////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam.h"
#include "XAM_Router.h"
#include "XAM_Logger.h"
#include "XAM_Utils.h"

// ============================================================================
// Methods for evaluating xam_status 
// ============================================================================

EXPORT xam_boolean DECL
XAM_GetErrorToken (const xam_handle_t inHandle,
                   const xam_status inStatus,
                   xam_string* const outToken)
{
    xam_boolean vSuccess = false;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_enter, 
                        "XAM_GetErrorToken");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                         component_api_param, 
                        "[in] [xam_status] %i", inStatus);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string*] %p", outToken);
    }

    vSuccess = XAM_Router::XAM_GetErrorToken(inHandle, inStatus, outToken);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetErrorToken");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_string] \"%s\"", _logString(*outToken));
    }

    return vSuccess;
}

// =============================================================================
// Methods for the XIterator 
// =============================================================================

EXPORT xam_status DECL
XAM_OpenFieldIterator (const xam_handle_t inHandle,  
                       const xam_string inPattern,
                       xiterator_handle* const outIterator)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_OpenFieldIterator");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPattern));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xiterator_handle] %"XAM_INT_FORMAT"d", outIterator);
    }

    vStatus = XAM_Router::XAM_OpenFieldIterator(inHandle, 
                                                inPattern, 
                                                outIterator);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_OpenFieldIterator");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xiterator_handle] %"XAM_INT_FORMAT"d", *outIterator);    

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XIterator_HasNext (const xiterator_handle inHandle, 
                   xam_boolean* const outValue)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XIterator_HasNext");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xiterator_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outValue);
    }

    vStatus = XAM_Router::XIterator_HasNext (inHandle, outValue);     

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XIterator_HasNext");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outValue ? "true" : "false");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XIterator_Next (const xiterator_handle inHandle, 
                xam_string* const outName)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XIterator_Next");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xiterator_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string*] %p", outName);
    }

    vStatus = XAM_Router::XIterator_Next (inHandle, outName);     

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XIterator_Next");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_string] \"%s\"", 
                            *outName ? _logString(*outName) : "(null)");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 
  
EXPORT xam_status DECL 
XIterator_Close (xiterator_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XIterator_Close");

        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xiterator_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XIterator_Close(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XIterator_Close");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for managing XAM Fields (properties or XStreams)
// ============================================================================

EXPORT xam_status DECL
XAM_ContainsField(const xam_handle_t inHandle,  
                  const xam_string inName,
                  xam_boolean* const outContained)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_ContainsField");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outContained);
    }

    vStatus = XAM_Router::XAM_ContainsField(inHandle, inName, outContained);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_ContainsField");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_boolean] %s", 
                        *outContained ? "true" : "false");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XAM_SetFieldAsBinding (const xset_handle inHandle,
                       const xam_string inName)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetFieldAsBinding");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
    }

    vStatus = XAM_Router::XAM_SetFieldAsBinding(inHandle, inName);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetFieldAsBinding");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetFieldAsNonbinding (const xset_handle inHandle,
                          const xam_string inName)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetFieldAsBinding");

        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);

        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
    }

    vStatus = XAM_Router::XAM_SetFieldAsNonbinding(inHandle, inName);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetFieldAsNonbinding");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetFieldType (const xam_handle_t inHandle,
                  const xam_string inName,
                  xam_string* const outType)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetFieldType");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string*] %p", outType);
    }

    vStatus = XAM_Router::XAM_GetFieldType(inHandle, inName, outType);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetFieldType");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG,                 
                            component_api_param, 
                            "[out] [xam_string] \"%s\"", 
                            *outType ? _logString(*outType) : "(null)");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetFieldLength (const xam_handle_t inHandle,
                    const xam_string inName,
                    xam_int* const outLength)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetFieldLength");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outLength);
    }

    vStatus = XAM_Router::XAM_GetFieldLength(inHandle, inName, outLength);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetFieldLength");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_int] %"XAM_INT_FORMAT"d", *outLength);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetFieldBinding (const xam_handle_t inHandle,
                     const xam_string inName,
                     xam_boolean* const outFixed)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetFieldBinding");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outFixed);
    }

    vStatus = XAM_Router::XAM_GetFieldBinding(inHandle, inName, outFixed);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetFieldBinding");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_boolean] %s", 
                        *outFixed ? "true" : "false");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetFieldReadOnly (const xam_handle_t inHandle,
                      const xam_string inName,
                      xam_boolean* const outReadOnly)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetFieldReadOnly");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outReadOnly);
    }

    vStatus = XAM_Router::XAM_GetFieldReadOnly(inHandle, inName, outReadOnly);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetFieldReadOnly");  
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outReadOnly ? "true" : "false");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_DeleteField (const xam_handle_t inHandle,
                 const xam_string inName)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_DeleteField");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
    }

    vStatus = XAM_Router::XAM_DeleteField(inHandle, inName);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_DeleteField");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for managing property fields
// ============================================================================

EXPORT xam_status DECL 
XAM_CreateBoolean (const xam_handle_t inHandle,
                   const xam_string inName,
                   const xam_boolean inBinding,
                   const xam_boolean inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateBoolean");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inValue ? "true" : "false");
    }

    vStatus = XAM_Router::XAM_CreateBoolean(inHandle, 
                                            inName, 
                                            inBinding, 
                                            inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateBoolean");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_CreateInt (const xam_handle_t inHandle,
               const xam_string inName,
               const xam_boolean inBinding,
               const xam_int inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateInt");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inValue);
    }

    vStatus = XAM_Router::XAM_CreateInt(inHandle, 
                                        inName, 
                                        inBinding, 
                                        inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateInt");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_CreateDouble (const xam_handle_t inHandle,
                 const xam_string inName,
                 const xam_boolean inBinding,
                 const xam_double inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateDouble");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_double] %f", inValue);
    }

    vStatus = XAM_Router::XAM_CreateDouble(inHandle, 
                                          inName, 
                                          inBinding, 
                                          inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateDouble");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_CreateXUID (const xam_handle_t inHandle,
                    const xam_string inName,
                    const xam_boolean inBinding,
                    const xam_xuid inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateXUID");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inValue);
    }

    vStatus = XAM_Router::XAM_CreateXUID(inHandle, 
                                         inName, 
                                         inBinding, 
                                         inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateXUID");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_CreateString (const xam_handle_t inHandle,
                      const xam_string inName,
                      const xam_boolean inBinding,
                      const xam_string inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateString");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inValue));
    }

    vStatus = XAM_Router::XAM_CreateString(inHandle, 
                                           inName, 
                                           inBinding, 
                                           inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateString");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_CreateDatetime (const xam_handle_t inHandle,
                    const xam_string inName,
                    const xam_boolean inBinding,
                    const xam_datetime inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateDatetime");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_datetime] %s", inValue);
    }

    vStatus = XAM_Router::XAM_CreateDatetime(inHandle, 
                                             inName, 
                                             inBinding, 
                                             inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateDatetime");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetBoolean (const xam_handle_t inHandle,
                const xam_string inName,
                const xam_boolean inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetBoolean");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inValue ? "true" : "false");
    }

    vStatus = XAM_Router::XAM_SetBoolean(inHandle, inName, inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetBoolean");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetInt (const xam_handle_t inHandle,
            const xam_string inName,
            const xam_int inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetInt");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inValue);
    }

    vStatus = XAM_Router::XAM_SetInt(inHandle, inName, inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetInt");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetDouble (const xam_handle_t inHandle,
              const xam_string inName,
              const xam_double inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetDouble");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_double] %f", inValue);
    }

    vStatus = XAM_Router::XAM_SetDouble(inHandle, inName, inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetDouble");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetXUID (const xam_handle_t inHandle,
             const xam_string inName,
             const xam_xuid inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetXUID");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inValue);
    }

    vStatus = XAM_Router::XAM_SetXUID(inHandle, inName, inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetXUID");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetString (const xam_handle_t inHandle,
               const xam_string inName,
               const xam_string inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetString");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] %s", _logString(inValue));
    }

    vStatus = XAM_Router::XAM_SetString(inHandle, inName, inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetString");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_SetDatetime (const xam_handle_t inHandle,
                 const xam_string inName,
                 const xam_datetime inValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_SetDatetime");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_datetime] %s", inValue);
    }

    vStatus = XAM_Router::XAM_SetDatetime(inHandle, inName, inValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_SetDatetime");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetBoolean (const xam_handle_t inHandle,
                const xam_string inName,
                xam_boolean* const outValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetBoolean");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outValue);
    }

    vStatus = XAM_Router::XAM_GetBoolean(inHandle, inName, outValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetBoolean");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outValue ? "true" : "false");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetInt (const xam_handle_t inHandle,
            const xam_string inName,
            xam_int* const outValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetInt");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outValue);
    }

    vStatus = XAM_Router::XAM_GetInt(inHandle, inName, outValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetInt");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_int] %"XAM_INT_FORMAT"d", *outValue);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetDouble (const xam_handle_t inHandle,
              const xam_string inName,
              xam_double* const outValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetDouble");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_double*] %p", outValue);
    }

    vStatus = XAM_Router::XAM_GetDouble(inHandle, inName, outValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetDouble");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_double] %f", *outValue);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetXUID (const xam_handle_t inHandle,
             const xam_string inName,
             xam_xuid* const outValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetXUID");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid*] %p", outValue);
    }

    vStatus = XAM_Router::XAM_GetXUID(inHandle, inName, outValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetXUID");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetString (const xam_handle_t inHandle,
               const xam_string inName,
               xam_string* const outValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetString");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string*] %p", outValue);
    }

    vStatus = XAM_Router::XAM_GetString(inHandle, inName, outValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetString");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_string] \"%s\"", 
                            *outValue ? _logString(*outValue) : "(null)");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XAM_GetDatetime (const xam_handle_t inHandle,
                 const xam_string inName,
                 xam_datetime* const outValue)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_GetDatetime");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_datetime*] %p", outValue);
    }

    vStatus = XAM_Router::XAM_GetDatetime(inHandle, inName, outValue);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_GetDatetime");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_datetime] \"%s\"", 
                        *outValue ? *outValue : "(null)");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 
// ============================================================================
// Methods for XStreams and XStream fields
// ============================================================================

EXPORT xam_status DECL 
XAM_CreateXStream (const xam_handle_t inHandle,
                   const xam_string inName,
                   const xam_boolean inBinding,
                   const xam_string inType,
                   xstream_handle* const outXStream)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_CreateXStream");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inType));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle*] %p", outXStream);
    }

    vStatus = XAM_Router::XAM_CreateXStream(inHandle, 
                                            inName, 
                                            inBinding, 
                                            inType, 
                                            outXStream);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_CreateXStream");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xstream_handle] %"XAM_INT_FORMAT"d", *outXStream);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XAM_OpenXStream (const xam_handle_t inHandle,
                 const xam_string inName,
                 const xam_string inMode,
                 xstream_handle* const outXStream)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_OpenXStream");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle_t] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inMode));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle*] %p", outXStream);
    }

    vStatus = XAM_Router::XAM_OpenXStream(inHandle, 
                                          inName, 
                                          inMode, 
                                          outXStream);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_OpenXStream");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xstream_handle] %"XAM_INT_FORMAT"d", *outXStream);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XStream_Read (const xstream_handle inHandle,
              char* const ioBuffer,
              const xam_int inBufferLength,
              xam_int* const outBytesRead)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_Read");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [const char*] %p", ioBuffer);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inBufferLength);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outBytesRead);
    }    

    vStatus = XAM_Router::XStream_Read(inHandle, 
                                       ioBuffer, 
                                       inBufferLength, 
                                       outBytesRead);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_Read");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_int] %"XAM_INT_FORMAT"d", *outBytesRead);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XStream_Write (const xstream_handle inHandle,
                   const char* inBuffer,
                   const xam_int inByteCount,
                   xam_int* const outBytesWritten)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_Write");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [const char*] %p", inBuffer);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inByteCount);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outBytesWritten);
    }

    vStatus = XAM_Router::XStream_Write(inHandle, 
                                        inBuffer, 
                                        inByteCount, 
                                        outBytesWritten);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_Write");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_int] %"XAM_INT_FORMAT"d", *outBytesWritten);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XStream_Seek (const xstream_handle inHandle,
                  const xam_int inOffset,
                  const xam_int inWhence)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_Seek");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inOffset);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inWhence);
    }

    vStatus = XAM_Router::XStream_Seek(inHandle, inOffset, inWhence);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit,
                        "Exit XStream_Seek");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XStream_Tell (const xstream_handle inHandle,
              xam_int* const outPosition)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_Tell");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outPosition);
    }

    vStatus = XAM_Router::XStream_Tell(inHandle, outPosition);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_Tell");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_int] %"XAM_INT_FORMAT"d", *outPosition);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XStream_Abandon(const xstream_handle inHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_Abandon");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XStream_Abandon(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_Abandon");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XStream_Close (xstream_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_Close");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XStream_Close(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_Close");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for managing the connection to a XAM storage system
// ============================================================================

EXPORT xam_status DECL 
XAMLibrary_Connect (const xam_string inXRI,
                    xsystem_handle* const outHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAMLibrary_Connect", inXRI, outHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inXRI));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle*] %p", outHandle);
    }

    vStatus = XAM_Router::XAMLibrary_Connect(inXRI, outHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAMLibrary_Connect");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xsystem_handle] %"XAM_INT_FORMAT"d", *outHandle);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSystem_Authenticate (const xsystem_handle inHandle,
                      const char* inBuffer,
                      const xam_int inByteCount,
                      xstream_handle* const outXStream)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_Authenticate");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [const char*] %p", inBuffer);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inByteCount);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle*] %p", outXStream);
    }

    vStatus = XAM_Router::XSystem_Authenticate(inHandle, 
                                               inBuffer, 
                                               inByteCount, 
                                               outXStream);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_Authenticate");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xstream_handle] %"XAM_INT_FORMAT"d", *outXStream);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSystem_Close (const xsystem_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_Close");

        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSystem_Close(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_Close");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSystem_Abandon (const xsystem_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_Abandon");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSystem_Abandon(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                            component_api_exit, 
                            "Exit XSystem_Abandon");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for XSet management
// ============================================================================

EXPORT xam_status DECL 
XSystem_DeleteXSet (const xsystem_handle inHandle,
                        const xam_xuid inXUID)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_DeleteXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
    }

    vStatus = XAM_Router::XSystem_DeleteXSet(inHandle, inXUID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_DeleteXSet");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSystem_HoldXSet (const xsystem_handle inHandle,
                  const xam_xuid inXUID,
                  const xam_string inHoldID)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_HoldXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inHoldID));
    }

    vStatus = XAM_Router::XSystem_HoldXSet(inHandle, inXUID, inHoldID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_HoldXSet");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSystem_ReleaseXSet (const xsystem_handle inHandle,
                     const xam_xuid inXUID,
                     const xam_string inHoldID)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_ReleaseXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inHoldID));
    }

    vStatus = XAM_Router::XSystem_ReleaseXSet(inHandle, inXUID, inHoldID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_ReleaseXSet");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSystem_AccessXSet (const xsystem_handle inHandle,
                    const xam_xuid inXUID,
                    const xam_int inMode,
                    xam_boolean* const outIsAccessible)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_AccessXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inMode);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outIsAccessible);
    }

    vStatus = XAM_Router::XSystem_AccessXSet(inHandle, 
                                             inXUID, 
                                             inMode, 
                                             outIsAccessible);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_AccessXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outIsAccessible ? "true" : "false");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSystem_IsXSetRetained (const xsystem_handle inHandle,
                        const xam_xuid inXUID,
                        xam_boolean* const outIsRetained)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_IsXSetRetained");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outIsRetained);
    }

    vStatus = XAM_Router::XSystem_IsXSetRetained(inHandle, 
                                                 inXUID, 
                                                 outIsRetained);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_IsXSetRetained");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outIsRetained ? "true" : "false");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSystem_GetXSetAccessTime (const xsystem_handle inHandle,
                           const xam_xuid inXUID,
                           xam_datetime* const outAccessTime)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_AccessXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_datetime*] %p", outAccessTime);
    }

    vStatus = XAM_Router::XSystem_GetXSetAccessTime(inHandle, 
                                                    inXUID, 
                                                    outAccessTime);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_AccessXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_datetime] %s", *outAccessTime);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

// ============================================================================
// Methods for creating an XSeT
// ============================================================================

EXPORT xam_status DECL 
XSystem_CreateXSet (const xsystem_handle inHandle,
                    const xam_string inMode,
                    xset_handle* const outXSet)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_CreateXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] %s", _logString(inMode));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle*] %p", outXSet);
    }

    vStatus = XAM_Router::XSystem_CreateXSet(inHandle, inMode, outXSet);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_CreateXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param,
                            "[out] [xset_handle] %"XAM_INT_FORMAT"d", *outXSet);

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for XSet policy management
// ============================================================================

EXPORT xam_status DECL
XSet_ApplyAccessPolicy (const xset_handle inHandle,
                        const xam_boolean inBinding,
                        const xam_string inPolicy)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyAccessPolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyAccessPolicy(inHandle, 
                                                 inBinding, 
                                                 inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyAccessPolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_ResetAccessFields (const xset_handle inHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ResetAccessFields");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSet_ResetAccessFields(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ResetAccessFields");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XSet_ApplyManagementPolicy (const xset_handle inHandle,
                            const xam_boolean inBinding,
                            const xam_string inPolicy)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyManagementPolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyManagementPolicy(inHandle, 
                                                     inBinding, 
                                                     inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyManagementPolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSet_ResetManagementFields (const xset_handle inHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ResetManagementFields");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSet_ResetManagementFields(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ResetManagementFields");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_CreateRetention (const xset_handle inHandle,
                      const xam_boolean inBinding,
                      const xam_string inRetentionID)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_CreateRetention");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
    }

    vStatus = XAM_Router::XSet_CreateRetention(inHandle,
                                               inBinding,
                                               inRetentionID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_CreateRetention");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_SetRetentionEnabledFlag (const xset_handle inHandle,
                              const xam_string inRetentionID,
                              const xam_boolean inBinding,
                              const xam_boolean inEnabled)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SetRetentionEnabledFlag");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", inEnabled ? "true" : "false");
    }

    vStatus = XAM_Router::XSet_SetRetentionEnabledFlag(inHandle,
                                                       inRetentionID,
                                                       inBinding,
                                                       inEnabled);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SetRetentionEnabledFlag");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_ApplyRetentionEnabledPolicy (const xset_handle inHandle,
                                  const xam_string inRetentionID,
                                  const xam_boolean inBinding,
                                  const xam_string inPolicy)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyRetentionEnabledPolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyRetentionEnabledPolicy(inHandle,
                                                           inRetentionID,
                                                           inBinding,
                                                           inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyRetentionEnabledPolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

xam_status
XSet_SetRetentionDuration (const xset_handle inHandle,
                           const xam_string inRetentionID,
                           const xam_boolean inBinding,
                           const xam_int inDuration)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SetRetentionDuration");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inDuration);
    }

    vStatus = XAM_Router::XSet_SetRetentionDuration(inHandle,
                                                    inRetentionID,
                                                    inBinding,
                                                    inDuration);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SetRetentionDuration");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_ApplyRetentionDurationPolicy (const xset_handle inHandle,
                                   const xam_string inRetentionID,
                                   const xam_boolean inBinding,
                                   const xam_string inPolicy)    
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyRetentionDurationPolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyRetentionDurationPolicy(inHandle, 
                                                            inRetentionID,
                                                            inBinding, 
                                                            inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyRetentionDurationPolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSet_SetRetentionStarttime (const xset_handle inHandle,
                            const xam_string inRetentionID,
                            const xam_boolean inBinding)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SetRetentionStarttime");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
    }

    vStatus = XAM_Router::XSet_SetRetentionStarttime(inHandle, 
                                                     inRetentionID,
                                                     inBinding);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SetRetentionStarttime");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_SetBaseRetention (const xset_handle inHandle,
                       const xam_boolean inBinding,
                       const xam_int inDuration)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SetBaseRetention");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inDuration);
    }

    vStatus = XAM_Router::XSet_SetBaseRetention(inHandle, 
                                                inBinding,
                                                inDuration);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SetBaseRetention");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

xam_status
XSet_ApplyBaseRetentionPolicy (const xset_handle inHandle,
                               const xam_boolean inBinding,
                               const xam_string inPolicy)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyBaseRetentionPolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyBaseRetentionPolicy(inHandle, 
                                                        inBinding,
                                                        inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyBaseRetentionPolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XSet_ApplyAutoDeletePolicy (const xset_handle inHandle,
                                const xam_boolean inBinding,
                                const xam_string inPolicy)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyAutoDeletePolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyAutoDeletePolicy(inHandle, 
                                                     inBinding, 
                                                     inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyAutoDeletePolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSet_SetAutoDelete (const xset_handle inHandle,
                        const xam_boolean inBinding,
                        const xam_boolean inAutoDelete)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SetAutoDelete");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inAutoDelete ? "true" : "false");           
    }

    vStatus = XAM_Router::XSet_SetAutoDelete(inHandle, 
                                             inBinding, 
                                             inAutoDelete);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SetAutoDelete");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSet_ApplyShredPolicy (const xset_handle inHandle,
                       const xam_boolean inBinding,
                       const xam_string inPolicy)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyShredPolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));
    }

    vStatus = XAM_Router::XSet_ApplyShredPolicy(inHandle, 
                                                inBinding, 
                                                inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyShredPolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XSet_SetShred (const xset_handle inHandle,
               const xam_boolean inBinding,
               const xam_boolean inShred)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SetShred");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inShred ? "true" : "false");       
    }

    vStatus = XAM_Router::XSet_SetShred(inHandle, 
                                        inBinding, 
                                        inShred);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SetShred");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XSet_ApplyStoragePolicy (const xset_handle inHandle,
                         const xam_boolean inBinding,
                         const xam_string inPolicy)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_ApplyStoragePolicy");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean] %s", 
                        inBinding ? "true" : "false");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inPolicy));           
    }

    vStatus = XAM_Router::XSet_ApplyStoragePolicy(inHandle, 
                                                  inBinding, 
                                                  inPolicy);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_ApplyStoragePolicy");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSet_GetActualRetentionDuration (const xset_handle inHandle,
                                 const xam_string inRetentionID,
                                 xam_int* const outDuration)  
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_GetActualRetentionDuration");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outDuration);        
    }

    vStatus = XAM_Router::XSet_GetActualRetentionDuration(inHandle, 
                                                          inRetentionID,
                                                          outDuration);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_GetActualRetentionDuration");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_int] %"XAM_INT_FORMAT"d", *outDuration);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSet_GetActualRetentionEnabled (const xset_handle inHandle,
                                const xam_string inRetentionID,
                                xam_boolean* const outEnabled)    
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_GetActualRetentionEnabled");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inRetentionID));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outEnabled);        
    }

    vStatus = XAM_Router::XSet_GetActualRetentionEnabled(inHandle, 
                                                         inRetentionID,
                                                         outEnabled);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_GetActualRetentionEnabled");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_boolean] %s", 
                        *outEnabled ? "true" : "false");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSet_GetActualAutoDelete (const xset_handle inHandle,
                              xam_boolean* const outEnabled)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_GetActualAutoDelete");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outEnabled);        
    }

    vStatus = XAM_Router::XSet_GetActualAutoDelete(inHandle, outEnabled);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_GetActualAutoDelete");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outEnabled ? "true" : "false");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

xam_status
XSet_GetActualShred (const xset_handle inHandle,
                     xam_boolean* const outEnabled)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_GetActualShred");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outEnabled);        
    }

    vStatus = XAM_Router::XSet_GetActualShred(inHandle, outEnabled);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_GetActualShred");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outEnabled ? "true" : "false");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

// ============================================================================
// Methods for editing an XSet
// ============================================================================

EXPORT xam_status DECL 
XSystem_OpenXSet (const xsystem_handle inHandle,
                  const xam_xuid inXUID,
                  const xam_string inMode,
                  xset_handle* const outXSet)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_OpenXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);  
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inMode));  
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle*] %p", outXSet); 
    }

    vStatus = XAM_Router::XSystem_OpenXSet(inHandle, inXUID, inMode, outXSet);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_OpenXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xset_handle] %"XAM_INT_FORMAT"d", *outXSet);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSystem_CopyXSet (const xsystem_handle inHandle,
                  const xam_xuid inXUID,
                  const xam_string inMode,
                  xset_handle* const outXSet)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_CopyXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);  
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inMode));  
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle*] %p", outXSet); 
    }

    vStatus = XAM_Router::XSystem_CopyXSet(inHandle, inXUID, inMode, outXSet);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_CopyXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xset_handle] %"XAM_INT_FORMAT"d", *outXSet);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XSet_Commit (const xset_handle inHandle,
                xam_xuid* const outXUID)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,  
                        component_api_enter, 
                        "XSet_Commit");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid*] %p", outXUID); 
    }

    vStatus = XAM_Router::XSet_Commit(inHandle, outXUID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_Commit");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_xuid] \"%s\"", 
                            *outXUID ? *outXUID : "(null)");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSet_Close (const xset_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_Close");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSet_Close(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_Close");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSet_Abandon (const xset_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_Abandon");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSet_Abandon(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_Abandon");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for XSet migration
// ============================================================================

EXPORT xam_status DECL 
XSet_OpenExportXStream (const xset_handle inHandle,
                            xstream_handle* const outXStream)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_OpenExportXStream");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle*] %p", outXStream);
    }

    vStatus = XAM_Router::XSet_OpenExportXStream(inHandle, outXStream);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_OpenExportXStream");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xstream_handle] %"XAM_INT_FORMAT"d", *outXStream);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSet_OpenImportXStream (const xset_handle inHandle,
                          xstream_handle* const outXStream)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_OpenImportXStream");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle*] %p", outXStream);
    }

    vStatus = XAM_Router::XSet_OpenImportXStream(inHandle, outXStream);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_OpenImportXStream");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xstream_handle] %"XAM_INT_FORMAT"d", *outXStream);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

// ============================================================================
// Methods for job control
// ============================================================================

EXPORT xam_status DECL 
XSet_SubmitJob (const xset_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_SubmitJob");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSet_SubmitJob(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_SubmitJob");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XSet_HaltJob (const xset_handle inHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_HaltJob");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XSet_HaltJob(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_HaltJob");

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

/* ========================================================================
 * methods for async i/o
 * =======================================================================*/

EXPORT xam_status DECL
XSystem_AsyncOpenXSet (const xsystem_handle inHandle,
                       const xam_xuid inXUID,
                       const xam_string inMode,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_AsyncOpenXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inMode));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);    
    }

    vStatus = XAM_Router::XSystem_AsyncOpenXSet(inHandle, 
                                                inXUID, 
                                                inMode, 
                                                inXOPID, 
                                                inCallback, 
                                                outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_AsyncOpenXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);  

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

xam_status
XSystem_AsyncCopyXSet (const xsystem_handle inHandle,
                       const xam_xuid inXUID,
                       const xam_string inMode,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSystem_AsyncCopyXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xsystem_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid] \"%s\"", inXUID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inMode));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);    
    }

    vStatus = XAM_Router::XSystem_AsyncCopyXSet(inHandle, 
                                                inXUID, 
                                                inMode, 
                                                inXOPID, 
                                                inCallback, 
                                                outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSystem_AsyncCopyXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);  

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL 
XAM_AsyncOpenXStream (const xam_handle_t inHandle,
                      const xam_string inName,
                      const xam_string inMode,
                      const XOPID inXOPID,
                      xasync_callback inCallback,
                      xasync_handle* const outAsyncHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAM_AsyncOpenXStream");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_handle] %p", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inName));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_string] \"%s\"", _logString(inMode));
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);    
    }

    vStatus = XAM_Router::XAM_AsyncOpenXStream(inHandle, 
                                               inName, 
                                               inMode, 
                                               inXOPID, 
                                               inCallback, 
                                               outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAM_AsyncOpenXStream");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);  

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XStream_AsyncRead (const xstream_handle inHandle,
                   char* const ioBuffer,
                   const xam_int inBufferLength,
                   const XOPID inXOPID,
                   xasync_callback inCallback,
                   xasync_handle* const outAsyncHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_AsyncRead");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [const char*] %p", ioBuffer);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inBufferLength);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);
    }    

    vStatus = XAM_Router::XStream_AsyncRead(inHandle, 
                                            ioBuffer, 
                                            inBufferLength, 
                                            inXOPID, 
                                            inCallback, 
                                            outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_AsyncRead");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL 
XStream_AsyncWrite (const xstream_handle inHandle,
                    const char* inBuffer,
                    const xam_int inByteCount,
                    const XOPID inXOPID,
                    xasync_callback inCallback,
                    xasync_handle* const outAsyncHandle)      
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_AsyncWrite");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [const char*] %p", inBuffer);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int] %"XAM_INT_FORMAT"d", inByteCount);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);
    }    

    vStatus = XAM_Router::XStream_AsyncWrite(inHandle, 
                                             inBuffer, 
                                             inByteCount, 
                                             inXOPID, 
                                             inCallback, 
                                             outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_AsyncWrite");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XSet_AsyncCommit (const xset_handle inHandle,
                  const XOPID inXOPID,
                  xasync_callback inCallback,
                  xasync_handle* const outAsyncHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XSet_AsyncCommit");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);    
    }

    vStatus = XAM_Router::XSet_AsyncCommit(inHandle, 
                                           inXOPID, 
                                           inCallback, 
                                           outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XSet_AsyncCommit");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);  

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
} 

EXPORT xam_status DECL
XStream_AsyncClose (const xstream_handle inHandle,
                    const XOPID inXOPID,
                    xasync_callback inCallback,
                    xasync_handle* const outAsyncHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XStream_AsyncClose");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID] %i", inXOPID);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_callback] %p", inCallback);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle*] %p", outAsyncHandle);    
    }

    vStatus = XAM_Router::XStream_AsyncClose(inHandle,
                                             inXOPID, 
                                             inCallback, 
                                             outAsyncHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XStream_AsyncClose");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xasync_handle] %"XAM_INT_FORMAT"d", *outAsyncHandle);  

        XAM_Logger::logStatus(XAM_LIBRARY_HANDLE, vStatus);
    }

    return vStatus;
}

/* ========================================================================
 * method prototypes for managing asyncronous operations
 * =======================================================================*/

EXPORT xam_status DECL
XAsync_Halt (const xasync_handle inHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_Halt");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XAsync_Halt(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_Halt");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_IsComplete (const xasync_handle inHandle,
                     xam_boolean* const outIsComplete)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_IsComplete");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_boolean*] %p", outIsComplete);
    }

    vStatus = XAM_Router::XAsync_IsComplete(inHandle, outIsComplete);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_IsComplete");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_boolean] %s", 
                            *outIsComplete ? "true" : "false");   

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetStatus (const xasync_handle inHandle,
                    xam_status* const outStatus)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetStatus");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_status*] %p", outStatus);
    }

    vStatus = XAM_Router::XAsync_GetStatus(inHandle, outStatus);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetStatus");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xam_status] %i", *outStatus);    

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetXOPID (const xasync_handle inHandle,
                   XOPID* const outXOPID)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetXOPID");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [XOPID*] %p", outXOPID);
    }

    vStatus = XAM_Router::XAsync_GetXOPID(inHandle, outXOPID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetXOPID");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [XOPID] %i", *outXOPID); 

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetXSet (const xasync_handle inHandle,
                xset_handle* const outXSet)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetXSet");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xset_handle*] %p", outXSet);
    }

    vStatus = XAM_Router::XAsync_GetXSet(inHandle, outXSet);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetXSet");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xset_handle] %"XAM_INT_FORMAT"d", *outXSet);        

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetXStream (const xasync_handle inHandle,
                   xstream_handle* const outXStream)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetXStream");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xstream_handle*] %p", outXStream);
    }

    vStatus = XAM_Router::XAsync_GetXStream(inHandle, outXStream);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetXStream");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                            component_api_param, 
                            "[out] [xstream_handle] %"XAM_INT_FORMAT"d", *outXStream);        

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetXUID (const xasync_handle inHandle,
                  xam_xuid* const outXUID)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetXUID");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_xuid*] %p", outXUID);
    }

    vStatus = XAM_Router::XAsync_GetXUID(inHandle, outXUID);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetXUID");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_xuid] %s", *outXUID); 

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetBytesRead (const xasync_handle inHandle,
                       xam_int* const outBytesRead)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetBytesRead");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outBytesRead);
    }

    vStatus = XAM_Router::XAsync_GetBytesRead(inHandle, outBytesRead);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetBytesRead");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_int] %"XAM_INT_FORMAT"d", *outBytesRead);  

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_GetBytesWritten (const xasync_handle inHandle,
                          xam_int* const outBytesWritten)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_GetBytesWritten");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xam_int*] %p", outBytesWritten);
    }

    vStatus = XAM_Router::XAsync_GetBytesWritten(inHandle, 
                                                 outBytesWritten);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_GetBytesWritten");
        if(!vStatus)
            XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[out] [xam_int] %"XAM_INT_FORMAT"d", *outBytesWritten); 

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}

EXPORT xam_status DECL
XAsync_Close (const xasync_handle inHandle)
{
    xam_status vStatus = (xam_status) XAM_UNKNOWN_ERROR;

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO,   
                        component_api_enter, 
                        "XAsync_Close");
        XAM_Logger::Log(LOGLEVEL_DEBUG, 
                        component_api_param, 
                        "[in] [xasync_handle] %"XAM_INT_FORMAT"d", inHandle);
    }

    vStatus = XAM_Router::XAsync_Close(inHandle);

    if(XAM_Logger::isEnabled())
    {
        XAM_Logger::Log(LOGLEVEL_INFO, 
                        component_api_exit, 
                        "Exit XAsync_Close");

        XAM_Logger::logStatus(inHandle, vStatus);
    }

    return vStatus;
}


