////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_XAsync.h"
#include "XAM_POSIX.h"
#include "XAM_Exception.h"
#include "XAM_Router.h"
#include "XAM_Logger.h"

// Method run by the asynchronous service thread
XAM_THREAD_FUNCTION_PTR
xam_library_xasync_thr_method(void *inInfo)
{
    XAM_LOG_DEBUG_PLUS_P2(component_xam_debug, "xam_library_xasync_thr_method(%p)", inInfo);

    XAM_ThreadInfo* vInfo = (XAM_ThreadInfo*) inInfo;
    if(vInfo)
    {
        XAM_XAsync* vOp = (XAM_XAsync*) vInfo->data;

        // Run the appropriate method
        xam_status vStatus = vOp->execSynchronousCall();
        vOp->setRunning(false);   
        if(!vOp->isHalted())
            vOp->notifyComplete(vStatus);
    }

    XAM_Thread::exit();

    return 0;
}

bool 
XAM_XAsync::read(const xstream_handle inHandle,
                 char* ioBuffer,
                 const xam_int inBufferLength)
{
    bool vRetVal = false;

    if(mMethod == NONE)
    {        
        // Spawn a thread to perform an XStream read
        mMethod = XSTREAM_READ;
        mXStream = inHandle;
        mBuffer = ioBuffer; 
        mBufferLen = inBufferLength;
        
        mThread = new XAM_Thread(xam_library_xasync_thr_method, (void*)this); 
        mThread->run();

        vRetVal = true;
    }
    else
    {
        XAMInvalidHandleException e;
        throw e;
    }

    return vRetVal;
}

bool
XAM_XAsync::write(const xstream_handle inHandle,
                  const char* inBuffer,
                  const xam_int inByteCount)
{
    bool vRetVal = false;

    if(mMethod == NONE)
    {        
        // Spawn a thread to perform an XStream write
        mMethod = XSTREAM_WRITE;
        mXStream = inHandle;
        mBuffer = (char*)inBuffer; 
        mBufferLen = inByteCount;
        
        mThread = new XAM_Thread(xam_library_xasync_thr_method, this); 
        mThread->run();

        vRetVal = true;
    }
    else
    {
        XAMInvalidHandleException e;
        throw e;
    }

    return vRetVal;
}

bool
XAM_XAsync::close(const xstream_handle inHandle)
{
    bool vRetVal = false;

    if(mMethod == NONE)
    {        
        // Spawn a thread to perform an XStream close
        mMethod = XSTREAM_CLOSE;
        mXStream = inHandle;
        
        mThread = new XAM_Thread(xam_library_xasync_thr_method, this);
        mThread->run();

        vRetVal = true;
    }
    else
    {
        XAMInvalidHandleException e;
        throw e;
    }

    return vRetVal;
}

bool
XAM_XAsync::openXStream(const xam_handle_t inHandle,
                        const xam_string inName,
                        const xam_string inMode)
{
    bool vRetVal = false;

    if(mMethod == NONE)
    {        
        // Spawn a thread to perform an XSystem open XSet
        mMethod = XAM_OPENXSTREAM;
        mHandle = inHandle;

        XAM_POSIX::xam_strcpy(mFieldName, inName);
        XAM_POSIX::xam_strcpy(mXStreamMode, inMode);
        
        mThread = new XAM_Thread(xam_library_xasync_thr_method, this); 
        mThread->run();

        vRetVal = true;
    }
    else
    {
        XAMInvalidHandleException e;
        throw e;
    }

    return vRetVal;
}

xam_status
XAM_XAsync::execSynchronousCall()
{
    xam_status vRetVal = XAM_UNKNOWN_ERROR;

    XAM_LOG_DEBUG_PLUS_P2(component_xam_debug, "XAM_XAsync::execSynchronousCall(%i)", (int)mMethod);

    // Call the synchronous version of the API from this service thread
    switch(mMethod)
    {
    case XAM_OPENXSTREAM:
        vRetVal = XAM_Router::XAM_OpenXStream(mHandle, 
                                              mFieldName, 
                                              mXStreamMode, 
                                              &mXStreamResult);
        if(!vRetVal && (mXStreamResult != 0))
        {
            // Delete the wrapper object. It will be wrapped at the router 
            // level again. We do this to support VIM handle wrapping at 
            // the router level and to keep things consistent.
            XStream* vXStream = (XStream*) mXStreamResult;
            mXStreamResult = vXStream->getHandle();
            XAM_HandleManager::deleteHandle(vXStream);
        }
        break;
    case XSTREAM_READ:
        vRetVal = XAM_Router::XStream_Read(mXStream, mBuffer, mBufferLen, &mBytes);
        break;
    case XSTREAM_WRITE:
        vRetVal = XAM_Router::XStream_Write(mXStream, mBuffer, mBufferLen, &mBytes);
        break;
    case XSTREAM_CLOSE:
        vRetVal = XAM_Router::XStream_Close(mXStream);
        break;
    case NONE:
        break;
    }

    return vRetVal;
}

void
XAM_XAsync::setRunning(xam_boolean inState, xam_boolean inObtainLock)
{
    if(inObtainLock == true)
    {
        XAM_CriticalSection vCritical(&mAsyncMutex);
        if(mThread)
            mThread->getInfo()->running = inState;
    }
    else
    {
        if(mThread)
            mThread->getInfo()->running = inState;
    }
}

void
XAM_XAsync::notifyComplete(xam_status inStatus, xam_boolean inObtainLock)
{
    if(inObtainLock == true)
    {
        // Protects multiple threads from affecting the callback notification
        XAM_CriticalSection vCritical(&mAsyncMutex);
        notifyCompleteHelper(inStatus);
    }
    else
    {
        notifyCompleteHelper(inStatus);
    }
}

void
XAM_XAsync::notifyCompleteHelper(xam_status inStatus)
{
    if(!mNotifyCompleteFinished && !mHalted)
    {
        XAM_LOG_DEBUG_PLUS_P2(component_xam_debug, "XAM_XAsync::notifyComplete(%i)", (int)inStatus);

        // Async operation is complete. Update status.
        mStatus = inStatus;

        // Don't allow the callback to be executed more than once
        mNotifyCompleteFinished = true;

        // Notify the user (in case a callback was registered)
        if(mCallback != (xasync_callback)0)
            mCallback((xasync_handle)this);
    }
}

xam_boolean 
XAM_XAsync::isComplete()
{
    xam_boolean vRetVal = false;

    if(mStatus != XAM_XASYNC_PENDING)
        vRetVal = true;

    return vRetVal;
}

xam_int 
XAM_XAsync::getBytesRead()
{
    if(mMethod != XSTREAM_READ)
    {
        XAMInvalidHandleException e;
        throw e;
    }

    // This operation is still pending
    if(mStatus == XAM_XASYNC_PENDING)
    {
        XAMAsyncOpPendingException e;
        throw e;
    }

    // This operation did not complete successfully!
    if(mStatus != 0)
    {
        XAM_Exception e(mStatus);
        throw e;
    }

    if(mHalted)
    {
        XAMXAsyncHaltedException e;
        throw e;
    }

    return mBytes;
}

xam_int 
XAM_XAsync::getBytesWritten()
{
    if(mMethod != XSTREAM_WRITE)
    {
        XAMInvalidHandleException e;
        throw e;
    }

    // This operation is still pending
    if(mStatus == XAM_XASYNC_PENDING)
    {
        XAMAsyncOpPendingException e;
        throw e;
    }

    // This operation did not complete successfully!
    if(mStatus != 0)
    {
        XAM_Exception e(mStatus);
        throw e;
    }

    if(mHalted)
    {
        XAMXAsyncHaltedException e;
        throw e;
    }

    return mBytes;
}

xstream_handle 
XAM_XAsync::getXStream()
{
    if(mMethod != XAM_OPENXSTREAM)
    {
        XAMInvalidHandleException e;
        throw e;
    }

    // This operation is still pending
    if(mStatus == XAM_XASYNC_PENDING)
    {
        XAMAsyncOpPendingException e;
        throw e;
    }

    // This operation did not complete successfully!
    if(mStatus != 0)
    {
        XAM_Exception e(mStatus);
        throw e;
    }

    if(mHalted)
    {
        XAMXAsyncHaltedException e;
        throw e;
    }

    return mXStreamResult;
}

xam_status 
XAM_XAsync::halt()
{
    xam_boolean mAlreadyDone = false;

    // This operation is a no-op if already halted or complete (per SNIA TWG discussions)
    if(!mHalted)
    {
        // Synchronize with notifyComplete
        {
            XAM_CriticalSection vCritical(&mAsyncMutex);

            // This operation is a no-op if we are already complete
            if(isComplete())
                mAlreadyDone = true;
            else
                mHalted = true;
        }

        if(!mAlreadyDone)
        {
            // Halt the active service thread
            switch(mMethod)
            {
                case XAM_OPENXSTREAM:
                    // To prevent memory leaks, we need to wait for this operation
                    // to complete and then close the handle to cleanup.
                    mThread->join();
                    XAM_Router::XStream_Close(mXStreamResult);
                    break;
                case XSTREAM_CLOSE:
                    // We need to let this operation finish. Otherwise, the user 
                    // won't know whether or not to close the XStream themselves.
                    mThread->join();
                    break;
                default:
                   // By default, kill the thread and return immediately
                   // If this operation fails, join instead.
                   if(!mThread->kill())            
                        mThread->join();            
                    break;
            }
        }
    }

    // Mark complete state for halted operations!
    mStatus = (xam_status)0;

    return mStatus;
}


XAM_XAsync::XAM_XAsync(const XOPID inXOPID, xasync_callback inCallback)
: mMethod(NONE),
  mXOPID(inXOPID),
  mHandle(0),
  mXStream(0),
  mBuffer(0),
  mBufferLen(0),
  mXStreamResult(0),
  mBytes(0),
  mNotifyCompleteFinished(false),
  mThread(0),
  mHalted(false),
  mCallback(inCallback),
  mStatus(XAM_XASYNC_PENDING)
{ 
    XAM_POSIX::xam_strcpy(mFieldName, "");
    XAM_POSIX::xam_strcpy(mXStreamMode, "");
}









