////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_strings.h"

#include "XAM_BoolProperty.h"
#include "XAM_IntProperty.h"
#include "XAM_DoubleProperty.h"
#include "XAM_XUIDProperty.h"
#include "XAM_StringProperty.h"
#include "XAM_DatetimeProperty.h"
#include "XAM_FieldContainer.h"
#include "XAM_FieldIterator.h"
#include "XAM_XAsync.h"
#include "XAM_XAsyncHandler.h"

void
XAM_FieldContainer::cache(XAM_FieldNameCache* inCache)
{
    XAM_CriticalSection vCritical(&mMutex);

    int vSz = mFieldArray.size();
    for(int ix=(vSz - 1) ; ix >= 0 ; ix--)
    {
        XAM_Field* vTempField = mFieldArray[ix];
        if(vTempField)
        {
            inCache->addField(vTempField->getName());
        }
    }

    if(mSyntheticFieldMap)
        mSyntheticFieldMap->cache(inCache);
}

void 
XAM_FieldContainer::addField(XAM_Field* inField)
{
    // Add the field to this container
    mFieldArray.push_back(inField);
}

const int 
XAM_FieldContainer::findFieldIndex(const char *inName)
{
    int vIndex = -1;

    // Iterate through the table and find the field with the same name (if any).
    // Backwards search for performance. The most likely
    // position for this entry is at the end of the list.
    int vSz = mFieldArray.size();
    for(int ix=(vSz - 1) ; ix >= 0 ; ix--)
    {
        XAM_Field* vTempField = mFieldArray[ix];
        if(vTempField)
        {
            // If this field name matches inName, return this field
            const char* vName = vTempField->getName();
            if(XAM_POSIX::xam_strcmp(vName, inName) == 0)
            {
                vIndex = ix;
                break;
            }
        }
    }

    return vIndex;
}

XAM_Field* 
XAM_FieldContainer::findField(const char *inName)
{
    XAM_Field* vField = (XAM_Field*)0;

    // Iterate through the table and find the field with the same name (if any).
    // Backwards search for performance. The most likely
    // position for this entry is at the end of the list.
    int vSz = mFieldArray.size();
    for(int ix=(vSz - 1) ; ix >= 0 ; ix--)
    {
        XAM_Field* vTempField = mFieldArray[ix];
        if(vTempField)
        {
            // If this field name matches inName, return this field
            const char* vName = vTempField->getName();
            if(XAM_POSIX::xam_strcmp(vName, inName) == 0)
            {
                vField = vTempField;
                break;
            }
        }
    }

    return vField;
}

void
XAM_FieldContainer::containsField (const xam_string inName,
                                   xam_boolean* const outExists)
{
    XAM_FieldIterator* vIter = new XAM_FieldIterator(this, inName);
    if(!vIter)
    {
        XAMOutOfMemoryException e;
        throw e;
    }

    *outExists = vIter->containsField(inName);
    delete vIter;
}

void 
XAM_FieldContainer::setFieldAsBinding(const xam_string inName)
{
    // This field container does not support the persisting of fields
    // to storage, so a "true" binding state is not allowed!
    XAMOperationNotAllowedException e;
    throw e;

    /*
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setFieldAsBinding(inName);
    else
    {
        // Locate the field and set the binding (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            vField->setBinding(true);
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
    */
}

void 
XAM_FieldContainer::setFieldAsNonbinding(const xam_string inName)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setFieldAsNonbinding(inName);
    else
    {
        // Locate the field and set nonbinding (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            vField->setBinding(false);
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setFieldAsReadOnly(const xam_string inName)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setFieldAsReadOnly(inName);
    else
    {
        // Locate the field and set read only (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            vField->setReadOnly(true);
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getFieldType(const xam_string inName,
                                 xam_string* const outType)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Get the field type
    getFieldTypeHelper(inName, outType);
}



void 
XAM_FieldContainer::getFieldTypeHelper(const xam_string inName, xam_string* const outType)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getFieldType(inName, outType);
    else
    {
        // Locate the field and get the field type (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            const char* vMimeType = vField->getMimeType();
            XAM_POSIX::xam_strcpy((char*)outType, vMimeType);
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getFieldLength(const xam_string inName, xam_int* const outLen)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getFieldLength(inName, outLen);
    else
    {
        // Locate the field and get the field length (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            *outLen = vField->getSize();
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getFieldBinding(const xam_string inName, xam_boolean* const outFixed)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getFieldBinding(inName, outFixed);
    else
    {
        // Locate the field and get the field binding (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            *outFixed = vField->getBinding();
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getFieldReadOnly(const xam_string inName, 
                                     xam_boolean* const outReadOnly)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getFieldReadOnly(inName, outReadOnly);
    else
    {
        // Locate the field and get the field read only state (if found)
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            *outReadOnly = vField->getReadOnly();
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::deleteField(const xam_string inName)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->deleteField(inName);
    else
    {
        // Locate the field and delete it (if found)
        int vIndex = findFieldIndex((char*)inName);
        if(vIndex >= 0)
        {
            // Throw an error if this field is in use
            XAM_Field* vField = mFieldArray[vIndex];
            if(vField->isOpen() == true)
            {
                XAMFieldInUseException e;
                throw e;
            }

            // Remove the field from the array and delete it
            vField = mFieldArray.remove(vIndex);
            if(vField)
            {
                delete vField;
            }
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }        
    }
}

void 
XAM_FieldContainer::createBoolean(const xam_string inName, 
                                  const xam_boolean inBinding, 
                                  const xam_boolean inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    createBooleanHelper(inName, inBinding, inValue);
}

void 
XAM_FieldContainer::createBooleanHelper(const xam_string inName, 
                                        const xam_boolean inBinding, 
                                        const xam_boolean inValue)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

    // Only create if the field doesn't exist
    if(!findField((char*)inName))
    {
        XAM_Field* vField = 
            (XAM_Field*) new XAM_BoolProperty((char*)inName, 
                                              inBinding, 
                                              false, 
                                              inValue);        
        addField(vField);
    }
    else
    {
        XAMFieldExistsException e;
        throw e;
    }
}

void 
XAM_FieldContainer::createIntHelper(const xam_string inName, 
                                    const xam_boolean inBinding, 
                                    const xam_int inValue)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

    // Only create if the field doesn't exist
    if(!findField((char*)inName))
    {
        XAM_Field* vField = 
            (XAM_Field*) new XAM_IntProperty((char*)inName, 
                                             inBinding, 
                                             false, 
                                             inValue);
         
        addField(vField);
    }
    else
    {
        XAMFieldExistsException e;
        throw e;
    }
}

void 
XAM_FieldContainer::createInt(const xam_string inName, 
                              const xam_boolean inBinding,
                              const xam_int inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    createIntHelper(inName, inBinding, inValue);
}

void 
XAM_FieldContainer::createDouble(const xam_string inName, 
                                const xam_boolean inBinding, 
                                const xam_double inValue)
{
   XAM_CriticalSection vCritical(&mMutex);
   createDoubleHelper(inName, inBinding,inValue);
}

void 
XAM_FieldContainer::createDoubleHelper(const xam_string inName, 
                                       const xam_boolean inBinding,
                                       const xam_double inValue)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

    // Only create if the field doesn't exist
    if(!findField((char*)inName))
    {
        XAM_Field* vField = 
            (XAM_Field*) new XAM_DoubleProperty((char*)inName,
                                               inBinding, 
                                               false, 
                                               inValue);
        
        addField(vField);
    }
    else
    {
        XAMFieldExistsException e;
        throw e;
    }
}

void 
XAM_FieldContainer::createXUID(const xam_string inName, 
                               const xam_boolean inBinding, 
                               const xam_xuid inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    createXUIDHelper(inName, inBinding,inValue);
}

void 
XAM_FieldContainer::createXUIDHelper(const xam_string inName, 
                                     const xam_boolean inBinding,
                                     const xam_xuid inValue)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

  

    // Only create if the field doesn't exist
    if(!findField((char*)inName))
    {
        XAM_Field* vField = 
            (XAM_Field*) new XAM_XUIDProperty((char*)inName, 
                                              inBinding, 
                                              false, 
                                              inValue);
                    
        addField(vField);
    }
    else
    {
        XAMFieldExistsException e;
        throw e;
    }
}

void 
XAM_FieldContainer::createString(const xam_string inName, 
                                 const xam_boolean inBinding, 
                                 const xam_string inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    createStringHelper(inName, inBinding,inValue);
}

void 
XAM_FieldContainer::createStringHelper(const xam_string inName, 
                                       const xam_boolean inBinding,
                                       const xam_string inValue)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

    // Only create if the field doesn't exist
    if(!findField((char*)inName))
    {
        XAM_Field* vField = 
            (XAM_Field*) new XAM_StringProperty((char*)inName, 
                                                inBinding, 
                                                false, 
                                                inValue);
        
        addField(vField);
    }
    else
    {
        XAMFieldExistsException e;
        throw e;
    }
}

void 
XAM_FieldContainer::createDatetime(const xam_string inName, 
                                   const xam_boolean inBinding, 
                                   const xam_datetime inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    createDatetimeHelper(inName,inBinding,inValue);
}

void 
XAM_FieldContainer::createDatetimeHelper(const xam_string inName,
                                         const xam_boolean inBinding,
                                         const xam_datetime inValue)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

    // Only create if the field doesn't exist
    if(!findField((char*)inName))
    {
        XAM_Field* vField = 
            (XAM_Field*) new XAM_DatetimeProperty((char*)inName,
                                                  inBinding, 
                                                  false, 
                                                  inValue);
        
        addField(vField);
    }
    else
    {
        XAMFieldExistsException e;
        throw e;
    }
}

void 
XAM_FieldContainer::assertFieldIsWritable(XAM_Field *vField)
{
    if(vField->getReadOnly())
    {
        XAMReadOnlyFieldException e;
        throw e;
    }
}

void 
XAM_FieldContainer::setBooleanHelper(const xam_string inName, 
                                     const xam_boolean inValue)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setBoolean(inName, inValue);
    else
    {
        // Locate the field and set the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            assertFieldIsWritable(vField);

            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_BOOLEAN_MIME_TYPE);

            vField->write((char*)&inValue, 0, sizeof(xam_boolean));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setBoolean(const xam_string inName, 
                               const xam_boolean inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    setBooleanHelper(inName, inValue);
}

void 
XAM_FieldContainer::setIntHelper(const xam_string inName, 
                                 const xam_int inValue)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setInt(inName, inValue);
    else
    {
        // Locate the field and set the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            assertFieldIsWritable(vField);

            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_INT_MIME_TYPE);

            vField->write((char*)&inValue, 0, sizeof(xam_int));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setInt(const xam_string inName, 
                           const xam_int inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    setIntHelper(inName, inValue);
}

void 
XAM_FieldContainer::setDoubleHelper(const xam_string inName, 
                                    const xam_double inValue)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setDouble(inName, inValue);
    else
    {
        // Locate the field and set the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            assertFieldIsWritable(vField);

            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_DOUBLE_MIME_TYPE);

            vField->write((char*)&inValue, 0, sizeof(xam_double));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setDouble(const xam_string inName, 
                              const xam_double inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    setDoubleHelper(inName, inValue);
}

void 
XAM_FieldContainer::setXUIDHelper(const xam_string inName, 
                                  const xam_xuid inValue)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setXUID(inName, inValue);
    else
    {
        // Locate the field and set the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            assertFieldIsWritable(vField);

            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_XUID_MIME_TYPE);

            vField->write((char*)inValue, 0, sizeof(xam_xuid));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setXUID(const xam_string inName, 
                            const xam_xuid inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    setXUIDHelper(inName, inValue);
}

void 
XAM_FieldContainer::setStringHelper(const xam_string inName, 
                                    const xam_string inValue)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setString(inName, inValue);
    else
    {
        // Locate the field and set the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            assertFieldIsWritable(vField);

            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_STRING_MIME_TYPE);

            // Write the string data to the field (include the NULL terminator)
            xam_int vMaxLen = sizeof(xam_string);
            xam_int vActualLen = XAM_POSIX::xam_strlen(inValue) + 1;
            vField->write((char*)inValue, 0, (vMaxLen > vActualLen) ? vActualLen : vMaxLen); 
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setString(const xam_string inName, 
                              const xam_string inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    setStringHelper(inName, inValue);
}

void 
XAM_FieldContainer::setDatetimeHelper(const xam_string inName,
                                      const xam_datetime inValue)
{
    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->setDatetime(inName, inValue);
    else
    {
        // Locate the field and set the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            assertFieldIsWritable(vField);

            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_DATETIME_MIME_TYPE);

            // Write the string data to the field (include the NULL terminator)
            xam_int vMaxLen = sizeof(xam_datetime);
            xam_int vActualLen = XAM_POSIX::xam_strlen(inValue) + 1;
            vField->write((char*)inValue, 0, (vMaxLen > vActualLen) ? vActualLen : vMaxLen); 
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::setDatetime(const xam_string inName,
                                const xam_datetime inValue)
{
    XAM_CriticalSection vCritical(&mMutex);
    setDatetimeHelper(inName, inValue);
}

void 
XAM_FieldContainer::getBoolean(const xam_string inName, 
                               xam_boolean* const outValue)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getBoolean(inName, outValue);
    else
    {
        // Locate the field and get the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_BOOLEAN_MIME_TYPE);

            vField->read((char*)outValue, 0, sizeof(xam_boolean));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getInt(const xam_string inName, 
                           xam_int* const outValue)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getInt(inName, outValue);
    else
    {
        // Locate the field and get the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_INT_MIME_TYPE);

            vField->read((char*)outValue, 0, sizeof(xam_int));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getDouble(const xam_string inName, 
                             xam_double* const outValue)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getDouble(inName, outValue);
    else
    {
        // Locate the field and get the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_DOUBLE_MIME_TYPE);

            vField->read((char*)outValue, 0, sizeof(xam_double));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getXUID(const xam_string inName, 
                            xam_xuid* const outValue)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getXUID(inName, outValue);
    else
    {
        // Locate the field and get the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_XUID_MIME_TYPE);

            vField->read((char*)outValue, 0, sizeof(xam_xuid));
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getString(const xam_string inName, 
                              xam_string* const outValue)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getString(inName, outValue);
    else
    {
        // Locate the field and get the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_STRING_MIME_TYPE);

            // Include the NULL terminator if there's room in the buffer
            xam_int vSz = vField->getSize();
            if(vSz < sizeof(xam_string))
                vField->read((char*)outValue, 0, vSz + 1); 
            else
                vField->read((char*)outValue, 0, vSz); 
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::getDatetime(const xam_string inName, 
                                xam_datetime* const outValue)
{
    XAM_CriticalSection vCritical(&mMutex);

    // Use the method mapping if it exists, otherwise use this container
    XAM_SyntheticField* vSynthField = (XAM_SyntheticField*)0;
    if(mSyntheticFieldMap && (vSynthField = mSyntheticFieldMap->isMapped(inName)))
        vSynthField->getDatetime(inName, outValue);
    else
    {
        // Locate the field and get the value
        XAM_Field* vField = findField((char*)inName);
        if(vField)
        {
            XAM_Utils::validateMIMEType(vField->getMimeType(), 
                                        XAM_DATETIME_MIME_TYPE);

            // Include the NULL terminator if there's room in the buffer
            xam_int vSz = vField->getSize();
            if(vSz < sizeof(xam_datetime))
                vField->read((char*)outValue, 0, vSz + 1); 
            else
                vField->read((char*)outValue, 0, vSz); 
        }
        else
        {
            XAMFieldNotFoundException e;
            throw e;
        }
    }
}

void 
XAM_FieldContainer::createXStream(const xam_string inName, 
                                  const xam_boolean inBinding, 
                                  const xam_string inType,
                                  xstream_handle* const outXStream)
{
    // This field container does not support the persisting of fields
    // to storage, so a binding state is not allowed!
    if(inBinding)
    {
        XAMOperationNotAllowedException e;
        throw e;
    }

    if(XAM_POSIX::xam_strlen(inType) > 0)
    {
        XAM_CriticalSection vCritical(&mMutex);

        // Only create if the field doesn't exist
        if(!findField((char*)inName))
        {
            XAM_XStream* vXStream = new XAM_XStream((char*)inName, 
                                                    (char*)inType, 
                                                    inBinding, 
                                                    false);  

            addField(vXStream);

            vXStream->open(XSTREAM_MODE_WRITEONLY_TRUNCATE, outXStream);
        }
        else
        {
            XAMFieldExistsException e;
            throw e;
        }
    }
    else
    {
        XAMInvalidMimeTypeException e;
        throw e;
    }
}

void 
XAM_FieldContainer::openXStream(const xam_string inName, 
                                const xam_string inMode, 
                                xstream_handle* const outXStream)
{
    XAM_XStream* vStream = (XAM_XStream*)0;

    XAM_CriticalSection vCritical(&mMutex);
    vStream = (XAM_XStream*) findField((char*)inName);

     // Validate MIME type
     xam_string vMimeType = "";
     getFieldTypeHelper(inName, &vMimeType);
     XAM_Utils::assertXStreamType(vMimeType);

    if(vStream)
    {
        vStream->open(inMode, outXStream);
    }
    else
    {
        XAMFieldNotFoundException e;
        throw e;
    }
}

void
XAM_FieldContainer::asyncOpenXStream (const xam_string inName,
                                      const xam_string inMode,
                                      const XOPID inXOPID,
                                      xasync_callback inCallback,
                                      xasync_handle* const outAsyncHandle)
{
    // Create an XAsync object and spawn the async operation thread
    XAM_XAsync* vXAM_XAsync = new XAM_XAsync(inXOPID, 
                                             inCallback ? XAM_AsyncCallback : (xasync_callback) 0);
    vXAM_XAsync->openXStream((xam_handle_t)this, inName, inMode);
    *outAsyncHandle = (xasync_handle) vXAM_XAsync;
}

void 
XAM_FieldContainer::openFieldIterator(const xam_string inPattern, 
                                        xiterator_handle* const outHandle)
{
    XAM_FieldIterator* vIter = new XAM_FieldIterator(this, inPattern);
    if(!vIter)
    {
        XAMOutOfMemoryException e;
        throw e;
    }

    *outHandle = (xiterator_handle) vIter;
}





