#ifndef __XAM_FIELD_CONTAINER_H__
#define __XAM_FIELD_CONTAINER_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_Field.h"
#include "XAM_Property.h"
#include "XAM_Handle.h"
#include "XAM_XStream.h"
#include "XAM_Critical.h"
#include "XAM_Array.h"
#include "XAM_SyntheticFieldMap.h"
#include "XAM_FieldNameCache.h"

/** 
 *  A XAM_Field container class
 *
 *  All public methods must be thread safe!
 */
class XAM_FieldContainer : public XAM_Handle
{
    protected:
         XAM_Array<XAM_Field> mFieldArray;          // Holds all XAM_Field objects
         XAM_SyntheticFieldMap* mSyntheticFieldMap; // Mapping for synthetic fields
         XAM_Mutex mMutex;                          // Covers this container

         /**
          * Add a field to this container
          *
          * @param inField A pointer to the XAM_Field to add
          */
         void 
         addField(XAM_Field* inField); 

         /**
          * Find a field of a given name in this container
          *
          * @param inName The name of the field to locate
          * @return A pointer to the located XAM_Field or NULL if not found
          */
         XAM_Field* 
         findField(const char* inName);

         /**
          * Find a index of field with the given name in this container
          *
          * @param inName The name of the field to locate
          * @return An index into the field array where located or -1 if not found
          */
         const int
         findFieldIndex(const char* inName);
 
         /**
          * Throws an exception if the given field is read only
          * 
          * @param inField - The XAM_Field being checked for read only state
          * @throws XAMReadOnlyFieldException if the field is read only
          */
         void
         assertFieldIsWritable(XAM_Field* inField);

    public:    

        /** 
         * Default Constructor
         *
         * @param inMap Optional method mapping to use when deciding whether to
         *              use cached field values or method lookups
         */
        XAM_FieldContainer(XAM_SyntheticFieldMap* inMap = 0) 
            : XAM_Handle(0,0), 
              mSyntheticFieldMap(inMap), 
              mFieldArray(true, kXA_USE_DELETE) {}

        /**
         * Destructor
         */
        virtual 
        ~XAM_FieldContainer() {}

        /**
         * Adds all field names in this container to the cache object 
         *
         * @param inCache The cache object to populate with field names
         */
        virtual void
        cache(XAM_FieldNameCache* inCache);

        /*
         * The following calls map directly to their corresponding XAM API
         * methods, and are used for the equivalent field operations.
         */ 

         /** @see xam.h for detailed description */
         virtual void
         containsField (const xam_string inName,
                        xam_boolean* const outExists);

         /** @see xam.h for detailed description */
         virtual void 
         setFieldAsBinding(const xam_string inName);

         /** @see xam.h for detailed description */
         virtual void 
         setFieldAsNonbinding(const xam_string inName);

         /** @see xam.h for detailed description */
         virtual void 
         setFieldAsReadOnly(const xam_string inName);

         /** @see xam.h for detailed description */
         virtual void 
         getFieldType(const xam_string inName, xam_string* const outType);

         /** Helper method to get field type without acquiring the lock */
         virtual void 
         getFieldTypeHelper(const xam_string inName, xam_string* const outType);

         /** @see xam.h for detailed description */
         virtual void 
         getFieldLength(const xam_string inName, xam_int* const outLen);

         /** @see xam.h for detailed description */
         virtual void 
         getFieldBinding(const xam_string inName, xam_boolean* const outBinding);

         /** @see xam.h for detailed description */
         virtual void 
         getFieldReadOnly(const xam_string inName, xam_boolean* const outReadOnly);


         /** @see xam.h for detailed description */
         virtual void 
         deleteField(const xam_string inName);

         /** @see xam.h for detailed description */
         virtual void 
         createBoolean(const xam_string inName, 
                       const xam_boolean inBinding, 
                       const xam_boolean inValue);

         /** @see xam.h for detailed description */
         virtual void 
         createInt(const xam_string inName, 
                   const xam_boolean inBinding, 
                   const xam_int inValue);
 
         /** @see xam.h for detailed description */
         virtual void 
         createDouble(const xam_string inName, 
                     const xam_boolean inBinding, 
                     const xam_double inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         createXUID(const xam_string inName, 
                    const xam_boolean inBinding, 
                    const xam_xuid inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         createString(const xam_string inName, 
                      const xam_boolean inBinding, 
                      const xam_string inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         createDatetime(const xam_string inName, 
                        const xam_boolean inBinding, 
                        const xam_datetime inValue);

         /** @see xam.h for detailed description */
         virtual void 
         setBoolean(const xam_string inName, const xam_boolean inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         setInt(const xam_string inName, const xam_int inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         setDouble(const xam_string inName, const xam_double inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         setXUID(const xam_string inName, const xam_xuid inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         setString(const xam_string inName, const xam_string inValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         setDatetime(const xam_string inName, const xam_datetime inValue);

         /** @see xam.h for detailed description */
         virtual void 
         getBoolean(const xam_string inName, xam_boolean* const outValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         getInt(const xam_string inName, xam_int* const outValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         getDouble(const xam_string inName, xam_double* const outValue);
         
#ifdef SUN
        using XAM_Interface::getXUID; // add base class getXUID definitions
#endif
         /** @see xam.h for detailed description */
         virtual void 
         getXUID(const xam_string inName, xam_xuid* const outValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         getString(const xam_string inName, xam_string* const outValue);
         
         /** @see xam.h for detailed description */
         virtual void 
         getDatetime(const xam_string inName, xam_datetime* const outValue);

         /** @see xam.h for detailed description */
         virtual void 
         createXStream(const xam_string inName, 
                       const xam_boolean inBinding, 
                       const xam_string inType, 
                       xstream_handle* const outXStream);
         
         /** @see xam.h for detailed description */
         virtual void 
         openXStream(const xam_string inName,       
                     const xam_string inMode, 
                     xstream_handle* const outXStream);

        /** @see xam.h for detailed description */
        virtual void
        asyncOpenXStream (const xam_string inName,
                          const xam_string inMode,
                          const XOPID inXOPID,
                          xasync_callback inCallback,
                          xasync_handle* const outAsyncHandle);

         /** @see xam.h for detailed description */
         virtual void 
         openFieldIterator(const xam_string inPattern, 
                           xiterator_handle* const outHandle);
         
        /* These functions perform the same operations as their 
        equivalent set<Type> functions except that they don't 
        grab the container lock. */
        void setDatetimeHelper(const xam_string inName,
                               const xam_datetime inValue);
        void setStringHelper(const xam_string inName, 
                             const xam_string inValue);
        void setXUIDHelper(const xam_string inName, 
                           const xam_xuid inValue);
        void setDoubleHelper(const xam_string inName, 
                             const xam_double inValue);
        void setIntHelper(const xam_string inName, 
                          const xam_int inValue);
        void setBooleanHelper(const xam_string inName, 
                              const xam_boolean inValue);

        /* These functions perform the same operations as their 
        equivalent create<Type> functions except that they don't 
        grab the container lock. */
        void createDatetimeHelper(const xam_string inName,
                                  const xam_boolean inBinding,
                                  const xam_datetime inValue);
        void createStringHelper(const xam_string inName, 
                                const xam_boolean inBinding,
                                const xam_string inValue);
        void createXUIDHelper(const xam_string inName, 
                              const xam_boolean inBinding,
                              const xam_xuid inValue);
        void createDoubleHelper(const xam_string inName, 
                                const xam_boolean inBinding,
                                const xam_double inValue);
        void createIntHelper(const xam_string inName, 
                             const xam_boolean inBinding,
                             const xam_int inValue);
        void createBooleanHelper(const xam_string inName, 
                                 const xam_boolean inBinding,
                                 const xam_boolean inValue);


     private:

        /* Private copy constructor - disallow copies */
        XAM_FieldContainer( XAM_FieldContainer& ) : XAM_Handle(0,0) {}
};

#endif // __XAM_FIELD_CONTAINER_H__
