////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_FieldIterator.h"
#include "XAM_Exception.h"
#include "XAM_POSIX.h"

void
XAM_FieldIterator::hasNext(xam_boolean& outValue)
{
    outValue = (mAtIndex >= mFieldNamesCache->getSize()) ? false : true;
}

void 
XAM_FieldIterator::getNext(xam_string& outFieldName)
{
    char* vFieldName = "";

    // Return the next pattern match in the cache, or empty string when
    // there are no more fields found in the cache. 
    for(int vSz = mFieldNamesCache->getSize() ; mAtIndex < vSz ; mAtIndex++)
    {
        char* vTmpFieldName = 
                 (char*) mFieldNamesCache->getNameAtIndex(mAtIndex);

        if(vTmpFieldName)
        {
            vFieldName = vTmpFieldName;
            mAtIndex++;
            break;
        }
    }

    XAM_POSIX::xam_strcpy(outFieldName, vFieldName);
}

xam_boolean
XAM_FieldIterator::containsField(const xam_string inName)
{
    return mFieldNamesCache->exists(inName);
}

XAM_FieldIterator::XAM_FieldIterator(XAM_FieldContainer* inContainer, 
                                     const xam_string inPattern) 
    : mAtIndex(0)
{
    // Create a cache of the names for thread safe iteration
    mFieldNamesCache = new XAM_FieldNameCache(inPattern, XAM_POSIX::xam_strlen(inPattern));

    // Fill the cache with fields from the container
    inContainer->cache(mFieldNamesCache);
}

XAM_FieldIterator::~XAM_FieldIterator()
{
    if(mFieldNamesCache)
        delete mFieldNamesCache;
}
