#ifndef __XAM_SYNTHETIC_FIELD__
#define __XAM_SYNTHETIC_FIELD__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_Interface.h"

/**
 * Interface for converting field requests to method calls 
 *
 * Provides a field interface, but calls a method to get/set 
 * the values associated with the field instead of caching 
 * these values.
 */
class XAM_SyntheticField : public XAM_Interface
{   
    public:
    
        /**
         * Destructor
         */
        virtual
        ~XAM_SyntheticField() {}

        /**
         *  Determines whether or not this method applies to 
         *  the field of the given name.
         * 
         *  @param inName The name of the field to check
         *  @return true if this field is supported by this method, 
         *          false otherwise
         */
        virtual const xam_boolean 
        isMapped(const xam_string inName) = 0;

        /**
         * Adds all field names for this method to the cache object 
         *
         * @param inCache The cache object to populate with field names
         */
        virtual void
        cache(XAM_FieldNameCache* inCache) = 0;

        /*
         * Default implementations of the XAM_Interface methods used 
         * by derived classes
         */

        virtual void 
        setFieldAsBinding (const xam_string inName)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        setFieldAsNonbinding (const xam_string inName)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        setFieldAsReadOnly (const xam_string inName)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        getFieldType (const xam_string inName,
                      xam_string* const outType)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        virtual void 
        getFieldLength (const xam_string inName,
                        xam_int* const outLength)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        virtual void
        getFieldBinding (const xam_string /*inName*/,
                         xam_boolean* const outBinding)
        {
            // All synthetic fields are nonbinding 
            *outBinding = false;
        }

        virtual void 
        getFieldReadOnly (const xam_string inName,
                          xam_boolean* const outReadOnly)
        {
            // All synthetic fields are read only (by default, but may be overridden in derived class)
            *outReadOnly = true;
        }

        virtual void 
        setBoolean (const xam_string inName,
                    const xam_boolean inValue)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        setInt (const xam_string inName,
                const xam_int inValue)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        setDouble (const xam_string inName,
                  const xam_double inValue)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        setXUID (const xam_string inName,
                 const xam_xuid inValue)
        {
            XAMReadOnlyFieldException e; throw e;
        }

        virtual void 
        setString (const xam_string inName,
                   const xam_string inValue)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        setDatetime (const xam_string inName,
                     const xam_datetime inValue)
        {
            XAMReadOnlyFieldException e; 
            throw e;
        }

        virtual void 
        getBoolean (const xam_string inName,
                    xam_boolean* const outValue)
        {
            XAMFieldNotFoundException e; 
            throw e;
        }

        virtual void 
        getInt (const xam_string inName,
                xam_int* const outValue)
        {
            XAMFieldNotFoundException e; 
            throw e;
        }

        virtual void 
        getDouble (const xam_string inName,
                  xam_double* const outValue)
        {
            XAMFieldNotFoundException e; 
            throw e;
        }

#ifdef SUN
        using XAM_Interface::getXUID; // add base class getXUID definitions
#endif
        virtual void 
        getXUID (const xam_string inName,
                 xam_xuid* const outValue)
        {
            XAMFieldNotFoundException e; 
            throw e;
        }

        virtual void 
        getString (const xam_string inName,
                   xam_string* const outValue)
        {
            XAMFieldNotFoundException e; 
            throw e;
        }

        virtual void 
        getDatetime (const xam_string inName,
                     xam_datetime* const outValue)
        {
            XAMFieldNotFoundException e; 
            throw e;
        }
};

#endif // __XAM_SYNTHETIC_FIELD__







