#ifndef __XAM_SYNTHETICFIELD_MAP__
#define __XAM_SYNTHETICFIELD_MAP__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"

#include "XAM_Array.h"
#include "XAM_Critical.h"
#include "XAM_FieldNameCache.h"
#include "XAM_SyntheticField.h"

/**
 * Abstract class for mapping field API requests to a specific method invocation.
 *
 * Derived classes are responsible for populating the mMappedFieldNames array
 * (with the names of fields which will be mapped to a method) as well as 
 * implementing the XAM_Interface methods which map the acutal call to the 
 * underlying method implementation.
 */
class XAM_SyntheticFieldMap
{
    protected:
        XAM_Array<XAM_SyntheticField> mSynthFields;  // Array of synthetic fields        
        XAM_Mutex mSynthFieldsMutex;  // Covers the synthentic field array

    public:

        /**
         * Destructor
         */
        virtual
        ~XAM_SyntheticFieldMap() {}

        /**
         * Registers a XAM_SyntheticField with the map
         *
         * @param inMethod The method to add
         */
        virtual void
        registerSyntheticField(XAM_SyntheticField* inMethod)
        {
            XAM_CriticalSection vCritical(&mSynthFieldsMutex);
            mSynthFields.push_back(inMethod);
        }            

        /**
         *  Determines whether or not this synthetic field mapping supports 
         *  the field of the given name.
         * 
         *  @param inName The name of the field to check for mapped support
         *  @return The field method if mapped, or NULL if not mapped
         */
        virtual XAM_SyntheticField* 
        isMapped(const xam_string inName)
        {
            XAM_SyntheticField* vSynthFieldFound = (XAM_SyntheticField*) 0;

            XAM_CriticalSection vCritical(&mSynthFieldsMutex);

            int vSz = mSynthFields.size();
            for(int i=0 ; i < vSz ; i++)
            {
                XAM_SyntheticField* vSynthField = mSynthFields[i];
                if(vSynthField && vSynthField->isMapped(inName))
                {
                    vSynthFieldFound = vSynthField;
                    break;
                }
            }

            return vSynthFieldFound;
        }

        /**
         * Adds all field names in this map to the cache object 
         *
         * @param inCache The cache object to populate with field names
         */
        virtual void
        cache(XAM_FieldNameCache* inCache)
        {
            XAM_CriticalSection vCritical(&mSynthFieldsMutex);

            int vSz = mSynthFields.size();
            for(int i=0 ; i < vSz ; i++)
            {
                XAM_SyntheticField* vSynthField = mSynthFields[i];
                if(vSynthField)
                    vSynthField->cache(inCache);
            }
        }

};



#endif // __XAM_SYNTHETICFIELD_MAP__