#ifndef __XAM_SYNTHETIC_LOGGING_FIELDS__
#define __XAM_SYNTHETIC_LOGGING_FIELDS__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_SyntheticField.h"
#include "XAM_LogFactory.h"

/**
 * Defines the XAM_SyntheticField interfaces for all logging fields
 */
class XAM_SyntheticLoggingFields : public XAM_SyntheticField
{   
    public:

        /**
         * Constructor
         */        
        XAM_SyntheticLoggingFields() {}
    
        /**
         * Destructor
         */
        virtual
        ~XAM_SyntheticLoggingFields() {}

        /**
         *  Determines whether or not this method applies to 
         *  the field of the given name.
         * 
         *  @param inName The name of the field to check
         *  @return true if this field is supported by this method, 
         *          false otherwise
         */
        virtual const xam_boolean 
        isMapped(const xam_string inName)
        {
            xam_boolean vMapped = false;

            if((XAM_POSIX::xam_strcmp(inName, XAM_LOG_LEVEL) == 0)   ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_VERBOSITY) == 0) ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_PATH) == 0)    ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_FORMAT) == 0)  ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_APPEND) == 0)  ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_SIZE) == 0)      ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_ROLLOVERS) == 0) ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MSG_FILTER) == 0)    ||
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_COMP_FILTER) == 0))
            {
                vMapped = true;
            }

            return vMapped;
        }

        /**
         * Adds all field names for this method to the cache object 
         *
         * @param inCache The cache object to populate with field names
         */
        virtual void
        cache(XAM_FieldNameCache* inCache)
        {
            inCache->addField(XAM_LOG_LEVEL);
            inCache->addField(XAM_LOG_VERBOSITY);
            inCache->addField(XAM_LOG_PATH);
            inCache->addField(XAM_LOG_FORMAT);
            inCache->addField(XAM_LOG_APPEND);
            inCache->addField(XAM_LOG_MAX_SIZE);
            inCache->addField(XAM_LOG_MAX_ROLLOVERS);
            inCache->addField(XAM_LOG_MSG_FILTER);
            inCache->addField(XAM_LOG_COMP_FILTER);
        }

        /** 
         * Handles all getFieldType requests this map has registered for
         */
        virtual void
        getFieldType(const xam_string inName,
                     xam_string* const outType)
        {
            // xam_int
            if((XAM_POSIX::xam_strcmp(inName, XAM_LOG_LEVEL) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_VERBOSITY) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_FORMAT) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_SIZE) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_ROLLOVERS) == 0))
            {
                // Retrieve the mime type of this field
                XAM_POSIX::xam_strcpy(*outType, XAM_INT_MIME_TYPE);
            }
            // xam_string
            else if((XAM_POSIX::xam_strcmp(inName, XAM_LOG_PATH) == 0) || 
                    (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MSG_FILTER) == 0) || 
                    (XAM_POSIX::xam_strcmp(inName, XAM_LOG_COMP_FILTER) == 0))
            {
                // Retrieve the mime type of this field
                XAM_POSIX::xam_strcpy(*outType, XAM_STRING_MIME_TYPE);
            }
            // xam_boolean
            else if(XAM_POSIX::xam_strcmp(inName, XAM_LOG_APPEND) == 0)
            {
                // Retrieve the mime type of this field
                XAM_POSIX::xam_strcpy(*outType, XAM_BOOLEAN_MIME_TYPE);
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }
        
        /** 
         * Handles all getFieldReadOnly requests this map has registered for
         */
        virtual void 
        getFieldReadOnly (const xam_string inName,
                          xam_boolean* const outReadOnly)
        {
            // All logging synthetic fields may be modified (and are not read only)
            *outReadOnly = false;
        }

        /** 
         * Handles all getFieldLength requests this map has registered for
         */
        virtual void 
        getFieldLength (const xam_string inName,
                        xam_int* const outLength)
        {
            // xam_int
            if((XAM_POSIX::xam_strcmp(inName, XAM_LOG_LEVEL) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_VERBOSITY) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_FORMAT) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_SIZE) == 0) || 
               (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_ROLLOVERS) == 0))
            {
                *outLength = sizeof(xam_int);
            }
            // xam_string
            else if((XAM_POSIX::xam_strcmp(inName, XAM_LOG_PATH) == 0) || 
                    (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MSG_FILTER) == 0) || 
                    (XAM_POSIX::xam_strcmp(inName, XAM_LOG_COMP_FILTER) == 0))
            {
                xam_string vStr;
                getString(inName, &vStr);

                *outLength = XAM_POSIX::xam_strlen(vStr);
            }
            // xam_boolean
            else if(XAM_POSIX::xam_strcmp(inName, XAM_LOG_APPEND) == 0)
            {
                *outLength = sizeof(xam_boolean);
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all setInt requests this map has registered for
         */
        virtual void 
        setInt (const xam_string inName,
                const xam_int inValue)
        {
            if(XAM_POSIX::xam_strcmp(inName, XAM_LOG_LEVEL) == 0)
            {
                // Apply the log level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setLevel(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
                XAM_Logger::initLogging(false);
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_VERBOSITY) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setVerbosityLevel(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
                XAM_Logger::initLogging(false);
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_FORMAT) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setFormat(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_SIZE) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setMaxSizeInKb(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_ROLLOVERS) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setMaxRollovers(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all getInt requests this map has registered for
         */
        virtual void 
        getInt (const xam_string inName,
                xam_int* const outValue)
        {
            if(XAM_POSIX::xam_strcmp(inName, XAM_LOG_LEVEL) == 0)
            {
                // Retrieve the log level state from the logging framework
                *outValue = XAM_Logger::mLogFactory.getLevel();
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_VERBOSITY) == 0)
            {
                // Retrieve the debug level state from the logging framework
                *outValue = XAM_Logger::mLogFactory.getVerbosityLevel();
            }        
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_FORMAT) == 0)
            {
                *outValue = XAM_Logger::mLogFactory.getFormat();
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_SIZE) == 0)
            {
                *outValue = XAM_Logger::mLogFactory.getMaxSizeInKb();
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MAX_ROLLOVERS) == 0)
            {
                *outValue = XAM_Logger::mLogFactory.getMaxRollovers();
            }
            else
            {
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all setString requests this map has registered for
         */
        virtual void 
        setString (const xam_string inName,
                   const xam_string inValue)
        {
            if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_PATH) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setPath(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MSG_FILTER) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setMsgFilter(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_COMP_FILTER) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setCompFilter(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all getString requests this map has registered for
         */
        virtual void 
        getString (const xam_string inName,
                   xam_string* const outValue)
        {   
            if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_PATH) == 0)
            {
                const char* vLogPath = XAM_Logger::mLogFactory.getLogPath();

                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_POSIX::xam_strcpy(*outValue, vLogPath ? vLogPath : "");
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_MSG_FILTER) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_POSIX::xam_strcpy(*outValue, 
                                      XAM_Logger::mLogFactory.getMsgFilter());
            }
            else if (XAM_POSIX::xam_strcmp(inName, XAM_LOG_COMP_FILTER) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_POSIX::xam_strcpy(*outValue, 
                                      XAM_Logger::mLogFactory.getCompFilter());
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all setBoolean requests this map has registered for
         */
        virtual void 
        setBoolean (const xam_string inName,
                    const xam_boolean inValue)
        {
            if(XAM_POSIX::xam_strcmp(inName, XAM_LOG_APPEND) == 0)
            {
                // Apply the debug level state change
                XAM_CriticalSection vCritical(&XAM_Logger::mLoggerLock);
                XAM_Logger::mLogFactory.setAppend(inValue);
                XAM_Logger::mLogFactory.applyOptions(0);
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all getBoolean requests this map has registered for
         */
        virtual void 
        getBoolean (const xam_string inName,
                    xam_boolean* const outValue)
        {   
            if(XAM_POSIX::xam_strcmp(inName, XAM_LOG_APPEND) == 0)
            {
                // Apply the debug level state change
                *outValue = XAM_Logger::mLogFactory.getAppend();
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

};

#endif // __XAM_SYNTHETIC_LOGGING_FIELDS__







