////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_strings.h"
#include "XAM_XStream.h"
#include "XAM_XStream_Instance.h"
#include "XAM_Exception.h"
#include "XAM_POSIX.h"

void 
XAM_XStream::open(const char* inMode, xstream_handle* const ioHandle)
{
    if(XAM_POSIX::xam_strcmp(inMode, XSTREAM_MODE_READ_ONLY) != 0)
    {
        // Reject the open for write/append if the field itself is read only
        if(XAM_Field::getReadOnly())
        {
            XAMReadOnlyFieldException e;
            throw e;
        }

        // Acquire the WRITE lock if we're opening for write/append.
        // This blocks other callers who may be attempting to open 
        // the same field for write.
        holdWriteLock();
    }

    try
    {
        if(XAM_POSIX::xam_strcmp(inMode, XSTREAM_MODE_WRITEONLY_TRUNCATE) == 0)
        {
            // Truncate all existing stream data
            reset();

            // Create an instance from the starting position
            *ioHandle = (xstream_handle) 
                new XAM_XStream_Instance(this, false, 0, FROM_START);
        }
        else if(XAM_POSIX::xam_strcmp(inMode, XSTREAM_MODE_WRITEONLY_APPEND) == 0)
        {
            // Create an instance from the end position
            *ioHandle = (xstream_handle)
                new XAM_XStream_Instance(this, false, 0, FROM_END);
        }
        else if(XAM_POSIX::xam_strcmp(inMode, XSTREAM_MODE_READ_ONLY) == 0)
        {
            // Create an instance from the starting position
            *ioHandle = (xstream_handle)
                new XAM_XStream_Instance(this, true, 0, FROM_START);
        }
        else
        {
            XAMInvalidStreamModeException e;
            throw e;
        }
    }
    catch(XAM_Exception &)
    {
        // Release the write lock on error
        releaseWriteLock();

        // Rethrow the original exception
        throw;    
    }
}

void 
XAM_XStream::reset()
{
    if(mReadOnly == false)    
        XAM_Field::reset();    
    else
    {
        XAMReadOnlyFieldException e;
        throw e;
    }
}

void
XAM_XStream::holdWriteLock()
{
    if(mWriteLockAcquired)
    {
        XAMFieldInUseException e;
        throw e;
    }
    mWriteLockAcquired = true;
    mWriteLock.lock();
}

void 
XAM_XStream::assertNotCorrupted()
{
    if ( mCorrupted )
    {
        XAMCorruptException e(XSTREAM_HANDLE);
        throw e;
    }
}

void 
XAM_XStream::releaseWriteLock()
{
    if(mWriteLockAcquired)
    {
        mWriteLock.unlock();
        mWriteLockAcquired = false;
    }
}

XAM_XStream::XAM_XStream(const char* inName, 
                         const char* inMimeType, 
                         const bool inBinding,
                         const bool inReadOnly)
    : XAM_Field(inName, inMimeType, inBinding, inReadOnly, 0, 0),  
      mWriteLockAcquired(false),
      mCorrupted(false),
      mOpenInstances(0)
{   
}   


