////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_XStream.h"
#include "XAM_XStream_Instance.h"
#include "XAM_Exception.h"
#include "XAM_POSIX.h"

void
XAM_XStream_Instance::close()
{
    // Release the WRITE lock if we had opened for write/append
    mXStream->releaseWriteLock();
    mXStream->mOpenInstances--;

    delete this;
}

void 
XAM_XStream_Instance::assertNotAbandoned()
{
    if ( mAbandoned )
    {
        XAMAbandonedException e(XSTREAM_HANDLE);
        throw e;
    }
}

void 
XAM_XStream_Instance::assertReadOnlyMode()
{    
    if(mReadOnlyMode == false)
    {
        XAMInvalidStreamModeException e;
        throw e;
    }
}

void 
XAM_XStream_Instance::assertWriteOnlyMode()
{     
    if(mReadOnlyMode == true)
    {
        XAMInvalidStreamModeException e;
        throw e;
    }
}

void 
XAM_XStream_Instance::seek(const xam_int inOffset, 
                           const XStream_Whence inWhence,
                           xam_boolean inCheckReadOnlyState)
{
    assertNotAbandoned();
    mXStream->assertNotCorrupted();

    // Can only seek if the stream was opened for "read"
    if(inCheckReadOnlyState)
        assertReadOnlyMode();

    switch(inWhence)
    {
        case FROM_START:
        {
            // Check for invalid offset parameters
            if((inOffset < 0) || (inOffset > mXStream->mDataSizeInBytes))
            {
                XAMInvalidParamException e;
                throw e;
            }

            // Set this position
            mDataPos = inOffset;
            break;
        }
        case FROM_CURRENT:
        {
            // Check for invalid offset parameters
            if(((inOffset + mDataPos) < 0) || ((inOffset + mDataPos) > mXStream->mDataSizeInBytes))
            {
                XAMInvalidParamException e;
                throw e;
            }

            // Set this position
            mDataPos += inOffset;
            break;
        }
        case FROM_END:
        {
            // Check for invalid offset parameters
            if((inOffset > 0) || ((inOffset * -1) > mXStream->mDataSizeInBytes))
            {
                XAMInvalidParamException e;
                throw e;
            }

            // Set this position
            mDataPos = mXStream->mDataSizeInBytes + inOffset;
            break;
        }
        default:
        {
            XAMInvalidParamException e;
            throw e;
        }
    }

    // Check for EOF state
    if(mDataPos == mXStream->mDataSizeInBytes)
        mAtEOF = true;
    else
        mAtEOF = false;
}

void 
XAM_XStream_Instance::tell(xam_int* const outPosition)
{
    assertNotAbandoned();
    mXStream->assertNotCorrupted();

    *outPosition = mDataPos;
}

const xam_int 
XAM_XStream_Instance::write(const char* inData, const xam_int inDataSizeInBytes)
{
    assertNotAbandoned();
    mXStream->assertNotCorrupted();

    // Can only seek if the stream was opened for "read"
    assertWriteOnlyMode();

    xam_int vWritten = 0;

    vWritten = mXStream->write(inData, mDataPos, inDataSizeInBytes);
    mDataPos += vWritten;

    return vWritten;
}

const xam_int 
XAM_XStream_Instance::read(char* const outData, 
                           const xam_int inDataSizeInBytes)
{
    assertNotAbandoned();
    mXStream->assertNotCorrupted();

    // Can only read if the stream was opened for "read"
    assertReadOnlyMode();

    xam_int vRead = 0;

    // If there's no data, we're at EOF!
    if(mXStream->mDataSizeInBytes == 0)
        mAtEOF = true;

    if(mAtEOF == true)
    {
        vRead = -1;
    }
    else
    {        
        if(!mAtEOF)
        {
            vRead = mXStream->read(outData, mDataPos, inDataSizeInBytes);
            mDataPos += vRead;    
        }

        if(mDataPos >= mXStream->mDataSizeInBytes)
            mAtEOF = true;
    }

    return vRead;
}

XAM_XStream_Instance::XAM_XStream_Instance(XAM_XStream* inXStream,
                                           xam_boolean inReadOnly, 
                                           xam_int inDataPos,
                                           XStream_Whence inWhence)
: mXStream(inXStream),
  mReadOnlyMode(inReadOnly),
  mAtEOF(false),
  mAbandoned(false),
  mDataPos(inDataPos) 
{
    seek(inDataPos, inWhence, false);
    mXStream->mOpenInstances++;
}

