#ifndef __XAM_XSTREAM_INSTANCE_H__
#define __XAM_XSTREAM_INSTANCE_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_XStream.h"

/**
 * XStream Instance Class
 */
class XAM_XStream_Instance
{        
    private:
        XAM_XStream* mXStream;  // Pointer to the parent XStream field

        xam_int mDataPos;          // Position in the data for r/w
        xam_boolean mAtEOF;        // Indicator for end of stream
        xam_boolean mAbandoned;    // If true, the instance is abandoned
        xam_boolean mReadOnlyMode; // If true, the instance is readonly

    public:

        /**
         * Constructor
         *
         * @param inXStream The XStream to open the instance on
         * @param inReadOnly true for readonly access, false for write
         * @param inDataPos The seek position count
         * @param inWhence The whence position to seek from
         */
        XAM_XStream_Instance(XAM_XStream* inXStream,
                             xam_boolean inReadOnly, 
                             xam_int inDataPos,
                             XStream_Whence inWhence);

        /**
         * Abandon this stream instance
         */
        void
        abandon()
        {
            mAbandoned = true;
        }

        /**
         * Close this stream instance
         */
        void 
        close(); 

        /**
         * Convenience method to verify that this XStream is 
         * XSTREAM_MODE_READ_ONLY
         * 
         * @throws XAMInvalidStreamModeException
         */
        void 
        assertReadOnlyMode();

        /*
         * Throws corresponding exception if the XStream instance 
         * is abandoned
         */
        void
        assertNotAbandoned();

        /**
         * Convenience method to verify that this XStream is 
         * XSTREAM_MODE_WRITEONLY_TRUNCATE or 
         * XSTREAM_MODE_WRITEONLY_APPEND
         * 
         * @throws XAMInvalidStreamModeException
         */
        void 
        assertWriteOnlyMode();

        /**
         * Move the stream position to the offset from the whence 
         * position
         *
         * @param inOffset  The offset into the stream to move
         * @param inWhence  Defines the starting position 
         */
        void 
        seek(const xam_int inOffset, 
             const XStream_Whence inWhence,
             xam_boolean inCheckReadOnlyState = true);

        /**
         * Get the current stream position
         *
         * @param outPosition The current position is written to 
         *                    this address
         */
        void 
        tell(xam_int* const outPosition);

        /**
         * Write data to this stream
         *
         * @param inData             Pointer to the buffer of data 
         *                           being written
         * @param inDataSizeInBytes  The number of bytes in the buffer 
         *                           to be written
         * @return                   The number of bytes successfully 
         *                           written to the stream (will be <= 
         *                           inDataSizeInBytes)
         */
        const xam_int 
        write(const char* inData, const xam_int inDataSizeInBytes);

        /**
         * Read data from this stream
         *
         * @param outData            Pointer to allocated space where the 
         *                           data will be written
         * @param inDataSizeInBytes  The number of bytes available in the 
         *                           allocated space
         * @return                   The actual number of bytes written to 
         *                           outData (will be <= inDataSizeInBytes)
         */
        const xam_int 
        read(char* const outData, const xam_int inDataSizeInBytes);
};


#endif // __XAM_XSTREAM_INSTANCE_H__

 
 

