#ifndef __XAM_HANDLE_WRAPPER__
#define __XAM_HANDLE_WRAPPER__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"

#include "XAM_Exception.h"
#include "XAM_Interface.h"
#include "XAM_VIM.h"
#include "XAM_Utils.h"
#include "XAM_Logger.h"

/** 
 *  Base wrapper class for xam_handle_t types 
 */
class XAM_Handle : public XAM_Interface
{
    protected:
        xam_handle_t mHandle;  // Wrapped xam_handle
        XAM_Handle* mParent;   // Parent handle (if any)
        XAM_VIM* mVim;         // Associated VIM (if any)
        XAMObjType mType;      // Type of xam_handle

    public:
        /**
         * Default Constructor
         *
         * @param inHandle The xam_handle to represent as an object
         * @param inParent The parent handle (or NULL if none)
         * @param inVim The associated VIM (or NULL to use the parent's 
         *              VIM, if any)
         */
        XAM_Handle(const xam_handle_t inHandle, 
                   XAM_Handle* inParent, 
                   XAM_VIM* inVim = 0)
            : mHandle(inHandle), 
              mParent(inParent), 
              mVim(inVim ? inVim : (inParent ? inParent->getVim() : 0)),
              mType(GENERIC_HANDLE) 
        {
            // If the handle was not specified, use this object
            if(inHandle == (xam_handle_t)0)
                mHandle = (xam_handle_t)this;
        }

        /** Destructor */
        virtual 
        ~XAM_Handle() {}

        /**
         * Retrieve the xam_handle_t wrapped by this object
         */
        virtual const xam_handle_t  
        getHandle() 
        { 
            return mHandle; 
        }
        
        /**
         * Modify the xam_handle_t wrapped by this object
         */
        virtual void
        setHandle(xam_handle_t* inHandle)
        {
            mHandle = *inHandle;
        }

        /**
         * Retrieve the parent handle of this object
         */
        XAM_Handle*   
        getParent() 
        { 
            return mParent; 
        }
        
       /**
        * Retrieve the VIM associated with this object
        */
        XAM_VIM*      
        getVim() 
        { 
            return mVim; 
        }
        
       /**
        * Retrieve this object's type
        */
        const XAMObjType    
        getType() 
        { 
            return mType; 
        }

       /**
        * Retrieve this object's type as a string
        */
        virtual const char*    
        getTypeString() 
        { 
            char* vRetVal = XAM_Utils::getStaticMemoryReference(12);

            switch(mType) 
            {
            case XAM_OBJECT_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal, "XAM_LIBRARY");
                break;
            case XSYSTEM_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal, "XSYSTEM");
                break;
            case XITERATOR_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal, "XITERATOR");
                break;
            case XSET_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal, "XSET");
                break;
            case XASYNC_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal, "XASYNC");
                break;
            case XSTREAM_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal, "XSTREAM");
                break;
            case GENERIC_HANDLE: 
                XAM_POSIX::xam_strcpy(vRetVal,  "GENERIC");
                break;
            default:
                XAM_POSIX::xam_strcpy(vRetVal, "<unknown>");
            }

            return vRetVal;
        }

        /**
         * Assert that this handle is of a given type
         *
         * @param inType The type of handle expected 
         * @throws XAMInvalidHandleException if this handle's 
         *         type does not match inType 
         */
        void 
        assertType(const XAMObjType inType)
        {
            if(mType != inType)
            {
                XAMInvalidHandleException e;
                throw e;
            }
        }

        /**
         * Notify this object that an Asynchronous operation is complete!
         * (only valid for XAsync handles, but needs to be exposed in a 
         *  generic way on XAM_Handle types)
         *
         * @throws XAMInvalidHandleException for all but XAsync handles
         */         
        virtual void
        notifyComplete()
        {
            XAMInvalidHandleException e;
            throw e;
        }

    private:
        /* Private copy constructor - disallow copies */
        XAM_Handle ( XAM_Handle& ) {}
};

#endif // __XAM_HANDLE_WRAPPER__



