#ifndef __XAM_HANDLE_MANAGER__
#define __XAM_HANDLE_MANAGER__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_Handle.h"
#include "XAM_Object.h"
#include "XAM_VIM_Manager.h"
#include "XAM_XAsyncHandler.h"
#include "XAM_Logger.h"

#include "XSystem.h"
#include "XIterator.h"
#include "XSet.h"
#include "XAsync.h"
#include "XStream.h"

/** 
 *  Scopes XAM_VIM objects utilizing XAM_Handle objects
 */
class XAM_HandleManager : public XAM_VIM_Manager
{
    public:    
    
        /**
         * Given a xam_handle_t, returns the equivalent XAM_Handle wrapper
         * @throws XAMInvalidHandleException if the handle is invalid
         */
        static XAM_Handle*
        getHandle(const xam_handle_t inHandle)
        {
            // Check for invalid handles
            if(inHandle == 0)
            {
                XAM_LOG_DEBUG_PLUS(component_xam_debug,
                                   "XAM_HandleManager::getHandle(00000000) -> Invalid!");

                XAMInvalidHandleException e;
                throw e;
            }

            // Convert the handle
            XAM_Handle* vHandle = 
                ((inHandle == XAM_LIBRARY_HANDLE) ? XAM_Object::getInstance() : ((XAM_Handle*) inHandle));

            // Log the handle value
            if(inHandle == (xam_handle_t) vHandle)
            {
                XAM_LOG_DEBUG_PLUS_P3(component_xam_debug,
                                      "XAM_HandleManager::getHandle(%"XAM_INT_FORMAT"d) -> (%s)", 
                                      inHandle,
                                      vHandle->getTypeString());
            }
            else
            {
                XAM_LOG_DEBUG_PLUS_P4(component_xam_debug,
                                      "XAM_HandleManager::getHandle(%"XAM_INT_FORMAT"d) -> (%s, %"XAM_INT_FORMAT"d)", 
                                      inHandle,
                                      vHandle->getTypeString(),
                                      (xam_handle_t) vHandle);
            }

            return vHandle;
        }

        /**
         * Creates an XIterator handle
         *
         * @param inHandle The handle to wrap with it's XAM_Handle equivalent 
         * @param inParent The parent XAM_Handle
         * @return An allocated XIterator handle wrapper 
         *        (use deleteHandle to dealloc)
         */
        static XIterator* 
        createXIteratorHandle(const xiterator_handle inHandle, 
                              XAM_Handle* inParent)
        {
            XIterator* vIter = new XIterator(inHandle, inParent);

            XAM_LOG_DEBUG_PLUS_P4(component_xam_debug,
                                  "XAM_HandleManager::createXIteratorHandle(%"XAM_INT_FORMAT"d, %p) -> (%"XAM_INT_FORMAT"d)", 
                                  inHandle, inParent, (xam_handle_t)vIter);
            return vIter;
        }

        /**
         * Creates an XSystem handle
         *
         * @param inHandle The handle to wrap with it's XAM_Handle equivalent 
         * @param inVIM The VIM from which this XSystem was originally created
         * @return An allocated XSystem handle wrapper 
         *        (use deleteHandle to dealloc)
         */
        static XSystem* 
        createXSystemHandle(const xsystem_handle inHandle, 
                            XAM_VIM* inVIM)
        {
            XSystem* vXSystem = new XSystem(inHandle, inVIM);

            XAM_LOG_DEBUG_PLUS_P4(component_xam_debug,
                                  "XAM_HandleManager::createXSystemHandle(%"XAM_INT_FORMAT"d, %p) -> (%"XAM_INT_FORMAT"d)", 
                                  inHandle, inVIM, (xam_handle_t)vXSystem);
            return vXSystem;
        }

        /**
         * Creates an XSet handle
         *
         * @param inHandle The handle to wrap with it's XAM_Handle equivalent 
         * @param inParent The parent XAM_Handle
         * @return An allocated XSystem handle wrapper
         *        (use deleteHandle to dealloc)
         */
        static XSet* 
        createXSetHandle(const xset_handle inHandle,
                         XAM_Handle* inParent)
        {
            XSet* vXSet = new XSet(inHandle, inParent);

            XAM_LOG_DEBUG_PLUS_P4(component_xam_debug,
                                  "XAM_HandleManager::createXSetHandle(%"XAM_INT_FORMAT"d, %p) -> (%"XAM_INT_FORMAT"d)", 
                                  inHandle, inParent, (xam_handle_t)vXSet);
            return vXSet;
        }

        /**
         * Creates an XStream handle
         *
         * @param inHandle The handle to wrap with it's XAM_Handle equivalent 
         * @param inParent The parent XAM_Handle
         * @return An allocated XSystem handle wrapper 
         *         (use deleteHandle to dealloc)
         */
        static XStream* 
        createXStreamHandle(const xstream_handle inHandle, 
                            XAM_Handle* inParent)
        {
            XStream* vXStream = new XStream(inHandle, inParent);

            XAM_LOG_DEBUG_PLUS_P4(component_xam_debug,
                                  "XAM_HandleManager::createXStreamHandle(%"XAM_INT_FORMAT"d, %p) -> (%"XAM_INT_FORMAT"d)", 
                                  inHandle, inParent, (xam_handle_t)vXStream);
            return vXStream;
        }

        /**
         * Creates an XAsync handle
         *
         * @param inCallback The user callback
         * @param inXOPID The XAsync operation unique identifier
         * @param inParent The parent XAM_Handle
         * @return An allocated XSystem handle wrapper 
         *        (use deleteHandle to dealloc)
         * @throws XAMInvalidParamException if the XOPID is not unique
         *
         * NOTE: The caller MUST subsequently populate the returned XAsync
         *       object with a valid xasync_handle using the setHandle
         *       method!
         */
        static XAsync* 
        createXAsyncHandle(xasync_callback inCallback,
                           XAM_Handle* inParent)
        {
            XAsync* vXAsync = new XAsync(inCallback, inParent);

            XAM_LOG_DEBUG_PLUS_P4(component_xam_debug,
                                  "XAM_HandleManager::createXAsyncHandle(%p, %p) -> (%p)", 
                                  inCallback, inParent, (XAM_Handle*)vXAsync);

            return vXAsync;
        }

        /**
         * Deletes a XAM_Handle
         *
         * @param inHandle The XAM_Handle to deallocate 
         */
        static void
        deleteHandle(XAM_Handle* inHandle)
        {
            XAM_LOG_DEBUG_PLUS_P2(component_xam_debug,
                                  "XAM_HandleManager::deleteHandle(%"XAM_INT_FORMAT"d)", 
                                  (xam_handle_t)inHandle);

            // Do not allow the deletion of the XAM_Object handle
            // All other handle types may be deleted
            if(inHandle && (inHandle->getType() != XAM_OBJECT_HANDLE))
                delete inHandle;
        }

};

#endif // __XAM_HANDLE_MANAGER__

