#ifndef __XAM_INTERFACE_H__
#define __XAM_INTERFACE_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_errors.h"
#include "XAM_Exception.h"

/** 
 *  Interface class for all XAM Objects defining methods corresponding to 
 *  each XAM API call.
 */
class 
XAM_Interface
{
    public:
        // ================================================================
        // methods for evaluating xam_status 
        // ================================================================

        /** @see xam.h for detailed description */
        virtual xam_boolean
        getErrorToken(const xam_status inStatusCode, 
                      xam_string* const outToken)
        {
            XAMInvalidHandleException e;
            throw e;
        }

        // ================================================================
        // methods for deallocating/abandoning handles 
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        abandon ()
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        close ()
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for the XIterator 
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        openFieldIterator (const xam_string inPattern,
                           xiterator_handle* const outIterator)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        hasNext (xam_boolean* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        next (xam_string* const outName)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for managing XAM Fields (properties or XStreams)
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        containsField (const xam_string inName,
                       xam_boolean* const outExists)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setFieldAsBinding (const xam_string inName)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setFieldAsNonbinding (const xam_string inName)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setFieldAsReadOnly (const xam_string inName)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getFieldType (const xam_string inName,
                      xam_string* const outType)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getFieldLength (const xam_string inName,
                        xam_int* const outLength)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getFieldBinding (const xam_string inName,
                         xam_boolean* const outFixed)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getFieldReadOnly (const xam_string inName,
                          xam_boolean* const outReadOnly)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        deleteField (const xam_string inName)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for managing property fields
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        createBoolean (const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_boolean inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        createInt (const xam_string inName,
                   const xam_boolean inBinding,
                   const xam_int inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        createDouble (const xam_string inName,
                     const xam_boolean inBinding,
                     const xam_double inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        createXUID (const xam_string inName,
                    const xam_boolean inBinding,
                    const xam_xuid inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        createString (const xam_string inName,
                      const xam_boolean inBinding,
                      const xam_string inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        createDatetime (const xam_string inName,
                        const xam_boolean inBinding,
                        const xam_datetime inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setBoolean (const xam_string inName,
                    const xam_boolean inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setInt (const xam_string inName,
                const xam_int inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setDouble (const xam_string inName,
                  const xam_double inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setXUID (const xam_string inName,
                 const xam_xuid inValue)
        {
            XAMInvalidHandleException e; throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setString (const xam_string inName,
                   const xam_string inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setDatetime (const xam_string inName,
                     const xam_datetime inValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getBoolean (const xam_string inName,
                    xam_boolean* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getInt (const xam_string inName,
                xam_int* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getDouble (const xam_string inName,
                  xam_double* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getXUID (const xam_string inName,
                 xam_xuid* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getString (const xam_string inName,
                   xam_string* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getDatetime (const xam_string inName,
                     xam_datetime* const outValue)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for XStreams and XStream fields
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        createXStream (const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_string inType,
                       xstream_handle* const outXStream)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        openXStream (const xam_string inName,
                     const xam_string inMode,
                     xstream_handle* const outXStream)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        read (char* const ioBuffer,
              const xam_int inBufferLength,
              xam_int* const outBytesRead)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        write (const char* inBuffer,
               const xam_int inByteCount,
               xam_int* const outByteWritten)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        seek (const xam_int inOffset,
              const xam_int inWhence)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        tell (xam_int* const outPosition)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for managing the connection to a XAM storage system
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        connect (const xam_string inXRI,
                 xsystem_handle* const outHandle)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        authenticate (const char* inBuffer,
                      const xam_int inByteCount,
                      xstream_handle* const outXStream)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for XSet management
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        deleteXSet (const xam_xuid inXUID)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        holdXSet (const xam_xuid inXUID,
                  const xam_string inHoldID)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        releaseXSet (const xam_xuid inXUID,
                     const xam_string inHoldID)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        accessXSet (const xam_xuid inXUID,
                    const xam_int inMode,
                    xam_boolean* const outIsAccessible)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        virtual void
        isXSetRetained (const xam_xuid inXUID,
                        xam_boolean* const outIsRetained)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        virtual void
        getXSetAccessTime (const xam_xuid inXUID,
                           xam_datetime* const outAccessTime)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // method for creating an XSet
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        createXSet (const xam_string inMode, xset_handle* const outXSet)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for XSet policy management
        // ================================================================

        virtual void
        applyAccessPolicy (const xam_boolean inBinding,
                           const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        virtual void
        resetAccessFields ()
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyManagementPolicy (const xam_boolean inBinding,
                               const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        resetManagementFields ()
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        createRetention (const xam_boolean inBinding,
                         const xam_string inRetentionID)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setRetentionEnabledFlag (const xam_string inRetentionID,
                                 const xam_boolean inBinding,
                                 const xam_boolean inEnabled)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyRetentionEnabledPolicy (const xam_string inRetentionID,
                                     const xam_boolean inBinding,
                                     const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setRetentionDuration (const xam_string inRetentionID,
                              const xam_boolean inBinding,
                              const xam_int inDuration)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyRetentionDurationPolicy (const xam_string inRetentionID,
                                      const xam_boolean inBinding,
                                      const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setRetentionStarttime (const xam_string inRetentionID,
                               const xam_boolean inBinding)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setBaseRetention (const xam_boolean inBinding,
                          const xam_int inDuration)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyBaseRetentionPolicy (const xam_boolean inBinding,
                                  const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyAutoDeletePolicy (const xam_boolean inBinding,
                               const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setAutoDelete (const xam_boolean inBinding,
                       const xam_boolean inAutoDelete)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyShredPolicy (const xam_boolean inBinding,
                          const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        setShred (const xam_boolean inBinding,
                  const xam_boolean inAutoDelete)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        applyStoragePolicy (const xam_boolean inBinding,
                            const xam_string inPolicy)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualRetentionDuration (const xam_string inRetentionID,
                                    xam_int* const outDuration)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualRetentionEnabled (const xam_string inRetentionID,
                                   xam_boolean* const outEnabled)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualAutoDelete (xam_boolean* const outEnabled)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualShred (xam_boolean* const outEnabled)
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for editing an XSet
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        openXSet (const xam_xuid inXUID,
                  const xam_string inMode,
                  xset_handle* const outXSet)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        copyXSet (const xam_xuid inXUID,
                  const xam_string inMode,
                  xset_handle* const outXSet)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        commit (xam_xuid* const outXUID)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for XSet migration
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        openExportXStream (xstream_handle* const outXStream)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        openImportXStream (xstream_handle* const outXStream)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for job control
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        submitJob ()        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        haltJob ()        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ========================================================================
        //  methods for async i/o
        // ========================================================================

        /** @see xam.h for detailed description */
        virtual void
        asyncOpenXSet (const xam_xuid inXUID,
                       const xam_string inMode,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        asyncCopyXSet (const xam_xuid inXUID,
                       const xam_string inMode,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        asyncOpenXStream (const xam_string inName,
                          const xam_string inMode,
                          const XOPID inXOPID,
                          xasync_callback inCallback,
                          xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        } 

        /** @see xam.h for detailed description */
        virtual void
        asyncRead (char* ioBuffer,
                   const xam_int inBufferLength,
                   const XOPID inXOPID,
                   xasync_callback inCallback,
                   xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        asyncWrite (const char* inBuffer,
                    const xam_int inByteCount,
                    const XOPID inXOPID,
                    xasync_callback inCallback,
                    xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        asyncClose (const XOPID inXOPID,
                    xasync_callback inCallback,
                    xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        asyncCommit (const XOPID inXOPID,
                     xasync_callback inCallback,
                     xasync_handle* const outAsyncHandle)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        // ================================================================
        // methods for managing asynchronous operations
        // ================================================================

        /** @see xam.h for detailed description */
        virtual void
        halt ()        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        isComplete (xam_boolean* const outIsComplete)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getStatus (xam_status* const outStatus)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getXOPID (XOPID* const outXOPID)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getXSet (xset_handle* const outXSet)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getXStream (xstream_handle* const outXStream)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getXUID (xam_xuid* const outXUID)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getBytesRead (xam_int* const outBytesRead)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }

        /** @see xam.h for detailed description */
        virtual void
        getBytesWritten (xam_int* const outBytesWritten)        
        {
            XAMInvalidHandleException e; 
            throw e;
        }  

};

#endif // __XAM_INTERFACE_H__


