////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_strings.h"

#include "XAM_Object.h"
#include "XAM_ObjectSyntheticFieldMap.h"
#include "XAM_Logger.h"
#include "XAM_Exception.h"
#include "XAM_Handle.h"
#include "XAM_Utils.h"
#include "XSystem.h"
#include "XAM_FieldIterator.h"
#include "XAM_Config.h"
#include "XAM_Router.h"

// Holds the instance for the singleton class
XAM_Object* XAM_Object::mInstance = (XAM_Object*)0;

// Lock covering the XAM Object instance
XAM_Mutex XAM_Object::mMutex;

const char* XAM_Object::sIdentity = "SNIA C XAM Library, version @version.full@";
const char* XAM_Object::sApiLevel = "@version.api@"; 

XAM_Object*
XAM_Object::getInstance()
{
    XAM_CriticalSection((XAM_Mutex*)&mMutex);

    if(mInstance == (XAM_Object*)0)
        mInstance = new XAM_Object();

    if(mInstance == (XAM_Object*)0)
    {
        XAMOutOfMemoryException e;
        throw e;
    }

    return mInstance;
}

void
XAM_Object::writePropertyToXSystem(XAM_VIM* inVim, 
                                   const xsystem_handle& inHandle, 
                                   const xam_string& inFieldType,
                                   const xam_string& inFieldName)
{
    if(XAM_POSIX::xam_strcmp(inFieldType, XAM_BOOLEAN_MIME_TYPE) == 0)
    {
        // Get the value of this field
        xam_boolean vVal;
        this->getBoolean(inFieldName, &vVal);

        // Create the field on the XSystem or modify it's value if it already exists
        try {        
            inVim->XSystem_CreateBoolean(inHandle, inFieldName, false, vVal);
        }
        catch(XAM_Exception &) {
            inVim->XSystem_SetBoolean(inHandle, inFieldName, vVal);
        }
    }
    else if(XAM_POSIX::xam_strcmp(inFieldType, XAM_INT_MIME_TYPE) == 0)
    {
        // Get the value of this field
        xam_int vVal;
        this->getInt(inFieldName, &vVal);

        // Create the field on the XSystem or modify it's value if it already exists
        try {        
            inVim->XSystem_CreateInt(inHandle, inFieldName, false, vVal);
        }
        catch(XAM_Exception &) {
            inVim->XSystem_SetInt(inHandle, inFieldName, vVal);
        }
    }
    else if(XAM_POSIX::xam_strcmp(inFieldType, XAM_DOUBLE_MIME_TYPE) == 0)
    {
        // Get the value of this field
        xam_double vVal;
        this->getDouble(inFieldName, &vVal);

        // Create the field on the XSystem or modify it's value if it already exists
        try {        
            inVim->XSystem_CreateDouble(inHandle, inFieldName, false, vVal);
        }
        catch(XAM_Exception &) {
            inVim->XSystem_SetDouble(inHandle, inFieldName, vVal);
        }
    }   
    else if(XAM_POSIX::xam_strcmp(inFieldType, XAM_XUID_MIME_TYPE) == 0)
    {
        // Get the value of this field
        xam_xuid vVal;
        this->getXUID(inFieldName, &vVal);

        // Create the field on the XSystem or modify it's value if it already exists
        try {        
            inVim->XSystem_CreateXUID(inHandle, inFieldName, false, vVal);
        }
        catch(XAM_Exception &) {
            inVim->XSystem_SetXUID(inHandle, inFieldName, vVal);
        }
    }   
    else if(XAM_POSIX::xam_strcmp(inFieldType, XAM_DATETIME_MIME_TYPE) == 0)
    {
        // Get the value of this field
        xam_datetime vVal;
        this->getDatetime(inFieldName, &vVal);

        // Create the field on the XSystem or modify it's value if it already exists
        try {        
            inVim->XSystem_CreateDatetime(inHandle, inFieldName, false, vVal);
        }
        catch(XAM_Exception &) {
            inVim->XSystem_SetDatetime(inHandle, inFieldName, vVal);
        }
    }    
    else if(XAM_POSIX::xam_strcmp(inFieldType, XAM_STRING_MIME_TYPE) == 0)
    {
        // Get the value of this field
        xam_string vVal;
        this->getString(inFieldName, &vVal);

        // Create the field on the XSystem or modify it's value if it already exists
        try {        
            inVim->XSystem_CreateString(inHandle, inFieldName, false, vVal);
        }
        catch(XAM_Exception &) {
            inVim->XSystem_SetString(inHandle, inFieldName, vVal);
        }
    }            
}

void
XAM_Object::writeXStreamToXSystem(XAM_VIM* inVim, 
                                  const xsystem_handle& inHandle, 
                                  const xam_string& inFieldType,
                                  const xam_string& inFieldName)
{
    char vBuffer[1024];
    xstream_handle vSrcStreamHandle;
    xstream_handle vTgtStreamHandle;
    xam_int vTotalBytesToMove = 0;

    // Cache the length of this stream
    this->getFieldLength(inFieldName, &vTotalBytesToMove);

    // Open the stream on this object
    this->openXStream(inFieldName, XSTREAM_MODE_READ_ONLY, &vSrcStreamHandle);
    XAM_XStream_Instance* vSrcStream = 
        (XAM_XStream_Instance*) vSrcStreamHandle;

    // Create or open the stream on the target object
    try {
        inVim->XSystem_CreateXStream(inHandle, inFieldName, false, inFieldType, &vTgtStreamHandle);
    }
    catch(XAM_Exception &) {
         inVim->XSystem_OpenXStream(inHandle, inFieldName, XSTREAM_MODE_WRITEONLY_TRUNCATE, &vTgtStreamHandle);
    }
         
    // Read from the source stream and write to the target stream
    if(vTotalBytesToMove > 0)
    {
        xam_int vReadThisPass = 0;
    
        // Write as long as we need to
        while(vReadThisPass >= 0)
        {          
            // Read data from the source stream (up to 1024 bytes)
            vReadThisPass = vSrcStream->read(vBuffer, 1024);

            // If we read anything, write it to the target stream
            if(vReadThisPass >= 0)
            {
                // Write until all read data is in the new stream
                xam_int vTotalWrittenThisPass = 0;  
                while(vTotalWrittenThisPass < vReadThisPass)
                {
                    xam_int vWrittenThisPass = 0;  
                    inVim->XStream_Write(vTgtStreamHandle, 
                                         &vBuffer[vTotalWrittenThisPass], 
                                         vReadThisPass - vTotalWrittenThisPass, 
                                         &vWrittenThisPass);
                    vTotalWrittenThisPass += vWrittenThisPass;
                }
            }
        }
    }

    // Close both streams
    vSrcStream->close();
    inVim->XStream_Close(vTgtStreamHandle);
}

void 
XAM_Object::writeXSystemFields(XAM_VIM* inVim, const xsystem_handle inHandle)
{
    xam_boolean vHasNext;
    xam_boolean vReadOnly;
    xam_string vFieldName;
    xam_string vFieldType;
    xiterator_handle vHandle;

    // Open an XIterator on this object
    this->openFieldIterator("", &vHandle);
    XAM_FieldIterator* vIter = (XAM_FieldIterator*)vHandle;

    try
    {
        // Copy each field to the XSystem        
        vIter->hasNext(vHasNext);
        while(vHasNext)
        {
            vIter->getNext(vFieldName);

            // Determine the field read only state
            this->getFieldReadOnly(vFieldName, &vReadOnly);

            // Do not copy readonly fields onto the XSystem, as there's
            // no VIM method available for setting a readonly state. 
            // Applications cannot set a readonly state, so this 
            // exclusion only applies to XAM Library read only fields.
            if(vReadOnly == false)
            {
                // Determine the field type
                this->getFieldType(vFieldName, &vFieldType);

                try
                {
                    // Write Property fields to the XSystem
                    XAM_Utils::validateSimpleType(vFieldType);
                    writePropertyToXSystem(inVim, inHandle, vFieldType, vFieldName);
                }
                catch(XAMInvalidMimeTypeException &)
                {
                    // Write XStream fields to the XSystem
                    writeXStreamToXSystem(inVim, inHandle, vFieldType, vFieldName);
                }
            }

            vIter->hasNext(vHasNext);
        }
    }
    catch(...)
    {
        // Cleanup...
        if(vIter)
            delete vIter;
        throw;
    }

    // Finished with the XIterator
    if(vIter)
        delete vIter;
}

void
XAM_Object::applyGlobalFieldSettings(XAM_VIM* inVim, const xsystem_handle inHandle)
{
    XAM_LOG_INFO_P2(component_vim_enter, 
                    "******* Start XSystem initialization (%s) *******",
                    inVim->getName());

    // Set a field indicating that this is a XAM library field update
    inVim->XSystem_CreateBoolean(inHandle, XAM_XSYSTEM_INITIALIZING, false, true);    

    // Write all XAM Object fields to the XSystem (processing accordingly)
    writeXSystemFields(inVim, inHandle);

    // Remove the XAM Library update field
    inVim->XSystem_DeleteField(inHandle, XAM_XSYSTEM_INITIALIZING);  

    XAM_LOG_INFO_P2(component_vim_exit, 
                    "******* End XSystem initialization (%s) *******",
                    inVim->getName());
}

void
XAM_Object::connect(XAM_VIM* inVim, 
                    xam_string inCxnStr,
                    xsystem_handle* const outHandle)
{
    // Create the XSystem instance
    inVim->CreateXSystem(outHandle);

    try
    {
        // Set XAM Object fields on the XSystem
        applyGlobalFieldSettings(inVim, *outHandle);

        // Connect the to XSystem
        inVim->XSystem_Connect(*outHandle, inCxnStr);
    }
    catch(XAM_Exception &e)
    {
        // Cleanup the allocated XSystem handle on error
        try
        {
            // Close the allocated XSystem handle
            inVim->XSystem_Close(*outHandle);
        }
        catch(XAM_Exception &)
        {
            // Something went wrong... Abandon!!!
            inVim->XSystem_Abandon(*outHandle);
            inVim->XSystem_Close(*outHandle); 
        }

        // Throw the original exception
        throw e;
    }
}

XAM_Object::XAM_Object()
{
    // Pre-register singleton for config sequence below
    mInstance = this;

    mType = XAM_OBJECT_HANDLE;
    mSyntheticFieldMap = new XAM_ObjectSyntheticFieldMap(this);

    // Add all "fixed" standard XAM object fields
    createString(XAM_IDENTITY, false, sIdentity); 
    setFieldAsReadOnly(XAM_IDENTITY);
    createString(XAM_API_LEVEL, false, sApiLevel); 
    setFieldAsReadOnly(XAM_API_LEVEL);

    // Load and apply configuration file settings
    XAM_Config vConfig;    
    vConfig.generatePropertyFields(mInstance);

    // Preload any VIM libraries that were specified in the config
    XAM_Router::preloadVims(mInstance);
}

XAM_Object::~XAM_Object()
{
    if(mSyntheticFieldMap)
        delete mSyntheticFieldMap;
}

