#ifndef __XAM_OBJECT__
#define __XAM_OBJECT__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_strings.h"

#include "XAM_FieldContainer.h"
#include "XAM_Handle.h"
#include "XAM_Mutex.h"
#include "XAM_VIM.h"

/** 
 *  A singleton class representing the top level XAM library object    
 */
class XAM_Object : public XAM_FieldContainer
{
    private:         
         static XAM_Object* mInstance; // Holds instance of this singleton
         static XAM_Mutex mMutex;      // Covers the instance

         // Constants
         static const char* sIdentity; // XAM_IDENTITY
         static const char* sApiLevel; // XAM_API_LEVEL

         XAM_Object(); // Singleton, private constructor. Use getInstance().

         /*
          * Apply global XAM Object field settings on the XSystem instance
          */
         void
         applyGlobalFieldSettings(XAM_VIM* inVim, const xsystem_handle inHandle);

         /*
          * Writes a XAM Object Property field to the VIM's XSystem instance
          */
         void
         writePropertyToXSystem(XAM_VIM* inVim, 
                                const xsystem_handle& inHandle, 
                                const xam_string& inFieldType,
                                const xam_string& inFieldName);
         /*
          * Writes a XAM Object Property field to the VIM's XSystem instance
          */
         void
         writeXStreamToXSystem(XAM_VIM* inVim, 
                               const xsystem_handle& inHandle,
                               const xam_string& inFieldType,
                               const xam_string& inFieldName);
         /*
          * Writes all XAM Object fields to the VIM's XSystem instance
          */
         void
         writeXSystemFields(XAM_VIM* inVim, const xsystem_handle inHandle);

    public:
         /**
          * Retrieve the XAM_Object instance
          */
         static XAM_Object* 
         getInstance();

#ifdef SUN
        using XAM_FieldContainer::connect; // add base class connect definitions
#endif
         /**
          * Connect to an XSystem
          *
          * @param inVim     The VIM to use for the connect operation
          * @param inCxnStr  The connection string from the XRI
          * @param outHandle A handle to the created XSystem object 
          *                 is written here
          * @throws XAM_Exception on error
          */
         void
         connect(XAM_VIM* inVim, 
                 xam_string inCxnStr, 
                 xsystem_handle* const outHandle);

         // Destructor
         virtual 
         ~XAM_Object();
};


#endif // __XAM_OBJECT__


