#ifndef __XAM_OBJECT_SYNTHETIC_FIELDS__
#define __XAM_OBJECT_SYNTHETIC_FIELDS__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_SyntheticField.h"
#include "XAM_LogFactory.h"
#include "XAM_Config.h"

/**
 * Defines the XAM_SyntheticField interfaces for the XAM object
 */
class XAM_ObjectSyntheticFields : public XAM_SyntheticField
{   
    private:
        XAM_FieldContainer* mParent; // Handle to the parent field container

    public:

        /**
         * Constructor
         */        
        XAM_ObjectSyntheticFields(XAM_FieldContainer* inParent) 
            : mParent(inParent) {}
    
        /**
         * Destructor
         */
        virtual
        ~XAM_ObjectSyntheticFields() {}

        /**
         *  Determines whether or not this method applies to 
         *  the field of the given name.
         * 
         *  @param inName The name of the field to check
         *  @return true if this field is supported by this method, 
         *          false otherwise
         */
        virtual const xam_boolean 
        isMapped(const xam_string inName)
        {
            xam_boolean vMapped = false;

            if((XAM_POSIX::xam_strcmp(inName, XAM_MAX_STRING_FIELD) == 0) ||
               (XAM_POSIX::xam_strcmp(inName, XAM_CONFIG_PATH) == 0))
            {
                vMapped = true;
            }

            return vMapped;
        }

        /**
         * Adds all field names for this method to the cache object 
         *
         * @param inCache The cache object to populate with field names
         */
        virtual void
        cache(XAM_FieldNameCache* inCache)
        {
            inCache->addField(XAM_MAX_STRING_FIELD);

            if(XAM_Config::getConfigFilePath() != (char*)0)
                inCache->addField(XAM_CONFIG_PATH);
        }

        /** 
         * Handles all getFieldType requests this map has registered for
         */
        virtual void
        getFieldType(const xam_string inName,
                     xam_string* const outType)
        {
            // xam_int
            if(XAM_POSIX::xam_strcmp(inName, XAM_MAX_STRING_FIELD) == 0)
            {
                // Retrieve the mime type of this field
                XAM_POSIX::xam_strcpy(*outType, XAM_INT_MIME_TYPE);
            }
            // xam string
            else if((XAM_POSIX::xam_strcmp(inName, XAM_CONFIG_PATH) == 0) && XAM_Config::getConfigFilePath())
            {
                // Retrieve the mime type of this field
                XAM_POSIX::xam_strcpy(*outType, XAM_STRING_MIME_TYPE);
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }
        
        /** 
         * Handles all getFieldLength requests this map has registered for
         */
        virtual void 
        getFieldLength (const xam_string inName,
                        xam_int* const outLength)
        {
            // xam_int
            if(XAM_POSIX::xam_strcmp(inName, XAM_MAX_STRING_FIELD) == 0)
            {
                *outLength = sizeof(xam_int);
            }
            // xam string
            else if((XAM_POSIX::xam_strcmp(inName, XAM_CONFIG_PATH) == 0) && XAM_Config::getConfigFilePath())
            {
                const char* vPath = XAM_Config::getConfigFilePath();
                if(vPath != (const char*)0)
                    *outLength = XAM_POSIX::xam_strlen(vPath);
                else
                    *outLength = 0;
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all setInt requests this map has registered for
         */
        virtual void 
        setInt (const xam_string inName,
                const xam_int inValue)
        {
           
            // Support for this field does not exist
            XAMFieldNotFoundException e;
            throw e;
        }

        /**
         * Handles all getInt requests this map has registered for
         */
        virtual void 
        getInt (const xam_string inName,
                xam_int* const outValue)
        {
            if(XAM_POSIX::xam_strcmp(inName, XAM_MAX_STRING_FIELD) == 0)
            {
                *outValue = XAM_MAX_STRING;
            }
            else
            {
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all setString requests this map has registered for
         */
        virtual void 
        setString (const xam_string inName,
                   const xam_string inValue)
        {
            if (XAM_POSIX::xam_strcmp(inName, XAM_CONFIG_PATH) == 0)
            {
                // Determine if this config path is accessible
                if(XAM_POSIX::xam_access(inValue, XAM_F_OK) == 0)
                {
                    // Read the new configuration file and apply the changes
                    XAM_Config::setConfigFilePath((char*)inValue);
                }
                else
                {
                    // Could not access the specified file
                    XAMFileSystemException e;
                    throw e;  
                }
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }

        /**
         * Handles all getString requests this map has registered for
         */
        virtual void 
        getString (const xam_string inName,
                   xam_string* const outValue)
        {   
            if((XAM_POSIX::xam_strcmp(inName, XAM_CONFIG_PATH) == 0) && XAM_Config::getConfigFilePath())
            {
                const char* vPath = XAM_Config::getConfigFilePath();
                if(vPath)
                    XAM_POSIX::xam_strcpy(*outValue, vPath);
                else
                    XAM_POSIX::xam_strcpy(*outValue, "");
            }
            else
            {               
                // Support for this field does not exist
                XAMFieldNotFoundException e;
                throw e;
            }
        }
};

#endif // __XAM_OBJECT_SYNTHETIC_FIELDS__


