#ifndef __XASYNC_OBJECT__
#define __XASYNC_OBJECT__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_Handle.h"
#include "XAM_VIM.h"
#include "XAM_XAsync.h"
#include "XAM_Logger.h"
#include "XAM_Mutex.h"
#include "XAM_XAsyncHandler.h"
#include <jni.h>


/** 
 *  Wrapper class for an XAsync object handle
 */
class XAsync : public XAM_Handle
{
    private:
        // used in JNI call backs
        jobject mJNIAsyncObj;  // Java XAsync object (used in JNI call back)
        jobject mJNIAsyncListenerObj; // Java XAsyncListener Object 
        JNIEnv *mJNIEnv; // JNI Env pointer
        char *mWriteBuff; // user Write buffer
        char *mReadBuff; // user Read buffer
        jobject mReadJNIBuff; // JNI Read buffer
        jboolean mASyncClose; // Only true if the JNI AsyncClose is called.
        jobject mJavaXStreamObj; // Reference to Java XStream Object. Only used in ASyncClose.
        xam_boolean mJavaEnvInited; // Java environment has been inited for callback
        xam_boolean mJavaIsComplete; // true if a java asyncop is complete
        xam_boolean mHandleSet; // A handle was set on this object

        // Cache the user's callback so that this object may call 
        // it on async complete
        xasync_callback mUserCallback;

        XAM_Mutex mGetSetHandleMutex; // Mutex for the get/set handle methods
                                      // (because this is not atomic on 32-bit platforms!)
        XAM_Mutex mIsJavaEnvInitedMutex;

        xam_boolean mComplete; // If true, this operation has completed

    public:    
        /** Constructor */
        XAsync(xasync_callback inCallback, 
               XAM_Handle* inParent, 
               XAM_VIM* inVim = 0)
            : XAM_Handle(0, inParent, inVim),
              mUserCallback(inCallback),
              mJNIAsyncObj(0), 
              mJNIAsyncListenerObj(0),
              mJNIEnv(0),
              mWriteBuff(0),
              mReadBuff(0),
              mReadJNIBuff(0),
              mASyncClose(false),
              mJavaXStreamObj(0),
              mJavaEnvInited(false),
              mJavaIsComplete(false),
              mHandleSet(false),
              mComplete(false)
        {
            mType = XASYNC_HANDLE;

            // Register the async operation if a callback method was provided.
            // This allows the XAM Library to synchronize callback operations.
            if(inCallback != (xasync_callback)0)
                XAM_XAsyncHandler::registerAsyncOp(this);
        }

        /** Destructor */
        virtual 
        ~XAsync() {}

        virtual const xam_handle_t  
        getHandle()
        {
            XAM_CriticalSection vCritical(&mGetSetHandleMutex);
            return mHandleSet ? mHandle : (xam_handle_t)0;
        }

        virtual void
        setHandle(xam_handle_t* inHandle)
        {
            XAM_CriticalSection vCritical(&mGetSetHandleMutex);
            mHandle = (xam_handle_t) *inHandle;
            mHandleSet = true;
        }
               
        /** 
            Stores the required JNI Environment so that if an async api is called from java
            we can call the java callback method.

            @note This function should only be called from the JNI.

            @param inJNIEnv JNI Environment pointer.
            @param inJNIAsyncObj The java async object that needs to be passed in to java callback method.
            @param inJNIAsyncListenerObj The java Async listener object
            @param inWriteBuffer Buffer used in write operations. we need to free this buffer
            @param inReadBuffer this is buffer passed in at the time of read api calls. we need to 
            copy the data to the java buffer before freeing.
        */

        void
        setJNIEnv(JNIEnv *inJNIEnv, 
                  jobject inJNIAsyncListenerObj, 
                  jobject inJNIAsyncObj, 
                  char* inWriteBuffer, 
                  char* inReadBuffer,
                  jobject inReadJNIBuffer,
                  jboolean inASyncClose,
                  jobject inJavaXStreamObj)
        {
            XAM_LOG_DEBUG_P2(component_xam_debug, 
                             "XAsync: setJNIENV Enter (%"XAM_INT_FORMAT"d)", 
                             (xasync_handle)this);

            mJNIEnv = inJNIEnv;
            mJNIAsyncListenerObj = inJNIAsyncListenerObj;
            mJNIAsyncObj = inJNIAsyncObj;
            mWriteBuff = inWriteBuffer;
            mReadBuff = inReadBuffer;
            mReadJNIBuff = inReadJNIBuffer;
            mASyncClose = inASyncClose;
            mJavaXStreamObj = inJavaXStreamObj;
            XAM_CriticalSection vCriticial(&mIsJavaEnvInitedMutex);
            mJavaEnvInited = true;
            XAM_LOG_DEBUG_P2(component_xam_debug, 
                             "XAsync: setJNIENV Exit (%s)", 
                             mJavaEnvInited ? "true" : "false");
        }

        /** 
            Returns the required JNI Environment so that if an async api is called from java
            we can call the java callback method.

            @note This function should only be called from the JNI.

            @param outJNIEnv JNI Environment pointer.
            @param outJNIAsyncObj The java async object that needs to be passed in to java callback method.
            @param outJNIAsyncListenerObj The java Async listener object
        */
        
        void
        getJNIEnv(JNIEnv* &outJNIEnv, 
                  jobject &outJNIAsyncListenerObj,  
                  jobject &outJNIAsyncObj, 
                  char* &outWriteBuffer, 
                  char* &outReadBuffer,
                  jobject &outReadJNIBuffer,
                  jboolean &outAsyncClose,
                  jobject &outJavaXStreamObj)
        {
            XAM_LOG_DEBUG_P2(component_xam_debug, 
                             "XAsync: getJNIENV Enter (%"XAM_INT_FORMAT"d)", 
                             (xasync_handle)this);

            outJNIEnv = mJNIEnv;
            outJNIAsyncListenerObj = mJNIAsyncListenerObj;
            outJNIAsyncObj = mJNIAsyncObj;
            outWriteBuffer = mWriteBuff;
            outReadBuffer = mReadBuff;
            outReadJNIBuffer = mReadJNIBuff;
            outAsyncClose = mASyncClose;
            outJavaXStreamObj = mJavaXStreamObj;
            
            XAM_LOG_DEBUG(component_xam_debug, 
                             "XAsync: getJNIENV Exit.");
        }

        /** 
         *   @return true if the JNI ENV is set on this async object. otherwise false.
         */        
        xam_boolean
        getJavaEnvInited()
        {
            XAM_CriticalSection vCriticial(&mIsJavaEnvInitedMutex);
            return mJavaEnvInited;
        }

        /**
         *  Sets the value determining whether or not the java async operation has completed to true.
         */
        void 
        setJavaIsComplete()
        {
            mJavaIsComplete = true;
        }

        /**
         *  Gets the value determining whether or not the java async operation has completed.
         *
         *  @returns True if the async op is complete, false otherwise.
         */
        xam_boolean
        getJavaIsComplete()
        {
            return mJavaIsComplete;
        }

        virtual void
        halt ()
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_Halt(mHandle);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                xAsync->halt();
            }

            // Unregister the handle (in case it is registered)
            // (This is a no-op if not registered)
            // should always be unregistered before it gets here
            if(mUserCallback != (xasync_callback)0)
                XAM_XAsyncHandler::unregisterAsyncOp(this);

        }

        virtual void
        isComplete (xam_boolean* const outIsComplete)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_IsComplete(mHandle, outIsComplete);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                *outIsComplete = xAsync->isComplete();
            }

            if(mUserCallback != (xasync_callback)0)               
            {
                // The operation is only really complete if the VIM
                // has responded (mComplete state)
                *outIsComplete = ((mComplete) && (*outIsComplete));
            }            

        }

        virtual void
        getStatus (xam_status* const outStatus)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetStatus(mHandle, outStatus);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                *outStatus = xAsync->getStatus();
            }
        }
        
        virtual void
        getXOPID (XOPID* const outXOPID)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetXOPID(mHandle, outXOPID);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                *outXOPID = xAsync->getXOPID();
            }
        }

        virtual void
        getXSet (xset_handle* const outXSet)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetXSet(mHandle, outXSet);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAMInvalidHandleException e;
                throw e;
            }
        }

        virtual void
        getXStream (xstream_handle* const outXStream)        
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetXStream(mHandle, outXStream);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                *outXStream = xAsync->getXStream();
            }
        }

#ifdef SUN
        using XAM_Interface::getXUID; // add base class getXUID definitions
#endif
        virtual void
        getXUID (xam_xuid* const outXUID)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetXUID(mHandle, outXUID);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAMInvalidHandleException e;
                throw e;
            }
        }

        virtual void
        getBytesRead (xam_int* const outBytesRead)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetBytesRead(mHandle, outBytesRead);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                *outBytesRead = xAsync->getBytesRead();
            }
        }

        virtual void
        getBytesWritten (xam_int* const outBytesWritten)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XAsync_GetBytesWritten(mHandle, outBytesWritten);
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;
                *outBytesWritten = xAsync->getBytesWritten();
            }
        }

        virtual void
        close ()
        {
            if(mVim != (XAM_VIM*)0)
            {
                mVim->XAsync_Close(mHandle);
            }
            else
            {
                // No VIM exists. Use the XAM Library equivalent.
                XAM_XAsync* xAsync = (XAM_XAsync*) mHandle;  
                if(xAsync)
                {
                    delete xAsync;
                    mHandle = 0;
                }
            }

            // Unregister the handle (in case it is still registered)
            // (This is a no-op if not registered)
            if(mUserCallback != (xasync_callback)0)
                XAM_XAsyncHandler::unregisterAsyncOp(this);

        }

        virtual void
        notifyComplete()
        {
            mComplete = true;

            // Call the user's callback
            if(mUserCallback != (xasync_callback)0)
            {
                XAM_LOG_DEBUG_P3(component_xam_debug, 
                                 "XAsync [%p]: Notifying user callback [%p]", 
                                 this, 
                                 mUserCallback);
                mUserCallback((xasync_handle)this);
            }
        }
};

#endif // __XASYNC_OBJECT__

