#ifndef __XITERATOR_OBJECT__
#define __XITERATOR_OBJECT__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_errors.h"
#include "XAM_Handle.h"
#include "XAM_Object.h"
#include "XAM_VIM.h"
#include "XAM_FieldIterator.h"

/** 
 *  Wrapper class for an XIterator object handle
 */
class XIterator : public XAM_Handle
{
    public:    
        /** Constructor */
        XIterator(const xam_handle_t inHandle, XAM_Handle* inParent)
            : XAM_Handle(inHandle, inParent) 
        {
            mType = XITERATOR_HANDLE;
        }

        /** Destructor */
        virtual 
        ~XIterator() {}

        virtual void
        hasNext (xam_boolean* const outValue)
        {
            if(mVim != (XAM_VIM*)0)
            {
                mVim->XIterator_HasNext(mHandle, outValue);
            }
            else
            {
                // Assumes use of the XAM_FieldIterator (XAM Object)
                XAM_FieldIterator* vIter = (XAM_FieldIterator*) mHandle;
                if(vIter)
                {
                    vIter->hasNext(*outValue); 
                }
                else
                {
                    *outValue = false;
                }
            }
        }

        virtual void 
        next (xam_string* const outName)
        {
            if(mVim != (XAM_VIM*)0)
            {
                mVim->XIterator_Next(mHandle, outName);
            }
            else
            {
                // Assumes use of the XAM_FieldIterator (XAM Object)
                XAM_FieldIterator* vIter = (XAM_FieldIterator*) mHandle;
                if(vIter)
                {
                    vIter->getNext(*outName); 
                }
                else
                {
                    XAM_POSIX::xam_strcpy(*outName, "");
                }
            }
        }

        virtual void 
        close ()
        {
            if(mVim != (XAM_VIM*)0)
            {
                mVim->XIterator_Close(mHandle);
            }
            else
            {
                // If no VIM exists, use the XAM Library object equivalent object
                XAM_FieldIterator* vXIter = (XAM_FieldIterator*) mHandle;
                if(vXIter)
                {
                    delete vXIter;
                    mHandle = 0;
                }
            }
        }
};

#endif // __XITERATOR_OBJECT__

