#ifndef __XSET_OBJECT__
#define __XSET_OBJECT__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_Handle.h"
#include "XAM_XAsync.h"
#include "XAM_XAsyncHandler.h"

/** 
 *  Wrapper class for an XSet object handle
 */
class XSet : public XAM_Handle
{
    public:    

        /** Constructor */
        XSet(xam_handle_t inHandle, XAM_Handle* inParent)
            : XAM_Handle(inHandle, inParent)   
        {
            mType = XSET_HANDLE;
        }

        /** Destructor */
        virtual
        ~XSet() {}

        virtual void
        applyAccessPolicy (const xam_boolean inBinding,
                           const xam_string inPolicy)
        {
            mVim->XSet_ApplyAccessPolicy(mHandle, inBinding, inPolicy);
        }

        virtual void
        resetAccessFields ()
        {
            mVim->XSet_ResetAccessFields(mHandle);
        }

        virtual void
        applyManagementPolicy (const xam_boolean inBinding,
                               const xam_string inPolicy)
        {
            mVim->XSet_ApplyManagementPolicy(mHandle, inBinding, inPolicy);
        }

        virtual void
        resetManagementFields ()    
        {  
            mVim->XSet_ResetManagementFields(mHandle);
        }

       /** @see xam.h for detailed description */
        virtual void
        createRetention (const xam_boolean inBinding,
                         const xam_string inRetentionID)
        {
            // Forbid "base" retention ID.
            if(XAM_POSIX::xam_strcasecmp(inRetentionID, "base") == 0)
            {
                XAMInvalidParamException e;
                throw e;
            }

            mVim->XSet_CreateRetention(mHandle, inBinding, inRetentionID);
        }

        /** @see xam.h for detailed description */
        virtual void
        setRetentionEnabledFlag (const xam_string inRetentionID,
                                 const xam_boolean inBinding,
                                 const xam_boolean inEnabled)
        {
            // Forbid "base" retention ID.
            if(XAM_POSIX::xam_strcasecmp(inRetentionID, "base") == 0)
            {
                XAMInvalidParamException e;
                throw e;
            }

            mVim->XSet_SetRetentionEnabledFlag(mHandle, inRetentionID, inBinding, inEnabled);
        }

        /** @see xam.h for detailed description */
        virtual void
        applyRetentionEnabledPolicy (const xam_string inRetentionID,
                                     const xam_boolean inBinding,
                                     const xam_string inPolicy)
        {
            // Forbid "base" retention ID.
            if(XAM_POSIX::xam_strcasecmp(inRetentionID, "base") == 0)
            {
                XAMInvalidParamException e;
                throw e;
            }
            mVim->XSet_ApplyRetentionEnabledPolicy(mHandle, inRetentionID, inBinding, inPolicy);
        }

        /** @see xam.h for detailed description */
        virtual void
        setRetentionDuration (const xam_string inRetentionID,
                              const xam_boolean inBinding,
                              const xam_int inDuration)
        {
            // Forbid "base" retention ID.
            if(XAM_POSIX::xam_strcasecmp(inRetentionID, "base") == 0)
            {
                XAMInvalidParamException e;
                throw e;
            }

            mVim->XSet_SetRetentionDuration(mHandle, inRetentionID, inBinding, inDuration);
        }

        /** @see xam.h for detailed description */
        virtual void
        applyRetentionDurationPolicy (const xam_string inRetentionID,
                                      const xam_boolean inBinding,
                                      const xam_string inPolicy)
        {
            // Forbid "base" retention ID.
            if(XAM_POSIX::xam_strcasecmp(inRetentionID, "base") == 0)
            {
                XAMInvalidParamException e;
                throw e;
            }

            mVim->XSet_ApplyRetentionDurationPolicy(mHandle, inRetentionID, inBinding, inPolicy);
        }

        /** @see xam.h for detailed description */
        virtual void
        setRetentionStarttime (const xam_string inRetentionID,
                               const xam_boolean inBinding)
        {
            // Forbid "base" retention ID.
            if(XAM_POSIX::xam_strcasecmp(inRetentionID, "base") == 0)
            {
                XAMInvalidParamException e;
                throw e;
            }

            mVim->XSet_SetRetentionStarttime(mHandle, inRetentionID, inBinding);
        }

        /** @see xam.h for detailed description */
        virtual void
        setBaseRetention (const xam_boolean inBinding,
                          const xam_int inDuration)
        {
            mVim->XSet_SetBaseRetention(mHandle, inBinding, inDuration);
        }

        /** @see xam.h for detailed description */
        virtual void
        applyBaseRetentionPolicy (const xam_boolean inBinding,
                                  const xam_string inPolicy)
        {
            mVim->XSet_ApplyBaseRetentionPolicy(mHandle, inBinding, inPolicy);
        }

        virtual void
        applyAutoDeletePolicy (const xam_boolean inBinding,
                               const xam_string inPolicy)
        {
            mVim->XSet_ApplyAutoDeletePolicy(mHandle, inBinding, inPolicy);
        }

        virtual void
        setAutoDelete (const xam_boolean inBinding,
                       const xam_boolean inAutoDelete)
        {
            mVim->XSet_SetAutoDelete(mHandle, inBinding, inAutoDelete);
        }

        /** @see xam.h for detailed description */
        virtual void
        applyShredPolicy (const xam_boolean inBinding,
                          const xam_string inPolicy)
        {
            mVim->XSet_ApplyShredPolicy(mHandle, inBinding, inPolicy);
        }

        /** @see xam.h for detailed description */
        virtual void
        setShred (const xam_boolean inBinding,
                  const xam_boolean inShred)
        { 
            mVim->XSet_SetShred(mHandle, inBinding, inShred);
        }

        /** @see xam.h for detailed description */
        virtual void
        applyStoragePolicy (const xam_boolean inBinding,
                            const xam_string inPolicy)
        {
            mVim->XSet_ApplyStoragePolicy(mHandle, inBinding, inPolicy);
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualRetentionDuration (const xam_string inRetentionID,
                                    xam_int* const outDuration)
        {
            mVim->XSet_GetActualRetentionDuration(mHandle, inRetentionID, outDuration);
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualRetentionEnabled (const xam_string inRetentionID,
                                   xam_boolean* const outEnabled)
        { 
            mVim->XSet_GetActualRetentionEnabled(mHandle, inRetentionID, outEnabled);
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualAutoDelete (xam_boolean* const outEnabled)
        {
            mVim->XSet_GetActualAutoDelete(mHandle, outEnabled);
        }

        /** @see xam.h for detailed description */
        virtual void
        getActualShred (xam_boolean* const outEnabled)
        {
            mVim->XSet_GetActualShred(mHandle, outEnabled);
        }

        /** @see xam.h for detailed description */
        virtual void
        commit (xam_xuid* const outXUID)
        {
            mVim->XSet_Commit(mHandle, outXUID);
        }

        /** @see xam.h for detailed description */
        virtual void
        close ()
        {
            mVim->XSet_Close(mHandle);
        }

        /** Abandon this handle */
        virtual void 
        abandon() 
        { 
            mVim->XSet_Abandon(mHandle);
        }

        virtual void
        openExportXStream (xstream_handle* const outXStream)
        {
            mVim->XSet_OpenExportXStream(mHandle, outXStream);
        }

        virtual void
        openImportXStream (xstream_handle* const outXStream)
        {
            mVim->XSet_OpenImportXStream(mHandle, outXStream);
        }

        virtual void
        submitJob ()
        {
            mVim->XSet_SubmitJob(mHandle);
        }

        virtual void
        haltJob ()
        {
            mVim->XSet_HaltJob(mHandle);
        }

        virtual void
        containsField (const xam_string inName,
                     xam_boolean* const outExists)
        {
            mVim->XSet_ContainsField(mHandle, inName, outExists);
        }

        virtual void 
        setFieldAsBinding(const xam_string inName)
        {
            mVim->XSet_SetFieldAsBinding(mHandle, inName);
        }

        virtual void 
        setFieldAsNonbinding(const xam_string inName)
        {
            mVim->XSet_SetFieldAsNonbinding(mHandle, inName);
        }

        virtual void 
        getFieldType(const xam_string inName, xam_string* const outType)
        {
            mVim->XSet_GetFieldType(mHandle, inName, outType);
        }

        virtual void 
        getFieldLength(const xam_string inName, xam_int* const outLen)
        {  
            mVim->XSet_GetFieldLength(mHandle, inName, outLen);
        }

        virtual void
        getFieldBinding(const xam_string inName, xam_boolean* const outFixed)
        {  
            mVim->XSet_GetFieldBinding(mHandle, inName, outFixed);
        }

        virtual void 
        getFieldReadOnly(const xam_string inName, xam_boolean* const outReadOnly)
        {
            mVim->XSet_GetFieldReadOnly(mHandle, inName, outReadOnly);
        }

        virtual void 
        deleteField(const xam_string inName)
        {
            mVim->XSet_DeleteField(mHandle, inName);
        }

        virtual void 
        createBoolean(const xam_string inName, 
                      const xam_boolean inBinding, 
                      const xam_boolean inValue)
        {
            mVim->XSet_CreateBoolean(mHandle, inName, inBinding, inValue);
        }

        virtual void 
        createInt(const xam_string inName, 
                  const xam_boolean inBinding, 
                  const xam_int inValue)
        { 
            mVim->XSet_CreateInt(mHandle, inName, inBinding, inValue);
        }    
        
        virtual void 
        createDouble(const xam_string inName, 
                    const xam_boolean inBinding, 
                    const xam_double inValue)
        {
            mVim->XSet_CreateDouble(mHandle, inName, inBinding, inValue);
        }    

        void 
        createXUID(const xam_string inName, 
                   const xam_boolean inBinding, 
                   const xam_xuid inValue)
        {
            mVim->XSet_CreateXUID(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        createString(const xam_string inName, 
                     const xam_boolean inBinding, 
                     const xam_string inValue)
        {
            mVim->XSet_CreateString(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        createDatetime(const xam_string inName, 
                       const xam_boolean inBinding, 
                       const xam_datetime inValue)
        {
            mVim->XSet_CreateDatetime(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        setBoolean(const xam_string inName, const xam_boolean inValue)
        {
            mVim->XSet_SetBoolean(mHandle, inName, inValue);
        }    

        virtual void 
        setInt(const xam_string inName, const xam_int inValue)
        {
            mVim->XSet_SetInt(mHandle, inName, inValue);
        }  

        virtual void 
        setDouble(const xam_string inName, const xam_double inValue)
        {
            mVim->XSet_SetDouble(mHandle, inName, inValue);
        }  

        virtual void 
        setXUID(const xam_string inName, const xam_xuid inValue)
        {
            mVim->XSet_SetXUID(mHandle, inName, inValue);
        }  

        virtual void 
        setString(const xam_string inName, const xam_string inValue)
        {
            mVim->XSet_SetString(mHandle, inName, inValue);
        }  

        virtual void 
        setDatetime(const xam_string inName, const xam_datetime inValue)
        {
            mVim->XSet_SetDatetime(mHandle, inName, inValue);
        }  

        virtual void 
        getBoolean(const xam_string inName, xam_boolean* const outValue)
        {
            mVim->XSet_GetBoolean(mHandle, inName, outValue);
        }  

        virtual void 
        getInt(const xam_string inName, xam_int* const outValue)
        {
            mVim->XSet_GetInt(mHandle, inName, outValue);
        }  

        virtual void 
        getDouble(const xam_string inName, xam_double* const outValue)
        { 
            mVim->XSet_GetDouble(mHandle, inName, outValue);
        }  

#ifdef SUN
        using XAM_Interface::getXUID; // add base class getXUID definitions
#endif
        virtual void 
        getXUID(const xam_string inName, xam_xuid* const outValue)
        {
            mVim->XSet_GetXUID(mHandle, inName, outValue);
        }  

        virtual void 
        getString(const xam_string inName, xam_string* const outValue)
        { 
            mVim->XSet_GetString(mHandle, inName, outValue);
        }  

        virtual void 
        getDatetime(const xam_string inName, xam_datetime* const outValue)
        {
            mVim->XSet_GetDatetime(mHandle, inName, outValue);
        }  

        virtual void 
        createXStream(const xam_string inName, 
                      const xam_boolean inBinding, 
                      const xam_string inType, 
                      xstream_handle* const outXStream)
        {
            mVim->XSet_CreateXStream(mHandle, 
                                     inName, 
                                     inBinding, 
                                     inType, 
                                     outXStream);
        }  

        virtual void 
        openXStream(const xam_string inName, 
                    const xam_string inMode, 
                    xstream_handle* const outXStream)
        {
            mVim->XSet_OpenXStream(mHandle, inName, inMode, outXStream);
        }  

        virtual void 
        openFieldIterator(const xam_string inPattern, 
                            xiterator_handle* const outHandle)
        {
            mVim->XSet_OpenFieldIterator(mHandle, inPattern, outHandle);
        }  

        virtual void
        asyncOpenXStream (const xam_string inName,
                          const xam_string inMode,
                          const XOPID inXOPID,
                          xasync_callback inCallback,
                          xasync_handle* const outAsyncHandle)        
        {
            xasync_callback vCallback = 
                    inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            mVim->XSet_AsyncOpenXStream(mHandle, 
                                        inName,
                                        inMode, 
                                        inXOPID,
                                        vCallback,
                                        outAsyncHandle);
        } 

        virtual void
        asyncCommit (const XOPID inXOPID,
                     xasync_callback inCallback,
                     xasync_handle* const outAsyncHandle)        
        {
            xasync_callback vCallback = 
                    inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            mVim->XSet_AsyncCommit(mHandle, 
                                   inXOPID,
                                   vCallback,
                                   outAsyncHandle);
        }
};

#endif // __XSET_OBJECT__


