#ifndef __XSTREAM_OBJECT__
#define __XSTREAM_OBJECT__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_Handle.h"
#include "XAM_VIM.h"
#include "XAsync.h"
#include "XAM_XStream_Instance.h"

/** 
 *  Wrapper class for an XStream object handle
 */
class XStream : public XAM_Handle
{
    public:    
        /** Constructor */
        XStream(xam_handle_t inHandle, XAM_Handle* inParent)
            : XAM_Handle(inHandle, inParent) 
        {
            mType = XSTREAM_HANDLE;
        }

        /** Destructor */
        virtual 
        ~XStream() {}

        virtual void 
        read (char* const ioBuffer,
              const xam_int inBufferLength, 
              xam_int* const outBytesRead)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XStream_Read(mHandle, ioBuffer, inBufferLength, outBytesRead);
            else
            {
                // If no VIM exists, use the XAM Library object equivalent
                XAM_XStream_Instance* vXStream = 
                    (XAM_XStream_Instance*)mHandle;
                *outBytesRead = vXStream->read(ioBuffer, inBufferLength);
            }
        }

        virtual void 
        write (const char* inBuffer,
               const xam_int inByteCount,
               xam_int* const outBytesWritten)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XStream_Write(mHandle, inBuffer, inByteCount, outBytesWritten);
            else
            {
                // If no VIM exists, use the XAM Library object equivalent
                XAM_XStream_Instance* vXStream = 
                    (XAM_XStream_Instance*)mHandle;
                *outBytesWritten = vXStream->write(inBuffer, inByteCount);
            }
        }

        virtual void
        seek (const xam_int inOffset,
              const xam_int inWhence)
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XStream_Seek(mHandle, inOffset, inWhence);
            else
            {
                // If no VIM exists, use the XAM Library object equivalent
                XAM_XStream_Instance* vXStream = 
                    (XAM_XStream_Instance*)mHandle;
                vXStream->seek(inOffset, (XStream_Whence)inWhence);
            }
        }

        virtual void
        tell (xam_int* const outPosition)
        {     
            if(mVim != (XAM_VIM*)0)
                mVim->XStream_Tell(mHandle, outPosition);
            else
            {
                // If no VIM exists, use the XAM Library object equivalent
                XAM_XStream_Instance* vXStream = 
                    (XAM_XStream_Instance*)mHandle;
                vXStream->tell(outPosition);
            }
        }

        virtual void
        close ()
        {
            if(mVim != (XAM_VIM*)0)
                mVim->XStream_Close(mHandle);
            else
            {
                // If no VIM exists, use the XAM Library object equivalent object
                XAM_XStream_Instance* vXStream = 
                    (XAM_XStream_Instance*) mHandle;
                if(vXStream)
                    vXStream->close();
            }
        }

        /** Abandon this handle */
        virtual void 
        abandon() 
        { 
            if(mVim != (XAM_VIM*)0)
                mVim->XStream_Abandon(mHandle);
            else
            {
                // If no VIM exists, use the XAM Library object equivalent object
                XAM_XStream_Instance* vXStream = 
                    (XAM_XStream_Instance*) mHandle;
                if(vXStream)
                    vXStream->abandon();
            }

        }

        virtual void
        asyncRead (char* ioBuffer,
                   const xam_int inBufferLength,
                   const XOPID inXOPID,
                   xasync_callback inCallback,
                   xasync_handle* const outAsyncHandle)
        { 
            xasync_callback vCallback = 
                    inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            if(mVim != (XAM_VIM*)0)
            {
                mVim->XStream_AsyncRead(mHandle,
                                        ioBuffer,
                                        inBufferLength,
                                        inXOPID,
                                        vCallback,
                                        outAsyncHandle);
            }                 
            else
            {
                // Create an XAsync object and spawn the async operation thread
                XAM_XAsync* vXAM_XAsync = new XAM_XAsync(inXOPID, 
                                                         vCallback);
                vXAM_XAsync->read((xstream_handle)this, ioBuffer, inBufferLength);
                *outAsyncHandle = (xasync_handle) vXAM_XAsync;
            }
        }

        virtual void
        asyncWrite (const char* inBuffer,
                    const xam_int inByteCount,
                    const XOPID inXOPID,
                    xasync_callback inCallback,
                    xasync_handle* const outAsyncHandle)
        { 
            xasync_callback vCallback = 
                    inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            if(mVim != (XAM_VIM*)0)
            {
                mVim->XStream_AsyncWrite(mHandle,
                                         inBuffer,
                                         inByteCount,
                                         inXOPID,
                                         vCallback,
                                         outAsyncHandle);
            }                 
            else
            {
                // Create an XAsync object and spawn the async operation thread
                XAM_XAsync* vXAM_XAsync = new XAM_XAsync(inXOPID,
                                                         vCallback);
                vXAM_XAsync->write((xstream_handle)this, inBuffer, inByteCount);
                *outAsyncHandle = (xasync_handle) vXAM_XAsync;
            }
        }

        virtual void
        asyncClose (const XOPID inXOPID,
                    xasync_callback inCallback,
                    xasync_handle* const outAsyncHandle)   
        {
            xasync_callback vCallback = 
                    inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            if(mVim != (XAM_VIM*)0)
            {
                mVim->XStream_AsyncClose(mHandle,
                                         inXOPID,
                                         vCallback,
                                         outAsyncHandle);
            }                 
            else
            {
                // Create an XAsync object and spawn the async operation thread
                XAM_XAsync* vXAM_XAsync = new XAM_XAsync(inXOPID, 
                                                         vCallback);
                vXAM_XAsync->close((xstream_handle)this);
                *outAsyncHandle = (xasync_handle) vXAM_XAsync;
            }
        }

};


#endif // __XSTREAM_OBJECT__

