#ifndef __XSYSTEM_OBJECT__
#define __XSYSTEM_OBJECT__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_Handle.h"
#include "XAM_VIM.h"
#include "XAM_XAsync.h"
#include "XAM_XAsyncHandler.h"

/** 
 *  Wrapper class for an XSystem object handle
 */
class XSystem : public XAM_Handle
{
    public:  

        /** Constructor */
        XSystem(xam_handle_t inHandle, XAM_VIM* inVim = 0)
            : XAM_Handle(inHandle, NULL /* XAM LIBRARY OBJECT*/, inVim) 
              // Only the XAM Object can be an XSystem parent
        {
            mType = XSYSTEM_HANDLE;
        }

        /** Destructor */
        virtual 
        ~XSystem() {}

        virtual xam_boolean
        getErrorToken(const xam_status inStatusCode, 
                      xam_string* const outString)
        {
            return mVim->XSystem_GetErrorToken(mHandle, inStatusCode, outString);
        }

        virtual void
        authenticate (const char* inBuffer,
                      const xam_int inByteCount,
                      xstream_handle* const outXStream)
        {
            mVim->XSystem_Authenticate(mHandle, inBuffer, inByteCount, outXStream);
        }

        virtual void
        deleteXSet (const xam_xuid inXUID)
        {
            mVim->XSystem_DeleteXSet(mHandle, inXUID);
        }

        virtual void
        holdXSet (const xam_xuid inXUID,
                  const xam_string inHoldID)
        {
            mVim->XSystem_HoldXSet(mHandle, inXUID, inHoldID);
        }

        virtual void
        releaseXSet (const xam_xuid inXUID,
                     const xam_string inHoldID)
        { 
            mVim->XSystem_ReleaseXSet(mHandle, inXUID, inHoldID);
        }

        virtual void
        accessXSet (const xam_xuid inXUID,
                    const xam_int inMode,
                    xam_boolean* const outIsAccessible)
        {
            mVim->XSystem_AccessXSet(mHandle, inXUID, inMode, outIsAccessible);
        }

        virtual void
        isXSetRetained (const xam_xuid inXUID,                   
                        xam_boolean* const outIsRetained)
        {
            mVim->XSystem_IsXSetRetained(mHandle, inXUID, outIsRetained);
        }

        virtual void
        getXSetAccessTime (const xam_xuid inXUID,
                           xam_datetime* const outAccessTime)
        {
            mVim->XSystem_GetXSetAccessTime(mHandle, inXUID, outAccessTime);
        }

        virtual void
        createXSet (const xam_string inMode, xset_handle* const outXSet)
        {
            mVim->XSystem_CreateXSet(mHandle, inMode, outXSet);
        }

        virtual void
        openXSet (const xam_xuid inXUID,
                  const xam_string inMode,
                  xset_handle* const outXSet)
        {
            mVim->XSystem_OpenXSet(mHandle, inXUID, inMode, outXSet);
        }

        virtual void
        copyXSet (const xam_xuid inXUID,
                  const xam_string inMode,
                  xset_handle* const outXSet)
        {  
            mVim->XSystem_CopyXSet(mHandle, inXUID, inMode, outXSet);
        }

        virtual void
        close ()
        {
            mVim->XSystem_Close(mHandle);
        }

        /** Abandon this handle */
        virtual void 
        abandon() 
        { 
            mVim->XSystem_Abandon(mHandle);
        }

        virtual void
        containsField (const xam_string inName,
                       xam_boolean* const outExists)
        {
            mVim->XSystem_ContainsField(mHandle, inName, outExists);
        }

        virtual void 
        getFieldType(const xam_string inName, xam_string* const outType)
        {
            mVim->XSystem_GetFieldType(mHandle, inName, outType);
        }

        virtual void 
        getFieldLength(const xam_string inName, xam_int* const outLen)
        { 
            mVim->XSystem_GetFieldLength(mHandle, inName, outLen);
        }

        virtual void 
        getFieldBinding(const xam_string inName, xam_boolean* const outBinding)
        {
            // There's no VIM API call for XSystem getBinding, so check if
            // the field exists and throw the appropriate error.
            xam_boolean vExists = false;
            mVim->XSystem_ContainsField(mHandle, inName, &vExists);
            if(!vExists)
            {
                XAMFieldNotFoundException e;
                throw e;
            }

            // All XSystem fields are nonbinding
            // (There's no VIM API call for XSystem field binding defined!)
            *outBinding = false;
        }

        virtual void 
        getFieldReadOnly(const xam_string inName, xam_boolean* const outReadOnly)
        {
            mVim->XSystem_GetFieldReadOnly(mHandle, inName, outReadOnly);
        }

        virtual void 
        deleteField(const xam_string inName)
        {
            mVim->XSystem_DeleteField(mHandle, inName);
        }

        virtual void 
        createBoolean(const xam_string inName, 
                      const xam_boolean inBinding, 
                      const xam_boolean inValue)
        {
            mVim->XSystem_CreateBoolean(mHandle, inName, inBinding, inValue);
        }

        virtual void 
        createInt(const xam_string inName, 
                  const xam_boolean inBinding, 
                  const xam_int inValue)
        {
            mVim->XSystem_CreateInt(mHandle, inName, inBinding, inValue);
        }    
        
        virtual void 
        createDouble(const xam_string inName, 
                    const xam_boolean inBinding, 
                    const xam_double inValue)
        {
            mVim->XSystem_CreateDouble(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        createXUID(const xam_string inName, 
                   const xam_boolean inBinding, 
                   const xam_xuid inValue)
        {
            mVim->XSystem_CreateXUID(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        createString(const xam_string inName, 
                     const xam_boolean inBinding, 
                     const xam_string inValue)
        {
            mVim->XSystem_CreateString(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        createDatetime(const xam_string inName, 
                       const xam_boolean inBinding, 
                       const xam_datetime inValue)
        {
            mVim->XSystem_CreateDatetime(mHandle, inName, inBinding, inValue);
        }    

        virtual void 
        setBoolean(const xam_string inName, const xam_boolean inValue)
        {
            mVim->XSystem_SetBoolean(mHandle, inName, inValue);
        }    

        virtual void 
        setInt(const xam_string inName, const xam_int inValue)
        { 
            mVim->XSystem_SetInt(mHandle, inName, inValue);
        }  

        virtual void 
        setDouble(const xam_string inName, const xam_double inValue)
        {
            mVim->XSystem_SetDouble(mHandle, inName, inValue);
        }  

        virtual void 
        setXUID(const xam_string inName, const xam_xuid inValue)
        {
            mVim->XSystem_SetXUID(mHandle, inName, inValue);
        }  

        virtual void 
        setString(const xam_string inName, const xam_string inValue)
        {  
            mVim->XSystem_SetString(mHandle, inName, inValue);
        }  

        virtual void 
        setDatetime(const xam_string inName, const xam_datetime inValue)
        {
            mVim->XSystem_SetDatetime(mHandle, inName, inValue);
        }  

        virtual void 
        getBoolean(const xam_string inName, xam_boolean* const outValue)
        {
            mVim->XSystem_GetBoolean(mHandle, inName, outValue);
        }  

        virtual void 
        getInt(const xam_string inName, xam_int* const outValue)
        {
            mVim->XSystem_GetInt(mHandle, inName, outValue);
        }  

        virtual void 
        getDouble(const xam_string inName, xam_double* const outValue)
        {
            mVim->XSystem_GetDouble(mHandle, inName, outValue);
        }  

#ifdef SUN
        using XAM_Interface::getXUID; // add base class getXUID definitions
#endif
        virtual void 
        getXUID(const xam_string inName, xam_xuid* const outValue)
        {
            mVim->XSystem_GetXUID(mHandle, inName, outValue);
        }  

        virtual void 
        getString(const xam_string inName, xam_string* const outValue)
        {
            mVim->XSystem_GetString(mHandle, inName, outValue);
        }  

        virtual void 
        getDatetime(const xam_string inName, xam_datetime* const outValue)
        {
            mVim->XSystem_GetDatetime(mHandle, inName, outValue);
        }  

        virtual void 
        createXStream(const xam_string inName, 
                      const xam_boolean inBinding, 
                      const xam_string inType, 
                      xstream_handle* const outXStream)
        {
            mVim->XSystem_CreateXStream(mHandle, 
                                        inName, 
                                        inBinding, 
                                        inType, 
                                        outXStream);
        }  

        virtual void 
        openXStream(const xam_string inName, 
                    const xam_string inMode, 
                    xstream_handle* const outXStream)
        {
            mVim->XSystem_OpenXStream(mHandle, inName, inMode, outXStream);
        }  

        virtual void 
        openFieldIterator(const xam_string inPattern, 
                            xiterator_handle* const outHandle)
        {
            mVim->XSystem_OpenFieldIterator(mHandle, inPattern, outHandle);
        }  

        virtual void
        asyncOpenXSet (const xam_xuid inXUID,
                       const xam_string inMode,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)
        {
            xasync_callback vCallback = 
                inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            mVim->XSystem_AsyncOpenXSet(mHandle, 
                                        inXUID, 
                                        inMode, 
                                        inXOPID, 
                                        vCallback,
                                        outAsyncHandle);
        }

        virtual void
        asyncCopyXSet (const xam_xuid inXUID,
                       const xam_string inMode,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)
        {
            xasync_callback vCallback = 
                inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            mVim->XSystem_AsyncCopyXSet(mHandle, 
                                        inXUID, 
                                        inMode, 
                                        inXOPID, 
                                        vCallback,
                                        outAsyncHandle);
        }

        virtual void
        asyncOpenXStream (const xam_string inName,
                          const xam_string inMode,
                          const XOPID inXOPID,
                          xasync_callback inCallback,
                          xasync_handle* const outAsyncHandle)        
        { 
            xasync_callback vCallback = 
                inCallback ? XAM_AsyncCallback : (xasync_callback) 0;

            mVim->XSystem_AsyncOpenXStream(mHandle,
                                           inName,
                                           inMode, 
                                           inXOPID, 
                                           vCallback,
                                           outAsyncHandle);
        } 
};

#endif // __XSYSTEM_OBJECT__



