#ifndef __XAM_ROUTER__
#define __XAM_ROUTER__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"

#include "XAM_VIM.h"
#include "XAM_VIM_Manager.h"
#include "XAM_HandleManager.h"

/** 
 *  Static class responsible for routing calls from the XAM API to 
 *  the appropriate VIM (or to the singleton XAM object, as appropriate).
 *
 *  All new xam_handles are scoped by wrapping them with the
 *  appropriate derived XAM_Handle object. These wrapper objects are 
 *  deallocated when the handle is closed.
 * 
 *  All method names map to the corresponding XAM API equivalent.
 *  See the XAM API documentation for details on each method.
 */
class XAM_Router : public XAM_HandleManager
{
    public:

        // ================================================================
        // methods for evaluating xam_status 
        // ================================================================

         /** @see xam.h for detailed description */
        static xam_boolean 
        XAM_GetErrorToken(const xam_handle_t inHandle, 
                          const xam_status inStatusCode, 
                          xam_string* const outToken);

        // ================================================================
        // methods for the XIterator 
        // ================================================================

         /** @see xam.h for detailed description */
        static xam_status
        XAM_OpenFieldIterator (const xam_handle_t inHandle,  
                               const xam_string inPattern,
                               xiterator_handle* const outIterator);

         /** @see xam.h for detailed description */
        static xam_status
        XIterator_HasNext (const xiterator_handle inHandle, 
                           xam_boolean* const outValue);

         /** @see xam.h for detailed description */
        static xam_status 
        XIterator_Next (const xiterator_handle inHandle, 
                        xam_string* const outName);
      
         /** @see xam.h for detailed description */
        static xam_status 
        XIterator_Close (xiterator_handle inHandle);


        // ================================================================
        // methods for managing XAM Fields (properties or XStreams)
        // ================================================================

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_ContainsField (const xam_handle_t inHandle,
                           const xam_string inName,
                           xam_boolean* const outExists);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetFieldAsBinding (const xset_handle inHandle,
                               const xam_string inName);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetFieldAsNonbinding (const xset_handle inHandle,
                                  const xam_string inName);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetFieldType (const xam_handle_t inHandle,
                          const xam_string inName,
                          xam_string* const outType);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetFieldLength (const xam_handle_t inHandle,
                            const xam_string inName,
                            xam_int* const outLength);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetFieldBinding (const xam_handle_t inHandle,
                             const xam_string inName,
                             xam_boolean* const outFixed);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetFieldReadOnly (const xam_handle_t inHandle,
                              const xam_string inName,
                              xam_boolean* const outReadOnly);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_DeleteField (const xam_handle_t inHandle,
                         const xam_string inName);

        // ================================================================
        // methods for managing property fields
        // ================================================================

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateBoolean (const xam_handle_t inHandle,
                           const xam_string inName,
                           const xam_boolean inBinding,
                           const xam_boolean inValue);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateInt (const xam_handle_t inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_int inValue);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateDouble (const xam_handle_t inHandle,
                         const xam_string inName,
                         const xam_boolean inBinding,
                         const xam_double inValue);

         /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateXUID (const xam_handle_t inHandle,
                        const xam_string inName,
                        const xam_boolean inBinding,
                        const xam_xuid inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateString (const xam_handle_t inHandle,
                          const xam_string inName,
                          const xam_boolean inBinding,
                          const xam_string inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateDatetime (const xam_handle_t inHandle,
                            const xam_string inName,
                            const xam_boolean inBinding,
                            const xam_datetime inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetBoolean (const xam_handle_t inHandle,
                        const xam_string inName,
                        const xam_boolean inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetInt (const xam_handle_t inHandle,
                    const xam_string inName,
                    const xam_int inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetDouble (const xam_handle_t inHandle,
                     const xam_string inName,
                     const xam_double inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetXUID (const xam_handle_t inHandle,
                     const xam_string inName,
                     const xam_xuid inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetString (const xam_handle_t inHandle,
                       const xam_string inName,
                       const xam_string inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_SetDatetime (const xam_handle_t inHandle,
                         const xam_string inName,
                         const xam_datetime inValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetBoolean (const xam_handle_t inHandle,
                        const xam_string inName,
                        xam_boolean* const outValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetInt (const xam_handle_t inHandle,
                    const xam_string inName,
                    xam_int* const outValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetDouble (const xam_handle_t inHandle,
                      const xam_string inName,
                      xam_double* const outValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetXUID (const xam_handle_t inHandle,
                     const xam_string inName,
                     xam_xuid* const outValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetString (const xam_handle_t inHandle,
                       const xam_string inName,
                       xam_string* const outValue);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_GetDatetime (const xam_handle_t inHandle,
                         const xam_string inName,
                         xam_datetime* const outValue);

        // ================================================================
        // methods for XStreams and XStream fields
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_CreateXStream (const xam_handle_t inHandle,
                           const xam_string inName,
                           const xam_boolean inBinding,
                           const xam_string inType,
                           xstream_handle* const outXStream);

        /** @see xam.h for detailed description */
        static xam_status 
        XAM_OpenXStream (const xam_handle_t inHandle,
                         const xam_string inName,
                         const xam_string inMode,
                         xstream_handle* const outXStream);

        /** @see xam.h for detailed description */
        static xam_status 
        XStream_Read (const xstream_handle inHandle,
                      char* const ioBuffer,
                      const xam_int inBufferLength,
                      xam_int* const outBytesRead);

        /** @see xam.h for detailed description */
        static xam_status 
        XStream_Write (const xstream_handle inHandle,
                       const char* inBuffer,
                       const xam_int inByteCount,
                       xam_int* const outByteWritten);

        /** @see xam.h for detailed description */
        static xam_status 
        XStream_Seek (const xstream_handle inHandle,
                      const xam_int inOffset,
                      const xam_int inWhence);

        /** @see xam.h for detailed description */
        static xam_status 
        XStream_Tell (const xstream_handle inHandle,
                      xam_int* const outPosition);

        /** @see xam.h for detailed description */
        static xam_status
        XStream_Abandon(const xstream_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status 
        XStream_Close (xstream_handle inHandle);

        // ================================================================
        // methods for managing the connection to a XAM storage system
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XAMLibrary_Connect (const xam_string inXRI,
                            xsystem_handle* const outHandle);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_Authenticate (const xsystem_handle inHandle,
                              const char* inBuffer,
                              const xam_int inByteCount,
                              xstream_handle* const outXStream);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_Close (const xsystem_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_Abandon (const xsystem_handle inHandle);

        // ================================================================
        // methods for XSet management
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_DeleteXSet (const xsystem_handle inHandle,
                            const xam_xuid inXUID);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_HoldXSet (const xsystem_handle inHandle,
                          const xam_xuid inXUID,
                          const xam_string inHoldID);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_ReleaseXSet (const xsystem_handle inHandle,
                             const xam_xuid inXUID,
                             const xam_string inHoldID);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_AccessXSet (const xsystem_handle inHandle,
                            const xam_xuid inXUID,
                            const xam_int inMode,
                            xam_boolean* const outIsAccessible);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_IsXSetRetained (const xsystem_handle inHandle,
                                const xam_xuid inXUID,
                                xam_boolean* const outIsRetained);

        /** @see xam.h for detailed description */
        static xam_status
        XSystem_GetXSetAccessTime (const xsystem_handle inHandle,
                                   const xam_xuid inXUID,
                                   xam_datetime* const outAccessTime);

        // ================================================================
        // method for creating an XSet
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_CreateXSet (const xsystem_handle inHandle,
                            const xam_string inMode, 
                            xset_handle* const outXSet);

        // ================================================================
        // methods for XSet policy management
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status
        XSet_ApplyAccessPolicy (const xset_handle inHandle,
                                const xam_boolean inBinding,
                                const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_ResetAccessFields (const xset_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_ApplyManagementPolicy (const xset_handle inHandle,
                                    const xam_boolean inBinding,
                                    const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_ResetManagementFields (const xset_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_CreateRetention (const xset_handle inHandle,
                              const xam_boolean inBinding,
                              const xam_string inRetentionID);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_SetRetentionEnabledFlag (const xset_handle inHandle,
                                      const xam_string inRetentionID,
                                      const xam_boolean inBinding,
                                      const xam_boolean inEnabled);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_ApplyRetentionEnabledPolicy (const xset_handle inHandle,
                                          const xam_string inRetentionID,
                                          const xam_boolean inBinding,
                                          const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_SetRetentionDuration (const xset_handle inHandle,
                                   const xam_string inRetentionID,
                                   const xam_boolean inBinding,
                                   const xam_int inDuration);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_ApplyRetentionDurationPolicy (const xset_handle inHandle,
                                           const xam_string inRetentionID,
                                           const xam_boolean inBinding,
                                           const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_SetRetentionStarttime (const xset_handle inHandle,
                                    const xam_string inRetentionID,
                                    const xam_boolean inBinding);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_SetBaseRetention (const xset_handle inHandle,
                               const xam_boolean inBinding,
                               const xam_int inDuration);

        /** @see xam.h for detailed description */
        static xam_status
        XSet_ApplyBaseRetentionPolicy (const xset_handle inHandle,
                                       const xam_boolean inBinding,
                                       const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_ApplyAutoDeletePolicy (const xset_handle inHandle,
                                    const xam_boolean inBinding,
                                    const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_SetAutoDelete (const xset_handle inHandle,
                            const xam_boolean inBinding,
                            const xam_boolean inAutoDelete);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_ApplyShredPolicy (const xset_handle inHandle,
                               const xam_boolean inBinding,
                               const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_SetShred (const xset_handle inHandle,
                       const xam_boolean inBinding,
                       const xam_boolean inShred);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_ApplyStoragePolicy (const xset_handle inHandle,
                                 const xam_boolean inBinding,
                                 const xam_string inPolicy);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_GetActualRetentionDuration (const xset_handle inHandle,
                                         const xam_string inRetentionID,
                                         xam_int* const outDuration);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_GetActualRetentionEnabled (const xset_handle inHandle,
                                        const xam_string inRetentionID,
                                        xam_boolean* const outEnabled);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_GetActualAutoDelete (const xset_handle inHandle,
                                  xam_boolean* const outEnabled);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_GetActualShred (const xset_handle inHandle,
                             xam_boolean* const outEnabled);

        // ================================================================
        // methods for editing an XSet
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_OpenXSet (const xsystem_handle inHandle,
                          const xam_xuid inXUID,
                          const xam_string inMode,
                          xset_handle* const outXSet);

        /** @see xam.h for detailed description */
        static xam_status 
        XSystem_CopyXSet (const xsystem_handle inHandle,
                          const xam_xuid inXUID,
                          const xam_string inMode,
                          xset_handle* const outXSet);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_Commit (const xset_handle inHandle,
                     xam_xuid* const outXUID);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_Close (const xset_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_Abandon (const xset_handle inHandle);

        // ================================================================
        // methods for XSet migration
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_OpenExportXStream (const xset_handle inHandle,
                                xstream_handle* const outXStream);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_OpenImportXStream (const xset_handle inHandle,
                                xstream_handle* const outXStream);

        // ================================================================
        // methods for job control
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_SubmitJob (const xset_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status 
        XSet_HaltJob (const xset_handle inHandle);

        // ================================================================
        //  methods for async i/o
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status
        XSystem_AsyncOpenXSet (const xsystem_handle inHandle,
                               const xam_xuid inXUID,
                               const xam_string inMode,
                               const XOPID inXOPID,
                               xasync_callback inCallback,
                               xasync_handle* const outAsyncHandle); 

        /** @see xam.h for detailed description */
        static xam_status
        XSystem_AsyncCopyXSet (const xsystem_handle inHandle,
                               const xam_xuid inXUID,
                               const xam_string inMode,
                               const XOPID inXOPID,
                               xasync_callback inCallback,
                               xasync_handle* const outAsyncHandle); 

        /** @see xam.h for detailed description */
        static xam_status
        XAM_AsyncOpenXStream (const xsystem_handle inHandle,
                               const xam_string inName,
                               const xam_string inMode,
                               const XOPID inXOPID,
                               xasync_callback inCallback,
                               xasync_handle* const outAsyncHandle); 

        /** @see xam.h for detailed description */
        static xam_status
        XStream_AsyncRead (const xstream_handle inHandle,
                           char* const ioBuffer,
                           const xam_int inBufferLength,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle);

        /** @see xam.h for detailed description */
        static xam_status
        XStream_AsyncWrite (const xstream_handle inHandle,
                            const char* inBuffer,
                            const xam_int inByteCount,
                            const XOPID inXOPID,
                            xasync_callback inCallback,
                            xasync_handle* const outAsyncHandle); 

        /** @see xam.h for detailed description */
        static xam_status
        XSet_AsyncCommit (const xset_handle inHandle,
                          const XOPID inXOPID,
                          xasync_callback inCallback,
                          xasync_handle* const outAsyncHandle); 

        /** @see xam.h for detailed description */
        static xam_status
        XStream_AsyncClose (const xstream_handle inHandle,
                            const XOPID inXOPID,
                            xasync_callback inCallback,
                            xasync_handle* const outAsyncHandle); 

        // ================================================================
        // methods for managing asynchronous operations
        // ================================================================

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_Halt (const xasync_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_Close (const xasync_handle inHandle);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_IsComplete (const xasync_handle inHandle,
                               xam_boolean* const outIsComplete);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetStatus (const xasync_handle inHandle,
                            xam_status* const outStatus);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetXOPID (const xasync_handle inHandle,
                           XOPID* const outXOPID);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetXSet (const xasync_handle inHandle,
                          xset_handle* const outXSet);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetXStream (const xasync_handle inHandle,
                           xstream_handle* const outXStream);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetXUID (const xasync_handle inHandle,
                          xam_xuid* const outXUID);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetBytesRead (const xasync_handle inHandle,
                               xam_int* const outBytesRead);

        /** @see xam.h for detailed description */
        static xam_status
        XAsync_GetBytesWritten (const xasync_handle inHandle,
                                  xam_int* const outBytesWritten);

};


#endif // __XAM_ROUTER__

