//////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2009, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////
#include "RemoteVIM.h"
#include "XAM_POSIX.h"
#include "XAM_Mutex.h"

static XAM_Mutex sMAIN1_MUTEX;
static XAM_Mutex sMAIN2_MUTEX;
static XAM_Mutex sMAIN3_MUTEX;
static XAM_Mutex sCACHE_MUTEX;

////////////////////////////////////////////////////////////////////////////////
// internal utilities
////////////////////////////////////////////////////////////////////////////////
void 
RemoteVIM::ExpandDirectBuffers (void)
{
	xam_int vNumBlocks = mNumAllocated + mBlockSize;
	xam_int vSize = vNumBlocks * sizeof(xsystem_handle);
	mHandle = (xsystem_handle*)XAM_POSIX::xam_realloc(mHandle, vSize);
	mRemoteHandle = (xsystem_handle*)XAM_POSIX::xam_realloc(mRemoteHandle, vSize);
	vSize = vNumBlocks * sizeof(std::string*);
	mAddress = (std::string**)XAM_POSIX::xam_realloc(mAddress, vSize);
	vSize = vNumBlocks * sizeof(XferBuffer*);
	mCache = (XferBuffer**)XAM_POSIX::xam_realloc(mCache, vSize);
	vSize = vNumBlocks * sizeof(xam_boolean);
	mActive = (xam_boolean*)XAM_POSIX::xam_realloc(mActive, vSize);
	for ( xam_int ix=mNumAllocated; ix<vNumBlocks; ix++ )
	{
		mHandle[ix] = (xsystem_handle)0;
		mRemoteHandle[ix] = (xsystem_handle)0;
		mAddress[ix] = (std::string*)0;
		mCache[ix] = (XferBuffer*)0;
		mActive[ix] = (xam_boolean)false;
	}
	mNumAllocated = vNumBlocks;
}

void 
RemoteVIM::ExpandIndirectBuffers (void)
{
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	xam_int vNumBlocks = mNumIndirectAllocated + mBlockSize;
	xam_int vSize = vNumBlocks * sizeof(xam_handle_t);
	mIndirectHandle = (xam_handle_t*)XAM_POSIX::xam_realloc(mIndirectHandle, vSize);
	mReferencedHandle = (xsystem_handle*)XAM_POSIX::xam_realloc(mReferencedHandle, vSize);
	vSize = vNumBlocks * sizeof(xasync_callback);
	mCallback = (xasync_callback*)XAM_POSIX::xam_realloc(mCallback, vSize);
	vSize = vNumBlocks * sizeof(xam_boolean);
	mIndirectActive = (xam_boolean*)XAM_POSIX::xam_realloc(mIndirectActive, vSize);
	vSize = vNumBlocks * sizeof(xam_int);
	mBuffersize = (xam_int*)XAM_POSIX::xam_realloc(mBuffersize, vSize);
	vSize = vNumBlocks * sizeof(char*);
	mBuffer = (char**)XAM_POSIX::xam_realloc(mBuffer, vSize);
	for ( xam_int ix=mNumIndirectAllocated; ix<vNumBlocks; ix++ )
	{
		mIndirectHandle[ix] = (xsystem_handle)0;
		mCallback[ix] = (xasync_callback)0;
		mReferencedHandle[ix] = (xsystem_handle)0;
		mIndirectActive[ix] = (xam_boolean)false;
		mBuffersize[ix] = 0;
		mBuffer[ix] = (char*)0;
	}
	mNumIndirectAllocated = vNumBlocks;
}

void 
RemoteVIM::ExpandVIMBuffers (void)
{
	XAM_CriticalSection vCritical(&sMAIN3_MUTEX);
	xam_int vNumBlocks = mNumVIMAllocated + mBlockSize;
	xam_int vSize = vNumBlocks * sizeof(SingleRemoteVIM*);
	mSingleVIM = (SingleRemoteVIM**)XAM_POSIX::xam_realloc(mSingleVIM, vSize);
	vSize = vNumBlocks * sizeof(xam_boolean);
	mIsVIMReserved = (xam_boolean*)XAM_POSIX::xam_realloc(mIsVIMReserved, vSize);
	for ( xam_int ix=mNumVIMAllocated; ix<vNumBlocks; ix++ )
	{
		mSingleVIM[ix] = (SingleRemoteVIM*)0;
		mIsVIMReserved[ix] = (xam_boolean)false;
	}
	mNumVIMAllocated = vNumBlocks;
}

xam_int 
RemoteVIM::FindDirectIndex(const xsystem_handle inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xam_int vIndex = 0;
	for ( ; vIndex<mNumAllocated; vIndex++ )
	{
		if ( mActive[vIndex] && mHandle[vIndex] == inHandle )
			break;
	}
	return (vIndex<mNumAllocated) ? vIndex : -1;
}

xam_int
RemoteVIM::FindIndirectIndex(const xam_handle_t inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	xam_int vIndex = 0;
	for ( ; vIndex<mNumIndirectAllocated; vIndex++ )
	{
		if ( mIndirectActive[vIndex] && mIndirectHandle[vIndex] == inHandle )
			break;
	}
	if ( vIndex >= mNumIndirectAllocated )
		vIndex = -1;
	else
		vIndex = FindDirectIndex(mReferencedHandle[vIndex]);
	return vIndex;
}

xam_int 
RemoteVIM::FindAvailableVIMIndex(void)
{
	XAM_CriticalSection vCritical(&sMAIN3_MUTEX);
	xam_int vIndex = 0;
	for ( ; vIndex<mNumVIMAllocated; vIndex++ )
	{
		if ( mSingleVIM[vIndex] == (SingleRemoteVIM*)0 )
			continue;
		if ( mIsVIMReserved[vIndex] == (xam_boolean)true )
			continue;

		break;
	}
	return (vIndex<mNumVIMAllocated) ? vIndex : -1;
}

////////////////////////////////////////////////////////////////////////////////
// manage the linkage/association between VIM and xsystem_handle
////////////////////////////////////////////////////////////////////////////////
void 
RemoteVIM::RegisterXSystem(const xsystem_handle inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xam_int vIndex = FindDirectIndex(inHandle);
	if ( vIndex == (-1) ) // not found, then add it to the list
	{
		// allocate a new block of entries if needed
		if ( mNumAllocated <= mNumActive )
			ExpandDirectBuffers();

		// insert the xsystem_handle into the table
		for ( xam_int ix = 0; ix<mNumAllocated; ix++ )
		{
			if ( mActive[ix] == false )
			{
				mAddress[ix] = (std::string*)0;
				mHandle[ix] = inHandle;
				mRemoteHandle[ix] = (xsystem_handle)0;
				mActive[ix] = true;
				mNumActive++;
				break;
			}
		}				
	}
}

void
RemoteVIM::BindToRemoteXSystem(const xsystem_handle inLocalHandle,
	                           const xsystem_handle inRemoteHandle)
{
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xam_int vIndex = FindDirectIndex(inLocalHandle);
	if ( vIndex != (-1) )
		mRemoteHandle[vIndex] = inRemoteHandle;
}

void 
RemoteVIM::BindToAddress (const xsystem_handle inHandle, const std::string inAddress)
{
	// find the handle in the list if it already exists
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xam_int vIndex = FindDirectIndex(inHandle);
	if ( vIndex != (-1) )
	{
		if ( mAddress[vIndex] != (std::string*)0 )
			delete mAddress[vIndex];
		mAddress[vIndex] = new std::string(inAddress);
	}
	else // if not found, then add it to the list
	{
		// allocate a new block of entries if needed
		if ( mNumAllocated <= mNumActive )
			ExpandDirectBuffers();

		// insert the xsystem_handle into the table
		for ( xam_int ix = 0; ix<mNumAllocated; ix++ )
		{
			if ( mActive[ix] == false )
			{
				mAddress[ix] = new std::string(inAddress);
				mHandle[ix] = inHandle;
				mRemoteHandle[ix] = (xsystem_handle)0;
				mActive[ix] = true;
				mNumActive++;
				break;
			}
		}				
	}
}

void
RemoteVIM::HaltPOLL(void)
{
	mInfo.running = false;
}

static XAM_THREAD_FUNCTION_PTR
async_thread_method(void *inInfo)
{
	XAM_ThreadInfo* vInfo = (XAM_ThreadInfo*)inInfo;
	if ( vInfo )
	{
		RemoteVIM* vVIM = (RemoteVIM*)vInfo->data;
		if ( vVIM )
		{
			while ( vVIM->POLL() )
				XAM_POSIX::xam_sleep(10);
			vVIM->HaltPOLL();
		}
	}
    XAM_POSIX::xam_thread_exit();
    return 0;
}

void 
RemoteVIM::BindToHandle (const xam_handle_t inHandle, 
                          const xasync_callback inCallback,
                          const xsystem_handle inXSystemHandle)
{
	// find the handle in the list if it already exists
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	xam_int vIndex = FindIndirectIndex(inHandle);
	if ( vIndex == (-1) ) // not found, then add it to the list
	{
		// allocate a new block of entries if needed
		if ( mNumIndirectAllocated <= mNumIndirectActive )
			ExpandIndirectBuffers();

		// insert the handle into the table
		for ( xam_int ix = 0; ix<mNumIndirectAllocated; ix++ )
		{
			if ( mIndirectActive[ix] == false )
			{
				mIndirectHandle[ix] = inHandle;
				mCallback[ix] = inCallback;
				mReferencedHandle[ix] = inXSystemHandle;
				mIndirectActive[ix] = true;
				mNumIndirectActive++;
				break;
			}
		}

		// start a thread to process the response
		if ( inCallback != (xasync_callback)0 && mInfo.running == false )
		{
			mInfo.data = (void*)this;
			if ( XAM_POSIX::xam_thread_create(mInfo, async_thread_method) == 0 )
				mInfo.running = true;
		}
	}
}

void 
RemoteVIM::BindToHandle (const xam_handle_t inHandle, 
                          const xsystem_handle inXSystemHandle)
{
	BindToHandle(inHandle, 0, inXSystemHandle);
}

void 
RemoteVIM::BindToBuffer (const xam_handle_t inHandle, 
                         xam_int inBuffersize,
                         char* inBuffer)
{
	// find the handle in the list if it already exists
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	xam_int vIndex = FindIndirectIndex(inHandle);
	if ( vIndex != (-1) ) // only works if found
	{
		mBuffersize[vIndex] = inBuffersize;
		mBuffer[vIndex] = inBuffer;
	}
}

void 
RemoteVIM::GetBuffer (const xam_handle_t inHandle, 
                      xam_int* outBuffersize,
                      char** outBuffer)
{
	// find the handle in the list if it already exists
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	xam_int vIndex = FindIndirectIndex(inHandle);
	if ( vIndex != (-1) ) // only works if found
	{
		*outBuffersize = mBuffersize[vIndex];
		*outBuffer = mBuffer[vIndex];

		mBuffersize[vIndex] = 0;
		mBuffer[vIndex] = (char*)0;
	}
}

xsystem_handle
RemoteVIM::GetRegisteredXSystem(const xam_handle_t inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	xsystem_handle vHandle = (xsystem_handle)0;
	xam_int vIndex = FindIndirectIndex(inHandle);
	if ( vIndex != (-1) )
		vHandle = mReferencedHandle[vIndex];
	return vHandle;
}

xsystem_handle 
RemoteVIM::GetRemoteHandle(const xsystem_handle inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xsystem_handle vRemoteHandle = (xsystem_handle)0;
	xam_int vIndex = FindDirectIndex(inHandle);
	if ( vIndex != (-1) )
		vRemoteHandle = mRemoteHandle[vIndex];
	return vRemoteHandle;
}

XferBuffer* 
RemoteVIM::GetCache(const xsystem_handle inHandle)
{
	XAM_CriticalSection vCritical(&sCACHE_MUTEX);
	XferBuffer* vCache = (XferBuffer*)0;
	xam_int vIndex = FindDirectIndex(inHandle);
	if ( vIndex != (-1) && (mAddress[vIndex] == (std::string*)0) )
	{
		// create a cache if one does not exist
		if ( mCache[vIndex] == (XferBuffer*)0 )
			mCache[vIndex] = new XferBuffer();

		// get the cache associated with the handle
		vCache = mCache[vIndex];
	}
	return vCache;
}

void
RemoteVIM::DeleteCache(const xsystem_handle inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xam_int vIndex = FindDirectIndex(inHandle);
	if ( vIndex != (-1) && mCache[vIndex] != (XferBuffer*)0 )
	{
		XAM_CriticalSection vCritical(&sCACHE_MUTEX);
		delete mCache[vIndex];
		mCache[vIndex] = (XferBuffer*)0;
	}
}

void
RemoteVIM::ReleaseDirectBinding(const xsystem_handle inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	for ( xam_int ix = 0; ix<mNumAllocated; ix++ )
	{
		if ( mActive[ix] && mHandle[ix] == inHandle )
		{
			mActive[ix] = false;
			mHandle[ix] = (xsystem_handle)0;
			mRemoteHandle[ix] = (xsystem_handle)0;
			if ( mCache[ix] )
				delete mCache[ix];
			mCache[ix] = (XferBuffer*)0;
			if ( mAddress[ix] )
				delete mAddress[ix];
			mAddress[ix] = (std::string*)0;
			mNumActive--;
			break;
		}
	}				
}

void
RemoteVIM::ReleaseIndirectBinding(const xam_handle_t inHandle)
{
	XAM_CriticalSection vCritical(&sMAIN2_MUTEX);
	for ( xam_int ix = 0; ix<mNumIndirectAllocated; ix++ )
	{
		if ( mIndirectActive[ix] && mIndirectHandle[ix] == inHandle )
		{
			mIndirectActive[ix] = false;
			mCallback[ix] = (xasync_callback)0;
			mIndirectHandle[ix] = (xam_handle_t)0;
			mReferencedHandle[ix] = (xsystem_handle)0;
			mBuffersize[ix] = 0;
			mBuffer[ix] = (char*)0;
			mNumIndirectActive--;
			break;
		}
	}				
}

/////////////////////////////////////////////////////////////////////////////////
// interact with a single remote VIM
////////////////////////////////////////////////////////////////////////////////
SingleRemoteVIM& 
RemoteVIM::GetVIM(void)
{
	XAM_CriticalSection vCritical(&sMAIN3_MUTEX);
	SingleRemoteVIM* vVIM = (SingleRemoteVIM*)0;
	xam_int vIndex = FindAvailableVIMIndex();
	if ( vIndex == (-1) ) // need a new VIM
	{
		if ( mNumVIMActive >= mNumVIMAllocated )
		{
			vIndex = mNumVIMAllocated;
			ExpandVIMBuffers();
		}
		else // there should be space... find it
		{
			for ( xam_int ix=0; ix<mNumVIMAllocated; ix++ )
			{
				if ( mSingleVIM[ix] == (SingleRemoteVIM*)0 )
				{
					vIndex = ix;
					break;
				}
			}
		}
		if ( vIndex != (-1) )
		{
			mSingleVIM[vIndex] = new SingleRemoteVIM();
			mNumVIMActive++;
		}
	}
	if ( vIndex != (-1) )
	{
		mIsVIMReserved[vIndex] = (xam_boolean)true;
		vVIM = mSingleVIM[vIndex];
	}
	return *vVIM;
}

SingleRemoteVIM&
RemoteVIM::GetVIM(const xsystem_handle inXSystemHandle, 
                  const char* const inCommand,
                  const xam_handle_t inOpHandle)
{
	SingleRemoteVIM& vVIM = GetVIM();

	XAM_CriticalSection vCritical(&sMAIN1_MUTEX);
	xam_int vIndex = FindDirectIndex(inXSystemHandle);
	if ( vIndex != (-1) )
	{
		vVIM.SetCommand(*(mAddress[vIndex]), inCommand);
		vVIM.ResetArgs(false);
		if ( inOpHandle == (xam_handle_t)0 )
			vVIM.AppendArg("handle", mRemoteHandle[vIndex]);
		else
			vVIM.AppendArg("handle", inOpHandle);
	}
	return vVIM;
}

void
RemoteVIM::ReleaseVIM(SingleRemoteVIM& inVIM)
{
	XAM_CriticalSection vCritical(&sMAIN3_MUTEX);
	for ( xam_int vIndex = 0; vIndex<mNumVIMAllocated; vIndex++ )
	{
		if ( mSingleVIM[vIndex] == (SingleRemoteVIM*)0 )
			continue;
		if ( inVIM.mID != mSingleVIM[vIndex]->mID )
			continue;

		mIsVIMReserved[vIndex] = (xam_boolean)false;
		break;
	}
}

xam_status
RemoteVIM::GetAsyncData (const xasync_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "GetData";
		SingleRemoteVIM& vVIM = GetVIM(inHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	// get the handle for any xstream that may contain data
	if ( vStatus == 0 )
	{
		// find the buffer and buffersize
		xam_int vMaxBytecount = 0;
		char* vBuffer = (char*)0;
		GetBuffer(inHandle, &vMaxBytecount, &vBuffer);

		// read the data into the buffer
		xam_int vBytecount;
		vData.GetBytecount(vBytecount);
		if ( vBytecount >= vMaxBytecount )
			vBytecount = vMaxBytecount;
		vData.GetBytes(vBuffer, vBytecount, 0);
	}
	return vStatus;
}

xam_boolean
RemoteVIM::POLL()
{
	xam_boolean vPollingComplete = true;
	xam_int vLimit = 10;
	for ( xam_int ix=0; ix<mNumAllocated; ix++ )
	{
		if ( mActive[ix] == false )
			continue;
		xsystem_handle vHandle = mRemoteHandle[ix];
		xam_status vStatus = (xam_status)0;
		XferBuffer vData;
		try
		{
			char* vCommand = "POLL";
			SingleRemoteVIM& vVIM = GetVIM(vHandle, vCommand);
			vVIM.AppendArg("limit", vLimit);
			try
			{
				std::string vHeader;
				vVIM.GET(&vHeader, &vData); 
			}
			catch(...){vStatus = (xam_status)1001;}
			try{ReleaseVIM(vVIM);}catch(...){}
		}
		catch(...){vStatus = (xam_status)1001;}
		if ( vStatus != (xam_status)0 )
			continue;

		xam_int vCount = 0;
		xam_boolean vPolling = true;
		char vName[16];
		XAM_POSIX::xam_strcat(vName, "async_complete0");
		while ( vPolling && (vCount < vLimit) )
		{
			// get the async handle
			vName[15] = (char)(vCount+48);
			xasync_handle vAsync = (xasync_handle)0;
			vStatus = vData.GetHandle(vName, vAsync);
			if ( vStatus != (xam_status)0 && vAsync != (xasync_handle)0 )
			{
				vPolling = false;
				break;
			}
			vCount++;

			// load up any buffers prior to invoking callback
			GetAsyncData(vAsync);

			// execute the callback
			xasync_callback vCallback = mCallback[ix];
			(*vCallback)(vAsync);

			// remove the callback from the list
			ReleaseIndirectBinding(vAsync);
		}
		if ( vCount >= vLimit )
			vPollingComplete = false;
	}
	return vPollingComplete ? false : true;
}

////////////////////////////////////////////////////////////////////////////////
// constructors
////////////////////////////////////////////////////////////////////////////////
RemoteVIM::RemoteVIM(void):mBlockSize(10)
{
	xam_int vSize = mBlockSize * sizeof(xam_handle_t);
	mHandle = (xsystem_handle*)XAM_POSIX::xam_malloc(vSize);
	mRemoteHandle = (xsystem_handle*)XAM_POSIX::xam_malloc(vSize);
	mIndirectHandle = (xam_handle_t*)XAM_POSIX::xam_malloc(vSize);
	mReferencedHandle = (xsystem_handle*)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(std::string*);
	mAddress = (std::string**)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(xasync_callback);
	mCallback = (xasync_callback*)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(XferBuffer*);
	mCache = (XferBuffer**)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(xam_boolean);
	mActive = (xam_boolean*)XAM_POSIX::xam_malloc(vSize);
	mIndirectActive = (xam_boolean*)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(SingleRemoteVIM*);
	mSingleVIM = (SingleRemoteVIM**)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(xam_boolean);
	mIsVIMReserved = (xam_boolean*)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(xam_int);
	mBuffersize = (xam_int*)XAM_POSIX::xam_malloc(vSize);
	vSize = mBlockSize * sizeof(char*);
	mBuffer = (char**)XAM_POSIX::xam_malloc(vSize);
	for ( xam_int ix=0; ix<mBlockSize; ix++ )
	{
		mHandle[ix] = (xsystem_handle)0;
		mRemoteHandle[ix] = (xsystem_handle)0;
		mAddress[ix] = (std::string*)0;
		mCache[ix] = (XferBuffer*)0;
		mActive[ix] = (xam_boolean)false;

		mIndirectHandle[ix] = (xam_handle_t)0;
		mCallback[ix] = (xasync_callback)0;
		mReferencedHandle[ix] = (xsystem_handle)0;
		mBuffersize[ix] = 0;
		mBuffer[ix] = (char*)0;
		mIndirectActive[ix] = (xam_boolean)false;

		mSingleVIM[ix] = (SingleRemoteVIM*)0;
		mIsVIMReserved[ix] = (xam_boolean)false;
	}
	mNumAllocated = mNumIndirectAllocated = mNumVIMAllocated = mBlockSize;
	mNumActive = mNumIndirectActive = mNumVIMActive = 0;

	mInfo.running = false;
    mInfo.killed = false;
    mInfo.thread_id = 0;
    mInfo.thread_handle = 0;
	mInfo.data = (void*)0;
}

////////////////////////////////////////////////////////////////////////////////
// destructor
////////////////////////////////////////////////////////////////////////////////
RemoteVIM::~RemoteVIM (void)
{
#if 0 // being invoked after DLL is unloaded - make into a noop
	// release direct table
	if ( mActive )
	{
		for ( xam_int ix=0; ix<mNumAllocated; ix++ )
		{
			if ( mActive[ix] && mVIM[ix] != (SingleRemoteVIM*)0 )
			{
				mActive[ix] = false;
				
				delete mVIM[ix];
				mVIM[ix] = (SingleRemoteVIM*)0;
			}
		}
	}
	mNumAllocated = 0;
	mNumActive = 0;
	if ( mHandle )
	{
		XAM_POSIX::xam_free(mHandle);
		mHandle = (xsystem_handle*)0;
	}
	if ( mVIM )
	{
		XAM_POSIX::xam_free(mVIM);
		mVIM = (SingleRemoteVIM**)0;
	}
	if ( mActive )
	{
		XAM_POSIX::xam_free(mActive);
		mActive = (xam_boolean*)0;
	}

	// release indirect table
	mNumIndirectAllocated = 0;
	mNumIndirectActive = 0;
	if ( mIndirectHandle )
	{
		XAM_POSIX::xam_free(mIndirectHandle);
		mIndirectHandle = (xam_handle_t*)0;
	}
	if ( mCallback )
	{
		XAM_POSIX::xam_free(mCallback);
		mCallback = (xasync_callback*)0;
	}
	if ( mReferencedHandle )
	{
		XAM_POSIX::xam_free(mReferencedHandle);
		mReferencedHandle = (xsystem_handle*)0;
	}
	if ( mIndirectActive )
	{
		XAM_POSIX::xam_free(mIndirectActive);
		mIndirectActive = (xam_boolean*)0;
	}
#endif
}
