#ifndef __RemoteVIMInterface_H__
#define __RemoteVIMInterface_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2009, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include <string>
#include "vim.h"
#include "curl/curl.h"
#include "XferBuffer.h"
#include "XAM_Exception.h"

class RemoteVIMInterface
{
 public:
	/**
	 * Modifies the URL by appending the command string
	 * @param inCommand the string containing the name of the command
	 */
    virtual void SetCommand(const char* const inCommand) = 0;

	/**
	 * Replaces any argument list with the specified name/value pair
	 * @param inName the string containing the name of the argument
	 * @param inValue the string containing the value of the argument
	 */
	virtual void SetArg(const char* const inName, const char* const inValue) = 0;

	/**
	 * Appends the specified name/value pair to the argument list
	 * @param inName the string containing the name of the argument
	 * @param inValue the string containing the value of the argument
	 */
    virtual void AppendArg(const char* const inName, const char* const inValue) = 0;

	/**
	 * Appends the specified name/value pair to the argument list
	 * @param inName the string containing the name of the argument
	 * @param inValue the xam_int containing the value of the argument
	 */
    virtual void AppendArg(const char* const inName, const xam_int inValue) = 0;

	/**
	 * Appends the specified name/value pair to the argument list
	 * @param inName the string containing the name of the argument
	 * @param inValue the xam_boolean containing the value of the argument
	 */
    virtual void AppendArg(const char* const inName, const xam_boolean inValue) = 0;

	/**
	 * Appends the specified name/value pair to the argument list
	 * @param inName the string containing the name of the argument
	 * @param inValue the xam_double containing the value of the argument
	 */
    virtual void AppendArg(const char* const inName, const xam_double inValue) = 0;

	/**
	 * Appends the specified name/value pair to the argument list
	 * @param inName the string containing the name of the argument
	 * @param inValue the xam_xuid containing the value of the argument
	 */
    virtual void AppendXUID(const char* const inName, const xam_xuid inValue) = 0;

	/**
	 * Appends the xsystem handle to the argument list
	 * @param inValue the xsystem_handle containing the value of the argument
	 */
    virtual void AppendHandle(const xsystem_handle inHandle) = 0;

	/**
	 * Issues the queued up command with the current args
	 * @param ioHeaderStr the header of the response
	 * @param ioDataBuffer the body of the response
	 */
	virtual void Execute(std::string* ioHeaderStr, XferBuffer* ioDataBuffer) const = 0;

	////////////////////////////////////////////////////////////////////////////
	// constructors
	////////////////////////////////////////////////////////////////////////////
 protected:
	 RemoteVIMInterface (void) {}

 	////////////////////////////////////////////////////////////////////////////
	// destructor
	////////////////////////////////////////////////////////////////////////////
 public:
	virtual ~RemoteVIMInterface (void) {}
};

#endif // __RemoteVIMInterface_H__