////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2009, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XferBuffer.h"
#include "XAM_POSIX.h"
#include "XAM_Exception.h"

void
XferBuffer::ParseBytes(void)
{
	// convert the buffer to a large string
	std::string vUnparsedByteStr;
	for ( xam_int ix=0; ix<mNumBlocks; ix++ )
		vUnparsedByteStr.append(mBlock[ix], (unsigned int)mBlockSize[ix]);
	xam_int vNumBytes = vUnparsedByteStr.length();

	// assume that the data consists of <name>=<value> strings, one per line
	// parse it into name value pairs now
	std::string vName;
    std::string vValue;
	unsigned int vIndex1 = 0;
	while ( vIndex1 < vNumBytes )
	{
		// find the end of the name, marked by an '='
		unsigned int vIndex2 = vUnparsedByteStr.find('=', vIndex1);
		if ( vIndex2 == std::string::npos ) // didn't find the separator
		{
			vName.clear();
			vValue.clear();
			break;
		}
		else // found the name now find the value
		{
			vName = vUnparsedByteStr.substr(vIndex1, vIndex2 - vIndex1);

			// find the end of the value, at the end of the line or the end of the data
			vIndex1 = vIndex2 + 1;
			if ( vIndex1 >= vNumBytes ) // nothing to copy
				vValue.clear();
			else // copy until the end of line (or end of data)
			{
				vIndex2 = vUnparsedByteStr.find('\x00a', vIndex1);
				if ( vIndex2 == std::string::npos )
					vIndex2 = (unsigned int)vNumBytes;
				vValue = vUnparsedByteStr.substr(vIndex1, vIndex2 - vIndex1);

				// insert the name/value pair into the map
				mMap[vName] = vValue;

				// prep for next iteration
				vIndex1 = vIndex2 + 1;
			}
		}
	}
}

xam_double
XferBuffer::StringToDouble(std::string& inString)
{
	xam_boolean vNegative = false;
	xam_int vExponent = 0;
	xam_int vMantissa = 0;
	xam_double vValue = 0;

	const char* const vStr = inString.c_str();
	xam_int vLen = inString.length();
	if ( vLen >= 1 )
	{
		vNegative = (vStr[0] == 43) ? true : false;
		if ( vLen >= 5 )
		{
			vExponent = (vStr[1]-48)*1000 + (vStr[2]-48)*100 + (vStr[3]-48)*10 + vStr[4]-48;
			if ( vLen >= 22 )
			{
				vMantissa = (vStr[6]-48)*1000000000000000LL + 
					        (vStr[7]-48)*100000000000000LL + 
					        (vStr[8]-48)*10000000000000LL + 
					        (vStr[9]-48)*1000000000000LL + 
					        (vStr[10]-48)*100000000000LL + 
					        (vStr[11]-48)*10000000000LL + 
					        (vStr[12]-48)*1000000000 + 
					        (vStr[13]-48)*100000000 + 
					        (vStr[14]-48)*10000000 + 
					        (vStr[15]-48)*1000000 + 
					        (vStr[16]-48)*100000 + 
					        (vStr[17]-48)*10000 + 
					        (vStr[18]-48)*1000 + 
					        (vStr[19]-48)*100 + 
							(vStr[20]-48)*10 + 
							 vStr[21]-48;

			}
		}
	}
	xam_double vRetVal = 0;
	xam_int* vVal = (xam_int*)&vRetVal;
	
	if ( vNegative )
		*vVal |= 0x8000;
	*vVal |= (vExponent << 52) & 0x7ff;
	*vVal |= vMantissa & 0xfffffffffffffLL;

	return vRetVal;
}
// 20 digits
// decimal 48 is zero
// decimal 57 is nine

xam_int
XferBuffer::StringToInt(std::string& inString)
{
	xam_int vMultiplier = 1;
	xam_int vValue = 0;
	xam_int vNumBytes = inString.length();
	if ( vNumBytes > 20 )
		vNumBytes = 20;
	const char* const vStr = inString.c_str();
	for ( xam_int ix=0; ix<vNumBytes; ix++ )
	{
		xam_int vDigit = (xam_int)vStr[(vNumBytes-1)-ix];
		if ( vDigit >= 48 )
		{
			if ( vDigit > 57 )
				vDigit = 9;
			else
				vDigit -= 48;
			for ( xam_int jx=0; jx<ix; jx++ )
				vDigit *= 10;
			vValue += vDigit;
		}
	}
	return vValue;
}

xam_boolean
XferBuffer::StringToBoolean(std::string& inString)
{
	return (xam_boolean)(XAM_POSIX::xam_strcasecmp(inString.c_str(), "true")?true:false);
}

void
XferBuffer::AddBytes(const char * inBuf, xam_int inBytecount)
{
	// is there room in the block array?
	if ( mNumBlocks%mBlockIncrement == 0 )
	{
		// add a number of blocks
		xam_int vNewNumTotalBlocks = mNumTotalBlocks + mBlockIncrement; 
		xam_int vSize = vNewNumTotalBlocks * sizeof(char*);
		if ( mBlock == (char**)0 )
			mBlock = (char**)XAM_POSIX::xam_malloc(vSize);
		else
			mBlock = (char**)XAM_POSIX::xam_realloc(mBlock, vSize);
		if ( mBlock == (char**)0 )
        { 
            XAMOutOfMemoryException e;
            throw e;
        }
		vSize = vNewNumTotalBlocks * sizeof(xam_int);
		if ( mBlock == (char**)0 )
			mBlockSize = (xam_int*)XAM_POSIX::xam_malloc(vSize);
		else
			mBlockSize = (xam_int*)XAM_POSIX::xam_realloc(mBlockSize, vSize);
		if ( mBlockSize == (xam_int*)0 )
        { 
            XAMOutOfMemoryException e;
            throw e;
        }
	}

	// add the buffer the the current block
	mBlock[mNumBlocks] = (char*)XAM_POSIX::xam_malloc(inBytecount);
	XAM_POSIX::xam_memcpy(mBlock[mNumBlocks], inBuf, inBytecount);
	mBlockSize[mNumBlocks] = inBytecount;

	// increment the block count
	mNumBlocks++;
}

void 
XferBuffer::GetBytecount(xam_int& ioBytecount)
{
	xam_int vBytecount = 0;
	for ( xam_int ix=0; ix<mNumBlocks; ix++ )
		vBytecount += mBlockSize[ix];
	ioBytecount = vBytecount;
}

void 
XferBuffer::GetBytes (char* ioBuf, xam_int inBytecount, xam_int inOffset)
{
	// clear the space (note this slows down stuff and is prolly not needed)
	XAM_POSIX::xam_memset(ioBuf, 0, inBytecount);

	// determine the block to start reading from
	xam_int vBlockIx = 0;
	xam_int vBlockOffset = 0;
	xam_int vTotalBytecount = 0;
	GetBytecount(vTotalBytecount);
	if ( (inOffset+inBytecount) > vTotalBytecount )
	{
        XAMOutOfMemoryException e; // for now, throw this one
        throw e;
	}
	while ( inOffset > (vBlockOffset+mBlockSize[vBlockIx]) )
		vBlockOffset += mBlockSize[vBlockIx++];
	xam_int vOffset = inOffset - vBlockOffset;

	// copy the data into the buffer, a block at a time
	xam_int vBytesCopied = 0;
	while ( (vBytesCopied < inBytecount) && (vBlockIx < mNumBlocks) )
	{
		// transfer the data from the block
		xam_int vBytecount = mBlockSize[vBlockIx] - vOffset;
		if ( (vBytecount + vBytesCopied) > inBytecount )
			vBytecount = inBytecount - vBytesCopied;
		XAM_POSIX::xam_memcpy(&ioBuf[vBytesCopied], 
		                      &mBlock[vBlockIx][vOffset],
							  vBytecount);

		// update the indices
		vBytesCopied += vBytecount;
		vOffset = 0;
		vBlockIx++;
	}
}

xam_status
XferBuffer::GetValue(const char* const inName, std::string& ioValue)
{
	// parse the data into name/value pairs
	xam_status vStatus = 0;
	if ( mParsed == false )
	{
		ParseBytes();
		mParsed = true;
	}

	// find the specified name
	std::map<std::string,std::string>::iterator vPtr;
	vPtr = mMap.find(inName);
	if ( vPtr == mMap.end() )
		ioValue.clear();
	else
		ioValue = vPtr->second;

	// find the status field
	vPtr = mMap.find("status");
	if ( vPtr != mMap.end() )
		vStatus = (xam_status)StringToInt(vPtr->second);
	return vStatus;
}

xam_status
XferBuffer::GetValue(std::string& inName, std::string& ioValue)
{
	return GetValue(inName.c_str(), ioValue);
}

xam_status
XferBuffer::GetCStr(const char* const inName, xam_string ioStr)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		XAM_POSIX::xam_strcpy(ioStr, vStr.c_str());
	else
		ioStr[0] = (char)0;
	return vStatus;
}

xam_status
XferBuffer::GetCStr(std::string& inName, xam_string ioStr)
{
	return GetCStr(inName.c_str(), ioStr);
}

xam_status
XferBuffer::GetHandle(const char* const inName, xam_handle_t& ioHandle)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		ioHandle = (xam_handle_t)StringToInt(vStr);
	else
		ioHandle = (xam_handle_t)0;
	return vStatus;
}

xam_status
XferBuffer::GetHandle(std::string& inName, xam_handle_t& ioHandle)
{
	return GetHandle(inName.c_str(), ioHandle);
}

xam_status
XferBuffer::GetBoolean(const char* const inName, xam_boolean& ioFlag)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		ioFlag = (xam_boolean)StringToBoolean(vStr);
	else
		ioFlag = (xam_boolean)0;
	return vStatus;
}

xam_status
XferBuffer::GetBoolean(std::string& inName, xam_boolean& ioFlag)
{
	return GetBoolean(inName.c_str(), ioFlag);
}


xam_status
XferBuffer::GetDatetime(const char* const inName, xam_datetime ioDatetime)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		XAM_POSIX::xam_strcpy(ioDatetime, vStr.c_str());
	else
		ioDatetime[0] = 0;
	return vStatus;
}

xam_status
XferBuffer::GetDatetime(std::string& inName, xam_datetime ioDatetime)
{
	return GetDatetime(inName.c_str(), ioDatetime);
}


xam_status
XferBuffer::GetInt(const char* const inName, xam_int& ioValue)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		ioValue = StringToInt(vStr);
	else
		ioValue = 0;
	return vStatus;
}

xam_status
XferBuffer::GetInt(std::string& inName, xam_int& ioValue)
{
	return GetInt(inName.c_str(), ioValue);
}

xam_status 
XferBuffer::GetDouble(const char* const inName, xam_double& ioValue)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		ioValue = StringToDouble(vStr);
	else
		ioValue = 0;
	return vStatus;
}

xam_status 
XferBuffer::GetDouble(std::string& inName, xam_double& ioValue)
{
	return GetDouble(inName.c_str(), ioValue);
}

xam_status
XferBuffer::GetXUID(const char* const inName, xam_xuid ioXUID)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		XAM_POSIX::xam_strncpy(ioXUID, vStr.c_str(), XAM_MAX_XUID);
	else
		ioXUID[0] = 0;
	return vStatus;
}

xam_status
XferBuffer::GetXUID(std::string& inName, xam_xuid ioXUID)
{
	return GetXUID(inName.c_str(), ioXUID);
}

xam_status
XferBuffer::GetStatus(const char* const inName, xam_status& ioStatus)
{
	std::string vStr;
	xam_status vStatus = GetValue(inName, vStr);
	if ( vStatus == 0 )
		ioStatus = (xam_status)StringToInt(vStr);
	else
		ioStatus = 0;
	return vStatus;
}

xam_status
XferBuffer::GetStatus(std::string& inName, xam_status& ioStatus)
{
	return GetStatus(inName.c_str(), ioStatus);
}

void
XferBuffer::Reset(void)
{
	if ( mBlock != (char**)0 )
	{
		for ( int ix=0; ix<mNumBlocks; ix++ )
			XAM_POSIX::xam_free(mBlock[ix]);
		XAM_POSIX::xam_free(mBlock);
		mBlock = (char**)0;
	}
	if ( mBlockSize != (xam_int*)0 )
	{
		XAM_POSIX::xam_free(mBlockSize);
		mBlockSize = (xam_int*)0;
	}
	mNumTotalBlocks = 0;
	mNumBlocks = 0;
	mParsed = false;
	mMap.clear();
}

////////////////////////////////////////////////////////////////////////////////
// constructors
////////////////////////////////////////////////////////////////////////////////
XferBuffer::XferBuffer(void)
{
	mBlockIncrement = 10;

	mBlock = (char**)0;
	mBlockSize = (xam_int*)0;
	mNumTotalBlocks = 0;
	mNumBlocks = 0;

	mParsed = false;
}

////////////////////////////////////////////////////////////////////////////////
// destructor
////////////////////////////////////////////////////////////////////////////////
XferBuffer::~XferBuffer(void)
{
	Reset();
}
