#ifndef __XferBuffer_H__
#define __XferBuffer_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2009, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "vim.h"
#include <string>
#include <map>

class XferBuffer
{
private:
	// raw data
	char** mBlock;           // c-style block array
	xam_int* mBlockSize;     // c-style array of block sizes
	xam_int mNumTotalBlocks; // total number of blocks
	xam_int mBlockIncrement; // amount of blocks allocated on overflow
	xam_int mNumBlocks;      // number of blocks in use

	// processed data
	xam_boolean mParsed;
	std::map<std::string, std::string> mMap;

	void ParseBytes(void);
	xam_double StringToDouble(std::string& inString);
	xam_int StringToInt(std::string& inString);
	xam_boolean StringToBoolean(std::string& inString);

public:
	/**
	 * Append data to the buffer
	 * @param inBuf buffer of bytes containing the bytes
	 * @param inBytecount number of bytes in the buffer
	 */
	void AddBytes(const char* const inBuf, xam_int inBytecount);

	/**
	 * Get the number of bytes currently stored
	 * @param ioBytecount reference to write the bytecount into
	 */
	void GetBytecount(xam_int& ioBytecount);

	/**
	 * Get the bytes currently stored
	 * @param inBytecount number of bytes to read
	 * @param inOffset offset to read the bytes from
	 */
	void GetBytes(char* ioBuf, xam_int inBytecount, xam_int inOffset=0);

	xam_status GetValue(std::string& inName, std::string& ioValue);
	xam_status GetValue(const char* const inName, std::string& ioValue);

	xam_status GetCStr(std::string& inName, xam_string ioStr);
	xam_status GetCStr(const char* const inName, xam_string ioStr);

	xam_status GetHandle(std::string& inName, xam_handle_t& ioHandle);
	xam_status GetHandle(const char* const inName, xam_handle_t& ioHandle);

	xam_status GetBoolean(std::string& inName, xam_boolean& ioFlag);
	xam_status GetBoolean(const char* const inName, xam_boolean& ioFlag);

	xam_status GetDatetime(std::string& inName, xam_datetime ioDatetime);
	xam_status GetDatetime(const char* const inName, xam_datetime ioDatetime);

	xam_status GetInt(std::string& inName, xam_int& ioValue);
	xam_status GetInt(const char* const inName, xam_int& ioValue);

	xam_status GetDouble(std::string& inName, xam_double& ioValue);
	xam_status GetDouble(const char* const inName, xam_double& ioValue);

	xam_status GetXUID(std::string& inName, xam_xuid ioXUID);
	xam_status GetXUID(const char* const inName, xam_xuid ioXUID);

	xam_status GetStatus(std::string& inName, xam_status& ioValue);
	xam_status GetStatus(const char* const inName, xam_status& ioValue);

	/**
	 * Reset the buffer to zero size (freeing all resources)
	 */
	void Reset(void);

	////////////////////////////////////////////////////////////////////////////
	// constructors
	////////////////////////////////////////////////////////////////////////////
private:
    XferBuffer(const XferBuffer&); // Copy constructor not available
    XferBuffer& operator = (const XferBuffer&); // Assignment op not available

public:
	/**
	 * Create a transfer buffer
	 */
	XferBuffer(void);

	////////////////////////////////////////////////////////////////////////////
	// destructor
	////////////////////////////////////////////////////////////////////////////
    ~XferBuffer(void);

};
#endif  // #ifndef __XferBuffer_H__
