////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2009, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM VIM HTTP Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "vim.h"

#include "XAM_Utils.h"
#include "RemoteVIM.h"

const char* const sADDRESS = ".org.snia.xam.vim.parameter.targetServer.ipAddress";
const char* const sPORT = ".org.snia.xam.vim.parameter.targetServer.port";
const char* const sVIMNAME = ".org.snia.xam.vim.parameter.targetServer.vimname";
const char* const sADDRESS_PARAM = "targetServer.ipAddress=";
const char* const sPORT_PARAM = "targetServer.port=";
const char* const sVIMNAME_PARAM = "targetServer.vimname=";
const char* const sPATH = "C:\\VIMS\\vim.properties";

static xam_int sHandleIndex = 19620729;
static XAM_Mutex sMUTEX;
static RemoteVIM sRemoteVIM;

static xam_boolean
get_address_from_xri(const std::string& inXRI, 
                     std::string& outVimname,
                     std::string& outAddress,
                     std::string& outPort)
{
	xam_boolean vSuccess = false;

	// find the index of the beginning of the parameter list
	std::string::size_type vIndex1 = inXRI.find('?');
	if ( vIndex1 != std::string::npos ) // found
	{
		// get the parameter list
		std::string vParams = inXRI.substr(vIndex1+1);
	
		// find the vimname parameter
		vIndex1 = vParams.find(sVIMNAME_PARAM);
		if ( vIndex1 != std::string::npos )
		{
			vIndex1 += (std::string::size_type)XAM_POSIX::xam_strlen(sVIMNAME_PARAM);
            std::string::size_type vIndex2 = vParams.find('&', vIndex1);
            if ( vIndex2 == std::string::npos )
				outVimname = vParams.substr(vIndex1);
			else
				outVimname = vParams.substr(vIndex1, vIndex2-vIndex1);
		}

		// find the address parameter
		vIndex1 = vParams.find(sADDRESS_PARAM);
		if ( vIndex1 != std::string::npos )
		{
			vIndex1 += (std::string::size_type)XAM_POSIX::xam_strlen(sADDRESS_PARAM);
            std::string::size_type vIndex2 = vParams.find('&', vIndex1);
            if ( vIndex2 == std::string::npos )
				outAddress = vParams.substr(vIndex1);
			else
			{
				outAddress = vParams.substr(vIndex1, vIndex2-vIndex1);

				// find the port parameter
				vIndex1 = vParams.find(sPORT_PARAM);
				if ( vIndex1 != std::string::npos )
				{
					vIndex1 += (std::string::size_type)XAM_POSIX::xam_strlen(sPORT_PARAM);
					vIndex2 = vParams.find('&', vIndex1);
					if ( vIndex2 == std::string::npos )
						outPort = vParams.substr(vIndex1);
					else
						outPort = vParams.substr(vIndex1, vIndex2-vIndex1);
				}
			}
			vSuccess = true;
		}
	}
	return vSuccess;
}

static xam_boolean
get_address_from_file(std::string& outVimname, 
                      std::string& outAddress,
                      std::string& outPort)
{
	xam_boolean vSuccess = false;

	// stat the file
	xam_int vLength = 0;
	xam_stat_t vStat;
	if ( XAM_POSIX::xam_stat(sPATH, &vStat) == 0 )
	{
		vLength = vStat.st_size;
		char* vBuf = (char*)XAM_POSIX::xam_malloc(vLength+1);
		if ( vBuf )
		{
			// read the file
			xam_int vNumRead = 0;
			FILE* vFile = XAM_POSIX::xam_fopen("C:\\VIMS\\vim.properties", "r");
			if ( vFile )
			{
				vNumRead = XAM_POSIX::xam_fread(vBuf, vLength, 1, vFile);
				vBuf[vLength] = (char)0;
				XAM_POSIX::xam_fclose(vFile);
			}

			// parse the address
			if ( vNumRead == vLength ) try
			{
				XferBuffer vXferBuffer;
				vXferBuffer.AddBytes(vBuf, vLength);

				vXferBuffer.GetValue(sVIMNAME, outVimname);
				vXferBuffer.GetValue(sADDRESS, outAddress);
				vXferBuffer.GetValue(sPORT, outPort);

				vSuccess = true;
			}
			catch(...) {}

			// release the resources
			XAM_POSIX::xam_free(vBuf);
		}
	}
	return vSuccess;
}

EXPORT xam_boolean DECL
VIM_XSystem_GetErrorToken (const xsystem_handle inHandle,
                           const xam_status inStatus,
                           xam_string* const outToken)
{
	xam_boolean vSuccess = true;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemGetErrorToken";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("status", inStatus);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vSuccess = (xam_boolean)false;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vSuccess = (xam_boolean)false;}

	if ( vSuccess )
		vSuccess = vData.GetCStr("token", *outToken) ? false : true;
	return vSuccess;
}

/**************************************************************************
 *  methods for the XIterator
 *************************************************************************/
EXPORT xam_status DECL
VIM_XSystem_OpenFieldIterator (const xsystem_handle inHandle,  
                               const xam_string inPattern,
                               xiterator_handle* const outIterator)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XAMCreateFieldIterator";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("class", "xsystem");
			vVIM.AppendArg("pattern", inPattern);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outIterator);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outIterator, inHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_OpenFieldIterator (const xset_handle inHandle,  
                            const xam_string inPattern,
                            xiterator_handle* const outIterator)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XAMCreateFieldIterator";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("class", "xset");
			vVIM.AppendArg("pattern", inPattern);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outIterator);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outIterator, vHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XIterator_HasNext (const xiterator_handle inHandle, 
                       xam_boolean* const outHasNext)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XIteratorHasNext";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("hasnext", *outHasNext);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XIterator_Next (const xiterator_handle inHandle, 
                    xam_string* const outName)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XIteratorNext";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetCStr("property", *outName);
	return vStatus;
}
  
EXPORT xam_status DECL
VIM_XIterator_Close (xiterator_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XIteratorClose";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		sRemoteVIM.ReleaseIndirectBinding(inHandle);
	return vStatus;
}

/**************************************************************************
 *  methods for managing XAM Fields (properties or XStreams)
 *************************************************************************/
EXPORT xam_status DECL
VIM_XSystem_ContainsField(const xsystem_handle inHandle,  
                          const xam_string inName,
                          xam_boolean* const outContained)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemContainsField";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outContained);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ContainsField(const xset_handle inHandle,  
                       const xam_string inName,
                       xam_boolean* const outContained)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetContainsField";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outContained);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetFieldAsBinding (const xset_handle inHandle,
                            const xam_string inName)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetFieldAsBinding";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetFieldAsNonbinding (const xset_handle inHandle,
                               const xam_string inName)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetFieldAsNonbinding";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetFieldType (const xsystem_handle inHandle,
                          const xam_string inName,
                          xam_string* const outType)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemGetPropertyType";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetCStr("value", *outType);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetFieldType (const xset_handle inHandle,
                       const xam_string inName,
                       xam_string* const outType)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetPropertyType";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetCStr("value", *outType);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetFieldLength (const xsystem_handle inHandle,
                            const xam_string inName,
                            xam_int* const outLength)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemGetFieldLength";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetInt("value", *outLength);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetFieldLength (const xset_handle inHandle,
                         const xam_string inName,
                         xam_int* const outLength)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetFieldLength";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetInt("value", *outLength);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetFieldBinding (const xset_handle inHandle,
                          const xam_string inName,
                          xam_boolean* const outBinding)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetFieldBinding";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outBinding);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetFieldReadOnly (const xsystem_handle inHandle,
                              const xam_string inName,
                              xam_boolean* const outReadOnly)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemGetFieldReadOnly";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outReadOnly);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetFieldReadOnly (const xset_handle inHandle,
                           const xam_string inName,
                           xam_boolean* const outReadOnly)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetFieldReadOnly";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outReadOnly);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_DeleteField (const xsystem_handle inHandle,
                         const xam_string inName)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{

		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
			;
		else
		{
			char* vCommand = "XSystemDeleteField";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData); 
			}
			catch(...){vStatus = (xam_status)1001;} 
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_DeleteField (const xset_handle inHandle,
                      const xam_string inName)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetDeleteField";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CreateBoolean (const xsystem_handle inHandle,
                           const xam_string inName,
                           const xam_boolean inBinding,
                           const xam_boolean inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			if ( XAM_POSIX::xam_strcmp(".xsystem.initializing", inName) != 0 )
				SingleRemoteVIM::CacheBoolean(inName, inValue, inBinding, *vCache);
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemCreateProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "boolean");
				vVIM.AppendArg("binding", inBinding);
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateBoolean (const xset_handle inHandle,
                        const xam_string inName,
                        const xam_boolean inBinding,
                        const xam_boolean inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "boolean");
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CreateInt (const xsystem_handle inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_int inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			if ( XAM_POSIX::xam_strcmp(".xsystem.initializing", inName) != 0 )
				SingleRemoteVIM::CacheInt(inName, inValue, inBinding, *vCache);
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemCreateProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "integer");
				vVIM.AppendArg("binding", inBinding);
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateInt (const xset_handle inHandle,
                    const xam_string inName,
                    const xam_boolean inBinding,
                    const xam_int inValue)
{
	// set the command onto the VIM
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "integer");
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CreateDouble (const xsystem_handle inHandle,
                          const xam_string inName,
                          const xam_boolean inBinding,
                          const xam_double inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			if ( XAM_POSIX::xam_strcmp(".xsystem.initializing", inName) != 0 )
				SingleRemoteVIM::CacheDouble(inName, inValue, inBinding, *vCache);
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemCreateProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "double");
				vVIM.AppendArg("binding", inBinding);
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateDouble (const xset_handle inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_double inValue)
{
	// set the command onto the VIM
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "double");
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CreateXUID (const xsystem_handle inHandle,
                        const xam_string inName,
                        const xam_boolean inBinding,
                        const xam_xuid inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			if ( XAM_POSIX::xam_strcmp(".xsystem.initializing", inName) != 0 )
				SingleRemoteVIM::CacheXUID(inName, inValue, inBinding, *vCache);
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemCreateProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "xuid");
				vVIM.AppendArg("binding", inBinding);
				vVIM.AppendXUID("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}


EXPORT xam_status DECL
VIM_XSet_CreateXUID (const xset_handle inHandle,
                     const xam_string inName,
                     const xam_boolean inBinding,
                     const xam_xuid inValue)
{
	// set the command onto the VIM
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "xuid");
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendXUID("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}


EXPORT xam_status DECL
VIM_XSystem_CreateString (const xsystem_handle inHandle,
                          const xam_string inName,
                          const xam_boolean inBinding,
                          const xam_string inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			if ( XAM_POSIX::xam_strcmp(".xsystem.initializing", inName) != 0 )
				SingleRemoteVIM::CacheString(inName, inValue, inBinding, *vCache);
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemCreateProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "string");
				vVIM.AppendArg("binding", inBinding);
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateString (const xset_handle inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_string inValue)
{
	// set the command onto the VIM
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "string");
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CreateDatetime (const xsystem_handle inHandle,
                            const xam_string inName,
                            const xam_boolean inBinding,
                            const xam_datetime inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			if ( XAM_POSIX::xam_strcmp(".xsystem.initializing", inName) != 0 )
				SingleRemoteVIM::CacheDatetime(inName, inValue, inBinding, *vCache);
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemCreateProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "datetime");
				vVIM.AppendArg("binding", inBinding);
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateDatetime (const xset_handle inHandle,
                         const xam_string inName,
                         const xam_boolean inBinding,
                         const xam_datetime inValue)
{
	// set the command onto the VIM
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "datetime");
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_SetBoolean (const xsystem_handle inHandle,
                        const xam_string inName,
                        const xam_boolean inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemSetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "boolean");
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetBoolean (const xset_handle inHandle,
                     const xam_string inName,
                     const xam_boolean inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "boolean");
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_SetInt (const xsystem_handle inHandle,
                    const xam_string inName,
                    const xam_int inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemSetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "integer");
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetInt (const xset_handle inHandle,
			 	 const xam_string inName,
				 const xam_int inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "integer");
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_SetDouble (const xsystem_handle inHandle,
 				      const xam_string inName,
				      const xam_double inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
			char* vCommand = "XSystemSetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "double");
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetDouble (const xset_handle inHandle,
				   const xam_string inName,
				   const xam_double inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "double");
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_SetXUID (const xsystem_handle inHandle,
				     const xam_string inName,
				     const xam_xuid inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemSetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "xuid");
				vVIM.AppendXUID("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetXUID (const xset_handle inHandle,
				  const xam_string inName,
				  const xam_xuid inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "xuid");
			vVIM.AppendXUID("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_SetString (const xsystem_handle inHandle,
				       const xam_string inName,
				       const xam_string inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemSetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "xuid");
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetString (const xset_handle inHandle,
				    const xam_string inName,
				    const xam_string inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "string");
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_SetDatetime (const xsystem_handle inHandle,
					     const xam_string inName,
					     const xam_datetime inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemSetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "datetime");
				vVIM.AppendArg("value", inValue);
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetDatetime (const xset_handle inHandle,
					  const xam_string inName,
					  const xam_datetime inValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "datetime");
			vVIM.AppendArg("value", inValue);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetBoolean (const xsystem_handle inHandle,
					    const xam_string inName,
					    xam_boolean* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemGetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "boolean");
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetBoolean (const xset_handle inHandle,
					 const xam_string inName,
					 xam_boolean* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "boolean");
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetInt (const xsystem_handle inHandle,
				    const xam_string inName,
				    xam_int* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemGetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "integer");
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
		vStatus = vData.GetInt("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetInt (const xset_handle inHandle,
				 const xam_string inName,
				 xam_int* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "integer");
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	if ( vStatus == 0 )
		vStatus = vData.GetInt("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetDouble (const xsystem_handle inHandle,
				      const xam_string inName,
				      xam_double* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemGetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "double");
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
		vStatus = vData.GetDouble("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetDouble (const xset_handle inHandle,
				   const xam_string inName,
				   xam_double* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "double");
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	if ( vStatus == 0 )
		vStatus = vData.GetDouble("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetXUID (const xsystem_handle inHandle,
				     const xam_string inName,
				     xam_xuid* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemGetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "xuid");
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
		vStatus = vData.GetXUID("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetXUID (const xset_handle inHandle,
				  const xam_string inName,
				  xam_xuid* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "xuid");
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	if ( vStatus == 0 )
		vStatus = vData.GetXUID("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetString (const xsystem_handle inHandle,
				       const xam_string inName,
				       xam_string* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemGetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "string");
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
		vStatus = vData.GetCStr("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetString (const xset_handle inHandle,
				    const xam_string inName,
				    xam_string* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "string");
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	if ( vStatus == 0 )
		vStatus = vData.GetCStr("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_GetDatetime (const xsystem_handle inHandle,
					     const xam_string inName,
					     xam_datetime* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		// get the VIM to use (locked resource)
		XferBuffer* vCache = sRemoteVIM.GetCache(inHandle);
		if ( vCache != (XferBuffer*)0 )
		{
			;
		}
		else
		{
			char* vCommand = "XSystemGetProperty";
			SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
			try
			{
				vVIM.AppendArg("name", inName);
				vVIM.AppendArg("ptype", "datetime");
				std::string vHeader;
				vVIM.GET(&vHeader, &vData);
			}
			catch (...) { vStatus = (xam_status)1001; }
			try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
		}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
		vStatus = vData.GetDatetime("value", *outValue);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetDatetime (const xset_handle inHandle,
					  const xam_string inName,
					  xam_datetime* const outValue)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetProperty";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("ptype", "datetime");
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;}

	if ( vStatus == 0 )
		vStatus = vData.GetDatetime("value", *outValue);
	return vStatus;
}

/**************************************************************************
 *  methods for XStreams and XStream fields
 *************************************************************************/
EXPORT xam_status DECL
VIM_XSystem_CreateXStream (const xsystem_handle inHandle,
					       const xam_string inName,
					       const xam_boolean inBinding,
					       const xam_string inType,
                           xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemCreateXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("type", inType);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXStream);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXStream, inHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateXStream (const xset_handle inHandle,
					    const xam_string inName,
					    const xam_boolean inBinding,
					    const xam_string inType,
                        xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("binding", inBinding);
			vVIM.AppendArg("type", inType);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXStream);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXStream, vHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_OpenXStream (const xsystem_handle inHandle,
					     const xam_string inName,
					     const xam_string inMode,
					     xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemOpenXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("mode", inMode);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXStream);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXStream, inHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_OpenXStream (const xset_handle inHandle,
					  const xam_string inName,
					  const xam_string inMode,
					  xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetOpenXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("name", inName);
			vVIM.AppendArg("mode", inMode);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXStream);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXStream, vHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XStream_Read (const xstream_handle inHandle,
				  char* const ioBuffer,
				  const xam_int inBufferLength,
				  xam_int* const outBytesRead)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamRead";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("numbytes", inBufferLength);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData); 
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		xam_int vBytecount;
		vData.GetBytecount(vBytecount);
		if ( vBytecount >= inBufferLength )
			vBytecount = inBufferLength;
		vData.GetBytes(ioBuffer, vBytecount, 0);
		*outBytesRead = vBytecount;
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XStream_Write (const xstream_handle inHandle,
				   const char* inBuffer,
				   const xam_int inByteCount,
				   xam_int* const outBytesWritten)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamWrite";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.POST(inBuffer,inByteCount, &vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetInt("numbytes", *outBytesWritten);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XStream_Seek (const xstream_handle inHandle,
				  const xam_int inOffset,
				  const xam_int inWhence)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamSeek";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			vVIM.AppendArg("offset", inOffset);
			vVIM.AppendArg("whence", inWhence);
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XStream_Tell (const xstream_handle inHandle,
				  xam_int* const outPosition)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamTell";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetInt("position", *outPosition);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XStream_Abandon (const xstream_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamAbandon";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XStream_Close (xstream_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	xsystem_handle vHandle = (xsystem_handle)0;
	try
	{
		vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamClose";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		try
		{
			std::string vHeader;
			vVIM.GET(&vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		sRemoteVIM.ReleaseIndirectBinding(inHandle);
	return vStatus;
}

/**************************************************************************
 *  Managing the connection to the XAM Storage System
 *************************************************************************/
static xam_handle_t
generate_handle (void)
{
	// not intrinsically threadsafe - requires a lock
	XAM_CriticalSection vCritical(&sMUTEX);
	xam_handle_t vHandle = sHandleIndex++;
	if ( sHandleIndex == XAM_INT_MAX )
		sHandleIndex = 1;
	return vHandle;
}

EXPORT xam_status DECL
VIM_CreateXSystem (xsystem_handle* const outHandle)
{
	xam_status vStatus = (xam_status)0;
	*outHandle = (xsystem_handle)generate_handle();
	if ( *outHandle != (xsystem_handle)0 )
		sRemoteVIM.RegisterXSystem(*outHandle);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_Connect (const xsystem_handle inHandle,
                     const xam_string inXRI)
{
	// get the remote address
	xam_status vStatus = (xam_status)0;
	std::string vXRI("snia-xam://");
	std::string vVimname;
	std::string vAddress;
	std::string vPort;
	xam_boolean vFound = get_address_from_xri(inXRI, vVimname, vAddress, vPort);
	if ( vFound == false )
		vFound = get_address_from_file(vVimname, vAddress, vPort);
	if ( vVimname.empty() )
		vXRI += inXRI;
	else
		vXRI += vVimname + "!" + inXRI;
	if ( vFound )
	{
		if ( vPort.length() != 0 )
		{
			vAddress += ":";
			vAddress += vPort;
		}
	}
	xam_int vBytecount = 0;
	char* vBuf = (char*)0;
	XferBuffer* vCache = (XferBuffer*)0;
	XferBuffer vData;

	// issue the command
	if ( vStatus == 0 ) try
	{
		// get the VIM to use (locked resource)
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM();
		try
		{
			// get any cached field info to post
			vCache = sRemoteVIM.GetCache(inHandle);
			if ( vCache )
			{
				vCache->GetBytecount(vBytecount);
				vBuf = (char*)XAM_POSIX::xam_malloc(vBytecount);
				vCache->GetBytes (vBuf, vBytecount, 0);
			}

			// associate the handle to the address of the remote VIM
			sRemoteVIM.BindToAddress(inHandle, vAddress);

			// format the command
			vVIM.SetCommand(vAddress, "XSystemConnect");
			vVIM.ResetArgs(false);
			vVIM.AppendArg("xri", vXRI.c_str()); // this could be part of the post
			std::string vHeader;
			vVIM.POST(vBuf, vBytecount, &vHeader, &vData);
		}
		catch(...){vStatus = (xam_status)1001;}
		if ( vBuf != (char*)0 )
			XAM_POSIX::xam_free(vBuf);

		// unlock resources
		try{sRemoteVIM.DeleteCache(inHandle);}catch(...){}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...) { vStatus = (xam_status)1001; }

	if ( vStatus == 0 )
	{
		// get the XSystem handle and store it on the remote VIM
		xsystem_handle vRemoteHandle;
		vStatus = vData.GetHandle("handle", vRemoteHandle);
		if ( vStatus == 0 )
			sRemoteVIM.BindToRemoteXSystem(inHandle, vRemoteHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_Authenticate (const xsystem_handle inHandle,
						  const char* inBuffer,
						  const xam_int inByteCount,
						  xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemAuthenticate");
		std::string vHeader;
		try { vVIM.POST(inBuffer, inByteCount, &vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	// get the handle for any xstream that may contain data
	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXStream);
		if ( vStatus == 0 ) // get the auth code associated with the stream
		{
			xam_int vAuthCode = 0;
			xam_status vStat = vData.GetInt("authenticate_status", vAuthCode);
			if ( vStat == 0 )
			{
				if ( vAuthCode == XAM_AUTH_FAILED )
					vStatus = (xam_status)XAM_AUTH_FAILED;
				else if ( vAuthCode == XAM_AUTH_DATA_NEEDED )
					vStatus = (xam_status)XAM_AUTH_DATA_NEEDED;
				else if ( vAuthCode == XAM_AUTH_REQUIRED )
					vStatus = (xam_status)XAM_AUTH_REQUIRED;
			}
		}
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_Close (const xsystem_handle inHandle)
{
	// set the command onto the VIM
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemClose");
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	// get rid of the entry in the xsystem table
	if ( vStatus == (xam_status)0 )
		sRemoteVIM.ReleaseDirectBinding(inHandle);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_Abandon (const xsystem_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemAbandon");
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_DeleteXSet (const xsystem_handle inHandle,
						const xam_xuid inXUID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemAbandon");
		vVIM.AppendArg("xuid", inXUID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_HoldXSet (const xsystem_handle inHandle,
					  const xam_xuid inXUID,
					  const xam_string inHoldID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemHoldXSet");
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("holdid", inHoldID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_ReleaseXSet (const xsystem_handle inHandle,
						 const xam_xuid inXUID,
						 const xam_string inHoldID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemReleaseXSet");
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("holdid", inHoldID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_AccessXSet (const xsystem_handle inHandle,
						const xam_xuid inXUID,
                        const xam_int inMode,
						xam_boolean* const outIsAccessible)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemAccessXSet");
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("mode", inMode);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("accessible", *outIsAccessible);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_IsXSetRetained (const xsystem_handle inHandle,
                            const xam_xuid inXUID,
                            xam_boolean* const outIsRetained)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemIsXSetRetained");
		vVIM.AppendArg("xuid", inXUID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetBoolean("retained", *outIsRetained);
	return vStatus;
}
 
EXPORT xam_status DECL
VIM_XSystem_GetXSetAccessTime (const xsystem_handle inHandle,
                               const xam_xuid inXUID,
                               xam_datetime* const outAccessTime)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemGetXSetAccessTime");
		vVIM.AppendArg("xuid", inXUID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vStatus = vData.GetDatetime("accesstime", *outAccessTime);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CreateXSet (const xsystem_handle inHandle,
                        const xam_string inMode,
						xset_handle* const outXSet)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, "XSystemCreateXSet");
		vVIM.AppendArg("mode", inMode);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXSet);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXSet, inHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ApplyAccessPolicy (const xset_handle inHandle,
                            const xam_boolean inBinding,
                            const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, "XSetApplyAccessPolicy", inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ResetAccessFields (const xset_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, "XSetResetAccessFields", inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_ApplyManagementPolicy (const xset_handle inHandle,
								const xam_boolean inBinding,
								const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyManagementPolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_ResetManagementFields (const xset_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetResetManagementFields";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_CreateRetention (const xset_handle inHandle,
                          const xam_boolean inBinding,
                          const xam_string inRetentionID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCreateRetention";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("retentionid", inRetentionID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetRetentionEnabledFlag (const xset_handle inHandle,
                                  const xam_string inRetentionID,
                                  const xam_boolean inBinding,
                                  const xam_boolean inEnabled)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetRetentionEnabledFlag";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("retentionid", inRetentionID);
		vVIM.AppendArg("enabled", inEnabled);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ApplyRetentionEnabledPolicy (const xset_handle inHandle,
                                      const xam_string inRetentionID,
                                      const xam_boolean inBinding,
                                      const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyRetentionEnabledPolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("retentionid", inRetentionID);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetRetentionDuration (const xset_handle inHandle,
                               const xam_string inRetentionID,
                               const xam_boolean inBinding,
                               const xam_int inDuration)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetRetentionDuration";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("retentionid", inRetentionID);
		vVIM.AppendArg("duration", inDuration);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ApplyRetentionDurationPolicy (const xset_handle inHandle,
                                       const xam_string inRetentionID,
                                       const xam_boolean inBinding,
                                       const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyRetentionDurationPolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("retentionid", inRetentionID);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetRetentionStarttime (const xset_handle inHandle,
                                const xam_string inRetentionID,
                                const xam_boolean inBinding)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetRetentionStarttime";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("retentionid", inRetentionID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetBaseRetention (const xset_handle inHandle,
                           const xam_boolean inBinding,
                           const xam_int inDuration)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetBaseRetention";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("duration", inDuration);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ApplyBaseRetentionPolicy (const xset_handle inHandle,
                                   const xam_boolean inBinding,
                                   const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyBaseRetentionPolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_ApplyAutoDeletePolicy (const xset_handle inHandle,
								const xam_boolean inBinding,
								const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyAutoDeletePolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_SetAutoDelete (const xset_handle inHandle,
						const xam_boolean inBinding,
						const xam_boolean inAutoDelete)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetAutoDelete";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("autodelete", inAutoDelete);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_ApplyShredPolicy (const xset_handle inHandle,
                           const xam_boolean inBinding,
                           const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyShredPolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_SetShred (const xset_handle inHandle,
                   const xam_boolean inBinding,
                   const xam_boolean inShred)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSetShred";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("shred", inShred);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_ApplyStoragePolicy (const xset_handle inHandle,
							 const xam_boolean inBinding,
							 const xam_string inPolicy)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetApplyStoragePolicy";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("binding", inBinding);
		vVIM.AppendArg("policy", inPolicy);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetActualRetentionDuration (const xset_handle inHandle,
                                     const xam_string inRetentionID,
                                     xam_int* const outDuration)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetActualRetentionDuration";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("retentionid", inRetentionID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vData.GetInt("duration", *outDuration);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetActualRetentionEnabled (const xset_handle inHandle,
                                    const xam_string inRetentionID,
                                    xam_boolean* const outEnabled)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetActualRetentionEnabled";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("retentionid", inRetentionID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vData.GetBoolean("enabled", *outEnabled);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_GetActualAutoDelete (const xset_handle inHandle,
							  xam_boolean* const outEnabled)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetActualAutoDelete";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vData.GetBoolean("autodelete", *outEnabled);
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSet_GetActualShred (const xset_handle inHandle,
                         xam_boolean* const outEnabled)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetGetActualShred";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vData.GetBoolean("shred", *outEnabled);
	return vStatus;
}

/**************************************************************************
 *  methods for editing an XSet
 *************************************************************************/

EXPORT xam_status DECL 
VIM_XSystem_OpenXSet (const xsystem_handle inHandle,
					  const xam_xuid inXUID,
                      const xam_string inMode,
					  xset_handle* const outXSet)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemOpenXSet";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("mode", inMode);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXSet);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXSet, inHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL
VIM_XSystem_CopyXSet (const xsystem_handle inHandle,
                      const xam_xuid inXUID,
                      const xam_string inMode,
                      xset_handle* const outXSet)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemCopyXSet";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("mode", inMode);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
	{
		vStatus = vData.GetHandle("handle", *outXSet);
		if ( vStatus == 0 )
			sRemoteVIM.BindToHandle(*outXSet, inHandle);
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_Commit (const xset_handle inHandle,
				 xam_xuid* const outXUID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetCommit";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		vData.GetXUID("xuid", *outXUID);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_Close (const xset_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetClose";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == 0 )
		sRemoteVIM.ReleaseIndirectBinding(inHandle);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_Abandon (const xset_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetAbandon";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

/**************************************************************************
 *  methods for XSet migration
 *************************************************************************/
EXPORT xam_status DECL 
VIM_XSet_OpenExportXStream (const xset_handle inHandle,
							xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetOpenExportXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vData.GetHandle("handle", *outXStream);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_OpenImportXStream (const xset_handle inHandle,
							xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetOpenImportXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vData.GetHandle("handle", *outXStream);
	return vStatus;
}

/**************************************************************************
 *  methods for job control
 *************************************************************************/
EXPORT xam_status DECL 
VIM_XSet_SubmitJob (const xset_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetSubmitJob";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_HaltJob (const xset_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetHaltJob";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

/**************************************************************************
 *  methods for asynchronous i/o
 **************************************************************************
 *  async handle manipulation
 *************************************************************************/
EXPORT xam_status DECL 
VIM_XAsync_IsComplete (const xasync_handle inHandle,
                       xam_boolean* const outIsComplete)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XASyncIsComplete";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetBoolean("async_complete", *outIsComplete);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetStatus (const xasync_handle inHandle,
                      xam_status* const outStatus)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetStatus";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetStatus("async_status", *outStatus);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetXOPID (const xasync_handle inHandle,
                     XOPID* const outXOPID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetXOPID";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetInt("async_xopid", *outXOPID);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetXSet (const xasync_handle inHandle,
                    xset_handle* const outXSet)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetXSet";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetHandle("async_xset", *outXSet);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetXStream (const xasync_handle inHandle,
                       xstream_handle* const outXStream)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetHandle("async_xstream", *outXStream);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetXUID (const xasync_handle inHandle,
                    xam_xuid* const outXUID)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetXUID";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetXUID("async_xuid", *outXUID);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetBytesRead (const xasync_handle inHandle,
                         xam_int* const outBytesRead)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetBytesRead";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetInt("async_bytesread", *outBytesRead);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_GetBytesWritten (const xasync_handle inHandle,
                            xam_int* const outBytesWritten)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncGetBytesWritten";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if( vStatus == 0 )
		vStatus = vData.GetInt("async_byteswritten", *outBytesWritten);
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_Close (const xasync_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem((xam_handle_t)inHandle);
		char* vCommand = "XAsyncClose";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	return vStatus;
}

/**************************************************************************
 *  async methods
 *************************************************************************/
EXPORT xam_status DECL 
VIM_XSystem_AsyncOpenXSet (const xsystem_handle inHandle,
                           const xam_xuid inXUID,
                           const xam_string inMode,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemAsyncOpenXSet";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("mode", inMode);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSystem_AsyncCopyXSet (const xsystem_handle inHandle,
                           const xam_xuid inXUID,
                           const xam_string inMode,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle)
{
   	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemAsyncCopyXSet";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		vVIM.AppendArg("xuid", inXUID);
		vVIM.AppendArg("mode", inMode);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSystem_AsyncOpenXStream (const xsystem_handle inHandle,
                              const xam_string inName,
                              const xam_string inMode,
                              const XOPID inXOPID,
                              xasync_callback inCallback,
                              xasync_handle* const outAsyncHandle)
{
   	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		char* vCommand = "XSystemAsyncOpenXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(inHandle, vCommand);
		vVIM.AppendArg("name", inName);
		vVIM.AppendArg("mode", inMode);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_AsyncOpenXStream (const xset_handle inHandle,
                           const xam_string inName,
                           const xam_string inMode,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle)
{
   	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetAsyncOpenXStream";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("name", inName);
		vVIM.AppendArg("mode", inMode);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XStream_AsyncRead (const xstream_handle inHandle,
                       char* const ioBuffer,
                       const xam_int inBufferLength,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle)
{
   	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamAsyncRead";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("numbytes", inBufferLength);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
			{
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
				sRemoteVIM.BindToBuffer(vAsyncHandle, inBufferLength, ioBuffer);
			}
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XStream_AsyncWrite (const xstream_handle inHandle,
                        const char* inBuffer,
                        const xam_int inByteCount,
                        const XOPID inXOPID,
                        xasync_callback inCallback,
                        xasync_handle* const outAsyncHandle)
{
   	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamAsyncWrite";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.POST(inBuffer,inByteCount, &vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XSet_AsyncCommit (const xset_handle inHandle,
                      const XOPID inXOPID,
                      xasync_callback inCallback,
                      xasync_handle* const outAsyncHandle)
{
   	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XSetAsyncCommit";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XStream_AsyncClose (const xstream_handle inHandle,
                        const XOPID inXOPID,
                        xasync_callback inCallback,
                        xasync_handle* const outAsyncHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XStreamAsyncClose";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		vVIM.AppendArg("xopid", inXOPID);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	if ( vStatus == (xam_status)0 )
	{
		// add the handle to the sRemoteVIM
        xasync_handle vAsyncHandle;
		vStatus = vData.GetHandle("handle", vAsyncHandle);
		if ( vStatus == (xam_status)0 )
		{
			if ( vAsyncHandle == (xasync_handle)0 )
				vStatus = XAM_INVALID_HANDLE;
			else
				sRemoteVIM.BindToHandle(vAsyncHandle, inCallback, inHandle);
			*outAsyncHandle = vAsyncHandle;
		}
	}
	return vStatus;
}

EXPORT xam_status DECL 
VIM_XAsync_Halt (const xasync_handle inHandle)
{
	xam_status vStatus = (xam_status)0;
	XferBuffer vData;
	try
	{
		xsystem_handle vHandle = sRemoteVIM.GetRegisteredXSystem(inHandle);
		char* vCommand = "XASyncHalt";
		SingleRemoteVIM& vVIM = sRemoteVIM.GetVIM(vHandle, vCommand, inHandle);
		std::string vHeader;
		try { vVIM.GET(&vHeader, &vData); }
		catch (...)
		{
			vStatus = (xam_status)1001;
		}
		try{sRemoteVIM.ReleaseVIM(vVIM);}catch(...){}
	}
	catch(...){vStatus = (xam_status)1001;} // unknown error

	// this might be premature... need to evaluate possible error conditions
	if ( vStatus == (xam_status)0 )
		sRemoteVIM.ReleaseIndirectBinding(inHandle);
	return vStatus;
}




