#ifndef XAM_TOOLKIT_H
#define XAM_TOOLKIT_H
////////////////////////////////////////////////////////////////////////////////
//  Copyright  2008, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright  2008, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

/* types and definitions */
#include "xam_types.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup XAMToolkit XAM Toolkit
    @{ */

/* =======================================================================
 * methods for XUID conversion and evaluation 
 * ======================================================================*/

/** @defgroup XUIDUtils XUID Utilities
    @{ */

/**
    Converts a binary XUID into it's equivalent displayable string format.

    An error is returned if the source XUID is not found to be valid.

    @param inBinaryXUID A valid XUID in binary form.
    @param outStringXUID A reference to valid storage for a xam_string. 
                         The value that is passed in is not used and is 
                         overwritten with the converted result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
XAMToolkit_XUIDtoString(const xam_xuid inBinaryXUID,
                        xam_string* const outStringXUID);

/**
    Converts a displayable string XUID into it's equivalent binary format.

    An error is returned if the converted XUID is not found to be valid.

    @param inStringXUID A valid XUID in displayable string form.
    @param outBinaryXUID A reference to valid storage for a xam_xuid. 
                         The value that is passed in is not used and is 
                         overwritten with the converted result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
XAMToolkit_StringtoXUID(const xam_string inStringXUID,
                        xam_xuid* const outBinaryXUID);

/**
    Determines if a given binary XUID is valid. 

    A XUID is valid if it's length and CRC bytes confirm the integrity
    of the identifier.

    @param inBinaryXUID A valid XUID in binary form.
    @param outIsValid A reference to valid storage for a xam_boolean. 
                      The value that is passed in is not used and is 
                      overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
XAMToolkit_IsValidXUID(const xam_xuid inBinaryXUID,
                       xam_boolean* const outIsValid);

/** @} */ /* XUIDUtils */

/* =======================================================================
 * methods for evaluating field attributes 
 * ======================================================================*/

/** @defgroup FieldAttribUtils Field Attribute Utilities
    @{ */

/**
    Evaluates the field type and determines if it is a property field. 
    If it is a property, then the method will set the passed in value 
    to TRUE; otherwise it will be set to FALSE.

    @param inHandle A valid xam_handle_t containing an XSet, XSystem, or
                    XAM Library object reference.
    @param inName A xam_string containing the name of the field.
    @param outIsProperty A reference to valid storage for a xam_boolean. 
                         The value that is passed in is not used and is 
                         overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
XAMToolkit_IsPropertyField (const xam_handle_t inHandle,
                            const xam_string inName,
                            xam_boolean* const outIsProperty);

/**
    Evaluates the field type and determines if it is an XStream field. 
    If it is an XStream, then the method will set the passed in value 
    to TRUE; otherwise it will be set to FALSE.

    @param inHandle A valid xam_handle_t containing an XSet, XSystem, or
                    XAM Library object reference.
    @param inName A xam_string containing the name of the field.
    @param outIsXStream A reference to valid storage for a xam_boolean. 
                         The value that is passed in is not used and is 
                         overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
XAMToolkit_IsXStreamField (const xam_handle_t inHandle,
                           const xam_string inName,
                           xam_boolean* const outIsXStream);

/** @} */ /* FieldAttribUtils */

/** @} */ /* XAMToolkit END */

#ifdef __cplusplus
} //extern "C"
#endif

#endif // XAM_TOOLKIT_H


     

