#ifndef XAM_TYPES_H
#define XAM_TYPES_H 1
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

/* ========================================================================= *\
 *                      Supporting definitions                               *
\* ========================================================================= */

#define XAM_MAX_STRING 512 
#define XAM_MAX_XUID 80

#define XAM_INT_MAX 0x7fffffffffffffffLL

#ifndef TRUE
  #define TRUE (unsigned char)1
#endif
#ifndef FALSE
  #define FALSE (unsigned char)0
#endif

#ifdef _WIN32
  #undef EXPORT
  #define EXPORT __declspec(dllexport)  
  #undef DECL
  #define DECL __cdecl      
#else
  #define EXPORT
  #define DECL
#endif

/* ======================================================================== *\
 *                          Primitive types                                 *
\* ======================================================================== */


#ifdef _WIN32
  #if defined (__GNUC__) 
    #include <stdint.h>
    typedef int64_t xam_int;                    /**< 8-byte signed integer   */
  #else                                                                                                  
    typedef __int64 xam_int;                    /**< 8-byte signed integer   */
  #endif                                                                                                                                                                                                             
#else // POSIX
   #include <inttypes.h>
   #if defined (__IBMC__) || defined (__IBMCPP__)
      typedef long long xam_int;                /**< 8-byte signed integer */
   #else
      typedef int64_t xam_int;                  /**< 8-byte signed integer */
   #endif                  
#endif /* WIN32 or POSIX */                                                                  

#if !defined(__cplusplus)
    typedef unsigned char           xam_boolean;
    #define true                    (unsigned char)1
    #define false                   (unsigned char)0
#else
    typedef bool                    xam_boolean;
#endif

typedef double                  xam_double;
typedef char                    xam_xuid[XAM_MAX_XUID];
typedef char                    xam_string[XAM_MAX_STRING];
typedef char                    xam_datetime[XAM_MAX_STRING];

typedef xam_xuid                XUID;

/* ======================================================================== *\
 *                           Method status                                  *
\* ======================================================================== */

#ifdef _WIN32
  #if defined (__GNUC__)                                                                                  
    typedef int32_t xam_status;                  /**< 4-byte signed integer */
  #else                                                                                                  
    typedef __int32 xam_status;                  /**< 4-byte signed integer */
  #endif                                                                                                                                                                                                             
#else // POSIX
   #if defined (__IBMC__) || defined (__IBMCPP__)
      typedef int xam_status;                    /**< 4-byte signed integer */
   #else
      typedef int32_t xam_status;                /**< 4-byte signed integer */
   #endif                  
#endif /* WIN32 or POSIX */                                                                  


/* ======================================================================== *\
 *                   Handles for the various XAM classes                    *
\* ======================================================================== */

typedef xam_int                xam_handle_t;
typedef xam_handle_t           xset_handle;
typedef xam_handle_t           xsystem_handle;
typedef xam_handle_t           xiterator_handle;
typedef xam_handle_t           xstream_handle;
typedef xam_handle_t           xasync_handle;

#define XAM_LIBRARY_HANDLE     (xam_handle_t)1   /**< XAM Library constant */

/* ======================================================================== *\
 *                  asynchronous operations typedefs                        *
\* ======================================================================== */

typedef xam_int                XOPID;

typedef void (*xasync_callback) (const xasync_handle inHandle);

/* ======================================================================== *\
 *                        Misc constants                                    *
\* ======================================================================== */

/* XAM Library log levels */
static const xam_int    XAM_LOG_ALL    = 5L;
static const xam_int    XAM_LOG_INFO   = 4L;
static const xam_int    XAM_LOG_WARN   = 3L;
static const xam_int    XAM_LOG_ERROR  = 2L;
static const xam_int    XAM_LOG_FATAL  = 1L;
static const xam_int    XAM_LOG_OFF    = 0L;

/* XStream whence values */
static const xam_int    XSTREAM_SEEK_SET  = 0L;
static const xam_int    XSTREAM_SEEK_CUR  = 1L;
static const xam_int    XSTREAM_SEEK_END  = 2L;

/* Access bits */
static const xam_int    XSET_ACCESS_READ_OK             = 0x80000000L; 
static const xam_int    XSET_ACCESS_WRITE_APP_OK        = 0x40000000L;
static const xam_int    XSET_ACCESS_WRITE_SYSTEM_OK     = 0x20000000L;
static const xam_int    XSET_ACCESS_CREATE_OK           = 0x10000000L;
static const xam_int    XSET_ACCESS_DELETE_OK           = 0x08000000L;
static const xam_int    XSET_ACCESS_HOLD_OK             = 0x04000000L;
static const xam_int    XSET_ACCESS_RETENTION_EVENT_OK  = 0x02000000L;
static const xam_int    XSET_ACCESS_JOB_OK              = 0x01000000L;
static const xam_int    XSET_ACCESS_JOB_COMMIT_OK       = 0x00800000L;

/* Access bit composites */
#define XSET_ACCESS_WRITE_OK  (XSET_ACCESS_WRITE_APP_OK|XSET_ACCESS_WRITE_SYSTEM_OK)
#define XSET_ACCESS_ALL_OK    (XSET_ACCESS_READ_OK|XSET_ACCESS_WRITE_OK|XSET_ACCESS_CREATE_OK|XSET_ACCESS_DELETE_OK)

#endif /* XAM_TYPES_H */


