/////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_Native.h"
#include "xam.h"
#include "xam_types.h"
#include "XAM_POSIX.h"
#include "XAsync.h"
#include "XAM_Logger.h"

static void throwException (JNIEnv *jniEnv, xam_status inStatus);
static void throwOutOfMemoryException (JNIEnv *jniEnv);
static jobject NewXAsync(JNIEnv *jniEnv, jlong inHandle);
void CallbackJNIFunction(const xasync_handle inHandle);

JavaVM *m_vm;

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetErrorToken
// Signature: (JI)Ljava/lang/String;
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jstring JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetErrorToken(JNIEnv *jniEnv,
                                                         jclass inClass,
                                                         jlong inHandle,
                                                         jint inStatus)
{
    xam_string  vTokenArray = "";
    xam_boolean vStatus = false;
    jstring vRetVal = NULL;

	vStatus = XAM_GetErrorToken(inHandle,inStatus,&vTokenArray);
	if(vStatus) // success case.
	{
        if(XAM_POSIX::xam_strlen(vTokenArray) >= sizeof(xam_string))
        {
            const xam_int vXAMStringSize = sizeof(xam_string);
            char vString[vXAMStringSize + 1];
            XAM_POSIX::xam_strncpy(vString, vTokenArray, vXAMStringSize);
            vString[vXAMStringSize] = '\0';
            vRetVal = jniEnv->NewStringUTF(vString);
        }
        else
        {
            vRetVal = (jstring)jniEnv->NewStringUTF(vTokenArray);
        }
	}
	else //throw an Exception
	{
        jclass vExceptionClass;
        vExceptionClass = jniEnv->FindClass("java/lang/Exception");
        if (vExceptionClass != NULL)
        {
		    jniEnv->ThrowNew(vExceptionClass, "");
        }
        jniEnv->DeleteLocalRef(vExceptionClass);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_OpenFieldIterator
// Signature: (JLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1OpenFieldIterator(JNIEnv *jniEnv,
                                                               jclass inClass,
                                                               jlong inHandle,
                                                               jstring inPattern)
{
	xiterator_handle vRetVal = -1;
	xam_status vStatus = 0;
	xiterator_handle vOutIterator = -1;
	jboolean vSuccess = true;
	const char* vPattern = (char*)0;
	xam_string vErrCode = "";

	vPattern = jniEnv->GetStringUTFChars(inPattern, NULL);
	if(vPattern != NULL)
    {
		vStatus = XAM_OpenFieldIterator(inHandle,vPattern,&vOutIterator);
		jniEnv->ReleaseStringUTFChars(inPattern, vPattern);

		if(vStatus == 0) //success
		{
			vRetVal = vOutIterator;
		}else //throw an Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_ContainsField
// Signature: (JLjava/lang/String;)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAM_1ContainsField(JNIEnv *jniEnv, 
                                                         jclass inClass, 
                                                         jlong inHandle, 
                                                         jstring inFieldName)
{
    xam_status vStatus = 0;
	jboolean vRetVal = false;
    xam_boolean vOutFieldExists = false;
	const char* vFieldName = (char*)0;
	xam_string vErrCode = "";

	vFieldName = jniEnv->GetStringUTFChars(inFieldName, NULL);
	if(vFieldName != NULL)
    {
        vStatus = XAM_ContainsField(inHandle, vFieldName, &vOutFieldExists);
        jniEnv->ReleaseStringUTFChars(inFieldName, vFieldName);
	    if(vStatus == 0) //success
	    {
		    vRetVal = vOutFieldExists;
	    }else
	    {
            throwException(jniEnv, vStatus);
	    }
    }
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XIterator_HasNext
// Signature: (J)Z;
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL 
  Java_org_snia_xam_natives_XAM_1Native_XIterator_1HasNext(JNIEnv *jniEnv, 
                                                           jclass inClass, 
                                                           jlong inHandle)
{
	xam_status vStatus = 0;
	jboolean vRetVal = false;
    xam_boolean vOutHasNext = false;

    vStatus = XIterator_HasNext(inHandle, &vOutHasNext);
	if(vStatus == 0) //success
	{
		vRetVal = vOutHasNext;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XIterator_Next
// Signature: (J)Ljava/lang/String;
///////////////////////////////////////////////////////////////////////////////
  
JNIEXPORT jstring JNICALL
  Java_org_snia_xam_natives_XAM_1Native_XIterator_1Next(JNIEnv *jniEnv,
                                                        jclass inClass,
                                                        jlong inHandle)
{
	xam_status vStatus = 0;
	jstring vRetVal = NULL;
	xam_string vNextIteratorArray = "";

	vStatus = XIterator_Next(inHandle,&vNextIteratorArray);
	if(vStatus == 0) //success
    {
        if(XAM_POSIX::xam_strlen(vNextIteratorArray) >= sizeof(xam_string))
        {
            const xam_int vXAMStringSize = sizeof(xam_string);
            char vString[vXAMStringSize + 1];
            XAM_POSIX::xam_strncpy(vString, vNextIteratorArray, vXAMStringSize);
            vString[vXAMStringSize] = '\0';
            vRetVal = jniEnv->NewStringUTF(vString);
        }
        else
        {
		    vRetVal = jniEnv->NewStringUTF(vNextIteratorArray);
        }
	}
	else //create an Exception
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XIterator_Close
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XIterator_1Close(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XIterator_Close(inHandle);

	if(vStatus != 0) //throw Exception
    {
        throwException(jniEnv, vStatus);
    }
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetFieldAsBinding
// Signature: (JLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetFieldAsBinding(JNIEnv *jniEnv,
                                                             jclass inClass,
                                                             jlong inHandle,
                                                             jstring inName)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
    {
		vStatus = XAM_SetFieldAsBinding(inHandle,vName);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus != 0)
	    {
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetFieldAsNonbinding
// Signature: (JLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
  Java_org_snia_xam_natives_XAM_1Native_XAM_1SetFieldAsNonbinding(JNIEnv *jniEnv,
                                                                  jclass inClass,
                                                                  jlong inHandle,
                                                                  jstring inName)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_SetFieldAsNonbinding(inHandle,vName);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus != 0)
	    {
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetFieldType
// Signature: (JLjava/lang/String;)Ljava/lang/String;
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jstring JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetFieldType(JNIEnv *jniEnv,
                                                        jclass inClass,
                                                        jlong inHandle,
                                                        jstring inName)
{
	xam_status vStatus = 0;
	jstring vRetVal = NULL;
	const char* vName = (char*)0;
	xam_string vOutType = "";

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
    {
		vStatus = XAM_GetFieldType(inHandle,vName,&vOutType);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus == 0) //success
        { 
            if(XAM_POSIX::xam_strlen(vOutType) >= sizeof(xam_string))
            {
                const xam_int vXAMStringSize = sizeof(xam_string);
                char vString[vXAMStringSize + 1];
                XAM_POSIX::xam_strncpy(vString, vOutType, vXAMStringSize);
                vString[vXAMStringSize] = '\0';
                vRetVal = jniEnv->NewStringUTF(vString);
            }
            else
            {
			    vRetVal = jniEnv->NewStringUTF(vOutType);
            }
		}else{
            throwException(jniEnv, vStatus);
        }
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetFieldLength
// Signature: (JLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetFieldLength(JNIEnv *jniEnv,
                                                          jclass inClass,
                                                          jlong inHandle,
                                                          jstring inName)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	const char* vName = (char*)0;
	xam_int outLength = 0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
    {
		vStatus = XAM_GetFieldLength(inHandle,vName,&outLength);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vRetVal = outLength;
		}else{
	        throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetFieldBinding
// Signature: (JLjava/lang/String;)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetFieldBinding(JNIEnv *jniEnv,
                                                           jclass inClass,
                                                           jlong inHandle,
                                                           jstring inName)
{
	xam_status vStatus = 0;
	jboolean vRetVal = false;
	const char* vName = (char*)0;
	xam_boolean outFieldBinding = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
    {
		vStatus = XAM_GetFieldBinding(inHandle,vName,&outFieldBinding);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vRetVal = outFieldBinding;
		}else
        {
            throwException(jniEnv, vStatus);
        }
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetFieldReadOnly
// Signature: (JLjava/lang/String;)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetFieldReadOnly(JNIEnv *jniEnv,
                                                            jclass inClass,
                                                            jlong inHandle,
                                                            jstring inName)
{
	xam_status vStatus = 0;
	jboolean vRetVal = false;
	const char* vName = (char*)0;
	xam_boolean outReadOnly = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
    {
		vStatus = XAM_GetFieldReadOnly(inHandle,vName,&outReadOnly);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vRetVal = outReadOnly;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_DeleteField
// Signature: (JLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1DeleteField(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle,
                                                       jstring inName)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
    {
		vStatus = XAM_DeleteField(inHandle,vName);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateBoolean
// Signature: (JLjava/lang/String;ZZ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateBoolean(JNIEnv *jniEnv,
                                                         jclass inClass,
                                                         jlong inHandle,
                                                         jstring inName,
                                                         jboolean inBinding,
                                                         jboolean inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	bool vBinding = false;
	bool vValue = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        vBinding = inBinding ? true : false;
        vValue = inValue ? true : false;
		
		vStatus = XAM_CreateBoolean(inHandle,vName, vBinding, vValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateInt
// Signature: (JLjava/lang/String;ZJ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateInt(JNIEnv *jniEnv,
                                                     jclass inClass,
                                                     jlong inHandle,
                                                     jstring inName,
                                                     jboolean inBinding,
                                                     jlong inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	bool vBinding = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        vBinding = inBinding ? true : false;
       
		vStatus = XAM_CreateInt(inHandle,vName, vBinding, inValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateDouble
// Signature: (JLjava/lang/String;ZD)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateDouble(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle,
                                                       jstring inName,
                                                       jboolean inBinding,
                                                       jdouble inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	bool vBinding = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        vBinding = inBinding ? true : false;

		vStatus = XAM_CreateDouble(inHandle,vName, vBinding, inValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateXUID
// Signature: (JLjava/lang/String;Z[B)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateXUID(JNIEnv *jniEnv,
                                                      jclass inClass,
                                                      jlong inHandle,
                                                      jstring inName,
                                                      jboolean inBinding,
                                                      jbyteArray inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	xam_xuid vValue = "";
	bool vBinding = false;
    xam_xuid vXUIDReqBytes = "";

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        jniEnv->GetByteArrayRegion(inValue,0,8,(jbyte *)vXUIDReqBytes);
        jbyte vActualLength = vXUIDReqBytes[5]; 
		jniEnv->GetByteArrayRegion(inValue,0,vActualLength,(jbyte *)vValue);

        vBinding = inBinding ? true : false;

        vStatus = XAM_CreateXUID(inHandle,vName, vBinding, vValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateString
// Signature: (JLjava/lang/String;ZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateString(JNIEnv *jniEnv,
                                                        jclass inClass,
                                                        jlong inHandle,
                                                        jstring inName,
                                                        jboolean inBinding,
                                                        jstring inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	const char* vValue = (char*)0;
	bool vBinding = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vValue = jniEnv->GetStringUTFChars(inValue, NULL);
		if(vValue != NULL)
		{
            vBinding = inBinding ? true : false;

			vStatus = XAM_CreateString(inHandle,vName, vBinding, vValue);
			jniEnv->ReleaseStringUTFChars(inName, vName);
			jniEnv->ReleaseStringUTFChars(inValue, vValue);
			if(vStatus != 0)
			{
                throwException(jniEnv, vStatus);
            }
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateDatetime
// Signature: (JLjava/lang/String;ZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateDatetime(JNIEnv *jniEnv,
                                                          jclass inClass,
                                                          jlong inHandle,
                                                          jstring inName,
                                                          jboolean inBinding,
                                                          jstring inValue)

{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	const char* vValue = (char*)0;
	bool vBinding = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vValue = jniEnv->GetStringUTFChars(inValue, NULL);
		if(vValue != NULL)
		{
            vBinding = inBinding ? true : false;
			vStatus = XAM_CreateDatetime(inHandle,vName, vBinding, vValue);
			jniEnv->ReleaseStringUTFChars(inName, vName);
			jniEnv->ReleaseStringUTFChars(inValue, vValue);

			if(vStatus !=0)
			{
                throwException(jniEnv, vStatus);
            }
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetBoolean
// Signature: (JLjava/lang/String;Z)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetBoolean(JNIEnv *jniEnv,
                                                      jclass inClass,
                                                      jlong inHandle,
                                                      jstring inName,
                                                      jboolean inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	bool vValue = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        vValue = inValue ? true : false;
		vStatus = XAM_SetBoolean(inHandle,vName, vValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
        }

	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetInt
// Signature: (JLjava/lang/String;J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetInt(JNIEnv *jniEnv,
                                                  jclass inClass,
                                                  jlong inHandle,
                                                  jstring inName,
                                                  jlong inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_SetInt(inHandle,vName, inValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetDouble
// Signature: (JLjava/lang/String;D)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetDouble(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle,
                                                    jstring inName,
                                                    jdouble inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_SetDouble(inHandle,vName, inValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetXUID
// Signature: (JLjava/lang/String;[B)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetXUID(JNIEnv *jniEnv,
                                                   jclass inClass,
                                                   jlong inHandle,
                                                   jstring inName,
                                                   jbyteArray inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	xam_xuid vValue = "";
    xam_xuid vXUIDReqBytes = "";

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        jniEnv->GetByteArrayRegion(inValue,0,8,(jbyte *)vXUIDReqBytes);
        jbyte vActualLength = vXUIDReqBytes[5]; 
		jniEnv->GetByteArrayRegion(inValue,0,vActualLength,(jbyte *)vValue);

		vStatus = XAM_SetXUID(inHandle,vName, vValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);
		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetString
// Signature: (JLjava/lang/String;Ljava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetString(JNIEnv *jniEnv,
                                                     jclass inClass,
                                                     jlong inHandle,
                                                     jstring inName,
                                                     jstring inValue)

{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	const char* vValue = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vValue = jniEnv->GetStringUTFChars(inValue, NULL);
		if(vValue != NULL)
		{
			vStatus = XAM_SetString(inHandle,vName, vValue);
			jniEnv->ReleaseStringUTFChars(inName, vName);
			jniEnv->ReleaseStringUTFChars(inValue, vValue);

			if(vStatus !=0)
			{
                throwException(jniEnv, vStatus);
            }
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_SetDatetime
// Signature: (JLjava/lang/String;Ljava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1SetDatetime(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle,
                                                       jstring inName,
                                                       jstring inValue)
{
	xam_status vStatus = 0;
	const char* vName = (char*)0;
	const char* vValue = (char*)0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vValue = jniEnv->GetStringUTFChars(inValue, NULL);
		if(vValue != NULL)
		{
			vStatus = XAM_SetDatetime(inHandle,vName, vValue);
			jniEnv->ReleaseStringUTFChars(inName, vName);
			jniEnv->ReleaseStringUTFChars(inValue, vValue);
			if(vStatus !=0)
			{
	            throwException(jniEnv, vStatus);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetBoolean
// Signature: (JLjava/lang/String;)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetBoolean(JNIEnv *jniEnv,
                                                      jclass inClass,
                                                      jlong inHandle,
                                                      jstring inName)
{
	xam_status vStatus = 0;
	jboolean vRetVal = false;
	const char* vName = (char*)0;
	xam_boolean vOutValue = false;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_GetBoolean(inHandle,vName, &vOutValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vRetVal = vOutValue;
		}else
		{
            throwException(jniEnv, vStatus);
        }
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetInt
// Signature: (JLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetInt(JNIEnv *jniEnv,
                                                  jclass inClass,
                                                  jlong inHandle,
                                                  jstring inName)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	const char* vName = (char*)0;
	xam_int vOutValue = 0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_GetInt(inHandle,vName,&vOutValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vRetVal = vOutValue;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetDouble
// Signature: (JLjava/lang/String;)D
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jdouble JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetDouble(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle,
                                                    jstring inName)
{
	xam_status vStatus = 0;
	jdouble vRetVal = 0.0;
	const char* vName = (char*)0;
	xam_double vOutValue = 0.0;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_GetDouble(inHandle,vName,&vOutValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vRetVal = vOutValue;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetXUID
// Signature: (JLjava/lang/String;)[B
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jbyteArray JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetXUID(JNIEnv *jniEnv,
                                                   jclass inClass,
                                                   jlong inHandle,
                                                   jstring inName)
{
	xam_status vStatus = 0;
	jbyteArray vRetVal = NULL;
	const char* vName = (char*)0;
	jstring vString = NULL;
	xam_xuid vOutValue = "";
	jbyteArray vByteArray = NULL;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_GetXUID(inHandle,vName,&vOutValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
			vByteArray=jniEnv->NewByteArray(80);
			jniEnv->SetByteArrayRegion(vByteArray, 0, 79, (jbyte *)&vOutValue) ;
			vRetVal = vByteArray;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetString
// Signature: (JLjava/lang/String;)Ljava/lang/String;
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jstring JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetString(JNIEnv *jniEnv,
                                                     jclass inClass,
                                                     jlong inHandle,
                                                     jstring inName)
{
	xam_status vStatus = 0;
	jstring vRetVal = NULL;
	const char* vName = (char*)0;
	xam_string vOutValue = "";

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_GetString(inHandle,vName,&vOutValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
		{
            if(XAM_POSIX::xam_strlen(vOutValue) >= sizeof(xam_string))
            {
                const xam_int vXAMStringSize = sizeof(xam_string);
                char vString[vXAMStringSize + 1];
                XAM_POSIX::xam_strncpy(vString, vOutValue, vXAMStringSize);
                vString[vXAMStringSize] = '\0';
                vRetVal = jniEnv->NewStringUTF(vString);
            }
            else
            {
			    vRetVal = jniEnv->NewStringUTF(vOutValue);
            }
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_GetDatetime
// Signature: (JLjava/lang/String;)Ljava/lang/String;
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jstring JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1GetDatetime(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle,
                                                       jstring inName)
{
	xam_status vStatus = 0;
	jstring vRetVal = NULL;
	const char* vName = (char*)0;
	jstring vString = NULL;
	xam_datetime vOutValue ="";

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vStatus = XAM_GetDatetime(inHandle, vName, &vOutValue);
		jniEnv->ReleaseStringUTFChars(inName, vName);

		if(vStatus == 0) //success
        {
            if(XAM_POSIX::xam_strlen(vOutValue) >= sizeof(xam_string))
            {
                const xam_int vXAMStringSize = sizeof(xam_string);
                char vString[vXAMStringSize + 1];
                XAM_POSIX::xam_strncpy(vString, vOutValue, vXAMStringSize);
                vString[vXAMStringSize] = '\0';
                vRetVal = jniEnv->NewStringUTF(vString);
            }
            else
            {   
			    vRetVal = jniEnv->NewStringUTF(vOutValue);
            }
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_CreateXStream
// Signature: (JLjava/lang/String;ZLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1CreateXStream(JNIEnv *jniEnv,
                                                         jclass inClass,
                                                         jlong inHandle,
                                                         jstring inName,
                                                         jboolean inBinding,
                                                         jstring inType)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	const char* vName = (char*)0;
	const char* vType = (char*)0;
	xstream_handle vOutXStream;
	bool vBinding = false;

    vBinding = inBinding ? true : false;
	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vType = jniEnv->GetStringUTFChars(inType, NULL);
		if(vType != NULL)
		{
			vStatus = XAM_CreateXStream(inHandle,vName,vBinding,vType,&vOutXStream);
			jniEnv->ReleaseStringUTFChars(inName, vName);
			jniEnv->ReleaseStringUTFChars(inType, vType);
			if(vStatus == 0) //success
			{
				vRetVal = vOutXStream;
			}else
			{
                throwException(jniEnv, vStatus);
			}
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_OpenXStream
// Signature: (JLjava/lang/String;Ljava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1OpenXStream(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle,
                                                       jstring inName,
                                                       jstring inMode)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	const char* vName = (char*)0;
	const char* vMode = (char*)0;
	xstream_handle vOutXStream = -1;

	vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
		vMode = jniEnv->GetStringUTFChars(inMode, NULL);
		if(vMode != NULL)
		{
			vStatus = XAM_OpenXStream(inHandle,vName,vMode,&vOutXStream);
			jniEnv->ReleaseStringUTFChars(inName, vName);
			jniEnv->ReleaseStringUTFChars(inMode, vMode);
			if(vStatus == 0) //success
			{
				vRetVal = vOutXStream;
			}else
			{
                throwException(jniEnv, vStatus);
			}
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_Read
// Signature: (J[BJ)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XStream_1Read(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle,
                                                    jbyteArray inBuffer,
                                                    jlong inBufferLength)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	xam_int vOutBytesRead = 0;
	char* vBuffer = 0;
	jboolean vIsCopy = false;

	vBuffer = (char *)jniEnv->GetByteArrayElements(inBuffer, &vIsCopy);
	if (vBuffer)
	{
        // JNI_OK = 0 - Always copy and free
		vStatus = XStream_Read(inHandle,vBuffer,inBufferLength,&vOutBytesRead);
        jniEnv->ReleaseByteArrayElements(inBuffer, (jbyte*)vBuffer, JNI_OK); 
		if(vStatus == 0) //success
		{
			vRetVal = vOutBytesRead;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_Write
// Signature: (J[BJ)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XStream_1Write(JNIEnv *jniEnv,
                                                     jclass inClass,
                                                     jlong inHandle,
                                                     jbyteArray inBuffer,
                                                     jlong inByteCount)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	xam_int outBytesWritten = 0;
	char* vBuffer = 0;
	jboolean vIsCopy = false;

	vBuffer = (char *)jniEnv->GetByteArrayElements(inBuffer, &vIsCopy);
	if (vBuffer)
	{
        // JNI_OK = 0 - Always copy and free
        // JNI_ABORT = 2 - Free without copying back to Java (for performance)
		vStatus = XStream_Write(inHandle, vBuffer, inByteCount, &outBytesWritten);
        jniEnv->ReleaseByteArrayElements(inBuffer, (jbyte*)vBuffer, vIsCopy ? JNI_ABORT : JNI_OK);
		if(vStatus == 0) //success
		{
			vRetVal = outBytesWritten;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_Seek
// Signature: (JJJ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XStream_1Seek(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle,
                                                    jlong inOffset,
                                                    jlong inWhence)
{
	xam_status vStatus = 0;
	vStatus = XStream_Seek(inHandle, inOffset, inWhence);
	if(vStatus != 0)
	{
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_Tell
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XStream_1Tell(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	xam_int vOutPosition = 0;

	vStatus = XStream_Tell(inHandle, &vOutPosition);

	if(vStatus == 0) //success
	{
		vRetVal = vOutPosition;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_Abandon
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XStream_1Abandon(JNIEnv *jniEnv,
                                                       jclass inClass,
                                                       jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XStream_Abandon(inHandle);
	if(vStatus != 0)
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_Close
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XStream_1Close(JNIEnv *jniEnv,
                                                     jclass inClass,
                                                     jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XStream_Close(inHandle);
	if(vStatus != 0)
	{
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAMLibrary_Connect
// Signature: (Ljava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XAM_1Connect(JNIEnv *jniEnv,
                                                   jclass inClass,
                                                   jstring inXRI)
{
	xam_status vStatus = 0;
	xsystem_handle vRetVal = -1;
	xsystem_handle vOutHandle = -1;
	const char* vXRI = (char*)0;

    vXRI = jniEnv->GetStringUTFChars(inXRI, NULL);
	if(vXRI != NULL)
	{
  	    vStatus =  XAMLibrary_Connect(vXRI,&vOutHandle);
	    jniEnv->ReleaseStringUTFChars(inXRI, vXRI);
	    if(vStatus == 0) //success
	    {
		   vRetVal = vOutHandle;
	    }else //create an Exception
		{
            throwException(jniEnv, vStatus);
        }
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_Authenticate
// Signature: (J[BJ)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1Authenticate(JNIEnv *jniEnv,
                                                            jclass inClass,
                                                            jlong inHandle,
                                                            jbyteArray inBuffer,
                                                            jlong inByteCount)
{
	xam_status vStatus = 0;
	jlong vRetVal = 0;
	xstream_handle vOutXStream = 0;
    char* vBuffer = (char*)0;
    jboolean vIsCopy = false;
    
    vBuffer = (char *)jniEnv->GetByteArrayElements(inBuffer, &vIsCopy);
	if (vBuffer != NULL)
	{
        // JNI_OK = 0 - Always copy and free
        // JNI_ABORT = 2 - Free without copying back to Java (for performance)
		vStatus = XSystem_Authenticate(inHandle, vBuffer, inByteCount, &vOutXStream);
        jniEnv->ReleaseByteArrayElements(inBuffer, (jbyte*)vBuffer, vIsCopy ? JNI_ABORT : JNI_OK);

		if((vStatus == 0) || (vStatus == XAM_AUTH_DATA_NEEDED)) //success
		{
			vRetVal = vOutXStream;
		}
		else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_Authenticate2
// Signature: (J[BLjava/io/ByteArrayOutputStream;)I
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jint 
JNICALL Java_org_snia_xam_natives_XAM_1Native_XSystem_1Authenticate2(JNIEnv *jniEnv, 
                                                                     jclass inClass, 
                                                                     jlong inHandle, 
                                                                     jbyteArray inBuffer,
                                                                     jlong inByteCount,
                                                                     jobject inObject)
{
	xam_status vStatus = 0;
    xam_status vXStreamReadStatus = 0;
	jlong vRetVal = 0;
    jint vAuthStatus = 0;
	xstream_handle vOutXStream = 0;
    char* vBuffer = (char*)0;
    jboolean vIsCopy = false;
    
    vBuffer = (char *)jniEnv->GetByteArrayElements(inBuffer, &vIsCopy);
	if (vBuffer != NULL)
	{
        // JNI_OK = 0 - Always copy and free
        // JNI_ABORT = 2 - Free without copying back to Java (for performance)
		vStatus = XSystem_Authenticate(inHandle, vBuffer, inByteCount, &vOutXStream);
        jniEnv->ReleaseByteArrayElements(inBuffer, (jbyte*)vBuffer, vIsCopy ? JNI_ABORT : JNI_OK);

		if((vStatus == 0) || (vStatus == XAM_AUTH_DATA_NEEDED)) //success
		{
            if(vOutXStream != NULL)
            {
                //write the XSystem responce in to the jobject.
                jclass vByteArrayOutputStreamClass = jniEnv->GetObjectClass(inObject);
                //jfieldID vFieldId = jniEnv->GetFieldID(vByteArrayOutputStreamClass, "buf","[B;");;
                jmethodID vMethodID = jniEnv->GetMethodID(vByteArrayOutputStreamClass, "write", "([BII)V");

                jbyte vReadData[512];
                
                xam_int outBytes = 0;

                while(outBytes >= 0)
                {
                    vXStreamReadStatus = XStream_Read(vOutXStream, (char*)vReadData, sizeof(xam_string), &outBytes);
                    

                    //call the user callback
                    if(vXStreamReadStatus != 0)
                    {
                        throwException(jniEnv, vXStreamReadStatus);
                    }else
                    {
                        if(outBytes >= 0)
                        {
                            jbyteArray vReadDataArray = jniEnv->NewByteArray((jsize)outBytes);
                            jniEnv->SetByteArrayRegion(vReadDataArray, 0, (jsize)outBytes, vReadData);
                            jniEnv->CallVoidMethod(inObject, vMethodID, vReadDataArray, 0, (int)outBytes);
                            jniEnv->DeleteLocalRef(vReadDataArray);
                        }
                    }
                }

                 // close the stream
                 XStream_Close(vOutXStream);
            }
			vAuthStatus = (jint)vStatus;
		}
		else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vAuthStatus;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_Close
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1Close(JNIEnv *jniEnv,
                                                     jclass inClass,
                                                     jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XSystem_Close(inHandle);
	if(vStatus !=0)
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_Abandon
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSystem_1Abandon(JNIEnv *jniEnv, 
                                                       jclass inClass, 
                                                       jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XSystem_Abandon(inHandle);
	if(vStatus !=0)
	{
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_CreateXSet
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1CreateXSet(JNIEnv *jniEnv,
                                                          jclass inClass,
                                                          jlong inHandle, 
                                                          jstring inMode)

{
	xam_status vStatus = 0;
	xset_handle vOutXSet = -1;
	jlong vRetVal = 0;
	const char* vMode = (char*)0;

    vMode = jniEnv->GetStringUTFChars(inMode, NULL);
	if(vMode != NULL)
	{
	    vStatus = XSystem_CreateXSet(inHandle, vMode, &vOutXSet);
		jniEnv->ReleaseStringUTFChars(inMode, vMode);

	    if(vStatus == 0) //success
	    {
		    vRetVal = vOutXSet;
	    }
        else
	    {
            throwException(jniEnv, vStatus);
	    }
    }
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_OpenXSet
// Signature: (J[BLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1OpenXSet(JNIEnv *jniEnv,
                                                        jclass inClass,
                                                        jlong inHandle,
                                                        jbyteArray inXUID,
                                                        jstring inMode)
{
	xam_status vStatus = 0;
	xset_handle vOutXSet = -1;
	xam_xuid vXUID = "";
	const char* vMode = (char*)0;
	jlong vRetVal = 0;
    xam_xuid vXUIDReqBytes = "";

    jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);

	vMode = jniEnv->GetStringUTFChars(inMode, NULL);
	if(vMode != NULL)
	{
		vStatus	= XSystem_OpenXSet(inHandle,vXUID,vMode,&vOutXSet);

		jniEnv->ReleaseStringUTFChars(inMode, vMode);
		if(vStatus == 0) //success
		{
		vRetVal = vOutXSet;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_CopyXSet
// Signature: (J[BLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1CopyXSet(JNIEnv *jniEnv,
                                                        jclass inClass,
                                                        jlong inHandle,
                                                        jbyteArray inXUID,
                                                        jstring inMode)
{
	xam_status vStatus = 0;
	xset_handle vOutXSet = -1;
	xam_xuid vXUID = "";
	const char* vMode = (char*)0;
	jlong vRetVal = 0;
    xam_xuid vXUIDReqBytes = "";

	jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);

	vMode = jniEnv->GetStringUTFChars(inMode, NULL);
	if(vMode != NULL)
	{
		vStatus	= XSystem_CopyXSet(inHandle,vXUID,vMode,&vOutXSet);

		jniEnv->ReleaseStringUTFChars(inMode, vMode);
		if(vStatus == 0) //success
		{
		vRetVal = vOutXSet;
		}else
		{
            throwException(jniEnv, vStatus);
		}
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_DeleteXSet
// Signature: (J[B)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1DeleteXSet(JNIEnv *jniEnv,
                                                          jclass inClass,
                                                          jlong inHandle,
                                                          jbyteArray inXUID)
{
	xam_status vStatus = 0;
	xam_xuid vXUID = "";
    xam_xuid vXUIDReqBytes = "";

    jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 

	jniEnv->GetByteArrayRegion(inXUID,0, vActualLength,(jbyte *)vXUID);
	vStatus = XSystem_DeleteXSet((xsystem_handle)inHandle,vXUID);
	if(vStatus !=0)
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_HoldXSet
// Signature: (J[BLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1HoldXSet(JNIEnv *jniEnv,
                                                        jclass inClass,
                                                        jlong inHandle,
                                                        jbyteArray inXUID,
                                                        jstring inHoldID)
{
	xam_status vStatus = 0;
	const char* vHoldID = (char*)0;
	xam_xuid vXUID = "";
    xam_xuid vXUIDReqBytes = "";

	jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);

	vHoldID = jniEnv->GetStringUTFChars(inHoldID, NULL);
	if(vHoldID != NULL)
	{
		vStatus = XSystem_HoldXSet(inHandle,vXUID,vHoldID);

		jniEnv->ReleaseStringUTFChars(inHoldID, vHoldID);
		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_ReleaseXSet
// Signature: (J[BLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1ReleaseXSet(JNIEnv *jniEnv,
                                                           jclass inClass,
                                                           jlong inHandle,
                                                           jbyteArray inXUID,
                                                           jstring inHoldID)
{
	xam_status vStatus = 0;
	const char* vHoldID = (char*)0;
	xam_xuid vXUID = "";
    xam_xuid vXUIDReqBytes = "";

	jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);
	vHoldID = jniEnv->GetStringUTFChars(inHoldID, NULL);
	if(vHoldID != NULL)
	{
		vStatus = XSystem_ReleaseXSet(inHandle,vXUID,vHoldID);
		jniEnv->ReleaseStringUTFChars(inHoldID, vHoldID);

		if(vStatus !=0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_AccessXSet
// Signature: (J[BJ)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1AccessXSet(JNIEnv *jniEnv,
                                                          jclass inClass,
                                                          jlong inHandle,
                                                          jbyteArray inXUID,
                                                          jlong inMode)
{
	xam_status vStatus = 0;
	xam_boolean vOutIsAccessible = false;
	jboolean vRetVal = false;
	xam_xuid vXUID = "";
    xam_xuid vXUIDReqBytes = "";

	jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);
	vStatus = XSystem_AccessXSet(inHandle,vXUID,inMode,&vOutIsAccessible);

	if(vStatus == 0) //success
	{
		vRetVal = vOutIsAccessible;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_IsXSetRetained
// Signature: (J[B)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSystem_1IsXSetRetained(JNIEnv *jniEnv, 
                                                              jclass inClass, 
                                                              jlong inHandle, 
                                                              jbyteArray inXUID)
{
	xam_status vStatus = 0;
	xam_boolean vOutIsRetained = false;
	jboolean vRetVal = false;
	xam_xuid vXUID = "";
    xam_xuid vXUIDReqBytes = "";

	jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);
    vStatus = XSystem_IsXSetRetained(inHandle,vXUID,&vOutIsRetained);

	if(vStatus == 0) //success
	{
		vRetVal = vOutIsRetained;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_GetXSetAccessTime
// Signature: (J[B)Ljava/lang/String;
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jstring JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSystem_1GetXSetAccessTime(JNIEnv *jniEnv,
                                                                 jclass inClass,
                                                                 jlong inHandle,
                                                                 jbyteArray inXUID)
{
	xam_status vStatus = 0;
	xam_datetime vOutAccessTime = "";
	jstring vRetVal = NULL;
	xam_xuid vXUID = "";
    xam_xuid vXUIDReqBytes = "";

	jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);
	vStatus = XSystem_GetXSetAccessTime(inHandle,vXUID,&vOutAccessTime);

	if(vStatus == 0) //success
    {
        if(XAM_POSIX::xam_strlen(vOutAccessTime) >= sizeof(xam_string))
        {
            const xam_int vXAMStringSize = sizeof(xam_string);
            char vString[vXAMStringSize + 1];
            XAM_POSIX::xam_strncpy(vString, vOutAccessTime, vXAMStringSize);
            vString[vXAMStringSize] = '\0';
            vRetVal = jniEnv->NewStringUTF(vString);
        }
        else
        {
		    vRetVal = jniEnv->NewStringUTF(vOutAccessTime);
        }
	}else
	{
         throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_Commit
// Signature: (J)[B
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jbyteArray JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1Commit(JNIEnv *jniEnv,
                                                   jclass inClass,
                                                   jlong inHandle)
{
	xam_status vStatus = 0;
	xam_xuid vOutXUID = "";
	jbyteArray vRetVal = NULL;

	vStatus	= XSet_Commit(inHandle,&vOutXUID);

	if(vStatus == 0) //success
    {
		vRetVal=jniEnv->NewByteArray(80);
		jniEnv->SetByteArrayRegion(vRetVal, 0, 79, (jbyte *)&vOutXUID) ;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_Close
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1Close(JNIEnv *jniEnv,
                                                  jclass inClass,
                                                  jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XSet_Close(inHandle);
	if(vStatus != 0) //throw Exception
    {
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_Abandon
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1Abandon(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XSet_Abandon(inHandle);
	if(vStatus != 0) //throw Exception
    {
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyAccessPolicy
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyAccessPolicy(JNIEnv *jniEnv, 
                                                              jclass inClass, 
                                                              jlong inHandle, 
                                                              jboolean inBinding, 
                                                              jstring inPolicyName)
{
    xam_status vStatus = 0;
	const char* vPolicy = (char*)0;
	bool vBinding = false;

	vPolicy = jniEnv->GetStringUTFChars(inPolicyName, NULL);
	if(vPolicy != NULL)
	{
        vBinding = inBinding ? true : false;
        vStatus = XSet_ApplyAccessPolicy(inHandle,vBinding,vPolicy);
		jniEnv->ReleaseStringUTFChars(inPolicyName, vPolicy);

		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ResetAccessFields
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1ResetAccessFields(JNIEnv *jniEnv, 
                                                              jclass inClass, 
                                                              jlong inHandle)
{
	xam_status vStatus = 0;
    vStatus = XSet_ResetAccessFields(inHandle);
	if(vStatus != 0) //throw Exception
    {
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyManagementPolicy
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyManagementPolicy(JNIEnv *jniEnv,
                                                                  jclass inClass,
                                                                  jlong inHandle,
                                                                  jboolean inBinding,
                                                                  jstring inPolicy)
{
	xam_status vStatus = 0;
	const char* vPolicy = (char*)0;
	bool vBinding = false;

	vPolicy = jniEnv->GetStringUTFChars(inPolicy, NULL);
	if(vPolicy != NULL)
	{
        vBinding = inBinding ? true : false;
		vStatus = XSet_ApplyManagementPolicy(inHandle,vBinding,vPolicy);
		jniEnv->ReleaseStringUTFChars(inPolicy, vPolicy);

		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ResetManagementFields
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1ResetManagementFields(JNIEnv *jniEnv,
                                                                    jclass inClass,
                                                                    jlong inHandle)

{
	xam_status vStatus = 0;
	vStatus = XSet_ResetManagementFields(inHandle);
	if(vStatus != 0) //throw an exception
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyAutoDeletePolicy
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyAutoDeletePolicy(
                                                                JNIEnv *jniEnv,
                                                                jclass inClass,
                                                                jlong inHandle,
                                                                jboolean inBinding,
                                                                jstring inPolicy)
{
	xam_status vStatus = 0;
	const char* vPolicy = (char*)0;
	bool vBinding = false;

	vPolicy = jniEnv->GetStringUTFChars(inPolicy, NULL);
	if(vPolicy != NULL)
	{
        vBinding = inBinding ? true : false;
		vStatus = XSet_ApplyAutoDeletePolicy(inHandle,vBinding,vPolicy);
		jniEnv->ReleaseStringUTFChars(inPolicy, vPolicy);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SetAutoDelete
// Signature: (JZZ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1SetAutoDelete(JNIEnv *jniEnv,
                                                          jclass inClass,
                                                          jlong inHandle,
                                                          jboolean inBinding,
                                                          jboolean inAutoDelete)
{
	xam_status vStatus = 0;
	bool vBinding = false;
	bool vAutoDelete = false;

    vBinding = inBinding ? true : false;
    vAutoDelete = inAutoDelete ? true : false;
	vStatus = XSet_SetAutoDelete(inHandle,vBinding,vAutoDelete);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyShredPolicy
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyShredPolicy(JNIEnv *jniEnv, 
                                                             jclass inClass, 
                                                             jlong inHandle, 
                                                             jboolean inBinding, 
                                                             jstring inPolicyName)
{
    xam_status vStatus = 0;
	const char* vPolicy = (char*)0;
	bool vBinding = false;

	vPolicy = jniEnv->GetStringUTFChars(inPolicyName, NULL);
	if(vPolicy != NULL)
	{
        vBinding = inBinding ? true : false;
        vStatus = XSet_ApplyShredPolicy(inHandle,vBinding,vPolicy);
		jniEnv->ReleaseStringUTFChars(inPolicyName, vPolicy);

		if(vStatus != 0)
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyStoragePolicy
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyStoragePolicy(JNIEnv *jniEnv,
                                                               jclass inClass,
                                                               jlong inHandle,
                                                               jboolean inBinding,
                                                               jstring inPolicy)
{
	xam_status vStatus = 0;
	const char* vPolicy = (char*)0;
	bool vBinding = false;

	vPolicy = jniEnv->GetStringUTFChars(inPolicy, NULL);
	if(vPolicy != NULL)
	{
        vBinding = inBinding ? true : false;
    	vStatus = XSet_ApplyStoragePolicy(inHandle,vBinding,vPolicy);
		jniEnv->ReleaseStringUTFChars(inPolicy, vPolicy);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_GetActualAutoDelete
// Signature: (J)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1GetActualAutoDelete(JNIEnv *jniEnv,
                                                                jclass inClass,
                                                                jlong inHandle)
{
	xam_status vStatus = 0;
	xam_boolean vOutEnabled = false;
	jboolean vRetVal = false;

	vStatus = XSet_GetActualAutoDelete(inHandle,&vOutEnabled);
	if(vStatus == 0) //success
	{
		vRetVal = vOutEnabled;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_GetActualShred
// Signature: (J)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1GetActualShred(JNIEnv *jniEnv, 
                                                           jclass inClass, 
                                                           jlong inHandle)
{
    jboolean vRetVal = false;
    xam_status vStatus = 0;
	xam_boolean vOutShred = 0;

    vStatus = XSet_GetActualShred(inHandle,&vOutShred);
	if(vStatus == 0) //success
	{
		vRetVal = vOutShred;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_CreateRetention
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1CreateRetention(JNIEnv *jniEnv, 
                                                            jclass inClass, 
                                                            jlong inHandle, 
                                                            jboolean inBinding, 
                                                            jstring inRetentionID)
{
	xam_status vStatus = 0;
	const char* vRetentionID = (char*)0;
	bool vBinding = false;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vBinding = inBinding ? true : false;
        vStatus = XSet_CreateRetention(inHandle,vBinding,vRetentionID);
		jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SetRetentionEnabledFlag
// Signature: (JLjava/lang/String;ZZ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1SetRetentionEnabledFlag(JNIEnv *jniEnv, 
                                                                    jclass inClass, 
                                                                    jlong inHandle, 
                                                                    jstring inRetentionID, 
                                                                    jboolean inBinding, 
                                                                    jboolean inEnabled)
{
	xam_status vStatus = 0;
	const char* vRetentionID = (char*)0;
	bool vBinding = false;
    bool vEnabled = false;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vBinding = inBinding ? true : false;
        vEnabled = inEnabled ? true : false;

        vStatus = XSet_SetRetentionEnabledFlag(inHandle,vRetentionID,vBinding,vEnabled);
		jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyEventRetentionEnabledPolicy
// Signature: (JLjava/lang/String;ZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyRetentionEnabledPolicy(JNIEnv *jniEnv, 
                                                                             jclass inClass, 
                                                                             jlong inHandle, 
                                                                             jstring inRetentionID, 
                                                                             jboolean inBinding, 
                                                                             jstring inPolicyName)
{
	xam_status vStatus = 0;
	const char* vRetentionID = (char*)0;
    const char* vPolicyName = (char*)0;
	bool vBinding = false;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vPolicyName = jniEnv->GetStringUTFChars(inPolicyName, NULL);
	    if(vPolicyName != NULL)
	    {
            vBinding = inBinding ? true : false;
            vStatus = XSet_ApplyRetentionEnabledPolicy(inHandle, vRetentionID, vBinding,vPolicyName);
		    jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
            jniEnv->ReleaseStringUTFChars(inPolicyName, vPolicyName);
		    if(vStatus != 0) //throw Exception
		    {
                throwException(jniEnv, vStatus);
		    }
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SetRetentionDuration
// Signature: (JLjava/lang/String;ZJ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1SetRetentionDuration(JNIEnv *jniEnv, 
                                                                 jclass inClass, 
                                                                 jlong inHandle, 
                                                                 jstring inRetentionID, 
                                                                 jboolean inBinding, 
                                                                 jlong inDuration)
{
   	xam_status vStatus = 0;
	const char* vRetentionID = (char*)0;
	bool vBinding = false;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vBinding = inBinding ? true : false;
        vStatus = XSet_SetRetentionDuration(inHandle, vRetentionID, vBinding, inDuration);
		jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyRetentionDurationPolicy
// Signature: (JLjava/lang/String;ZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyRetentionDurationPolicy(JNIEnv *jniEnv, 
                                                                         jclass inClass, 
                                                                         jlong inHandle, 
                                                                         jstring inRetentionID, 
                                                                         jboolean inBinding, 
                                                                         jstring inPolicyName)
{
   	xam_status vStatus = 0;
	const char* vRetentionID = (char*)0;
    const char* vPolicyName = (char*)0;
	bool vBinding = false;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vPolicyName = jniEnv->GetStringUTFChars(inPolicyName, NULL);
	    if(vPolicyName != NULL)
	    {
            vBinding = inBinding ? true : false;
            vStatus = XSet_ApplyRetentionDurationPolicy(inHandle, vRetentionID, vBinding,vPolicyName);
		    jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
            jniEnv->ReleaseStringUTFChars(inPolicyName, vPolicyName);
		    if(vStatus != 0) //throw Exception
		    {
                throwException(jniEnv, vStatus);
		    }
        }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SetRetentionStarttime
// Signature: (JLjava/lang/String;Z)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1SetRetentionStarttime(JNIEnv *jniEnv, 
                                                                  jclass inClass, 
                                                                  jlong inHandle, 
                                                                  jstring inRetentionID, 
                                                                  jboolean inBinding)
{
    xam_status vStatus = 0;
	const char* vRetentionID = (char*)0;
	bool vBinding = false;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vBinding = inBinding ? true : false;
        vStatus = XSet_SetRetentionStarttime(inHandle, vRetentionID, vBinding);
		jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SetBaseRetention
// Signature: (JZJ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1SetBaseRetention(JNIEnv *jniEnv, 
                                                             jclass inClass, 
                                                             jlong inHandle, 
                                                             jboolean inBinding, 
                                                             jlong inDuration)
{
    xam_status vStatus = 0;
	bool vBinding = false;
    vBinding = inBinding ? true : false;
    vStatus = XSet_SetBaseRetention(inHandle, vBinding, inDuration);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_ApplyBaseRetentionPolicy
// Signature: (JZLjava/lang/String;)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1ApplyBaseRetentionPolicy(JNIEnv *jniEnv, 
                                                                     jclass inClass, 
                                                                     jlong inHandle, 
                                                                     jboolean inBinding, 
                                                                     jstring inPolicyName)
{
    xam_status vStatus = 0;
	const char* vPolicyName = (char*)0;
	bool vBinding = false;

	vPolicyName = jniEnv->GetStringUTFChars(inPolicyName, NULL);
	if(vPolicyName != NULL)
	{
        vBinding = inBinding ? true : false;
        vStatus = XSet_ApplyBaseRetentionPolicy(inHandle, vBinding, vPolicyName);
		jniEnv->ReleaseStringUTFChars(inPolicyName, vPolicyName);
		if(vStatus != 0) //throw Exception
		{
            throwException(jniEnv, vStatus);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SetShred
// Signature: (JZZ)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1SetShred(JNIEnv *jniEnv, 
                                                     jclass inClass, 
                                                     jlong inHandle, 
                                                     jboolean inBinding, 
                                                     jboolean inShred)
{
    xam_status vStatus = 0;
	bool vBinding = false;
	bool vShred = false;

    vBinding = inBinding ? true : false;
    vShred = inShred ? true : false;
	vStatus = XSet_SetShred(inHandle,vBinding,vShred);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_GetActualRetentionDuration
// Signature: (JLjava/lang/String;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1GetActualRetentionDuration(JNIEnv *jniEnv, 
                                                                       jclass inClass, 
                                                                       jlong inHandle, 
                                                                       jstring inRetentionID)
{
	xam_status vStatus = 0;
	xam_int vOutRetention = 0;
	jlong vRetVal = 0;
    const char* vRetentionID = (char*)0;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
	    vStatus	= XSet_GetActualRetentionDuration(inHandle, vRetentionID, &vOutRetention);
        jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
	    if(vStatus == 0) //success
	    {
		    vRetVal = vOutRetention;
	    }else
	    {
            throwException(jniEnv, vStatus);
	    }
    }
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_GetActualRetentionEnabled
// Signature: (JLjava/lang/String;)Z
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jboolean JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1GetActualRetentionEnabled(JNIEnv *jniEnv, 
                                                                      jclass inClass, 
                                                                      jlong inHandle, 
                                                                      jstring inRetentionID)
{
    xam_status vStatus = 0;
	jboolean vRetVal = false;
    xam_boolean vOutRetentionEnabled = false;
    const char* vRetentionID = (char*)0;

	vRetentionID = jniEnv->GetStringUTFChars(inRetentionID, NULL);
	if(vRetentionID != NULL)
	{
        vStatus = XSet_GetActualRetentionEnabled(inHandle, vRetentionID, &vOutRetentionEnabled);
        jniEnv->ReleaseStringUTFChars(inRetentionID, vRetentionID);
	    if(vStatus == 0) //success
	    {
		    vRetVal = vOutRetentionEnabled;
	    }else
	    {
            throwException(jniEnv, vStatus);
	    }
    }
    return vRetVal;
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_OpenExportXStream
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1OpenExportXStream(JNIEnv *jniEnv,
                                                              jclass inClass,
                                                              jlong inHandle)
{
	xam_status vStatus = 0;
	xstream_handle vOutXStream = -1;
	jlong vRetVal = 0;

	vStatus	= XSet_OpenExportXStream(inHandle,&vOutXStream);
	if(vStatus == 0) //success
	{
		vRetVal = vOutXStream;
	}
    else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_OpenImportXStream
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1OpenImportXStream(JNIEnv *jniEnv,
                                                              jclass inClass,
                                                              jlong inHandle)
{
	xam_status vStatus = 0;
	xstream_handle vOutXStream = -1;
	jlong vRetVal = 0;

	vStatus	= XSet_OpenImportXStream(inHandle,&vOutXStream);
	if(vStatus == 0) //success
	{
		vRetVal = vOutXStream;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_SubmitJob
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1SubmitJob(JNIEnv *jniEnv,
                                                      jclass inClass,
                                                      jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XSet_SubmitJob(inHandle);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_HaltJob
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT void JNICALL
Java_org_snia_xam_natives_XAM_1Native_XSet_1HaltJob(JNIEnv *jniEnv,
                                                    jclass inClass,
                                                    jlong inHandle)
{
	xam_status vStatus = 0;
	vStatus = XSet_HaltJob(inHandle);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}



///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_AsyncOpenXSet
// Signature: (J[BLjava/lang/String;JLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSystem_1AsyncOpenXSet(JNIEnv *jniEnv, 
                                                             jclass inClass, 
                                                             jlong inHandle, 
                                                             jbyteArray inXUID, 
                                                             jstring inMode, 
                                                             jlong inXOPID, 
                                                             jobject inListener)
{
	xam_status vStatus = 0;
	xasync_handle vAsyncHandle = -1;
	xam_xuid vXUID = "";
	const char* vMode = (char*)0;
	jobject vRetVal = 0;
    xam_xuid vXUIDReqBytes = "";
    
    //store the VM
    jniEnv->GetJavaVM(&m_vm );

    jobject vGlobalListener = 0;
    jobject vGlobalAsyncObj = 0;

    jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);
    vMode = jniEnv->GetStringUTFChars(inMode, NULL);
    
    if(vMode != NULL)
    {
        vStatus	= XSystem_AsyncOpenXSet(inHandle, 
                                        vXUID, 
                                        vMode, 
                                        inXOPID, 
                                        CallbackJNIFunction, 
                                        &vAsyncHandle);

        jniEnv->ReleaseStringUTFChars(inMode, vMode);
	    if(vStatus == 0) //success
	    {
           vRetVal = NewXAsync(jniEnv,vAsyncHandle);
           XAsync *vAsync = (XAsync *)vAsyncHandle;
           
           vGlobalListener = jniEnv->NewGlobalRef(inListener); 
           vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 

           vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, NULL, NULL, NULL, false, NULL);
	    }else
	    {
            throwException(jniEnv, vStatus);
	    }
    }
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSystem_AsyncCopyXSet
// Signature: (J[BLjava/lang/String;JLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSystem_1AsyncCopyXSet(JNIEnv *jniEnv, 
                                                             jclass inClass, 
                                                             jlong inHandle, 
                                                             jbyteArray inXUID, 
                                                             jstring inMode, 
                                                             jlong inXOPID, 
                                                             jobject inListener)
{
 	xam_status vStatus = 0;
	xasync_handle vAsyncHandle = -1;
	xam_xuid vXUID = "";
	const char* vMode = (char*)0;
	jobject vRetVal = 0;
    xam_xuid vXUIDReqBytes = "";

    //store the VM
    jniEnv->GetJavaVM(&m_vm );

    jobject vGlobalListener = 0;
    jobject vGlobalAsyncObj = 0;

    jniEnv->GetByteArrayRegion(inXUID,0,8,(jbyte *)vXUIDReqBytes);
    jbyte vActualLength = vXUIDReqBytes[5]; 
	jniEnv->GetByteArrayRegion(inXUID,0,vActualLength,(jbyte *)vXUID);

    vMode = jniEnv->GetStringUTFChars(inMode, NULL);
    if(vMode != NULL)
    {
        vStatus	= XSystem_AsyncCopyXSet(inHandle, 
                                        vXUID, 
                                        vMode, 
                                        inXOPID, 
                                        CallbackJNIFunction, 
                                        &vAsyncHandle);

        jniEnv->ReleaseStringUTFChars(inMode, vMode);
	    if(vStatus == 0) //success
	    {
           vRetVal = NewXAsync(jniEnv,vAsyncHandle);
           XAsync *vAsync = (XAsync *)vAsyncHandle;
           vGlobalListener = jniEnv->NewGlobalRef(inListener); 
           vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 
           vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, NULL, NULL, NULL, false, NULL);
	    }else
	    {
            throwException(jniEnv, vStatus);
	    }
    }
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAM_AsyncOpenXStream
// Signature: (JLjava/lang/String;Ljava/lang/String;JLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAM_1AsyncOpenXStream(JNIEnv *jniEnv, 
                                                            jclass inClass, 
                                                            jlong inHandle, 
                                                            jstring inName, 
                                                            jstring inMode, 
                                                            jlong inXOPID, 
                                                            jobject inListener)
{
 	xam_status vStatus = 0;
	xasync_handle vAsyncHandle = -1;
	const char* vMode = (char*)0;
    const char* vName = (char*)0;
	jobject vRetVal = 0;

    //store the VM
    jniEnv->GetJavaVM(&m_vm );

    jobject vGlobalListener = 0;
    jobject vGlobalAsyncObj = 0;

    vName = jniEnv->GetStringUTFChars(inName, NULL);
	if(vName != NULL)
	{
        vMode = jniEnv->GetStringUTFChars(inMode, NULL);
        if(vMode != NULL)
        {
            vStatus	= XAM_AsyncOpenXStream(inHandle, 
                                           vName, 
                                           vMode, 
                                           inXOPID, 
                                           CallbackJNIFunction, 
                                           &vAsyncHandle);

            jniEnv->ReleaseStringUTFChars(inMode, vMode);
            jniEnv->ReleaseStringUTFChars(inName, vName);

	        if(vStatus == 0) //success
	        {
               vRetVal = NewXAsync(jniEnv,vAsyncHandle);
               XAsync *vAsync = (XAsync *)vAsyncHandle;
               vGlobalListener = jniEnv->NewGlobalRef(inListener); 
               vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 
               vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, NULL, NULL, NULL, false, NULL);
	        }else
	        {
                throwException(jniEnv, vStatus);
	        }
        }
    }
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_AsyncRead
// Signature: (J[BJJLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////

JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XStream_1AsyncRead(JNIEnv *jniEnv, 
                                                         jclass inClass, 
                                                         jlong inHandle, 
                                                         jbyteArray inBuffer, 
                                                         jlong inBufferLength, 
                                                         jlong inXOPID, 
                                                         jobject inListener)
{
 	xasync_handle vAsyncHandle = -1;
	jobject vRetVal = 0;
	xam_status vStatus = 0;
	char* vBuffer = 0;
	jboolean vIsCopy = false;

	vBuffer = (char *)jniEnv->GetByteArrayElements(inBuffer, &vIsCopy);

    if (vBuffer)
    {
        //jniEnv->ReleaseByteArrayElements(inBuffer, (jbyte*)vBuffer, vIsCopy ? JNI_ABORT : JNI_OK);
            //store the VM
            jniEnv->GetJavaVM(&m_vm );
            jobject vGlobalListener = 0;
            jobject vGlobalAsyncObj = 0;
            jobject vGlobalReadArray = 0;

            vStatus	= XStream_AsyncRead(inHandle, 
                                        vBuffer, 
                                        inBufferLength, 
                                        inXOPID, 
                                        CallbackJNIFunction, /* Always call the JNI callback */
                                        &vAsyncHandle);

  		    if(vStatus == 0) //success
		    {
			    vRetVal = NewXAsync(jniEnv,vAsyncHandle);
                XAsync *vAsync = (XAsync *)vAsyncHandle;
                vGlobalListener = jniEnv->NewGlobalRef(inListener); 
                vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 
                vGlobalReadArray = jniEnv->NewGlobalRef(inBuffer);

                vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, NULL, vBuffer, vGlobalReadArray, false, NULL);
		    }
            else
		    {
                throwException(jniEnv, vStatus);
		    }
	    }
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_AsyncWrite
// Signature: (J[BJJLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XStream_1AsyncWrite(JNIEnv *jniEnv, 
                                                          jclass inClass, 
                                                          jlong inHandle, 
                                                          jbyteArray inBuffer, 
                                                          jlong inBufferLength, 
                                                          jlong inXOPID, 
                                                          jobject inListener)
{
    xasync_handle vAsyncHandle = -1;
	jobject vRetVal = 0;
	xam_status vStatus = 0;
	char* vBuffer = 0;
	jboolean vIsCopy = false;

    char *vCachedBuff = (char *)XAM_POSIX::xam_malloc(inBufferLength);
	vBuffer = (char *)jniEnv->GetByteArrayElements(inBuffer, &vIsCopy);
	
    if(vCachedBuff != NULL)
    {
        if (vBuffer)
	    {
            // JNI_OK = 0 - Always copy and free
            // JNI_ABORT = 2 - Free without copying back to Java (for performance)
            XAM_POSIX::xam_memcpy(vCachedBuff,vBuffer,inBufferLength);
            jniEnv->ReleaseByteArrayElements(inBuffer, (jbyte*)vBuffer, vIsCopy ? JNI_ABORT : JNI_OK);

            //store the VM
            jniEnv->GetJavaVM(&m_vm );

            jobject vGlobalListener = 0;
            jobject vGlobalAsyncObj = 0;

            vStatus	= XStream_AsyncWrite(inHandle, 
                                         vCachedBuff, 
                                         inBufferLength, 
                                         inXOPID, 
                                         CallbackJNIFunction,
                                         &vAsyncHandle);

		    if(vStatus == 0) //success
		    {
			    vRetVal = NewXAsync(jniEnv,vAsyncHandle);
                XAsync *vAsync = (XAsync *)vAsyncHandle;
                vGlobalListener = jniEnv->NewGlobalRef(inListener); 
                vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 
                vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, vCachedBuff, NULL, NULL, false, NULL);
		    }else
		    {
                throwException(jniEnv, vStatus);
		    }
	    }
    }else
    {
        throwOutOfMemoryException(jniEnv);
    }
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XSet_AsyncCommit
// Signature: (JJLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XSet_1AsyncCommit(JNIEnv *jniEnv, 
                                                        jclass inClass, 
                                                        jlong inHandle, 
                                                        jlong inXOPID, 
                                                        jobject inListener)
{
    xasync_handle vAsyncHandle = -1;
	jobject vRetVal = 0;
	xam_status vStatus = 0;

    //store the VM
    jniEnv->GetJavaVM(&m_vm );

    jobject vGlobalListener = 0;
    jobject vGlobalAsyncObj = 0;

	vStatus	= XSet_AsyncCommit(inHandle, 
                               inXOPID, 
                               CallbackJNIFunction, 
                               &vAsyncHandle);

	if(vStatus == 0) //success
    {
		vRetVal = NewXAsync(jniEnv,vAsyncHandle);
        XAsync *vAsync = (XAsync *)vAsyncHandle;
        vGlobalListener = jniEnv->NewGlobalRef(inListener); 
        vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 
        vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, NULL, NULL, NULL, false, NULL);
	}else
	{
        throwException(jniEnv, vStatus);
	}

	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XStream_AsyncClose
// Signature: (JJJLorg/snia/xam/XAsyncListener;)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jobject JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XStream_1AsyncClose(JNIEnv *jniEnv, 
                                                          jclass inClass, 
                                                          jlong inHandle, 
                                                          jlong inXOPID, 
                                                          jobject inListener,
                                                          jobject inXStreamObj)
{
    xasync_handle vAsyncHandle = -1;
	jobject vRetVal = 0;
	xam_status vStatus = 0;
   	
    //store the VM
    jniEnv->GetJavaVM(&m_vm );

    jobject vGlobalListener = 0;
    jobject vGlobalAsyncObj = 0;
    jobject vGlobalXStreamObject = 0;

	vStatus	= XStream_AsyncClose(inHandle, 
                                 inXOPID, 
                                 CallbackJNIFunction, /* Always call the JNI callback function */
                                 &vAsyncHandle);

	if(vStatus == 0) //success
    {
		vRetVal = NewXAsync(jniEnv,vAsyncHandle);
        XAsync *vAsync = (XAsync *)vAsyncHandle;
        vGlobalListener = jniEnv->NewGlobalRef(inListener); 
        vGlobalAsyncObj = jniEnv->NewGlobalRef(vRetVal); 
        vGlobalXStreamObject = jniEnv->NewGlobalRef(inXStreamObj); 
        vAsync->setJNIEnv(jniEnv, vGlobalListener, vGlobalAsyncObj, NULL, NULL, NULL, true, vGlobalXStreamObject);
	}else
	{
        throwException(jniEnv, vStatus);
	}
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_Halt
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1Halt(JNIEnv *jniEnv, 
                                                   jclass inClass, 
                                                   jlong inHandle)
{
	xam_status vStatus = 0;
    vStatus = XAsync_Halt(inHandle);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_IsComplete
// Signature: (J)Z
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jboolean JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1IsComplete(JNIEnv *jniEnv, 
                                                         jclass inClass, 
                                                         jlong inHandle)
{
    xam_status vStatus = 0;
	jboolean vRetVal = false;
    xam_boolean vIsComplete = false;

    vStatus = XAsync_IsComplete(inHandle, &vIsComplete);
	if(vStatus == 0) //success
	{
        if(vIsComplete)
        {
            // Determine if the callback has propagated changes back through JNI to Java
            // before allowing this operation to appear as complete.
            XAsync* vAsyncHandle = (XAsync*)inHandle;
		    vIsComplete = ((vIsComplete) && (vAsyncHandle->getJavaIsComplete()));
        }

        vRetVal = vIsComplete;
	}
    else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetStatus
// Signature: (J)I
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jint JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetStatus(JNIEnv *jniEnv, 
                                                        jclass inClass, 
                                                        jlong inHandle)
{
    jint vRetVal = 0;
    xam_status vStatus = 0;
    xam_status vOutStatus = 0;

    vStatus = XAsync_GetStatus(inHandle, &vOutStatus);
	if(vStatus == 0) //success
	{
		vRetVal = vOutStatus;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetXOPID
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jlong JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetXOPID(JNIEnv *jniEnv, 
                                                       jclass inClass, 
                                                       jlong inHandle)
{
    jlong vRetVal = 0;
    xam_status vStatus = 0;
    XOPID vOutXOPID = 0;

    vStatus = XAsync_GetXOPID(inHandle, &vOutXOPID);
	if(vStatus == 0) //success
	{
		vRetVal = vOutXOPID;
	}else
	{
        throwException(jniEnv, vStatus);
	}
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetXSet
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jlong JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetXSet(JNIEnv *jniEnv, 
                                                      jclass inClass, 
                                                      jlong inHandle)
{
    jlong vRetVal = 0;
    xam_status vStatus = 0;
    xset_handle vOutXSet = 0;

    vStatus = XAsync_GetXSet(inHandle, &vOutXSet);
	if(vStatus == 0) //success
	{
		vRetVal = vOutXSet;
	}else
	{
        throwException(jniEnv, vStatus);
	}
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetXStream
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jlong JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetXStream(JNIEnv *jniEnv, 
                                                         jclass inClass, 
                                                         jlong inHandle)
{
    jlong vRetVal = 0;
    xam_status vStatus = 0;
    xstream_handle vOutXSet = 0;

    vStatus = XAsync_GetXStream(inHandle, &vOutXSet);
	if(vStatus == 0) //success
	{
		vRetVal = vOutXSet;
	}else
	{
        throwException(jniEnv, vStatus);
	}
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetXUID
// Signature: (J)[B
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jbyteArray JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetXUID(JNIEnv *jniEnv, 
                                                      jclass inClass, 
                                                      jlong inHandle)
{
	xam_status vStatus = 0;
	jbyteArray vRetVal = NULL;
	xam_xuid vOutValue = "";
	jbyteArray vByteArray = NULL;

	vStatus = XAsync_GetXUID(inHandle,&vOutValue);

    if(vStatus == 0) //success
	{
		vByteArray=jniEnv->NewByteArray(80);
		jniEnv->SetByteArrayRegion(vByteArray, 0, 79, (jbyte *)&vOutValue) ;
		vRetVal = vByteArray;
	}else
	{
        throwException(jniEnv, vStatus);
	}
	return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetBytesRead
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jlong JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetBytesRead(JNIEnv *jniEnv, 
                                                           jclass inClass, 
                                                           jlong inHandle)
{
    jlong vRetVal = 0;
    xam_status vStatus = 0;
    xam_int vOutBytesRead = 0;

    vStatus = XAsync_GetBytesRead(inHandle, &vOutBytesRead);
	if(vStatus == 0) //success
	{
		vRetVal = vOutBytesRead;
	}else
	{
        throwException(jniEnv, vStatus);
	}
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_GetBytesWritten
// Signature: (J)J
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT jlong JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1GetBytesWritten(JNIEnv *jniEnv, 
                                                              jclass inClass, 
                                                              jlong inHandle)
{
    jlong vRetVal = 0;
    xam_status vStatus = 0;
    xam_int vOutBytesWritten = 0;

    vStatus = XAsync_GetBytesWritten(inHandle, &vOutBytesWritten);
	if(vStatus == 0) //success
	{
		vRetVal = vOutBytesWritten;
	}else
	{
        throwException(jniEnv, vStatus);
	}
    return vRetVal;
}

///////////////////////////////////////////////////////////////////////////////
// Class:     org_snia_xam_natives_XAM_Native
// Method:    XAsync_Close
// Signature: (J)V
///////////////////////////////////////////////////////////////////////////////
JNIEXPORT void JNICALL 
Java_org_snia_xam_natives_XAM_1Native_XAsync_1Close(JNIEnv *jniEnv, 
                                                    jclass inClass, 
                                                    jlong inHandle)
{
	xam_status vStatus = 0;
    vStatus = XAsync_Close(inHandle);
	if(vStatus != 0) //throw Exception
	{
        throwException(jniEnv, vStatus);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Helper functions
///////////////////////////////////////////////////////////////////////////////

void 
CallbackJNIFunction(const xasync_handle inHandle)
{
    jobject mJNIAsyncObj = 0;
    jobject mJNIAsyncListenerObj = 0;
    JNIEnv *mJNIEnv = 0;
    char *mCachedBuff = (char *)0; // user write buffer
    char *mReadBuff = (char *)0; // user read buffer
    jobject mReadJNIBuff = NULL; // JNI Read jbyteArray
    jboolean mAsyncClose = false; // true if call back is from ASyncClose.
    jobject mJavaXStreamObj = NULL; // XStream Object Only for ASyncClose.
    XAsync *vAsync = (XAsync *)inHandle;

    // Poll until Java has been successfully setup
    while(!vAsync->getJavaEnvInited())
    {
        // Poll until the Java Environment has been initialized 
        // in the XAsync object!
    }

    // Retrieve the JNI environment from the XAsync object
    vAsync->getJNIEnv(mJNIEnv, mJNIAsyncListenerObj, mJNIAsyncObj, mCachedBuff, mReadBuff, mReadJNIBuff, mAsyncClose, mJavaXStreamObj);
   
    m_vm->AttachCurrentThread((void**) &mJNIEnv, NULL );

    if(mCachedBuff != NULL)
    {
        XAM_POSIX::xam_free(mCachedBuff);
    }

    if(mReadBuff != NULL)
    {
        if(mReadJNIBuff != NULL)
        {
            //copy the data to java buffer
            mJNIEnv->ReleaseByteArrayElements((jbyteArray)mReadJNIBuff, (jbyte*)mReadBuff, JNI_OK);
            mJNIEnv->DeleteGlobalRef(mReadJNIBuff);
        }
    }

    // Mark this operation as java complete
    vAsync->setJavaIsComplete();

    // Call the user listener as needed
    if(mJNIAsyncListenerObj != 0)
    {
        jclass vCallBackClass = mJNIEnv->GetObjectClass(mJNIAsyncListenerObj);
        jmethodID vMethodID = mJNIEnv->GetMethodID(vCallBackClass, "XAsyncCallback", "(Lorg/snia/xam/XAsync;)V");

        //call the user callback
        mJNIEnv->CallVoidMethod(mJNIAsyncListenerObj, vMethodID, mJNIAsyncObj);
    }

    if(mAsyncClose)
    {
        jclass vXStreamImplClass = mJNIEnv->GetObjectClass(mJavaXStreamObj);
        jfieldID vFieldId = mJNIEnv->GetFieldID(vXStreamImplClass, "mHandle", "J");
        if(vFieldId != NULL)
        {
            mJNIEnv->SetLongField(mJavaXStreamObj, vFieldId, 0);
        }
    }

    // Cleanup
    mJNIEnv->DeleteGlobalRef(mJNIAsyncListenerObj);
    mJNIEnv->DeleteGlobalRef(mJNIAsyncObj);
    m_vm->DetachCurrentThread ();
}

static void throwException (JNIEnv *jniEnv, xam_status inStatus)
{ 
    xam_string vErrCode = "";
    jclass vExceptionClass;
	vExceptionClass = jniEnv->FindClass("java/lang/Exception");
	if (vExceptionClass != NULL)
	{
		XAM_POSIX::xam_sprintf(vErrCode,"%d", inStatus);
	    jniEnv->ThrowNew(vExceptionClass, vErrCode);
    }
	jniEnv->DeleteLocalRef(vExceptionClass);
}

static void throwOutOfMemoryException (JNIEnv *jniEnv)
{ 
    jclass vExceptionClass;
	vExceptionClass = jniEnv->FindClass("java/lang/OutOfMemoryError");
	if (vExceptionClass != NULL)
	{
	    jniEnv->ThrowNew(vExceptionClass, "OutOfMemoryError from JNI");
    }
	jniEnv->DeleteLocalRef(vExceptionClass);
}

 static jobject
 NewXAsync(JNIEnv *jniEnv, jlong inHandle)
 {
     jclass XAsyncClass;
     jmethodID vConstructor;
     jobject vRetVal;
 
     XAsyncClass = jniEnv->FindClass("org/snia/xam/XAsyncImpl");
     if (XAsyncClass == NULL) {
         vRetVal = NULL; /* exception thrown */
     }

    /* Get the Constructor ID*/
     vConstructor = jniEnv->GetMethodID(XAsyncClass,"<init>", "(J)V");
     if (vConstructor == NULL) {
          vRetVal = NULL; /* exception thrown */
     }
 
     /* Construct a org/snia/xam/XAsyncImpl object */
     vRetVal = jniEnv->NewObject(XAsyncClass, vConstructor, inHandle);
 
     /* Free local references */
      jniEnv->DeleteLocalRef(XAsyncClass);
     return vRetVal;
 }
