////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_strings.h"

#include "XAM_Logger.h"
#include "XAM_POSIX.h"
#include "XAM_LogFactory.h"

// Pointer to the active logger
XAM_LogFormatter* XAM_LogFactory::mLogBase = (XAM_LogFormatter*)0;

XAM_LogTabFormatter*
XAM_LogFactory::createLogTab(XAM_LogState* inOptions )
{
    // Create a tab delimited logger object
    XAM_LogTabFormatter *vLogTab = (XAM_LogTabFormatter*)0;
    try
    {
        vLogTab = new XAM_LogTabFormatter(inOptions);
    }
    catch (...)
    {
        vLogTab = (XAM_LogTabFormatter*)0;
    }

    return vLogTab;
}

void
XAM_LogFactory::applyOptions(XAM_LogState* inLogState)
{
    // If inLogState is NULL, reload the existing configuration
    XAM_LogState* vNewLogState = inLogState ? inLogState : this;

    // If logging is disabled, apply the settings and enable it (if now active)
    if (mLogBase == (XAM_LogFormatter*)0)
    {        
        // Apply the settings. Cache the log state before applying the change, 
        // as the vNewLogState could be the same as the one we're modifying
        XAM_LogState vLogState;
        vLogState.setLogState(vNewLogState);

        // Update this object to reflect the new state
        this->setLogState(&vLogState);

        if(isEnabled())
        {
            // Inform the XAM Logger of the state change
            XAM_Logger::setState(ON, vNewLogState->getLevel(), vNewLogState->getComponentMask());

            xam_int vFormat = vNewLogState->getFormat();
            if (vFormat == TAB)
                mLogBase = createLogTab(vNewLogState);
        }
    } 
    else // Logging is active
    {
        // If the log file path hasn't changed, update it's current settings
        const char* vCurrentPath = mLogBase->getLogPath();
        const char* vNewPath = vNewLogState->getLogPath();
        if(vCurrentPath && vNewPath && 
           XAM_POSIX::xam_strcmp(vNewPath, vCurrentPath) == 0)
        {
            mLogBase->update(vNewLogState);
        }
        else
        {
            // Cleanup and recreate this log.
            closeLog();
            applyOptions(vNewLogState); // Recursive. 
                                        // Logging is inactive next time through.
        }
    }
}

void
XAM_LogFactory::closeLog()
{
    // Deleting the active logger effectively closes any active log file
    if(mLogBase != (XAM_LogFormatter*)0)
    {
        delete mLogBase;
        mLogBase = (XAM_LogFormatter*)0;
    }
}

XAM_LogFormatter*
XAM_LogFactory::getInstance()
{
    XAM_LogFormatter *vRetVal = (XAM_LogFormatter*)0;

    if (mLogBase) 
    {
        vRetVal = mLogBase;                
    }
    else
    {
        if(isEnabled())
        {
            // Create a new logger of the configured format
            xam_int vFormat = getFormat();
            if(vFormat == TAB)
                mLogBase = createLogTab(this);

            vRetVal = mLogBase;
        }

    }
    return vRetVal;
}

XAM_LogFactory::XAM_LogFactory()
{
}

XAM_LogFactory::~XAM_LogFactory()
{
    if (mLogBase)
    {
        // Delete the active logger
        delete mLogBase;
        mLogBase = (XAM_LogFormatter*)0;
    }
}


