#ifndef __XAM_LOGFACTORY__
#define __XAM_LOGFACTORY__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_LogFormatter.h"
#include "XAM_LogTabFormatter.h"
#include "XAM_LogState.h"

/**
 * A factory class for XAM_LogFormatter instances
 *
 * An instance of this is 'statically' created in XAM_Logger.
 * So, we can ensure that the destructor is called when the
 * program exits.
 */
class 
XAM_LogFactory : public XAM_LogState // Composition
{
    /* Ensure that only XAM_Logger can instantiate/access the factory */
    friend class XAM_Logger;

    private:
        /* Only one logger should be instantiated */
        static XAM_LogFormatter* mLogBase;

        /* Creates and initializes the tab-delimited log
         * @return XAM_LogTabFormatter* - a pointer to the newly created log,
         *          or NULL on error
         */
        XAM_LogTabFormatter* 
        createLogTab(XAM_LogState* inOptions);

        /* Private Constructor
         *
         * Only friend classes can create/access the log factory 
         */
        XAM_LogFactory();


    public:
        /** 
         *  This routine will initialize the logging mechanism,
         *  if it has not been initialized yet.         
         *
         *  A pointer to the logger object of the configured type 
         *  (e.g. XML, TAB, etc.) is returned from this call.
         * 
         *  @return If logging is enabled, return a LogBasic
         *          pointer, which can be used to log messages.  If
         *          logging is disabled, return NULL.
         */
        XAM_LogFormatter* 
        getInstance();

        /**
         * Dynamically change the current logger to reflect the new state.
         *
         * @param inLogState The new log state to apply or NULL to apply 
         *                   the existing configuration
         */
        void 
        applyOptions(XAM_LogState* inLogState);

        /**
         * Deletes any active loggers, effectively force closing any open log 
         * files 
         */
        void
        closeLog();

        ~XAM_LogFactory();

};

#endif // __XAM_LOGFACTORY__


