////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_LogFormatter.h"
#include "XAM_Logger.h"
#include "XAM_Exception.h"

void
XAM_LogFormatter::log (const loglevel_t inLevel,
                       const component_t inComponent,
                       const char *inFormat,
                       ...)
{
    va_list ap;
    va_start(ap, inFormat);

    // In order for logging to occur, the log level must be  
    // configured for at least the requested log level and 
    // the component must exist in the configured mask.
    if ((inLevel <= mLogState.getLevel()) && 
        (inComponent & mLogState.getComponentMask()))
    {
        if (mLogFile)
        {
            xam_boolean vFilteredOut = false;

            // Check the message filter for restrictions
            const char* vMsgFilter = mLogState.getMsgFilter();
            if(XAM_POSIX::xam_strcmp(vMsgFilter, "") != 0)
            {
                // If the string is not found, do not log this event
                if(XAM_POSIX::xam_strstr(inFormat, vMsgFilter) == 0)
                    vFilteredOut = true;
            }

            // Check the component filter for restrictions
            const char* vCompFilter = mLogState.getCompFilter();
            if(XAM_POSIX::xam_strcmp(vCompFilter, "") != 0)
            {
                // If the string is not found, do not log this event  
                const char* vStr = getComponentMaskString(inComponent);
                if(XAM_POSIX::xam_strstr(vStr, vCompFilter) == 0)
                    vFilteredOut = true;
            }

            // If not filtered out, log the event!
            if(!vFilteredOut)
            {
                // Rollover event handling for bounded log files
                if(mLogState.getMaxSizeInKb() != XAM_LOG_SIZE_UNBOUNDED)
                {
                    handleRolloverEvent(inFormat, ap);    
                    va_end(ap);
                    va_start(ap, inFormat);
                }

                // Log this specific event with the derived formatter class
                logEvent(inLevel, inComponent, inFormat, ap);
            }
        }
    }

  va_end(ap) ;
}

const char*
XAM_LogFormatter::getComponentMaskString(const component_t inComponent) 
{
    const char *vComp = (char*)0;

    switch (inComponent) 
    {
        case component_none            : vComp = "none" ;        break ;
        case component_api_enter       : vComp = "api_enter" ;   break ;
        case component_api_exit        : vComp = "api_exit" ;    break ;
        case component_api_param       : vComp = "api_param" ;   break ;
        case component_api_status      : vComp = "api_status" ;  break ;      
        case component_logger          : vComp = "logger";       break ;
        case component_vim_enter       : vComp = "vim_enter";    break ;
        case component_vim_exit        : vComp = "vim_exit";     break ;
        case component_vim_param       : vComp = "vim_param";    break ;
        case component_vendor_vim      : vComp = "vendor_vim";   break ;
        case component_lib             : vComp = "lib";          break ;
        case component_app             : vComp = "application" ; break ;  
        case component_xam_debug       : vComp = "xam_debug";    break ;
        case component_all             : vComp = "---" ;         break ;     
        default                        : vComp = "???" ;         break ;
    }
    return vComp;
}

const char*
XAM_LogFormatter::getVerbosityString(const loglevel_t inVerbosity) 
{
    static const char *vVerbosityStrings[] =
    {
        // Should not exceed 5 characters for formatting purposes
        ("none"),
        ("fatal"),
        ("error"),
        ("warn"),
        ("info"),
        ("debug"),
        ("dbg+")
    };
    
    // Ensure that we don't overrun our vVerbosityString array
    loglevel_t vLevel = inVerbosity;
    if (vLevel > LOGLEVEL_DEBUG_PLUS)
        vLevel = LOGLEVEL_DEBUG_PLUS;
    if (vLevel < LOGLEVEL_NONE)
        vLevel = LOGLEVEL_NONE;

    const char *vRetVal = vVerbosityStrings[vLevel];

    return vRetVal;
}

void
XAM_LogFormatter::getTimestamp(struct timeval& inTimestamp)
{
    // Get the current time (local time)
    if(XAM_POSIX::xam_gettimeofday(&inTimestamp) == -1) 
    {
        inTimestamp.tv_sec  = 0;
        inTimestamp.tv_usec = 0;
    }
}

FILE*
XAM_LogFormatter::createLogFile(const xam_boolean inKeepExisting)
{
    const char* vFileName = mLogState.getLogPath();
    xam_boolean vAppend = mLogState.getAppend();

    // Re-create the log file
    if (mLogFile)
    {
        preCloseEvent();
        XAM_POSIX::xam_fclose(mLogFile);
        mLogFile = (FILE*)0;
    }

    if(vFileName)
    {
        xam_stat_t vStat;
        xam_int vStatResult;

        if (vAppend)
        {
            // Append to the existing file
            mLogFile = XAM_POSIX::xam_fopen(vFileName, "a");
        }
        else
        {
            if (inKeepExisting)
            {
                // Create a new file, and open it with 'append'

                // allocate 25 extra characters to append to the filename
                xam_int vFileNameLen = XAM_POSIX::xam_strlen(vFileName);
                xam_int vFileNameMax = vFileNameLen + 25;
                char vFileAppendText[25];

                char *vNewFile = (char*) XAM_POSIX::xam_malloc(sizeof(char) * 
                                                               vFileNameMax);

                if (vNewFile == (char*)0)
                {
                    XAMOutOfMemoryException e;
                    throw e;
                }
                else {
                    XAM_POSIX::xam_strcpy(vNewFile, vFileName);

                    xam_boolean vFileFound = false;
                    for (xam_int vTries = 0; 
                        ((vTries < 100) && (vFileFound == false)); 
                        vTries++)
                    {
                        struct timeval vTimestamp;
                        getTimestamp(vTimestamp);

                        // change to milliseconds
                        xam_int vTimeSec = vTimestamp.tv_sec; 
                        xam_int vTimeMsec = vTimestamp.tv_usec / 1000;

                        // increment 1 msec to increase the chance that the
                        // file does not exist
                        vTimeMsec += vTries; 

                        XAM_POSIX::xam_sprintf(vFileAppendText, 
                                               ".%d%.3d", vTimeSec, vTimeMsec);

                        vNewFile[vFileNameLen] = (char)0;
                        if (vTries)
                        {
                            XAM_POSIX::xam_strcat(&vNewFile[vFileNameLen], 
                                                  vFileAppendText);
                        }

                        vStatResult = XAM_POSIX::xam_stat(vNewFile, &vStat);
                        if (vStatResult == -1)
                        {
                            // This file doesn't exist - use it. (open it for
                            // append to cover a race condition in which 2 
                            // processes discover that this particular file
                            // doesn't exist at the same time, and both try 
                            // to use it)
                            mLogFile = XAM_POSIX::xam_fopen(vNewFile, "a");
                            vFileFound = true;
                        }
                    }
                }
            }
            else
            {
                // Overwrite the existing file
                mLogFile = XAM_POSIX::xam_fopen(vFileName, "w");
            }
        }

        // If we're opening in append mode, set mBytesWritten accordingly for 
        // rollover
        if(vAppend && !XAM_POSIX::xam_stat(vFileName, &vStat))
            mBytesWritten = (xam_int)vStat.st_size;
        else
            mBytesWritten = 0;

        // On success, generate a post open log event.
        if(mLogFile != (FILE*)0)
            postOpenEvent();
        else
        {
            XAMFileSystemException e;
            throw e;
        }
    }

    return(mLogFile);
}

void
XAM_LogFormatter::handleRolloverEvent(const char* inFormat, va_list ap)
{
    xam_int vBytesWritten = mBytesWritten + getPrependLength();

    // Rollover this (and all existing) log files, 
    // and start a new one as needed.
    if(rolloverThresholdReached(inFormat, ap, vBytesWritten))
    {
        // Re-create the log file
        if (mLogFile)
        {
            preCloseEvent();
            XAM_POSIX::xam_fclose(mLogFile);
            mLogFile = (FILE*)0;
        }

        rolloverLogFiles();
        mBytesWritten = 0;

        // Throws on error 
        // Note: if no ROLLOVERS are allowed, we can't append to a file 
        // that has exceeded the max size!
        mLogFile = createLogFile(mKeepExisting);
    }
}

const xam_boolean 
XAM_LogFormatter::rolloverThresholdReached(const char* inFormat, 
                                           va_list ap, 
                                           const xam_int inBytesWritten)
{
    xam_boolean vReached = false;
    xam_int vMaxKbSize = mLogState.getMaxSizeInKb();
    if(vMaxKbSize == 0)
        vReached = true;
    else
    {
        xam_int vFinalStringSize = 0;
        char* vTempBuf = (char*)0;

        // Don't rollover if there is no size limitation for the
        // existing file.
        if(vMaxKbSize != XAM_LOG_SIZE_UNBOUNDED) 
        {    
            if(inBytesWritten >= (vMaxKbSize*1024))
               vReached = true;
            else
            {
                // Determine how big the log event string will be
                vTempBuf = allocateBuffer(inFormat, ap, vFinalStringSize);  
                
                if((vFinalStringSize > 0) && 
                   ((inBytesWritten + (xam_int)vFinalStringSize) >= 
                    (vMaxKbSize*1024)))
                   vReached = true;            
            }
        }

        if(vTempBuf)
            XAM_POSIX::xam_free(vTempBuf);
    }

    return(vReached);
}

void 
XAM_LogFormatter::rolloverLogFiles()
{
    const char* vFileName = mLogState.getLogPath();
    xam_int vMaxRollovers = mLogState.getMaxRollovers(); 

    // This rollover is kind of slow, but we need to support a Linux-like 
    // log rollover in which 'file_2.txt' is the oldest of: 'file.txt', 
    // 'file_1.txt', 'file_2.txt'.

    if((vFileName != (char*)0) && (vMaxRollovers > 0))       
    {
        char* vInitialFileName = XAM_POSIX::xam_strdup(vFileName); 
        if(vInitialFileName == (char*)0)
        {
            XAMOutOfMemoryException e;
            throw e;
        }
        else 
        {
            // Allocate enough space for the filename, rollover string, 
            // and number.
            xam_int vFileStringSize 
                        = XAM_POSIX::xam_strlen(vFileName) +
                          XAM_POSIX::xam_strlen(XAM_LOG_ROLLOVER_STRING) +
                          40;   

            char* vOldFileName = (char*)XAM_POSIX::xam_malloc(vFileStringSize);
            char* vNewFileName = (char*)XAM_POSIX::xam_malloc(vFileStringSize);

            char* vDot = XAM_POSIX::xam_strrchr(vInitialFileName, '.');  
            char* vExtension = (char*)0;                
            if(vDot)
            {
                // Copy the extension
                vExtension = (char*)XAM_POSIX::xam_malloc(XAM_POSIX::xam_strlen(vDot) + 1);
                XAM_POSIX::xam_strcpy(vExtension, vDot);

                // Cut the filename off at the extension (if found)
                *vDot = '\0';
            }

            if((vOldFileName == (char*)0) || (vNewFileName == (char*)0))
            {
                XAMOutOfMemoryException e;
                throw e;
            }
            else
            {    
                // Since this is a rollover event, all existing log files 
                // should have their numbers incremented and the largest file
                // at the overflow threshold should be deleted.
                for(int i=(int)vMaxRollovers ; i >= 1 ; i--)
                {
                    // For this loop, vOldFileName should reflect the file 
                    // to be moved
                    XAM_POSIX::xam_sprintf(vOldFileName, "%s%s%i%s", 
                                           vInitialFileName, 
                                           XAM_LOG_ROLLOVER_STRING, 
                                           i, 
                                           (vExtension != (char*)0) ? vExtension : ""); 

                    // If the file exists, roll it to the next highest 
                    // (or possible delete it)

                    // Exists check
                    if(XAM_POSIX::xam_access(vOldFileName, XAM_F_OK) == 0) 
                    {
                        // Are we at the max overflow? Delete the max file for 
                        // replacement in the next loop.
                        if(i == vMaxRollovers)
                        {
                            XAM_POSIX::xam_remove(vOldFileName);
                        }
                        else // Rename the existing file (+1)
                        {
                            XAM_POSIX::xam_sprintf(vNewFileName, "%s%s%i%s", 
                                                   vInitialFileName, 
                                                   XAM_LOG_ROLLOVER_STRING, 
                                                   (i+1), 
                                                   (vExtension != (char*)0) ? vExtension : "");     

                            XAM_POSIX::xam_rename(vOldFileName, vNewFileName);                          
                        }
                    }
                }

                // Now roll the current log file to '.1'
                XAM_POSIX::xam_strcpy(vOldFileName, vFileName); 
                XAM_POSIX::xam_sprintf(vNewFileName, "%s%s1%s", 
                                       vInitialFileName, 
                                       XAM_LOG_ROLLOVER_STRING, 
                                       (vExtension != (char*)0) ? vExtension : "");                            
                XAM_POSIX::xam_rename(vOldFileName, vNewFileName);

            }

            // Free resources
            if(vExtension)
                XAM_POSIX::xam_free(vExtension);
            if(vOldFileName)
                XAM_POSIX::xam_free(vOldFileName);
            if(vNewFileName)
                XAM_POSIX::xam_free(vNewFileName);

        }

        if(vInitialFileName)
            XAM_POSIX::xam_free(vInitialFileName);  
    }
}

void 
XAM_LogFormatter::update(XAM_LogState* inLogState)
{
    mLogState.setLogState(inLogState);
}

void 
XAM_LogFormatter::bytesWritten(const xam_int inWritten) 
{ 
    // Only keep track of the bytes written if there is no 
    // max size threshold
    if(mLogState.getMaxSizeInKb() != XAM_LOG_SIZE_UNBOUNDED) 
        mBytesWritten += inWritten; 
}

char* 
XAM_LogFormatter::allocateBuffer(const char* inFormat, 
                                 va_list ap, 
                                 xam_int &outStrLen)
{
    xam_int vFinalStringSize = 0;
    xam_int vResultSize = 0;

    // Determine how big the log event string is
    xam_int vTempSize = 1024;
    char* vTempBuf = (char*) XAM_POSIX::xam_malloc((size_t)vTempSize);
    if ( vTempBuf == (char*)0 )
    { 
        XAMOutOfMemoryException e;
        throw e;
    }

    // Determine the final size of this log buffer
    while(!vFinalStringSize)
    {
        vResultSize = XAM_POSIX::xam_vsnprintf(vTempBuf, 
                                               (size_t)vTempSize, 
                                               inFormat, ap);

        // If we have enough room, set vFinalStringSize. 
        // Otherwise, increase vTempSize and reallocate 
        // enough room for next pass.
        if(vResultSize > 0)
        {
            if(vResultSize >= vTempSize)
                vTempSize = vResultSize + 1; // Include '\0'.
            else
                vFinalStringSize = vResultSize + 1; // Include '\0'.
        }
        else
            vFinalStringSize = vTempSize;

        // Final size not found, we need more buffer space for next pass
        if(vFinalStringSize == 0)
        {
            vTempBuf = (char*)XAM_POSIX::xam_realloc(vTempBuf, 
                                                     (size_t)vTempSize);                
            if(vTempBuf == (char*)0)
            { 
                XAMOutOfMemoryException e;
                throw e;
            }
        }
    }

    outStrLen = vFinalStringSize;
    return vTempBuf;
}

const int 
XAM_LogFormatter::logToFile(FILE* inFile, const char* inFormat, ...)
{
    int vRetVal = 0;

    va_list ap;
    va_start(ap, inFormat);

    // Perform the log event
#ifdef __MVS__
    vRetVal = XAM_POSIX::xam_vfprintf(inFile, inFormat, (char**)ap);
#else
    vRetVal = XAM_POSIX::xam_vfprintf(inFile, inFormat, ap);
#endif
    va_end(ap);

    if(vRetVal > 0)
    {
        bytesWritten(vRetVal + 1);
        XAM_POSIX::xam_fflush(inFile);
    }

    return(vRetVal);
}

const int 
XAM_LogFormatter::logListToFile(FILE* inFile, const char* inFormat, va_list ap)
{
    int vRetVal = 0;

    // Perform the log event
#ifdef __MVS__
    vRetVal = XAM_POSIX::xam_vfprintf(inFile, inFormat, (char**)ap);
#else
    vRetVal = XAM_POSIX::xam_vfprintf(inFile, inFormat, ap);
#endif

    if(vRetVal > 0)
    {
        bytesWritten(vRetVal + 1);
        XAM_POSIX::xam_fflush(inFile);
    }
    
    return(vRetVal);
}

XAM_LogFormatter::XAM_LogFormatter()
: mLogFile(0), 
  mKeepExisting(false), 
  mBytesWritten(0)
{
}

XAM_LogFormatter::~XAM_LogFormatter()
{
    if (mLogFile)
    {
        // Close the log file
        XAM_POSIX::xam_fclose(mLogFile);
        mLogFile = (FILE*)0;;
    }
}



