////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_LogState.h"
#include "XAM_LogTypes.h"
#include "XAM_Logger.h"
#include "XAM_Exception.h"

void
XAM_LogState::loadEnvOptions() 
{
    // Log path
    if(getLogPath() == (char*)0)
    {
        const char* vFileName = XAM_POSIX::xam_getenv(XAM_LOG_PATH_ENV_VAR);
        if ((vFileName != NULL) && (XAM_POSIX::xam_strlen(vFileName) != 0))
        {
            setPath(vFileName);
        }
    }

    // Log level
    char *vLevel = XAM_POSIX::xam_getenv(XAM_LOG_LEVEL_ENV_VAR);
    if ((vLevel != NULL) && (XAM_POSIX::xam_strlen(vLevel) != 0))
    {
        xam_int vLogLevel = XAM_POSIX::xam_atoi(vLevel);
        setLevel(vLogLevel);
    }

    // Debug Level (Component Filter)
    char* vVerbosity = XAM_POSIX::xam_getenv(XAM_LOG_VERBOSITY_ENV_VAR);
    if ((vVerbosity != NULL) && (XAM_POSIX::xam_strlen(vVerbosity) != 0))
    {
        xam_int vDebugLevel = XAM_POSIX::xam_atoi(vVerbosity);
        setVerbosityLevel(vDebugLevel);
    }

    // Format
    char *vFormat = XAM_POSIX::xam_getenv(XAM_LOG_FORMAT_ENV_VAR);
    if ((vFormat != NULL) && (XAM_POSIX::xam_strlen(vFormat) != 0))
    {
        xam_int vFormatType = XAM_POSIX::xam_atoi(vFormat);
        setFormat(vFormatType);
    }

    // Append or Overwrite
    char *vAppendEnvStr = XAM_POSIX::xam_getenv(XAM_LOG_APPEND_ENV_VAR);
    if (vAppendEnvStr != NULL)
    {
        if (XAM_POSIX::xam_strlen(vAppendEnvStr) != NULL)
        {
            xam_boolean vAppend = 
                (XAM_POSIX::xam_strcasecmp(vAppendEnvStr, "true") == 0);

            setAppend(vAppend);
        }
    }

    // Maximum log file size (kilobytes)
    char *vMaxSize = XAM_POSIX::xam_getenv(XAM_LOG_MAX_SIZE_ENV_VAR);
    if ((vMaxSize != NULL) && (XAM_POSIX::xam_strlen(vMaxSize) != 0))
    {
        xam_int vSizeUsed = XAM_POSIX::xam_atoi(vMaxSize);

        // Default values to 1 GB
        if (XAM_POSIX::xam_strcmp(vMaxSize, "0") == 0)
            vSizeUsed = 1048576; 

        setMaxSizeInKb(vSizeUsed);
    }

    // Maximum number of log rollover files
    char *vMaxRollovers = XAM_POSIX::xam_getenv(XAM_LOG_MAX_ROLLOVERS_ENV_VAR);
    if ((vMaxRollovers != NULL) && (XAM_POSIX::xam_strlen(vMaxRollovers) != 0))
        setMaxRollovers(XAM_POSIX::xam_atoi(vMaxRollovers));

    // Message filter
    char *vMsgFilter = XAM_POSIX::xam_getenv(XAM_LOG_MSG_FILTER_ENV_VAR);
    if ((vMsgFilter != NULL) && (XAM_POSIX::xam_strlen(vMsgFilter) != 0))
        setMsgFilter(vMsgFilter);

    // Component filter
    char *vCompFilter = XAM_POSIX::xam_getenv(XAM_LOG_COMP_FILTER_ENV_VAR);
    if ((vCompFilter != NULL) && (XAM_POSIX::xam_strlen(vCompFilter) != 0))
        setCompFilter(vCompFilter);
}

xam_int
XAM_LogState::getComponentList(xam_int inDebugLevel)
{
    xam_int vCompMask = component_api_status; // Always allow api status 
                                              // for warnings/errors

    // Determine which components will be logged for a given debug level
    int vDebugLevel = (int) inDebugLevel;
    switch(vDebugLevel)
    {     
    case 1:
        // Log XAM API entry and exit
        vCompMask |= (component_logger | component_api_enter | 
                      component_api_exit);
        break;
    case 2:
        // Log XAM API entry, exit, and params
        vCompMask |= (component_logger | component_api);
        break;
    case 3:
        // Log all XAM API and VIM_API entry/exit
        vCompMask |= (component_logger | component_api | component_vim_enter | 
                      component_vim_exit);
        break;
    case 4:
        // Log all XAM API, all VIM API, and library loading/unloading msgs
        vCompMask |= (component_logger | component_api | component_vim | 
                      component_lib);
        break;
    case 5:
        // Log all XAM API, all VIM API, lib load/unload, and debug messages
        vCompMask |= (component_logger | component_api | component_vim | 
                      component_lib | component_xam_debug);
        break;
    }

    return vCompMask;
}

void
XAM_LogState::loadDefaultOptions() 
{
    // Set default values for all fields
    setFormat(TAB);
    setMaxSizeInKb(1048576); // 1 GB;
    setMaxRollovers(1);
    setLevel(0);
    setVerbosityLevel(0);
    setAppend(false); // overwrite
    setMsgFilter("");
    setCompFilter("");

    loadEnvOptions();

    // Use the default log file path if one has not been specified
    if(getLogPath() == (char*)0)
        setPath((char*) DEFAULT_XAM_LOG_FILE);
}

void
XAM_LogState::loadDefaults() 
{
    // Use default options
    loadDefaultOptions();
}

void
XAM_LogState::loadFile(const char* inFilePath) 
{
    // The file path must be valid
    if(inFilePath == (char*)0)
    {      
        XAMInvalidParamException e;
        throw e;        
    }

    // Attempt to access the file at this location
    if(XAM_POSIX::xam_access(inFilePath, XAM_F_OK) != 0)
    {
        XAMFileSystemException e;
        throw e;  
    }

    // Cache here so that it's always safe to cleanup the log options
    char* vStateFilePath = XAM_POSIX::xam_strdup(inFilePath);
    if ( vStateFilePath == (char*)0 )
    { 
        XAMOutOfMemoryException e;
        throw e;
    }

    cleanup();

    // Load the current file into the properties object
    mProps->readFromFile(vStateFilePath);    

    // Clear out the log state and apply the new properties
    loadDefaultOptions();

    // Log path
    xam_string vFileName = "";
    xam_boolean vSuccess = mProps->getValue(XAM_LOG_PATH, &vFileName);
    if (vSuccess && (XAM_POSIX::xam_strlen(vFileName) != 0))
    {
        if(mLogFileName != (char*)0)
            XAM_POSIX::xam_free(mLogFileName);
        mLogFileName = (char*)XAM_POSIX::xam_strdup(vFileName);
        if ( mLogFileName == (char*)0 )
        { 
            XAMOutOfMemoryException e;
            throw e;
        }
    }

    // Log level
    xam_string vLevel = "";
    vSuccess = mProps->getValue(XAM_LOG_LEVEL, &vLevel);
    xam_int vLogLevel = 0; // default
    if (vSuccess && (XAM_POSIX::xam_strlen(vLevel) != 0))
    {
        vLogLevel = XAM_POSIX::xam_atoi(vLevel);
    }
    mLogLevel = vLogLevel;

    // Debug level
    xam_string vDebugLevel = "";
    vSuccess = mProps->getValue(XAM_LOG_VERBOSITY, &vDebugLevel);
    component_t vCompBitmask = component_all;
    if (vSuccess && (XAM_POSIX::xam_strlen(vDebugLevel) != 0))
    {
        xam_int vDbgLogLevel = XAM_POSIX::xam_atoi(vDebugLevel);

        mVerbosityLevel = vDbgLogLevel;
        vCompBitmask = (component_t) getComponentList(vDbgLogLevel);
    }
    mCompBitmask = vCompBitmask;

    // Format
    xam_string vFormat = "";
    vSuccess = mProps->getValue(XAM_LOG_FORMAT, &vFormat);
    LogFormat vLogFormat = TAB;
    if (vSuccess && (XAM_POSIX::xam_strlen(vFormat) != 0))
    {
        vLogFormat = (LogFormat) XAM_POSIX::xam_atoi(vFormat);
    }
    mLogFormat = vLogFormat;

    // Append or Overwrite
    xam_string vAppendStr = "";
    vSuccess = mProps->getValue(XAM_LOG_APPEND, &vAppendStr);
    xam_boolean vAppend = false;
    if (vSuccess && (XAM_POSIX::xam_strlen(vAppendStr) != 0))
    {
        if(XAM_POSIX::xam_strcmp(vAppendStr, "true") == 0)
            vAppend = true;
    }
    mLogAppend = vAppend;

    // Maximum log file size (kilobytes)
    xam_string vMaxSize = "";
    vSuccess = mProps->getValue(XAM_LOG_MAX_SIZE, &vMaxSize);
    xam_int vSizeUsed = XAM_LOG_SIZE_UNBOUNDED;
    if (vSuccess && (XAM_POSIX::xam_strlen(vMaxSize) != 0) &&
        (XAM_POSIX::xam_strcasecmp(vMaxSize, "XAM_LOG_SIZE_UNBOUNDED") != 0))
    {
        vSizeUsed = XAM_POSIX::xam_atoi(vMaxSize);

        // Default values to XAM_LOG_SIZE_UNBOUNDED
        if ((XAM_POSIX::xam_strcmp(vMaxSize, "0") != 0))
        {
            vSizeUsed = XAM_LOG_SIZE_UNBOUNDED;
        }
    }
    mMaxLogSizeInKb = vSizeUsed;

    // Maximum number of log rollover files
    xam_string vMaxRollovers = "";
    vSuccess = mProps->getValue(XAM_LOG_MAX_ROLLOVERS, &vMaxRollovers);
    xam_int vRollovers = 1;
    if (vSuccess && (XAM_POSIX::xam_strlen(vMaxRollovers) != 0))
    {
        vRollovers = XAM_POSIX::xam_atoi(vMaxRollovers);

        // Default values to unbounded log size
        if ((vRollovers == 0) && 
            (XAM_POSIX::xam_strcmp(vMaxRollovers, "0") != 0)) 
        {
            vSizeUsed = XAM_LOG_SIZE_UNBOUNDED;
        }
    }
    mMaxLogRollovers = vRollovers;

    // Message filter
    xam_string vMsgFilter = "";
    vSuccess = mProps->getValue(XAM_LOG_MSG_FILTER, &vMsgFilter);
    if (vSuccess && (XAM_POSIX::xam_strlen(vMsgFilter) != 0))
    {
        XAM_POSIX::xam_strcpy(mMsgFilter, vMsgFilter);
    }

    // Component filter
    xam_string vCompFilter = "";
    vSuccess = mProps->getValue(XAM_LOG_COMP_FILTER, &vCompFilter);
    if (vSuccess && (XAM_POSIX::xam_strlen(vCompFilter) != 0))
    {
        XAM_POSIX::xam_strcpy(mCompFilter, vCompFilter);
    }
}

void
XAM_LogState::writeToFile(const char* inFilePath) 
{
    // Apply settings to the XAM_Properties object
    mProps->set(XAM_LOG_PATH, mLogFileName);
    mProps->set(XAM_LOG_LEVEL, mLogLevel);     
    mProps->set(XAM_LOG_FORMAT, mLogFormat);
    mProps->set(XAM_LOG_VERBOSITY, mVerbosityLevel);  
    mProps->set(XAM_LOG_APPEND, mLogAppend);

    // Use a special indicator for log size unbounded
    if(mMaxLogSizeInKb == XAM_LOG_SIZE_UNBOUNDED)
        mProps->set(XAM_LOG_MAX_SIZE, "XAM_LOG_SIZE_UNBOUNDED");
    else
        mProps->set(XAM_LOG_MAX_SIZE, mMaxLogSizeInKb);

    mProps->set(XAM_LOG_MAX_ROLLOVERS, mMaxLogRollovers);    

    if(XAM_POSIX::xam_strcmp(mCompFilter, "") != 0)
        mProps->set(XAM_LOG_COMP_FILTER, mCompFilter);

    if(XAM_POSIX::xam_strcmp(mMsgFilter, "") != 0)
        mProps->set(XAM_LOG_MSG_FILTER, mMsgFilter);

    // Write out the XAM_Properties object to a file
    mProps->writeToFile(inFilePath);
}


void 
XAM_LogState::setPath(const char* inFilePath) 
{
    if(mLogFileName != (char*)0)
        XAM_POSIX::xam_free((char*)mLogFileName);

    if(inFilePath != (char*)0)
        mLogFileName = (char*)XAM_POSIX::xam_strdup(inFilePath);
    else    
        mLogFileName = (char*)0;    
}

const char* 
XAM_LogState::getLogPath() 
{
    return mLogFileName;
}

void 
XAM_LogState::setAppend(const xam_boolean inAppend) 
{
    mLogAppend = inAppend;
}

const xam_boolean
XAM_LogState::getAppend() 
{
    return mLogAppend;
}

void 
XAM_LogState::setFormat(const xam_int inFormat) 
{
    mLogFormat = inFormat;
}

const xam_int 
XAM_LogState::getFormat() 
{
    return mLogFormat;
}

void 
XAM_LogState::setLevel(const xam_int inLevel) 
{
    mLogLevel = inLevel;
}

const xam_int 
XAM_LogState::getLevel() 
{
    return mLogLevel;
}

void 
XAM_LogState::setVerbosityLevel(const xam_int inLevel) 
{
    mVerbosityLevel = inLevel;

    // Apply changes to the component mask
    mCompBitmask = getComponentList(mVerbosityLevel);
}

const xam_int 
XAM_LogState::getVerbosityLevel() 
{
    return mVerbosityLevel;
}

const xam_int 
XAM_LogState::getComponentMask() 
{
    return mCompBitmask;
}

void 
XAM_LogState::setMaxSizeInKb(const xam_int inMaxKbSize) 
{
    if((inMaxKbSize > 0) || (inMaxKbSize == XAM_LOG_SIZE_UNBOUNDED))
        mMaxLogSizeInKb = inMaxKbSize;
    else
    {
        // The max size in KB cannot be zero or negative
        XAMInvalidParamException e;
        e.setMessage("XAM_LogState::setMaxSizeInKb - Requested max size is invalid!");
        throw e;
    }
}

const xam_int 
XAM_LogState::getMaxSizeInKb() 
{
    return mMaxLogSizeInKb;
}

void 
XAM_LogState::setMaxRollovers(const xam_int inMaxRollovers) 
{
    mMaxLogRollovers = inMaxRollovers;
}

const xam_int 
XAM_LogState::getMaxRollovers() 
{
    return mMaxLogRollovers;
}

void 
XAM_LogState::setMsgFilter(const char* inMsg)
{
    if(XAM_POSIX::xam_strlen(inMsg) < XAM_MAX_STRING)
        XAM_POSIX::xam_strcpy(mMsgFilter, inMsg);
}

const char* 
XAM_LogState::getMsgFilter()
{
    return mMsgFilter;
}

void 
XAM_LogState::setCompFilter(const char* inMsg)
{
    if(XAM_POSIX::xam_strlen(inMsg) < XAM_MAX_STRING)
        XAM_POSIX::xam_strcpy(mCompFilter, inMsg);
}

const char* 
XAM_LogState::getCompFilter()
{
    return mCompFilter;
}

void
XAM_LogState::setLogState(XAM_LogState* inLogState)
{
    // Deallocate all existing string options
    cleanup();

    // Apply changes to the log path
    const char* vLogPath = inLogState->getLogPath();
    if((vLogPath != (char*)0) && (XAM_POSIX::xam_strcmp(vLogPath, "") != 0))
        mLogFileName = XAM_POSIX::xam_strdup(inLogState->getLogPath());
    else
        mLogFileName = (char*)0;

    // Apply changes to all other fields
    mLogAppend = inLogState->getAppend();
    mCompBitmask = inLogState->getComponentMask();
    mVerbosityLevel = inLogState->getVerbosityLevel();
    mLogFormat = inLogState->getFormat();
    mLogLevel = inLogState->getLevel();
    mMaxLogRollovers = inLogState->getMaxRollovers();
    mMaxLogSizeInKb = inLogState->getMaxSizeInKb();

    XAM_POSIX::xam_strcpy(mMsgFilter, inLogState->getMsgFilter());
    XAM_POSIX::xam_strcpy(mCompFilter, inLogState->getCompFilter());
}

void
XAM_LogState::cleanup()
{
    if(mLogFileName != (char*)0) {
        XAM_POSIX::xam_free(mLogFileName);
        mLogFileName = (char*)0;
    } 
}

xam_boolean
XAM_LogState::isEnabled()
{
    xam_boolean vRetVal = false;
    if(mLogLevel > 0)
    {
        vRetVal = true;
    }
    return vRetVal;
}

XAM_LogState::XAM_LogState() 
: mLogFileName(0)
{
    mProps = new XAM_Properties("XAM_LogState");
    this->loadDefaults();
}

XAM_LogState::XAM_LogState(char* inFilePath) 
: mLogFileName(0)
{
    mProps = new XAM_Properties("XAM_LogState");
    this->loadFile(inFilePath);
}

XAM_LogState::~XAM_LogState() 
{
    delete mProps;
    cleanup();
}



