////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_LogTabFormatter.h"
#include "XAM_Logger.h"
#include "XAM_POSIX.h"
#include "XAM_Critical.h"
#include "XAM_Utils.h"

xam_boolean  
XAM_LogTabFormatter::formatTimestamp (struct timeval& inTimestamp, 
                                      char* const outBuf, 
                                      int outBufLen)
{    
    xam_boolean vSuccess = true;

    // Timestamp format
    static const char vFormat[] = "%Y-%m-%d %H:%M:%S";
    time_t vSeconds = (time_t) inTimestamp.tv_sec;
    char vMilliBuffer[64];

    // Validate params
    if (outBuf == 0 || outBufLen <= 0 || 
        inTimestamp.tv_sec < 0 || inTimestamp.tv_usec < 0) 
    {
        vSuccess = false;
    }
    else
    {
        // format the time (secs) part
        outBuf[0] = (char) 1; // make sure it doesn't start with zero

        struct tm* vBrokenTime = XAM_POSIX::xam_gmtime(&vSeconds);

        // on Z/OS, expects ASCII input, and gives ASCII output
        xam_int vRet = XAM_POSIX::xam_strftime(outBuf, 
                                               outBufLen - 5, 
                                               vFormat, 
                                               vBrokenTime); 
        if(vRet == 0 && outBuf[0] != '\0') 
            return NULL; // errno has been set
      
        // add the milliseconds
        XAM_POSIX::xam_sprintf (vMilliBuffer, ".%03ld", 
                                (long) (inTimestamp.tv_usec / 1000L)) ;

        #ifdef __MVS__
            XAM_Utils::convert_ASCII_to_EBCDIC(outBuf, 
                                               outBuf, 
                                               XAM_POSIX::xam_strlen(outBuf));
        #endif

        XAM_POSIX::xam_strcat (outBuf, vMilliBuffer) ; 
    }

    return vSuccess;
}

void
XAM_LogTabFormatter::postOpenEvent()
{
    // Write headers
    if(mLogFile != (FILE*)0)
    {
        logToFile(mLogFile, "Time         \t");
        logToFile(mLogFile, "FormattedTime        \t");
        logToFile(mLogFile, "LogLevel\t");
        logToFile(mLogFile, "PID.Thread   \t") ;
        logToFile(mLogFile, "Component \t");
        logToFile(mLogFile, "Message");
        logToFile(mLogFile, "\n"); 
    }

    // always write the version of the SDK to the head of the log file
    log(LOGLEVEL_NONE, component_all, "XAM SDK Logger Version @version.full@");
}


/** Logs the information to the file managed by the base formatter class */
void
XAM_LogTabFormatter::logEvent (const loglevel_t inLevel,
                               component_t inComponent,
                               const char *inFormat,
                               va_list ap)
{
    // Get the timestamp
    struct timeval vTimeStamp;
    getTimestamp(vTimeStamp);

    // Grab the process ID
    xam_int vProcID = XAM_POSIX::xam_getpid();

    // Grab the thread ID
    xam_int vThreadID = XAM_POSIX::xam_thread_self();

    // Format a timestamp
    char vTimeString[128];
    xam_boolean vTimestampOK = formatTimestamp(vTimeStamp, vTimeString, 128);
    if(!vTimestampOK)
        XAM_POSIX::xam_strcpy(vTimeString, "?????????????????????????"); 

    // Time in milliseconds, time in formatted string
    xam_string vTimeBuf;
    XAM_POSIX::xam_sprintf(vTimeBuf, 
                           "%d%.3d\t%s\t", 
                            vTimeStamp.tv_sec, 
                            (vTimeStamp.tv_usec / 1000),
                            vTimeString);

    // Log level, <proc id>.<thread id>, component
    const char* vComp = getComponentMaskString(inComponent);
    const char* vVerbosity = getVerbosityString(inLevel);

    // Populate the buffer
    xam_string vDataBuf;
    XAM_POSIX::xam_sprintf(vDataBuf, "[%s]\t\t%ld.%ld\t[%s]   \t",
                           vVerbosity,
                           (long)vProcID,
                           (long)vThreadID,
                           vComp);

    // Build the time and data buffers
    xam_int vLogSz = XAM_POSIX::xam_strlen(vTimeBuf) + 
                     XAM_POSIX::xam_strlen(vDataBuf) + 
                     (XAM_POSIX::xam_strlen(inFormat) * 4);

    char* vLogMsg = (char*)XAM_POSIX::xam_malloc(1024*10);
    if(vLogMsg == (char*)0)
    {
        XAMOutOfMemoryException e;
        throw e;
    }

    XAM_POSIX::xam_strcpy(vLogMsg, vTimeBuf);
    XAM_POSIX::xam_strcat(vLogMsg, vDataBuf);

    // Pad the message output for readability
    switch(inComponent)
    {
     case component_api_param:
     case component_vim_param:
         XAM_POSIX::xam_strcat(vLogMsg, "  ");
         break;
     case component_api_status:
         XAM_POSIX::xam_strcat(vLogMsg, " ");
         break;
    }
    
    // Clearly identify exceptions (top)
    if(inComponent == component_api_status)
    {
        if(inLevel == LOGLEVEL_ERROR)
        {
            logToFile(mLogFile, "*** ERROR ************************************");
            logToFile(mLogFile, "**********************************************");
            logToFile(mLogFile, "****************************************\n"); 
        }
        else if(inLevel == LOGLEVEL_FATAL)
        {
            logToFile(mLogFile, "*** FATAL ************************************");
            logToFile(mLogFile, "**********************************************");
            logToFile(mLogFile, "****************************************\n"); 
        }
    }

    // Add full string with message
    int vSz = (int) XAM_POSIX::xam_strlen(vLogMsg);
    XAM_POSIX::xam_vsprintf(vLogMsg + vSz, inFormat, ap);
    XAM_POSIX::xam_strcat(vLogMsg, "\n");

    logToFile(mLogFile, _logString(vLogMsg));

    // Clearly identify exceptions (bottom)
    if((inComponent == component_api_status) && 
       ((inLevel == LOGLEVEL_ERROR) || (inLevel == LOGLEVEL_FATAL)))
    {
        logToFile(mLogFile, "**********************************************");
        logToFile(mLogFile, "**********************************************");
        logToFile(mLogFile, "****************************************\n");
    }

    XAM_POSIX::xam_free(vLogMsg);
}

XAM_LogTabFormatter::XAM_LogTabFormatter(XAM_LogState* inLogState)
{              
    try
    {
        mLogState.setLogState(inLogState);
       
        mBytesWritten = 0;
   
        // Throws on error
        mLogFile = createLogFile(mKeepExisting);

    }
    catch (...)
    {
        if (mLogFile)
        {
            XAM_POSIX::xam_fclose(mLogFile);
            mLogFile = (FILE*)0;
        }

        throw;
    }
}

XAM_LogTabFormatter::~XAM_LogTabFormatter()
{
    preCloseEvent();
}



