#ifndef __XAM_LOGTYPES__
#define __XAM_LOGTYPES__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_strings.h"
#include "xam_types.h"

// Environment variables 
// These are checked on XAM_LogState construction and will affect 
// the default setting of the corresponding fields in all 
// XAM_LogState objects.
static const char* const XAM_LOG_PATH_ENV_VAR               = "XAM_LOG_PATH";
static const char* const XAM_LOG_LEVEL_ENV_VAR              = "XAM_LOG_LEVEL";
static const char* const XAM_LOG_VERBOSITY_ENV_VAR          = "XAM_LOG_VERBOSITY";
static const char* const XAM_LOG_FORMAT_ENV_VAR             = "XAM_LOG_FORMAT";
static const char* const XAM_LOG_APPEND_ENV_VAR             = "XAM_LOG_APPEND";
static const char* const XAM_LOG_MAX_SIZE_ENV_VAR           = "XAM_LOG_MAX_SIZE";
static const char* const XAM_LOG_MAX_ROLLOVERS_ENV_VAR      = "XAM_LOG_MAX_ROLLOVERS";
static const char* const XAM_LOG_MSG_FILTER_ENV_VAR         = "XAM_LOG_MSG_FILTER";
static const char* const XAM_LOG_COMP_FILTER_ENV_VAR        = "XAM_LOG_COMP_FILTER";

// Max log size in KB options
static const xam_int XAM_LOG_SIZE_UNBOUNDED       = -1;

// Default file paths (working directory)
static const char* const DEFAULT_XAM_LOG_FILE     = "xam.log";

#ifdef __cplusplus
extern "C" {
#endif

typedef enum  {
    LOGLEVEL_NONE = 0,   // lowest level
    LOGLEVEL_FATAL,      // Fatal Error Messages
    LOGLEVEL_ERROR,      // Error messages
    LOGLEVEL_WARN,       // Warning messages
    LOGLEVEL_INFO,       // Informational messages
    LOGLEVEL_DEBUG,      // Debug level messages (support)
    LOGLEVEL_DEBUG_PLUS  // Debug level message  (dev)
} loglevel_t;

// The component of the XAM SDK from which a message is logged.
typedef enum  {
    component_none          = 0,
    component_api_enter     = 0x00000001,
    component_api_exit      = 0x00000002,
    component_api_param     = 0x00000004,
    component_api_status    = 0x00000008,
    component_api           = component_api_enter | component_api_exit | 
                              component_api_param | component_api_status,
    component_logger        = 0x00000010,
    component_vim_enter     = 0x00000020,
    component_vim_exit      = 0x00000040,
    component_vim_param     = 0x00000100,
    component_vendor_vim    = 0x00000200,
    component_vim           = component_vim_enter | component_vim_exit | 
                              component_vim_param | component_vendor_vim,
    component_lib           = 0x00000400,
    component_app           = 0x00000800,
    component_xam_debug     = 0x00001000,
    component_all           = 0xFFFFFFFF
} component_t ;

// Log format options
typedef enum {
    TAB = 1
} LogFormat;

// Append mode options
typedef enum {
    OVERWRITE = 0,
    APPEND
} LogFileCreation;

// Active log state values
typedef enum {
  OFF = 0,      // logging is turned off            (FALSE)
  ON,           // logging is turned on             (TRUE)
  UNSET         // logging is uninitialized/unknown (TRUE)          
} LoggingState;


#ifdef __cplusplus
}
#endif
  
#endif // __XAM_LOGTYPES__


