////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_POSIX.h"
#include "XAM_Exception.h"
#include "XAM_Critical.h"
#include "XAM_Utils.h"

static XAM_Mutex sMutex; // Protects all non-thread safe POSIX calls

void
XAM_POSIX::xam_sleep(xam_int inSeconds)
{
#ifdef _WIN32
	Sleep((DWORD)inSeconds*1000);
#else
	//sleep(inSeconds);
#endif
}
void*
XAM_POSIX::xam_malloc(xam_int inSize)
{
    return malloc((size_t) inSize);
}

void 
XAM_POSIX::xam_free(void* inPtr)
{
    free(inPtr);
}

void* 
XAM_POSIX::xam_realloc(void* inPtr, xam_int inSize)
{
    return realloc(inPtr, (size_t) inSize);
}

void* 
XAM_POSIX::xam_memcpy(void* inDest, const void* inSrc, xam_int inLen)
{
    return memcpy(inDest, inSrc, (size_t) inLen);
}

void* 
XAM_POSIX::xam_memset(void *inPtr, int c, xam_int inSize)
{
    return memset(inPtr, c, (size_t) inSize);
}

char* 
XAM_POSIX::xam_adjust_library_name(const char* inLibName)
{
    char* vAdjustedLibName = (char*)0;

    // Determine the length of all components
    xam_int vParamLen  = xam_strlen(inLibName);
    xam_int vPrefixLen = xam_strlen(OS_LIB_PREFIX);
    xam_int vExtLen    = xam_strlen(OS_LIB_EXT);
    xam_int vArchLen   = xam_strlen(OS_LIB_ARCH); 

    // Determine if a prefix, suffix, and/or architecture is required in the lib name
    xam_boolean vUsePrefix = (vPrefixLen > 0) ? (xam_strncmp(inLibName, OS_LIB_PREFIX, vPrefixLen) != 0) : false;
    xam_boolean vUseExtension = (vExtLen > 0) ? (xam_strstr(inLibName, OS_LIB_EXT) == (char*)0) : false; 
    xam_boolean vUseArch = (vArchLen > 0) ? (xam_strstr(inLibName, OS_LIB_ARCH) == (char*)0) : false; 

    // Adjust the library name if any 'required' components are missing
    if(vUsePrefix || vUseExtension || vUseArch)
    {
        char* vLocalLibName = xam_strdup(inLibName);
        if(vLocalLibName == (char*)0)
        {
            XAMOutOfMemoryException e;
            throw e;
        }

        // Truncate the library name at the current extension (if any)
        char* vExtension = xam_strstr(vLocalLibName, ".");
        if(vExtension)
        {
            *vExtension = '\0';
            vUseExtension = true;
            vParamLen  = xam_strlen(vLocalLibName);
        }

        xam_int vTotLen = (vParamLen + 
                          (vUsePrefix ? vPrefixLen : 0) + 
                          (vUseExtension ? vExtLen : 0) + 
                          (vUseArch ? vArchLen : 0));

        // Allocate a new adjusted name buffer and populate it's contents
        vAdjustedLibName = (char*)xam_malloc(vTotLen + 1);
        if (vAdjustedLibName)
        {
            xam_sprintf(vAdjustedLibName, 
                        "%s%s%s%s", 
                        vUsePrefix ? OS_LIB_PREFIX : "",
                        vLocalLibName,
                        vUseArch ? OS_LIB_ARCH : "",
                        vUseExtension ? OS_LIB_EXT : "");
        }

        xam_free(vLocalLibName);
    }

    return vAdjustedLibName;
}

void* 
XAM_POSIX::xam_dlopen(const char* inLibPath, XAM_DLOPEN_MODE inMode)
{
    void* vLib = (void*)0;

#ifdef _WIN32
    if(inLibPath == (char*)0)
    {
        vLib = GetModuleHandle(NULL);
    }
    else
    {
        vLib = LoadLibrary(inLibPath); 
    }
#else
    vLib = dlopen(inLibPath, (int)inMode);
#endif

    // will be NULL on failure
    return vLib;
}

void* 
XAM_POSIX::xam_dlsym(void* inHandleToLib, const char* inNameOfFunction)
{
    void* vFuncHandle = (void*)0;
#ifdef _WIN32
    vFuncHandle = (void*)GetProcAddress((HINSTANCE)inHandleToLib, inNameOfFunction);
#elif defined __MVS__
    vFuncHandle = dllqueryfn((dllhandle*)inHandleToLib, inNameOfFunction);
#else
    vFuncHandle = dlsym(inHandleToLib, inNameOfFunction);
#endif
    return vFuncHandle;
}

int 
XAM_POSIX::xam_dlclose(void* inHandleToLib)
{
    int vRetVal = -1;
#ifdef _WIN32
    // this method returns NON ZERO if succeeds, zero on error, which is opposite of POSIX.    
    vRetVal = FreeLibrary((HINSTANCE)inHandleToLib) ? 0 : -1; 
#elif defined __MVS__
    vRetVal = dllfree((dllhandle*)inHandleToLib);
#else
    vRetVal = dlclose(inHandleToLib); // returns ZERO if succeeds and non-zero on error
#endif
    return vRetVal; // returns ZERO if succeeds, per POSIX call
}

const char* 
XAM_POSIX::xam_dlerror()
{
#ifdef _WIN32
    DWORD vLastErr;
    DWORD vFormatMsgErr;
    char *vLpMsgBuf;
    char *vErrMsg = (char*)0;
    xam_int vLen;

    // Get the value of the last error and format the message
    vLastErr = GetLastError();
    if (!FormatMessage(
            FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM |
            FORMAT_MESSAGE_IGNORE_INSERTS,
            NULL,
            vLastErr,
            // Default language
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),(LPTSTR) &vLpMsgBuf, 0, NULL ))
    {
        // Handle the error.
        vFormatMsgErr = GetLastError();
        vErrMsg = (char*)XAM_POSIX::xam_malloc(256);
        if(vErrMsg == (char*)0)
        {
            XAMOutOfMemoryException e;
            throw e;
        }
        XAM_POSIX::xam_sprintf(vErrMsg, "FormatMessage() failed with errno: %ld", vFormatMsgErr);
        return vErrMsg;
    }
    vLen = XAM_POSIX::xam_strlen(vLpMsgBuf);
    vErrMsg = (char*)XAM_POSIX::xam_malloc(vLen + 1);
    if(vErrMsg == (char*)0)
    {
        XAMOutOfMemoryException e;
        throw e;
    }
    XAM_POSIX::xam_strncpy(vErrMsg, vLpMsgBuf, vLen);

    // Free the buffer and return our own, as recommended by MS
    LocalFree(vLpMsgBuf);
    return vErrMsg;
#else
    return dlerror();
#endif
}

int 
XAM_POSIX::xam_thread_create(XAM_ThreadInfo &outInfo, XAM_THREAD_METHOD inMethod)
{
    int vRetVal = 0;
#ifdef _WIN32
    outInfo.thread_handle = CreateThread(NULL, 
                                         0, 
                                         (LPTHREAD_START_ROUTINE)inMethod, 
                                         (void*)&outInfo, 
                                         0, 
                                         &outInfo.thread_id);
    if (!outInfo.thread_handle)
        vRetVal = -1;
#elif defined SUN
    vRetVal = thr_create(0, 0, inMethod, (void*) &outInfo, 0, (thread_t*) &outInfo.thread_id);
#else // POSIX
    vRetVal = pthread_create((pthread_t*) &outInfo.thread_id, 
                             (pthread_attr_t *)0, 
                             inMethod, 
                             (void*)&outInfo) ? errno : ENOERR;
#endif
    return vRetVal;
}

int 
XAM_POSIX::xam_thread_join(XAM_ThreadInfo &ioInfo)
{
    int vRetVal = 0;
#ifdef _WIN32
    vRetVal = WaitForSingleObject(&ioInfo.thread_handle, INFINITE); 
#elif defined SUN
    vRetVal = thr_join((thread_t) ioInfo.thread_id, 0, 0);
#else // POSIX
    vRetVal = pthread_join ((pthread_t) ioInfo.thread_id, 0);
#endif 
    return vRetVal;
}

bool 
XAM_POSIX::xam_thread_kill(XAM_ThreadInfo &ioInfo)
{
    bool vSuccess = false;
#ifdef _WIN32
    vSuccess = (TerminateThread(ioInfo.thread_handle, 0)) ? true : false;
#elif defined SUN
    vSuccess = thr_kill((thread_t) ioInfo.thread_id, 0);
#else // POSIX
    vSuccess = (pthread_kill ((pthread_t) ioInfo.thread_id, 0)) ? false : true;
#endif
    return vSuccess;
}

void 
XAM_POSIX::xam_thread_exit()
{
#ifdef _WIN32
    ExitThread(0);
#elif defined SUN
    thr_exit(0);
#else // POSIX
    pthread_exit (NULL);
#endif
}

int 
XAM_POSIX::xam_mutex_init(xam_mutex_t* inMutex)
{
#ifdef _WIN32
    InitializeCriticalSection (inMutex);
    return 0;
#else
    return pthread_mutex_init (inMutex, 
                              (pthread_mutexattr_t*)0) ? errno : ENOERR; 
                              // default attribs
#endif
}

int 
XAM_POSIX::xam_mutex_lock(xam_mutex_t* inMutex)
{
#ifdef _WIN32
    EnterCriticalSection(inMutex);  
    return 0;
#else
    return pthread_mutex_lock(inMutex) ? errno : ENOERR;
#endif
}

int 
XAM_POSIX::xam_mutex_unlock(xam_mutex_t* inMutex)
{
#ifdef _WIN32
    LeaveCriticalSection(inMutex); 
    return 0;
#else
    return pthread_mutex_unlock(inMutex) ? errno : ENOERR;
#endif
}

int 
XAM_POSIX::xam_mutex_destroy(xam_mutex_t* inMutex)
{
#ifdef _WIN32
    DeleteCriticalSection(inMutex);  
    return 0;
#else
    return pthread_mutex_destroy (inMutex) ? errno : ENOERR;
#endif
}

xam_thread_t 
XAM_POSIX::xam_thread_self()
{
#ifdef _WIN32
    return (xam_thread_t) GetCurrentThreadId ();
#else
    #ifdef __MVS__
	    union tl{
	      xam_thread_t  p;
	      int           i1;
	      int           i2;
	    };
	    tl k;
        k.p = pthread_self();
	    return ((xam_thread_t) k.i2);
    #else
        return pthread_self();
    #endif
#endif 
}

xam_pid_t 
XAM_POSIX::xam_getpid()
{
#ifdef _WIN32
    return GetCurrentProcessId ();
#else
    return getpid();
#endif
}

char* 
XAM_POSIX::xam_strstr(const char* inString, const char* inSearchString)
{
    char* vString = (char*)0;

    vString = (char*)strstr(inString, inSearchString);

    return vString;
}

char* 
XAM_POSIX::xam_strcasestr(const char* inString, const char* inSearchString)
{
    // Emulated to support all platforms

    char* vStrFound = (char*)0;

    xam_int vSrcStrLen = xam_strlen (inSearchString);
    xam_int vStrLen = xam_strlen (inString);

    if ((vSrcStrLen > vStrLen) || (vStrLen <= 0))
        vStrFound = (char*)0;
    else if (vSrcStrLen <= 0) 
        vStrFound = (char*)inString;
    else
    {
        for (xam_int i = 0; i <= (vStrLen - vSrcStrLen); i++) 
        {
            if (xam_strncasecmp (inString + i, inSearchString, vSrcStrLen) == 0) 
            {
                vStrFound = (char*)(inString + i);
                break;
            }
        }
    }

    return vStrFound;
}

char* 
XAM_POSIX::xam_strcpy(char *inDest, const char *inSrc)
{
    char* vString = (char*)0;

    vString = strcpy(inDest, inSrc);

    return vString;
}

char* 
XAM_POSIX::xam_strncpy(char *inDest, const char *inSrc, xam_int inLen)
{
    char* vString = (char*)0;

    vString = strncpy(inDest, inSrc, (size_t) inLen);

    return vString;
}

int
XAM_POSIX::xam_sprintf(char* inDest, const char *inFormat, ...)
{
  int vResult;
  va_list ap;

  va_start (ap, inFormat);
  vResult = xam_vsprintf (inDest, inFormat, ap);
  va_end (ap);

  return vResult;
}

int 
XAM_POSIX::xam_snprintf(char *inDest, xam_int inSize, const char* inFormat, ...)
{
  int vResult;
  va_list ap;

  va_start (ap, inFormat);
  vResult = xam_vsnprintf (inDest, inSize, inFormat, ap);
  va_end (ap);

  return vResult;
}

int 
XAM_POSIX::xam_vsprintf(char *inDest, const char* inFormat, va_list ap)
{
    return vsprintf(inDest, inFormat, ap);
}

int 
XAM_POSIX::xam_vsnprintf(char *inDest, 
                         xam_int inSize, 
                         const char* inFormat, 
                         va_list ap)
{
    return vsnprintf(inDest, (size_t) inSize, inFormat, ap);
}

xam_int 
XAM_POSIX::xam_strlen(const char *inString)
{
    return strlen(inString);
}

char*
XAM_POSIX::xam_strdup(const char *inString)
{
    char* vString = (char*)0;

    if(inString)
    {
        xam_int vLen = xam_strlen(inString);
        vString = (char*)xam_malloc(vLen + 1);
        if(vString != (char*)0)
        {
            xam_strncpy(vString, inString, vLen);
            vString[vLen] = '\0';
        }
        else
        {
            XAMOutOfMemoryException e;
            throw e;
        }
    }

    return vString;
}

int 
XAM_POSIX::xam_strcmp(const char* inStr1, const char* inStr2)
{
    return strcmp(inStr1, inStr2);
}

int 
XAM_POSIX::xam_strncmp(const char* inStr1, const char* inStr2, xam_int inLen)
{
   return strncmp(inStr1, inStr2, (size_t) inLen);
}

int 
XAM_POSIX::xam_strcasecmp(const char* inStr1, const char* inStr2)
{
#ifdef __MVS__
    int n_Rc = 0;
    for ( int ix=0;; ix++ )
    {
            // not equal, check
        if (*(s1+ix) != *(s2+ix))
        {
            char c1 = *(s1+ix);
            char c2 = *(s2+ix);
            if ( c1 >= 65 && c1 <= 90 ) // compare UNICODE values
            {
                if ( c2 != (c1+32) )
                {
                    n_Rc = (c1>c2) ? 1 : -1;
                    break;
                }
            }
            else if ( c1 >= 97 && c1 <= 122 ) // compare UNICODE values
            {
                if ( c2 != (c1-32) )
                {
                    n_Rc = (c1>c2) ? 1 : -1;
                    break;
                }
            }
            else // not uppercase or lowercase 
            {
                n_Rc = (c1>c2) ? 1 : -1;
                break;
            }
        }
        else
            if (*(s1+ix) == '\0')
                break;
    }
    return(n_Rc);
#elif _WIN32
   return _strcmpi(inStr1, inStr2);
#else
   return strcasecmp(inStr1, inStr2);
#endif // __MVS__
}

int 
XAM_POSIX::xam_strncasecmp(const char* inStr1, 
                           const char* inStr2, 
                           xam_int inLen)
{
#if _WIN32
   return _strnicmp(inStr1, inStr2, (size_t) inLen);
#else
   return strncasecmp(inStr1, inStr2, inLen);
#endif // __MVS__
}

char*
XAM_POSIX::xam_strcat(char* inStr, char* inAppendStr)
{
    return strcat(inStr, inAppendStr);
}

char* 
XAM_POSIX::xam_strchr(char* inStr, int c)
{
    return strchr(inStr, c);
}

char* 
XAM_POSIX::xam_strrchr(char* inString, int c)
{
    return strrchr(inString, c);
}

char* 
XAM_POSIX::xam_strtok_r(char* inStr, 
                        const char* inSeparator, 
                        char** inLast)
{
#ifdef _WIN32
    if (inStr == 0)
        inStr = *inLast;
    else
        *inLast = inStr;

    if (*inStr == 0)   // We have reached the end
        return 0;

    xam_int l_org = xam_strlen (inStr);
    inStr = strtok (inStr, inSeparator);
    if (inStr == 0)
        return 0;

    xam_int l_sub = xam_strlen (inStr);
    if (inStr + l_sub < *inLast + l_org)
        *inLast = inStr + l_sub + 1;
    else
        *inLast = inStr + l_sub;

    return inStr ;
#else
    return strtok_r(inStr, inSeparator, inLast);
#endif
}

FILE* 
XAM_POSIX::xam_fopen(const char* inPath, const char* inMode)
{
    return fopen(inPath, inMode);
}

xam_int 
XAM_POSIX::xam_fread(void* inPtr, xam_int inSize, xam_int inNum, FILE* inFile)
{
    return fread(inPtr, (size_t) inSize, (size_t) inNum, inFile);
}

int 
XAM_POSIX::xam_fseek(FILE* inFile, xam_int inOffset, int inWhence)
{
#ifdef _WIN32
    // For Windows, use the 64-bit fseek version for xam_int's
    return _fseeki64(inFile, inOffset, inWhence);
#else
    // Here longs are 64-bit, so use xam_int directly
    return fseek(inFile, inOffset, inWhence);
#endif
}

xam_int 
XAM_POSIX::xam_fwrite(const void* inPtr, 
                      xam_int inSize, 
                      xam_int inNum, 
                      FILE* inFile)
{
    return fwrite(inPtr, (size_t) inSize, (size_t) inNum, inFile);
}

int 
XAM_POSIX::xam_fclose(FILE* inFile)
{
    return fclose(inFile);
}

int 
XAM_POSIX::xam_stat(const char* inPath, xam_stat_t *outStat)
{
    return stat(inPath, outStat);
}

int 
XAM_POSIX::xam_fflush(FILE* inFile)
{
    return fflush(inFile);
}

int 
XAM_POSIX::xam_rename(const char* inOldName, const char* inNewName)
{
    return rename(inOldName, inNewName);
}

int 
XAM_POSIX::xam_access(const char* inPath, int inMode)
{
#ifdef _WIN32
  int vResult = -1;

  FILE* handle = fopen (inPath, "r");  
  if(handle)
  {
      vResult = 0;
      fclose (handle);
  }

  return vResult;
#else
    return access(inPath, inMode);
#endif
}

int 
XAM_POSIX::xam_remove(const char* inPath)
{
    return remove(inPath);
}

int 
XAM_POSIX::xam_vfprintf(FILE* inFile, const char* inFormat, va_list ap)
{
    return vfprintf(inFile, inFormat, ap);
}

char* 
XAM_POSIX::xam_getenv(const char *inName)
{
// Need special version for MVS w/ conversion between ASCII and EBCDIC
#ifdef __MVS__
    char  ach_Symbol[2048];
    char *pach_Value = NULL;

    ach_Symbol[sizeof(ach_Symbol) - 1] = '\0';
    strncpy(ach_Symbol,symbol,sizeof(ach_Symbol) - 1);
  #if !defined(ECOMPILE)
    // switch to EBCDIC for rest of function, especially 
    // to call system functions
    XAM_Utils::convert_ASCII_to_EBCDIC(ach_Symbol,
                                       ach_Symbol,
                                       xam_strlen(ach_Symbol));
  #endif
    pach_Value = xam_getenv (ach_Symbol);
  #if !defined(ECOMPILE)
    // if we switched to EBCIDIC to call the system function
    // switch back to ASCII to use the value programmatically
    if (pach_Value) 
    {
        XAM_Utils::convert_EBCDIC_to_ASCII(pach_Value, 
                                           pach_Value, 
                                           xam_strlen(pach_Value));
    }
  #endif
    return pach_Value;
#else
    return getenv(inName);
#endif
}

#ifdef __MVS__

    void addquotes(char *s)
    { /* put quotes around string */
     char tempstr[102];
     strcpy(tempstr,NXLT("'"));
     strcat(tempstr,s);
     strcat(tempstr,NXLT("'"));
     strcpy(s,tempstr);
    }
              
    char * xam_zOS_tmpname(char * s_input)
    { /* create long z/OS datasetname of the form:
       * USERID.Taaaaaa.Dmmddyy.Thhmmss.Rrrrrr
         where aaaaaa is TCB address Dmmddyy is date Thhmmss 
         is time and rrrrr is a random number */
      int j;
      int useridlen;
      char tmpbuf[100];
      char * s_retname;
      void  *jnameptr;
      char  *useridptr;
      char  jobname[50];
      char *c_jnameptr;
      static  short int hlqset=false;
      static  char hlq[10];
      char  qual1[100];
      char  qual2[100];  
      char  qual3[100];
      char  qual4[10];
      void *tcbaddress;
     int ch;
     time_t temp;
     struct tm *timeptr;

     XAMTMPVL(&tmpfilevalptr); //use assembler to get some values

     temp = time(NULL);
     timeptr = localtime(&temp); // hlq for temp dataset 
             
     
        if (hlqset==false)  /* if hlq was never set 
                                then set it */
        {  /* was it zapped */
      
            if (strlen(tmpfilevalptr->hlqname))
               {  
                 memset(hlq,0x00,sizeof(hlq));
                 strcpy(hlq,tmpfilevalptr->hlqname);
                 hlq[strlen(tmpfilevalptr->hlqname)]
                  =NXLT('.');
               }
            else { 
                
                 /* get HLQ from userid of job */           
                  XAMGTUSR(&useridptr);
                  useridlen=useridptr[0];
                  memset(hlq,0x00,sizeof(hlq));
                  memcpy(hlq,&useridptr[1],useridlen);
                  hlq[useridlen]=NXLT('.');
                 }
           hlqset=true;
        }
            strcpy(qual1,hlq);

      
            XAMTCBADR(&tcbaddress);
            sprintf(qual2,
                   NXLT("%8X"),
                   (int) tcbaddress);  

           
             memcpy(qual2,NXLT("TC"),2);  // overlay the first two bytes


             ch = strftime(qual3,sizeof(qual3)-1,
                         NXLT(".D%m%d%y.T%H%M%S"),timeptr);
           
             //sprintf(qual4,NXLT(".R%05d"),rand());
             sprintf(qual4,NXLT(".R%05d"),rand());
             strcat(qual3,qual4);              
             strcat(qual2,qual3);
             strcat(qual1,qual2);



            if (s_input)    // if parm passed then copy back   
            {
             strcpy(s_input,qual1);
             }

           // allocate dataset name to return 
           s_retname=(char *)malloc(strlen(qual1)+1);
           strcpy(s_retname,qual1);

           return s_retname ;
              
    }

    FILE * xam_zOS_tmpfile_mem()
    {
        // use memory files 
        char dsn[100];
        tmpname(dsn); 
        addquotes(dsn);
        FILE *pStream=fopen(dsn,NXLT("wb+,type=memory"));
        return pStream;
    }

    FILE * xam_zOS_tmpfile_hiperspace()
    {

    // use hiperspace memory files
    // preprocessing script is not handling the 
    // type=memory(hiperspace) correctly
    // so we are explicitly coding the convlit

       char dsn[100];
       tmpname(dsn);
       addquotes(dsn);
    #pragma convlit(suspend)
       FILE *pStream=fopen(dsn,"wb+,type=memory(hiperspace)");
    #pragma convlit(resume)

       return pStream;
    }

      
    FILE * xam_zOS_tmpfile_disk()
         {    /* allocate using HLQ of JOB owner ... we are subjected to 
                 SMS rules for
                 userid of owner */

    #define HIORDERMASK 0x80000000
    #define NUMTXT 14
             
             int unitlen;           // length of unitname
             char unitnametxt[100]; // unit name text unit 
             char prialoctxt[100]; // primary space allocation text unit
             char secaloctxt[100]; // secondary space allocation text unit
             char dsntxt[100];   // space for dsn allocate text unit  
             char retddtxt[100]; // space for the return of ddname 
             int dsnlen;         // length of dataset
             char dsn[100];      // area for tmpfile generated dataset name
             int ddnamelen;      // length of DDNAME
             char *c_ptr,*c_ptr1;
             char ddname[15];
             int rc;
             struct __S99struc parmlist;
             char *cptr;
             char unitname_ebcdic[10];
             char *envptr;
             int  envaloc;


    //* this array has only EBCDIC values so we will do our own pragma */

    #pragma convlit(suspend)
              char *s[NUMTXT] = {                      /* array of text pointers */
             /* text units follow */
                 "\0\x02\0\x01\0",                     /*0 DSN   beginning of text unit 2      */
                 "\0\x04\0\x01\0\x01\x04",             /*1 DISP=NEW  new allocation            */
                 "\0\x05\0\x01\0\x01\x02",             /*2 DISP=(,DELETE) normal disp          */
                 "\0\x06\0\x01\0\x01\x04",             /*3 DISP=(,,DELETE) abnormal disp       */
                 "\0\x08\0\0",                         /*4 SPACE=(CYL,..   cylinder allocation */             
                 "\0\x0A\0\x01\0\x03\0\0\x10",         /*5 primary=16      primary allocation  */
                 "\0\x0B\0\x01\0\x03\0\0\x2",          /*6 secondary=2     secondary allocation*/
                 "\0\x15\0\x01\0\x05""SYSDA    ",      /*7 UNIT=SYSDA      unit is SYSDA but could be SYSALLDA  */
                 "\0\x1C\0\0",                         /*8  UNALLOC AT CLOSE a fclose deallocate */
                 "\0\x30\0\x01\0\x02\x17\xC0",         /*9 BLKSIZE=6080   use same blocksize as tmpfile */
                 "\0\x3C\0\x01\0\x02\x40\0",           /*10 DSORG=PS       sequential dataset */
                 "\0\x42\0\x01\0\x02\0\x50",           /*11 LRECL=80       lrecl */
                 "\0\x49\0\x01\0\x01\x98",             /*12 RECFM=FBS      recfm */
                 "\0\x55\0\x01\0\x08""12345678" };     /*13 return of DDNAME   */
    #pragma convlit(resume)             

             xam_zOS_tmpname(dsn);  /* get our own tmp name with routine */
               
             if (envptr=getenv(XAMZOSTMPUNITNAME)) {
                 XAM_Utils::convert_ASCII_to_EBCDIC(envptr,unitname_ebcdic,strlen(envptr));
                memcpy(unitnametxt,s[7],5);
                unitlen=strlen(envptr);
                cptr=(char *)&unitlen;
                unitnametxt[5]=cptr[3];
                memcpy(&unitnametxt[6],unitname_ebcdic,unitlen);
                s[7]=unitnametxt;

             }
             else
             {         
             memcpy(unitnametxt,s[7],5);
             unitlen=strlen(tmpfilevalptr->unitname);
             cptr=(char *)&unitlen;
             unitnametxt[5]=cptr[3];
             memcpy(&unitnametxt[6],tmpfilevalptr->unitname,unitlen);
             s[7]=unitnametxt;
             }

             // setup primary space allocation
             if (envptr=getenv(XAMZOSTMPPRIALC)) {
               envaloc=atoi(envptr);
               cptr=(char *)&envaloc;
               memcpy(prialoctxt,s[5],6);
               memcpy(&prialoctxt[6],&cptr[1],3);
              s[5]=prialoctxt;
            
             } else

             { memcpy(prialoctxt,s[5],6);
               cptr=(char *)&tmpfilevalptr->pricyl;
               memcpy(&prialoctxt[6],&cptr[1],3);
               s[5]=prialoctxt;
             }

              // setup primary space allocation
             if (envptr=getenv(XAMZOSTMPSECALC)) {
               envaloc=atoi(envptr);
               cptr=(char *)&envaloc;
                memcpy(secaloctxt,s[6],6);
                memcpy(&secaloctxt[6],&cptr[1],3);
                s[6]=secaloctxt; 
               
             }

             else

             {    // setup secondary space allocation
               memcpy(secaloctxt,s[6],6);
               cptr=(char *)&tmpfilevalptr->seccyl;
               memcpy(&secaloctxt[6],&cptr[1],3);
               s[6]=secaloctxt;
                     }

             memcpy(retddtxt,s[NUMTXT-1],6); /* setup return ddname verb */
             dsnlen=strlen(dsn); /* get length of specified dataset name */
             c_ptr=(char *)&dsnlen; /* point to dsn length */           
             memcpy(dsntxt,s[0],5); /* copy dsn txt unit */
             dsntxt[5]=c_ptr[3]; // copy length        
             memcpy((void *)&dsntxt[6],dsn,dsnlen);  //copy dataset name
             s[0]=dsntxt;  /* overlay dsn txt unit */
             s[NUMTXT-1]=retddtxt; /* overlay return dd txt unit */

             memset(&parmlist, 0, sizeof(parmlist));
             parmlist.__S99RBLN = 20;
             parmlist.__S99VERB = 1;            /* verb for dsname allocation       */
             parmlist.__S99FLAG1 = 0x4000;      /* do not use existing allocation   */
             parmlist.__S99TXTPP = s;           /* pointer to pointer to text units */
             s[NUMTXT-1] = (char *)((long unsigned) (s[NUMTXT-1]) | HIORDERMASK);
             rc = svc99(&parmlist);
             if (rc != 0) {   //* there is an error in dynamic allocation
                              //* we could consider using memory files

                fprintf(stderr,NXLT("z/OS tmpfile dynalloc error: Error code = x%04X Info code = x%04X\n"),
                   (unsigned short) parmlist.__S99ERROR, (unsigned short) parmlist.__S99INFO);
                 return NULL;
             }

             /* svc99 allocation...take the returned DDNAME and set it up for fopen
                the format for fopen is DD:DDNAME */

             ddnamelen=(unsigned char)retddtxt[5]; /* length of returned ddname */
             memset(ddname,0x00,sizeof(ddname));
             strcpy(ddname,NXLT("DD:"));
             memcpy(&ddname[3],&retddtxt[6],ddnamelen);         
             FILE *f=fopen(ddname,NXLT("wb+"));   
              /* even though DDNAME is useful to z/OS, all we will do is return the FILE PTR */
             return f;
        }

    FILE * xam_zOS_tmpfile_ambersand()
         {   /* use &DSN type allocation */

             int unitlen;           // length of unitname
             char unitnametxt[100]; // unit name text unit 
             char prialoctxt[100]; // primary space allocation text unit
             char secaloctxt[100]; // secondary space allocation text unit
             char dsntxt[100];   // space for dsn allocate text unit  
             char retddtxt[100]; // space for the return of ddname 
             int dsnlen;         // length of dataset
             char dsn[100];      // area for tmpfile generated dataset name
             int ddnamelen;      // length of DDNAME
             char *c_ptr,*c_ptr1;
             char ddname[15];
             int rc;
             struct __S99struc parmlist;
             char *cptr;
             void *tcbaddress;
             char unitname_ebcdic[10];
             char *envptr;
             int  envaloc;


    //* this array has only EBCDIC values so we will do our own pragma */

    #pragma convlit(suspend)
              char *s[NUMTXT] = {                      /* array of text pointers */
             /* text units follow */
                 "\0\x02\0\x01\0",                     /*0 DSN   beginning of text unit 2      */
                 "\0\x04\0\x01\0\x01\x04",             /*1 DISP=NEW  new allocation            */
                 "\0\x05\0\x01\0\x01\x04",             /*2 DISP=(,DELETE) normal disp          */
                 "\0\x06\0\x01\0\x01\x04",             /*3 DISP=(,,DELETE) abnormal disp       */
                 "\0\x08\0\0",                         /*4 SPACE=(CYL,..   cylinder allocation */             
                 "\0\x0A\0\x01\0\x03\0\0\x10",         /*5 primary=16      primary allocation  */
                 "\0\x0B\0\x01\0\x03\0\0\x2",          /*6 secondary=2     secondary allocation*/
                 "\0\x15\0\x01\0\x05""SYSDA    ",      /*7 UNIT=SYSDA      unit is SYSDA but could be SYSALLDA  */
                 "\0\x1C\0\0",                         /*8  UNALLOC AT CLOSE a fclose deallocate */
                 "\0\x30\0\x01\0\x02\x17\xC0",         /*9 BLKSIZE=6080   use same blocksize as tmpfile */
                 "\0\x3C\0\x01\0\x02\x40\0",           /*10 DSORG=PS       sequential dataset */
                 "\0\x42\0\x01\0\x02\0\x50",           /*11 LRECL=80       lrecl */
                 "\0\x49\0\x01\0\x01\x98",             /*12 RECFM=FBS      recfm */
                 "\0\x55\0\x01\0\x08""12345678" };     /*13 return of DDNAME   */
    #pragma convlit(resume)             

             XAMTCBADR(&tcbaddress);   /* get the tcb address ... it is unique */
             //srand((unsigned int)tcbaddress); /* create a unique seed */
             srand((unsigned int)tcbaddress); /* create a unique seed */
             /* try to allocate 10 times */

             for (int i_dynalc=0;i_dynalc<10;i_dynalc++)
             { 
             
             strcpy(dsn,NXLT("&"));  /* first character is ambersand */

             //sprintf(&dsn[1],NXLT("T%05d"),rand());
             sprintf(&dsn[1],NXLT("T%05d"),rand());


             if (envptr=getenv(XAMZOSTMPUNITNAME)) {
                 XAM_Utils::convert_ASCII_to_EBCDIC(envptr,unitname_ebcdic,strlen(envptr));
               memcpy(unitnametxt,s[7],5);
               unitlen=strlen(envptr);
               cptr=(char *)&unitlen;
               unitnametxt[5]=cptr[3];
               memcpy(&unitnametxt[6],unitname_ebcdic,unitlen);
               s[7]=unitnametxt;

                }
               else
                {         
                  memcpy(unitnametxt,s[7],5);
                  unitlen=strlen(tmpfilevalptr->unitname);
                  cptr=(char *)&unitlen;
                  unitnametxt[5]=cptr[3];
                  memcpy(&unitnametxt[6],tmpfilevalptr->unitname,unitlen);
                  s[7]=unitnametxt;
                }

               // setup primary space allocation
               if (envptr=getenv(XAMZOSTMPPRIALC)) {
                 envaloc=atoi(envptr);
                 cptr=(char *)&envaloc;
                 memcpy(prialoctxt,s[5],6);
                 memcpy(&prialoctxt[6],&cptr[1],3);
                 s[5]=prialoctxt;


                } else

                { memcpy(prialoctxt,s[5],6);
                  cptr=(char *)&tmpfilevalptr->pricyl;
                  memcpy(&prialoctxt[6],&cptr[1],3);
                  s[5]=prialoctxt;
                 }

                 // setup secondary space allocation
                if (envptr=getenv(XAMZOSTMPSECALC)) {
                 envaloc=atoi(envptr);
                 cptr=(char *)&envaloc;
                 memcpy(secaloctxt,s[6],6);
                 memcpy(&secaloctxt[6],&cptr[1],3);
                 s[6]=secaloctxt; 
            
                 }
                 else
                  {    // setup secondary space allocation
                   memcpy(secaloctxt,s[6],6);
                   cptr=(char *)&tmpfilevalptr->seccyl;
                   memcpy(&secaloctxt[6],&cptr[1],3);
                   s[6]=secaloctxt;
                  }
             
             memcpy(retddtxt,s[NUMTXT-1],6); /* setup return ddname verb */
             dsnlen=strlen(dsn); /* get length of specified dataset name */
             c_ptr=(char *)&dsnlen; /* point to dsn length */           
             memcpy(dsntxt,s[0],5); /* copy dsn txt unit */
             dsntxt[5]=c_ptr[3]; // copy length        
             memcpy((void *)&dsntxt[6],dsn,dsnlen);  //copy dataset name
             s[0]=dsntxt;  /* overlay dsn txt unit */
             s[NUMTXT-1]=retddtxt; /* overlay return dd txt unit */

             memset(&parmlist, 0, sizeof(parmlist));
             parmlist.__S99RBLN = 20;
             parmlist.__S99VERB = 1;            /* verb for dsname allocation       */
             parmlist.__S99FLAG1 = 0x4000;      /* do not use existing allocation   */
             parmlist.__S99TXTPP = s;           /* pointer to pointer to text units */
             s[NUMTXT-1] = (char *)((long unsigned) (s[NUMTXT-1]) | HIORDERMASK);
             rc = svc99(&parmlist);
             if (rc==0) break;
            }

              if (rc != 0) {   // there is an error in dynamic allocation
                               // we could consider using memory files
                fprintf(stderr,NXLT("z/OS tmpfile dynalloc error: Error code = x%04X   Info code = x%04X\n"),
                 (unsigned short) parmlist.__S99ERROR, (unsigned short) parmlist.__S99INFO);
                 return NULL;
             }
            
             /* svc99 allocation...take the returned DDNAME and set it up for fopen
                the format for fopen is DD:DDNAME */

             ddnamelen=(unsigned char)retddtxt[5]; /* length of returned ddname */
             memset(ddname,0x00,sizeof(ddname));
             strcpy(ddname,NXLT("DD:"));
             memcpy(&ddname[3],&retddtxt[6],ddnamelen);         

             FILE *f=fopen(ddname,NXLT("wb+"));   
              /* even though DDNAME is useful to z/OS, all we will do is return the FILE PTR */
             return f;
        }

    FILE * xam_zOS_tmpfile_system()
         {   /* create system generated unique names by not specifying dataset name  */

             char unitname_ebcdic[10];
             int unitlen;           // length of unitname
             char unitnametxt[100]; // unit name text unit 
             char prialoctxt[100]; // primary space allocation text unit
             char secaloctxt[100]; // secondary space allocation text unit
             char dsntxt[100];   // space for dsn allocate text unit  
             char retddtxt[100]; // space for the return of ddname
             char retdsnametxt[100]; // space for return of dsn
             int dsnlen;         // length of dataset
             char dsn[100];      // area for tmpfile generated dataset name
             int ddnamelen;      // length of DDNAME
             char *c_ptr,*c_ptr1;
             char ddname[15];
             int rc;
             struct __S99struc parmlist;
             char *cptr;
             void *tcbaddress;
             char *envptr;
             int  envaloc;

    //* this array has only EBCDIC values so we will do our own pragma */

    #pragma convlit(suspend)
              char *s[14] = {                      /* array of text pointers */
                                                   /* text units follow */
                 "\0\x04\0\x01\0\x01\x04",             /*0 DISP=NEW  new allocation            */
                 "\0\x05\0\x01\0\x01\x04",             /*1 DISP=(,DELETE) normal disp          */
                 "\0\x06\0\x01\0\x01\x04",             /*2 DISP=(,,DELETE) abnormal disp       */
                 "\0\x08\0\0",                         /*3 SPACE=(CYL,..   cylinder allocation */             
                 "\0\x0A\0\x01\0\x03\0\0\x10",         /*4 primary=16      primary allocation  */
                 "\0\x0B\0\x01\0\x03\0\0\x2",          /*5 secondary=2     secondary allocation*/
                 "\0\x15\0\x01\0\x05""SYSDA    ",      /*6 UNIT=SYSDA      unit is SYSDA but could be SYSALLDA  */
                 "\0\x1C\0\0",                         /*7  UNALLOC AT CLOSE a fclose deallocate */
                 "\0\x30\0\x01\0\x02\x17\xC0",         /*8 BLKSIZE=6080   use same blocksize as tmpfile */
                 "\0\x3C\0\x01\0\x02\x40\0",           /*9 DSORG=PS       sequential dataset */
                 "\0\x42\0\x01\0\x02\0\x50",           /*10 LRECL=80       lrecl */
                 "\0\x49\0\x01\0\x01\x98",             /*11 RECFM=FBS      recfm */
                 "\0\x55\0\x01\0\x08""12345678  ",     /*12 return of DDNAME       */
                 "\0\x56\0\x01\0\x44" };               /*13 return of DSNAME       */
    #pragma convlit(resume)             


             for (int i_dynalc=0;i_dynalc<10;i_dynalc++)
             { 
       
             if (envptr=getenv(XAMZOSTMPUNITNAME)) {
                XAM_Utils::convert_ASCII_to_EBCDIC(envptr,unitname_ebcdic,strlen(envptr));
                memcpy(unitnametxt,s[6],5);
                unitlen=strlen(envptr);
                cptr=(char *)&unitlen;
                unitnametxt[5]=cptr[3];
                memcpy(&unitnametxt[6],unitname_ebcdic,unitlen);
                s[6]=unitnametxt;

             }
             else
             {         
             memcpy(unitnametxt,s[6],5);
             unitlen=strlen(tmpfilevalptr->unitname);
             cptr=(char *)&unitlen;
             unitnametxt[5]=cptr[3];
             memcpy(&unitnametxt[6],tmpfilevalptr->unitname,unitlen);
             s[6]=unitnametxt;
             }

             // setup primary space allocation
             if (envptr=getenv(XAMZOSTMPPRIALC)) {
               envaloc=atoi(envptr);
               cptr=(char *)&envaloc;
               memcpy(prialoctxt,s[4],6);
               memcpy(&prialoctxt[6],&cptr[1],3);
              s[4]=prialoctxt;

             } else

             { memcpy(prialoctxt,s[4],6);
               cptr=(char *)&tmpfilevalptr->pricyl;
               memcpy(&prialoctxt[6],&cptr[1],3);
               s[4]=prialoctxt;
             }

              // setup primary space allocation
             if (envptr=getenv(XAMZOSTMPSECALC)) {
               envaloc=atoi(envptr);
               cptr=(char *)&envaloc;
                memcpy(secaloctxt,s[5],6);
                memcpy(&secaloctxt[6],&cptr[1],3);
                s[5]=secaloctxt;     
             }

             else

             {    // setup secondary space allocation
               memcpy(secaloctxt,s[5],6);
               cptr=(char *)&tmpfilevalptr->seccyl;
               memcpy(&secaloctxt[6],&cptr[1],3);
               s[5]=secaloctxt;
             }
             
             memcpy(retddtxt,s[12],6); /* setup return ddname verb */
             s[12]=retddtxt; /* overlay return dd txt unit */

             memset(retdsnametxt,0x00,sizeof(retdsnametxt));
             memcpy(retdsnametxt,s[13],6);
             s[13]=retdsnametxt;

             memset(&parmlist, 0, sizeof(parmlist));
             parmlist.__S99RBLN = 20;
             parmlist.__S99VERB = 1;            /* verb for dsname allocation       */
             parmlist.__S99FLAG1 = 0x4000;      /* do not use existing allocation   */
             parmlist.__S99TXTPP = s;           /* pointer to pointer to text units */
             s[13] = (char *)((long unsigned) (s[13]) | HIORDERMASK);
             rc = svc99(&parmlist);
             if (rc==0) break;
            }

              if (rc != 0) {   // there is an error in dynamic allocation
                               // we could consider using memory files
                fprintf(stderr,NXLT("z/OS tmpfile dynalloc error: Error code = x%04X   Info code = x%04X\n"),
                 (unsigned short) parmlist.__S99ERROR, (unsigned short) parmlist.__S99INFO);
                 return NULL;
             }
            
             /* svc99 allocation...take the returned DDNAME and set it up for fopen
                the format for fopen is DD:DDNAME */

             ddnamelen=(unsigned char)retddtxt[5]; /* length of returned ddname */
             memset(ddname,0x00,sizeof(ddname));
             strcpy(ddname,NXLT("DD:"));
             memcpy(&ddname[3],&retddtxt[6],ddnamelen);         
             FILE *f=fopen(ddname,NXLT("wb+"));   
              /* even though DDNAME is useful to z/OS, all we will do is return the FILE PTR */
             return f;
        }
#endif // MVS

FILE* 
XAM_POSIX::xam_tmpfile()
{
     FILE* vFile = (FILE*) 0;

#ifdef __MVS__      
    int tmpfile_type=0;
    char *envptr;
    if (envptr = xam_getenv(XAMZOSTMPTYPE)) {
        if (xam_strcmp(envptr,XAMZOSTMPSYSGEN)==0)      {  tmpfile_type=TMPSYSGEN;    }
        if (xam_strcmp(envptr,XAMZOSTMPMEMORY)==0)      {  tmpfile_type=TMPMEM;       }
        if (xam_strcmp(envptr,XAMZOSTMPHIPER)==0)       {  tmpfile_type=TMPHIPER;     }
        if (xam_strcmp(envptr,XAMZOSTMPAMBERSAND)==0)   {  tmpfile_type=TMPAMBERSAND; }
        if (xam_strcmp(envptr,XAMZOSTMPSDKGEN)==0)      {  tmpfile_type=TMPSDKGEN;    }
    }

    /* zOS cannot use tmpfile() as it will break when multitasking
       so we use our own routines */

    XAMTMPVL(&tmpfilevalptr); //use assembler to get some values

    if (tmpfile_type==0) {
    tmpfile_type=tmpfilevalptr->memtype;
    }

    switch (tmpfile_type)
    {
    case 1: vFile=xam_zOS_tmpfile_disk();
            break;
    case 2: vFile=xam_zOS_tmpfile_mem();
            break;
    case 3: vFile=xam_zOS_tmpfile_hiperspace();
            break;
    case 4: vFile=xam_zOS_tmpfile_ambersand();
            break;
    default:
            vFile=xam_zOS_tmpfile_system();
            break;
    }   
#else // POSIX / WINDOWS   
    vFile = tmpfile();
#endif 
    
    return vFile;  
}

int
XAM_POSIX::xam_gettimeofday(struct timeval *inTP)
{
#ifdef _WIN32
    FILETIME        vFT;
    LARGE_INTEGER   vLI;
    __int64         t;

    if (inTP)
    {
        GetSystemTimeAsFileTime(&vFT);
        vLI.LowPart  = vFT.dwLowDateTime;
        vLI.HighPart = vFT.dwHighDateTime;
        t  = vLI.QuadPart;         // 100-nanosec intervals
        t -= EPOCHFILETIME;       // offset to the Epoch time
        t /= 10;                  // to microseconds
        inTP->tv_sec  = (long)(t / 1000000);
        inTP->tv_usec = (long)(t % 1000000);
    }

    return 0;
#else
    return gettimeofday(inTP, 0);
#endif
}

xam_int 
XAM_POSIX::xam_strftime(char *s, 
                        xam_int inMaxSize, 
                        const char *inFormat, 
                        const struct tm *inTimePtr)
{
    return strftime(s, (size_t) inMaxSize, inFormat, inTimePtr);
}

struct tm* 
XAM_POSIX::xam_gmtime (const time_t *inClock)
{
#ifdef _WIN32
    return gmtime(inClock); // MS version is already thread safe
#else
    XAM_CriticalSection vCritical(&sMutex);
    static struct tm vTM;
    return gmtime_r(inClock, &vTM);
#endif
}

int 
XAM_POSIX::xam_atoi(const char *inPtr)
{
    return atoi(inPtr);
}

XAM_DIR* 
XAM_POSIX::xam_opendir(const char* inDirPath)
{
#ifdef _WIN32
    XAM_DIR* vDir = (XAM_DIR*)0;
    XAM_DIR* vLocDir = (XAM_DIR*) xam_malloc(sizeof(XAM_DIR));
    if(vLocDir == (XAM_DIR*)0)
    {
        XAMOutOfMemoryException e;
        throw e;
    }

    char vDirPath[1024*4];
    xam_sprintf(vDirPath, "%s\\*", inDirPath);

    vLocDir->findHandle = FindFirstFile(vDirPath, &(vLocDir->fileData));
    if(vLocDir->findHandle == INVALID_HANDLE_VALUE)    
        xam_free(vLocDir);    
    else
    {
        vDir = vLocDir;
        vDir->entry = (XAM_DIR_ENTRY*) xam_malloc(sizeof(XAM_DIR_ENTRY));
        if(vDir->entry == (XAM_DIR_ENTRY*)0)
        {
            XAMOutOfMemoryException e;
            throw e;
        }
    }

    return vDir;
#else
    return (XAM_DIR*)opendir(inDirPath);
#endif
}

XAM_DIR_ENTRY* 
XAM_POSIX::xam_readdir(XAM_DIR* inDir)
{
#ifdef _WIN32
    XAM_DIR_ENTRY* vEntry = (XAM_DIR_ENTRY*) 0;
    BOOL vFound = FindNextFile(inDir->findHandle, &(inDir->fileData));
    if(vFound)
    {
        vEntry = inDir->entry;
        vEntry->d_name = inDir->fileData.cFileName;
    }
    return vEntry;
#else
    return (XAM_DIR_ENTRY*)readdir(inDir);
#endif
}

int 
XAM_POSIX::xam_closedir(XAM_DIR* inDir)
{
#ifdef _WIN32
    BOOL vRetVal = FindClose(inDir->findHandle);
    xam_free(inDir->entry);
    xam_free(inDir);
    return (vRetVal);
#else
    return closedir(inDir);
#endif
}
