#ifndef __XAM_POSIX_H__
#define __XAM_POSIX_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include <xam_types.h>

// ------------------------------------
// Platform independent headers
// ------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

// ------------------------------------
// Platform independent defines
// ------------------------------------
#define ENOERR 0

// ------------------------------------
// Platform dependent headers
// ------------------------------------

#ifdef _WIN32
  #include <windows.h>
  #include <time.h>
#else
  #if defined (__MVS__)
    #include <dll.h>
  #else // POSIX
    #include <dlfcn.h>
  #endif

  #include <unistd.h>
  #include <strings.h>
  #include <pthread.h>
  #include <signal.h>
  #include <sys/time.h>
  #include <sys/types.h>
  #include <errno.h>

  #include <dirent.h>
#endif

// ------------------------------------
// typedefs
// ------------------------------------

#ifdef _WIN32
    typedef CRITICAL_SECTION xam_mutex_t; 
    typedef DWORD xam_thread_t;
    typedef HANDLE xam_hthread_t;
    typedef xam_int xam_pid_t;

    typedef struct stat xam_stat_t;
    typedef HANDLE XAM_FILE_HANDLE;

    #ifndef __GNUC__
      #define EPOCHFILETIME (116444736000000000i64) 
    #else
      #define EPOCHFILETIME (116444736000000000LL)
    #endif

    // for access operations (these do nothing on win32)
    #define XAM_R_OK   0
    #define XAM_W_OK   0
    #define XAM_F_OK   0
    #define XAM_X_OK   0

    #define XAM_FILE_SEP '\\'

    typedef enum XAM_DLOPEN_MODE
    {
        LAZY=0,
        FULL
    } XAM_DLOPEN_MODE;

    typedef struct XAM_DIR_ENTRY
    {
        HANDLE foundHandle;       
        char* d_name;
    } XAM_DIR_ENTRY;

    typedef struct XAM_DIR
    {
        WIN32_FIND_DATA fileData;
        HANDLE findHandle;
        XAM_DIR_ENTRY* entry;
    } XAM_DIR;

#else // POSIX
   #if defined SUN
       #include <thread.h>  // Use Solaris threads
        typedef thread_t xam_thread_t;
   #else
        typedef pthread_t xam_thread_t;
   #endif

    typedef pthread_mutex_t xam_mutex_t;
    typedef pthread_t xam_hthread_t;
    typedef pid_t xam_pid_t;

    typedef struct stat xam_stat_t;
    typedef FILE* XAM_FILE_HANDLE;

    #define XAM_FILE_SEP '/'

    // for access operations
    //! See POSIX
    #define XAM_R_OK     R_OK
    //! See POSIX
    #define XAM_W_OK     W_OK
    //! See POSIX
    #define XAM_F_OK     F_OK
    //! See POSIX
    #define XAM_X_OK     X_OK

    typedef enum
    {
        LAZY = RTLD_LAZY,
        FULL = RTLD_NOW
    } XAM_DLOPEN_MODE;

    typedef DIR XAM_DIR;
    typedef dirent XAM_DIR_ENTRY;
#endif

#ifdef _WIN32
  typedef DWORD XAM_THREAD_FUNCTION_PTR;
#else // POSIX
  typedef void* XAM_THREAD_FUNCTION_PTR;
#endif

// Used for formatting xam_int values (int64 vs. long long)
#ifdef _WIN32
  #define XAM_INT_FORMAT "I64"
#else // POSIX
  #define XAM_INT_FORMAT "ll"
#endif

/* Pointer to a thread method */
typedef XAM_THREAD_FUNCTION_PTR (*XAM_THREAD_METHOD)(void *);

/** Thread information structure */
typedef struct {
  xam_thread_t thread_id;
  xam_hthread_t thread_handle;
  xam_boolean running;
  xam_boolean killed;
  void* data;
} XAM_ThreadInfo;

/* Thread status codes */
typedef enum
{
  THREAD_RUNNING = -2,
  THREAD_KILLED = -1,
  THREAD_OK = 0
} XAM_ThreadStatus;

/* Library name prefix, suffix, and architecture */
#if defined WIN32 | WIN64 | __MVS__
    #define OS_LIB_PREFIX ""
    #define OS_LIB_ARCH "" 
    #define OS_LIB_EXT  ".dll"
#else // POSIX
    // All supported unix type platforms have a lib prefix in library name
    #define OS_LIB_PREFIX "lib"

    #if defined XAM32BIT
       #define OS_LIB_ARCH "32" 
    #elif defined XAM64BIT 
       #define OS_LIB_ARCH "64" 
    #else 
       #define OS_LIB_ARCH "" 
    #endif

    // Determine the library name suffix to use for these platforms
    #if defined HPUX
        #if defined ITANIUM
	        #define OS_LIB_EXT ".so"
        #else
	        #define OS_LIB_EXT ".sl"
        #endif
    #elif defined AIX51
    	#define OS_LIB_EXT ".a"
    #else
        // SUN, LINUX, etc.
        #define OS_LIB_EXT ".so"
    #endif
#endif // POSIX

/* Define misc methods */
#define xam_sscanf sscanf 

/**
 * A POSIX compatibility interface for the XAM SDK
 */
class XAM_POSIX
{
    private:
        XAM_POSIX() {}  // This object cannot be constructed

    public:
        
        // NOTE: See POSIX documentation for a detailed description of these 
        // methods.    

        // ------------------------------------
        // Memory management methods
        // ------------------------------------

        static void* xam_malloc(xam_int size);
        static void  xam_free(void *inPtr);
        static void* xam_realloc(void* inPtr, xam_int size);
        static void* xam_calloc(xam_int inCount, xam_int inSize);
        static void* xam_memcpy(void *inDest, const void *inSrc, xam_int inLen);
        static void* xam_memset(void *inPtr, int c, xam_int inSize);
        static char* xam_strdup(const char *inString);

        // ------------------------------------
        // Library management methods
        // ------------------------------------
        
        static void* xam_dlopen(const char *inLibPath,
                                XAM_DLOPEN_MODE inMode);

        static void* xam_dlsym(void *inHandleToLib, 
                               const char *inNameOfFunction);

        static int   xam_dlclose(void *inHandleToLib);

        static const char* xam_dlerror();

        // ------------------------------------
        // Thread methods
        // ------------------------------------

		static void xam_sleep(xam_int inSeconds);

        static int xam_thread_create(XAM_ThreadInfo &outInfo, 
                                     XAM_THREAD_METHOD inMethod);

        static int xam_thread_join(XAM_ThreadInfo &ioInfo);

        static bool xam_thread_kill(XAM_ThreadInfo &ioInfo);

        static void xam_thread_exit();

        static xam_thread_t xam_thread_self();

        // ------------------------------------
        // Thread management methods
        // ------------------------------------

        static int xam_mutex_init(xam_mutex_t *inMutex);
        static int xam_mutex_lock(xam_mutex_t *inMutex);
        static int xam_mutex_unlock(xam_mutex_t *inMutex);
        static int xam_mutex_destroy(xam_mutex_t *inMutex);
        static xam_pid_t xam_getpid();

        // ------------------------------------
        // String manipulation methods
        // ------------------------------------

        static char* xam_strstr(const char *inString, 
                                const char *inSearchString);

        static char* xam_strcasestr(const char *inString, 
                                    const char *inSearchString);

        static char* xam_strcpy(char *inDest, const char *inSrc);
        static char* xam_strncpy(char *inDest, const char *inSrc, xam_int len);

        static int xam_sprintf(char *inDest, const char *inFormat, ...);

        static int xam_snprintf(char *inDest, 
                                xam_int size, 
                                const char* inFormat, ...);

        static int xam_vsprintf(char *inDest, const char* inFormat, va_list ap);

        static int xam_vsnprintf(char *inDest, 
                                xam_int size, 
                                const char* inFormat, 
                                va_list ap);

        static xam_int xam_strlen(const char *inString);

        static int xam_strcmp(const char* inStr1,
                              const char* inStr2);

        static int xam_strncmp(const char* inStr1, 
                               const char* inStr2, xam_int len);

        static int xam_strcasecmp(const char* inStr1, 
                                  const char* inStr2);

        static int xam_strncasecmp(const char* inStr1, 
                                  const char* inStr2, 
                                  xam_int len);

        static char* xam_strcat(char* inStr, char* inAppendStr);

        static char* xam_strchr(char* inStr, int c);

        static char* xam_strrchr(char* inString, int c);

        static char* xam_strtok_r(char* inStr, 
                                  const char* inSeparator, 
                                  char** inLast);

        // ------------------------------------
        // File management methods
        // ------------------------------------

        static FILE* xam_fopen(const char* inPath, const char* inMode);

        static xam_int xam_fread(void* inPtr,
                                xam_int inSize, 
                                xam_int inNum,
                                FILE* inFile);

        static int xam_fseek(FILE* inFile, xam_int inOffset, int inWhence); 

        static xam_int xam_fwrite(const void* inPtr, 
                                 xam_int inSize, 
                                 xam_int inNum, 
                                 FILE* inFile);

        static int xam_fclose(FILE* inFile);

        static int xam_stat(const char* inPath, xam_stat_t *outStat);

        static int xam_fflush(FILE* inFile);

        static int xam_rename(const char* inOldName, const char* inNewName);

        static int xam_access(const char* inPath, int mode);

        static int xam_remove(const char* inPath);

        static int xam_vfprintf(FILE* inFile, 
                                const char* inFormat, 
                                va_list ap);

        static FILE* xam_tmpfile();

        // ------------------------------------
        // Time management methods
        // ------------------------------------

        static int xam_gettimeofday(struct timeval *inTimeVal);

        static xam_int xam_strftime(char *s, 
                                   xam_int inMaxSize, 
                                   const char *inFormat, 
                                   const struct tm *inTimePtr);

        static struct tm* xam_gmtime (const time_t *inClock);

        // ------------------------------------
        // Directory access / search
        // ------------------------------------

        static XAM_DIR* xam_opendir(const char* inDirPath);
        static XAM_DIR_ENTRY* xam_readdir(XAM_DIR* inDir);
        static int xam_closedir(XAM_DIR* inDir);    

        // ------------------------------------
        // Misc
        // ------------------------------------ 

        static char* xam_getenv(const char *name);
        static int xam_atoi(const char *inPtr);

        // Adjusts a library name to include a platform specific 
        // prefix and/or suffix, as needed.
        //
        // @param inLibName The library name to adjust
        // @returns An allocated string containing the adjusted name
        //          (caller must free this buffer with xam_free!)
        //          or NULL if the library name could not be adjusted.
        static char* xam_adjust_library_name(const char* inLibName);
};

#endif // __XAM_POSIX_H__


