#ifndef XAM_BASE64_H
#define XAM_BASE64_H
////////////////////////////////////////////////////////////////////////////////
//  Copyright  2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright  2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////
//
//    Based on "base64 implementation in c"
//    Copyright (c) 2001 Bob Trower, Trantor Standard Systems Inc.
//
//    Permission is hereby granted, free of charge, to any person
//    obtaining a copy of this software and associated
//    documentation files (the "Software"), to deal in the
//    Software without restriction, including without limitation
//    the rights to use, copy, modify, merge, publish, distribute,
//    sublicense, and/or sell copies of the Software, and to
//    permit persons to whom the Software is furnished to do so,
//    subject to the following conditions:
//
//    The above copyright notice and this permission notice shall
//    be included in all copies or substantial portions of the
//    Software.
//
//    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
//    KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//    WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//    PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
//    OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//    OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
//    OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

static const char sCB64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
static const char sCD64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";

void 
encodeblock (unsigned char inBlock[3], unsigned char outBlock[4], int inLen)
{
    outBlock[0] = sCB64[inBlock[0] >> 2 ];
    outBlock[1] = sCB64[((inBlock[0] & 0x03) << 4) | ((inBlock[1] & 0xf0) >> 4) ];
    outBlock[2] = (unsigned char) (inLen > 1 ? sCB64[((inBlock[1] & 0x0f) << 2) | ((inBlock[2] & 0xc0) >> 6)] : '=');
    outBlock[3] = (unsigned char) (inLen > 2 ? sCB64[inBlock[2] & 0x3f] : '=');
}

void 
decodeblock (unsigned char inBlock[4], unsigned char outBlock[3])
{   
    outBlock[0] = (unsigned char) (inBlock[0] << 2 | inBlock[1] >> 4);
    outBlock[1] = (unsigned char) (inBlock[1] << 4 | inBlock[2] >> 2);
    outBlock[2] = (unsigned char) (((inBlock[2] << 6) & 0xc0) | inBlock[3]);
}

// make sure outDstBuf is at least (inSrcLen+2)/3 * 4 bytes long
void 
base64_encode (const char *inSrcBuf, 
               int inSrcLen, 
               char *outDstBuf, 
               int *outDstLen)
{
    unsigned char inBlock[3], outBlock[4];
    int src_ix = 0;
    int dst_ix = 0;
    int len = 0;
    while ( src_ix < inSrcLen )
    {
        len = 0;
        for ( int ix=0; ix<3; ix++ ) 
        {
            if ( src_ix >= inSrcLen )
                inBlock[ix] = (unsigned char)0;
            else
            {
                inBlock[ix] = (unsigned char)inSrcBuf[src_ix++];
                len++;
            }
        }
        if ( len ) 
        {
            encodeblock(inBlock, outBlock, len);
            for ( int ix=0; ix<4; ix++ )
                    outDstBuf[dst_ix++] = (char)outBlock[ix];
        }
    }
    *outDstLen = dst_ix;
}

// make sure outBuf is at least (inSrcLen+3)/4 * 3 bytes long
void 
base64_decode (const char *inSrcBuf, 
               int inSrcLen, 
               char *outDstBuf, 
               int *outDstLen)
{
    unsigned char inBlock[4], outBlock[3], v;
    int src_ix = 0;
    int dst_ix = 0;
    int ix, len;

    while ( src_ix < inSrcLen )
    {
        for ( len=0, ix=0; ix<4 && src_ix<inSrcLen; ix++ ) 
        {
            v = 0;
            while ( src_ix<inSrcLen && v==0 ) 
            {
                v = (unsigned char)inSrcBuf[src_ix++];
                v = (unsigned char)((v < 43 || v > 122) ? 0 : sCD64[ v - 43 ]);

                if ( v )
                    v = (unsigned char)((v == '$') ? 0 : v - 61);
            }
            if ( v == 0 ) 
                inBlock[ix] = 0;
            else
            {
                len++;
                inBlock[ix] = (unsigned char)(v - 1);
            }
        }

        if ( len ) 
        {
            decodeblock(inBlock, outBlock);
            for ( ix=0; ix<len-1; ix++ )
                    outDstBuf[dst_ix++] = outBlock[ix];
        }
    }
    *outDstLen = dst_ix;
}


#endif // XAM_BASE64_H

