////////////////////////////////////////////////////////////////////////////////
//  Copyright  2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright  2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include <string.h>

#include "xuid_toolkit.h"
#include "xam_errors.h"
#include "base64.h"
#include "crc.h"

// For endian POSIX methods
#ifdef POSIX
#include <netinet/in.h>
#ifndef __MVS__
#include <netinet/tcp.h>
#endif // __MVS__
#endif
#ifdef _WIN32
#include <winsock.h>
#endif 

xam_status
XUID_ToString(const xam_xuid inBinaryXUID,
              xam_string* const outStringXUID)
{
    xam_status vResult = 0;

    // Check input params
    if(!inBinaryXUID || !outStringXUID)
        vResult = XAM_INVALID_PARAM;
    else
    {
        // Determine binary XUID validity
        xam_boolean vValidXUID;
        XUID_IsValid(inBinaryXUID, &vValidXUID);

        if(vValidXUID == false)
            vResult = XAM_BAD_XUID_FORMAT;
        else
        {
            // Encode the binary XUID
            int vLen = sizeof(xam_string);
            base64_encode(inBinaryXUID, (int)(inBinaryXUID[5]), *outStringXUID, &vLen);
            (*outStringXUID)[vLen] = '\0';
        }
    }

    return vResult;
}

xam_status
XUID_ToBinary(const xam_string inStringXUID,
              xam_xuid* const outBinaryXUID)
{
    xam_status vResult = 0;

    if(!inStringXUID || !outBinaryXUID)
        vResult = XAM_INVALID_PARAM;
    else
    {
        // Decode the string XUID
        int vLen = sizeof(xam_xuid);
        base64_decode(inStringXUID, (int)strlen(inStringXUID), *outBinaryXUID, &vLen);

        // Pad the rest with zeros (as per the spec)
        int vDiff = sizeof(xam_xuid) - vLen;
        for(int i=0 ; i < vDiff ; i++)
            (*outBinaryXUID)[vLen + i] = '\0';

        // Determine binary XUID validity
        xam_boolean vValidXUID;
        XUID_IsValid(*outBinaryXUID, &vValidXUID);
        if(vValidXUID == false)
            vResult = XAM_BAD_XUID_FORMAT;
    }

    return vResult;
}

xam_status
XUID_IsValid(const xam_xuid inBinaryXUID,
             xam_boolean* const outIsValid)
{
    // Cache the current xam_xuid w/ padded bytes
    xam_xuid vXUID; 
    memset(vXUID, 0, sizeof(xam_xuid));
    memcpy(vXUID, inBinaryXUID, inBinaryXUID[5]);

    // Replace the 6th and 7th byte with 0 to compute the CRC 
    memset(&vXUID[6], 0, 2 * sizeof(char));

    // Run the fast CRC-16 algorithm on this data
    unsigned short vActualCrc = crcFast(vXUID, vXUID[5]);

    // Determine if the actual CRC matches the expected CRC
    unsigned short* vShortPtr = ((unsigned short*)inBinaryXUID) + 3;
    unsigned short vExpectedCrc = *vShortPtr;

    // Convert computed CRC platform byte order to network byte order
    vActualCrc = htons(vActualCrc);

    // If the expected CRC matches the actual CRC, this XUID is valid!
    if(vExpectedCrc == vActualCrc)
        *outIsValid = true;
    else
        *outIsValid = false;

    return 0;
}

