#ifndef __XAM_ASSOCIATIVE_ARRAY__
#define __XAM_ASSOCIATIVE_ARRAY__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"

#include "XAM_Exception.h"
#include "XAM_POSIX.h"

/**
 * An array which stores key-value pairs
 *
 * A class that implements a simple associative array that stores 
 * key-value pairs, associating a value with each key, and that
 * lets insert and retrieve elemement quickly based on their keys.
 */
class XAM_AssociativeArray
{
    private:
        int  mMaxNumberOfAttributes;  // Maximum # of attribs allowed
        int  mNumberOfAttributes;     // Actual # of attribs
        char** mName;                 // Name pointer
        char** mValue;                // Value pointer
        
        char** 
        copyNameArray ()
        {
           char** vCopiedArray = (char**)0;
            if (mMaxNumberOfAttributes > 0)
            {
                xam_int vSize = size();
                vCopiedArray = 
                    (char**)XAM_POSIX::xam_calloc(mMaxNumberOfAttributes, 
                                                  sizeof(*vCopiedArray));

                for ( xam_int ix=0; ix<vSize; ix++ )
                {
                  if ( mName[ix] )
                     vCopiedArray[ix] = XAM_POSIX::xam_strdup(mName[ix]);
                }
            }
            return vCopiedArray;
        }

        char** 
        copyValueArray ()
        {
            char** vCopiedArray = (char**)0;
            xam_int vSize = size();
            int vMaxSz = mMaxNumberOfAttributes;
            if (vMaxSz > 0)
            {
               vCopiedArray = 
                   (char**)XAM_POSIX::xam_calloc(vMaxSz, 
                                                 sizeof(*vCopiedArray));

               for ( xam_int ix=0; ix<vSize; ix++ )
               {
                  if ( mValue[ix] )
                     vCopiedArray[ix] = XAM_POSIX::xam_strdup(mValue[ix]);
               }
            }
            return vCopiedArray;
        }

        int 
        _put (char* inName, char* inValue)
        {
            // ensure the list is large enough to hold the new pair
            if ( mNumberOfAttributes >= mMaxNumberOfAttributes )
            {
                if ( mMaxNumberOfAttributes <= 0 )
                    resize(2);  
                else         
                    resize(2*mMaxNumberOfAttributes);
            }

            // insert the pair
            int vIndex = -1;
            for ( int ix=0; ix<mMaxNumberOfAttributes; ix++ )
            {
                if ( mName[ix] == (char*)0 )
                {
                    mName[ix] = inName;
                    if ( mName[ix] )
                    {
                        mValue[ix] = inValue;
                        if ( mValue[ix] == (char*)0 )
                            mName[ix] = (char*)0;
                        else
                        {
                            mNumberOfAttributes++;
                            vIndex = ix;
                        }
                    }
                    break;
                }
            }
            return vIndex;
        }

        void 
        releaseMem()
        {
          if ( mMaxNumberOfAttributes > 0 )
            {
                xam_int ix = 0;
                if ( mName )
                {
                    for ( ix=0; ix<mMaxNumberOfAttributes; ix++ )
                    { 
                        if ( mName[ix] )
                            XAM_POSIX::xam_free(mName[ix]);
                    }
                    XAM_POSIX::xam_free(mName);
                }
                if ( mValue )
                {
                    for ( ix=0; ix<mMaxNumberOfAttributes; ix++ )
                    { 
                        if ( mValue[ix] )
                        {
                            XAM_POSIX::xam_free(mValue[ix]);
                            mValue[ix] = (char*)0;
                        }
                    }
                    XAM_POSIX::xam_free(mValue);
                }
            }
        }
        
    public:
        
        void 
        resize (const int inAdditionalAttributeCount)
        {
            int vCount = mNumberOfAttributes + inAdditionalAttributeCount;
            if ( vCount > mMaxNumberOfAttributes )
            {
                // create some new arrays
                xam_int vSz = vCount * sizeof(char*);
                char** vName  = (char**)XAM_POSIX::xam_malloc((size_t)vSz);
                char** vValue = (char**)XAM_POSIX::xam_malloc((size_t)vSz);
                if((vName == (char**)0) || (vValue == (char**)0))
                {
                    XAMOutOfMemoryException e;
                    throw e;
                }

                // copy existing elements into the 'old' area of the arrays
                int ix = 0;
                int vNumberOfAttributes = 0;
                for ( ix=0; ix<mNumberOfAttributes; ix++ )
                {
                    // skip "empty" names
                    if ( mName[ix] == (char*)0 ) 
                    {
                        // release resources associated with "nameless" values
                        if ( mValue[ix] )
                            XAM_POSIX::xam_free(mValue[ix]);
                        continue;
                    }

                    // copy the name/value pair
                    vName[vNumberOfAttributes] = mName[ix];
                    vValue[vNumberOfAttributes] = mValue[ix];
                    vNumberOfAttributes++;
                }

                // clear the remainder of the arrays
                for ( ix=vNumberOfAttributes; ix<vCount; ix++ )
                {
                    vName[ix]  = (char*)0;
                    vValue[ix] = (char*)0;
                }

                // assign the new arrays and update the counters
                if ( mName )
                    XAM_POSIX::xam_free(mName);
                mName = vName;
                if ( mValue )
                    XAM_POSIX::xam_free(mValue);
                mValue = vValue;
                mMaxNumberOfAttributes = vCount;
                mNumberOfAttributes = vNumberOfAttributes;
            }
        }

        /** insert name-value pair (both must be non-null) */
        int 
        put (const char* inName, const char* inValue)
        {
            int vIndex = -1;
            if ( inName && inValue )
            {
                char* vName = XAM_POSIX::xam_strdup(inName);
                if ( vName )
                {
                    char* vValue = XAM_POSIX::xam_strdup(inValue);
                    if ( vValue == (char*)0 )
                        XAM_POSIX::xam_free(vName);
                    else
                    {
                        vIndex = _put(vName, vValue);
                        if ( vIndex == (-1) )
                        {
                            XAM_POSIX::xam_free(vName);
                            XAM_POSIX::xam_free(vValue);
                        }
                    }
                }
            }
            return vIndex;
        }
       
        /** insert a list of name-value pairs */
        int 
        putList (const char** inPairs, const int inNumPairs)
        {
            int vNewSize = inNumPairs/2;
            int vCount = vNewSize * 2;
            int vNumInserted = 0;
            for ( int ix=0; ix<vCount; ix+=2 )
            {
                if ( put(inPairs[ix], inPairs[ix+1]) != (-1) )
                    vNumInserted++;
            }
            return vNumInserted;
        }

        /** 
         * Insert a delimited list into the map, delimited by inDelim, with the
         * value = inValue
         */
        int 
        putList (const char* inNameList, 
                 const char* inValue, 
                 const char inDelim)
        {
            int vNumInserted = 0;
            if ( inNameList && inValue )
            {
                char* vValue = XAM_POSIX::xam_strdup(inValue);
                if ( vValue != (char*)0 )
                {
                    char* vStrList = const_cast<char*>(inNameList);
                    char* vBegin = (char*)vStrList;
                    char* vEnd;
                    int vIndex = -1;

                    while ( vBegin )
                    {
                        vEnd = XAM_POSIX::xam_strchr(vBegin, (char)inDelim);
                        if ( vEnd == (char*)0 )
                        {
                            vIndex = put(vBegin, vValue);
                            if ( vIndex == -1 )
                                break;
                            vBegin = (char*)0;
                        }
                        else
                        {
                            *vEnd = (char)0;
                            vIndex = put(vBegin, vValue);
                            if ( vIndex == -1 )
                                break;
                            vBegin = vEnd + 1;
                        }
                        vNumInserted++;
                    }

                    XAM_POSIX::xam_free(vValue);
                }
            }
            return vNumInserted;
        }

        /** 
         * Replace the value at the specified name, 
         * add name/value if not yet present 
         */
        void 
        replace (const char* inName, const char* inValue)
        {
            bool vSet = false;
            for ( int ix=0; ix<mMaxNumberOfAttributes; ix++ )
            {
                if ( mName[ix] ) // skip NULL names
                {
                    const char* vName = mName[ix];
                    if ( XAM_POSIX::xam_strcasecmp(vName, inName) == 0 )
                    {
                        // name found, replace value
                        if ( mValue[ix] )
                            XAM_POSIX::xam_free(mValue[ix]);
                        mValue[ix] = XAM_POSIX::xam_strdup(inValue);
                        vSet = true;
                        break;
                    }
                }
            }
            if ( vSet == false ) // name not found, add value
                put(inName, inValue);
        }

        /** Remove the specified pair name-value */
        int 
        remove (const char* inName)
        {
            int vIndex = -1;
            for ( int ix=0; ix<mMaxNumberOfAttributes; ix++ )
            { 
                if ( mName[ix] )
                {
                    const char* vName = mName[ix];
                    if ( XAM_POSIX::xam_strcasecmp(vName, inName) == 0 )
                    {
                        // free the element
                        XAM_POSIX::xam_free(mName[ix]);
                        if ( mValue[ix] )
                            XAM_POSIX::xam_free(mValue[ix]);

                        // move the higher up elements down
                        for (xam_int jx=ix; jx<mMaxNumberOfAttributes-1; jx++)
                        { 
                            mName[jx] = mName[jx+1];
                            mValue[jx] = mValue[jx+1];
                        }

                        // zero the last element
                        mName[mMaxNumberOfAttributes-1] = 0;
                        mValue[mMaxNumberOfAttributes-1] = 0;
                        mNumberOfAttributes--;
                        vIndex = ix;
                        break;
                    }
                }
            }
            return vIndex;
        }

        /** Return the (const) name at the specified index */
        const char* 
        getName (int inIndex)
        {
            char* vName = ""; 
            if ( (0 <= inIndex) && 
                 (inIndex < mMaxNumberOfAttributes) && (mName[inIndex]) )
            {
                vName = mName[inIndex];
            }

            return vName;
        }

        /** Return the (const) value at the specified index */
        const char* 
        getValue (int inIndex)
        {
            char* vValue = "";
            if ( inIndex >= 0 && inIndex < mMaxNumberOfAttributes )
            {
                if ( mName[inIndex] )
                    vValue = mValue[inIndex];
            }
            return vValue;
        }

        /** 
         * Return the (const) value for the specified name, 
         * or NULL if not found 
         */
        const char* 
        getValue (const char* inName)
        {
            char* vValue = (char*)0;
            for ( int ix=0; ix<mMaxNumberOfAttributes; ix++ )
            {
                if ( mName[ix] ) // skip NULL names
                {
                    const char* vName = mName[ix];
                    if ( XAM_POSIX::xam_strcasecmp(vName, inName) == 0 )
                    {
                        vValue = mValue[ix];
                        break;
                    }
                }
            }
            return vValue;
        }

        /** Return the current number of pairs */
        int 
        size () const
        {
            return mNumberOfAttributes;
        }

        /** Copy each elements of this object to newly allocated space.
         *  caller should take care of the previous allocated space of 
         *  this object
         */
        void 
        makeUnique ()
        {
            char** vName  = (char**)0;
            char** vValue = (char**)0;
            if (mMaxNumberOfAttributes > 0)
            {
                vName = (char**)XAM_POSIX::xam_malloc(mMaxNumberOfAttributes * 
                                                      sizeof(*vName));
                vValue = (char**)XAM_POSIX::xam_malloc(mMaxNumberOfAttributes * 
                                                       sizeof(*vValue));

                if((vName == (char**)0) || (vValue == (char**)0))
                {
                    XAMOutOfMemoryException e;
                    throw e;
                }

                for ( xam_int i=0; i<mMaxNumberOfAttributes; i++ )
                {   
                    if ( mName[i] )
                        vName[i] = (char*)XAM_POSIX::xam_strcpy(
                                   (char*)XAM_POSIX::xam_malloc(
                                   (XAM_POSIX::xam_strlen(mName[i])+1) *
                                   sizeof(*mName[i])), 
                                   mName[i]);
                    else
                        vName[i] = 0;
                    if ( mValue[i] )
                        vValue[i] = (char*)XAM_POSIX::xam_strcpy(
                                    (char*)XAM_POSIX::xam_malloc(
                                    (XAM_POSIX::xam_strlen(mValue[i])+1) *
                                    sizeof(*mValue[i])), 
                                    mValue[i]);
                    else
                        vValue[i] = 0;
                }
            }
            mName = vName;
            mValue = vValue;
        }

        //////////////////////////////////////////////////////////////////////
        // Constructors/Destructor
        //////////////////////////////////////////////////////////////////////
        XAM_AssociativeArray () : 
            mMaxNumberOfAttributes(0), 
            mNumberOfAttributes(0), 
            mName(0), 
            mValue(0)
        {    
        }

        // copy constructor
        XAM_AssociativeArray(const XAM_AssociativeArray& inObj)
        {
           mMaxNumberOfAttributes = inObj.mMaxNumberOfAttributes;
           mNumberOfAttributes = inObj.mNumberOfAttributes;
           mName = const_cast<XAM_AssociativeArray&>(inObj).copyNameArray();
           mValue = const_cast<XAM_AssociativeArray&>(inObj).copyValueArray();
        }

        ~XAM_AssociativeArray ()
        {
            releaseMem();
        }

        ///////////////////////////////////////////////////////////////////////
        // assignment operator
        ///////////////////////////////////////////////////////////////////////
        XAM_AssociativeArray& 
        operator= (XAM_AssociativeArray& inObj)
        {
            if (this != &inObj) // we only need to assign if inObj is not this 
                                // object 
            {
                // Release the memory associated with the object being assigned 
                // to before assigning the new values. If not there could be a 
                // memory leak.
                releaseMem(); 
                
                // now do the assignment
                mMaxNumberOfAttributes = inObj.mMaxNumberOfAttributes;
                mNumberOfAttributes = inObj.mNumberOfAttributes;
                mName = const_cast<XAM_AssociativeArray&>(inObj).copyNameArray();
                mValue = const_cast<XAM_AssociativeArray&>(inObj).copyValueArray();
            }
            return *this;
        }
};

#endif // __XAM_ASSOCIATIVE_ARRAY__







