////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_strings.h"
#include "XAM_Config.h"
#include "XAM_Exception.h"
#include "XAM_Object.h"
#include "XAM_Critical.h"

// The active configuration file path (may be empty string)
xam_string XAM_Config::mConfigFilePath;

// Lock covering the XAM configuration updates
XAM_Mutex XAM_Config::mMutex;

void
XAM_Config::loadFile(const char* inFilePath) 
{
    // The file path must be valid
    if(inFilePath == (char*)0)
    {      
        XAMInvalidParamException e;
        throw e;        
    }

    // Attempt to access the file at this location
    if(XAM_POSIX::xam_access(inFilePath, XAM_F_OK) != 0)
    {
        XAMFileSystemException e;
        throw e;  
    }

    // Load the current file into the properties object
    mProps->readFromFile(inFilePath);    
}

void
XAM_Config::writeToFile(const char* inFilePath) 
{
    // Write out the XAM_Properties object to a file
    mProps->writeToFile(inFilePath);
}

xam_boolean
XAM_Config::readTypeNamespace(char*& inPropName, 
                              xam_string& ioMimeTypeStr)
{
    xam_boolean vFound = false;

    /* Maps supported types to MIME type strings */
    XAM_PrefixTypeMap vMap[] = 
    {
        { "xam_boolean.",     XAM_BOOLEAN_MIME_TYPE }, 
        { "xam_int.",         XAM_INT_MIME_TYPE }, 
        { "xam_double.",      XAM_DOUBLE_MIME_TYPE }, 
        { "xam_xuid.",        XAM_XUID_MIME_TYPE }, 
        { "xam_datetime.",    XAM_DATETIME_MIME_TYPE }, 
        { "xam_string.",      XAM_STRING_MIME_TYPE }
    };

    xam_int vNumTypes = sizeof(vMap)/sizeof(XAM_PrefixTypeMap);
    for (int i = 0; i < vNumTypes; i++)
    {
        xam_int vTypePrefixLen = XAM_POSIX::xam_strlen(vMap[i].sTypePrefix);
        if(XAM_POSIX::xam_strncmp(vMap[i].sTypePrefix, 
                                  inPropName, 
                                  vTypePrefixLen) == 0)
        {
            XAM_POSIX::xam_strcpy(ioMimeTypeStr, vMap[i].sMimeType);
            inPropName += vTypePrefixLen;
            vFound = true;
            break;
        }
    }    

    return vFound;
}

void
XAM_Config::generateBoolProperty(XAM_FieldContainer* inContainer, 
                                 xam_string inName,
                                 xam_boolean inBinding,
                                 xam_boolean inValue)
{
    // Generate a field of the given type and value (ignoring any errors)
    try
    {
        try
        {
            // Set the new value (throws if the field does not exist)
            inContainer->setBooleanHelper(inName, inValue);
        }
        catch(XAMFieldNotFoundException &)
        {
            // Creates the new property 
            inContainer->createBooleanHelper(inName, inBinding, inValue);
        }
    }
    catch(...)
    {
        // Errors are ignored!
    }
}

void
XAM_Config::generateIntProperty(XAM_FieldContainer* inContainer, 
                                xam_string inName,
                                xam_boolean inBinding,
                                xam_int inValue)
{
    // Generate a field of the given type and value (ignoring any errors)
    try
    {
        try
        {
            // Set the new value (throws if the field does not exist)
            inContainer->setIntHelper(inName, inValue);
        }
        catch(XAMFieldNotFoundException &)
        {
            // Creates the new property 
            inContainer->createIntHelper(inName, inBinding, inValue);
        }
    }
    catch(...)
    {
        // Errors are ignored!
    }
}

void
XAM_Config::generateStringProperty(XAM_FieldContainer* inContainer, 
                                   xam_string inName,
                                   xam_boolean inBinding,
                                   xam_string inValue)
{
    // Generate a field of the given type and value (ignoring any errors)
    try
    {
        try
        {
            // Set the new value (throws if the field does not exist)
            inContainer->setStringHelper(inName, inValue);
        }
        catch(XAMFieldNotFoundException &)
        {
            // Creates the new property 
            inContainer->createStringHelper(inName, inBinding, inValue);
        }
    }
    catch(...)
    {
        // Errors are ignored!
    }
}

void
XAM_Config::generateDoubleProperty(XAM_FieldContainer* inContainer, 
                                   xam_string inName,
                                   xam_boolean inBinding,
                                   xam_double inValue)
{
    // Generate a field of the given type and value (ignoring any errors)
    try
    {
        try
        {
            // Set the new value (throws if the field does not exist)
            inContainer->setDoubleHelper(inName, inValue);
        }
        catch(XAMFieldNotFoundException &)
        {
            // Creates the new property 
            inContainer->createDoubleHelper(inName, inBinding, inValue);
        }
    }
    catch(...)
    {
        // Errors are ignored!
    }
}

void
XAM_Config::generateXUIDProperty(XAM_FieldContainer* inContainer, 
                                 xam_string inName,
                                 xam_boolean inBinding,
                                 xam_xuid inValue)
{
    // Generate a field of the given type and value (ignoring any errors)
    try
    {
        try
        {
            // Set the new value (throws if the field does not exist)
            inContainer->setXUIDHelper(inName, inValue);
        }
        catch(XAMFieldNotFoundException &)
        {
            // Creates the new property 
            inContainer->createXUIDHelper(inName, inBinding, inValue);
        }
    }
    catch(...)
    {
        // Errors are ignored!
    }
}

void
XAM_Config::generateDatetimeProperty(XAM_FieldContainer* inContainer, 
                                     xam_string inName,
                                     xam_boolean inBinding,
                                     xam_datetime inValue)
{
    // Generate a field of the given type and value (ignoring any errors)
    try
    {
        try
        {
            // Set the new value (throws if the field does not exist)
            inContainer->setDatetimeHelper(inName, inValue);
        }
        catch(XAMFieldNotFoundException &)
        {
            // Creates the new property 
            inContainer->createDatetimeHelper(inName, inBinding, inValue);
        }
    }
    catch(...)
    {
        // Errors are ignored!
    }
}

void
XAM_Config::getPropertiesByNamespace(xam_string inPattern,
                                     XAM_AssociativeArray& ioAliasMap,
                                     XAM_Array<char>& ioTypeArray)
{
    xam_int vPatternLen = XAM_POSIX::xam_strlen(inPattern);

    // Write all properties matching the pattern to the caller's hashmap
    int vSz = mProps->getSize();
    for(int i=0 ; i < vSz ; i++)
    {
        char* vPropName = const_cast<char*>(mProps->getNameAtIndex(i));

        // Skips any predefined "type" namespaces and updates the pointer
        xam_string vTypeStr;
        XAM_POSIX::xam_strcpy(vTypeStr, XAM_STRING_MIME_TYPE);
        readTypeNamespace(vPropName, vTypeStr); 

        // Handle matching patterns
        if(XAM_POSIX::xam_strncmp(inPattern, vPropName, vPatternLen) == 0)
        {
            const char* vPropValue = mProps->getValueAtIndex(i);
            ioAliasMap.put(vPropName, vPropValue);

            // Add the type to the array at this index
            char* vTypeBuffer = (char*)XAM_POSIX::xam_malloc(XAM_POSIX::xam_strlen(vTypeStr) + 1);
            XAM_POSIX::xam_strcpy(vTypeBuffer, vTypeStr);
            ioTypeArray.push_back(vTypeBuffer);
        }
    }
}

void
XAM_Config::getPropertiesByType(xam_string inMimeType,
                                XAM_AssociativeArray& ioAliasMap)
{
    // Write all properties matching the pattern to the caller's hashmap
    int vSz = mProps->getSize();
    for(int i=0 ; i < vSz ; i++)
    {
        char* vPropName = const_cast<char*>(mProps->getNameAtIndex(i));

        // Determines the MIME type and increments the pointer (if needed)
        // If not specified, the type is xam_string!
        xam_string vTypeStr;
        XAM_POSIX::xam_strcpy(vTypeStr, XAM_STRING_MIME_TYPE);
        readTypeNamespace(vPropName, vTypeStr); 

        // Add matching property MIME types to the caller's map
        if(XAM_POSIX::xam_strcmp(inMimeType, vTypeStr) == 0)
        {
            const char* vPropValue = mProps->getValueAtIndex(i);
            ioAliasMap.put(vPropName, vPropValue);
        }
    }
}

void
XAM_Config::getPropertiesByTypeAndNamespace(xam_string inMimeType,
                                            xam_string inPattern,
                                            XAM_AssociativeArray& ioAliasMap)
{
    xam_int vPatternLen = XAM_POSIX::xam_strlen(inPattern);

    // Write all properties matching the pattern to the caller's hashmap
    int vSz = mProps->getSize();
    for(int i=0 ; i < vSz ; i++)
    {
        char* vPropName = const_cast<char*>(mProps->getNameAtIndex(i));

        // Determines the MIME type and increments the pointer (if needed)
        // If not specified, the type is xam_string!
        xam_string vTypeStr;
        XAM_POSIX::xam_strcpy(vTypeStr, XAM_STRING_MIME_TYPE);
        readTypeNamespace(vPropName, vTypeStr); 

        // Handle matching patterns
        if(XAM_POSIX::xam_strncmp(inPattern, vPropName, vPatternLen) == 0)
        {
            // Add matching property MIME types to the caller's map
            if(XAM_POSIX::xam_strcmp(inMimeType, vTypeStr) == 0)
            {
                const char* vPropValue = mProps->getValueAtIndex(i);
                ioAliasMap.put(vPropName, vPropValue);                
            }
        }
    }
}

void
XAM_Config::applyAndRemoveLogStateControls(XAM_FieldContainer* inContainer)
{
    // Cache logging properties
    XAM_AssociativeArray vHashMap;
    XAM_Array<char> vTypeArray(10, 10, true, kXA_USE_FREE); // destruct elems with free
    getPropertiesByNamespace(".xam.log", vHashMap, vTypeArray);
    int vSz = vHashMap.size();
    for(int i=0 ; i < vSz ; i++)
    {
        try
        {
            // Get the property name/value
            char* vPropNamePtr = const_cast<char*>(vHashMap.getName(i));
            char* vPropValuePtr = const_cast<char*>(vHashMap.getValue(i));

            // Get the property type (strips type prefixes)
            xam_string vTypeStr;
            XAM_POSIX::xam_strcpy(vTypeStr, vTypeArray[i]);

            // Cache the name/value
            xam_string vName;
            XAM_POSIX::xam_strcpy(vName, vPropNamePtr);
            xam_string vValue; 
            XAM_POSIX::xam_strcpy(vValue, vPropValuePtr);

            // Generate a property field by type
            if(XAM_POSIX::xam_strcmp(vTypeStr, XAM_BOOLEAN_MIME_TYPE) == 0)
            {
                // Generate xam_boolean logging property
                if(XAM_POSIX::xam_strcasecmp(vValue, "false") == 0)
                    generateBoolProperty(inContainer, vName, false, false);
                else
                    generateBoolProperty(inContainer, vName, false, true);
                mProps->remove(vName);
            }
            else if(XAM_POSIX::xam_strcmp(vTypeStr, XAM_INT_MIME_TYPE) == 0)
            {
                // Generate xam_int logging property
                generateIntProperty(inContainer, vName, false, XAM_POSIX::xam_atoi(vValue));
                mProps->remove(vName);
            }
            else if(XAM_POSIX::xam_strcmp(vTypeStr, XAM_STRING_MIME_TYPE) == 0)
            {       
                // Generate xam_string logging property
                generateStringProperty(inContainer, vName, false, vValue);
                mProps->remove(vName);
            }
        }
        catch(...)
        {
            // Ignore errors
        }
    }
}

void
XAM_Config::generatePropertyFields(XAM_FieldContainer* inContainer)
{
    // Only generate fields if we found properties
    if(mProps->getSize() > 0)
    {
        // Apply logging fields first, so that config files can be logged
        applyAndRemoveLogStateControls(inContainer);

        XAM_LOG_DEBUG(component_xam_debug, 
                      "******* Start XAMLibrary Apply Configuration Updates *******");

        // The config path property value is not allowed, so 
        // just remove it if someone tries to use it!
        mProps->remove((char*)XAM_CONFIG_PATH);

        // Create the XAM_CONFIG_PATH property field
        if(XAM_POSIX::xam_strcmp(mConfigFilePath, "") != 0)
        {
            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "  CONFIG [xam_string] %s = \"%s\"", XAM_CONFIG_PATH, mConfigFilePath);
        }

        // Create property fields of all supported stypes
        generateBooleanPropertyFields(inContainer);
        generateIntPropertyFields(inContainer);
        generateStringPropertyFields(inContainer);
        generateDoublePropertyFields(inContainer);
        generateDatetimePropertyFields(inContainer);

        // Can't support this unless we add base64 encoding/decoding services to the lib
        // (as XUID strings are base64 encoded and need to be decoded into binary format)
        //generateXUIDPropertyFields(inContainer);

        XAM_LOG_DEBUG(component_xam_debug, 
                      "******* End XAMLibrary Apply Configuration Updates *******");
    }
}

void 
XAM_Config::generateStringPropertyFields(XAM_FieldContainer* inContainer)
{
    xam_string vTypeStr;
    XAM_POSIX::xam_strcpy(vTypeStr, XAM_STRING_MIME_TYPE);

    // Cache xam_string properties
    XAM_AssociativeArray vHashMap;
    getPropertiesByType(vTypeStr, vHashMap);
    int vSz = vHashMap.size();
    for(int i=0 ; i < vSz ; i++)
    {
        xam_string vName;
        xam_string vValue;
        try
        {
            XAM_POSIX::xam_strcpy(vName, vHashMap.getName(i)); 
            XAM_POSIX::xam_strcpy(vValue, vHashMap.getValue(i));

            generateStringProperty(inContainer, vName, false, vValue);

            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "  CONFIG [xam_string] %s = \"%s\"", vName, vValue);
        }
        catch(...)
        {
            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "   ERROR [xam_string] %s = \"%s\"", vName, vValue);
        }
    }
}

void 
XAM_Config::generateDoublePropertyFields(XAM_FieldContainer* inContainer)
{
    xam_string vTypeStr;
    XAM_POSIX::xam_strcpy(vTypeStr, XAM_DOUBLE_MIME_TYPE);

    // Cache xam_double properties
    XAM_AssociativeArray vHashMap;
    getPropertiesByType(vTypeStr, vHashMap);
    int vSz = vHashMap.size();
    for(int i=0 ; i < vSz ; i++)
    {
        xam_string vName;
        const char* vValStr;
        xam_double vDoubleVal = 0.0;
        try
        {
            XAM_POSIX::xam_strcpy(vName, vHashMap.getName(i));
            vValStr = vHashMap.getValue(i);

            // Convert string to double
            xam_sscanf(vValStr, "%lf", &vDoubleVal);

            // Check for invalid conversion
            if((vDoubleVal == 0.0) && (!XAM_POSIX::xam_strstr(vValStr, "0")))
            {
                XAMInvalidParamException e;
                throw e;
            }

            generateDoubleProperty(inContainer, vName, false, vDoubleVal);

            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "  CONFIG [xam_double] %s = \"%lf\"", vName, vDoubleVal);
        }
        catch(...)
        {
            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "   ERROR [xam_double] %s = \"%s\"", vName, vValStr);
        }
    }
}

void 
XAM_Config::generateDatetimePropertyFields(XAM_FieldContainer* inContainer)
{
    xam_string vTypeStr;
    XAM_POSIX::xam_strcpy(vTypeStr, XAM_DATETIME_MIME_TYPE);

    // Cache xam_datetime properties
    XAM_AssociativeArray vHashMap;
    getPropertiesByType(vTypeStr, vHashMap);
    int vSz = vHashMap.size();
    for(int i=0 ; i < vSz ; i++)
    {
        xam_string vName;
        xam_string vValue; 
        try
        {
            XAM_POSIX::xam_strcpy(vName, vHashMap.getName(i));
            XAM_POSIX::xam_strcpy(vValue, vHashMap.getValue(i));
            generateDatetimeProperty(inContainer, vName, false, vValue);

            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "  CONFIG [xam_datetime] %s = \"%s\"", vName, vValue);
        }
        catch(...)
        {
            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "   ERROR [xam_datetime] %s = \"%s\"", vName, vValue);
        }
    }
}

void 
XAM_Config::generateIntPropertyFields(XAM_FieldContainer* inContainer)
{
    xam_string vTypeStr;
    XAM_POSIX::xam_strcpy(vTypeStr, XAM_INT_MIME_TYPE);

    // Cache xam_int properties
    XAM_AssociativeArray vHashMap;
    getPropertiesByType(vTypeStr, vHashMap);
    int vSz = vHashMap.size();
    for(int i=0 ; i < vSz ; i++)
    {
        xam_string vName;
        xam_string vValue;
        xam_int vIntVal = 0;
        try
        {
            XAM_POSIX::xam_strcpy(vName, vHashMap.getName(i));
            XAM_POSIX::xam_strcpy(vValue, vHashMap.getValue(i));
            vIntVal = XAM_POSIX::xam_atoi(vValue);

            // Handle conversion errors
            if((vIntVal == 0) && (XAM_POSIX::xam_strcmp(vValue, "0") != 0)) 
            {
                XAMInvalidParamException e;                
                throw e;
            }

            generateIntProperty(inContainer, vName, false, vIntVal);

            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "  CONFIG [xam_int] %s = \"%"XAM_INT_FORMAT"d\"", vName, vIntVal);
        }
        catch(...)
        {
            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "   ERROR [xam_int] %s = \"%s\"", vName, vValue);
        }
    }
}

void 
XAM_Config::generateBooleanPropertyFields(XAM_FieldContainer* inContainer)
{
    xam_string vTypeStr;
    XAM_POSIX::xam_strcpy(vTypeStr, XAM_BOOLEAN_MIME_TYPE);

    // Cache xam_boolean properties
    XAM_AssociativeArray vHashMap;
    getPropertiesByType(vTypeStr, vHashMap);
    int vSz = vHashMap.size();
    for(int i=0 ; i < vSz ; i++)
    {
        xam_string vName;
        xam_string vValue;
        try
        {
            XAM_POSIX::xam_strcpy(vName, vHashMap.getName(i));
            XAM_POSIX::xam_strcpy(vValue, vHashMap.getValue(i));

            // The value is true for all inputs except "false" and "" (empty string)
            xam_boolean vBoolVal = true;
            if((XAM_POSIX::xam_strcasecmp(vValue, "false") == 0) ||
               (XAM_POSIX::xam_strcmp(vValue, "") == 0))
                vBoolVal = false;

            generateBoolProperty(inContainer, vName, false, vBoolVal);

            XAM_LOG_DEBUG_P3(component_xam_debug, 
                             "  CONFIG [xam_boolean] %s = \"%s\"", 
                             vName, 
                             vBoolVal ? "true" : "false");
        }
        catch(...)
        {
            XAM_LOG_DEBUG_P3(component_xam_debug, 
                          "   ERROR [xam_boolean] %s = \"%s\"", vName, vValue);
        }
    }
}

void
XAM_Config::setConfigFilePath(const char* inPath)
{
    if(inPath)
    {
        {
            // Only 1 thread/process may update the config at a time
            XAM_CriticalSection vCritical(&mMutex);

            // Use this configration file
            XAM_POSIX::xam_strcpy(mConfigFilePath, inPath);
        }

        // Generate properties
        XAM_Config vConfig(mConfigFilePath);
        vConfig.generatePropertyFields(XAM_Object::getInstance());
    }
}

const char*
XAM_Config::getConfigFilePath()
{
    const char* vConfigPath = (const char*)0;

    // Only 1 thread/process may access the config path at a time
    XAM_CriticalSection vCritical(&mMutex);

    if(XAM_POSIX::xam_strcmp(mConfigFilePath, "") != 0)
        vConfigPath = mConfigFilePath;

    return vConfigPath;
}

XAM_Config::XAM_Config(char* inFilePath) 
{
    // Only 1 thread/process may update the config at a time
    XAM_CriticalSection vCritical(&mMutex);

    mProps = new XAM_Properties("XAM_Config");

    XAM_POSIX::xam_strcpy(mConfigFilePath, inFilePath ? inFilePath : "");

    if(inFilePath != (char*)0)
    {
        // Load the caller specified configuration file
        this->loadFile(inFilePath);
        XAM_POSIX::xam_strcpy(mConfigFilePath, inFilePath);
    }
    else
    {
        // Check env var for properties file path (and load if found)
        const char* vConfigEnvVarValue = XAM_POSIX::xam_getenv(XAM_CONFIG_PATH_ENV_VAR_NAME);
        if(vConfigEnvVarValue && (XAM_POSIX::xam_access(vConfigEnvVarValue, XAM_F_OK) == 0))
        {
            // Load the config file defined by the environment variable
            this->loadFile(vConfigEnvVarValue);
            XAM_POSIX::xam_strcpy(mConfigFilePath, vConfigEnvVarValue);
        }
        else if(XAM_POSIX::xam_access(XAM_DEFAULT_CONFIG_PATH, XAM_F_OK) == 0)
        {
            // Load the default configuration file (if found)
            this->loadFile(XAM_DEFAULT_CONFIG_PATH);
            XAM_POSIX::xam_strcpy(mConfigFilePath, XAM_DEFAULT_CONFIG_PATH);
        }
    }
}

XAM_Config::~XAM_Config() 
{
    delete mProps;
}



