#ifndef __XAM_CONFIG__
#define __XAM_CONFIG__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_Properties.h"
#include "XAM_FieldContainer.h"

#define XAM_CONFIG_PATH_ENV_VAR_NAME  "XAM_CONFIG_PATH"
#define XAM_DEFAULT_CONFIG_PATH       "xam.properties"

/**
 * A class that encapsulates all XAM Library configuration information and 
 * provides serialization/de-serialization (save/open) to and from a 
 * properties file.
 */
class XAM_Config 
{
    private:
         XAM_FieldContainer* mParent;  // Pointer to the parent object for this config
         XAM_Properties* mProps;       // Properties (name/value pairs in hashmap)

         static XAM_Mutex mMutex;             // Protects against concurrent config updates
         static xam_string mConfigFilePath;   // Config file path

        /** 
         * Struct for mapping supported type prefix strings to MIME type strings 
         */
        typedef struct 
        {
            xam_string sTypePrefix;
            const char* sMimeType;
        } XAM_PrefixTypeMap;

        /**
         * Checks the first few characters if the property string and
         * obtains the MIME type to use while incrementing the input string
         * accordingly.
         *
         * @param inPropName Reference to a pointer to the string
         * @param ioMimeTypeStr Reference to a xam_string where the MIME type is written
         * @return true if in a type namespace, false otherwise.
         */
        xam_boolean
        readTypeNamespace(char*& inPropName, 
                          xam_string& ioMimeTypeStr);


        void 
        generateBooleanPropertyFields(XAM_FieldContainer* inContainer);

        void 
        generateIntPropertyFields(XAM_FieldContainer* inContainer);

        void 
        generateStringPropertyFields(XAM_FieldContainer* inContainer);

        void 
        generateDoublePropertyFields(XAM_FieldContainer* inContainer);

        void 
        generateDatetimePropertyFields(XAM_FieldContainer* inContainer);

        //void 
        //generateXUIDPropertyFields(XAM_FieldContainer* inContainer);

        void
        applyAndRemoveLogStateControls(XAM_FieldContainer* inContainer);

        /**
        * Loads all parameters from the given file.
        * Deserializes/saves an XAM_LogState properties file.
        *
        * @throws XAM_ParameterException if the inFilePath is null.
        * @throws XAM_FileSystemException if the inFilePath can't be read
        */
        void 
        loadFile(const char* inFilePath);

        /**
        * Serializes/creates an XAM_LogState properties file
        * based on the current state of this object.
        */
        void 
        writeToFile(const char* inFilePath);

        /**
        * Writes all property name/value pairs under a certain namespace 
        * to the caller supplied hash map.
        *
        * @param inPattern The namespace prefix for selected properties
        * @param ioAliasMap The hashmap provided by the caller which will
        *                   be populated with any properties matching the
        *                   namespace pattern.
        * @param ioTypeArray The property type for each index is written here
        */
        void
        getPropertiesByNamespace(xam_string inPattern,
                                 XAM_AssociativeArray& ioAliasMap,
                                 XAM_Array<char>& ioTypeArray);

        /**
        * Writes all property name/value pairs of a certain MIME type 
        * to the caller supplied hash map.
        *
        * @param inMimeType The MIME type string for selected properties
        * @param ioAliasMap The hashmap provided by the caller which will
        *                   be populated with any properties matching the
        *                   MIME type.
        */
        void
        getPropertiesByType(xam_string inMimeType,
                            XAM_AssociativeArray& ioAliasMap);

        /**
        * Writes all property name/value pairs under a certain namespace 
        * and of a certain MIME type to the caller supplied hash map.
        *
        * @param inMimeType The MIME type string for selected properties
        * @param inPattern The namespace prefix for selected properties
        * @param ioAliasMap The hashmap provided by the caller which will
        *                   be populated with any properties matching the
        *                   namespace pattern.
        */
        void
        getPropertiesByTypeAndNamespace(xam_string inMimeType,
                                        xam_string inPattern,
                                        XAM_AssociativeArray& ioAliasMap);

        /**
         * Property generator fields
         *
         * These methods create a property field if it doesn't exist
         * or sets a property value if it does. All errors are ignored.
         */

        static void
        generateBoolProperty(XAM_FieldContainer* inContainer, 
                             xam_string inName,
                             xam_boolean inBinding,
                             xam_boolean inValue);
        static void
        generateIntProperty(XAM_FieldContainer* inContainer, 
                            xam_string inName,
                            xam_boolean inBinding,
                            xam_int inValue);
        static void
        generateStringProperty(XAM_FieldContainer* inContainer, 
                               xam_string inName,
                               xam_boolean inBinding,
                               xam_string inValue);
        static void
        generateDoubleProperty(XAM_FieldContainer* inContainer, 
                               xam_string inName,
                               xam_boolean inBinding,
                               xam_double inValue);
        static void
        generateXUIDProperty(XAM_FieldContainer* inContainer, 
                             xam_string inName,
                             xam_boolean inBinding,
                             xam_xuid inValue);
        static void
        generateDatetimeProperty(XAM_FieldContainer* inContainer, 
                                 xam_string inName,
                                 xam_boolean inBinding,
                                 xam_datetime inValue);
    public:       

        /**
         * Get the active config file path
         *
         * @return The full path to the file, or NULL if no config file 
         *         is being utilized
         */
        static const char*
        getConfigFilePath();

        /**
         * Set the active config file path, forcing a load of all 
         * properties contained within the file (if it exists).
         *
         * @return The full path to the file, or NULL if no config file 
         *         is being utilized
         */
        static void
        setConfigFilePath(const char* inPath);

        /**
        * Creates property fields on the parent field container 
        * corresponding to the property name/value pairs in the 
        * configuration file.
        *
        * @param inContainer The field container where property fields 
        *                    will be created/updated.
        */
        void
        generatePropertyFields(XAM_FieldContainer* inContainer);

        /**
        * Constructor - Loads an existing log state configuration file
        *
        *  Config File Precedence:          
        *     1. Any specified config file paths passed to this method
        *     2. The XAM_CONFIG_PATH_ENV_VAR_NAME environment variable
        *     3. The XAM_DEFAULT_CONFIG_PATH
        *          
        * @param inFilePath Full path to a config properties file on disk.
        *                   If NULL, the XAM_CONFIG_PATH_ENV_VAR_NAME will 
        *                   be checked and the referenced properties file 
        *                   will be loaded. If the env var is not set, the 
        *                   default config file XAM_DEFAULT_CONFIG_PATH in 
        *                   the working directory will be loaded (if found).
        */
        XAM_Config(char* inFilePath = 0); 

        /**
        * Destructor
        */
        ~XAM_Config();
};


#endif // __XAM_CONFIG__


