#ifndef __XAM_EXCEPTION_H_
#define __XAM_EXCEPTION_H_
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_errors.h"

#include "XAM_POSIX.h"
#include "XAM_Utils.h"

/** 
 *  Base class for all XAM related exceptions
 */
class XAM_Exception
{
    protected:    
         xam_string mMsg;         /* Exception message */
         xam_status mStatusCode;  /* Exception status code */

    public:

         /** 
          * Constructor
          *
          * @param inStatusCode The status code for this exception
          */
         XAM_Exception(xam_status inStatusCode)
         {
             // Clear the exception message
             XAM_POSIX::xam_strcpy(mMsg, "");
             mStatusCode = inStatusCode;
         }

         /** 
          * Constructor for specifying an exception message
          *
          * @param inStatusCode The status code for this exception
          * @param inMsg The message to set for this exception
          */
         XAM_Exception(xam_status inStatusCode, const char* inMsg)
         {
             // Clear the exception message
             XAM_POSIX::xam_strcpy(mMsg, inMsg);
             mStatusCode = inStatusCode;
         }

         /**
          * Retrieve the exception message
          *
          * @return The exception message
          */
         const char* 
         getMessage() 
         { 
             return mMsg ; 
         }

         /**
          * Set an exception message
          *
          * @param inMsg The message to apply to this exception
          */
         void 
         setMessage(const char* inMsg, ...)
         {
            va_list ap;
            int vResult = 0;

            va_start (ap, inMsg);
            vResult = XAM_POSIX::xam_vsprintf (mMsg, inMsg, ap);
            va_end (ap);
         }

         /**
          * Get the status code associated with the exception
          *
          * @return The exception status code
          */
         xam_status 
         getStatusCode() 
         { 
             return mStatusCode; 
         }
};

/**
 * Out of memory exception
 */
class XAMOutOfMemoryException : public XAM_Exception
{
    public:
    XAMOutOfMemoryException() : XAM_Exception(XAM_OUT_OF_MEMORY) {}
};

/**
 * VIM not found exception
 */
class XAMVimNotFoundException : public XAM_Exception
{
    public:
    XAMVimNotFoundException() : XAM_Exception(XAM_VIM_NOT_FOUND) {}    
};

/**
 * Symbol not found exception
 */
class XAMSymbolNotFoundException : public XAM_Exception
{
    public:
    XAMSymbolNotFoundException() : XAM_Exception(XAM_VIM_SYMBOL_NOT_FOUND) {}    
};

/**
 * Param not UTF-8 exception
 */
class XAMParamNotUTF8Exception : public XAM_Exception
{
    public:
    XAMParamNotUTF8Exception() : XAM_Exception(XAM_PARAM_NOT_UTF8) {}    
};

/**
 * Field not found exception
 */
class XAMFieldNotFoundException : public XAM_Exception
{
    public:
    XAMFieldNotFoundException() : XAM_Exception(XAM_FIELD_NOT_FOUND) {}    
};

/**
 * Invalid handle exception
 */
class XAMInvalidHandleException : public XAM_Exception
{
    public:
    XAMInvalidHandleException() : XAM_Exception(XAM_INVALID_HANDLE) {}    
};

/**
 * Read only field exception
 */
class XAMReadOnlyFieldException: public XAM_Exception
{
    public:
    XAMReadOnlyFieldException() : XAM_Exception(XAM_FIELD_IS_READ_ONLY) {}
};

/**
 * Invalid parameter exception
 */
class XAMInvalidParamException: public XAM_Exception
{
    public:
    XAMInvalidParamException() : XAM_Exception(XAM_INVALID_PARAM) {}
};

/**
 * Filesystem error exception
 */
class XAMFileSystemException: public XAM_Exception
{
    public:
    XAMFileSystemException() : XAM_Exception(XAM_FILESYSTEM_ERROR) {}
};

/**
 * Field already exists exception
 */
class XAMFieldExistsException: public XAM_Exception
{
    public:
    XAMFieldExistsException() : XAM_Exception(XAM_FIELD_EXISTS) {}
};

/**
 * Invalid mime type exception
 */
class XAMInvalidMimeTypeException : public XAM_Exception
{
    public:
    XAMInvalidMimeTypeException() : XAM_Exception(XAM_INVALID_MIME_TYPE) {}
};

/**
 * Invalid stream mode exception
 */
class XAMInvalidStreamModeException : public XAM_Exception
{
    public:
    XAMInvalidStreamModeException() : XAM_Exception(XAM_INVALID_XSTREAM_MODE) {}
};

/**
 * Invalid XRI exception
 */
class XAMInvalidXRIException : public XAM_Exception
{
    public:
    XAMInvalidXRIException() : XAM_Exception(XAM_INVALID_XRI) {}
};

/**
 * The asynchronous operation is pending exception
 */
class XAMAsyncOpPendingException : public XAM_Exception
{
    public:
    XAMAsyncOpPendingException() : XAM_Exception(XAM_XASYNC_PENDING) {}
};

/**
 * Abandoned object exception
 */
class XAMAbandonedException : public XAM_Exception
{
    public:
    XAMAbandonedException(XAMObjType inType) : XAM_Exception(XAM_UNKNOWN_ERROR) 
    {
        switch(inType)
        {
        case XSTREAM_HANDLE:
            mStatusCode = XAM_XSTREAM_ABANDONED;
            break;
        case XSYSTEM_HANDLE:
            mStatusCode = XAM_XSYSTEM_ABANDONED;
            break;
        case XSET_HANDLE:
            mStatusCode = XAM_XSET_ABANDONED;
            break;
        }
    }
};

/**
 * Corrupt object exception
 */
class XAMCorruptException : public XAM_Exception
{
    public:
    XAMCorruptException(XAMObjType inType) : XAM_Exception(XAM_UNKNOWN_ERROR) 
    {
        switch(inType)
        {
        case XSTREAM_HANDLE:
            mStatusCode = XAM_XSTREAM_CORRUPT;
            break;
        case XSYSTEM_HANDLE:
            mStatusCode = XAM_XSYSTEM_CORRUPT;
            break;
        case XSET_HANDLE:
            mStatusCode = XAM_XSET_CORRUPT;
            break;
        }
    }
};

/**
 * Field is in use exception
 */
class XAMFieldInUseException : public XAM_Exception
{
    public:
    XAMFieldInUseException() : XAM_Exception(XAM_FIELD_IN_USE) {}
};

/**
 * Invalid Field Name
 */
class XAMInvalidFieldNameException : public XAM_Exception
{
    public:
    XAMInvalidFieldNameException() : XAM_Exception(XAM_INVALID_FIELD_NAME) {}
};

/**
 * Max field limit exceeded
 */
class XAMMaxFieldLimitExceededException : public XAM_Exception
{
    public:
    XAMMaxFieldLimitExceededException() : XAM_Exception(XAM_MAX_FIELDS_EXCEEDED) {}
};

/**
 * Operation Not Allowed
 */
class XAMOperationNotAllowedException : public XAM_Exception
{
    public:
        XAMOperationNotAllowedException() : XAM_Exception(XAM_OPERATION_NOT_ALLOWED) {}
};

/**
 * XAsync operation halted
 */
class XAMXAsyncHaltedException : public XAM_Exception
{
    public:
    XAMXAsyncHaltedException() : XAM_Exception(XAM_XASYNC_HALTED) {}
};

#endif // __XAM_EXCEPTION_H_


