////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"

#include "XAM_Properties.h"
#include "XAM_Exception.h"
#include "XAM_Utils.h"

#define NULL_STR  "<NULL>"

xam_boolean 
XAM_Properties::set(const char* inName, const char* inValue) 
{
    xam_boolean vRetVal = (xam_boolean)false;

    // Get value string (or use NULL_STR if null)
    const char* vVal = inValue;
    if(inValue == (char*)0)
        vVal = NULL_STR; 

    // Apply the new value
    // (Scope the lock to the replace operation only)
    {
        XAM_PropertiesLocker(this);
        mHashTable.replace(inName, vVal);
    }

    // Determine success
    xam_string vReadVal;
    xam_boolean vSuccess = getValue(inName, &vReadVal);
    if(vSuccess && (XAM_POSIX::xam_strcmp(vReadVal, vVal) == 0))
    {
        vRetVal = (xam_boolean)true;
    }

    return vRetVal;
}

xam_boolean 
XAM_Properties::set(const char* inName, xam_int inValue) 
{
    // Convert int to string and use the other set
    return set(inName, (char*)XAM_Utils::toString(inValue));
}

xam_boolean 
XAM_Properties::remove(char* inName) 
{
    xam_boolean vRetVal = (xam_boolean)false;

    // Remove this name
    // (Scope the lock to the remove operation only)
    {
        XAM_PropertiesLocker(this);
	    mHashTable.remove(inName);	
    }

    // Determine success
    xam_string vVal;
    if(!getValue(inName, &vVal))
        vRetVal = (xam_boolean)true;

    return vRetVal;
}

void 
XAM_Properties::clear() 
{
    // Remove all elements
    XAM_PropertiesLocker(this);
    for (int ix=mHashTable.size()-1; ix>=0; ix--)
    {
        const char* x = mHashTable.getName(ix);
        mHashTable.remove(x);
    }
}

xam_boolean 
XAM_Properties::getValue(const char* inName, xam_string* const outString) 
{
    xam_boolean vRetVal = false;

    // Get the value for this name
    XAM_PropertiesLocker(this);
    const char* vFound = mHashTable.getValue(inName);
    if(vFound != (char*)0)
    {
       if((XAM_POSIX::xam_strcasecmp(vFound, "") == 0) || 
          (XAM_POSIX::xam_strcasecmp(vFound, NULL_STR) == 0))
       {
           vRetVal = false;
       }
       else
       {
           XAM_POSIX::xam_strcpy(*outString, vFound);
           vRetVal = true;
       }
    }

    return vRetVal;
}

xam_int 
XAM_Properties::getIntValue(const char* inName) 
{
    // Convert the value to an int before returning
    xam_int vRetVal = -1;
    xam_string vVal;
    xam_boolean vSuccess = getValue(inName, &vVal);
    if(vSuccess)
    {
        vRetVal = XAM_Utils::toInt(vVal);
    }

    return vRetVal;
}

int 
XAM_Properties::getSize() 
{
    return mHashTable.size();
}

xam_boolean 
XAM_Properties::writeToFile(const char* inFilePath)
{
    xam_boolean vRetVal = (xam_boolean)false;

    // Open the file
    FILE* vFilePtr = XAM_POSIX::xam_fopen(inFilePath, "w");
    if(vFilePtr == (FILE*)0)
    {
        // Failed to open the file
        XAMFileSystemException e;
        throw e;
    }
    else
    {
        // Write the name/value pairs to the file
        char* vProp = toString();
        XAM_POSIX::xam_fwrite (vProp, sizeof (char), 
                               XAM_POSIX::xam_strlen(vProp), 
                               vFilePtr);

        XAM_POSIX::xam_fclose (vFilePtr);
        XAM_POSIX::xam_free(vProp);

        vRetVal = true;
    }

    return vRetVal;
}


xam_boolean 
XAM_Properties::readFromFile(const char* inFilePath) 
{
    xam_boolean vRetVal = (xam_boolean)false;

    // Do we have access to the file?
    if(XAM_POSIX::xam_access(inFilePath, XAM_F_OK) != 0)
    {
        XAMFileSystemException e;
        throw e;
    }

    // Open the file, handle errors.
    FILE* vFilePtr = XAM_POSIX::xam_fopen(inFilePath, "r");
    if(vFilePtr == (FILE*)0)
    {
        XAMFileSystemException e;
        throw e;
    }
    else 
    {
        // Delete any existing name/value pairs
        clear();

        // Lock for the duration of this read
        XAM_PropertiesLocker(this);

        int vStringSize = 1024*5;
        char* vString = (char*) XAM_POSIX::xam_malloc(vStringSize);
        if(!vString)
        {
            XAMOutOfMemoryException e;
            throw e;
        }

        XAM_POSIX::xam_strcpy(vString, "");

        const int vSize = 1024; // Arbitrary read size
        char vBuff[vSize] = "";
        vBuff[vSize - 1] = '\0'; // paranoia
        xam_int vBytesRead  = 0;

        // Read the file into a string buffer
        do{
            vBytesRead = XAM_POSIX::xam_fread(vBuff, 1, vSize - 1, vFilePtr);
            vBuff[vBytesRead] = '\0';

            if((int)(XAM_POSIX::xam_strlen(vString) + vSize) > (vStringSize - 1))
            {
                vStringSize *= 2;
                vString = (char*)XAM_POSIX::xam_realloc(vString, vStringSize);
            }

            if(vBytesRead > 0){
                XAM_POSIX::xam_strcat(vString, vBuff);
            }
        }
        while(!feof(vFilePtr) && (vBytesRead > 0));

        XAM_POSIX::xam_fclose(vFilePtr);
        vRetVal = true;

        parsePropertiesFile(vString);

        if(vString)
            XAM_POSIX::xam_free(vString);
    }

    return vRetVal;
}

void XAM_Properties::parsePropertiesFile(char* vString)
{
    // Parse the buffer into properties
    char *vLineData, *vNameStr, *vValueStr;
    char *vLastNewLine, *vLastEquals, *vNameToValueStr;
    try
    {
        vLineData = vString;
        for (int j = 1; ; j++) 
        {
            // Tokenize on new lines
            vNameStr = XAM_POSIX::xam_strtok_r(vLineData, "\n", 
                &vLastNewLine);
            if (vNameStr == (char*)0)
                break;
            else if(XAM_POSIX::xam_strncmp(vNameStr, "#", 1) == 0) {
                // Skip commented lines
                vLineData = (char*)0;
                continue;
            }
            else
                vLineData = (char*)0;

            char* vName = (char*)0;
            char* vValue = (char*)0;

            // Tokenize on equals '='
            for (vNameToValueStr = vNameStr; ; vNameToValueStr = (char*)0) 
            {
                vValueStr = XAM_POSIX::xam_strtok_r(vNameToValueStr, 
                                                    "=", 
                                                    &vLastEquals);
                if (vValueStr == (char*)0)
                    break;

                if(vName == (char*)0) {    
                    vName = vValueStr;   
                }
                else if(vValue == (char*)0) {
                    vValue = vValueStr;                
                }

                if((vName != (char*)0) && (vValue != (char*)0)) {
                    mHashTable.replace(vName, vValue);                
                    vName = (char*)0;
                    vValue = (char*)0;
                }
            }
        }
    }
    catch(XAM_Exception&)
    {
        if(vString)
            XAM_POSIX::xam_free(vString);

        throw;
    }
}

char* 
XAM_Properties::toString() 
{
    char* vRetVal = (char*)0;
    int vSz = getSize();
    if(vSz > 0) 
    {
        vRetVal = (char*)XAM_POSIX::xam_malloc(1024*((size_t)vSz));
        if ( vRetVal == (char*)0 )
        { 
            XAMOutOfMemoryException e;
            throw e;
        }
        else
        {
            // Write the header
            char* vSeparator = "##################################";
            XAM_POSIX::xam_strcpy(vRetVal, vSeparator);
            XAM_POSIX::xam_strcat(vRetVal, "\n# ");
            XAM_POSIX::xam_strcat(vRetVal, mName);
            XAM_POSIX::xam_strcat(vRetVal, "\n");
            XAM_POSIX::xam_strcat(vRetVal, vSeparator);
            XAM_POSIX::xam_strcat(vRetVal, "\n\n");

            // Lock for the rest of the write
            XAM_PropertiesLocker(this);

            // Add each name/value pair to the buffer
            for(int i=0 ; i < vSz ; i++) {
                XAM_POSIX::xam_strcat(vRetVal, (char*)mHashTable.getName(i));
                XAM_POSIX::xam_strcat(vRetVal, "=");
                XAM_POSIX::xam_strcat(vRetVal, (char*)mHashTable.getValue(i));
                XAM_POSIX::xam_strcat(vRetVal, "\n");
            }
        }
    }

    return vRetVal;
}

XAM_Properties::XAM_Properties(char* inPropertiesName) 
{
    if(inPropertiesName != (char*)0)
        mName = XAM_POSIX::xam_strdup(inPropertiesName);
    else
        mName = XAM_POSIX::xam_strdup("XAM_Properties");
}

XAM_Properties::~XAM_Properties() 
{
    XAM_POSIX::xam_free(mName);
}


