#ifndef __XAM_STATUS_H__
#define __XAM_STATUS_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_errors.h"
#include "xam_strings.h"

#include "XAM_POSIX.h"
#include "XAM_Array.h"

/* Successful xam_status */
static const xam_status XAM_OK = 0;  

/** 
 *  Provides methods to convert standard xam_status codes into tokens
 */
class XAM_Status
{
    private:

        /** Struct for mapping xam_status error codes to tokens */
        typedef struct 
        {
            xam_status sErrorCode;
            const char* sErrorToken;
        } XAM_StatusTokenMap;

    public:    

        /**
         * Get the token associated with the xam_status code provided
         *
         * @param inCode The xam_status code
         * @param outToken The token associated with the code is written here
         */
        static xam_boolean 
        getToken(xam_status inCode, xam_string* const outToken)
        {
            xam_boolean vFoundError = false;

            /* Maps error tokens to error codes */
            XAM_StatusTokenMap vErrorMap[] = 
            {
                { XAM_OK,                      XAM_OK_TOKEN }, 
                { XAM_UNKNOWN_ERROR,           XAM_UNKNOWN_ERROR_TOKEN }, 
                { XAM_OUT_OF_MEMORY,           XAM_OUT_OF_MEMORY_TOKEN }, 
                { XAM_INVALID_PARAM,           XAM_INVALID_PARAM_TOKEN }, 
                { XAM_PARAM_NOT_UTF8,          XAM_PARAM_NOT_UTF8_TOKEN }, 
                { XAM_INVALID_HANDLE,          XAM_INVALID_HANDLE_TOKEN }, 
                { XAM_INVALID_MIME_TYPE,       XAM_INVALID_MIME_TYPE_TOKEN }, 
                { XAM_INVALID_XSTREAM_MODE,    XAM_INVALID_XSTREAM_MODE_TOKEN }, 
                { XAM_INVALID_XRI,             XAM_INVALID_XRI_TOKEN },
                { XAM_INVALID_XSET_MODE,       XAM_INVALID_XSET_MODE_TOKEN },
                { XAM_INVALID_FIELD_NAME,      XAM_INVALID_FIELD_NAME_TOKEN },
                { XAM_VIM_NOT_FOUND,           XAM_VIM_NOT_FOUND_TOKEN }, 
                { XAM_VIM_SYMBOL_NOT_FOUND,    XAM_VIM_SYMBOL_NOT_FOUND_TOKEN }, 
                { XAM_FIELD_NOT_FOUND,         XAM_FIELD_NOT_FOUND_TOKEN }, 
                { XAM_FIELD_IS_READ_ONLY,      XAM_FIELD_IS_READ_ONLY_TOKEN }, 
                { XAM_FIELD_EXISTS,            XAM_FIELD_EXISTS_TOKEN }, 
                { XAM_FIELD_IN_USE,            XAM_FIELD_IN_USE_TOKEN },
                { XAM_MAX_FIELDS_EXCEEDED,     XAM_MAX_FIELDS_EXCEEDED_TOKEN },
                { XAM_FILESYSTEM_ERROR,        XAM_FILESYSTEM_ERROR_TOKEN },
                { XAM_XSYSTEM_ABANDONED,       XAM_XSYSTEM_ABANDONED_TOKEN },
                { XAM_XSET_ABANDONED,          XAM_XSET_ABANDONED_TOKEN },
                { XAM_XSTREAM_ABANDONED,       XAM_XSTREAM_ABANDONED_TOKEN },
                { XAM_XSYSTEM_CORRUPT,         XAM_XSYSTEM_CORRUPT_TOKEN },
                { XAM_XSET_CORRUPT,            XAM_XSET_CORRUPT_TOKEN },
                { XAM_XSTREAM_CORRUPT,         XAM_XSTREAM_CORRUPT_TOKEN },
                { XAM_CONNECT_FAILED,          XAM_CONNECT_FAILED_TOKEN },
                { XAM_AUTH_REQUIRED,           XAM_AUTH_REQUIRED_TOKEN },
                { XAM_AUTH_DATA_NEEDED,        XAM_AUTH_DATA_NEEDED_TOKEN },
                { XAM_AUTH_FAILED,             XAM_AUTH_FAILED_TOKEN },
                { XAM_BAD_XUID_FORMAT,         XAM_BAD_XUID_FORMAT_TOKEN },
                { XAM_XSET_INACCESSIBLE,          XAM_XSET_INACCESSIBLE_TOKEN },
                { XAM_XASYNC_PENDING,                 XAM_XASYNC_PENDING_TOKEN },
                { XAM_NOT_SUPPORTED,           XAM_NOT_SUPPORTED_TOKEN },
                { XAM_OPERATION_NOT_ALLOWED,   XAM_OPERATION_NOT_ALLOWED_TOKEN },
                { XAM_OBJECT_IN_USE,           XAM_OBJECT_IN_USE_TOKEN },
                { XAM_NOT_A_JOB,               XAM_NOT_A_JOB_TOKEN },
                { XAM_JOB_INVALID_CMD,         XAM_JOB_INVALID_CMD_TOKEN },
                { XAM_JOB_INVALID_CMD_SYNTAX,  XAM_JOB_INVALID_CMD_SYNTAX_TOKEN },
                { XAM_JOB_LEVEL_NOT_SUPPORTED, XAM_JOB_LEVEL_NOT_SUPPORTED_TOKEN },
                { XAM_JOB_INSUFFICIENT_PERMISSIONS, XAM_JOB_INSUFFICIENT_PERMISSIONS_TOKEN },
                { XAM_JOB_INSUFFICIENT_RESOURCES, XAM_JOB_INSUFFICIENT_RESOURCES_TOKEN },
                { XAM_JOB_RUNNING,             XAM_JOB_RUNNING_TOKEN },
                { XAM_XSET_UNDER_RETENTION,    XAM_XSET_UNDER_RETENTION_TOKEN },
                { XAM_XSET_UNDER_HOLD,         XAM_XSET_UNDER_HOLD_TOKEN },
                { XAM_XSET_INVALID_HOLD_ID,      XAM_XSET_INVALID_HOLD_ID_TOKEN },
                { XAM_XSET_INVALID_RETENTION_VALUE,  XAM_XSET_INVALID_RETENTION_VALUE_TOKEN },
                { XAM_INVALID_POLICY_NAME,    XAM_INVALID_POLICY_NAME_TOKEN },
                { XAM_INVALID_POLICY_VALUE,    XAM_INVALID_POLICY_VALUE_TOKEN },
                { XAM_XASYNC_HALTED,          XAM_XASYNC_HALTED_TOKEN }
            };

            /* Search the table for the given error code and (if found) write the 
             * token data to outToken */
            xam_int vNumErrors = sizeof(vErrorMap)/sizeof(XAM_StatusTokenMap);
            for (xam_int i = 0; i < vNumErrors; i++)
            {
                if(vErrorMap[i].sErrorCode == inCode)
                {
                    XAM_POSIX::xam_strcpy((char*)outToken, 
                                          (char*)vErrorMap[i].sErrorToken);

                    vFoundError = true;
                    break;
                }
            }

            /* Could not locate the token. Default to unknown error */
            if(vFoundError == false)
                getToken(XAM_UNKNOWN_ERROR, outToken);

            return vFoundError;
        }
};

#endif // __XAM_STATUS_H__

