////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_TempFile.h"
#include "XAM_Exception.h"
#include "XAM_Logger.h"

XAM_TempFile::XAM_TempFile() : 
    mFile(0)
{
    xam_int vErrorCode = createTempFile();
    if (vErrorCode != 0)
    {
        // Temp file creation failed
        XAMFileSystemException e;
        throw e;
    }
}

XAM_TempFile::~XAM_TempFile()
{
#ifdef __MVS__ 

 // for MVS or z/OS we deal with 4 different temporary files
 // temporary disk dataset, & temp disk dataset, memory files or 
 // hiperspace memory files

 char removedsnq[100];  // buffer to put in z/os temporary dataset 
                        // name with quotes
 char ddname[10];       // ddname allocated for for temporary dataset
 int rc;                // return code for fldata
 fldata_t fdata;        // structure used to return info from fldata call
 int b_removedsn=false; // boolean to determine if temp dataset needs to 
                        // be removed
  
 // with zOS we need to use z/os call to get file information from FILE pointer 

 rc=fldata(mFile,ddname,&fdata);
 
 if (rc) return;  // non-zero from fldata is an error

 // check the type of dataset
   
 if ((fdata.__dsorgMem) || (fdata.__dsorgHiper) )
     b_removedsn=true;  // memory or hiperspace temp dataset needs 
                        // to be deleted via remove call

 
 // set up a string containing dataset with quotes around it
 // only if dataset has to be removed

 if (b_removedsn) {
     XAM_POSIX::xam_strcpy(removedsnq,NXLT("'"));
     XAM_POSIX::xam_strcat(removedsnq,fdata.__dsname);
     XAM_POSIX::xam_strcat(removedsnq,NXLT("'"));  
                }
 // in any case the dataset is closed  
 XAM_POSIX::xam_fclose(mFile);
 if (b_removedsn==true)
 { 
     rc=XAM_POSIX::xam_remove(removedsnq); 
 }
 // end of z/OS temporary file processing
#else
    if (mFile)
        XAM_POSIX::xam_fclose(mFile);
#endif
}

const xam_int 
XAM_TempFile::createTempFile()
{
    // Create a temporary file
    xam_int vErrCode = 0;
    mFile = XAM_POSIX::xam_tmpfile();

    if (mFile == NULL)
    {
        XAM_LOG_ERROR(component_xam_debug, "Failed to create temp file!");
        XAM_LOG_ERROR(component_xam_debug, "  - Check tmp directory permissions and free space.");

        vErrCode = XAM_FILESYSTEM_ERROR;
    }

    return vErrCode;
}

const xam_int 
XAM_TempFile::write(const char* inData, 
                    const xam_int inDataOffset, 
                    const xam_int inDataSizeInBytes)
{
    // Seek to the requested offset
    int vSeek = XAM_POSIX::xam_fseek(mFile, inDataOffset, SEEK_SET);
    if(vSeek == -1)
    {
        XAMFileSystemException e;
        throw e;
    }

    // Perform the write, return the number of bytes written
    xam_int vSzWritten = XAM_POSIX::xam_fwrite(inData, 
                                               1, 
                                               inDataSizeInBytes,
                                               mFile);
    return vSzWritten;
}

const xam_int 
XAM_TempFile::read(char* const outData, 
                   const xam_int inDataOffset, 
                   const xam_int inDataSizeInBytes)
{
    // Seek to the requested offset
    int vSeek = XAM_POSIX::xam_fseek(mFile, inDataOffset, SEEK_SET);
    if(vSeek == -1)
    {
        XAMFileSystemException e;
        throw e;
    }

    // Perform the read, return the number of bytes read
    return XAM_POSIX::xam_fread(outData, 
                                1, 
                                inDataSizeInBytes, 
                                mFile);
}


