#ifndef __XAM_TEMPFILE__
#define __XAM_TEMPFILE__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include <XAM_POSIX.h>

#include <xam_types.h>
#include <xam_errors.h>

/** 
 *  Class representing a temporary file on disk
 */
class XAM_TempFile
{
    private:
        FILE* mFile;  /* only this class is allowed to close its file */

        /*
         * Create a temp file. The name of the file shall be built from unique names.
         * The destructor shall remove the file from the system.
         * @return An error code if one is encountered, 
         *         or zero (0) if no error is encountered.
         */
        const xam_int 
        createTempFile();

    public:
        /**
         * Default Constructor
         *
         *  Creates a temporary file
         */
        XAM_TempFile();

        /**
         * Destructor
         *
         *  Closes the temporary file (effectively deleting it)
         */
        ~XAM_TempFile();

        /**
         * Write data to this file
         *
         * @param inData             Pointer to the buffer of data being 
         *                           written
         * @param inDataOffset       The offset position into the file to 
         *                           start the write to
         * @param inDataSizeInBytes  The number of bytes to be written
         * @return The number of bytes successfully written to the file 
         *         (will be <= inDataSizeInBytes)
         */
        const xam_int 
        write(const char* inData, 
              const xam_int inDataOffset, 
              const xam_int inDataSizeInBytes);

        /**
         * Read data from this file
         *
         * @param outData            Pointer to allocated space where the 
         *                           data will be written
         * @param inDataOffset       The offset position into the file to 
         *                           start the read from
         * @param inDataSizeInBytes  The number of bytes available in the 
         *                           allocated space
         * @return The actual number of bytes written to outData 
         *         (will be <= inDataSizeInBytes)
         */
        const xam_int 
        read(char* const outData, 
             const xam_int inDataOffset, 
             const xam_int inDataSizeInBytes); 
};

#endif /* __XAM_TEMPFILE__ */


