////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_Thread.h"
#include "XAM_Logger.h"

int
XAM_Thread::run()
{
    int vRetVal = 0;

    XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Starting thread...");
    XAM_CriticalSection vCritical(&mThreadSyncMutex);

    // Spawn a new thread to execute the method
    vRetVal = XAM_POSIX::xam_thread_create(mInfo, mMethod);
    if(!vRetVal)
    {
        XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Thread started!");
        mInfo.running = true;
        mInfo.killed = false;
    }
    else
    {
        XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Failed to start thread!");        
    }

    return vRetVal;
}

int
XAM_Thread::join()
{
    int vRetVal = 0;

    XAM_CriticalSection vCritical(&mThreadSyncMutex);
    vRetVal = joinWithoutLock();

    return vRetVal;
}


int
XAM_Thread::joinWithoutLock()
{
    int vRetVal = 0;

    XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Joining thread...");

    // Join the thread if it is running
    if(mInfo.running)
    {
        vRetVal = XAM_POSIX::xam_thread_join(mInfo);
        mInfo.running = false;

        XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Thread joined!");
    }
    else
    {
        XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Thread not joined (not running!)");
    }

    return vRetVal;
}

bool 
XAM_Thread::kill()
{
    bool vSuccess = false;

    XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Killing thread...");
    XAM_CriticalSection vCritical(&mThreadSyncMutex);

    // Kill the thread if it is running 
    if(mInfo.running)
    {
        vSuccess = XAM_POSIX::xam_thread_kill(mInfo);
        if(vSuccess)
        {
            XAM_LOG_DEBUG_PLUS(component_xam_debug, 
                "[XAM_Thread] Thread killed! Joining killed thread...");

            // Join the thread even after it has been successfully killed. 
            // This is required because not all platforms successfully kill the
            // thread at this point and we want to ensure that it's dead before
            // we return from this method.
            int vJoinResult = joinWithoutLock();
            mInfo.killed = true;

            XAM_LOG_DEBUG_PLUS_P2(component_xam_debug,
                "[XAM_Thread] Thread killed and joined! (Join status: %i)", vJoinResult);
        }
        else
        {
            XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Could not kill thread!");   
        }
    }
    else
    {
        XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Could not kill thread (not running!)");    
    }

    return vSuccess;
}

xam_thread_t 
XAM_Thread::self()
{
    xam_thread_t vID = 0;

    // Get the thread ID of the calling thread
    vID = XAM_POSIX::xam_thread_self();
    
    return vID;
}

void 
XAM_Thread::exit()
{
    XAM_LOG_DEBUG_PLUS(component_xam_debug, "[XAM_Thread] Thread exiting!");

    // Exit the calling thread
    XAM_POSIX::xam_thread_exit();
}

XAM_Thread::XAM_Thread(XAM_THREAD_METHOD inMethod, void* inThreadData)
{
    mMethod = inMethod;
    mInfo.running = false;
    mInfo.killed = false;
    mInfo.thread_id = 0;
    mInfo.thread_handle = 0;
    mInfo.data = inThreadData;
}

XAM_Thread::~XAM_Thread()
{
    // Attempt to cleanly kill any running thread
    if(mInfo.running)
        kill();
}



