#ifndef __XAM_THREAD_H__
#define __XAM_THREAD_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include <xam_types.h>

#include "XAM_POSIX.h"
#include "XAM_Mutex.h"

/** 
 *  A portable thread class.
 */
class XAM_Thread
{
    private:    
        XAM_ThreadInfo mInfo;        /* Thread information structure */
        XAM_THREAD_METHOD mMethod;   /* Method called by run() */
        XAM_Mutex mThreadSyncMutex;  /* Protects concurrent thread operations */

        /** 
         * Join this thread (without obtaining the thread lock)
         * @return status of the join operation
         */
        int
        joinWithoutLock();

    public:

        /** 
         * Constructor
         *  
         *  Contructs a thread object, which (when run) will call the 
         *  given function.
         *
         *  @param inMethod The method to execute when the thread is run
         *  @param inThreadData A pointer to any data structure for which
         *                      the thread needs access                
         */ 
        XAM_Thread(XAM_THREAD_METHOD inMethod, void* inThreadData);

        /** 
         * Destructor
         */
        ~XAM_Thread();

        /** Retrieve the thread info structure */
        XAM_ThreadInfo* 
        getInfo() 
        { 
            return &mInfo; 
        }

        /** Retrieve the thread id */
        xam_thread_t 
        getID() 
        { 
            return mInfo.thread_id; 
        }

        /** 
         * Retrieve the thread status 
         *
         * @return THREAD_RUNNING if the thread is still running
         *         THREAD_KILLED if the thread was killed
         *         THREAD_OK for normal status (e.g. success, not yet run)
         */
        int
        getStatus() 
        { 
            int vStatus = THREAD_OK;

            if(mInfo.running == true)
                vStatus = THREAD_RUNNING;
            else if(mInfo.killed == true)
                vStatus = THREAD_KILLED;

            return vStatus;
        }
       
        /** Create/spawn the thread */
        int 
        run();  

        /** Join this thread */
        int 
        join(); 

        /** Terminate this thread */
        bool 
        kill(); 

        /** Get the thread ID of the calling thread */
        static xam_thread_t 
        self();

        /** Exit the currently running thread (use in callback) */
        static void 
        exit();  
};


#endif // __XAM_THREAD_H__


