////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_strings.h"
#include "xam_errors.h"

#include "XAM_Utils.h"
#include "XAM_Exception.h"

// Unicode space (' ') character
const int   XAM_Utils::INVALID_UCS4_CODEPOINT = 0x20;

// The string denoting that a given string is too large to fit
// in a static memory block
const char* XAM_Utils::TOO_LARGE_STRING = "-";

// NULL string indicator for logs
const char* XAM_Utils::XAM_NULL_STRING = "(nil)";

// Percent ascii character
const char XAM_Utils::XAM_PERCENT_DELIMITER = (char)0x25;

// Covers the static memory reference request
XAM_Mutex XAM_Utils::sMutex;          

// The size of the static memory buffer attributes
const long kBufferSize = 1024*1024; // 1 MB
const long kLargeStringThresholdSize = 2 *1024;
const long kTruncatedBufferSize = 1024;
const long kTruncatedCharSize = 1000;

// Buffer used to generate static memory ref
char XAM_Utils::sBuffer[kBufferSize];

// Pointer to next block of static memory
char* XAM_Utils::sNextPtr = (char*)0;


xam_boolean 
XAM_Utils::startsWith(const char* inString, const char* inStartString)
{
	xam_boolean vRetVal = false;
    xam_int i = 0;
	xam_int vStartStringLength = XAM_POSIX::xam_strlen(inStartString);

    for(i = 0; i < vStartStringLength; i++)
    {
        if(inString[i] == (char)'\0')
        {
           break;
        }else
        {
            if(inString[i] != inStartString[i])
            {
                break;
            }
        }
    }

    if(i == vStartStringLength) 
    {
        vRetVal = true;
    }

	return vRetVal;
}


xam_boolean 
XAM_Utils::isStandardErrCode(const xam_status inCode)
{
    // If the first bit is set (i.e. the status code is negative), 
    // then it's a non-standard error code 
    xam_boolean vIsStandardCode = ((inCode >= 0) ? true : false);
    return vIsStandardCode;
}

char*
XAM_Utils::getStaticMemoryReference (const xam_int inSize)
{
    // initialize the 'next buffer pointer' on the first pass
    XAM_CriticalSection vCritical(&sMutex);
    if ( sNextPtr == (char*)0 )
        sNextPtr = sBuffer;

    // make sure there is room in the buffer for the requested size
    char* vReturn = (char*)0;
    if ( (sNextPtr+inSize) < (sBuffer+kBufferSize) )
    {
        vReturn = sNextPtr;
        sNextPtr += inSize;
    }
    else // not enough room
    {
        if ( inSize <= kBufferSize )
        {
            // Loop back to the start of the static block
            vReturn = sBuffer;
            sNextPtr = sBuffer + inSize;
        }
    }
    return vReturn;
}

const char*
XAM_Utils::adjustPercentStringForLogging (const char* inString) 
{
    char* vBuffer = (char*) inString;

    if ( vBuffer == (char*)0)            
        vBuffer = (char*) XAM_NULL_STRING;
    else if(XAM_POSIX::xam_strchr((char*)inString, XAM_PERCENT_DELIMITER))
    {
        char* vCurrentStr = (char*)0;
        xam_int vCurrentSize = XAM_POSIX::xam_strlen(inString);
        if ( vCurrentSize > kLargeStringThresholdSize )
        {
            // copy in enough of arg + "...<truncated>"
            vCurrentStr = getStaticMemoryReference(kTruncatedBufferSize+1);
            XAM_POSIX::xam_snprintf(vCurrentStr, kTruncatedCharSize, inString);
            XAM_POSIX::xam_strcat(vCurrentStr, "...");
            vCurrentStr[kTruncatedBufferSize] = '\0';
        }

        char* vPercentLocation = (char*)0;
        int vPercentCount = 0;
        if (vCurrentStr == (char*)0)
            vCurrentStr = const_cast<char*>(inString);

        vPercentLocation = XAM_POSIX::xam_strchr(vCurrentStr, 
                                                 XAM_PERCENT_DELIMITER);
        while (vPercentLocation != (char*)0)
        {
            vPercentCount++;
            vPercentLocation = XAM_POSIX::xam_strchr(++vPercentLocation, 
                                                     XAM_PERCENT_DELIMITER);
        }

        // if any percent chars, parse them out
        if (vPercentCount > 0)
        {
            // get the new block for the new string
            xam_int vNewSize = vCurrentSize + vPercentCount + 1;
            vBuffer = getStaticMemoryReference(vNewSize);

            // copy chars, replacing the '%' with the sequence "%%"
            if ( vBuffer )
            {
                char* vPtr = vBuffer;
                vPercentLocation = XAM_POSIX::xam_strchr(vCurrentStr, 
                                                         XAM_PERCENT_DELIMITER);
                xam_int vSubSize = 0;
                xam_int vBytesToCopy = 0;
                while (vPercentLocation != (char*)0)
                {
                    vCurrentSize = XAM_POSIX::xam_strlen(vCurrentStr);
                    vSubSize = XAM_POSIX::xam_strlen(vPercentLocation);
                    vBytesToCopy = vCurrentSize - vSubSize + 1;
                    XAM_POSIX::xam_strncpy(vPtr, vCurrentStr, vBytesToCopy);
                    vPtr += vBytesToCopy;
                    *vPtr = (char)XAM_PERCENT_DELIMITER;
                    vPtr++;
                    vCurrentStr = vPercentLocation + 1;
                    if(vCurrentStr != (char*)0)
                    {
                        vPercentLocation = XAM_POSIX::xam_strchr(vCurrentStr, 
                                                                 XAM_PERCENT_DELIMITER);
                    }
                    else
                    {
                        vPercentLocation = (char*)0;
                    }
                }
                if (vCurrentStr != (char*)0)
                    XAM_POSIX::xam_strcpy(vPtr, vCurrentStr);
                vBuffer[vNewSize-1] = (char)0;
            }
            else
            {
                vBuffer = getStaticMemoryReference(XAM_POSIX::xam_strlen(TOO_LARGE_STRING)+1);
                XAM_POSIX::xam_strcpy(vBuffer, TOO_LARGE_STRING);
            }
        }
        // if no percent chars, just do the work here
        else
        {
            vBuffer = getStaticMemoryReference(XAM_POSIX::xam_strlen(inString)+1);
            if ( vBuffer )
                XAM_POSIX::xam_strcpy(vBuffer, inString);
            else
            {
                vBuffer = 
                    getStaticMemoryReference(XAM_POSIX::xam_strlen(TOO_LARGE_STRING)+1);
                XAM_POSIX::xam_strcpy(vBuffer, TOO_LARGE_STRING);
            }
        }
    }

    return vBuffer;
}

const char* 
XAM_Utils::toString(const xam_int inValue)
{
    // Retrieve a static memory reference and fill it with 
    // the converted int string
    char* vBuffer = getStaticMemoryReference(22); // 21 int chars max (+ null)
    if ( vBuffer )
    {
        xamIntToString(inValue, vBuffer);
    }
    else
    {
        // Couldn't get a static memory block large enough to fit
        // Return the "too large" string instead.
        vBuffer = 
            getStaticMemoryReference(XAM_POSIX::xam_strlen(TOO_LARGE_STRING)+1);

        XAM_POSIX::xam_strcpy(vBuffer, TOO_LARGE_STRING);
    }
    return vBuffer;
}

void
XAM_Utils::validateMIMEType(const char* inMime1, const char* inMime2)
{
    // Compare the mime type strings and throw if they are 
    // different
    if(XAM_POSIX::xam_strcasecmp(inMime1, inMime2) != 0)
    {
        XAMInvalidMimeTypeException e;
        throw e;
    }
}

void 
XAM_Utils::validateMode(const char* inMode)
{
    // Throw is the mode string is anything other than the following
    if((XAM_POSIX::xam_strcasecmp(inMode, XSTREAM_MODE_READ_ONLY) != 0) && 
       (XAM_POSIX::xam_strcasecmp(inMode, XSTREAM_MODE_WRITEONLY_TRUNCATE) != 0) &&
       (XAM_POSIX::xam_strcasecmp(inMode, XSTREAM_MODE_WRITEONLY_APPEND) != 0))
    {
        XAMInvalidStreamModeException e;
        throw e;
    }
}

xam_int 
XAM_Utils::validateSimpleType(const char* inMimeType, const char* inData)
{
    xam_int vSize = 0;

    // Throw if the mime type is not one of the defined "stypes" 
    if(XAM_POSIX::xam_strcmp(inMimeType, XAM_BOOLEAN_MIME_TYPE) == 0)
        vSize = sizeof(xam_boolean);
    else if(XAM_POSIX::xam_strcmp(inMimeType, XAM_INT_MIME_TYPE) == 0)
        vSize = sizeof(xam_int);
    else if(XAM_POSIX::xam_strcmp(inMimeType, XAM_DOUBLE_MIME_TYPE) == 0)
        vSize = sizeof(xam_double);
    else if(XAM_POSIX::xam_strcmp(inMimeType, XAM_XUID_MIME_TYPE) == 0)
        vSize = sizeof(xam_xuid);
    else if(XAM_POSIX::xam_strcmp(inMimeType, XAM_STRING_MIME_TYPE) == 0)
    {
        // Determine the actual string size included the NULL terminator,
        // but capping the length at the maximum size of the input buffer.
        xam_int vMaxSize = sizeof(xam_string) + 1; // include the NULL
        xam_int vActualSize = (inData) ? (XAM_POSIX::xam_strlen(inData) + 1) : vMaxSize; 

        if(vActualSize < vMaxSize)
            vSize = vActualSize;
        else
            vSize = vMaxSize;
    }
    else if(XAM_POSIX::xam_strcmp(inMimeType, XAM_DATETIME_MIME_TYPE) == 0)
    {
        // Determine the actual string size included the NULL terminator,
        // but capping the length at the maximum size of the input buffer.
        xam_int vMaxSize = sizeof(xam_datetime) + 1; // include the NULL
        xam_int vActualSize = (inData) ? (XAM_POSIX::xam_strlen(inData) + 1) : vMaxSize; 

        if(vActualSize < vMaxSize)
            vSize = vActualSize;
        else
            vSize = vMaxSize;
    }
    else
    {
        XAMInvalidMimeTypeException e;
        throw e;
    }

    return vSize;
}

#ifdef __MVS__

int 
XAM_Utils::convert_EBCDIC_to_ASCII (char *pach_EBCDIC_Chars,
                                    char *pach_ASCII_Chars,
                                    int   n_Length)
{
  int   n_Rc = 0;             /* return code    */
  int   n_Worksub;            /* work subscript */
  char *pach_Input_Chars;     /* work pointer for input  area */
  char *pach_Output_Chars;    /* work pointer for output area */

  /*
   * A few basic checks first.
   */

  if (pach_EBCDIC_Chars == NULL
      ||
      pach_ASCII_Chars  == NULL
      ||
      n_Length < 1) 
      return -1;

  /*
   * Set our work pointers to point to the start of the input and output area.
   */

  pach_Input_Chars  = pach_EBCDIC_Chars;
  pach_Output_Chars = pach_ASCII_Chars;

  /*
   * We'll just loop through the input area, taking the value of each byte and using
   * it as an index into the e2a_mvs table, where the equivalent ASCII value resides, 
   * and place this ASCII value into the corresponding byte in the output array..
   */

  for (n_Worksub = 0;n_Worksub < n_Length;n_Worksub++) 
  {
#ifdef ECOMPILE
      *pach_Output_Chars = (char) *pach_Input_Chars;
#else
      *pach_Output_Chars = (char) e2a_mvs[*pach_Input_Chars];
#endif 
       pach_Output_Chars++;
       pach_Input_Chars++;
  }

  return n_Rc;   /* return */
}


int 
XAM_Utils::convert_ASCII_to_EBCDIC (char *pach_ASCII_Chars,
                                    char *pach_EBCDIC_Chars,
                                    int   n_Length)
{
  int   n_Rc = 0;              /* return code    */
  int   n_Worksub;             /* work subscript */
  char *pach_Input_Chars;      /* work pointer for input  area */
  char *pach_Output_Chars;     /* work pointer for output area */

  /*
   * A few basic checks first
   */

  if (pach_ASCII_Chars  == NULL
      ||
      pach_EBCDIC_Chars == NULL
      ||
      n_Length < 1) 
      return -1;

  /*
   * Set our work pointers to point to the start of the input and output area.
   */

  pach_Input_Chars  = pach_ASCII_Chars;
  pach_Output_Chars = pach_EBCDIC_Chars;

  /*
   * We'll just loop through the input area, taking the value of each byte and using
   * it as an index into the a2e_mvs table, where the equivalent EBCDIC value resides, 
   * and place this EBCDIC value into the corresponding byte in the output array..
   */

  for (n_Worksub = 0;n_Worksub < n_Length;n_Worksub++) 
  {
#ifdef ECOMPILE
      *pach_Output_Chars = *pach_Input_Chars;
#else
      *pach_Output_Chars = a2e_mvs[*pach_Input_Chars];
#endif       
       pach_Output_Chars++;
       pach_Input_Chars++;
  }

  return n_Rc;
}

#endif // __MVS__

/*
 * Any Unicode character except...
 * - the only legal control sequences below 0x20 are \t \r and \n
 * - surrogate blocks are not valid
 * - 0xfffe and 0xffff are not valid
 */
bool
XAM_Utils::isCodepointValid (const int inChar)
{
    bool vIsValid = false;

    if (( inChar >= 0x0020 && inChar <= 0xd7ff )         ||
        ( inChar >= 0xe000 && inChar <= 0xfffd )         ||
        ( inChar >= 0x00010000 && inChar <= 0x0010ffff ) ||
        ( inChar == 0x09 )  /* '\t' */                   ||
        ( inChar == 0x0a )  /* '\n' */                   ||
        ( inChar == 0x0d )  /* '\r' */ )
    {
        vIsValid = true;
    }

    return vIsValid;
}

int
XAM_Utils::utf8_to_ucs4 (const unsigned char inMBChar[], 
                         xam_int& outChar,
                         bool inValidate)
{
    int vNumberOfMBChars = 0;
    if ( inMBChar[0] < 0x80 ) // single byte
    {
        // Determine validity of this single byte char
        if ( isCodepointValid((int)inMBChar[0])) 
        {
            outChar = inMBChar[0];
            vNumberOfMBChars = 1;
        }
        else // illegal codepoint
        {
            if (inValidate)
            {   
                XAMParamNotUTF8Exception e;
                throw e;
            }
            else
            {
                // Set invalid codepoint
                outChar = INVALID_UCS4_CODEPOINT;
                vNumberOfMBChars = 1;
            }
        }
    }
    else if ( inMBChar[0] >= 0xc2 ) // multibyte
    {      
        // Double byte
        if ( inMBChar[0] < 0xe0 )
        {
            if ( inMBChar[1] == (char)0 )
            {    
                if ( inValidate )
                {
                    XAMParamNotUTF8Exception e;
                    throw e;
                }
                else
                {
                    outChar = INVALID_UCS4_CODEPOINT;
                    vNumberOfMBChars = 1;
                }
            }
            else 
            {
                if ( (inMBChar[1]^0x80) < 0x40 )
                {
                    outChar = ((int)(inMBChar[0]&0x1f)<<6) | 
                               (int)(inMBChar[1]^0x80);
                    vNumberOfMBChars = 2;
                }
                else
                {
                    if ( inValidate )
                    {
                        XAMParamNotUTF8Exception e;
                        throw e;
                    }
                    else
                    {
                        // Set invalid codepoint
                        outChar = INVALID_UCS4_CODEPOINT;
                        vNumberOfMBChars = 2;
                    }
                }
            }
        }
        else if ( inMBChar[0] < 0xf0 ) // Triple byte
        {
            if ( inMBChar[1] == (char)0 || inMBChar[2] == (char)0 )
            {
                if ( inValidate )
                {
                    XAMParamNotUTF8Exception e;
                    throw e;
                }
                else
                {
                    // Set invalid codepoint
                    outChar = INVALID_UCS4_CODEPOINT;
                    if ( inMBChar[1] == (char)0 )
                        vNumberOfMBChars = 1;
                    else
                        vNumberOfMBChars = 2;
                }
            }
            else
            {
                if ( (inMBChar[1]^0x80) < 0x40 && 
                     (inMBChar[2]^0x80) < 0x40 && 
                     (inMBChar[0] >= 0xe1 || inMBChar[1] >= 0xa0) )
                {
                    outChar = ((int)(inMBChar[0]&0x0f)<<12) |
                              ((int)(inMBChar[1]^0x80)<<6)  | 
                               (int)(inMBChar[2]^0x80);
                    vNumberOfMBChars = 3;
                }
                else
                {
                    if ( inValidate )
                    {
                        XAMParamNotUTF8Exception e;
                        throw e;
                    }
                    else
                    {
                        // Set invalid codepoint
                        outChar = INVALID_UCS4_CODEPOINT;
                        vNumberOfMBChars = 3;
                    }
                }
            }
        }
        else if ( inMBChar[0] < 0xf8 ) // Quad byte
        {
            if ( inMBChar[1] == (char)0 || inMBChar[2] == (char)0 || 
                 inMBChar[3] == (char)0 )
            {                    
                if ( inValidate )
                {
                    XAMParamNotUTF8Exception e;
                    throw e;
                }
                else
                {
                    // Set invalid codepoint
                    outChar = INVALID_UCS4_CODEPOINT;
                    if ( inMBChar[1] == (char)0 )
                        vNumberOfMBChars = 1;
                    else if ( inMBChar[2] == (char)0 )
                        vNumberOfMBChars = 2;
                    else
                        vNumberOfMBChars = 3;
                }
            }
            else
            {
                if ( (inMBChar[1]^0x80) < 0x40 && 
                     (inMBChar[2]^0x80) < 0x40 && 
                     (inMBChar[3]^0x80) < 0x40 && 
                     (inMBChar[0] >= 0xf1 || inMBChar[1] >= 0x90) && 
                     (inMBChar[0] < 0xf4 || 
                       (inMBChar[0] == 0xf4 && inMBChar[1] < 0x90)) )
                {
                    outChar = ((int)(inMBChar[0] & 0x07)<<18) | 
                              ((int)(inMBChar[1]^0x80)<<12)   | 
                              ((int)(inMBChar[2]^0x80)<<6)    | 
                               (int)(inMBChar[3]^0x80);
                    vNumberOfMBChars = 4;

                }
                else
                {
                    if ( inValidate )
                    {
                        XAMParamNotUTF8Exception e;
                        throw e;
                    }
                    else
                    {
                        // Set invalid codepoint
                        outChar = INVALID_UCS4_CODEPOINT;
                        vNumberOfMBChars = 4;
                    }
                }
            }
        }
        else if ( inMBChar[0] < 0xfc ) // Quint byte
        {
            if ( inMBChar[1] == (char)0 || inMBChar[2] == (char)0 || 
                 inMBChar[3] == (char)0 || inMBChar[4] == (char)0 )
            {                    
                if ( inValidate )
                {
                    XAMParamNotUTF8Exception e;
                    throw e;
                }
                else
                {
                    // Set invalid codepoint
                    outChar = INVALID_UCS4_CODEPOINT;
                    if ( inMBChar[1] == (char)0 )
                        vNumberOfMBChars = 1;
                    else if ( inMBChar[2] == (char)0 )
                        vNumberOfMBChars = 2;
                    else if ( inMBChar[3] == (char)0 )
                        vNumberOfMBChars = 3;
                    else
                        vNumberOfMBChars = 4;
                }
            }
            else
            {
                if ( (inMBChar[1]^0x80) < 0x40 && 
                     (inMBChar[2]^0x80) < 0x40 && 
                     (inMBChar[3]^0x80) < 0x40 && 
                     (inMBChar[4]^0x80) < 0x40 && 
                     (inMBChar[0] >= 0xf9 || inMBChar[1] >= 0x88) )
                {
                    outChar = ((int)(inMBChar[0]&0x03)<<24) | 
                              ((int)(inMBChar[1]^0x80)<<18) | 
                              ((int)(inMBChar[2]^0x80)<<12) | 
                              ((int)(inMBChar[3]^0x80)<<6)  | 
                               (int)(inMBChar[4]^0x80);
                    vNumberOfMBChars = 5;
                }
                else
                {
                    if ( inValidate )
                    {
                        XAMParamNotUTF8Exception e;
                        throw e;
                    }
                    else
                    {
                        // Set invalid codepoint
                        outChar = INVALID_UCS4_CODEPOINT;
                        vNumberOfMBChars = 5;
                    }
                }
            }
        }
        else // if ( inMBChar[0] < 0xfe ) // Hex byte
        {
            if ( inMBChar[1] == (char)0 || inMBChar[2] == (char)0 || 
                 inMBChar[3] == (char)0 || inMBChar[4] == (char)0 )
            {                    
                if ( inValidate )
                {
                    XAMParamNotUTF8Exception e;
                    throw e;
                }
                else
                {
                    // Set invalid codepoint
                    outChar = INVALID_UCS4_CODEPOINT;
                    if ( inMBChar[1] == (char)0 )
                        vNumberOfMBChars = 1;
                    else if ( inMBChar[2] == (char)0 )
                        vNumberOfMBChars = 2;
                    else if ( inMBChar[3] == (char)0 )
                        vNumberOfMBChars = 3;
                    else if ( inMBChar[4] == (char)0 )
                        vNumberOfMBChars = 4;
                    else
                        vNumberOfMBChars = 5;
                }
            }
            else
            {
                if ( (inMBChar[1]^0x80) < 0x40 && 
                     (inMBChar[2]^0x80) < 0x40 && 
                     (inMBChar[3]^0x80) < 0x40 && 
                     (inMBChar[4]^0x80) < 0x40 && 
                     (inMBChar[5]^0x80) < 0x40 && 
                     (inMBChar[0] >= 0xfd || inMBChar[1] >= 0x84) )
                {
                    outChar = ((int)(inMBChar[0]&0x01)<<30) | 
                              ((int)(inMBChar[1]^0x80)<<24) |
                              ((int)(inMBChar[2]^0x80)<<18) | 
                              ((int)(inMBChar[3]^0x80)<<12) | 
                              ((int)(inMBChar[4]^0x80)<<6)  | 
                               (int)(inMBChar[5]^0x80);
                    vNumberOfMBChars = 6;
                }
                else
                {
                    if ( inValidate )
                    {
                        XAMParamNotUTF8Exception e;
                        throw e;
                    }
                    else
                    {
                        // Set invalid codepoint
                        outChar = INVALID_UCS4_CODEPOINT;
                        vNumberOfMBChars = 6;
                    }
                }
            }
        }
    }
    else
    {
        if ( inValidate )
        {
            XAMParamNotUTF8Exception e;
            throw e;
        }
        else
        {
            // Set invalid codepoint
            outChar = INVALID_UCS4_CODEPOINT;
            vNumberOfMBChars = 1;
        }
    }

    return vNumberOfMBChars;
}

void 
XAM_Utils::validateUTF8(const char inChar[], const int inNumChars)
{
    xam_boolean vIsValid = true;

    // The buffer must be non-zero
    if(inChar == 0)
    {
        XAMInvalidParamException e;
        throw e;
    }

    // Determine the number of characters to convert
    xam_int vNumChars = inNumChars;
    if ( vNumChars == (-1) )
    {
        vNumChars = 0;
        while ( inChar[vNumChars++] );
        vNumChars--;
    }

    if(vNumChars > 0)
    {
        // Validate the characters
        xam_boolean vBadPrefix = false;
        xam_int vNumWChars = 0;
        xam_int vTempChar  = 0;
        for ( xam_int ix=0; ix<vNumChars; )
        {
            try 
            { 
                // Convert from UTF8 to UCS4. Throws on error.
                // This is how we determine valid UTF8 chars
                ix += utf8_to_ucs4((unsigned char *)&inChar[ix], 
                                   vTempChar, 
                                   true); 
            }
            catch ( ... )
            {
                vIsValid = false;
                break;
            }
            vNumWChars++;
        }
        if ( vIsValid && vNumWChars == 0 )
            vIsValid = false;
    }

    // This string is not valid UTF-8
    if(vIsValid == false)
    {
        XAMParamNotUTF8Exception e;
        throw e;
    }
}

void
XAM_Utils::xamIntToString (const xam_int inValue, char outString[22])
{
    // easy case first
    if ( inValue == 0 )
    {
        outString[0] = (char)48;
        outString[1] = (char)0;
    }
    else
    {
        // set the sign only on negative numbers
        xam_int vStringIndex = 0;
        xam_int vValue = inValue;
        if ( vValue < 0 )
        {
            outString[vStringIndex++] = (char)45; // minus sign
            vValue *= -1;
        }

        // find the start of the number
        xam_int vMultiplier = 1;
        xam_int vTemp = vValue;
        while ( vTemp = vTemp/10 ) vMultiplier *= 10;

        // now process the number
        while ( vMultiplier != 0 )
        {
            xam_int vDigit = (xam_int)(vValue / vMultiplier);
            outString[vStringIndex++] = (char)(48 + vDigit);
            vValue -= vDigit * vMultiplier;
            vMultiplier /= 10;
        }
        outString[vStringIndex++] = (char)0;
    }
}


xam_int
XAM_Utils::stringToXamInt (const char* inString, 
                           const xam_boolean inStopAtBadDigit) 
{
    xam_int vValue = 0;
    xam_int vSign = 1;
    if ( inString )
    {
        bool vStringStarted = false;
        xam_int vLen = XAM_POSIX::xam_strlen(inString);
        char vChar;
        for ( int ix=0; ix<vLen; ix++ )
        {
            // ignore leading whitespace
            vChar = inString[ix];
            if ( vChar <= 32 && vStringStarted == false )
                continue;

            if ( vChar >= 48 && vChar <= 57 )
            {
                vValue *= 10;
                vValue += vChar - 48;
            }
            else
            {
                // the 'first' non-space character can be a '+' or a '-'
                if ( vStringStarted == false && (vChar == 43 || vChar == 45) )
                {
                    if ( vChar == 45 )
                        vSign = -1;
                }
                else // terminate processing
                {
                    if ( inStopAtBadDigit )
                        break;
                    else
                    {
                        XAMInvalidParamException e;
                        throw e;
                    }
                }
            }
            if ( vStringStarted == false )
                vStringStarted = true;
        }
    }
    vValue *= vSign;
    return vValue;
}

void 
XAM_Utils::assertXStreamType(const char* inMimeType)
{
    // Throw if the mime type is one of the defined "stypes" 
    if((XAM_POSIX::xam_strcmp(inMimeType, XAM_BOOLEAN_MIME_TYPE) == 0) ||
       (XAM_POSIX::xam_strcmp(inMimeType, XAM_INT_MIME_TYPE) == 0) ||
       (XAM_POSIX::xam_strcmp(inMimeType, XAM_DOUBLE_MIME_TYPE) == 0) ||
       (XAM_POSIX::xam_strcmp(inMimeType, XAM_XUID_MIME_TYPE) == 0) ||
       (XAM_POSIX::xam_strcmp(inMimeType, XAM_STRING_MIME_TYPE) == 0) ||
       (XAM_POSIX::xam_strcmp(inMimeType, XAM_DATETIME_MIME_TYPE) == 0))
    {
        XAMInvalidMimeTypeException e;
        throw e;
    }
}


