#ifndef __XAM_UTILS_H__
#define __XAM_UTILS_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "XAM_POSIX.h"
#include "XAM_Critical.h"
#if 0
#include "XAM_XStream.h"
#endif

/**
 * XAM Handle Type Options
 */
typedef enum
{
  XAM_OBJECT_HANDLE = 0, 
  XSYSTEM_HANDLE,
  XITERATOR_HANDLE,
  XSET_HANDLE,
  XASYNC_HANDLE,
  XSTREAM_HANDLE,
  GENERIC_HANDLE
} XAMObjType;

/** 
 *  A static class containing various utility methods
 */
class XAM_Utils
{
    public:
        /**
         * Determine if an error code is a standard error code
         *
         * @param inCode The xam_status code to check
         * @return true if the status us a standard error code, 
         *         false otherwise.     
         */
        static xam_boolean 
        isStandardErrCode(const xam_status inCode);

        /**
         * Determine whether the inString starts with inStartString
         *
         * @param inString The String that needs to be examined for 
         * the start string
         *
         * @param inStartString The start String
         * @return true if the inString starts with inStartString, 
         *         false otherwise.     
         */
        static xam_boolean
        startsWith(const char* inString, const char* inStartString);

        /**
         * Convert a xam_int to it's string equivalent
         *
         * @param inInt The integer to convert
         * @return Pointer to static memory containing the string result
         */
        static const char* 
        toString(const xam_int inInt);

        /**
         * Convert an integer string value to a xam_int
         *
         * @param inStr The string to convert to an integer
         * @return converted integer
         */
        static xam_int 
        toInt(const char* inStr) 
        { 
            return stringToXamInt(inStr); 
        }

        /**
         * Verify that a string contains all UTF-8 characters
         * 
         * @param inChar Array of characters to validate
         * @param inNumChars The number of chars to check (or -1 
         *                   to check the full string).
         * @throws XAMParamNotUTF8Exception
         */
        static void 
        validateUTF8(const char inChar[], 
                     const int inNumChars=-1);

        /**
         * Verify that a string is XSTREAM_MODE_READ_ONLY, XSTREAM_MODE_WRITEONLY_TRUNCATE, 
         * or XSTREAM_MODE_WRITEONLY_APPEND
         * 
         * @param inMode The mode string to validate
         * @throws XAMInvalidStreamModeException
         */
        static void 
        validateMode(const char *inMode);

        /**
         * Convenience method for verifying that the given mime types match
         *
         * @param inMime1 Any valid mime type string
         * @param inMime2 Any valid mime type string
         * @throws XAMInvalidMimeTypeException If they don't match
         */
        static void 
        validateMIMEType(const char* inMime1, const char* inMime2);

        /**
         * Convenience method for verifying that the given mimetype is 
         * a XAM "simple type" (ie. stype)   
         *
         * @param inMimeType Any valid mime type string
         * @param inData Optional pointer to data stream
         *               If non-null, string types will use the pointer for determining
         *               the actual field length
         * @return Size of the stype (in bytes)
         * @throws XAMInvalidMimeTypeException If the mime type is not an stype
         */
        static xam_int 
        validateSimpleType(const char* inMimeType, const char* inData=0);

    #ifdef __MVS__
        /* The following 2 methods will only work on the mainframe */

        /**
         *  Converts EBCDIC to ASCII Characters.
         *
         *  Convert an input array of EBCDIC characters into the output 
         *  area as ASCII characters. 
         *
         *  @param1: Pointer to input  array of EBCDIC characters to be 
         *           converted.
         *  @param2: Pointer to output array for ASCII characters to be 
         *           placed.
         *  @param3: Length of array to convert.
         *
         *  Note that parm 1 and parm 2 can point to the same address.
         */
        static int 
        convert_EBCDIC_to_ASCII(char *pach_EBCDIC_Chars,
                                char *pach_ASCII_Chars, 
                                int n_Length);

        /**
         *  Convert ASCII to EBCDIC Characters.
         *
         *  Convert an input array of ASCII characters into the output 
         *  area as EBCDIC characters. 
         *
         *  @param1: Pointer to input  array of  ASCII  characters to 
         *           be converted.
         *  @param2: Pointer to output array for EBCDIC characters to 
         *           be placed.
         *  @param3: Length of array to convert.
         *
         *  Note that parm 1 and parm 2 can point to the same address.
         */
        static int 
        convert_ASCII_to_EBCDIC (char *pach_ASCII_Chars,
                                 char *pach_EBCDIC_Chars,
                                 int   n_Length);
    #endif // __MVS__

        /*
         * Retrieve a pointer to a block of static memory
         *
         * @param inSize The size requested
         * @returns A pointer to the block of static memory
         *          or NULL if the requested size is too large 
         *          (i.e. exceeds the static kBufferSize)
         */
        static char* 
        getStaticMemoryReference (const xam_int inSize);

        /*
         * Adjusts percent characters from the given string for 
         * logging purposes
         *
         * @param inString The string to double percent characters on
         * @return A static memory reference of inString with doubled 
         *         percent characters
         */
        static const char*
        adjustPercentStringForLogging (const char* inString); 

        /**
         * Convenience method for asserting that the given mimetype is not
         * a XAM "simple type" (ie. stype)   
         *
         * @param inMimeType Any valid mime type string
         * @throws XAMInvalidMimeTypeException If the mime type is an stype
         */
        static void 
        assertXStreamType(const char* inMimeType);
    private:

        /* An invalid UCS4 codepoint */
        static const int INVALID_UCS4_CODEPOINT; 

        /* String indicating that a buffer is too large to fit in 
           the static memory block */
        static const char* TOO_LARGE_STRING;   

        static const char XAM_PERCENT_DELIMITER;
        static const char* XAM_NULL_STRING;

        static XAM_Mutex sMutex; /* Covers the static memory reference request */
        static char sBuffer[];   /* Buffer used to generate static memory ref */
        static char* sNextPtr;   /* Pointer to next block of static memory */

        /*
         * Determine if a UTF-8 codepoint is valid
         *
         * @param inChar The UTF-8 character to validate
         * @returns true if valid, false otherwise
         */
        static bool 
        isCodepointValid (const int inChar);

        /*
         * Convert a UTF8 string to UCS4
         *
         * @param inMBChar[] The character array to check
         * @param outChar Will contain the INVALID_CODEPOINT char on error
         *                or the converted character byte(s) on success
         * @param inValidate If true, this method throws an 
         *                   XAMParamNotUTF8Exception on error 
         * @returns The number of characters converted
         */
        static int 
        utf8_to_ucs4 (const unsigned char inMBChar[], 
                      xam_int& outChar,
                      bool inValidate);
        /*
         * Converts a xam_int to a string.
         * If each element of passed in buffer is not a number an exception 
         * is thrown
         * @param inValue xam_int holding the value of chars in the buffer
         * @param outString pointer to string to write a long value
         */
        static void 
        xamIntToString (const xam_int inValue, char outString[22]);

        /*
         * Converts the string to a xam_int.
         * If each element of passed in buffer is not a number an exception 
         * is thrown
         * @param inString pointer to string containing a long value
         * @return xam_int holding the value of chars in the buffer
         * @throws XAM_InvalidParamException If any of the elements of the 
         *                                   passed in buffer is not a number 
         *                                   between 0 - 9 inclusive.
         */
        static xam_int 
        stringToXamInt (const char* inString, 
                        const xam_boolean inStopAtBadDigit = false);
};

#define _logString(ss) XAM_Utils::adjustPercentStringForLogging(ss)

#endif // __XAM_UTILS_H__



