////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_strings.h"

#include "XAM_VIM_Manager.h"
#include "XAM_Logger.h"
#include "XAM_Object.h"
#include "XSystem.h"
#include "XSet.h"
#include "XAM_FieldIterator.h"

// Array of loaded VIM objects
XAM_Array<XAM_VIM> XAM_VIM_Manager::mVimArray;

// Mutex covering the VIM array
XAM_Mutex XAM_VIM_Manager::mVimArrayMutex;

// XRI element constants
const char* XAM_VIM_Manager::kProtocol = XAM_PROTOCOL;
const char* XAM_VIM_Manager::kVimToken = XAM_VIM_LIBRARY_TOKEN;     

// Namespaces for VIM aliases
const char* XAM_VIM_Manager::sVimAliasNamespace = ".xam.config.vim.alias.";      
const char* XAM_VIM_Manager::sVimStackAliasNamespace = ".xam.config.vim.alias.stack."; 

// Character sequence between VIM libraries (or alias) in a VIM stack
const char* XAM_VIM_Manager::sVimStackDelimiter = ":";

// Namespace for VIMs to be preloaded
const char* XAM_VIM_Manager::sVimPreloadNamespace = ".xam.config.vim.preload.";

// Namespace for definind lookup order
const char* XAM_VIM_Manager::sVimLookupOrderNamespace = ".xam.config.vim.lookuporder.";

void
XAM_VIM_Manager::parseXRI(const char* inString, 
                          xam_string &outVimName, 
                          xam_string &outVimCxnStr)
{
    // initialize
    outVimName[0] = (char)0;
    outVimCxnStr[0] = (char)0;
    char *vStart = (char*)0;

   xam_boolean vSniaProtocol = XAM_Utils::startsWith(inString, kProtocol);

    // Protocol is required
    //char *vStart = XAM_POSIX::xam_strstr(inString, kProtocol);
    if ( vSniaProtocol == false ) // protocol string not found, or invalid
    { 
        XAMInvalidXRIException e;
        throw e;
    }
    
    vStart = (char *)inString;

    // get the vim name and xsytem name elements of the XRI
    char *vEnd = XAM_POSIX::xam_strstr(inString, kVimToken);
    if(!vEnd) // no delimeter: assume everything is xsystem name and parameters
        {
        // copy the xsystem name and parameters into passed in storage 
            vEnd = vStart + XAM_POSIX::xam_strlen(kProtocol);
        XAM_POSIX::xam_strcpy(outVimCxnStr, (vEnd && *vEnd) ? vEnd : inString);
        }
    else // vimname delimiter found
        {
        // copy the vim name into the passed in storage
            vStart = vStart + (XAM_POSIX::xam_strlen(kProtocol));

            xam_int vVimNameLen = (vEnd - vStart);
            if(vVimNameLen > 0)
            {                
                XAM_POSIX::xam_strncpy(outVimName, vStart, vVimNameLen);
                outVimName[vVimNameLen] = '\0';
            }

            // Move past the VIM lib name identifier
            vEnd = vEnd + XAM_POSIX::xam_strlen(kVimToken);

        // copy the xsystem name and parameters into passed in storage
        XAM_POSIX::xam_strcpy(outVimCxnStr, (vEnd && *vEnd) ? vEnd : inString);

    // Translate any VIM name alias to it's actual configured value
    if(XAM_POSIX::xam_strcmp(outVimName, "") != 0)
    {
        // Obtain a handle to the XAM object
        XAM_FieldContainer* vXAMObject = 
            (XAM_FieldContainer*)XAM_Object::getInstance();

        // Append any configured name/value pairs to the VIM connection string
        appendConfiguredNameValuePairs(vXAMObject, outVimName, outVimCxnStr);

        // Find an alias for this VIM name
        xam_string vAlias;
        xam_boolean vAliasFound = 
            findAliasValue(vXAMObject, &outVimName[0], vAlias);

        // If an alias could be found, replace the existing VIM name
        // (only up to XAM_MAX_STRING characters!)
        if(vAliasFound && 
          (XAM_POSIX::xam_strlen(vAlias) < XAM_MAX_STRING))
        {
            XAM_POSIX::xam_strcpy(outVimName, vAlias);
        }
    }
}
}

bool
XAM_VIM_Manager::findVimLibInPath(const char* inLibName, 
                                  const char* inSearchPath,
                                  xam_string* const outFoundPath,
                                  int inRetryCount)
{
    bool vFound = false;
    xam_int vNumFound = 0;

    if(outFoundPath == (xam_string*)0)
    {
        XAMInvalidParamException e;
        throw e;
    }

    char* vLibName = (char*)inLibName;

    // For unspecified lib names, try libraries with "vim" in the name 
    // (Note: assumes that VIM libraries always have "vim" in the lib name.
    if(vLibName == (char*)0)
        vLibName = "vim";
    
    // Open the directory and locate a library with the given name.
    xam_int vLen = XAM_POSIX::xam_strlen(vLibName);
    XAM_DIR* vDir = XAM_POSIX::xam_opendir(inSearchPath);
    if(vDir != (XAM_DIR*)0)
    {
        // Search all directory contents for a matching VIM name
        XAM_DIR_ENTRY* vDp;
        while ((vDp = XAM_POSIX::xam_readdir(vDir)) != (XAM_DIR_ENTRY*)0)
        {
            xam_int vNameLen = XAM_POSIX::xam_strlen(vDp->d_name);

            // Determine if this directory element name matches the 
            // requested name (case insensitive)
            if (vNameLen >= vLen && 
                (XAM_POSIX::xam_strcasestr(vDp->d_name, vLibName) != 0)) 
            {
                // A match was found! Build the full path to the 
                // VIM for the caller
                XAM_POSIX::xam_sprintf(*outFoundPath, 
                                       "%s%c%s\0", 
                                       inSearchPath, 
                                       XAM_FILE_SEP, 
                                       vDp->d_name);

                // For retry, we skip "inRetryCount" matches
                if(inRetryCount == vNumFound)
                {
                    // Stop searching
                    vFound = true;
                    break;
                }

                // Found a match, but it was skipped. Search again...
                vNumFound++;
            }
        }

        XAM_POSIX::xam_closedir(vDir);
    }

    return vFound;
}

XAM_VIM* 
XAM_VIM_Manager::discoverVIM(int inRetryCount)
{
    XAM_VIM* vVim = (XAM_VIM*)0;
    xam_boolean vKeepLooking = true;
    xam_boolean vVimAlreadyLoaded = false;
    int vRetryCount = inRetryCount;

    // Select an available VIM (if any) from the VIM table first.
    // The existing VIM table will be exhausted before we attempt 
    // to look for more VIM libraries in the XAM_VIM_PATH. This is
    // driven by the inRetryCount parameter.
    vVim = selectExistingVim(0, inRetryCount);

    while((vVim == (XAM_VIM*)0) && vKeepLooking)
    {       
        // Load the first VIM library that we find in the XAM_VIM_PATH (if any)
        // Note: Assumes that the discovery method is to look in the 
        // "XAM_VIM_PATH_ENV_VAR" for the VIM directory to check for VIM's.
        const char* vPath = XAM_POSIX::xam_getenv(XAM_VIM_PATH_ENV_VAR);
        if((vPath != (const char*)0) && 
           (XAM_POSIX::xam_strcmp(vPath, "") != 0))
        {
            xam_string vLibPath;

            // The existing VIM array was checked first, so normalize
            // the retry count for this call based on the size of that
            // array. Retries for this call should start at 0.
            int vNormalizedRetryValue = vRetryCount - mVimArray.size();

            // Locate any VIM in the given path
            vKeepLooking = 
                findVimLibInPath((const char*)0, // Find any VIM library
                                 vPath, 
                                 &vLibPath, 
                                 vNormalizedRetryValue);
            if(vKeepLooking)
            {
                // Retry again, using the next unused index
                vRetryCount++;

                // The VIM library was located. Load it!
                // On success, this VIM will be returned.
                vVim = loadVIM(vLibPath, &vVimAlreadyLoaded);

                // If we've already tried this VIM, skip it and try
                // the next VIM in the path
                if(vVimAlreadyLoaded)                
                    vVim = (XAM_VIM*)0;                
            }
        }
        else
        {
            // There is no XAM_VIM_PATH to search
            vKeepLooking = false;
        }
    }

    // VIM library was identified
    if(vVim != (XAM_VIM*)0)
    {
        XAM_LOG_DEBUG_P2(component_vim_enter, 
                         "VIM_DISCOVERED: \"%s\"", vVim->getName());
    }

    return vVim;
}

XAM_VIM* 
XAM_VIM_Manager::selectVim(const char* inXRI, 
                           xam_string &outVimCxnStr,
                           xam_boolean &outSpecifiedVimName,
                           int& ioRetryCount)
{
    XAM_VIM* vVim = (XAM_VIM*)0;
    xam_string vVimName = "";

    // Assume that we don't have a VIM name specified
    outSpecifiedVimName = false;

        // Covers the VIM array throughout this operation
        XAM_CriticalSection vCritical(&mVimArrayMutex);

    try
    {
        // If an XRI string was provided, parse and apply it
        if(inXRI != (const char*)0)
        {
            xam_boolean vVimAlreadyLoaded = false;

            // Parse the XRI, checking for any requested 
            // vendor implementation name.  
            parseXRI(inXRI, vVimName, outVimCxnStr);

            // A library token was found in the XRI
            if(XAM_POSIX::xam_strcmp(vVimName, "") != 0)
            {
                // A VIM name was specified!
                outSpecifiedVimName = true;

                // Next, check if the XAM_VIM_PATH_ENV_VAR is being used
                const char* vEnvPath = 
                    XAM_POSIX::xam_getenv(XAM_VIM_PATH_ENV_VAR);

                // The path in the env var must exist and be accessible.
                // Otherwise, the env var is ignored.
                if((vEnvPath != (const char*)0) && 
                   (XAM_POSIX::xam_strcmp(vEnvPath, "") != 0) &&
                   (XAM_POSIX::xam_access(vEnvPath, XAM_F_OK) == 0))
                {
                    // Load the VIM library from the specified VIM_PATH
                    xam_string vLibPath = "";

                    xam_boolean vFound = findVimLibInPath(vVimName, 
                                                          vEnvPath, 
                                                          &vLibPath);
                    if(vFound)
                    {
                        // The library was found in the XAM_VIM_PATH
                        // Load the full path to the library directly.
                        vVim = loadVIM(vLibPath, &vVimAlreadyLoaded);

                        // On failure, try using the platform search rules
                        if(vVim == (XAM_VIM*)0)
                        {
                            vVim = loadVIM(vVimName, &vVimAlreadyLoaded);                        
                    }
                    }
                    else
                    {
                        // Not found in the XAM_VIM_PATH
                        // Use platform search rules to load it instead.
                        vVim = loadVIM(vVimName, &vVimAlreadyLoaded); 
                    }
                }
                else
                {
                    // Load the specified VIM library using the platform search 
                    // rules given just the library name.
                    // (If the specified library is already loaded, this selects 
                    // the appropriate VIM from the loaded VIM array)
                    vVim = loadVIM(vVimName, &vVimAlreadyLoaded);
                }

                // The VIM library requested in the XRI was not found
                if(vVim == (XAM_VIM*)0)
                {
                    XAMVimNotFoundException e;
                    throw e;
                }
            }
        }

        // The XRI was either not specified, or no valid library was found.
        if(vVim == (XAM_VIM*)0)
        {
            // Check the XAM Library configuration for a predefined VIM lookup order
            int vLookupOrderCount = getLookupOrderConfigCount();

            // Only check the lookuporder if we haven't tried them all
            if(ioRetryCount < vLookupOrderCount)
            {
                xam_boolean vMoreLookupOrderFieldsFound = true;
                while((vVim == (XAM_VIM*)0) && vMoreLookupOrderFieldsFound)
                {
                    vVim = selectVimFromLookupOrderConfig(ioRetryCount, 
                                                          vMoreLookupOrderFieldsFound);  
            }
            }

            // We don't have a VIM at this point. Attempt to discover one.
            if(vVim == (XAM_VIM*)0)
            {
                vVim = discoverVIM(ioRetryCount - vLookupOrderCount);
                ioRetryCount++;
            }
        }
    }
    catch(...)
    {
        // A retryable error has occurred!
        // Increment the retry count needed for subsequent calls to this method
        ioRetryCount++;
        throw;
    }

    return vVim;
}

XAM_VIM*
XAM_VIM_Manager::loadVIM(const char* inLibPath, 
                         xam_boolean* const outVimAlreadyLoaded)
{
    XAM_VIM* vVim = (XAM_VIM*)0;

    if(inLibPath != (char*)0)    
    {
        // If already loaded, use the existing XAM_VIM
        vVim = selectExistingVim(inLibPath);

        // Adjust the requested VIM name, and look for an existing VIM
        if(vVim == (XAM_VIM*)0)
            vVim = selectExistingVim(inLibPath, 0, true);

        if(vVim != (XAM_VIM*)0)
        {
            if(outVimAlreadyLoaded)
                *outVimAlreadyLoaded = true;
        }
        else // VIM not found in the array. Add it.
        {
            if(outVimAlreadyLoaded)
                *outVimAlreadyLoaded = false;

            try
            {
                // Attempt to create the VIM object for this library
                // (throws on error)
                vVim = new XAM_VIM(inLibPath); 

                // Add this VIM to the VIM array
                mVimArray.push_back(vVim);       

                // Register this VIM with the XAM Object vim list 
                // (.xam.vim.list.* fields)
                xam_string vString;
                XAM_POSIX::xam_sprintf(vString, 
                                       "%s.%s", XAM_VIM_LIST, vVim->getName());

                XAM_Object* vXAMObject = XAM_Object::getInstance();
                vXAMObject->createString(vString, false, vVim->getName());
                vXAMObject->setFieldAsReadOnly(vString);

                // Log the vim library load event
                XAM_LOG_DEBUG_P2(component_lib, 
                                 "VIM_LOADED: \"%s\"", vVim->getPath());                
            }
            catch(XAM_Exception &)
            {
                // Ignore errors, just return a NULL VIM pointer
                if(vVim)
                {
                    delete vVim;
                    vVim = (XAM_VIM*)0;
                }
            }
        }
    }

    return vVim;
}

XAM_VIM* 
XAM_VIM_Manager::selectExistingVim(const char* inLibPath, 
                                   int inRetryCount, 
                                   xam_boolean inAdjustLibPath)
{
    XAM_VIM* vVim = (XAM_VIM*) 0;

    int vSz = mVimArray.size();

    if(inAdjustLibPath)
    {
        // If we want to adjust the library name first, perform the adjustment
        // and make a recursive call with the new name.
        char* vAdjustedLibPath = XAM_POSIX::xam_adjust_library_name(inLibPath);
        if(vAdjustedLibPath)
        {
            vVim = selectExistingVim(vAdjustedLibPath, inRetryCount, false);
            XAM_POSIX::xam_free(vAdjustedLibPath);
        }
    }
    else
    {
        // Iterate through the array and find the VIM with the same 
        // name or path.
        if(inLibPath)
        {
            // Backwards search for performance. The most likely
            // position for this VIM is at the end of the list.
            for(int ix=(vSz - 1) ; ix >= 0 ; ix--)
            {
                XAM_VIM* vTempVim = mVimArray[ix];
                if(vTempVim)
                {
                    // Compare the VIM paths, select the VIM if it matches
                    const char* vPath = vTempVim->getPath();
                    if(XAM_POSIX::xam_strcmp(vPath, inLibPath) == 0)
                    {
                        vVim = vTempVim;
                        break;
                    }
                    else
                    {
                        // Compare the VIM names, select the VIM if it matches
                        const char* vName = vTempVim->getName();
                        if(XAM_POSIX::xam_strcmp(vName, inLibPath) == 0)
                        {
                            vVim = vTempVim;
                            break;
                        }
                    }
                }
            }  
        }
        else
        {
            // Select the VIM found given a retry count
            // Start from the end, and work backwards for performance 
            // (the most likely VIM to use is the most recently loaded)
            if((vSz > 0) && (vSz > inRetryCount))
                vVim = mVimArray[vSz - inRetryCount - 1];
        }
    }

    return vVim;
}

int
XAM_VIM_Manager::getLookupOrderConfigCount()
{
    int vTotalFound = 0;

    XAM_FieldContainer* inContainer = (XAM_FieldContainer*) XAM_Object::getInstance();
    if(inContainer)
    {
        xam_boolean vKeepChecking = true;
        xam_int vIndex = 0;
        while(vKeepChecking)
        {
            // Determine if the lookuporder field exists for this index
            xam_boolean vFieldFound = false;
            xam_string vLookupField;
            XAM_POSIX::xam_sprintf(vLookupField, "%s%i", sVimLookupOrderNamespace, vIndex);
            inContainer->containsField(vLookupField, &vFieldFound);

            // Check if the next index exists (and notify the caller)
            xam_string vNextField;
            XAM_POSIX::xam_sprintf(vNextField, "%s%i", sVimLookupOrderNamespace, (vIndex + 1));
            inContainer->containsField(vNextField, &vKeepChecking);

            if(vFieldFound || vKeepChecking)
                vTotalFound++;

            vIndex++;
        }
    }

    return vTotalFound;
}

XAM_VIM*
XAM_VIM_Manager::selectVimFromLookupOrderConfig(int& ioIndex, 
                                                xam_boolean& ioKeepLooking)
{
    XAM_VIM* vSelectedVim = (XAM_VIM*)0;
    ioKeepLooking = false;

    if(ioIndex >= 0)
    {
        XAM_FieldContainer* inContainer = (XAM_FieldContainer*) XAM_Object::getInstance();
        if(inContainer)
        {
            // Determine if the lookuporder field exists for this index
            xam_boolean vFieldFound = false;
            xam_string vLookupField;
            XAM_POSIX::xam_sprintf(vLookupField, "%s%i", sVimLookupOrderNamespace, ioIndex);
            inContainer->containsField(vLookupField, &vFieldFound);

            // Check if the next index exists (and notify the caller)
            xam_string vNextField;
            XAM_POSIX::xam_sprintf(vNextField, "%s%i", sVimLookupOrderNamespace, (ioIndex + 1));
            inContainer->containsField(vNextField, &ioKeepLooking);

            // Increment the retry index
            if(vFieldFound || ioKeepLooking)
                ioIndex++;

            // Process the field found at this index
            if(vFieldFound)
            {
                // The type must be xam_string
                xam_string vType;
                inContainer->getFieldType(vLookupField, &vType);
                if(XAM_POSIX::xam_strcmp(vType, XAM_STRING_MIME_TYPE) == 0)
                {
                    // Get the field value
                    xam_string vVimName;
                    inContainer->getString(vLookupField, &vVimName);

                    // Check for aliases for this value
                    xam_string vNewName;
                    xam_boolean vNewNameFound = findAliasValue(inContainer, 
                                                               vVimName, 
                                                               vNewName, 
                                                               0, 
                                                               true);
                    if(vNewNameFound)
                    {
                        int vSz = sizeof(xam_string)-1;
                        XAM_POSIX::xam_strncpy(vVimName, vNewName, vSz);
                        vVimName[vSz] = '\0'; // NULL terminate
                    }

                    try
                    {
                        // Load this VIM
                        xam_boolean vAlreadyLoaded;
                        vSelectedVim = loadVIM(vVimName, &vAlreadyLoaded);  
                    }
                    catch(...)
                    {
                        // Ignore loading errors
                    }
                }
            }
        }
    }

    return vSelectedVim;
}

void
XAM_VIM_Manager::buildVimAliasMap(XAM_FieldContainer* inContainer,
                                  XAM_AssociativeArray& outVimAliasMap,
                                  xam_boolean inIncludeVimAliases,
                                  xam_boolean inIncludeVimStackAliases)
{
    if(inContainer)
    {
        xam_boolean vKeepLooking = false;

        // Iterate over the VIM alias namespace
        XAM_FieldIterator vIter(inContainer, sVimAliasNamespace);
        vIter.hasNext(vKeepLooking);
        while(vKeepLooking)
        {
            // Get the field name found
            xam_string vFieldName;
            vIter.getNext(vFieldName);

            // The type must be xam_string
            xam_string vType;
            inContainer->getFieldType(vFieldName, &vType);
            if(XAM_POSIX::xam_strcmp(vType, XAM_STRING_MIME_TYPE) != 0)
            {
                vIter.hasNext(vKeepLooking);
                continue;
            }

            // Handle VIM stack aliases
            if(XAM_POSIX::xam_strstr(vFieldName, sVimStackAliasNamespace) && 
               inIncludeVimStackAliases)
            {
                // Bypass the vim stack alias namespace
                const char* vAliasName = 
                    &vFieldName[XAM_POSIX::xam_strlen(sVimStackAliasNamespace)];

                // Skip alias param namespaces
                if(XAM_POSIX::xam_strstr(vAliasName, ".param."))
                {
                    vIter.hasNext(vKeepLooking);
                    continue;
                }

                // Get the field value
                xam_string vFieldValue;
                inContainer->getString(vFieldName, &vFieldValue);

                // Strip off the VIM stack from the first library path (or alias)
                char* vDelimiter = XAM_POSIX::xam_strstr(vFieldValue, 
                                                         sVimStackDelimiter);
                if(vDelimiter)
                    *vDelimiter = '\0';

                // Add to the caller's hashmap
                outVimAliasMap.put(vAliasName, vFieldValue);
            }
            // Handle VIM aliases
            else if(inIncludeVimAliases)
            {
                // Bypass the vim alias namespace
                const char* vAliasName = 
                    &vFieldName[XAM_POSIX::xam_strlen(sVimAliasNamespace)];

                // Get the field value
                xam_string vFieldValue;
                inContainer->getString(vFieldName, &vFieldValue);

                // Add to the caller's hashmap
                outVimAliasMap.put(vAliasName, vFieldValue);                
            }

            vIter.hasNext(vKeepLooking);
        }
    }
}

xam_boolean
XAM_VIM_Manager::findAliasValue(XAM_FieldContainer* inContainer,
                                const char* inAliasName,
                                xam_string& outAliasValue,
                                XAM_AssociativeArray* inAliasMap,
                                xam_boolean inCheckAliasForAlias)
{
    xam_boolean vAliasFound = false;
    XAM_AssociativeArray* vAliasMap = (XAM_AssociativeArray*)0;

    // Use the map from the caller
    if(inAliasMap != (XAM_AssociativeArray*)0)
        vAliasMap = inAliasMap;
    else
    {
        // Build a map of VIM name aliases
        vAliasMap = new XAM_AssociativeArray();
        buildVimAliasMap(inContainer, *vAliasMap, true, true);
    }

    // Loop through all aliases to find a match
    int vSz = vAliasMap->size();
    for(int i=0 ; i < vSz ; i++)
    {
        const char* vName = vAliasMap->getName(i);

        // Check for a match!
        if(XAM_POSIX::xam_strcmp(vName, inAliasName) == 0)
        {
            const char* vValue = vAliasMap->getValue(i);

            // Store the alias as a return candidate
            XAM_POSIX::xam_strncpy(outAliasValue, vValue, XAM_MAX_STRING);
            outAliasValue[XAM_MAX_STRING - 1] = '\0';
            vAliasFound = true;
        }

        // Recursively check for aliases which reference aliases.
        // Bail out after more than "n" layers deep, as there may be 
        // infinite recursion setup in the configuration!    
        int n = 20;
        if(vAliasFound && inCheckAliasForAlias)
        {        
            xam_string vNewAlias;
            xam_boolean vNewAliasFound = findAliasValue(inContainer, 
                                                        outAliasValue, 
                                                        vNewAlias,
                                                        vAliasMap, 
                                                        false);
            while(vNewAliasFound && n--)
            {
                // Store the new alias as a return candidate
                XAM_POSIX::xam_strncpy(outAliasValue, vNewAlias, XAM_MAX_STRING);
                outAliasValue[XAM_MAX_STRING - 1] = '\0';

                vNewAliasFound = findAliasValue(inContainer, 
                                                vNewAlias,
                                                vNewAlias,
                                                vAliasMap, 
                                                false);
            }

            // Select the alias (return false on recursion error)
            if(n < 0)        
                vAliasFound = false;
        }
    }

    // Deallocate the alias map if allocated in this call
    if(!inAliasMap)
        delete vAliasMap;

    return vAliasFound;
}

void
XAM_VIM_Manager::preloadVims(XAM_FieldContainer* inContainer)
{
    if(inContainer)
    {
        // Build a current alias list
        XAM_AssociativeArray vAliases;
        buildVimAliasMap(inContainer, vAliases, true, true);

        xam_boolean vKeepLooking = false;

        // Iterate over the VIM preload namespace
        XAM_FieldIterator vIter(inContainer, sVimPreloadNamespace);
        vIter.hasNext(vKeepLooking);

        xam_boolean vFound = vKeepLooking;
        if(vFound)
        {
            XAM_LOG_DEBUG(component_xam_debug, 
                          "******* Start XAMLibrary VIM Preload Sequence *******");
        }

        while(vKeepLooking)
        {
            // Get the field name found
            xam_string vFieldName;
            vIter.getNext(vFieldName);

            // The type must be xam_string
            xam_string vType;
            inContainer->getFieldType(vFieldName, &vType);
            if(XAM_POSIX::xam_strcmp(vType, XAM_STRING_MIME_TYPE) != 0)
            {
                vIter.hasNext(vKeepLooking);
                continue;
            }

            // Get the field value
            xam_string vVimName;
            inContainer->getString(vFieldName, &vVimName);

            // Check for aliases for this value
            xam_string vNewName;
            xam_boolean vNewNameFound = findAliasValue(inContainer, 
                                                       vVimName, 
                                                       vNewName, 
                                                       &vAliases);
            if(vNewNameFound)
            {
                int vSz = sizeof(xam_string)-1;
                XAM_POSIX::xam_strncpy(vVimName, vNewName, vSz);
                vVimName[vSz] = '\0'; // NULL terminate
            }

            try
            {
                // Load this VIM
                xam_boolean vAlreadyLoaded;
                XAM_VIM* vVim = loadVIM(vVimName, &vAlreadyLoaded);               

                if(vVim && !vAlreadyLoaded)
                {
                    XAM_LOG_DEBUG_P2(component_xam_debug, 
                                     "VIM_PRELOAD [SUCCESS] : %s", vVimName);
                }
                else if(!vVim)
                {
                    XAMVimNotFoundException e;
                    throw e;
                }
        
            }
            catch(...)
            {
                // Log errors
                XAM_LOG_DEBUG_P2(component_xam_debug, 
                                 "VIM_PRELOAD [**FAILED**] : %s", vVimName);
            }

            // Check for another VIM to preload
            vIter.hasNext(vKeepLooking);
        }

        if(vFound)
        {
            XAM_LOG_DEBUG(component_xam_debug, 
                          "******* End XAMLibrary VIM Preload Sequence *******");
        }
    }
}

void
XAM_VIM_Manager::appendConfiguredNameValuePairs(XAM_FieldContainer* inContainer,
                                                xam_string inSpecifiedVimName, 
                                                xam_string& ioCxnStr)
{
    if(inContainer)
    {        
        // Build the namespace buffer to search
        xam_string vParamNamespace;
        XAM_POSIX::xam_sprintf(vParamNamespace, 
                               "%s%s.param.", 
                               sVimAliasNamespace, 
                               inSpecifiedVimName);

        // Iterate over the param namespace for this VIM name
        xam_boolean vKeepLooking = false;
        XAM_FieldIterator vIter(inContainer, vParamNamespace);
        vIter.hasNext(vKeepLooking);

        // Loop through all params and append to the connection string
        while(vKeepLooking)
        {
            // Get the field name found
            xam_string vFieldName;
            vIter.getNext(vFieldName);

            // The type must be xam_string
            xam_string vType;
            inContainer->getFieldType(vFieldName, &vType);
            if(XAM_POSIX::xam_strcmp(vType, XAM_STRING_MIME_TYPE) != 0)
            {
                vIter.hasNext(vKeepLooking);
                continue;
            }

            // Get the field name
            xam_string vName;
            xam_int vSz = XAM_POSIX::xam_strlen(vParamNamespace);
            XAM_POSIX::xam_strcpy(vName, &vFieldName[vSz]);

            // Get the field value
            xam_string vValue;
            inContainer->getString(vFieldName, &vValue);

            // Determine if we already have params specified
            xam_boolean vNeedsQuestionMark = (XAM_POSIX::xam_strstr(ioCxnStr, "?") == 0);

            // Determine if we already have at least 1 param specified
            xam_boolean vHasParams = (XAM_POSIX::xam_strstr(ioCxnStr, "=") != 0);

            // Build the final name/value pair
            xam_string vFinalParam;
            XAM_POSIX::xam_sprintf(vFinalParam, 
                                  "%s%s%s=%s", 
                                  vNeedsQuestionMark ? "?" : "",
                                  vHasParams ? "&" : "",
                                  vName,
                                  vValue);

            // Append name/value pair to the connection string (if it fits)
            xam_int vCxnLen = XAM_POSIX::xam_strlen(ioCxnStr);
            if((XAM_POSIX::xam_strlen(vFinalParam) + vCxnLen) < XAM_MAX_STRING)
            {
                XAM_POSIX::xam_strcat(ioCxnStr, vFinalParam);
                vIter.hasNext(vKeepLooking);
            }
            else
            {
                // No more room to append, so stop looking for params
                vKeepLooking = false;
            }
        }
    }
}



