////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_XAsyncHandler.h"
#include "XAM_Critical.h"
#include "XAM_Logger.h"
#include "XAM_Handle.h"

// Array used to register XAsync handles requiring callback notification 
XAM_Array<XAM_Handle> XAM_XAsyncHandler::mActivePublicXAsyncHandleTable;

// Mutex covering the Async arrays
XAM_Mutex XAM_XAsyncHandler::mAsyncHandlerMutex;   

EXPORT void DECL
XAM_AsyncCallback (const xasync_handle inHandle)
{
    XAM_LOG_DEBUG_P2(component_xam_debug, 
                     "XAM_AsyncCallback: Entered (%"XAM_INT_FORMAT"d)",
                     inHandle);

    // Notify the handler of the async operation completion
    // (This will result in the user's callback being executed, if provided).
    XAM_XAsyncHandler::asyncOperationComplete(inHandle);

    XAM_LOG_DEBUG_P2(component_xam_debug, 
                     "XAM_AsyncCallback: Exited (%"XAM_INT_FORMAT"d)",
                     inHandle);
}

void
XAM_XAsyncHandler::registerAsyncOp(XAM_Handle* inHandle)
{
    XAM_CriticalSection vCritical(&mAsyncHandlerMutex);

    // Ensure that the XAsync instance is unique
    const int vIndex = mActivePublicXAsyncHandleTable.find(inHandle);
    if(vIndex != -1)
    {
        XAM_LOG_ERROR_P2(component_xam_debug, 
                         "XAM_XAsyncHandler: REGISTER FAILURE: The XAsync handle [%p] is already registered!",
                         inHandle);

        // The handle is already registered
        XAMInvalidParamException e;
        throw e;
    }
    else
    {
        XAM_LOG_DEBUG_P2(component_xam_debug, 
                         "XAM_XAsyncHandler: REGISTERED : XAsync handle [%p]",
                         inHandle);
    }

    // Add this handle to the active XAsync array
    mActivePublicXAsyncHandleTable.push_back(inHandle);
}

void
XAM_XAsyncHandler::unregisterAsyncOp(XAM_Handle* inHandle)
{
    XAM_CriticalSection vCritical(&mAsyncHandlerMutex);

    // Remove this handle from the active XAsync arrays
    const int vIndex = mActivePublicXAsyncHandleTable.find(inHandle);
    if(vIndex >= 0)
    {
        // Remove the stored XAsync handle from the discovered index
        // (The caller will dealloc this with a call to XAsync_Close().
        mActivePublicXAsyncHandleTable.remove(vIndex);

        XAM_LOG_DEBUG_P2(component_xam_debug, 
                         "XAM_XAsyncHandler: UNREGISTERED : XAsync handle [%p]",
                         inHandle);
    }
}

int
XAM_XAsyncHandler::findPrivateHandleIndex(const xasync_handle inPrivateHandle)
{
    int vRetVal = -1;

    // Iterate the public table looking for a handle which references the inPrivateHandle
    int vSz = mActivePublicXAsyncHandleTable.size();
    for(int i=0 ; i < vSz ; i++)
    {
        // Grab the public handle
        XAM_Handle* vPublicHandle = mActivePublicXAsyncHandleTable[i];

        // Extract the private handle
        xasync_handle vPrivateHandle = vPublicHandle->getHandle();

        // The private handle must be valid and match the input to return this index
        if(vPrivateHandle && (inPrivateHandle == vPrivateHandle))
        {
            // Found a match!
            vRetVal = i;
            break;
        }
    }

    return vRetVal;
}

void
XAM_XAsyncHandler::asyncOperationComplete(const xasync_handle inPrivateHandle)
{
    XAM_Handle* vXAsync = (XAM_Handle*)0;

    // PROBLEM:
    // We don't want to notify the XAsync object until it has been 
    // fully initialized, but the async operation may complete before 
    // that happens! The problem occurs because you can't fully init 
    // an XAsync object until the xasync_handle is returned. By the 
    // time the handle is returned, the async operation may have 
    // already finished and we would miss the notification completely!
    //
    // To solve this, we will loop the calling thread here until the 
    // XAsync object has been fully populated with a valid handle. 
    xam_boolean vFound = false;
    while(!vFound)
    {
       // START CriticalSection
        XAM_CriticalSection vCritical(&mAsyncHandlerMutex);

        // Determine the index of this private handle 
        // (if it was not previously registered, -1 will be returned)
        const int vIndex = findPrivateHandleIndex(inPrivateHandle);
        if(vIndex >= 0)
        {
            // Grab the handle and XOPID from the discovered index
            vXAsync = mActivePublicXAsyncHandleTable[vIndex];
            vFound = true;
        } 
       // END CriticalSection
    }

    XAM_LOG_DEBUG_P2(component_xam_debug, 
                     "XAM_XAsyncHandler: COMPLETE: Async operation for XAsync handle [%p]", 
                     vXAsync);

    // Notification complete! Unregister the table entry...
    unregisterAsyncOp(vXAsync);

    // Notify the user's callback
    vXAsync->notifyComplete();
}

