#ifndef __XAM_XASYNC_HANDLER_H__
#define __XAM_XASYNC_HANDLER_H__
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "XAM_Handle.h"
#include "XAM_Thread.h"
#include "XAM_Mutex.h"
#include "XAM_Array.h"

/* Callback to be utilized for VIM API async operations */
EXPORT void DECL
XAM_AsyncCallback (const xasync_handle inHandle);

/** 
 * Handles and manages XAsync status requests from the VIMs
 */
class XAM_XAsyncHandler
{
    private:
        // Array used to register XAsync handles requiring callback notification 
        static XAM_Array<XAM_Handle> mActivePublicXAsyncHandleTable;

        // Mutex covering the registered XAsync handle array
        static XAM_Mutex mAsyncHandlerMutex;     

        /**
         * Returns the index in the public handle array which references the 
         * private handle provided.
         *
         * @note This method is not thread safe and must be called under the
         *       mAsyncHandlerMutex mutex!
         *
         * @param inPrivateHandle The private (VIM) xasync_handle to search for
         * @returns The index of the corresponding public handle in the array, 
         *          or -1 if not found.         
         */
        static int
        findPrivateHandleIndex(const xasync_handle inPrivateHandle);

    public:        

        /**
         * Registers an async operation requiring a callback notification 
         * from the VIM.
         *
         * @param inHandle An XAsync instance wrapper handle to register
         * @throws XAMInvalidParamException If the handle is already registered
         */
        static void
        registerAsyncOp(XAM_Handle* inHandle);

        /**
         * Removes an entry from the active XAsync handle table.
         *
         * Should be utilized after a callback has been notified 
         * (or if the XAsync object has been closed)
         *
         * @param inHandle The XAsync handle to unregister
         */
        static void
        unregisterAsyncOp(XAM_Handle* inHandle);

        /**
         * Reports async operation complete notifications from the VIM 
         *
         * @param inPrivateHandle Private (VIM) handle to the async object
         */
        static void
        asyncOperationComplete(const xasync_handle inPrivateHandle);
};

#endif // __XAM_XASYNC_HANDLER_H__

