#ifndef __VIM_H_
#define __VIM_H_
////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007, The Storage Networking Industry Association.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - Neither the name of The Storage Networking Industry Association (SNIA)
//      nor the names of its contributors may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
// ============================================================================
//  Copyright (c) 2007, EMC Corporation.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of EMC Corporation may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
//  XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////

#include "xam_types.h"
#include "xam_strings.h"
#include "xam_errors.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
    Generates an error token from the xam_status. If passed an XSystem 
    reference, it will be able to generate error tokens for non-standard 
    status. Otherwise, non-standard status will always generate the 
    xam/unknown error token.

    This method does not require any passed in XSystem to be authenticated. 
    It will also work on an XSystem that is in a corrupted or aborted state . 
    It returns TRUE on success, and FALSE on failure.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.


    @param inHandle A valid xam_handle, containing an XSystem or a XAM 
                    library object reference. 
    @param inStatus A valid xam_status.  
    @param outToken A reference to valid storage for a xam_string. The 
                    value that is passed in is not used, and is overwritten 
                    with the result
    @return true if the error token was found and written outToken,
            false otherwise
 */
EXPORT xam_boolean DECL
VIM_XSystem_GetErrorToken (const xsystem_handle inHandle,
                           const xam_status inStatus,
                           xam_string* const outToken);

/**************************************************************************
 *  method prototypes for the XIterator
 *************************************************************************/

/**
    A factory interface, creating an XIterator from an XSystem. This iterator
    is used to discover the field names of fields on the XSystem. Only those 
    fields whose names begin with the distinct bit sequence as specified in the
    pattern will be included in the enumeration.

    Resources associated with the XIterator must be explicitly released. Once 
    the resources are released, the XIterator will no longer be valid.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xam_handle_t, containing an XSet, XSystem or XAM 
                    Object reference. This is the object that contains the 
                    fields to be enumerated. 
    @param inPattern A valid xam_string, containing a valid, null terminated 
                     utf-8 byte sequence. The pattern in this xam_string will 
                     be used to filter the fields which will be enumerated  
                     those fields that do not being with the specified pattern 
                     will not be included in the enumeration. The pattern is 
                     very simple  the byte sequence is treated as an explicit 
                     prefix, if the beginning of a field name does not match 
                     the exact bit sequence of the specified pattern it will be
                     filtered out of the results. All fields are considered to 
                     begin with an empty string, thus specifying an empty 
                     string in the pattern will result in no fields being 
                     filtered.  
    @param outIterator A reference to valid storage for an xiterator_handle. 
                       The value that is passed in is not used, and is 
                       overwritten with the result.
    @return The status code generated by calling this function. Use the 
        XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
VIM_XSystem_OpenFieldIterator (const xsystem_handle inHandle,  
                               const xam_string inPattern,
                               xiterator_handle* const outIterator);

/**
    A factory interface, creating an XIterator from an XSet. This iterator is 
    used to discover the field names of fields on the XSystem. Only those 
    fields whose names begin with the distinct bit sequence as specified in the
    pattern will be included in the enumeration.

    Resources associated with the XIterator must be explicitly released. Once 
    the resources are released, the XIterator will no longer be valid.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete

    @param inHandle A valid xam_handle_t, containing an XSet, XSystem or XAM 
                    Object reference. This is the object that contains the 
                    fields to be enumerated. 
    @param inPattern A valid xam_string, containing a valid, null terminated 
                     utf-8 byte sequence. The pattern in this xam_string will 
                     be used to filter the fields which will be enumerated  
                     those fields that do not being with the specified pattern 
                     will not be included in the enumeration. The pattern is 
                     very simple  the byte sequence is treated as an explicit 
                     prefix, if the beginning of a field name does not match 
                     the exact bit sequence of the specified pattern it will be
                     filtered out of the results. All fields are considered to 
                     begin with an empty string, thus specifying an empty 
                     string in the pattern will result in no fields being 
                     filtered.  
    @param outIterator A reference to valid storage for an xiterator_handle. 
                       The value that is passed in is not used, and is 
                       overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_OpenFieldIterator (const xset_handle inHandle,  
                            const xam_string inPattern,
                            xiterator_handle* const outIterator);

/**
    Determines if there are more field names available to be read 
    from the XIterator using the VIM_XIterator_Next method.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xiterator_handle. 
    @param outHasNext A reference to valid storage for a xam_boolean. 
                      If additional field names may be read from this XIterator,
                      "true" is written here. Otherwise, "false" is written. 
                      The value that is passed in is not used, and is overwritten
                      with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XIterator_HasNext (const xiterator_handle inHandle, 
                       xam_boolean* const outHasNext);

/**
    Copies the field name of the field at the current cursor of the iteration 
    into the provided storage. The cursor is then advanced to the next field. 
    Upon reading past the last field, an empty string will be returned.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xiterator_handle. 
    @param outName  A reference to valid storage for a xam_string. The result
                    is the name of the field following the current cursor (e.g.
                    the field name of the field at the current cursor/position 
                    in the iteration). The value that is passed in is not used, 
                    and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XIterator_Next (const xiterator_handle inHandle, 
                    xam_string* const outName);
  
/**
    Releases the resources associated with an open XIterator. After this method 
    is called, the XIterator may no longer be used.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xiterator_handle
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XIterator_Close (xiterator_handle inHandle);

/**************************************************************************
 *  method prototypes for managing XAM Fields (properties or XStreams)
 *************************************************************************/

/**
    Sets the xam_boolean value to true if the named field exists in this
    object, or to false otherwise.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing a valid XSystem reference. 
                    This is the object on which to determine the existence 
                    of the named field. 
    @param inName A xam_string containing the name of the field to locate.
    @param outContained A reference to valid storage for a xam_boolean. The result 
                        is true if the named field exists in the object; 
                        or false otherwise. The value that is passed in is not 
                        used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_ContainsField(const xsystem_handle inHandle,  
                          const xam_string inName,
                          xam_boolean* const outContained);

/**
    Sets the xam_boolean value to true if the named field exists in this
    object, or to false otherwise.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing a valid XSet reference. 
                    This is the object on which to determine the existence 
                    of the named field. 
    @param inName A xam_string containing the name of the field to locate.
    @param outExists A reference to valid storage for a xam_boolean. The result 
                     is true if the named field exists in the object; 
                     or false otherwise. The value that is passed in is not 
                     used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ContainsField(const xset_handle inHandle,  
                       const xam_string inName,
                       xam_boolean* const outExists);

/**
    Sets the binding attribute of a field to true.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet reference. This 
           is the object that contains the named field.
    @param inName A xam_string containing the name of the field to manipulate.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetFieldAsBinding (const xset_handle inHandle,
                            const xam_string inName);

/**
    Sets the binding attribute of a field to false.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet reference. This 
           is the object that contains the named field.
    @param inName A xam_string containing the name of the field to manipulate.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetFieldAsNonbinding (const xset_handle inHandle,
                               const xam_string inName);


/**
    Copies the mime-type of the named field into the provided xam_string.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that contains the named 
                    field. 
    @param inName A xam_string containing the name of the field to manipulate.
    @param outType A reference to valid storage for a xam_string. The result 
                   is the mime-type of the named field in the object. The value 
                   that is passed in is not used, and is overwritten with the 
                   result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetFieldType (const xsystem_handle inHandle,
                          const xam_string inName,
                          xam_string* const outType);

/**
    Copies the mime-type of the named field into the provided xam_string.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object reference.
                    This is the object that contains the named field. 
    @param inName A xam_string containing the name of the field to manipulate.
    @param outType A reference to valid storage for a xam_string. The result 
                   is the mime-type of the named field in the object. The value 
                   that is passed in is not used, and is overwritten with the 
                   result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetFieldType (const xset_handle inHandle,
                       const xam_string inName,
                       xam_string* const outType);

/**
    Copies the length of the named field into the provided xam_int.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object
                    reference. This is the object that contains the named 
                    field. 
    @param inName A xam_string containing the name of the field to manipulate.
    @param outLength A reference to valid storage for a xam_int. The result is 
                     the number of bytes of the value of the named field in the 
                     object. The value that is passed in is not used, and is 
                     overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetFieldLength (const xsystem_handle inHandle,
                            const xam_string inName,
                            xam_int* const outLength);
/**
    Copies the length of the named field into the provided xam_int.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object
                    reference. This is the object that contains the named 
                    field. 
    @param inName A xam_string containing the name of the field to manipulate.
    @param outLength A reference to valid storage for a xam_int. The result is 
                     the number of bytes of the value of the named field in the 
                     object. The value that is passed in is not used, and is 
                     overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetFieldLength (const xset_handle inHandle,
                         const xam_string inName,
                         xam_int* const outLength);

/**
    Sets the xam_boolean value to true if the binding attribute of the named 
    field is true, or to false otherwise.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object
                    reference. This is the object that contains the named 
                    field.
    @param inName A xam_string containing the name of the field to manipulate.
    @param outBinding A reference to valid storage for a xam_boolean. The result 
                      is true if the binding attribute of the named field is 
                      true; 
                      or false otherwise. The value that is passed in is not 
                         used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetFieldBinding (const xset_handle inHandle,
                          const xam_string inName,
                          xam_boolean* const outBinding);

/**
    Sets the xam_boolean value to true if the binding attribute of the named 
    field is true, or to false otherwise.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object
                    reference. This is the object that contains the named 
                    field.
    @param inName A xam_string containing the name of the field to manipulate.
    @param outReadOnly A reference to valid storage for a xam_boolean. The 
                      result is true if the read only attribute of the named 
                      field is true; or false otherwise. The value that is 
                      passed in is not used, and is overwritten with the 
                      result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetFieldReadOnly (const xsystem_handle inHandle,
                              const xam_string inName,
                              xam_boolean* const outReadOnly);

/**
    Sets the xam_boolean value to true if the read-only attribute of the named
    field is true, or to false otherwise.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete

    @param inHandle A valid xset_handle, containing an XSet Object reference.
                    This is the object that contains the named field. 
    @param inName A xam_string containing the name of the field to manipulate.
    @param outReadOnly A reference to valid storage for a xam_boolean. The 
                       result is true if the read-only attribute of the named 
                       field is true; or false otherwise. The value that is 
                       passed in is not used, and is overwritten with the 
                       result.
   @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
*/
EXPORT xam_status DECL
VIM_XSet_GetFieldReadOnly (const xset_handle inHandle,
                           const xam_string inName,
                           xam_boolean* const outReadOnly);

/**
    Removes a field from the XSet.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that contains the named 
                    field. 
    @param inName A xam_string containing the name of the field to delete.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_DeleteField (const xsystem_handle inHandle,
                         const xam_string inName);

/**
    Removes a field from the XSet.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that contains the named 
                    field. 
    @param inName A xam_string containing the name of the field to delete.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_DeleteField (const xset_handle inHandle,
                      const xam_string inName);

/**************************************************************************
 *  method prototypes for managing property fields
 *************************************************************************/

/**
    Creates a property field with a type set to 
    application/vnd.snia.xam.boolean on the object referenced by the passed
    in xam_handle_t. Its name, value and binding attributes will be set in 
    accordance with the user provided parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_boolean containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateBoolean (const xsystem_handle inHandle,
                           const xam_string inName,
                           const xam_boolean inBinding,
                           const xam_boolean inValue);

/**
    Creates a property field with a type set to 
    application/vnd.snia.xam.boolean on the object referenced by the passed
    in xam_handle_t. Its name, value and binding attributes will be set in 
    accordance with the user provided parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_boolean containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateBoolean (const xset_handle inHandle,
                        const xam_string inName,
                        const xam_boolean inBinding,
                        const xam_boolean inValue);

/**
    Creates a property field with a type set to application/vnd.snia.xam.int 
    on the object referenced by the passed in xam_handle_t. Its name, value and 
    binding attributes will be set in accordance with the user provided 
    parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_int containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateInt (const xsystem_handle inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_int inValue);

/**
    Creates a property field with a type set to application/vnd.snia.xam.int 
    on the object referenced by the passed in xam_handle_t. Its name, value and 
    binding attributes will be set in accordance with the user provided 
    parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_int containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateInt (const xset_handle inHandle,
                    const xam_string inName,
                    const xam_boolean inBinding,
                    const xam_int inValue);
/**
    Creates a property field with a type set to application/vnd.snia.xam.float 
    on the object referenced by the passed in xam_handle_t. Its name, value and 
    binding attributes will be set in accordance with the user provided 
    parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_double containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateDouble (const xsystem_handle inHandle,
                          const xam_string inName,
                          const xam_boolean inBinding,
                          const xam_double inValue);

/**
    Creates a property field with a type set to application/vnd.snia.xam.float 
    on the object referenced by the passed in xam_handle_t. Its name, value and 
    binding attributes will be set in accordance with the user provided 
    parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_double containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateDouble (const xset_handle inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_double inValue);

/**
    Creates a property field with a type set to application/vnd.snia.xam.xuid 
    on the object referenced by the passed in xam_handle_t. Its name, value and 
    binding attributes will be set in accordance with the user provided 
    parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_xuid containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateXUID (const xsystem_handle inHandle,
                        const xam_string inName,
                        const xam_boolean inBinding,
                        const xam_xuid inValue);

/**
    Creates a property field with a type set to application/vnd.snia.xam.xuid 
    on the object referenced by the passed in xam_handle_t. Its name, value and 
    binding attributes will be set in accordance with the user provided 
    parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_xuid containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateXUID (const xset_handle inHandle,
                     const xam_string inName,
                     const xam_boolean inBinding,
                     const xam_xuid inValue);

/**
    Creates a property field with a type set to 
    application/vnd.snia.xam.string on the object referenced by the passed in 
    xam_handle_t. Its name, value and binding attributes will be set in 
    accordance with the user provided parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_string containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateString (const xsystem_handle inHandle,
                          const xam_string inName,
                          const xam_boolean inBinding,
                          const xam_string inValue);

/**
    Creates a property field with a type set to 
    application/vnd.snia.xam.string on the object referenced by the passed in 
    xam_handle_t. Its name, value and binding attributes will be set in 
    accordance with the user provided parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_string containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateString (const xset_handle inHandle,
                       const xam_string inName,
                       const xam_boolean inBinding,
                       const xam_string inValue);

/**
    Creates a property field with a type set to 
    application/vnd.snia.xam.datetime on the object referenced by the passed 
    in xam_handle_t. Its name, value and binding attributes will be set in 
    accordance with the user provided parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_datetime containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateDatetime (const xsystem_handle inHandle,
                            const xam_string inName,
                            const xam_boolean inBinding,
                            const xam_datetime inValue);

/**
    Creates a property field with a type set to 
    application/vnd.snia.xam.datetime on the object referenced by the passed 
    in xam_handle_t. Its name, value and binding attributes will be set in 
    accordance with the user provided parameters.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inValue A xam_datetime containing the value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateDatetime (const xset_handle inHandle,
                         const xam_string inName,
                         const xam_boolean inBinding,
                         const xam_datetime inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.boolean on the object referenced by the passed 
    in xam_handle_t. Its value will be set in accordance with the user provided 
    parameter. 

    @note If the field is binding, this will result in a new XUID being assigned 
          to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_boolean containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_SetBoolean (const xsystem_handle inHandle,
                        const xam_string inName,
                        const xam_boolean inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.boolean on the object referenced by the passed 
    in xam_handle_t. Its value will be set in accordance with the user provided 
    parameter. 

    @note If the field is binding, this will result in a new XUID being assigned 
          to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_boolean containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetBoolean (const xset_handle inHandle,
                     const xam_string inName,
                     const xam_boolean inValue);

/**
    Changes a property field with a type set to application/vnd.snia.xam.int 
    on the object referenced by the passed in xam_handle_t. Its value will be 
    set in accordance with the user provided parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

   Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_int containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_SetInt (const xsystem_handle inHandle,
                    const xam_string inName,
                    const xam_int inValue);

/**
    Changes a property field with a type set to application/vnd.snia.xam.int 
    on the object referenced by the passed in xam_handle_t. Its value will be 
    set in accordance with the user provided parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

   Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_int containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetInt (const xset_handle inHandle,
			 	 const xam_string inName,
				 const xam_int inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.float on the object referenced by the passed in 
    xam_handle_t. Its value will be set in accordance with the user provided 
    parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_double containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_SetDouble (const xsystem_handle inHandle,
				      const xam_string inName,
				      const xam_double inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.float on the object referenced by the passed in 
    xam_handle_t. Its value will be set in accordance with the user provided 
    parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.
    
    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_double containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetDouble (const xset_handle inHandle,
				   const xam_string inName,
				   const xam_double inValue);


/**
    Changes a property field with a type set to application/vnd.snia.xam.xuid 
    on the object referenced by the passed in xam_handle_t. Its value will be 
    set in accordance with the user provided parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_xuid containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_SetXUID (const xsystem_handle inHandle,
				     const xam_string inName,
				     const xam_xuid inValue);

/**
    Changes a property field with a type set to application/vnd.snia.xam.xuid 
    on the object referenced by the passed in xam_handle_t. Its value will be 
    set in accordance with the user provided parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_xuid containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetXUID (const xset_handle inHandle,
				  const xam_string inName,
				  const xam_xuid inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.string on the object referenced by the passed in
    xam_handle_t. Its value will be set in accordance with the user provided 
    parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_string containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_SetString (const xsystem_handle inHandle,
				       const xam_string inName,
				       const xam_string inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.string on the object referenced by the passed in
    xam_handle_t. Its value will be set in accordance with the user provided 
    parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_string containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetString (const xset_handle inHandle,
				    const xam_string inName,
				    const xam_string inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.datetime on the object referenced by the passed 
    in xam_handle_t. Its value will be set in accordance with the user provided
    parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_datetime containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_SetDatetime (const xsystem_handle inHandle,
					     const xam_string inName,
					     const xam_datetime inValue);

/**
    Changes a property field with a type set to 
    application/vnd.snia.xam.datetime on the object referenced by the passed 
    in xam_handle_t. Its value will be set in accordance with the user provided
    parameter. 

    @note If the field is binding, this will result in a new XUID being 
          assigned to the XSet upon commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inValue A xam_datetime containing the new value to be stored.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetDatetime (const xset_handle inHandle,
					  const xam_string inName,
					  const xam_datetime inValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.boolean on the object referenced by the passed
    in xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_boolean. The value 
                    of the named field is written into this value. The value 
                    that is passed in is not used, and is overwritten with the 
                    result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetBoolean (const xsystem_handle inHandle,
					    const xam_string inName,
					    xam_boolean* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.boolean on the object referenced by the passed
    in xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_boolean. The value 
                    of the named field is written into this value. The value 
                    that is passed in is not used, and is overwritten with the 
                    result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetBoolean (const xset_handle inHandle,
					 const xam_string inName,
					 xam_boolean* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.int on the object referenced by the passed in 
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_int. The value of the 
                    named field is written into this value. The value that is 
                    passed in is not used, and is overwritten with the result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetInt (const xsystem_handle inHandle,
				    const xam_string inName,
				    xam_int* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.int on the object referenced by the passed in 
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_int. The value of the 
                    named field is written into this value. The value that is 
                    passed in is not used, and is overwritten with the result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetInt (const xset_handle inHandle,
				 const xam_string inName,
				 xam_int* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.float on the object referenced by the passed in 
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_double. The value of 
                    the named field is written into this value. The value that 
                    is passed in is not used, and is overwritten with the 
                    result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetDouble (const xsystem_handle inHandle,
				      const xam_string inName,
				      xam_double* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.float on the object referenced by the passed in 
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_double. The value of 
                    the named field is written into this value. The value that 
                    is passed in is not used, and is overwritten with the 
                    result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetDouble (const xset_handle inHandle,
				   const xam_string inName,
				   xam_double* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.xuid on the object referenced by the passed in 
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_xuid. The value of 
                    the named field is written into this value. The value that 
                    is passed 
                    in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetXUID (const xsystem_handle inHandle,
				     const xam_string inName,
				     xam_xuid* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.xuid on the object referenced by the passed in 
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_xuid. The value of 
                    the named field is written into this value. The value that 
                    is passed 
                    in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetXUID (const xset_handle inHandle,
				  const xam_string inName,
				  xam_xuid* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.string on the object referenced by the passed in
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_string. The value of 
                    the named field is written into this value. The value that 
                    is passed in is not used, and is overwritten with the 
                    result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetString (const xsystem_handle inHandle,
				       const xam_string inName,
				       xam_string* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.string on the object referenced by the passed in
    xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_string. The value of 
                    the named field is written into this value. The value that 
                    is passed in is not used, and is overwritten with the 
                    result. 
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetString (const xset_handle inHandle,
				    const xam_string inName,
				    xam_string* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.datetime on the object referenced by the passed 
    in xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_datetime. The value 
                    of the named field is written into this value. The value 
                    that is passed in is not used, and is overwritten with the
                    result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetDatetime (const xsystem_handle inHandle,
					     const xam_string inName,
					     xam_datetime* const outValue);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.datetime on the object referenced by the passed 
    in xam_handle_t. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param outValue A reference to valid storage for a xam_datetime. The value 
                    of the named field is written into this value. The value 
                    that is passed in is not used, and is overwritten with the
                    result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetDatetime (const xset_handle inHandle,
					  const xam_string inName,
					  xam_datetime* const outValue);

/**************************************************************************
 *  method prototypes for XStreams and XStream fields
 *************************************************************************/

/**
    Creates an XStream field with a type set to the user defined mime-type on 
    the object referenced by the passed in xam_handle_t. Its name, mime-type 
    and binding attributes will be set in accordance with the user provided 
    parameters. The XStream field is opened in writeonly mode. 

    @note The value is not set by the method. This method will create an 
          XStream with a length of zero  other methods must be used to add 
          data to this field.
    
    @note If the xam_handle_t contains an XSet, this method may fail with an
          error if the maximum number of fields supported on an XSet is 
          reached. All XSystems must support at least 2^14 (16,384) fields on
          an XSet. However, some XAM storage systems may support more than 
          this. To determine the actual maximum number of fields allowed on an
          XSet an application should evaluate the 
          .xsystem.limits.maxFieldsPerXSet field on the XSystem. For more 
          information on this topic please consult the XAM architecture 
          document.
    
    @note Call the XStream_Close() function one done with the outXStream so 
          others can use if needed.
    
    @note Call the XAM_DeleteField() function to release the resources 
          associated with the created outXStream.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inType A xam_string that contains the mime-type of the field.
    @param outXStream A reference to valid storage for an xstream_handle. The
                      value that is passed in is not used, and is overwritten 
                      with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateXStream (const xsystem_handle inHandle,
					       const xam_string inName,
					       const xam_boolean inBinding,
					       const xam_string inType,
                           xstream_handle* const outXStream);

/**
    Creates an XStream field with a type set to the user defined mime-type on 
    the object referenced by the passed in xam_handle_t. Its name, mime-type 
    and binding attributes will be set in accordance with the user provided 
    parameters. The XStream field is opened in writeonly mode. 

    @note The value is not set by the method. This method will create an 
          XStream with a length of zero  other methods must be used to add 
          data to this field.

    @note If the xam_handle_t contains an XSet, this method may fail with an
          error if the maximum number of fields supported on an XSet is 
          reached. All XSystems must support at least 2^14 (16,384) fields on
          an XSet. However, some XAM storage systems may support more than 
          this. To determine the actual maximum number of fields allowed on an
          XSet an application should evaluate the 
          .xsystem.limits.maxFieldsPerXSet field on the XSystem. For more 
          information on this topic please consult the XAM architecture 
          document.
    
    @note Call the XStream_Close() function one done with the outXStream so 
          others can use if needed.
    
    @note Call the XAM_DeleteField() function to release the resources 
          associated with the created outXStream.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field.
    @param inName A xam_string containing the name of the field to be created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inType A xam_string that contains the mime-type of the field.
    @param outXStream A reference to valid storage for an xstream_handle. The
                      value that is passed in is not used, and is overwritten 
                      with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateXStream (const xset_handle inHandle,
					    const xam_string inName,
					    const xam_boolean inBinding,
					    const xam_string inType,
                        xstream_handle* const outXStream);

/**
    Creates an open XStream in either readonly or writeonly mode, based on
    the mode argument. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xsystem_handle, containing an XSystem Object 
                    reference. This is the object that will contain the new
                    field.  
    @param inName A xam_string containing the name of the field to be created.
    @param inMode A string indicating the mode to open the XStream in:
        o readonly: open for reading. Write methods will fail on the XStream 
                      instance.
        o writeonly: open for writing. Read and seek methods will fail on the 
                       XStream instance. 
        o appendonly: open for writing. Read and seek methods will fail on the 
                        XStream instance. Appends the existing data in the XStream.
    @param outXStream A reference to valid storage for an xstream_handle. The 
                      value that is passed in is not used, and is overwritten 
                      with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_OpenXStream (const xsystem_handle inHandle,
					     const xam_string inName,
					     const xam_string inMode,
					     xstream_handle* const outXStream);

/**
    Creates an open XStream in either readonly or writeonly mode, based on
    the mode argument. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle, containing an XSet Object 
                    reference. This is the object that will contain the new
                    field. 
    @param inName A xam_string containing the name of the field to be created.
    @param inMode A string indicating the mode to open the XStream in:
        o readonly: open for reading. Write methods will fail on the XStream 
                      instance.
        o writeonly: open for writing. Read and seek methods will fail on the 
                       XStream instance. 
        o appendonly: open for writing. Read and seek methods will fail on the 
                        XStream instance. Appends the existing data in the XStream.
    @param outXStream A reference to valid storage for an xstream_handle. The 
                      value that is passed in is not used, and is overwritten 
                      with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_OpenXStream (const xset_handle inHandle,
					  const xam_string inName,
					  const xam_string inMode,
					  xstream_handle* const outXStream);

/**
    Transfers data from the storage system into the target buffer, up to the 
    number of bytes requested.
    
    @note If the inBufferLength is set to a size larger than the actual number
          of bytes of storage available in the inBuffer, undefined results may 
          occur (this includes but is not limited to data loss and data 
          corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method does not block until data is completely read, but will 
        indicate the amount of data that was read in each call. Subsequent 
        calls may be needed to read the remainder of the data.

    @param inHandle An xstream_handle that must have been opened in read mode.  
    @param ioBuffer A byte array to read the data into.
    @param inBufferLength A xam_int set to the number of bytes in the buffer.
    @param outBytesRead A reference to valid storage for a xam_int. On return 
                        this will contain the actual number of bytes read. This 
                        will be less than or equal to the inBufferLength. When 
                        there is no more data to be read, a value of -1 will be 
                        set.  The value that is passed in is not used, and is 
                        overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XStream_Read (const xstream_handle inHandle,
				  char* const ioBuffer,
				  const xam_int inBufferLength,
				  xam_int* const outBytesRead);

/**
    Transfers data from the source buffer to the XAM storage system, up to the 
    number of bytes requested.

    @note This method may fail with an error if the maximum number of bytes 
          supported in an XStream is reached. All XSystems must support at 
          least 2^36 bytes in an XStream. However, some XAM storage systems
          may support more than this. To determine the actual maximum number 
          of bytes allowed in an XStream an application should evaluate the 
          .xsystem.limits.maxSizeOfXStream field on the XSystem. For more 
          information on this topic please consult the XAM architecture 
          document.
    
    @note If the inByteCount is set to a size larger than the actual number of
          bytes of storage available in the inBuffer, undefined results may 
          occur (this includes but is not limited to data loss and data 
          corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method does not block until all the data in the buffer is 
        completely written, but it will indicate the amount of data that was 
        written in each call. Subsequent calls may be needed to write the all 
        of the data.

    @param inHandle An xstream_handle that must have been opened in write mode.
    @param inBuffer A byte array containing the data to be written.
    @param inByteCount A xam_int set to the number of bytes in the buffer to be 
                       written.
    @param outBytesWritten A reference to valid storage for a xam_int. On 
                           return this will contain the actual number of bytes 
                           written. This will be less than or equal to the 
                           inByteCount. The value that is passed in is not used, 
                           and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XStream_Write (const xstream_handle inHandle,
				   const char* inBuffer,
				   const xam_int inByteCount,
				   xam_int* const outBytesWritten);

/**
    Sets the position indicator for the XStream. The new position, measured in 
    bytes, is obtained by adding inOffset bytes to the position specified by 
    inWhence. If inWhence is set to 0, 1, or 2, then the offset is relative to 
    the start of the XStream, the current position, or end-of-data, 
    respectively.

    @note This method can only be used for XStreams opened for read. In 
          addition, this method cannot be used to create sparse files. It is an 
          error to seek past the end of the data in the XStream, as indicated by 
          the field attribute length. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xstream_handle that must have been opened in read mode. 
    @param inOffset A xam_int containing the number of bytes to change the 
                    position by. A positive value moves the cursor forward.
                    A negative value moves the cursor backward.
    @param inWhence A xam_int containing a 0, 1 or 2 (indicating where the 
                    offset should be measured from). 
                    The following constants are provided: 
                    XSTREAM_SEEK_SET(0) - Seek from the start position
                    XSTREAM_SEEK_CUR(1) - Seek from the current position
                    XSTREAM_SEEK_END(2) - Seek from the end position    
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XStream_Seek (const xstream_handle inHandle,
				  const xam_int inOffset,
				  const xam_int inWhence);

/**
    Obtains the current value of the XStream position indicator.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xstream_handle. 
    @param outPosition A xam_int containing the position in the XStream.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XStream_Tell (const xstream_handle inHandle,
				  xam_int* const outPosition);

/**
    An XStream in its normal state will generate an error when an application 
    attempts to close it if there are open asynchronous operations being 
    performed on it. Making this call will change the state of the XStream and 
    allow it to be closed without regard for any open asynchronous operations. 
    Note that the XStream will no longer be usable after this call is made, and 
    the only call that will succeed is an XStream.Close.

    @note This is a VERY DANGEROUS call that may result in data loss if used 
          inappropriately. It is recommended that applications track all open 
          asynchronous operations, and close the asynchronous operations 
          properly as opposed to making this call.
    
    @note If the XStream has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xstream_handle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XStream_Abandon (const xstream_handle inHandle);

/**
    Closes a previously opened XStream. Any resources that were allocated can 
    be released at this point.  
    
    @note Closing an already closed XStream can produce undefined results (this 
          includes but is not limited to data loss and data corruption)

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xstream_handle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XStream_Close (xstream_handle inHandle);

/**************************************************************************
 *  Managing the connection to the XAM Storage System
 *************************************************************************/

/**
    XAM Applications connect to XAM storage systems by calling the connect 
    API method in the XAM API, and specifying the XSystems Uniform Resource 
    Identifier (XRI) string as its parameter. It is expected that the XRI will 
    be specified by the local storage system administrators, and applications 
    should strive to make this easily configured at rum time.

    The XAM Library utilizes this method to create a VIM specific XSystem
    instance handle on which fields may be created. The connection to the
    storage system does not occur until the XAM Library calls the 
    VIM_XSystem_Connect method on this handle.

    @note The XSystem is not fully usable until it has been connected 
          and authenticated.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param outHandle  A reference to valid storage for an xsystem_handle. On 
                      return this will contain the XSystem handle that was 
                      created, on which fields may be created/updated. 
                      The value that is passed in is not used, and is 
                      overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_CreateXSystem (xsystem_handle* const outHandle);

/**
    XAM Applications connect to XAM storage systems by calling the connect 
    API method in the XAM API, and specifying the XSystems Uniform Resource 
    Identifier (XRI) string as its parameter. It is expected that the XRI will 
    be specified by the local storage system administrators, and applications 
    should strive to make this easily configured at rum time.

    The XAM Library utilizes this method to initiate a connection to 
    an (already created) XSystem instance.

    @note The XSystem is not fully usable until it has been authenticated.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A reference to valid storage for an xsystem_handle. 
                    This contains an XSystem handle that was created by a call to
                    VIM_CreateXSystem.
    @param inXRI A xam_string. It contains the XSystems Uniform Resource 
                 Identifier. A BNF of this format is listed below:
                    
                      xsystemname[?param=value[{&param=value}]]

                 The xsystemname is vendor specific  it may be an IP address, or 
                 some other id. It may not contain /, ?, or ! characters.
                 Additionally, param=value pairs can be specified
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_Connect (const xsystem_handle inHandle,
                     const xam_string inXRI);

/**
    Allows an application to authenticate an XSystem. It provides a generic 
    interface to exchange data as part of the authentication process. The 
    application should check the XSystem property xyz to determine which 
    patterns of authentication (mechanisms) are available for use. After a 
    pattern is selected, the appropriate sequence of data exchanges should be 
    made (using this call) in order to authenticate. A failed authentication 
    will make the XSystem unusable  applications cannot repeat failed 
    authentications using the same XSystem.
    
    @note The outXStream must be closed (using XStream_Close() function) when
          the application has finished its authentication processing.
    
    @note If the XSystem has been closed, or if the inByteCount is set to a 
          size larger than the actual number of bytes of storage available in 
          the inBuffer, undefined results may occur (this includes but is not 
          limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inBuffer Data that is being passed to the authentication mechanism 
                    is passed in this array of bytes.
    @param inByteCount The number of significant bytes in the passed in buffer.
    @param outXStream A reference to valid storage for an xstream_handle. On 
                      return this will contain the XStream handle that was 
                      created, and which contains the systems response to the 
                      authentication information. The value that is passed in 
                      is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_Authenticate (const xsystem_handle inHandle,
						  const char* inBuffer,
						  const xam_int inByteCount,
						  xstream_handle* const outXStream);

/**
    Called to release any resources associated with an XSystem. After calling
    this method, the closed XSystem should not be used. 

    @note This call will fail if there are any open XSets associated with this
          XSystem.

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.
    
    @param inHandle An xsystem_handle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_Close (const xsystem_handle inHandle);

/**
    An XSystem in its normal state will generate an error when an application 
    attempts to close it if it has open XSets in it. Making this call will 
    change the state of the XSystem and allow it to be closed without regard 
    for any open XSets. Note that the XSystem will no longer be usable after 
    this call is made, and the only call that will succeed is an XSystem.Close.

    @note This is a VERY DANGEROUS call that may result in data loss if used 
          inappropriately. It is recommended that applications track all open 
          XSets, and close the XSets properly as opposed to making this call.

   @note If the XSystem has been closed undefined results may occur (this 
         includes but is not limited to data loss and data corruption).
   
   Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_Abandon (const xsystem_handle inHandle);

/**
    Deletes an XSet from the XSystem.  
    
    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be deleted.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_DeleteXSet (const xsystem_handle inHandle,
						const xam_xuid inXUID);

/**
    Places an XSet on hold. A held XSet cannot be changed in any way (e.g. the 
    XSet can only be opened in read mode and commits of a held XSet will fail).  
    
    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be held.
    @param inHoldID A xam_string that contains the ID to be associated with the
                    hold.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_HoldXSet (const xsystem_handle inHandle,
					  const xam_xuid inXUID,
					  const xam_string inHoldID);
/**
    Releases a specific hold on an XSet (associated with the hold id).  

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be held.
    @param inHoldID A xam_string that contains the ID associated with the hold.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_ReleaseXSet (const xsystem_handle inHandle,
						 const xam_xuid inXUID,
						 const xam_string inHoldID);

/**
    Checks the accessibility of an XSet on the XSystem. It is not an error if 
    the XSet does not exist on the XSystem: such an XSet shall be noted as 
    being inaccessible.  
    
    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be checked.
    @param inMode The bitwise OR of the access permissions to be checked: 
                   - XSET_ACCESS_READ_OK for read permission
                   - XSET_ACCESS_WRITE_OK for write permission
                   - XSET_ACCESS_DELETE_OK for delete permission
    @param outIsAccessible A reference to valid storage for a xam_boolean. 
                           On return this will be set to true if the XSet is 
                           accessible in accordance with the access permissions
                           set by mode, false otherwise. The value that is 
                           passed in is not used, and is overwritten with the 
                           result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_AccessXSet (const xsystem_handle inHandle,
						const xam_xuid inXUID,
                        const xam_int inMode,
						xam_boolean* const outIsAccessible);

/**
    Evaluates all retention criteria that exists on a given XSet, specified 
    as a xam_xuid, and shall return TRUE if there exists retention criteria 
    which would prohibit XSet deletion. The method returns FALSE if all XSet
    retention criteria, i.e., retention criteria associated with the 
    base, event, and .xset.retention.list.<retention id> identifiers, have
    been met. This method does not evaluate the on-hold status. 
    
    A non-fatal error will be returned if the specified XUID is improperly 
    formatted, does not exist in the XSystem, or if the caller is not 
    authorized to access the XSet.
    
    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be checked.
    @param outIsRetained A reference to valid storage for a xam_boolean. 
                         On return this will be set to true if the XSet is 
                         under retention in accordance with the XSet retention
                         criteria, false otherwise. The value that is 
                         passed in is not used, and is overwritten with the 
                         result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_IsXSetRetained (const xsystem_handle inHandle,
                            const xam_xuid inXUID,
                            xam_boolean* const outIsRetained);
 
/**
    Gets the time at which the XSet was last opened or committed, whichever is 
    the most recent.  
    
    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be checked.
    @param outAccessTime A reference to valid storage for a xam_datetime. On 
                         return this will be set to the time at which the XSet
                         was last opened or committed, whichever is the most 
                         recent. The value that is passed in is not used, and 
                         is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_GetXSetAccessTime (const xsystem_handle inHandle,
                               const xam_xuid inXUID,
                               xam_datetime* const outAccessTime);

/**
    Creates a new empty XSet associated with the XSystem. Note that this XSet 
    will not exist on the XSystem unless that XSet is committed.  
    
    @note If the XSystem has been closed undefined results may occur (this
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inMode A string indicating the mode to open the XSet in. 
        Possible values are:
        o restricted - open for reading and limited writing. Adding, deleting
                         or modifying fields that are binding is not allowed. 
                         Changing fields from binding to nonbinding (or vice 
                         versa) is not allowed. Commit of the XSet instance 
                         will fail if any binding fields havebeen modified. 
                         Successful commit of the XSet will never generate a 
                         new XUID.
        o unrestricted - open for reading and writing. There are no limits 
                           on adding, deleting or modifying fields; nor are 
                           there limits on changing fields from binding to 
                           nonbinding (or vice versa). Successful commit of the
                           XSet will generate a new XUID if any binding fields 
                           have been added, deleted, or modified, or if any 
                           fields have been changed from binding to nonbinding 
                           (or vice versa).
    @param outXSet A reference to valid storage for an xset_handle. The value
                   that is passed in is not used, and is overwritten with the 
                   result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CreateXSet (const xsystem_handle inHandle,
                        const xam_string inMode,
						xset_handle* const outXSet);

/**
    Creates or modifies a property field with the name of 
    .xset.access.policy and a type set to application/vnd.snia.xam.string 
    on the object referenced by the passed-in xset_handle. Its value and 
    binding attributes will be set according to the user-provided parameters. 
    This field will be used by the XAM Storage System to determine the policies
    to use when accessing this XSet..

    @note If an access policy has not been applied to an XSet at the time of 
          the initial commit, then the property will be created and set as the 
          default access policy of the XSystem (i.e. the first string in the 
          XSystem AccessPolicyList).

    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ApplyAccessPolicy (const xset_handle inHandle,
                            const xam_boolean inBinding,
                            const xam_string inPolicy);

/**
    Removes all access fields from the XSet. 

    @note If an access policy has not been applied to an XSet at the time of 
          the initial commit, then the property will be created and set as 
          the default access policy of the XSystem (i.e., the first string 
          in the XSystem AccessPolicyList).
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that contains 
                    the access field(s).
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ResetAccessFields (const xset_handle inHandle);

/**
    Creates a property field with the name of xam.management_policy and a 
    type set to application/vnd.snia.xam.string on the object referenced by
    the passed in xam_handle_t. Its value and binding attributes will be set in
    accordance with the user provided parameters. This field will be used by 
    the XAM storage system to determine the default policies to use when 
    managing this XSet.

    @note If a management policy has not been applied to an XSet at the time 
          of the initial commit, then the property will be created and set as
          the default management policy of the XSystem (i.e. first string in 
          the XSystem ManagementPolicyList).

    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_ApplyManagementPolicy (const xset_handle inHandle,
								const xam_boolean inBinding,
								const xam_string inPolicy);

/**
    Removes all management fields from the XSet. This includes the 
    .xset.retention.start_time; because this is a binding field, calling this 
    method will always result in a new XUID being assigned to this XSet at the 
    next commit. 

    @note If a management policy has not been applied to an XSet at the time of
          the initial commit, then the property will be created and set as the 
          default management policy of the XSystem (i.e. first string in the 
          XSystem ManagementPolicyList).
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_ResetManagementFields (const xset_handle inHandle);

/**
    Creates a scope to for storing and evaluating retention criteria. It 
    creates a field with a type of application/vnd.snia.xam.string and 
    sets the value to the retention id. The field name is formed by appending
    the retention id to the following prefix: .xset.retention.list.. Thus 
    the final format of the name is .xset.retention.list.<retention id>. It 
    will have its binding attribute set in accordance with the binding flag 
    set by the application.

    @note Changing this field from binding to nonbinding (or vice versa) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being created.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_CreateRetention (const xset_handle inHandle,
                          const xam_boolean inBinding,
                          const xam_string inRetentionID);

/**
    Enables or disables retention scoped by the specified retention id. 
    This flag is stored in a field of type application/vnd.snia.xam.boolean.
    The name of the field is formed by inserting the retention id between a 
    prefix (.xset.retention.) and a suffix (.enabled); thus the final format 
    of the name is .xset.retention.<retention id>.enabled. If the field does 
    not exist it will be created; otherwise the value will be updated if and 
    only if the value is changed from false to true - if the value is set to 
    true it cannot be changed. It will have its binding attribute set in 
    accordance with the binding flag set by the application.

    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inEnabled A xam_boolean containing a flag indicating if event 
                     retention is enabled on this XSet or not. If the flag is
                     set to true, event retention is enabled, otherwise it is 
                     disabled.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetRetentionEnabledFlag (const xset_handle inHandle,
                                  const xam_string inRetentionID,
                                  const xam_boolean inBinding,
                                  const xam_boolean inEnabled);

/**
    This method will enabled or disable retention scoped by the specified 
    retention id. The policy name of the policy holding the enabled flag 
    is stored in a field of type application/vnd.snia.xam.string. The name
    of the field is formed by inserting the retention id between a prefix 
    (.xset.retention.) and a suffix (.enabled.policy); thus the final format 
    of the name is .xset.retention.<retention id>.enabled.policy. If the field 
    does not exist it will be created; otherwise the value will be updated if 
    and only if the value is changed from false to true - if the value is set 
    to true it cannot be changed. It will have its binding attribute set in 
    accordance with the binding flag set by the application.

    @note If the .xset.retention.<retention id>.enabled field is also present 
          on the XSet, it will be used by the XAM Storage System in preference 
          to this field.
    
    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ApplyRetentionEnabledPolicy (const xset_handle inHandle,
                                      const xam_string inRetentionID,
                                      const xam_boolean inBinding,
                                      const xam_string inPolicy);

/**
    Sets the duration of retention scoped by the specified retention id. 
    This flag is stored in a field of type application/vnd.snia.xam.int. 
    The name of the field is formed by inserting the retention id between 
    a prefix (.xset.retention.) and a suffix (.duration); thus the final 
    format of the name is .xset.retention.<retention id>.duration. If the 
    field does not exist it will be created; otherwise the value will be 
    updated if and only if the duration is increased. It will have its 
    binding attribute set in accordance with the binding flag set by the 
    application.

    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inDuration A xam_int containing the amount of time (measured in 
                      milliseconds from the time of commit) to retain the XSet. 
                      Zero indicates no retention, while a negative one (-1) 
                      indicates infinite retention.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetRetentionDuration (const xset_handle inHandle,
                               const xam_string inRetentionID,
                               const xam_boolean inBinding,
                               const xam_int inDuration);

/**
    Sets the duration of retention scoped by the specified retention id. This 
    policy name is stored in a field of type application/vnd.snia.xam.string.
    The name of the field is formed by inserting the retention id between a 
    prefix (.xset.retention.) and a suffix (.duration.policy); thus the final 
    format of the name is .xset.retention.<retention id>.duration.policy. If 
    the field does not exist it will be created; otherwise the value will be 
    updated if and only if the duration is increased. It will have its binding 
    attribute set in accordance with the binding flag set by the application.

    @note If the .xset.retention.<retention id>.duration field is also present 
          on the XSet, it will be used by the XAM Storage System in preference 
          to this field.
    
    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ApplyRetentionDurationPolicy (const xset_handle inHandle,
                                       const xam_string inRetentionID,
                                       const xam_boolean inBinding,
                                       const xam_string inPolicy);
/**
    Sets the start time of retention scoped by the specified retention id. The 
    current time of the XSystem is stored in a field of type 
    application/vnd.snia.xam.datetime. The name of the field is formed by 
    inserting the retention id between a prefix (.xset.retention.) and a suffix 
    (.starttime); thus the final format of the name is 
    .xset.retention.<retention id>.starttime. If the field does not exist it 
    will be created; otherwise an error will be generate as it is not allowed 
    to change the starttimme once set. It will have its binding attribute
    set in accordance with the binding flag set by the application.

    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being created.
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetRetentionStarttime (const xset_handle inHandle,
                                const xam_string inRetentionID,
                                const xam_boolean inBinding);

/**
    If this XSet does not already contain the field .xset.retention.list.base,
    this method will create the field with a type of 
    application/vnd.snia.xam.string and set the value to base. It will also 
    create the application/vnd.snia.xam.boolean field 
    .xset.retention.base.enabled and set the value to true. The duration will 
    be stored in a field named .xset.retention.base.duration. This field is of 
    type application/vnd.snia.xam.int. If the field already exists, its value 
    will be changed to match the passed in duration if and only if the duration
    of the retention is not reduced; the method will generate an error if the 
    duration is reduced. If the field does not already exist, it will be created 
    with the specified duration as the value. These fields will have their 
    binding attribute set in accordance with the binding flag set by the 
    application. These fields will be used by the XAM Storage System to 
    determine the base retention duration to use when managing this XSet.

    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    @note When an XSet instance containing the field .xset.retention.list.base 
          is first committed, the field .xset.retention.base.starttime will be 
          created and have its value set to .xset.xuidtime.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inDuration A xam_int containing the amount of time (measured in 
                      milliseconds from the time of commit) to retain the XSet. 
                      Zero indicates no retention, while a negative one (-1) 
                      indicates infinite retention.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetBaseRetention (const xset_handle inHandle,
                           const xam_boolean inBinding,
                           const xam_int inDuration);

/**
    If this XSet does not already contain the field .xset.retention.list.base,
    this method will create the field with a type of 
    application/vnd.snia.xam.string and set the value to base. It will also
    create the application/vnd.snia.xam.boolean field 
    .xset.retention.base.enabled and set the value to true. The duration policy 
    will be stored in a field named .xset.retention.base.duration.policy. This 
    field is of type application/vnd.snia.xam.string. If the field already 
    exists, its value will be changed to match the passed in policy if and only 
    if the policy would not reduce the duration of the retention; the method 
    will generate an error if the policy reduces the duration. If the field 
    does not already exist, it will be created with the specified policy name
    as the value. These fields will have their binding attribute set in 
    accordance with the binding flag set by the application. These fields will 
    be used by the XAM Storage System to determine the base retention duration 
    to use when managing this XSet.

    @note If the .xset.retention.base.duration field is also present on the 
          XSet, it will be used by the XAM Storage System in preference to 
          this policy field.

    @note When an XSet instance containing the field .xset.retention.list.base
          is first committed, the field .xset.retention.base.starttime will be 
          created and have its value set to .xset.xuidtime.
    
    @note Changing this field from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ApplyBaseRetentionPolicy (const xset_handle inHandle,
                                   const xam_boolean inBinding,
                                   const xam_string inPolicy);

/**
    Creates a property field on the specified XSet with the name of 
    xam.autodelete_policy and a type set to application/vnd.snia.xam.string
    Its value and binding attributes will be set in accordance with the user 
    provided parameters. This field will be used by the XAM storage system to 
    determine if the XSet should be automatically deleted upon expiration of 
    retention. Applying the policy will also remove the xam.autodelete from 
    the XSet.

    @note If the explicit duration field is also present on the XSet 
          (xam.autodelete) it will be used by the XAM storage system in 
          preference to this field. 
    
    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.  
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_ApplyAutoDeletePolicy (const xset_handle inHandle,
								const xam_boolean inBinding,
								const xam_string inPolicy);

/**
    Creates a property field on the specified XSet with the name of 
    xam.autodelete and a type set to application/vnd.snia.xam.boolean. Its
    value and binding attributes will be set in accordance with the user 
    provided parameters. This field will be used by the XAM storage system to 
    determine if the XSet should be automatically deleted upon expiration of 
    retention. Applying the policy will also remove the xam.autodelete_policy
    field from the XSet.

    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being assigned
          on commit. 
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inAutoDelete A xam_boolean containing a flag indicating if 
                        autodelete is enabled on this XSet or not. If the flag
                        is set to true, autodelete is enabled, otherwise it is 
                        disabled.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_SetAutoDelete (const xset_handle inHandle,
						const xam_boolean inBinding,
						const xam_boolean inAutoDelete);


/**
    If this XSet does not have an auto shred policy applied to it, this method 
    will create a property field on the specified XSet with the name of 
    .xset.shred.policy and a type set to application/vnd.snia.xam.string. 
    Its value and binding attributes will be set according to the user-provided 
    parameters. If the field already exists on the XSet, then its value will be 
    updated with the specified value. This field will be used by the XAM Storage 
    System to determine if the XSet should be shredded after XSet deletion. 
    If the .xset.shred field is also present on the XSet it will be used by the 
    XAM Storage System in preference to this field.

    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_ApplyShredPolicy (const xset_handle inHandle,
                           const xam_boolean inBinding,
                           const xam_string inPolicy);

/**
    If this XSet does not have auto shred set on it, this method will create a 
    property field on the specified XSet with the name of .xset.shred and a 
    type set to application/vnd.snia.xam.boolean. Its value and binding 
    attributes will be set according to the user-provided parameters. If the 
    field already exists on the XSet, then its value will be updated with the 
    specified value. This field will be used by the XAM Storage System to
    determine if the XSet should be shredded after deletion.

    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being assigned
          on commit. 
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inShred A xam_boolean containing a flag indicating if shredding is 
                   enabled on this XSet or not. If the flag is set to TRUE, 
                   shredding is enabled, otherwise it is disabled.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_SetShred (const xset_handle inHandle,
                   const xam_boolean inBinding,
                   const xam_boolean inShred);

/**
    Creates a property field on the specified XSet with the name of 
    xam.storage_policy and a type set to application/vnd.snia.xam.string. 
    Its value and binding attributes will be set in accordance with the user 
    provided parameters. This field will be used by the XAM storage system to 
    determine the storage policy of the XSet. 

    @note Changing this policy from binding to nonbinding (or the converse) 
          will result in a new XSet being created and a new XUID being 
          assigned on commit.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field. 
    @param inBinding A xam_boolean set to true if the field should be binding; 
                     or false otherwise.
    @param inPolicy A xam_string containing the name of the policy to be 
                    applied.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_ApplyStoragePolicy (const xset_handle inHandle,
							 const xam_boolean inBinding,
							 const xam_string inPolicy);

/**
    Evaluates all factors that affect the retention duration that is 
    currently in effect for the given retention id, and returns that
    duration to the caller.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain 
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being checked.
    @param outDuration A reference to valid storage for a xam_int. On return 
                       this will be set to the actual minimum retention duration 
                       that is currently being in effect for the XSet after 
                       evaluating the policies. The value that is passed in is 
                       not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetActualRetentionDuration (const xset_handle inHandle,
                                     const xam_string inRetentionID,
                                     xam_int* const outDuration);

/**
    Evaluates all factors that affect if this retention is enabled for the 
    XSet, and return that enabled state to the caller.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param inRetentionID A xam_string containing the retention identifier of the 
                         retention being checked.
    @param outEnabled A reference to valid storage for a xam_boolean. On return
                      this will be set to match the enabled state in effect for 
                      the XSet after evaluating the policies. The value that is 
                      passed in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetActualRetentionEnabled (const xset_handle inHandle,
                                    const xam_string inRetentionID,
                                    xam_boolean* const outEnabled);
/**
    Evaluates all factors that affect if auto delete is enabled for the XSet, 
    and return that enabled state to the caller.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param outEnabled A reference to valid storage for a xam_boolean. On return
                      this will be set to match the enabled state in effect for 
                      the XSet after evaluating the policies. The value that is 
                      passed in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_GetActualAutoDelete (const xset_handle inHandle,
							  xam_boolean* const outEnabled);

/**
    Evaluates all factors that affect if shredding is enabled for the XSet and 
    return that enabled state to the caller.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle A valid xset_handle. This is the object that will contain
                    the new field. 
    @param outEnabled A reference to valid storage for a xam_boolean. On return
                      this will be set to match the enabled state in effect for 
                      the XSet after evaluating the policies. The value that is 
                      passed in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSet_GetActualShred (const xset_handle inHandle,
                         xam_boolean* const outEnabled);

/**************************************************************************
 *  method prototypes for editing an XSet
 *************************************************************************/

/**
    Opens an XSet in the XSystem.  

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be opened.
    @param inMode A string indicating the mode to open the XSet in:
        o readonly - open for reading. Adding, deleting or modifying fields 
                       is not allowed. Commit of the XSet instance will fail.
        o restricted - open for reading and limited writing. Adding, deleting
                         or modifying fields that are binding is not allowed. 
                         Changing fields from binding to nonbinding (or vice 
                         versa) is not allowed. Commit of the XSet instance 
                         will fail if any binding fields havebeen modified. 
                         Successful commit of the XSet will never generate a 
                         new XUID.
        o unrestricted - open for reading and writing. There are no limits 
                           on adding, deleting or modifying fields; nor are 
                           there limits on changing fields from binding to 
                           nonbinding (or vice versa). Successful commit of the
                           XSet will generate a new XUID if any binding fields 
                           have been added, deleted, or modified, or if any 
                           fields have been changed from binding to nonbinding 
                           (or vice versa).
        o copy  open for reading and writing. There are no limits on adding,
                   deleting or modifying fields; nor are there limits on 
                   changing fields from binding to nonbinding (or vice versa). 
                   The first successful commit will always generate a new XUID. 
                   Subsequent successful commits of the XSet will generate a 
                   new XUID if any binding fields have been added, deleted, or 
                   modified, or if any fields have been changed from binding to 
                   nonbinding (or vice versa). 
    @param outXSet A reference to valid storage for a xset_handle. On return 
                   this will contain the XSet handle. The value that is passed
                   in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSystem_OpenXSet (const xsystem_handle inHandle,
					  const xam_xuid inXUID,
                      const xam_string inMode,
					  xset_handle* const outXSet);

/**
    Creates a copy of an XSet in the XSystem, returning a handle to an
    XSet instance associated with the XSystem. This XSet will not exist 
    on the XSystem unless that XSet instance is committed.  

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete. For applications that wish 
        to use a non-blocking version of this method, refer to 
        "XSystem_AsyncCopyXSet".

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be opened.
    @param inMode A string indicating the mode to open the XSet in:
        o restricted - open for reading and limited writing. Adding, deleting
                         or modifying fields that are binding is not allowed. 
                         Changing fields from binding to nonbinding (or vice 
                         versa) is not allowed. Commit of the XSet instance 
                         will fail if any binding fields havebeen modified. 
                         Successful commit of the XSet will never generate a 
                         new XUID.
        o unrestricted - open for reading and writing. There are no limits 
                           on adding, deleting or modifying fields; nor are 
                           there limits on changing fields from binding to 
                           nonbinding (or vice versa). Successful commit of the
                           XSet will generate a new XUID if any binding fields 
                           have been added, deleted, or modified, or if any 
                           fields have been changed from binding to nonbinding 
                           (or vice versa).
    @param outXSet A reference to valid storage for a xset_handle. On return 
                   this will contain the XSet handle. The value that is passed
                   in is not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL
VIM_XSystem_CopyXSet (const xsystem_handle inHandle,
                      const xam_xuid inXUID,
                      const xam_string inMode,
                      xset_handle* const outXSet);

/**
    Stores an XSet in the XSystem. Note this does not close the XSet, which can
    still be modified as allowed by the authorization of the XSystem. A XUID 
    will be assigned by the XAM storage system and this XUID will be returned.

    Open XStreams will not cause the commit to fail. Only the data that was 
    successfully written to such XSteams will be committed.

    If this is a modified XSet (e.g. an existing XSet was opened and changed) 
    then a new XUID may or may not be assigned by the commit, in accordance 
    with the following rules:
        - If only variable fields are edited (created, deleted, or changed) 
          then the XAM storage system may not assign a new XUID.
        - If any binding fields are edited (created, deleted, or changed) then
          the XAM storage system must assign a new XUI.

   In any case, an application should be coded to handle cases where the XUID 
   changes when a modified XSet is committed.  

   If a management policy has not been applied to the XSet prior to commit, a 
   default management policy will be applied to the XSet at the time of commit.

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xset_handle.
    @param outXUID A reference to valid storage for a XUID. On return this will
                   contain the XUID that was assigned to the XSet by the XAM 
                   storage system. The value that is passed in is not used, and 
                   is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_Commit (const xset_handle inHandle,
				 xam_xuid* const outXUID);

/**
    Releases any resources associated with an XSet. After calling this method, 
    the closed XSet should not be used. 

    @note This call will fail if there are any open XStreams associated with 
          this XSet.
    
    @note if the XSet has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xset_handle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_Close (const xset_handle inHandle);

/**
    An XSet in its normal state will generate an error when an application 
    attempts to close it if there are open XStreams in it. Making this call 
    will change the state of the XSet and allow it to be closed without regard
    for any open XStreams. Note that the XSet will no longer be usable after 
    this call is made, and the only call that will succeed is an XSet.Close.

    @note this is a VERY DANGEROUS call that may result in data loss if used
          inappropriately. It is recommended that applications track all open 
          XStreams, and close the XStreams properly as opposed to making this 
          call.

    @note If the XSet has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xset_handle
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_Abandon (const xset_handle inHandle);

/**************************************************************************
 *  method prototypes for XSet migration
 *************************************************************************/

/**
    Opens an export XStream for the XSet. The XSet must have been committed, 
    and must not have been modified since it was opened / committed. The XSet 
    will enter an import/export state, and will thus generate errors if used 
    for any operation until the export XStream is closed. The original XSet 
    referred to by the XSet handle will be overwritten. 

    The XStream will contain a canonical representation of the XSet. This data 
    can be read from the XStream using normal XStream calls and semantics. When 
    the XStream is closed the XSet will return to a normal state.
    
    
    @note If the XSet has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xset_handle.
    @param outXStream A reference to valid storage for a xstream_handle. On 
                      return this will contain the XStream handle of an XStream
                      opened in readonly mode. The value that is passed in is 
                      not used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_OpenExportXStream (const xset_handle inHandle,
							xstream_handle* const outXStream);

/**
    Opens an import XStream for the XSet. The XSet will enter an import/export
    state, and will thus generate errors if used for any operation until the 
    XStream is closed. Any data in the original XSet instance will be 
    overwritten. 

    It is expected that a data stream containing the canonical representation 
    of an XSet will be written into the XStream. When the XStream is closed, 
    the data will be validated. If the data is determined to be valid, then the
    XSet will return to a normal state (i.e. will no longer generate errors 
    when operated on) but it will now refer to the XSet that was described by 
    the canonical data that was written to the XStream. If the validation of the
    data fails (i.e. it contains invalid or improperly formatted data) then the 
    XSet will enter a corrupted state. It will no longer be recoverable and all 
    operations except abandon (followed by close) will fail.

    After a successful validation, the XSet fields can be examined as any 
    normal fields. The XSet can be modified. The XSet is not committed, but it
    is in all ways a normal XSet, and may be committed as per normal XSet 
    semantics. If the XSet is committed prior to any modification to binding 
    fields (adding, modifying or deleting binding fields; or changing the 
    binding attribute of any fields) then the XUID will be the XUID described 
    by the import XStream. Modification to any binding fields as decribed above 
    will result in a new XUID being assigned upon commit.

    @note If the XSet has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle an xset_handle.
    @param outXStream A reference to valid storage for a xstream_handle. On 
                      return this will contain the XStream handle of an XStream
                      opened in w mode. The value that is passed in is not 
                      used, and is overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_OpenImportXStream (const xset_handle inHandle,
							xstream_handle* const outXStream);

/**************************************************************************
 *  method prototypes for job control
 *************************************************************************/

/**
    Submits a job request to the XAM storage system. Fields on the XSet will be
    evaluated as input to the job in accordance with the semantics of the XAM 
    job control subsystem (refer to the XAM architecture document for more 
    details). This XSet will be used to communicate health and status 
    information about the job, as well as any results from the job.  

    @note If the XSet has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xset_handle
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_SubmitJob (const xset_handle inHandle);

/**
    Stops a currently running job in XAM storage system, if the XSet was used
    to start a job. Fields on the XSet will be evaluated as input to the job in
    accordance with the semantics of the XAM job control subsystem (refer to 
    the XAM architecture document for more details).  

    @note If the XSet has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xset_handle
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_HaltJob (const xset_handle inHandle);


/**************************************************************************
 *  method prototypes for asynchronous i/o
 *************************************************************************/

/**
    Asynchronously opens an XSet in the XSystem.  

    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be opened.
    @param inMode A string indicating the mode to open the XSet in:
       o readonly - open for reading. Adding, deleting or modifying fields 
                       is not allowed. Commit of the XSet instance will fail.
       o restricted - open for reading and limited writing. Adding, deleting
                        or modifying fields that are binding is not allowed. 
                        Changing fields from binding to nonbinding (or vice 
                        versa) is not allowed. Commit of the XSet instance 
                        will fail if any binding fields havebeen modified. 
                        Successful commit of the XSet will never generate a 
                        new XUID.
       o unrestricted - open for reading and writing. There are no limits 
                          on adding, deleting or modifying fields; nor are 
                          there limits on changing fields from binding to 
                          nonbinding (or vice versa). Successful commit of the
                          XSet will generate a new XUID if any binding fields 
                          have been added, deleted, or modified, or if any 
                          fields have been changed from binding to nonbinding 
                          (or vice versa).
    
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSystem_AsyncOpenXSet (const xsystem_handle inHandle,
                           const xam_xuid inXUID,
                           const xam_string inMode,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle);

/**
    Begins the asynchronous copying of an XSet in the XSystem, ultimately 
    returning a handle to an XSet instance associated with the XSystem. 
    The specified callback will be invoked as part of the asynchronous 
    copying. To monitor the status of this operation, the application can 
    poll the Async instance that is generated by this method. A handle to 
    an XAsync instance is also passed to any provided callback method when 
    that callback method is invoked.

    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle An xsystem_handle.
    @param inXUID The XUID of the XSet to be opened.
    @param inMode A string indicating the mode to copy the XSet in:
       o restricted - open for reading and limited writing. Adding, deleting
                        or modifying fields that are binding is not allowed. 
                        Changing fields from binding to nonbinding (or vice 
                        versa) is not allowed. Commit of the XSet instance 
                        will fail if any binding fields havebeen modified. 
                        Successful commit of the XSet will never generate a 
                        new XUID.
       o unrestricted - open for reading and writing. There are no limits 
                          on adding, deleting or modifying fields; nor are 
                          there limits on changing fields from binding to 
                          nonbinding (or vice versa). Successful commit of the
                          XSet will generate a new XUID if any binding fields 
                          have been added, deleted, or modified, or if any 
                          fields have been changed from binding to nonbinding 
                          (or vice versa).
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSystem_AsyncCopyXSet (const xsystem_handle inHandle,
                           const xam_xuid inXUID,
                           const xam_string inMode,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle);

/**
    Asynchronously creates an open XStream instance in either readonly 
    or writeonly mode, based on the mode argument. 

    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle A valid XSystem handle which contains the XStream field.
    @param inName A xam_string containing the name of the field to be created.
    @param inMode A string indicating the mode to open the XStream in:
        o readonly: open for reading. Write methods will fail on the XStream 
                      instance.
        o writeonly: open for writing. Read and seek methods will fail on the 
                       XStream instance. 
        o appendonly: open for writing. Read and seek methods will fail on the 
                        XStream instance. Appends the existing data in the XStream.
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSystem_AsyncOpenXStream (const xsystem_handle inHandle,
                              const xam_string inName,
                              const xam_string inMode,
                              const XOPID inXOPID,
                              xasync_callback inCallback,
                              xasync_handle* const outAsyncHandle);

/**
    Asynchronously creates an open XStream instance in either readonly 
    or writeonly mode, based on the mode argument. 

    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle A valid XSet handle which contains the XStream field.
    @param inName A xam_string containing the name of the field to be created.
    @param inMode A string indicating the mode to open the XStream in:
        o readonly: open for reading. Write methods will fail on the XStream 
                      instance.
        o writeonly: open for writing. Read and seek methods will fail on the 
                       XStream instance. 
        o appendonly: open for writing. Read and seek methods will fail on the 
                        XStream instance. Appends the existing data in the XStream.
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_AsyncOpenXStream (const xset_handle inHandle,
                           const xam_string inName,
                           const xam_string inMode,
                           const XOPID inXOPID,
                           xasync_callback inCallback,
                           xasync_handle* const outAsyncHandle);

/**
    Asynchronously transfers data from the storage system into the target 
    buffer, up to the number of bytes requested.
   
    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    @note If the inBufferLength is set to a size larger than the actual 
          number of bytes of storage available in the inBuffer, undefined 
          results may occur (this includes but is not limited to data loss and
          data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle An xstream_handle that must have been opened in read mode. 
    @param ioBuffer A byte array to read the data into.
    @param inBufferLength A xam_int set to the number of bytes in the buffer.
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XStream_AsyncRead (const xstream_handle inHandle,
                       char* const ioBuffer,
                       const xam_int inBufferLength,
                       const XOPID inXOPID,
                       xasync_callback inCallback,
                       xasync_handle* const outAsyncHandle);

/**
    Asynchronously transfers data from the source buffer to the XAM storage
    system, up to the number of bytes requested.

    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    @note This method may fail with an error if the maximum number of bytes
          supported in an XStream is reached. All XSystems must support at 
          least 2^36 bytes in an XStream. However, some XAM storage systems 
          may support more than this. To determine the actual maximum number
          of bytes allowed in an XStream an application should evaluate the 
          .xsystem.limits.maxSizeOfXStream field on the XSystem. For more 
          information on this topic please consult the XAM architecture 
          document.
    
    @note If the inByteCount is set to a size larger than the actual number of
          bytes of storage available in the inBuffer, undefined results may 
          occur (this includes but is not limited to data loss and data 
          corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle An xstream_handle that must have been opened in write mode. 
    @param inBuffer A byte array containing the data to be written.
    @param inByteCount A xam_int set to the number of bytes in the buffer to be 
                       written.
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XStream_AsyncWrite (const xstream_handle inHandle,
                        const char* inBuffer,
                        const xam_int inByteCount,
                        const XOPID inXOPID,
                        xasync_callback inCallback,
                        xasync_handle* const outAsyncHandle); 

/**
    Asynchronously stores an XSet in the XSystem. Note this does not close 
    the XSet, which can still be modified as allowed by the authorization 
    of the XSystem. A XUID will be assigned by the XAM storage system and this
    XUID will be returned.

    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    Open XStreams will not cause the commit to fail. Only the data that was 
    successfully written to such XSteams will be committed.

    If this is a modified XSet (e.g. an existing XSet was opened and changed) 
    then a new XUID may or may not be assigned by the commit, in accordance 
    with the following rules:
        - If only variable fields are edited (created, deleted, or changed) 
          then the XAM storage system may not assign a new XUID.
        - If any binding fields are edited (created, deleted, or changed) then
          the XAM storage system must assign a new XUI.

   In any case, an application should be coded to handle cases where the XUID 
   changes when a modified XSet is committed.  

   If a management policy has not been applied to the XSet prior to commit, a 
   default management policy will be applied to the XSet at the time of commit.

    @note If the XSystem has been closed undefined results may occur (this 
          includes but is not limited to data loss and data corruption).

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle An xset_handle.
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XSet_AsyncCommit (const xset_handle inHandle,
                      const XOPID inXOPID,
                      xasync_callback inCallback,
                      xasync_handle* const outAsyncHandle); 

/**
    Asynchronously closes a previously opened XStream. 
    Any resources that were allocated can be released at this point.  
    
    The VIM is tasked with starting the asynchronous operation and 
    immediately returning a valid XAsync handle corresponding to the
    given XOPID. When the operation is complete, the VIM must notify
    the user of it's completion by invoking the provided callback.

    @note Closing an already closed XStream can produce undefined results (this 
          includes but is not limited to data loss and data corruption)

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will return immediately.

    @param inHandle An xstream_handle.
    @param inXOPID Unique ID that is specified by the application to identify 
                   the asynchronous operation.
    @param inCallback A pointer to a function that is called when the 
                      asynchronous operation completes. The parameter passed to 
                      the call back function can be probed for information.
    @param outAsyncHandle A handle to the asynchronous operation.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XStream_AsyncClose (const xstream_handle inHandle,
                        const XOPID inXOPID,
                        xasync_callback inCallback,
                        xasync_handle* const outAsyncHandle); 

/**************************************************************************
 *  method prototypes for managing asyncronous operations
 *************************************************************************/

/**
    Stops the operation associated with the passed inHandle

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_Halt (const xasync_handle inHandle);

/**
    Allows the caller to discover if the asynchronous operation relating to the
    passed inHandle is complete or not.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions
    @param outIsComplete A reference to valid storage for a xam_boolean. The
                         result is true if the async operation related to the
                         passed inHandle is complete, 
                         or false otherwise. 
                         The value that is passed in is not used, and is 
                         overwritten with the result.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_IsComplete (const xasync_handle inHandle,
                       xam_boolean* const outIsComplete);


/**
    Gets the status of the completed asynchronous operation that relates 
    to the passed inHandle.

    @note The passed inHandle must relate to an operation that performed an
          asynchronous read or this function will not be successful.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions
    @param outStatus A reference to valid storage for a xam_status. 
           On input this param is not used, on output this param is populated 
           with the status of the completed asynchronous operation that relates 
           to the passed inHandle. 
           
           If the underlying asynchronous operation is not complete 
           this function will fail and return a status for this call which 
           relates to the failure.

    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetStatus (const xasync_handle inHandle,
                      xam_status* const outStatus);

/**
    Gets the XOPID that was set by the application for the asynchronous 
    operation that relates to the passed inHandle
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @param outXOPID A reference to valid storage for a XOPID. 
                    On input this param is not used.
                    On output (if function is successful) this param is 
                              populated with the XOPID of the asynchronous
                              operation that relates to the passed inHandle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetXOPID (const xasync_handle inHandle,
                     XOPID* const outXOPID);


/**
    Gets the XSet of the completed asynchronous operation that relates to the
    passed inHandle. The return status of this function is set appropriately on
    success of failure of this call.

    @note The passed inHandle must relate to an operation that performed an
          asynchronous read or this function will not be successful.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @param outXSet A reference to valid storage for an xset_handle. 
                   On input this param is not used, 
                   On output (if function is successful) this param is 
                             populated with the XSet of the asynchronous 
                             operation that relates to the passed inHandle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetXSet (const xasync_handle inHandle,
                    xset_handle* const outXSet);


/**
    Gets the XStream from the completed asynchronous operation that relates to the
    passed inHandle. The return status of this function is set appropriately on
    success of failure of this call.

    @note The passed inHandle must relate to an operation that performed an
          asynchronous read or this function will not be successful.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @param outXStream A reference to valid storage for an xstream_handle. 
                      On input this param is not used, 
                      On output (if function is successful) this param is 
                                populated with the XStream from the asynchronous 
                                operation that relates to the passed inHandle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetXStream (const xasync_handle inHandle,
                       xstream_handle* const outXStream);

/**
    Gets the value from a property field with a type set to 
    application/vnd.snia.xam.xuid on the object referenced by the passed 
    inHandle. 

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @param outXUID A reference to valid storage for a xam_xuid. 
                   On input this param is not used, 
                   On output (if function is successful) this param is 
                             populated with the xam_xuid of the asynchronous 
                             operation that relates to the passed inHandle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetXUID (const xasync_handle inHandle,
                    xam_xuid* const outXUID);


/**
    Gets the number of bytes read from the completed asynchronous operation 
    that relates to the passed inHandle. The return status of this function 
    is set appropriately on success of failure of this call.

    @note The passed inHandle must relate to an operation that performed an
          asynchronous read or this function will not be successful.
    
    @note The asynchronous operation that relates to the passed inHandle must
          be completed for this function call to be successful.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @param outBytesRead A reference to valid storage for a xam_int. 
                   On input this param is not used, 
                   On output (if function is successful) this param is 
                             populated with the number of bytes read
                             during the asynchronous operation that relates to
                             the passed inHandle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetBytesRead (const xasync_handle inHandle,
                         xam_int* const outBytesRead);

/**
    Gets the number of bytes written for the completed asynchronous operation 
    that relates to the passed inHandle. The return status of this function 
    is set appropriately on success of failure of this call.

    @note The passed inHandle must relate to an operation that performed an
          asynchronous write or this function will not be successful.
    
    @note The asynchronous operation that relates to the passed inHandle must
          be completed for this function call to be successful.

    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @param outBytesWritten A reference to valid storage for a xam_int. 
                   On input this param is not used, 
                   On output (if function is successful) this param is 
                             populated with the number of bytes written
                             during the asynchronous operation that relates to
                             the passed inHandle.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_GetBytesWritten (const xasync_handle inHandle,
                            xam_int* const outBytesWritten);

/**
    Releases resources associated with the completed asynchronous operation 
    that relates to the passed inHandle.
    
    Concurrency requirements:
        This method is thread-safe.
    Blocking:
        This method will block until complete.

    @param inHandle An xasync_handle as retrieved by calling anyone of the 
                    XXX_AsynchXXX functions.
    @return The status code generated by calling this function. Use the 
            XAM_GetErrorToken function to determine the meaning of this value.
 */
EXPORT xam_status DECL 
VIM_XAsync_Close (const xasync_handle inHandle);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // __VIM_H_
