/******************************************************************************
 *
 * Copyright  2007-2009, The Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    - Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - Neither the name of The Storage Networking Industry Association (SNIA)
 *      nor the names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ============================================================================
 *  Copyright  2007, EMC Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    - Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - The name of EMC Corporation may not be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*****************************************************************************
 * FieldIterator Version @version.full@
 *****************************************************************************/

package org.snia.xam;

import org.snia.xam.natives.XAM_Native;

/**
 * An object which iterates over the fields of a XAM library object, an XSystem
 * instance, or an XSet instance: it returns the names of the fields. It returns
 * the items once only, in sequence, by successive calls to the next method.
 * 
 * @since 1.0
 */
public class FieldIterator implements XIterator {
    protected long     mHandle      = 0;
    protected String   mLastElement = null;
    private XAMLibraryObj mXAMLibrary  = null;
    private XSystem    mXSystem     = null;
    
    // /////////////////////////////////////////////////////////////////////////
    // Iterator methods
    // /////////////////////////////////////////////////////////////////////////
    public boolean hasNext() {
        boolean vRetValue;
        try {
            vRetValue = XAM_Native.XIterator_HasNext(mHandle);
        } catch (Exception ignore) {
            vRetValue = false;
        }
        return vRetValue;
    }
    
    public String next() {
        String vElement = null;
        try {
            vElement = XAM_Native.XIterator_Next(mHandle);
        } catch (Exception vException) {
            vElement = null;
        }
        mLastElement = vElement;
        
        // if we do not have any elements, C API returns a empty string
        if (vElement.length() == 0) {
            vElement = null;
        }
        return vElement;
    }
    
    public void remove() {
        throw new java.lang.UnsupportedOperationException();
    }
    
    protected void finalize() {
        if (mHandle != 0) {
            mLastElement = null;
            try {
                XAM_Native.XIterator_Close(mHandle);
            } catch (Exception ignore) {
            }
            mHandle = 0;
        }
    }
    
    public void close() throws XAMException {
        // Check to see whether XStream is closed?
        assertClosed();
        
        synchronized(this){
            if (mHandle != 0) {
                mLastElement = null;
                try {
                    XAM_Native.XIterator_Close(mHandle);
                } catch (Exception vException) {
                    if (mXSystem != null)
                        XAMErrors.throwConvertedException(
                                ((XSystemImpl) mXSystem).getHandle(), vException);
                    else
                        XAMErrors.throwConvertedException(
                                ((XAMLibraryObj) mXAMLibrary).getHandle(), vException);
                }
                mHandle = 0;
            }
        }
    }
    
    private void assertClosed() throws XAMException {
        synchronized(this){
            if (this.mHandle == 0)
                throw new XAMException("xam/XIterator already closed.");
        }
    }
    
    /**
     * @return the FieldIterator Handle
     */
    long getHandle() {
        return mHandle;
    }
    
    // /////////////////////////////////////////////////////////////////////////
    // constructor methods
    // /////////////////////////////////////////////////////////////////////////
    /*
     * package scoped constructor
     */
    FieldIterator(long inHandle, XAMLibraryObj inXAMlLibrary, XSystem inXSystem) {
        mHandle = inHandle;
        mXAMLibrary = inXAMlLibrary;
        mXSystem = inXSystem;
    }
}
