/******************************************************************************
 *
 * Copyright  2007, The Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    - Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - Neither the name of The Storage Networking Industry Association (SNIA)
 *      nor the names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ============================================================================
 *  Copyright  2007, EMC Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    - Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - The name of EMC Corporation may not be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*****************************************************************************
 * XStreamImpl Version @version.full@
 *****************************************************************************/

package org.snia.xam;

import org.snia.xam.XAMException;
import org.snia.xam.natives.XAM_Native;

public class XStreamImpl implements XStream {
    private long          mHandle      = 0;
    private XAMLibraryObj mXAMLibrary  = null;
    private XSystem       mXSystem     = null;
    
    public void seek(long inOffset, long inWhence) throws XAMException {
        // Check to see whether XStream is closed?
        assertClosed();
        
        // Check for InvalidWhence
        if ((inWhence == SEEK_CUR) || (inWhence == SEEK_END)
                || (inWhence == SEEK_SET)) {
            try {
                XAM_Native.XStream_Seek(mHandle, inOffset, inWhence);
            } catch (Exception vException) {
                try {
                    if (mXSystem != null)
                        XAMErrors.throwConvertedException(
                                ((XSystemImpl) mXSystem).getHandle(),
                                vException);
                    else
                        XAMErrors.throwConvertedException(
                                ((XAMLibraryObj) mXAMLibrary).getHandle(),
                                vException);
                } catch (XAMException e) {
                    //If seek throws InvalidArgumentException because we
                    // attempt to seek outside boundaries of the stream. 
                    // we should throw XStreamException 
                    if (e instanceof InvalidArgumentException) {
                        throw new XStreamException(e.getMessage(), e.getStatusCode());
                    } else {
                        throw e;
                    }
                }
            }
        } else {
            throw new InvalidArgumentException(
                    XAMErrors.sInvalidArgumentExceptionMessage,
                    XAMErrors.sInvalidArgumentException);
        }
    }
    
    public long tell() throws XAMException {
        // Check to see whether XStream is closed?
        assertClosed();
        
        long vOffset = 0;
        try {
            vOffset = XAM_Native.XStream_Tell(mHandle);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        return vOffset;
    }
    
    public long read(byte[] inBuffer, long inOffset, long inBytesToRead)
            throws XAMException {
        // check whether the buffer is null.
        if (inBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // check for invalid offsets and length
        if ((inOffset + inBytesToRead) > inBuffer.length)
            throw new InvalidArgumentException(
                    "xam/size mismatch between arguments.",
                    XAMErrors.sInvalidArgumentException);
        if ((inOffset < 0) || (inBytesToRead < 0))
            throw new InvalidArgumentException(
                    "xam/size mismatch between arguments.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        // check for invalid offset and length.
        int vLength = inBuffer.length;
        if (inOffset > vLength)
            throw new InvalidArgumentException(
                    "xam/offset is greater than buffer length.",
                    XAMErrors.sInvalidArgumentException);
        if (inOffset < 0)
            throw new InvalidArgumentException("xam/offset is less than zero.",
                    XAMErrors.sInvalidArgumentException);
        
        // read the bytes
        long vBytesToRead = inBytesToRead;
        long vBytesRead = 0;
        if (inOffset == 0) {
            // read into the provided buffer
            try {
                vBytesRead = XAM_Native.XStream_Read(mHandle, inBuffer,
                        vBytesToRead);
            } catch (Exception vException) {
                if (mXSystem != null)
                    XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                            .getHandle(), vException);
                else
                    XAMErrors.throwConvertedException(
                            ((XAMLibraryObj) mXAMLibrary).getHandle(),
                            vException);
            }
        } else // read to an offset (less efficient)
        {
            // read into a temporary buffer
            byte[] vBuffer = new byte[(int) inBytesToRead];
            try {
                vBytesRead = XAM_Native.XStream_Read(mHandle, vBuffer,
                        inBytesToRead);
            } catch (Exception vException) {
                if (mXSystem != null)
                    XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                            .getHandle(), vException);
                else
                    XAMErrors.throwConvertedException(
                            ((XAMLibraryObj) mXAMLibrary).getHandle(),
                            vException);
            }
            
            // copy into the provided buffer
            if (vBytesRead > 0)
                System.arraycopy(vBuffer, 0, inBuffer, (int) inOffset,
                        (int) vBytesRead);
        }
        
        return vBytesRead;
    }
    
    public long read(byte inBuffer[], long inBytesToRead) throws XAMException {
        // check whether the buffer is null.
        if (inBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // check for invalid offsets and length
        if (inBytesToRead > inBuffer.length)
            throw new InvalidArgumentException(
                    "xam/size mismatch between arguments.",
                    XAMErrors.sInvalidArgumentException);
        if (inBytesToRead < 0)
            throw new InvalidArgumentException(
                    "xam/size mismatch between arguments.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        // check for invalid length.
        int vLength = inBuffer.length;
        
        if (inBytesToRead > vLength)
            throw new InvalidArgumentException(
                    "xam/bytes to read is greater than buffer length.",
                    XAMErrors.sInvalidArgumentException);
        
        // read the bytes
        long vBytesToRead = inBytesToRead;
        long vBytesRead = 0;
        
        // read into the provided buffer
        try {
            vBytesRead = XAM_Native.XStream_Read(mHandle, inBuffer,
                    vBytesToRead);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        
        return vBytesRead;
    }
    
    public long read(byte inBuffer[]) throws XAMException {
        // check whether the buffer is null.
        if (inBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        // read the bytes
        long vBytesToRead = inBuffer.length;
        long vBytesRead = 0;
        
        // read into the provided buffer
        try {
            vBytesRead = XAM_Native.XStream_Read(mHandle, inBuffer,
                    vBytesToRead);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }

        return vBytesRead;
    }
    
    public long write(byte[] inBuffer, long inOffset, long inBytesToWrite)
            throws XAMException {
        // Check to see whether the inBuffer is null
        if (inBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        int vLength = inBuffer.length;
        
        // check for invalid offset and length.
        if (inOffset > vLength)
            throw new InvalidArgumentException(
                    "xam/offset is greater than buffer length.",
                    XAMErrors.sInvalidArgumentException);
        
        if (inOffset < 0)
            throw new InvalidArgumentException("xam/offset is less than 0.",
                    XAMErrors.sInvalidArgumentException);
        
        if (inBytesToWrite > (vLength - inOffset))
            throw new InvalidArgumentException(
                    "xam/Invalid number of bytes to write.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        long vBytesWritten = 0;
        if (inOffset == 0) {
            try {
                vBytesWritten = XAM_Native.XStream_Write(mHandle, inBuffer,
                        inBytesToWrite);
            } catch (Exception vException) {
                if (mXSystem != null)
                    XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                            .getHandle(), vException);
                else
                    XAMErrors.throwConvertedException(
                            ((XAMLibraryObj) mXAMLibrary).getHandle(),
                            vException);
            }
        } else // write from an offset (less efficient)
        {
            // copy into a temprary buffer
            byte[] vBuffer = new byte[(int) inBytesToWrite];
            System.arraycopy(inBuffer, (int) inOffset, vBuffer, 0,
                    (int) inBytesToWrite);
            
            // write from the temp buffer
            try {
                vBytesWritten = XAM_Native.XStream_Write(mHandle, vBuffer,
                        inBytesToWrite);
            } catch (Exception vException) {
                if (mXSystem != null)
                    XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                            .getHandle(), vException);
                else
                    XAMErrors.throwConvertedException(
                            ((XAMLibraryObj) mXAMLibrary).getHandle(),
                            vException);
            }
        }
        return vBytesWritten;
    }
    
    public long write(byte inBuffer[]) throws XAMException {
        // Check to see whether the inBuffer is null
        if (inBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        long vBytesWritten = 0;
        try {
            vBytesWritten = XAM_Native.XStream_Write(mHandle, inBuffer,
                    inBuffer.length);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        return vBytesWritten;
    }
    
    public long write(byte inBuffer[], long inBytesToWrite) throws XAMException {
        // Check to see whether the inBuffer is null
        if (inBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        // check for invalid offset and length.
        int vLength = inBuffer.length;
        if (inBytesToWrite > vLength)
            throw new InvalidArgumentException(
                    "xam/bytes to write is greater than buffer length.",
                    XAMErrors.sInvalidArgumentException);
        
        long vBytesWritten = 0;
        try {
            vBytesWritten = XAM_Native.XStream_Write(mHandle, inBuffer,
                    inBytesToWrite);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        return vBytesWritten;
    }
    
    public XAsync asyncRead(byte ioBuffer[], long inBufferLength, long inXOPID,
            XAsyncListener inListener) throws XAMException {
        
        if (ioBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        // check for invalid offsets and length
        if (inBufferLength > ioBuffer.length)
            throw new InvalidArgumentException(
                    "xam/size mismatch between arguments.",
                    XAMErrors.sInvalidArgumentException);
        
        XAsync vRetVal = null;
        try {
            vRetVal = XAM_Native.XStream_AsyncRead(mHandle, ioBuffer,
                    inBufferLength, inXOPID, inListener);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        
        // set the XAMLibrary and XSystem Handles
        ((XAsyncImpl) vRetVal).setXAMLibrary(mXAMLibrary);
        ((XAsyncImpl) vRetVal).setXSystem(mXSystem);
        
        return vRetVal;
    }
    
    public XAsync asyncWrite(byte ioBuffer[], long inBufferLength,
            long inXOPID, XAsyncListener inListener) throws XAMException {
        // Check to see whether the ioBuffer is null
        if (ioBuffer == null)
            throw new InvalidArgumentException("xam/buffer is null.",
                    XAMErrors.sInvalidArgumentException);
        
        // Check to see whether XStream is closed?
        assertClosed();
        
        XAsync vRetVal = null;
        try {
            vRetVal = XAM_Native.XStream_AsyncWrite(mHandle, ioBuffer,
                    inBufferLength, inXOPID, inListener);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        
        // set the XAMLibrary and XSystem Handles
        ((XAsyncImpl) vRetVal).setXAMLibrary(mXAMLibrary);
        ((XAsyncImpl) vRetVal).setXSystem(mXSystem);
        
        return vRetVal;
    }
    
    public XAsync asyncClose(long inXOPID, XAsyncListener inListener)
            throws XAMException {
        // Check to see whether XStream is closed?
        assertClosed();
        
        XAsync vRetVal = null;
        try {
            vRetVal = XAM_Native.XStream_AsyncClose(mHandle, inXOPID,
                    inListener, this);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
        
        // set the XAMLibrary and XSystem Handles
        ((XAsyncImpl) vRetVal).setXAMLibrary(mXAMLibrary);
        ((XAsyncImpl) vRetVal).setXSystem(mXSystem);
        
        return vRetVal;
    }
    
    // /////////////////////////////////////////////////////////////////////////
    // XSystem management methods
    // /////////////////////////////////////////////////////////////////////////
    
    public void abandon() throws XAMException {
        try {
            XAM_Native.XStream_Abandon(mHandle);
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
    }
    
    public void close() throws XAMException {
        // Check to see whether XStream is closed?
        assertClosed();
        
        try {
            synchronized (this) {
                XAM_Native.XStream_Close(mHandle);
                mHandle = 0;
            }
        } catch (Exception vException) {
            if (mXSystem != null)
                XAMErrors.throwConvertedException(((XSystemImpl) mXSystem)
                        .getHandle(), vException);
            else
                XAMErrors.throwConvertedException(((XAMLibraryObj) mXAMLibrary)
                        .getHandle(), vException);
        }
    }
    
    protected void finalize() {
        if (mHandle != 0) {
            try {
                close();
            } catch (Exception ignore) {
            }
            mHandle = 0;
        }
    }
    
    /**
     * @return the XStreamHandle
     */
    long getHandle() {
        return mHandle;
    }
    
    private void assertClosed() throws XStreamException {
        synchronized (this) {
            if (this.mHandle == 0)
                throw new XStreamException("xam/XStream already closed.");
        }
    }
    
    // /////////////////////////////////////////////////////////////////////////
    // constructor methods
    // /////////////////////////////////////////////////////////////////////////
    /**
     * package scoped constructor
     */
    XStreamImpl(long inHandle, XAMLibraryObj inXAMLibrary, XSystem inXSystem)
            throws XAMException {
        mHandle = inHandle;
        mXAMLibrary = inXAMLibrary;
        mXSystem = inXSystem;
    }
}
