/******************************************************************************
 *
 * Copyright  2007, The Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    - Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - Neither the name of The Storage Networking Industry Association (SNIA)
 *      nor the names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ============================================================================
 *  Copyright  2007, EMC Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    - Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - The name of EMC Corporation may not be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*****************************************************************************
 * XAMXUID Version @version.full@
 *****************************************************************************/

package org.snia.xam.toolkit;

import java.util.Arrays;
import org.snia.xam.XAMException;
import org.snia.xam.InvalidXUIDException;
import org.snia.xam.XUID;
import org.snia.xam.toolkit.util.*;

public class XAMXUID implements XUID {
    public static final int sXUIDLENGTH = XUID.MAX_LENGTH;
    private byte[]          mXUID       = new byte[sXUIDLENGTH];
    
    /**
     * Returns the byte[] corresponding to the XUID
     */
    public byte[] toBytes() {
        byte[] vBytes = this.mXUID;
        return vBytes;
    }
    
    /**
     * Returns the base64 encoded form of the real XUID bytes if the XUID is
     * valid. Other wise this method returns null
     */
    public String toString() {
        String vString = null;
        byte[] vXUIDBytes = null;
        
        int vXUIDLength = mXUID[5];
        vXUIDBytes = new byte[vXUIDLength];
        System.arraycopy(mXUID, 0, vXUIDBytes, 0, vXUIDLength);
        vString = Base64EncoderDecoder.encodeBase64(vXUIDBytes);
        
        return vString;
    }
    
    /**
     * Returns true if the real XUID bytes of the provided XUID object are equal
     * to the current XUID objects real XUID bytes
     */
    public boolean equals(Object inXUID) {
        boolean vEquals = false;
        
        byte[] vInXUIDBytes = ((XAMXUID) inXUID).toBytes();
        byte[] vBytes = toBytes();
        int vXUIDLen = vBytes[5];
        
        byte[] vInXUID = new byte[vXUIDLen];
        byte[] vCurrentXUID = new byte[vXUIDLen];
        
        System.arraycopy(vInXUIDBytes, 0, vInXUID, 0, vXUIDLen);
        System.arraycopy(toBytes(), 0, vCurrentXUID, 0, vXUIDLen);
        
        // check only the real bytes
        vEquals = Arrays.equals(vInXUID, vCurrentXUID);
        
        return vEquals;
    }
    
    // /////////////////////////////////////////////////////////////////////////
    // private methods
    // /////////////////////////////////////////////////////////////////////////
    /**
     * Checks to see whether the provided byte[] follows a valid XUID format
     * 
     * @param byte[]
     *            bytes to be examined.
     * @return true - if the byte[] follows XUIDSyntax false otherwise
     */
    private boolean isValid(byte[] inBytes) {
        boolean vValid = true;
        
        if(inBytes == null)
            vValid = false;
        
        // Check for the length
        if ((vValid)
                && ((inBytes.length < 9) || (inBytes.length > sXUIDLENGTH)))
            vValid = false;
        
        // Check for reserved bits
        if ((vValid) && ((inBytes[0] != '\0') || (inBytes[4] != '\0')))
            vValid = false;
        
        // Check OID
        if ((vValid)
                && (((inBytes[1] == 0) && (inBytes[2] == 0) && (inBytes[3] == 0))))
            vValid = false;
        
        if (vValid) {
            int vActualLen = inBytes[5];
            byte[] vBytes = new byte[vActualLen];
            System.arraycopy(inBytes, 0, vBytes, 0, vActualLen);
            vBytes[6] = 0;
            vBytes[7] = 0;
            int vCalcCRC = CRC16.getInstancec().calcCRC(vBytes, vActualLen);
            String vCalcHex = Integer.toHexString(vCalcCRC);
            
            // Calculate the expected CRC-16
            int vHighCRCByte = inBytes[7] & 0x000000FF;
            vHighCRCByte <<= 8;
            int vLowCRCByte = inBytes[6] & 0x000000FF;
            int vCRC = vHighCRCByte | vLowCRCByte;
            String vExpectedHex = Integer.toHexString(vCRC);
            
            if (vCalcHex.equals(vExpectedHex))
                vValid = true;
        }
        return vValid;
    }
    
    // /////////////////////////////////////////////////////////////////////////
    // constructor methods
    // /////////////////////////////////////////////////////////////////////////
    /*
     * public scoped constructor
     */
    public XAMXUID(byte[] inXUID) throws XAMException {
        // Check for the length
        if (!isValid(inXUID))
            throw new InvalidXUIDException();
        
        // get the actual Length of the XUID
        int vActualLength = inXUID[5];
        System.arraycopy(inXUID, 0, mXUID, 0, vActualLength);
        
        // pad the mXUID with '\0'
        for (int i = vActualLength; i < sXUIDLENGTH; i++) {
            mXUID[i] = '\0';
        }
    }
    
    /*
     * public scoped constructor
     */
    public XAMXUID(String inBase64String) throws XAMException {
        byte[] vXUID = null;
        
        try{
        vXUID = Base64EncoderDecoder.decodeBase64(inBase64String);
        }catch(Exception e)
        {
        	//If any Exception decoding throw InvalidXUIDException
        	throw new InvalidXUIDException();
        }
        
        if (!isValid(vXUID))
            throw new InvalidXUIDException();
        
        // get the actual Length of the XUID
        int vActualLength = vXUID[5];
        System.arraycopy(vXUID, 0, mXUID, 0, vActualLength);
        
        // pad the mXUID with '\0'
        for (int i = vActualLength; i < sXUIDLENGTH; i++) {
            mXUID[i] = '\0';
        }
    }
}
