/*
 * Copyright (c) 2009, Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package org.snia.xam.vim.http.client;

import java.util.HashMap;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.XAMException;
import org.snia.xam.XIterator;
import org.snia.xam.vim.isolation.server.http.handlers.ProtocolConstants;

public class HTTPXIterator
   implements XIterator
{
   private String m_host   = null;
   private String m_handle = null;
   
   HTTPXIterator( String host, String handle )
   {
      m_host = host;
      m_handle = handle;
   }

   /**
    * Release resources being used by the XIterator
    * @see org.snia.xam.XIterator#close()
    */
   public void close()
      throws AuthenticationExpiredException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XITERATOR_CLOSE_HANDLER,
         ProtocolConstants.XITERATOR_HANDLE, m_handle);
      responses.clear();
   }

   /**
    * Determine if there are more names to be retuned from the XIterator.
    * @see org.snia.xam.XIterator#hasNext()
    */
   public boolean hasNext()
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XITERATOR_HASNEXT_HANDLER,
            ProtocolConstants.XITERATOR_HANDLE, m_handle);
         return Boolean.valueOf(responses.get(ProtocolConstants.HASNEXT_PROPERTY));
      }
      catch( Exception e )
      {
         throw new RuntimeException( "hasNext failed due to underlying error", e );
      }
   }

   /**
    * Return the next name from the XIterator
    * @see org.snia.xam.XIterator#next()
    */
   public String next()
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XITERATOR_NEXT_HANDLER,
            ProtocolConstants.XITERATOR_HANDLE, m_handle);
         return responses.get(ProtocolConstants.PROPERTY_PROPERTY);
      }
      catch( Exception e )
      {
         throw new RuntimeException( "hasNext failed due to underlying error", e );
      }
   }

   /**
    * Remove is not supported by XIterators. This method is provided to be compatible with Java Iterator.
    */
   public void remove()
   {
      throw new UnsupportedOperationException( "XIterator does not support 'remove'" );
   }
}
