/*
 * Copyright (c) 2009, Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package org.snia.xam.vim.http.client;

import java.util.HashMap;
import java.util.logging.Logger;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.AuthorizationException;
import org.snia.xam.FieldExistsException;
import org.snia.xam.InvalidArgumentException;
import org.snia.xam.InvalidFieldNameException;
import org.snia.xam.InvalidXUIDException;
import org.snia.xam.JobCommandException;
import org.snia.xam.JobRunningException;
import org.snia.xam.JobUnsupportedException;
import org.snia.xam.MaximumFieldException;
import org.snia.xam.ObjectInUseException;
import org.snia.xam.PolicyMismatchException;
import org.snia.xam.PolicyNameException;
import org.snia.xam.RetentionValueException;
import org.snia.xam.XAMException;
import org.snia.xam.XAsync;
import org.snia.xam.XAsyncListener;
import org.snia.xam.XSet;
import org.snia.xam.XSetAbandonException;
import org.snia.xam.XSetCorruptException;
import org.snia.xam.XSetUnderHoldException;
import org.snia.xam.XStream;
import org.snia.xam.XUID;
import org.snia.xam.base.DefaultXUID;
import org.snia.xam.vim.isolation.server.http.handlers.ProtocolConstants;

public class HTTPXSet
   extends HTTPFieldContainer
   implements XSet
{
   Logger m_logger = null;
   
   HTTPXSet( String host, String xset_handle)
   {
      m_host                            = host;
      m_handle_label                    = ProtocolConstants.XSYSTEM_HANDLE;
      m_handle                          = xset_handle;
      m_create_property_command         = ProtocolConstants.XSET_CREATEPROPERTY_HANDLER_NAME;
      m_create_xstream_command          = ProtocolConstants.XSET_CREATE_XSTREAM_HANDLER;
      m_get_property_command            = ProtocolConstants.XSET_GETPROPERTY_HANDLER_NAME;
      m_delete_field_command            = ProtocolConstants.XSET_DELETEFIELD_HANDLER_NAME;
      m_get_field_length_command        = ProtocolConstants.XSET_GET_FIELDLENGTH_HANDLER_NAME;
      m_get_field_type_command          = ProtocolConstants.XSET_GETPROPERTYTYPE_HANDLER_NAME;
      m_get_field_readonly_command      = ProtocolConstants.XSET_GET_FIELD_READONLY_HANDLER;
      m_get_field_binding_command       = ProtocolConstants.XSET_GET_FIELD_BINDING_HANDLER;
      m_set_property_command            = ProtocolConstants.XSET_SETPROPERTY_HANDLER_NAME;
      m_set_field_as_binding_command    = ProtocolConstants.XSET_SET_FIELD_AS_BINDING_HANDLER;
      m_set_field_as_nonbinding_command = ProtocolConstants.XSET_SET_FIELD_AS_NONBINDING_HANDLER;
      m_open_xstream_command            = ProtocolConstants.XSET_OPEN_XSTREAM_HANDLER;
      m_open_field_iterator_command     = ProtocolConstants.XAM_CREATE_FIELD_ITERATOR_HANDLER;
      m_xiterator_type_property         = ProtocolConstants.XITERATOR_CLASS_XSET;
      m_contains_field_command          = ProtocolConstants.XSET_CONTAINS_FIELD_HANDLER;
      m_async_open_xstream_command      = ProtocolConstants.XSET_ASYNC_OPENXSRTEAM_HANDLER;
   }
   
   public void abandon()
      throws AuthenticationExpiredException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_ABANDON_HANDLER_NAME,
         ProtocolConstants.XSET_HANDLE, m_handle);
      responses.clear();
   }

   public void applyAccessPolicy(boolean binding, String policyName)
      throws AuthenticationExpiredException, MaximumFieldException,
      PolicyNameException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_ACCESS_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policyName, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      } catch( UnsupportedEncodingException e ) 
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyAutoDeletePolicy(boolean binding, String policyName)
      throws AuthenticationExpiredException, PolicyNameException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_AUTODELETE_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policyName, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      }
      catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyBaseRetentionPolicy(boolean binding, String policyName)
      throws AuthenticationExpiredException, PolicyNameException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_BASE_RETENTION_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policyName, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyManagementPolicy(boolean binding, String policyName)
      throws AuthenticationExpiredException, MaximumFieldException,
      PolicyNameException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_MANAGEMENT_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policyName, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyRetentionDurationPolicy(String retentionID,
                                            boolean binding,
                                            String inPolicy)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, PolicyNameException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_RETENTION_DURATION_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(inPolicy, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      }
      catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyRetentionEnabledPolicy(String retentionID,
                                           boolean binding,
                                           String policy)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, PolicyNameException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_RETENTION_ENABLED_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policy, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      }
      catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyShredPolicy(boolean binding, String policyName)
      throws AuthenticationExpiredException, MaximumFieldException,
      PolicyNameException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_SHRED_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policyName, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      }
      catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void applyStoragePolicy(boolean binding, String policyName)
      throws AuthenticationExpiredException, MaximumFieldException,
      PolicyNameException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      try
      {
         HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_APPLY_STORAGE_POLICY_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_POLICY, URLEncoder.encode(policyName, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      }
      catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public XAsync asyncCommit(long xopid, XAsyncListener listener)
      throws AuthenticationExpiredException, InvalidArgumentException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_ASYNC_COMMIT_HANDLER,
         ProtocolConstants.XSET_HANDLE, m_handle,
         ProtocolConstants.ASYNC_XOPID, Long.toString(xopid) );
      String asyncHandle = responses.get(ProtocolConstants.XASYNC_HANDLE);
      responses.clear();
      return new HTTPXAsync( m_host, asyncHandle, listener, m_logger );
   }

   public void close()
      throws AuthenticationExpiredException, ObjectInUseException,
      XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_CLOSE_HANDLER_NAME,
         ProtocolConstants.XSYSTEM_HANDLE, m_handle );
      responses.clear();
   }

   public XUID commit()
      throws AuthenticationExpiredException, AuthorizationException,
      JobRunningException, XSetUnderHoldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_COMMIT_HANDLER_NAME,
         ProtocolConstants.XSYSTEM_HANDLE, m_handle );
      XUID xuid = null;
      try
      {
         String xuid_string = responses.get(ProtocolConstants.XUID_PROPERTY);
         xuid = DefaultXUID.valueOf(xuid_string);
         responses.clear();
      }
      catch( Exception e )
      {
         throw new InvalidXUIDException( "Can not properly decode XUID response from remote server" );
      }
      return xuid;
   }

   public void createRetention(boolean binding, String retentionID)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      HashMap<String, String> responses;
      try
      {
         responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_CREATE_RETENTION_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      }
      catch (UnsupportedEncodingException e)
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );   
      }
   }

   public boolean getActualAutoDelete()
      throws AuthenticationExpiredException, ObjectInUseException,
      PolicyMismatchException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses;
      responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_GET_ACTUAL_AUTODELETE_HANDLER,
         ProtocolConstants.XSET_HANDLE, m_handle);
      boolean autodelete = Boolean.parseBoolean(responses.get(ProtocolConstants.ARGUMENT_AUTODELETE));
      responses.clear();
      return autodelete;
   }

   public long getActualRetentionDuration(String retentionID)
      throws AuthenticationExpiredException, InvalidArgumentException,
      PolicyMismatchException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      HashMap<String, String> responses;
      try
      {
         responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_GET_ACTUAL_RETENTION_DURATION_HANDLER,
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         long duration = Long.parseLong(responses.get(ProtocolConstants.ARGUMENT_DURATION));
         responses.clear();
         return duration;
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public boolean getActualRetentionEnabled(String retentionID)
      throws AuthenticationExpiredException, InvalidArgumentException,
      PolicyMismatchException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      HashMap<String, String> responses;
      try
      {
         responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_GET_ACTUAL_RETENTION_ENABLED_HANDLER,
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.XSET_HANDLE, m_handle);
         boolean enables = Boolean.parseBoolean(responses.get(ProtocolConstants.ARGUMENT_ENABLED));
         responses.clear();
         return enables;
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public boolean getActualShred()
      throws AuthenticationExpiredException, ObjectInUseException,
      PolicyMismatchException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses;
      responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_GET_ACTUAL_SHRED_HANDLER,
         ProtocolConstants.XSET_HANDLE, m_handle);
      boolean autodelete = Boolean.parseBoolean(responses.get(ProtocolConstants.ARGUMENT_SHRED));
      responses.clear();
      return autodelete;
   }

   public void haltJob()
      throws AuthorizationException, AuthenticationExpiredException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_HALTJOB_HANDLER_NAME,
         ProtocolConstants.XSET_HANDLE, m_handle );
      responses.clear();
   }

   public XStream openExportXStream()
      throws AuthenticationExpiredException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_OPEN_EXPORT_XSTREAM_HANDLER,
         m_handle_label, m_handle);
      String stream_handle = responses.get(ProtocolConstants.XSTREAM_HANDLE);
      responses.clear();
      return new HTTPXStream( m_host, stream_handle, m_logger );
   }

   public XStream openImportXStream()
      throws AuthenticationExpiredException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_OPEN_IMPORT_XSTREAM_HANDLER,
         m_handle_label, m_handle);
      String stream_handle = responses.get(ProtocolConstants.XSTREAM_HANDLE);
      responses.clear();
      return new HTTPXStream( m_host, stream_handle, m_logger );
   }

   public void resetAccessFields()
      throws AuthenticationExpiredException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_RESET_ACCESS_FIELDS_HANDLER,
         ProtocolConstants.XSET_HANDLE, m_handle );
      responses.clear();
   }

   public void resetManagementFields()
      throws AuthenticationExpiredException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_RESET_MANAGEMENT_FIELDS_HANDLER,
         ProtocolConstants.XSET_HANDLE, m_handle );
      responses.clear();
   }

   public void setAutoDelete(boolean binding, boolean autoDelete)
      throws AuthenticationExpiredException, MaximumFieldException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses;
      responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_SET_AUTODELETE_HANDLER,
         ProtocolConstants.ATTRIBUTE_BINDING,   Boolean.toString(binding),
         ProtocolConstants.ARGUMENT_AUTODELETE, Boolean.toString(autoDelete),
         ProtocolConstants.XSET_HANDLE, m_handle);
      responses.clear();
   }

   public void setBaseRetention(boolean binding, long duration)
      throws AuthenticationExpiredException, RetentionValueException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_SET_BASE_RETENTION_HANDLER,
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_DURATION, Long.toString(duration),
            ProtocolConstants.XSET_HANDLE, m_handle);
      responses.clear();
   }

   public void setRetentionDuration(String retentionID,
                                    boolean binding,
                                    long duration)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, RetentionValueException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses;
      try
      {
         responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_SET_RETENTION_DURATION_HANDLER,
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_DURATION, Long.toString(duration),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void setRetentionEnabledFlag(String retentionID,
                                       boolean binding,
                                       boolean enabled)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XAMException
   {
      HashMap<String, String> responses;
      try
      {
         responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_SET_RETENTION_ENABLED_FLAG_HANDLER,
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.ARGUMENT_ENABLED, Boolean.toString(enabled),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void setRetentionStarttime(String retentionID, boolean binding)
      throws AuthenticationExpiredException, FieldExistsException,
      InvalidArgumentException, InvalidFieldNameException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses;
      try
      {
         responses = HTTPRequest.request(m_host,
            ProtocolConstants.XSET_SET_RETENTION_STARTTIME_HANDLER,
            ProtocolConstants.ARGUMENT_RETENTION_ID, URLEncoder.encode(retentionID, ProtocolConstants.UTF8),
            ProtocolConstants.ATTRIBUTE_BINDING, Boolean.toString(binding),
            ProtocolConstants.XSET_HANDLE, m_handle);
         responses.clear();
      } catch( UnsupportedEncodingException e )
      {
         throw new XAMException( ProtocolConstants.TRANSPORT_ENCODE_ERROR, e );
      }
   }

   public void setShred(boolean binding, boolean shred)
      throws AuthenticationExpiredException, MaximumFieldException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XAMException
   {
      HashMap<String, String> responses;
      responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_SET_SHRED_HANDLER,
         ProtocolConstants.ATTRIBUTE_BINDING,   Boolean.toString(binding),
         ProtocolConstants.ARGUMENT_SHRED, Boolean.toString(shred),
         ProtocolConstants.XSET_HANDLE, m_handle);
      responses.clear();
   }

   public void submitJob()
      throws AuthenticationExpiredException, AuthorizationException,
      JobCommandException, JobUnsupportedException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XAMException
   {
      HashMap<String, String> responses = HTTPRequest.request(m_host,
         ProtocolConstants.XSET_SUBMITJOB_HANDLER_NAME,
         ProtocolConstants.XSET_HANDLE, m_handle );
      responses.clear();
   }
}
