/*
 * Copyright (c) 2009, Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package org.snia.xam.vim.http.client;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.FieldDoesNotExistException;
import org.snia.xam.FieldExistsException;
import org.snia.xam.InvalidArgumentException;
import org.snia.xam.InvalidFieldNameException;
import org.snia.xam.InvalidOperationException;
import org.snia.xam.MaximumFieldException;
import org.snia.xam.ObjectInUseException;
import org.snia.xam.XAMException;
import org.snia.xam.XAsync;
import org.snia.xam.XAsyncListener;
import org.snia.xam.XSetAbandonException;
import org.snia.xam.XSetCorruptException;
import org.snia.xam.XStream;
import org.snia.xam.XSystemAbandonException;
import org.snia.xam.XSystemCorruptException;
import org.snia.xam.base.AbstractFieldContainer;

public class TempFieldContainer
   extends AbstractFieldContainer
{
   static final String MSG = "Not supported on the TempFieldContainer";
   
   TempFieldContainer() { ; }
   
   /**
    * Override the Abstract class method. Since this class is used as a temp 
    * container, no protected name space is required.
    * If it is a {@link InvalidFieldNameException} is thrown.
    * 
    * @param fieldName The field name
    * @throws XAMException If the field name is in the protected name space.
    */
   protected void checkProtectedNameSpace(String fieldName)
      throws XAMException
   {
   }

   public XAsync asyncOpenXStream(String name,
                                  String mode,
                                  long xopid,
                                  XAsyncListener listener)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, FieldDoesNotExistException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException(MSG);
   }

   public XStream createXStream(String fieldName,
                                boolean binding,
                                String mimeType)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, InvalidOperationException,
      FieldExistsException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException(MSG);
   }

   public XStream openXStream(String fieldName, String mode)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      throw new InvalidOperationException(MSG);
   }
}
