/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *  
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


package org.snia.xam.vim.isolation.server.contextes;


public interface ContextRepository {

    /**
     * Adds a <code>Context</code> to the repository using 
     * its <code>ContextKey</code>
     *
     * @param key Identifies the context to be added
     * @param ctx object to be added
     * @return    <code>Context</code> that has been added
     */
    public Context put(ContextKey key, Context ctx);

    /**
     * Removes a <code>Context</code> to the repository using 
     * its <code>ContextKey</code>
     *
     * @param key Identifies the context to be removed
     * @return    <code>Context</code> that has been removed or null if 
     *            the context does not exist
     */
    public Context remove(ContextKey key);

    /**
     * Retrieve a <code>Context</code> to the repository from 
     * its <code>ContextKey</code>
     *
     * @param key Identifies the context to be retrieved
     * @return    <code>Context</code> to be retrieved or null if 
     *            the context does not exist
     */
    public Context get(ContextKey key);

    /**
     * Returns true is the <code>Context</code> associated to the
     * <code>ContextKey</code> exists in the repository.
     *
     * @param key Identifies the context
     * @return    true if the the context exists
     */
    public boolean containsKey(ContextKey key);

    /**
     * Returns whether or not the repository is emoty.
     *
     * @return    true if the repository is not empty
     */
    public boolean isEmpty();

    /**
     * Returns the number of element in the repository.
     */
    public int size();

    /**
     * Removes all the entries from this repository.
     */
    public void clear();

}