/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *  
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


package org.snia.xam.vim.isolation.server.contextes;

import java.util.Map;
import java.util.HashMap;

public class ContextRepositoryMap implements ContextRepository {

    private Map repository;
    
    /**
     * Create a new context repository 
     */
    public ContextRepositoryMap() {
        repository = new HashMap();
    }

    /**
     * Create a new context repository with the specified initial capacity
     * @param initialCapacity the intial capacity of the repository
     */
    public ContextRepositoryMap(int initialCapacity) {
        repository = new HashMap(initialCapacity);
    }

    /**
     * @see ContextRepository#put(ContextKey, Context)
     */
    public Context put(ContextKey key, Context ctx) {
        synchronized(repository) {
            Context res = (Context) repository.put(key, ctx);
            return res;
        }
    }

    /**
     * @see ContextRepository#remove(ContextKey)
     */
    public Context remove(ContextKey key) {
        synchronized(repository) {
            Context res = (Context) repository.remove(key);
            return res;
        }        
    }

    /**
     * @see ContextRepository#get(ContextKey)
     */
    public Context get(ContextKey key) {
        synchronized(repository) {
            Context res = (Context) repository.get(key);
            return res;
        }
    }

    /**
     * @see ContextRepository#containsKey(ContextKey)
     */
    public boolean containsKey(ContextKey key) {
        synchronized(repository) {
            return repository.containsKey(key);
        }
    }

    /**
     * @see ContextRepository#isEmpty()
     */
    public boolean isEmpty() {
        synchronized(repository) {
            return repository.isEmpty();
        }
    }

    /**
     * @see ContextRepository#size()
     */
    public int size() {
        synchronized(repository) {
            return repository.size();
        }
    }

    /**
     * @see ContextRepository#clear()
     */
    public void clear() {
        synchronized(repository) {
            repository.clear();
        }
    }
}
