/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *  
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package org.snia.xam.vim.isolation.server.handlers;

import java.io.IOException;

import org.snia.xam.vim.isolation.server.ifc.StorageServerException;
import org.snia.xam.vim.isolation.server.contextes.Context;
import org.snia.xam.vim.isolation.server.resources.ByteBufferList;

public interface WriteHandler extends StorageHandler {

    /**
     * Writes the data identified by the buffer to the object
     * referred to by the <code>ctx</code>.
     *
     * @param  ctx        context identifying the object to write to
     * @param  bufferList data be appended to the object
     */
    public void write(Context ctx, ByteBufferList bufferList)
        throws IOException, StorageServerException;

    /**
     * Flush the data to persistent storage so the operation can
     * be resumed if it failed in the middle.
     *
     * @param ctx     context identifying the object
     * @see <a href="StorageHandler.html">StorageHandler</a>
     */
    public void flush(Context ctx)
        throws IOException, StorageServerException;
}