/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *  
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package org.snia.xam.vim.isolation.server.http;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;

import org.mortbay.http.HttpResponse;

import org.snia.xam.vim.isolation.server.ifc.StorageServerException;
import org.snia.xam.vim.isolation.server.handlers.StorageHandler;
import org.snia.xam.vim.isolation.server.contextes.Context;
import org.snia.xam.vim.isolation.server.resources.ByteBufferList;
import org.snia.xam.vim.isolation.server.handlers.ReadHandler;

public class HttpReadGlue extends HttpGenericHandler
{

    public HttpReadGlue(StorageHandler handler, String contextPath) 
        throws StorageServerException {
        super(handler, contextPath);
    }


    protected void handle(Context ctx,
      OutputStream outputStr, HttpResponse response)
        throws IOException, StorageServerException {
        
        
        byte[] bytes = new byte[getOptimalRequiredSize()];
        ByteBufferList bufferList = new ByteBufferList();
        
        long nbRead = 0;
        do {
            try {
                nbRead = read(ctx, bufferList);
                if (nbRead < 1) {
                    break;
                }
                writeBufferList(bufferList, bytes, outputStr);
            } finally {
                bufferList.clear();
            }
        } while(nbRead > 0);
    }

    public long read(Context ctx, ByteBufferList bufferList)
        throws IOException, StorageServerException {
        return ((ReadHandler) handler).read(ctx, bufferList);
    }

    private void writeBufferList(ByteBufferList bufferList,
      byte[] bytes,
      OutputStream out) throws IOException {

        ByteBuffer[] buffers = bufferList.getBuffers();

        logger.finest("writeBufferList " + bufferList.remaining());

        for (int i = 0; i < buffers.length; i++) {
            int toWrite = buffers[i].remaining();
            buffers[i].get(bytes, 0, toWrite);
            out.write(bytes, 0, toWrite);
        }
    }

}
