/*
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *  
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* Copyright (c) 2007, Sun Microsystems, Inc. */

package org.snia.xam.vim.isolation.server.http;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;

import org.mortbay.http.HttpInputStream;
import org.mortbay.http.HttpResponse;

import org.snia.xam.vim.isolation.server.ifc.StorageServerException;
import org.snia.xam.vim.isolation.server.handlers.StorageHandler;
import org.snia.xam.vim.isolation.server.contextes.Context;
import org.snia.xam.vim.isolation.server.resources.ByteBufferPool;
import org.snia.xam.vim.isolation.server.resources.ByteBufferList;
import org.snia.xam.vim.isolation.server.handlers.WriteHandler;

public class HttpWriteGlue extends HttpGenericHandler
{

    public HttpWriteGlue(StorageHandler handler, String contextPath ) 
        throws StorageServerException{
        super(handler, contextPath);
    }

    protected void handle(Context ctx, 
      OutputStream outputStr, HttpResponse response)
        throws IOException, StorageServerException {
       
        ByteBufferPool bufferPool = ByteBufferPool.getInstance();
        ByteBufferList bufferList = new ByteBufferList();
        int bufferSize = getOptimalRequiredSize();
        byte[] bytes = new byte[bufferSize];
        int read = 0;
        
        HttpInputStream inputStr = (HttpInputStream) response.getInputStream();
        
        while (read >= 0) {
            
            ByteBuffer buffer = bufferPool.checkOutBuffer(bufferSize);
            int bufferRead = 0;
            
            try {
                while (bufferRead < bufferSize &&
                  (read = inputStr.read(bytes, bufferRead, 
                    bufferSize - bufferRead)) >= 0) {
                    bufferRead += read;
                }

                if (bufferRead >= 0) {
                    buffer.put(bytes, 0, bufferRead);
                    buffer.flip();
		    bufferList.appendBuffer(buffer);
                    write(ctx, bufferList);
                }
            } finally {
                bufferList.clear();
            }
        }
    }
    

    private void write(Context ctx, ByteBufferList bufferList)
        throws IOException, StorageServerException {
        ((WriteHandler) handler).write(ctx, bufferList);
    }

    private void flush(Context ctx)
        throws IOException, StorageServerException {
        ((WriteHandler) handler).flush(ctx);
    }
}