/*
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 *      * Neither the name of Storage Networking Industry Association.
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* Copyright (c) 2007, Sun Microsystems, Inc.                                       */

package org.snia.xam.vim.isolation.server.http.handlers;

/**
 * A set of constants used for various XAM handlers.  Constants in this file
 * should apply generally.
 */
public interface ProtocolConstants
{

    /**
     * Property key for the Connect operation's XRI target
     */
    public static final String CONNECT_XRI = "xri";
    
    public static final String VIM_NAME = "vimname";

    /**
     * The header value for the status
     */
    public static final String STATUS_CODE = "status";
    public static final String STATUS_MSG  = "status_msg";

    // Handles
    public static final String XSYSTEM_HANDLE             = "handle";
    public static final String QUALIFIED_XSYSTEM_HANDLE   = "xsystem_handle";
    public static final String XSET_HANDLE                = "handle";
    public static final String XSTREAM_HANDLE             = "handle";
    public static final String XITERATOR_HANDLE           = "handle";
    public static final String XASYNC_HANDLE              = "handle";
    
    /* Handler names, not javadoc'ed because name is self explanatory */
    public static final String VIM_CREATE_XSYSTEM_HANDLER_NAME      = "VIMCreateXSystem";

    public static final String XAM_CREATE_FIELD_ITERATOR_HANDLER    = "XamCreateFieldIterator";    

    public static final String XITERATOR_CLOSE_HANDLER              = "XIteratorClose";
    public static final String XITERATOR_HASNEXT_HANDLER            = "XIteratorHasNext";    
    public static final String XITERATOR_NEXT_HANDLER               = "XIteratorNext";    
    
    public static final String XASYNC_CLOSE_HANDLER             = "XAsyncClose";
    public static final String XASYNC_GET_BYTES_READ_HANDLER    = "XAsyncGetByteRead";
    public static final String XASYNC_GET_BYTES_WRITTEN_HANDLER = "XAsyncGetBytesWritten";
    public static final String XASYNC_GET_STATUS                = "XAsyncGetStatus";
    public static final String XASYNC_GET_XOPID                 = "XAsyncGetXOPID";
    public static final String XASYNC_GET_XSET                  = "XAsyncGetXSet";
    public static final String XASYNC_GET_XSTREAM               = "XAsyncGetXStream";
    public static final String XASYNC_GET_XUID                  = "XAsyncGetXUID";
    public static final String XASYNC_HALT                      = "XAsyncHalt";
    public static final String XASYNC_IS_COMPLETE               = "XAsyncIsComplete";
    public static final String ASYNC_CALLBACK_POLL              = "POLL";    // Internal method
    public static final String ASYNC_CALLBACK_GETDATA           = "GetData"; // Internal method
        
    public static final String XSYSTEM_ABANDON_HANDLER_NAME            = "XSystemAbandon";
    public static final String XSYSTEM_ACCESSXSET_HANDLER_NAME         = "XSystemAccessXSet";
    public static final String XSYSTEM_ASYNC_COPYXSET_HANDLER_NAME     = "XSystemAsyncCopyXSet";
    public static final String XSYSTEM_ASYNC_OPENXSET_HANDLER          = "XSystemAsyncOpenXSet";
    public static final String XSYSTEM_ASYNC_OPENXSTREAM_HANDLE        = "XSystemAsyncOpenXStream";
    public static final String XSYSTEM_AUTHENTICATE_HANDLER            = "XSystemAuthenticate";
    public static final String XSYSTEM_DELETEFIELD_HANDLER_NAME        = "XSystemDeleteField";
    public static final String XSYSTEM_DELETEXSET_HANDLER_NAME         = "XSystemDeleteXSet";
    public static final String XSYSTEM_CONTAINS_FIELD_HANDLER          = "XSystemContainsField";
    public static final String XSYSTEM_CONNECT_HANDLER_NAME            = "XSystemConnect";
    public static final String XSYSTEM_COPY_XSET_HANDLER_NAME          = "XSystemCopyXSet";
    public static final String XSYSTEM_CLOSE_HANDLER_NAME              = "XSystemClose";
    public static final String XSYSTEM_CREATEPROPERTY_HANDLER_NAME     = "XSystemCreateProperty";
    public static final String XSYSTEM_CREATEXSET_HANDLER_NAME         = "XSystemCreateXSet";
    public static final String XSYSTEM_CREATE_XSTREAM_HANDLER          = "XSystemCreateXStream";
    public static final String XSYSTEM_GET_FIELDLENGTH_HANDLER_NAME    = "XSystemGetFieldLength";
    public static final String XSYSTEM_GETPROPERTY_HANDLER_NAME        = "XSystemGetProperty";
    public static final String XSYSTEM_GETPROPERTYTYPE_HANDLER_NAME    = "XSystemGetPropertyType";
    public static final String XSYSTEM_GET_XSET_ACCESS_TIME            = "XSystemGetXSetAccessTime";
    public static final String XSYSTEM_HOLD_XSET                       = "XSystemHoldXSet";
    public static final String XSYSTEM_IS_XSET_RETAINED                = "XSystemIsXSetRetained";
    public static final String XSYSTEM_OPENXSET_HANDLER_NAME           = "XSystemOpenXSet";
    public static final String XSYSTEM_OPEN_XSTREAM_HANDLER            = "XSystemOpenXStream";
    public static final String XSYSTEM_GET_FIELD_BINDING_HANDLER       = "XSystemGetFieldBinding";
    public static final String XSYSTEM_GET_FIELD_READONLY_HANDLER      = "XSystemGetFieldReadOnly";
    public static final String XSYSTEM_RELEASE_XSET                    = "XSystemReleaseXSet";
    public static final String XSYSTEM_SET_FIELD_AS_BINDING_HANDLER    = "XSystemSetFieldAsBinding";
    public static final String XSYSTEM_SET_FIELD_AS_NONBINDING_HANDLER = "XSystemSetFieldAsNonbinding";
    public static final String XSYSTEM_SETPROPERTY_HANDLER_NAME        = "XSystemSetProperty";
 
    public static final String XSET_ABANDON_HANDLER_NAME                    = "XSetAbandon";
    public static final String XSET_ASYNC_OPENXSRTEAM_HANDLER               = "XSetAsyncOpenXStream";
    public static final String XSET_ASYNC_COMMIT_HANDLER                    = "XSetXAsyncCommit";
    public static final String XSET_APPLY_ACCESS_POLICY_HANDLER             = "XSetApplyAccessPolicy";
    public static final String XSET_APPLY_BASE_RETENTION_POLICY_HANDLER     = "XSetApplyBaseRetentionPolicy";
    public static final String XSET_APPLY_AUTODELETE_POLICY_HANDLER         = "XSetApplyAutoDeletePolicy";
    public static final String XSET_APPLY_MANAGEMENT_POLICY_HANDLER         = "XSetApplyManagementPolicy";
    public static final String XSET_APPLY_RETENTION_DURATION_POLICY_HANDLER = "XSetApplyRetentionDurationPolicy";
    public static final String XSET_APPLY_RETENTION_ENABLED_POLICY_HANDLER  = "XSetApplyRetentionEnabledPolicy";
    public static final String XSET_APPLY_AUTODELETE_ENABLED_POLICY_HANDLER = "XSetApplyAutoDeletePolicy";
    public static final String XSET_APPLY_SHRED_POLICY_HANDLER              = "XSetApplyShredPolicy";
    public static final String XSET_APPLY_STORAGE_POLICY_HANDLER            = "XSetStoragePolicy";
    public static final String XSET_CLOSE_HANDLER_NAME                      = "XSetClose";
    public static final String XSET_COMMIT_HANDLER_NAME                     = "XSetCommit";
    public static final String XSET_CONTAINS_FIELD_HANDLER                  = "XSetContainsField";
    public static final String XSET_CREATEPROPERTY_HANDLER_NAME             = "XSetCreateProperty";
    public static final String XSET_CREATE_RETENTION_HANDLER                = "XSetCreateRetention";
    public static final String XSET_CREATE_XSTREAM_HANDLER                  = "XSetCreateXStream";
    public static final String XSET_GET_ACTUAL_AUTODELETE_HANDLER           = "XSetGetActualAutoDelete";
    public static final String XSET_GET_ACTUAL_SHRED_HANDLER                = "XSetGetActualShred";
    public static final String XSET_GET_ACTUAL_RETENTION_DURATION_HANDLER   = "XSetGetActualRetentionDuration";
    public static final String XSET_GET_ACTUAL_RETENTION_ENABLED_HANDLER    = "XSetGetActualRetentionEnabled";
    public static final String XSET_HALTJOB_HANDLER_NAME                    = "XSetHaltJob";
    public static final String XSET_DELETEFIELD_HANDLER_NAME                = "XSetDeleteField";
    public static final String XSET_GET_FIELD_BINDING_HANDLER               = "XSetGetFieldBinding";
    public static final String XSET_GET_FIELDLENGTH_HANDLER_NAME            = "XSetGetFieldLength";
    public static final String XSET_GET_FIELD_READONLY_HANDLER              = "XSetGetFieldReadOnly";
    public static final String XSET_GETPROPERTY_HANDLER_NAME                = "XSetGetProperty";
    public static final String XSET_GETPROPERTYTYPE_HANDLER_NAME            = "XSetGetPropertyType";    
    public static final String XSET_OPEN_EXPORT_XSTREAM_HANDLER             = "XSetOpenExportXStream";
    public static final String XSET_OPEN_IMPORT_XSTREAM_HANDLER             = "XSetOpenImportXStream";
    public static final String XSET_OPEN_XSTREAM_HANDLER                    = "XSetOpenXStream";
    public static final String XSET_RESET_ACCESS_FIELDS_HANDLER             = "XSetResetAccessFields";
    public static final String XSET_RESET_MANAGEMENT_FIELDS_HANDLER         = "XSetResetManagementFields";
    public static final String XSET_SET_AUTODELETE_HANDLER                  = "XSetSetAutoDelete";
    public static final String XSET_SET_BASE_RETENTION_HANDLER              = "XSetSetBaseRetention";
    public static final String XSET_SET_FIELD_AS_BINDING_HANDLER            = "XSetSetFieldAsBinding";
    public static final String XSET_SET_FIELD_AS_NONBINDING_HANDLER         = "XSetSetFieldAsNonbinding";
    public static final String XSET_SETPROPERTY_HANDLER_NAME                = "XSetSetProperty";
    public static final String XSET_SET_RETENTION_DURATION_HANDLER          = "XSetSetRetentionDuration";
    public static final String XSET_SET_RETENTION_ENABLED_FLAG_HANDLER      = "XSetSetRetentionEnabledFlag";
    public static final String XSET_SET_RETENTION_STARTTIME_HANDLER         = "XSetSetRetentionStarttime";
    public static final String XSET_SET_SHRED_HANDLER                       = "XSetSetShred";
    public static final String XSET_SUBMITJOB_HANDLER_NAME                  = "XSetSubmitJob";    

    public static final String XSTREAM_ABANDON_HANDLER              = "XStreamAbandon";    
    public static final String XSTREAM_ASYNC_CLOSE_HANDLER          = "XStreamAsyncClose";
    public static final String XSTREAM_ASYNC_READ_HANDLER           = "XStreamAsyncRead";
    public static final String XSTREAM_ASYNC_WRITE_HANDLER          = "XStreamAsyncWrite";
    public static final String XSTREAM_CLOSE_HANDLER                = "XStreamClose";
    public static final String XSTREAM_READ_HANDLER                 = "XStreamRead";
    public static final String XSTREAM_SEEK_HANDLER                 = "XStreamSeek";
    public static final String XSTREAM_TELL_HANDLER                 = "XStreamTell";
    public static final String XSTREAM_WRITE_HANDLER                = "XStreamWrite";
   

    /**
     * Return code from authenticate call.
     */
    public static final String AUTHENTICATE_STATUS                  = "authenticate_status";
    
    /**
     * A flag key typically inserted into the context to relay that
     * the operation is complete
     */
    public static final String COMPLETION_FLAG = "complete";

    /**
     * Property key for a XUID
     */
    public static final String XUID_PROPERTY = "xuid";

    static final String XRI_HOST_ARG    = "targetServer.ipAddress";
    static final String XRI_PORT_ARG    = "targetServer.port";
    static final String XRI_VIM_ARG     = "targetServer.vimname";

    /**
     * Property key for a mode
     */
    public static final String MODE_PROPERTY = "mode";

    public static final String XSYSTEM_ACCESSIBLE_PROPERTY = "accessible";
    
    public static final String XITERATOR_CLASS_XSYSTEM = "XSystem";
    public static final String XITERATOR_CLASS_XSET    = "XSet";

    // Field Attributes
    public static final String ATTRIBUTE_TYPE    = "ptype";
    public static final String ATTRIBUTE_NAME    = "name";
    public static final String TYPE_PROPERTY     = "type";
    public static final String ATTRIBUTE_BINDING = "binding";
    public static final String ATTRIBUTE_VALUE   = "value";
    
    // XAsync values
    public static final String ASYNC_COMPLETE      = "async_complete";
    public static final String ASYNC_STATUS        = "async_status";
    public static final String ASYNC_STATUS_MSG    = "async_statusMsg";
    public static final String ASYNC_XOPID         = "async_xopid";
    public static final String ASYNC_XSET          = "async_xset";
    public static final String ASYNC_XSTREAM       = "async_xstream";
    public static final String ASYNC_XUID          = "async_xuid";
    public static final String ASYNC_BYTES_READ    = "async_bytesread";
    public static final String ASYNC_BYTES_WRITTEN = "async_byteswritten";

    // XStream 
    public static final String XSTREAM_NUMBYTES = "numbytes";
    public static final String XSTREAM_WHENCE   = "whence";
    public static final String XSTREAM_OFFSET   = "offset";
    public static final String XSTREAM_POSITION = "position";
    
    // XIterator constants.
    public static final String CLASS_TYPE_PROPERTY = "class";
    public static final String PATTERN_PROPERTY    = "pattern";
    public static final String PROPERTY_PROPERTY   = "property";
    public static final String HASNEXT_PROPERTY    = "hasnext";
    
    // Property types
    public static final String PTYPE_BOOLEAN  = "boolean";
    public static final String PTYPE_INTEGER  = "integer"; 
    public static final String PTYPE_DOUBLE   = "double";
    public static final String PTYPE_XUID     = "xuid";
    public static final String PTYPE_STRING   = "string";
    public static final String PTYPE_DATETIME = "date_time";
    
    public static final String XAM_BOOLEAN  = "xam_boolean";
    public static final String XAM_INTEGER  = "xam_int"; 
    public static final String XAM_DOUBLE   = "xam_double";
    public static final String XAM_XUID     = "xam_xuid";
    public static final String XAM_STRING   = "xam_string";
    public static final String XAM_DATETIME = "xam_datetime";

    // XSystem method parameters
    public static final String HOLD_ID        = "holdid";

   // Policy arguments
   public static final String ARGUMENT_POLICY       = "policy";
   public static final String ARGUMENT_RETENTION_ID = "retentionid";
   public static final String ARGUMENT_AUTODELETE   = "autodelete";
   public static final String ARGUMENT_SHRED        = "shred";
   public static final String ARGUMENT_DURATION     = "duration";
   public static final String ARGUMENT_ENABLED      = "enabled";
   public static final String ARGUMENT_ACCESSTIME   = "accesstime";
   public static final String ARGUMENT_RETAINED     = "retained";

   // Polling arguments
   public static final String ARGUMENT_LIMIT        = "limit";
   // For URL encoding
   public static final String UTF8                   = "utf-8";
   public static final String TRANSPORT_ENCODE_ERROR = "Error performing URL safe encoding";

   public static final String UNEXPECTED_EXCEPTION = "Operation failed due to an unexpected exception: ";
   public static final String XUID_DECODE_ERROR    = "Hex Encoded XUID value is malformed";
}

