/*
 *
 * Copyright (c) 2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 *      * Neither the name of the Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package org.snia.xam.vim.isolation.server.http.handlers;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.logging.Level;
import org.snia.xam.XAMException;
import org.snia.xam.XAsync;
import org.snia.xam.vim.isolation.server.contextes.ConnectContextKey;
import org.snia.xam.vim.isolation.server.contextes.Context;
import org.snia.xam.vim.isolation.server.contextes.ContextKey;
import org.snia.xam.vim.isolation.server.ifc.StorageServerException;
import org.snia.xam.vim.isolation.server.resources.ByteBufferList;
import org.snia.xam.vim.isolation.server.util.XAsyncCache;

/**
 * Implements the handler for the HTTP wire "XAsyncGetBytesRead" operation.
 */
public class XAsyncGetBytesRead extends AbstractReadHandler {
    
    protected ContextKey contextKey = null;
    
    public XAsyncGetBytesRead() throws StorageServerException {
        m_name = ProtocolConstants.XASYNC_GET_BYTES_READ_HANDLER;
        setupLogger();
    }
    
     public ContextKey createContext(Map<String,String> inProps, InputStream input, Map<String,String> outProps, OutputStream output, Context ctx) {
        contextKey = new ConnectContextKey();
        logProperties( inProps );

        getAndRegister( inProps, ctx, ProtocolConstants.XASYNC_HANDLE, DEFAULT_HANDLE );
                
        return contextKey;
    }
    
     /** Perform the XAsyncGetBytesRead logic */
    public long read(Context ctx, ByteBufferList bufferList) {
        if(exists( ctx, ProtocolConstants.COMPLETION_FLAG) ) return 0;
        logger.entering(this.getClass().getName(), "read");
        
        long   status     = XAMException.SUCCESS;
        String statusMsg  = null;
        long   bytes      = -1;
        try { 
           Long handleAsLong = getHandle( ctx, ProtocolConstants.XASYNC_HANDLE);
        
           long handle = handleAsLong.longValue();
           
           XAsyncCache xac      = XAsyncCache.getInstance(logger);
           XAsync      async    = xac.getXAsync(handle);
           if( logger.isLoggable( Level.INFO ))
              logger.info( "XAsync[" + handle + "].getBytesRead()" );
           bytes = async.getBytesRead();
        } catch (XAMException ex) {
           status = ex.getStatusCode();
           statusMsg = ex.getMessage();
        } catch(Exception e) {
           status = XAMException.sXAMException;
           statusMsg = ProtocolConstants.UNEXPECTED_EXCEPTION + e.getMessage();
           logger.log( Level.SEVERE, ProtocolConstants.UNEXPECTED_EXCEPTION, e );
        }
        
        StringBuffer response = new StringBuffer();
        respond( response, ProtocolConstants.STATUS_CODE, status );
        if( statusMsg != null )
           respond( response, ProtocolConstants.STATUS_MSG, statusMsg );

        if( status == XAMException.SUCCESS ) 
           respond( response, ProtocolConstants.ASYNC_BYTES_READ, bytes );
        
        logger.exiting(this.getClass().getName(), "read");
        return packResponse( ctx, bufferList, response );
    } 
}
