/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *  
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

package org.snia.xam.vim.isolation.server.ifc;

import org.snia.xam.vim.isolation.server.handlers.StorageHandler;

public interface StorageServer {

    /**
     * Register the handler at the specific version.
     *
     * @param  handler handler to be registered
     * @param  version version of the specific handler
     * @throws StorageServerException if the handler is invalid or has
     *         already been registered.
     * @see <a href="StorageHandler.html">StorageHandler</a>
     */
    public void register(StorageHandler handler, String version)
        throws StorageServerException;

    /**
     * Unregister the handler at the specific version.
     *
     * @param  handler handler to be registered
     * @param  version version of the specific handler
     * @throws StorageServerException is the handler is not registered.
     * @see           <code>StorageHandler</code>
     */
    public void unregister(StorageHandler handler, String version)
        throws StorageServerException;


    /**
     * Set the minimum number of threads that will handle incoming connection.
     * By default the minum is 1.
     *
     * @param nbThreads min number of threads
     * @throws StorageServerException is the server is already running
     */
    public void setMinThreads(int nbThreads)
        throws StorageServerException;

    /**
     * Set the maximum number of threads that will handle incoming connection.
     * By default the minumum is 5.
     *
     * @param nbThreads max number of threads
     * @throws StorageServerException is the server is already running
     */
    public void setMaxThreads(int nbThreads)
        throws StorageServerException;

    /**
     * Start the <code>StorageServer</code>
     *
     * @throws StorageServerException if the port specified is not available.
     */
    public void start()
        throws StorageServerException;

    /**
     * Stop the <code>StorageServer</code>
     *
     * @param force specifes if we should wait for the threads to complete
     */
    public void stop(boolean force)
        throws InterruptedException;

    /**
     * Returns whether or not the <code>StorageServer</code> has been started.
     *
     * @return true if the server has been started
     */
    public boolean isStarted();
}