/*
 *
 * Copyright (c) 2007, Sun Microsystems, Inc.
 * Copyright (c) 2007-2009, Storage Networking Industry Association
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 *      * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 *      * Neither the name of Storage Networking Industry Association
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package org.snia.xam.vim.isolation.server.util;

/**
 * A set of constants used for various Isolation Server startup operations.
 */
public interface Constants
{
    /**
     * Property file used for the Isolation VIM startup parameters
     */
    static final String STARTUP_PROPERTY_FILE = "startup.properties";
    
    /**
     * Property file used for the Isolation VIM handlers for loading
     */
    static final String LOAD_HANDLER_FILE = "handlers.properties";
    
    /**
     * Property file used for determining the VIMs that 
     */
    static final String LOAD_VIM_FILE = "vim.properties";
    
    /**
     * Default value of the server version if the STARTUP_PROPERTY_FILE
     * cannot be found
     * @see Constants#STARTUP_PROPERTY_FILE
     */
    static final String SERVER_VERSION = "1.0";

    /**
     * Default value of the server host if the STARTUP_PROPERTY_FILE
     * cannot be found.  This is specified as in IP Address.
     * @see Constants#STARTUP_PROPERTY_FILE
     */
    static final String SERVER_HOST = "127.0.0.1";

    /**
     * Default value of the port if the STARTUP_PROPERTY_FILE
     * cannot be found.  Note that the port is specified as a 
     * String here as opposed to an int.  Use of a String will
     * make processing of the property file and default value
     * simpler (same path).
     * @see Constants#STARTUP_PROPERTY_FILE
     */
    static final String SERVER_PORT = "9923";
    
    /**
     * Default value for the set of handlers to load
     */
    static final String RUNTIME_HANDLERS = "Connect";
    
    /**
     * List of all possible handlers to load
     */
    static final String[][] AVAILABLE_HANDLERS = new String[][]{
        {"Connect","org.snia.xam.vim.isolation.server.http.handlers.Connect"}
    };
    
    /**
     * The default version for all connectors implemented 
     */
    static final String DEFAULT_VERSION = "1.0";
    
    /**
     * A constant representing the namespace value that is used in
     * the configuration file to indicate a default vimname
     */
    static final String DEFAULT_VIMNAME = ".org.snia.xam.vim.default";
    
    /**
     * A constant representing the alias namespace
     */
    static final String ALIAS_NAMESPACE = ".org.snia.xam.vim.alias";
    
    /**
     * HTTP VIM Server is going to use a log which is separate from the XSystem logs.
     */
    static final String LOGGER_NAME = "HTTPVimServer.log";
}
