/* Copyright (c) 2009, The Storage Networking Industry Association.

   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA) nor 
   the names of its contributors may be used to endorse or promote products 
   derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
   THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.snia.xam.vim.http;

import org.snia.xam.XSet;
import org.snia.xam.XStream;
import org.snia.xam.XSystem;
import org.snia.xam.util.SASLUtils;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;

public class HTTPXStreamTest
   extends XAMTest
{
   public HTTPXStreamTest( String name )
   { 
      super(name);
   }
   
   public static int runTest(String[] args, Test test)
   {
       TestResult res = junit.textui.TestRunner.run(test);
       return Math.max(res.failureCount(), res.errorCount());
   }
   
  public static Test suite()
  {
     TestSuite suite = new TestSuite();
     suite.addTest(new HTTPXStreamTest("testXStreamReadWrite"));
     suite.addTest(new HTTPXStreamTest("testHideAndGoSeek"));
     return suite;
  }

   public void testXStreamReadWrite() throws Exception
   {
      final String SNAME = "XStreamUnitTest";
      final String TEST  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      
      XSystem xsystem = s_xam.connect(s_xri);
      SASLUtils.authenticatePlain( xsystem, "testuser", "testpasswd" );
      XSet xs = xsystem.createXSet(XSet.MODE_UNRESTRICTED);
      XStream stream = xs.createXStream( SNAME, false, "text/plain" );
      
      StringBuffer testBuf = new StringBuffer();
      for(int i = 0; i < 50; i++ )
         testBuf.append(TEST);
      String test_str = testBuf.toString();
      stream.write( test_str.getBytes() );
      stream.close();
      
      int bufferSize = test_str.length();
      XStream s2 = xs.openXStream( SNAME, XStream.MODE_READ_ONLY );
      byte buffer[] = new byte[bufferSize];
      long len = s2.read( buffer );
      if( len < test_str.length() )
         fail( "Not enough bytes read from the stream: " + len);
      
      String test = new String(buffer, 0, (int) len );
      assertEquals( "String did not read back from stream correctly", test_str, test );
      s2.close();
      test = null;
      
      XStream s3 = xs.openXStream( SNAME, XStream.MODE_READ_ONLY );
      byte buffer3[] = new byte[bufferSize];
      long len3 = s3.read( buffer, 4 );
      assertEquals( "Wrong # bytes read from stream s3", 4, len3 );
      String test3 = new String(buffer, 0, (int) len3 );
      assertEquals( "String 3 did not read back from stream correctly", test_str.substring(0, 4), test3 );      
      s3.close();
      
      XStream s4 = xs.openXStream( SNAME, XStream.MODE_READ_ONLY );
      byte buffer4[] = new byte[bufferSize];
      long len4 = s4.read( buffer, 20, 4 );
      assertEquals( "Wrong # bytes read from stream s4", 4, len4 );
      String test4 = new String(buffer, 20, (int) len3 );
      assertEquals( "String 3 did not read back from stream correctly", test_str.substring(0, 4), test4 );      
      s4.close();
      
      xs.close();
      xsystem.close();
   }
   
   public void testHideAndGoSeek() throws Exception
   {
      final String SNAME = "XStreamUnitTest";
      final String TEST  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      
      XSystem xsystem = s_xam.connect(s_xri);
      SASLUtils.authenticatePlain( xsystem, "testuser", "testpasswd" );
      XSet xs = xsystem.createXSet(XSet.MODE_UNRESTRICTED);
      XStream stream = xs.createXStream( SNAME, false, "text/plain" );
      
      StringBuffer testBuf = new StringBuffer();
      for(int i = 0; i < 50; i++ )
         testBuf.append(TEST);
      String test_str = testBuf.toString();
      stream.write( test_str.getBytes() );
      stream.close();
      
      int bufferSize = test_str.length();
      XStream s2 = xs.openXStream( SNAME, XStream.MODE_READ_ONLY );
      s2.seek( TEST.length(), XStream.SEEK_CUR );
      byte buffer[] = new byte[TEST.length()];
      long len = s2.read( buffer );
      if( len < TEST.length() )
         fail( "Not enough bytes read from the stream: " + len);
      String testStr = new String(buffer, 0, (int) len );
      assertEquals( "Substrings did not compare", TEST, testStr );
      
      long position = s2.tell();
      assertEquals( "Stream cursor not where expected", TEST.length() * 2, position );      
      s2.close();
      
      xs.close();
      xsystem.close();
   }
   
   /**
    * Stand alone tests.
    */
   public static void main(String[] args)
   {
      System.exit(runTest(args, suite()));
   }
}
