/******************************************************************************
 *
 * Copyright (c) 2007-2008, The Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *    - Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    - Neither the name of The Storage Networking Industry Association (SNIA)
 *      nor the names of its contributors may be used to endorse or promote 
 *      products derived from this software without specific prior written 
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

package org.snia.xam;

/**
 * The base exception for the XAM interface. These exceptions must be caught.
 * 
 * All XAMExceptions will contain a status code, as required by the XAM
 * Architecture Specification. The status code is to be maintained and
 * retrievable by applications. The XAM Exception message string is identical to
 * the <i>error token</i> as defined in the XAM Architecture Specification.
 */
public class XAMException
   extends Exception
{
   // These status codes are lifted from the C XAM Library.
   // The codes listed here are NOT standardized as part of the XAM Architecture Specification,
   // but are included here so that sub-classed exceptions can set the status appropriately.
   public static final int    SUCCESS                           = 0;
   public static final int    sXAMException                     = 1001;
   public static final int    sXAM_OUT_OF_MEMORY                = 1002;
   public static final int    sInvalidArgumentException         = 1003;
   public static final int    sXAM_PARAM_NOT_UTF8               = 1004;
   public static final int    sXAM_INVALID_HANDLE               = 1005;
   public static final int    sInvalidFieldTypeException        = 1006;
   public static final int    sInvalidXStreamModeException      = 1007;
   public static final int    sInvalidXRIException              = 1008;
   public static final int    sInvalidXSetModeException         = 1009;
   public static final int    sInvalidFieldNameException        = 1010;
   public static final int    sVIMLoadException                 = 1011;
   public static final int    sXAM_VIM_SYMBOL_NOT_FOUND         = 1012;
   public static final int    sFieldDoesNotExistException       = 1013;
   public static final int    sFieldReadOnlyException           = 1014;
   public static final int    sFieldExistsException             = 1015;
   public static final int    sFieldInUseException              = 1016;
   public static final int    sMaximumFieldException            = 1017;
   public static final int    sXAM_FILESYSTEM_ERROR             = 1018;
   public static final int    sXSystemAbandonException          = 1019;
   public static final int    sXSetAbandonException             = 1020;
   public static final int    sXStreamAbandonException          = 1021;
   public static final int    sXSystemCorruptException          = 1022;
   public static final int    sXSetCorruptException             = 1023;
   public static final int    sXStreamCorruptException          = 1024;
   public static final int    sConnectException                 = 1025;
   public static final int    sAuthorizationException           = 1026;
   public static final int    sXAM_AUTH_DATA_NEEDED             = 1027;
   public static final int    sAuthenticationException          = 1028;
   public static final int    sInvalidXUIDException             = 1029;
   public static final int    sXSetDoesNotExistException        = 1030;
   public static final int    sAsyncPendingException            = 1031;
   public static final int    sXAM_NOT_SUPPORTED                = 1032;
   public static final int    sInvalidOperationException        = 1033;
   public static final int    sObjectInUseException             = 1034;
   public static final int    sXAM_NOT_A_JOB                    = 1035;
   public static final int    sJobCommandException              = 1036;
   public static final int    sXAM_JOB_INVALID_CMD_SYNTAX       = 1037;
   public static final int    sJobUnsupportedException          = 1038;
   public static final int    sJobPermissionsException          = 1039;
   public static final int    sJobResourcesException            = 1040;
   public static final int    sJobRunningException              = 1041;
   public static final int    sXSetUnderRetentionException      = 1042;
   public static final int    sXSetUnderHoldException           = 1043;
   public static final int    sHoldIdException                  = 1044;
   public static final int    sRetentionValueException          = 1045;
   public static final int    sPolicyNameException              = 1046;
   public static final int    sXAM_XASYNC_HALTED                = 1047;
   public static final int    sPolicyMismatchException          = 1048;

   long m_status = sXAMException;

   public XAMException()
   {
   }

   public XAMException(String arg0)
   {
      super(arg0);
   }

   public XAMException(String arg0, long status)
   {
      super(arg0);
      m_status = status;
   }

   public XAMException(Throwable arg0)
   {
      super(arg0);
   }

   public XAMException(Throwable arg0, long status)
   {
      super(arg0);
      m_status = status;
   }

   public XAMException(String arg0, Throwable arg1)
   {
      super(arg0, arg1);
   }

   public XAMException(String arg0, Throwable arg1, long status)
   {
      super(arg0, arg1);
      m_status = status;
   }

   public long getStatusCode()
   {
      return m_status;
   }
}
