/******************************************************************************
 *
 * Copyright (c) 2007-2009, The Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *    - Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    - Neither the name of The Storage Networking Industry Association (SNIA)
 *      nor the names of its contributors may be used to endorse or promote 
 *      products derived from this software without specific prior written 
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

package org.snia.xam;

/**
 * A XSet is the basic unit of storage for the XAM System. XSet instances are
 * implemented by storage vendors in the storage vendor's packages. These XSet
 * instances will perform storage vendor specific interactions with the XAM
 * Storage System, as well as acting as a factory source for XStream objects
 * (via create/open methods).<br>
 * NOTE: Changing the BINDING information on any persistently stored XSet Field
 * will result in a new XSet being created upon commit. This is only true when
 * the XSet has been opened in UNRESTRICTED mode. Changes may be altering the
 * value of a BOUND field, or changing the BINDING attribute of a field.
 */
public interface XSet extends FieldContainer
{
    /**
     * Open an XSet in READ ONLY Mode. Only reading of field operations is
     * permitted on an XSet opened in this mode. All attempts to commit will
     * fail. Modification attempts should fail as early as possible.
     */
    public final static String MODE_READ_ONLY = "readonly";

    /**
     * Open an XSet in RESTRICTED Mode. Reading of all fields is allowed and
     * modifications of UNBOUND field is permitted. Committing an XSet with a
     * BINDING modification will fail, and attempts to modify a BOUND field, or
     * change the BINDING attribute of a field should fail as soon as possible.
     */
    public final static String MODE_RESTRICTED = "restricted";

    /**
     * Open an XSet in UNRESTRICTED Mode. This mode allows reading and
     * modification of all fields. Changing the BINDING attribute, or changing
     * the value of a BOUND field will result in a the creation of a new XSet
     * with a new XUID value upon commit.
     */
    public final static String MODE_UNRESTRICTED = "unrestricted";

    // == Basic Built in Fields ==
    public final static String XAM_TIME_CREATION = ".xset.time.creation";

    public final static String XAM_TIME_XUID = ".xset.time.xuid";
    
    public final static String XAM_TIME_COMMIT = ".xset.time.commit";

    public final static String XAM_TIME_ACCESS = ".xset.time.access";

    public final static String XAM_TIME_RESIDENCY = ".xset.time.residency";

    public final static String XAM_XUID = ".xset.xuid";

    public final static String XAM_DIRTY = ".xset.dirty";

    // == Management Fields ==/
    public final static String XAM_RETENTION_LIST = ".xset.retention.list.";
    
    public final static String XAM_RETENTION_LIST_BASE = ".xset.retention.list.base";
    
    public final static String XAM_RETENTION_LIST_EVENT = ".xset.retention.list.event";
    
    public final static String XAM_RETENTION_BASE_STARTTIME = ".xset.retention.base.starttime";
    
    public final static String XAM_RETENTION_BASE_ENABLED = ".xset.retention.base.enabled";
    
    public final static String XAM_RETENTION_BASE_DURATION = ".xset.retention.base.duration";
    
    public final static String XAM_RETENTION_BASE_ENABLED_POLICY = ".xset.retention.base.enabled.policy";
    
    public final static String XAM_RETENTION_BASE_DURATION_POLICY = ".xset.retention.base.duration.policy";
    
    public final static String XAM_RETENTION_EVENT_STARTTIME = ".xset.retention.event.starttime";
    
    public final static String XAM_RETENTION_EVENT_DURATION = ".xset.retention.event.duration";
    
    public final static String XAM_RETENTION_EVENT_ENABLED = ".xset.retention.event.enabled";
    
    public final static String XAM_RETENTION_EVENT_DURATION_POLICY = ".xset.retention.event.duration.policy";

    public final static String XAM_RETENTION_EVENT_ENABLED_POLICY = ".xset.retention.event.enabled.policy";
    
    public final static String XAM_MANAGEMENT_POLICY = ".xset.management.policy";

    public static final String XSET_HOLD = ".xset.hold";
    
    public static final String XSET_HOLD_LIST = ".xset.hold.list.";

    public final static String XAM_DELETION_AUTODELETE = ".xset.deletion.autodelete";

    public final static String XAM_DELETION_AUTODELETE_POLICY = ".xset.deletion.autodelete.policy";
    
    public final static String XAM_DELETION_SHRED = ".xset.deletion.shred";

    public final static String XAM_DELETION_SHRED_POLICY = ".xset.deletion.shred.policy";

    public final static String XAM_STORAGE_POLICY = ".xset.storage.policy";
    
    public final static String XAM_ACCESS_POLICY = ".xset.access.policy";
    
    // == Job Properties/Values
    public final static String XAM_JOB_STATUS = ".xam.job.status";

    public final static String XAM_JOB_STATUS_NEW = "NEW";

    public final static String XAM_JOB_STATUS_STARTING = "STARTING";

    public final static String XAM_JOB_STATUS_RUNNING = "RUNNING";

    public final static String XAM_JOB_STATUS_SHUTTING_DOWN = "SHUTTING DOWN";

    public final static String XAM_JOB_STATUS_COMPLETE = "COMPLETE";

    public final static String XAM_JOB_STATUS_SUSPENDED = "SUSPENDED";

    public final static String XAM_JOB_STATUS_HALTED = "HALTED";

    public final static String XAM_JOB_STATUS_KILLED = "KILLED";

    public final static String XAM_JOB_ERRORHEALTH = ".xam.job.errorhealth";

    public final static String XAM_JOB_ERRORHEALTH_OK = "OK";

    public final static String XAM_JOB_ERRORHEALTH_ERROR = "ERROR";

    public final static String XAM_JOB_ERROR = ".xam.job.error";

    // == Query Job Fields ==
    public final static String XAM_JOB_QUERY = "xam.job.query";

    public final static String XAM_JOB_COMMAND = "org.snia.xam.job.command";

    public final static String XAM_JOB_QUERY_COMMAND = "xam.job.query.command";

    public final static String XAM_JOB_QUERY_RESULTS = "xam.job.query.results";

    public final static String XAM_JOB_QUERY_RESULTS_COUNT = "xam.job.query.results.count";

    public final static String XAM_JOB_QUERY_LEVEL = "xam.job.query.level";

    public static final String XAM_JOB_ERROR_LEVEL_NOT_SUPPORTED =
        "xam.job.query::level_not_supported";

    public static final String XAM_JOB_ERROR_INVALID_COMMAND_SYNTAX =
        "xam.job.query::invalid_command_syntax";

    public static final String XAM_JOB_ERROR_INSUFFICIENT_PERMISSIONS =
        "xam.job.query::insufficient_permission";

    public static final String XAM_JOB_ERROR_INSUFFICIENT_RESOURCES =
        "xam.job.query::insufficient_resources";

    // == Data Management ==
    /**
     * This method will create or modify a property field with the name of \
     * '.xset.access.policy' and a type set to 'application/vnd.snia.xam.string' on the 
     * XSet. Its value and binding attributes will be set according to the 
     * user-provided parameters. This field will be used by the XAM Storage System 
     * to determine the policies to use when accessing this XSet.
     * <b>Note</b>: If an access policy has not been applied to an XSet at the 
     * time of the initial commit, then the property will be created and set as 
     * the default access policy of the XSystem (i.e. the first string in the XSystem
     * AccessPolicyList).
     * 
     * @param binding
     *            TRUE if this value is to be BOUND.
     * @param policyName
     *            Name of the policy.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws MaximumFieldException Too many fields in the XSet.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyAccessPolicy( boolean binding, String policyName )
       throws AuthenticationExpiredException, MaximumFieldException, 
              PolicyNameException, ObjectInUseException, 
              XSetAbandonException, XSetCorruptException, XAMException;
    
    /**
     * This method will remove all access fields from the XSet.
     * 
     * <b>Note</b>: If an access policy has not been applied to an XSet at the 
     * time of the initial commit, then the property will be created and set as 
     * the default access policy of the XSystem (i.e., the first string in the 
     * XSystem AccessPolicyList).
     *  
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void resetAccessFields( )
       throws AuthenticationExpiredException, ObjectInUseException,
              XSetAbandonException, XSetCorruptException, XAMException;
    
    /**
     * This method creates and sets the XAM String property
     * ".xam.management.policy". This field is used by the XAM Storage System to
     * determine the default policies when managing the XSet. If the management
     * policy has not been set at the time of first commit, then the property
     * will be created and set as the default management policy of the XSystem.
     * 
     * @param binding
     *            TRUE if this value is to be BOUND.
     * @param policyName
     *            Name of the policy.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws MaximumFieldException Too many fields in the XSet.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyManagementPolicy(boolean binding, String policyName)
       throws AuthenticationExpiredException, MaximumFieldException, PolicyNameException, 
              ObjectInUseException, XSetAbandonException, 
              XSetCorruptException, XAMException;
    
    /**
     * Remove all management fields from the XSet. This includes the field named
     * ".xset.base.retention.starttime". Because this is a binding field, a
     * new XSet will be created upon commit, if the XSet was opened in the
     * unrestricted mode.
     * 
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void resetManagementFields() 
       throws AuthenticationExpiredException, ObjectInUseException, 
              XSetAbandonException, XSetCorruptException, XAMException;
    /**
     * This method will create a scope for storing and evaluating retention criteria. 
     * It creates a field with the type of 'application/vnd.snia.xam.string' and sets 
     * the value to the retention ID. The field name is formed by appending the retention id to the 
     * prefix '.xset.retention.list'. The final form of the field name is 
     * .xset.retention.list.<retentionID>. 
     * An exception is thrown if the retentionID is "base".
     * The binding attribute of this field is set according to the parameter <i>binding</i>.
     * 
     * @param binding Makes the retention settings bound if TRUE, otherwise unbound.
     * @param retentionID The unique ID for this retention scope.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws InvalidFieldNameException The generated field name is too long.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void createRetention( boolean binding,
    							                             String  retentionID )
       throws AuthenticationExpiredException, InvalidArgumentException, 
              InvalidFieldNameException, ObjectInUseException, 
              XSetAbandonException, XSetCorruptException, XAMException;
    
    /**
     * This method will enable or disable retention scoped by the specified retention id. 
     * This flag is stored in a field of type 'application/vnd.snia.xam.boolean'. 
     * The name of the field is formed by inserting the retention id between a prefix 
     * (.xset.retention.) and a suffix (.enabled); thus the final format of the name is
     * .xset.retention.<retention id>.enabled. If the field does not exist it will be created; 
     * otherwise the value will be updated if and only if the value is changed from FALSE to 
     * TRUE - if the value is set to true it cannot be changed. An exception is thrown if 
     * the retentionID is "base". It will have its binding 
     * attribute set in accordance with the binding flag set by the application.
     * 
     * @param retentionID
     *            The identifier for the retention being enabled.
     * @param binding
     *            TRUE if the property is BOUND.
     * @param enabled
     *            TRUE if retention is to be enabled.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws InvalidFieldNameException The generated field name is too long.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void setRetentionEnabledFlag(String retentionID, boolean binding, boolean enabled)
       throws AuthenticationExpiredException, InvalidArgumentException, 
              InvalidFieldNameException, ObjectInUseException, 
              XSetAbandonException, XSetCorruptException, XAMException;

    /**
     * This method will enable or disable retention scoped by the specified retention id as determined 
     * by the named policy. The policy name of the policy holding the enabled flag is stored 
     * in a field of type 'application/vnd.snia.xam.string'. The name of the field is formed 
     * by inserting the retention id between a prefix (.xset.retention.) and a suffix (.enabled.policy); 
     * thus the final format of the name is .xset.retention.<retention id>.enabled.policy. 
     * If the field does not exist it will be created; otherwise the value will be updated if 
     * and only if the value is changed from false to true - if the value is set to TRUE it 
     * cannot be changed. An exception is thrown if the retentionID is "base". 
     * It will have its binding attribute set in accordance with the binding 
     * flag set by the application.
     * 
     * @param retentionID
      *            The identifier of the retention to which the policy shall apply
     * @param binding
     *            TRUE if the value is to be BOUND.
     * @param policy
     *            Name of the policy.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws InvalidFieldNameException The generated field name is too long.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyRetentionEnabledPolicy( String retentionID, boolean binding, String policy)
       throws AuthenticationExpiredException, InvalidArgumentException,
              InvalidFieldNameException, PolicyNameException, 
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * Evaluates all management settings for this XSet to return the effective event retention.
     * @param retentionID The String name of the retention.
     * @return The boolean value of the enabled flag for the specified retention.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws PolicyMismatchException When an imported XSet's policy does not match the policy present in the XSystem.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public boolean getActualRetentionEnabled( String retentionID )
       throws AuthenticationExpiredException, InvalidArgumentException,
              PolicyMismatchException,
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * This method will set the duration of retention scoped by the specified retention id. 
     * This flag is stored in a field of type 'application/vnd.snia.xam.int'. 
     * The name of the field is formed by inserting the retention id between a prefix 
     * (.xset.retention.) and a suffix (.duration); thus the final format of the name is
     * .xset.retention.<retention id>.duration. An exception is thrown is the retentionID is "base". 
     * If the field does not exist it will be created; 
     * otherwise the value will be updated if and only if the duration is increased. 
     * It will have its binding attribute set in accordance with the binding flag set by the application.
     * 
     * @param retentionID
     *            The identifier for the retention for the specified duration
     * @param binding
     *            True if the value is to be BOUND.
     * @param duration
     *            The duration (milliSeconds) for the retention duration,
     *            relative to .xam.ctime.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws InvalidFieldNameException The generated field name is too long.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws RetentionValueException Thrown if the duration parameter specifies an illegal duration, such as a value which attempts to shorten the existing duration value.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void setRetentionDuration( String retentionID, boolean binding, long duration)
       throws AuthenticationExpiredException, InvalidArgumentException,
              InvalidFieldNameException, RetentionValueException,
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    /**
     * This method will set the duration of retention scoped by the specified retention id as 
     * specified by the named policy. This policy name is stored in a field of type 
     * 'application/vnd.snia.xam.string'. The name of the field is formed by inserting the 
     * retention id between a prefix (.xset.retention.) and a suffix (.duration.policy); 
     * thus the final format of the name is .xset.retention.<retention id>.duration.policy. 
     * An exception is thrown is the retentionID is "base". If the
     * field does not exist it will be created; otherwise the value will be updated if and only 
     * if the duration is increased. It will have its binding attribute set in accordance 
     * with the binding flag set by the application.
     * 
     * @param retentionID
     *            The identifier of the retention to which the policy shall apply
     * @param binding
     *            TRUE if this value is to be BOUND.
     * @param inPolicy
     *            Name of the policy
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws InvalidFieldNameException The generated field name is too long.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyRetentionDurationPolicy( String retentionID,
                                              boolean binding,
                                              String  inPolicy) 
       throws AuthenticationExpiredException, InvalidArgumentException,
              InvalidFieldNameException, PolicyNameException,
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * Evaluates all management settings for this XSet to return the effective retention duration.
     * @param retentionID The retention ID name String.
     * @return The effective retention duration (milliSeconds) of specified retention.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws PolicyMismatchException When an imported XSet's policy does not match the policy present in the XSystem.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public long getActualRetentionDuration( String retentionID )
       throws AuthenticationExpiredException, InvalidArgumentException,
              PolicyMismatchException,
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * Create the property ".xset.event.retention.starttime" on the XSet. The
     * value of this property is the current time. If this property already
     * exists, a runtime exception will be thrown. An exception is thrown is the retentionID is "base".
     * This field is used by the XAM Storage system to determine the event retention epoch to use when
     * managing this XSet.
     * 
     * @param retentionID The identifier of the retention to start
     * @param binding TRUE if this value is to be BOUND.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws FieldExistsException The property .xset.retention.<retentionID>.starttime exists, meaning that the start time has already been set for this retention ID.
     * @throws InvalidArgumentException The parameter retentionID is null or malformed.
     * @throws InvalidFieldNameException The generated field name is too long.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void setRetentionStarttime( String retentionID, boolean binding ) 
       throws AuthenticationExpiredException, FieldExistsException,
              InvalidArgumentException, InvalidFieldNameException,
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * If this XSet does not already contain the field .xset.retention.list.base, this method will 
     * create the field with a type of 'application/vnd.snia.xam.string' and set the value to 'base'. 
     * It will also create the 'application/vnd.snia.xam.boolean' field .xset.retention.base.enabled 
     * and set the value to true. The duration will be stored in a field named .xset.retention.base.duration. 
     * This field is of type 'application/vnd.snia.xam.int'. If the field already exists, its value will 
     * be changed to match the passed in duration if and only if the duration of the retention is not reduced; 
     * the method will generate an error if the duration is reduced. If the field does not already exist, 
     * it will be created with the specified duration as the value. The ".xset.retention.base.duration"
     * field will have its binding attribute set according to the binding flag that is set by the 
     * application. The ".xset.retention.list.base" is always a binding flag.
     * 
     * @param binding TRUE if this value is to be BOUND.
     * @param duration The duration (milliSeconds) for the retention duration, relative to the retention starttime.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws RetentionValueException Thrown if the duration parameter specifies an illegal duration, such as a value which attempts to shorten the existing duration value.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void setBaseRetention( boolean binding, 
    		                      long    duration )
       throws AuthenticationExpiredException, RetentionValueException, 
              ObjectInUseException, XSetAbandonException, 
              XSetCorruptException, XAMException;  
    
    /**
     * If this XSet does not already contain the field .xset.retention.list.base, this method will create the 
     * field with a type of 'application/vnd.snia.xam.string' and set the value to 'base'. It will also create 
     * the 'application/vnd.snia.xam.boolean' field .xset.retention.base.enabled and set the value to true. 
     * The duration policy will be stored in a field named .xset.retention.base.duration.policy. 
     * This field is of type 'application/vnd.snia.xam.string'. If the field already exists, its value 
     * will be changed to match the passed in policy if and only if the policy would not reduce the 
     * duration of the retention; the method will generate an error if the policy reduces the duration. 
     * If the field does not already exist, it will be created with the specified policy name as the value. 
     * These fields will have their binding attribute set in accordance with the binding flag set by the application.
     * 
     * @param binding TRUE if this value is to be BOUND.
     * @param policyName Name of the policy
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyBaseRetentionPolicy( boolean binding,
                                          String  policyName )
       throws AuthenticationExpiredException, PolicyNameException, 
              ObjectInUseException, XSetAbandonException, 
              XSetCorruptException, XAMException;    
    
    /**
     * Create or update the auto delete property, ".xset.deletion.autodelete", on the
     * specified XSet. This field will be used by the XAM Storage system to
     * determine if the XSet should automatically be deleted upon expiration of
     * retention.
     * 
     * @param binding
     *            TRUE if the property is BOUND.
     * @param autoDelete
     *            True if the system should auto-delete the object on
     *            expiration, false otherwise.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws MaximumFieldException Too many fields in the XSet.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void setAutoDelete(boolean binding, boolean autoDelete)
       throws AuthenticationExpiredException, MaximumFieldException, 
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * Create or update the auto delete policy, ".xset.deletetion.autodelete.policy", on
     * the specified XSet. <br>
     * NOTE: If the explicit autodelete field is present on the XSet, it will be
     * used by the XAM Storage System in preference to this field.
     * 
     * @param binding
     *            TRUE if the property is BOUND.
     * @param policyName
     *            Name of the policy.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyAutoDeletePolicy(boolean binding, String policyName)
       throws AuthenticationExpiredException, PolicyNameException,
              ObjectInUseException, XSetAbandonException, 
              XSetCorruptException, XAMException;
    
    /**
     * Evaluates all management settings for this XSet to return the effective
     * auto delete settings for this XSet.
     * 
     * @return Effective auto delete settings.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws PolicyMismatchException When an imported XSet's policy does not match the policy present in the XSystem.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public boolean getActualAutoDelete() 
       throws AuthenticationExpiredException, ObjectInUseException,
       PolicyMismatchException, XSetAbandonException, XSetCorruptException, XAMException;

    /**
     * If this XSet does not have the shred property set, this method will create a property 
     * field on the XSet with the name .xset.deletion.shred with type 'application/vnd.snia.xam.boolean'. 
     * Its value and binding attributes will be set according to the user-provided parameters. 
     * If the field already exists on the XSet, then its value will be updated with the specified value. 
     * This field will be used by the XAM Storage System to determine if the XSet should be shredded after deletion.
     * 
     * @param binding TRUE if the property is BOUND.
     * @param shred TRUE if the is to be shred upon deletion.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws MaximumFieldException Too many fields in the XSet.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void setShred( boolean binding, boolean shred )
       throws AuthenticationExpiredException, MaximumFieldException, 
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * If this XSet does not have an auto shred policy applied to it, this method will create a 
     * property field on the specified XSet with the name of .xset.deletion.shred.policy set to type 
     * 'application/vnd.snia.xam.string'. Its value and binding attributes will be set according 
     * to the user-provided parameters. If the field already exists on the XSet, then its value 
     * will be updated with the specified value. This field will be used by the XAM Storage
     * System to determine if the XSet should be shredded after XSet deletion. If the .xset.shred field is also
     * present on the XSet, it will be used by the XAM Storage System in preference to this field.
     * 
     * @param binding TRUE if the property is BOUND.
     * @param policyName The name of the policy to be applied for the shred setting.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws MaximumFieldException Too many fields in the XSet.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyShredPolicy( boolean binding, String policyName )
       throws AuthenticationExpiredException, MaximumFieldException,
              PolicyNameException, ObjectInUseException, 
              XSetAbandonException, XSetCorruptException, XAMException;
    
    /**
     * Evaluates all management settings for this XSet to return the effective auto shred settings for this XSet.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws PolicyMismatchException When an imported XSet's policy does not match the policy present in the XSystem.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public boolean getActualShred() 
       throws AuthenticationExpiredException, ObjectInUseException,
       PolicyMismatchException, XSetAbandonException, XSetCorruptException, XAMException;  
    
    /**
     * Create or set the storage policy, ".xam.storage.policy", on the specified
     * XSet.
     * 
     * @param binding
     *            TRUE if the property is BOUND.
     * @param policyName
     *            Name of the policy.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws PolicyNameException The policy name is null or invalid for this operation.
     * @throws MaximumFieldException Too many fields in the XSet.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void applyStoragePolicy(boolean binding, String policyName)
       throws AuthenticationExpiredException, MaximumFieldException, 
              PolicyNameException, ObjectInUseException, 
              XSetAbandonException, XSetCorruptException, XAMException;
    
    // == XSet Operations ==
    /**
     * This method will store an XSet in the XSystem. Note this does not close the 
     * XSet, which can still be modified as allowed by the authorization of the XSystem. 
     * A XUID will be assigned by the XAM Storage System and this XUID will be returned.
     * Open XStreams will not cause the commit to fail. Only the data that was 
     * successfully written to such XSteams will be committed. 
     * If this is a modified XSet (e.g., an existing XSet was opened and changed), 
     * then a new XUID may or may not be assigned by the commit, according to the 
     * following rules:
     * - If only nonbinding fields are edited (created, deleted, or changed), then the XAM Storage System may not assign a new XUID.
     * - If any binding fields are edited (created, deleted, or changed), then the XAM Storage System must assign a new XUID.
     * In any case, an application should be coded to handle cases where the XUID changes 
     * when a modified XSet is committed.
     * If a management policy has not been applied to the XSet before commit, a 
     * default management policy will be applied to the XSet at the time of commit.
     * 
     * @return The XUID of the stored XSet.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate
     * @throws AuthorizationException The application is not authorized to perform this operation.
     * @throws JobRunningException This XSet is running a job and the XSystem does not support saving running jobs.
     * @throws XSetUnderHoldException The XSet is being held and may not be changed.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public XUID commit() 
       throws AuthenticationExpiredException, AuthorizationException, 
              JobRunningException, XSetUnderHoldException, 
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    /**
     * Closes the XSet, making it unavailable for further use. Any resources
     * used by the open XSet will be released. After calling this method, the
     * XSet instance may not be reused.
     * 
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void close() 
       throws AuthenticationExpiredException, ObjectInUseException,
              XSetCorruptException, XAMException;
    
    
    /**
     * Allows an XSet to be closed after it has been corrupted. This method will
     * allow the XSet to be closed without regard to any open XStreams. After
     * calling this method, the only usable method is XSet.close(), all other
     * XSet operations will fail. unsaved XSet changes. <br>
     * <b>WARNING: This method should only be used upon a corrupt XSet. Calling
     * it at other inappropriate times can result in data loss. Applications are
     * encouraged to track open XSets and close them appropriately.</b>
     * 
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void abandon() throws AuthenticationExpiredException, XAMException;

    // == Job Methods ==
    /**
     * Submit a job request to the XAM Storage System. Fields in the XSet are
     * evaluated as input to the job in accordance with with the XAM Job Control
     * subsystem.
     * 
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws AuthorizationException The application is not authorized to perform this operation.
     * @throws JobCommandException One or more of the required job fields are not present in the XSet.
     * @throws JobUnsupportedException The specified job is not supported by the XSystem.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void submitJob() 
       throws AuthenticationExpiredException, AuthorizationException,
              JobCommandException, JobUnsupportedException, ObjectInUseException,
              XSetAbandonException, XSetCorruptException, XAMException;
              
    /**
     * If this XSet is a running job, halt the job, fill in the appropriate
     * fields for the completion of the job.
     * 
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws AuthorizationException The application is not authorized to perform this operation.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public void haltJob() 
       throws AuthorizationException, AuthenticationExpiredException, 
              ObjectInUseException, XSetAbandonException,
              XSetCorruptException, XAMException;
    
    // == Import/Export ==
    /**
     * Open a new XStream contains the normalized export format for the
     * specified XSet. The XSet must have been committed and the instance must
     * have been unchanged since opening. The application may read the XStream
     * using the normal XStream methods. The application must close this XStream
     * in order to be able to use the XSet for other XSet operations. <br>
     * NOTE: The XSet can not be modified while the export XStream is open.
     * 
     * @return The XStream containing the XSet export format.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public XStream openExportXStream() 
       throws AuthenticationExpiredException, ObjectInUseException,
       XSetAbandonException, XSetCorruptException, XAMException;
    
    /**
     * Opens an import stream which the application will use to write the
     * normalized import format for the XSet. The XSet must have been created
     * using either the RESTRICTED or UNRESTRICTED mode. When the import stream
     * has been completely written and closed, the XAM Storage System will
     * validate that the date from the import stream. If the data has been found
     * to be invalid, or improperly formatted, the XSet will enter the corrupt
     * state, and only the <i>abandon()</i> method will succeed. After a
     * successful import the XSet may be committed via the normal commit
     * semantics and the returned XUID is the value described in the normalized
     * import stream. Any modification to binding fields or binding attributes
     * in the XSet will result in a new XSet and new XUID value.
     * 
     * @return The XStream the application will use to write the cannonical
     *         export format data.
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             Any error encountered during import, including if the XSet
     *             has been opened in XSET_MODE_READ_ONLY.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public XStream openImportXStream() 
       throws AuthenticationExpiredException, ObjectInUseException,
              XSetAbandonException, XSetCorruptException, XAMException; 
    
    /**
     * This method is an asynchronous version of XSet.commit. 
     * @param xopid The application supplied XOPID of this XAsync operation.
     * @param listener listener An optional parameter, may be null, of the XASyncListener object to be notified when the operation has completed.
     * @return XAsync object representing this operation
     * @throws AuthenticationExpiredException The latest authentication has expired and the application must reauthenticate.
     * @throws InvalidArgumentException Xopid id may already be in use.
     * @throws ObjectInUseException The XSet has open import/export streams.
     * @throws XSetAbandonException The XSet is in the abandoned state and may only be closed.
     * @throws XSetCorruptException The XSet is in the corrupt state and may only be abandoned.
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe, the XAsyncListener must be coded in a thread safe manner
     *             <DT><b>Blocking:</b>
     *             <DD>This method will return immediately
     */
    public XAsync asyncCommit( long           xopid,
                               XAsyncListener listener )
       throws AuthenticationExpiredException, InvalidArgumentException, 
              ObjectInUseException, XSetAbandonException, 
              XSetCorruptException, XAMException;
  }
