/******************************************************************************
 *
 * Copyright (c) 2007-2008, The Storage Networking Industry Association
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *    - Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    - Neither the name of The Storage Networking Industry Association (SNIA)
 *      nor the names of its contributors may be used to endorse or promote 
 *      products derived from this software without specific prior written 
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

package org.snia.xam;

/**
 * This represents a XAM XUID.
 */
public interface XUID
{
   /** A XUID may be at most 80 bytes long. */
   final public int MAX_LENGTH = 80;
   
   /** A XUID must be at least 9 bytes long. */
   final public int MIN_LENGTH = 9;
   
    /**
     * Get the raw array of bytes representing this XUID
     * 
     * @return The byte array.
     *         <DT><b>Thread Safety:</b>
     *         <DD>This method is thread safe
     *         <DT><b>Blocking:</b>
     *         <DD>This method blocks until completion
     */
    public byte[] toBytes();

    /**
     * Convert this XUID to a base64 encoded String
     * 
     * @return The base 64 version of this XUID
     *         <DT><b>Thread Safety:</b>
     *         <DD>This method is thread safe
     *         <DT><b>Blocking:</b>
     *         <DD>This method blocks until completion
     */
    public String toString();

    /**
     * Compare two XUID values.
     * 
     * @return True if the two values are the same
     *         <DT><b>Thread Safety:</b>
     *         <DD>This method is thread safe
     *         <DT><b>Blocking:</b>
     *         <DD>This method blocks until completion
     */
    public boolean equals(Object x);
}
