////////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2007-2008, SNIA.
//
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//    - Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright 
//      notice, this list of conditions and the following disclaimer in the 
//      documentation and/or other materials provided with the distribution.
//    - The name of SNIA may not be used to endorse or promote 
//      products derived from this software without specific prior written 
//      permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
//  THE POSSIBILITY OF SUCH DAMAGE.
////////////////////////////////////////////////////////////////////////////////
// XAM SDK Source File Build Version @version.full@
////////////////////////////////////////////////////////////////////////////////
package org.snia.xam.vim;

import org.snia.xam.XAMException;
import org.snia.xam.XSystem;

/**
 * This interface provides a series of methods allowing the XAM library to
 * communicate to VIMs. These methods aren't available to application programs
 * as VIM objects are not available.
 * 
 * A VIM object implements the vendor specific logic to connect to a particular
 * XAM Storage System. The connect factgory method is used by the XAM library to
 * cause the VIM to perform vendor specific connection logic, and provide the
 * XSystem. The provided XSystem instance is implemented in the vendor's package
 * space but implements org.snia.xam.XSystem interface.
 */
public interface VIM
{
    /**
     * Create an XSystem which is initialized by the XAM Library, then connected
     * and returned to the application. This method is called in response to a 
     * XAMLibrary <i>connect</i> method.
     * 
     * @throws XAMException An error exists with the underlying XAM Storage System or VIM.
     *             <DT><b>Thread Safety:</b>
     *             <DD>This method is thread safe
     *             <DT><b>Blocking:</b>
     *             <DD>This method blocks until completion
     */
    public XSystem createXSystem() throws XAMException;
}
