/*
 * Copyright (c) 2009, Sun Microsystems, Inc.
 * Copyright (c) 2009, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */
//package examples;

package examples;

import java.io.FileOutputStream;
import org.snia.xam.XSet;
import org.snia.xam.XStream;
import org.snia.xam.XSystem;
import org.snia.xam.XUID;
import org.snia.xam.base.Base64;

/**
 * Simple client to connect to an XSystem, and export an XSet.
 * Note that for this example the Reference VIM is started and loaded into the
 * current process' JVM. In a production VIM implementation the VIM is likely
 * to be running in a separate process as a server. 
 * So in this example application calls are made to the Java XAM Library to 
 * load and register the Reference VIM.
 */
public class ExportXSetClient 
   extends ExampleBase
{
    /**
     * 
     * @param args connection parameters for VIM
     */
    public static void main(String[] args) throws Exception {
        try {
            System.out.print("XAM XSet Export Program - ");
            System.out.println("Uses Reference VIM by default");
            // Have to init and load VIM since it's being run in this JVM 
            initLibrary();
            System.out.println("\n=============================================");
            System.out.println("Client application example program testing:");
            
            // Now do the client connect to the VIM
            System.out.println("Client application connecting to the VIM");            
            XSystem xsystem = connectToVIM(s_xri);
            
            // Authenticate our user credentials
            System.out.println("\nClient application authenticating user credentials");
            authenticate(xsystem);
            
            // Open the Export stream, and write it out.
            XUID xuid = xsystem.createXUID(args[0]);
            String filename = Base64.safe64Encode(xuid.toBytes());
            
            XSet x = xsystem.openXSet(xsystem.createXUID(args[0]), XSet.MODE_READ_ONLY);
            XStream s = x.openExportXStream();
            long bytesRead = 0;
            byte buffer[] = new byte[4096];
            FileOutputStream out = new FileOutputStream(filename+".dat");
            while( (bytesRead = s.read(buffer )) > 0)
            {
               out.write( buffer, 0, (int) bytesRead );
            }
            out.close();
            
           System.out.println("=============================================\n");
        } catch (Exception ex) {
            ex.printStackTrace();
        }    
    }
   }
