/*
 * Copyright (c) 2008-2009, Sun Microsystems, Inc.
 * Copyright (c) 2008-2009, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */
package examples;

import java.util.Calendar;
import org.snia.xam.XSet;
import org.snia.xam.XSystem;
import org.snia.xam.XUID;


/**
 * Simple client example program that loads and connects to the Reference VIM.
 * Note that for this example the Reference VIM is started and loaded into the
 * current process' JVM. In a production VIM implementation the VIM is likely
 * to be running in a separate process as a server. 
 * So in this example application calls are made to the Java XAM Library to 
 * load and register the Reference VIM.
 */
public class ReferenceXSetClient 
   extends ExampleBase {
    /**
     * 
     * @param args connection parameters for VIM
     */
    public static void main(String[] args) {
        try {
            System.out.print("XAM XSet Client Example Program - ");
            System.out.println("Uses Reference VIM by default");
            // Have to init and load VIM since it's being run in this JVM 
            initLibrary();
            System.out.println("\n=============================================");
            System.out.println("Client application example program testing:");
            
            // Now do the client connect to the VIM
            System.out.println("Client application connecting to the VIM");            
            XSystem xsystem = connectToVIM(s_xri);
            
            // Authenticate our user credentials
            System.out.println("\nClient application authenticating user credentials");
            authenticate(xsystem);
            
            // Do actual work of creating XSet
            System.out.println("\nClient application creating & persisting XSet with properties.");
            testProperties(xsystem);
            System.out.println("=============================================\n");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
    }
   
    /**
     * Test creating XSet with some properties and then commiting the XSet.
     * @param system the XSystem instance 
     * @throws java.lang.Exception
     */
    private static void testProperties(XSystem system) throws Exception {
        String booleanPropName = "test.boolean";
        String intPropName = "test.int";
        String doublePropName = "test.double";
        String stringPropName = "test.string";
        // Create an empty XSet
        XSet xset = system.createXSet(XSet.MODE_UNRESTRICTED);
    	// commit it
    	XUID xuid  = xset.commit();
        System.out.println("XSet created - XUID: " + xuid.toString());
        // Create some properties
    	xset.createProperty(booleanPropName, false, true);
    	xset.createProperty(intPropName, false, 12345L);
    	xset.createProperty(doublePropName, false, 1234.5);
    	Calendar cal = Calendar.getInstance();
    	xset.createProperty("test.date", false, cal);
    	String str = "testing string....";
    	xset.createProperty(stringPropName, true, str);
    	xset.createProperty("test.xuid", false, xuid);
        // commit should get new XUID cause changed/added binding properties
        xuid = xset.commit();
        System.out.println("XSet updated (binding change) - XUID: " 
                + xuid.toString());
        System.out.println("Close & reopen XSet and test getting some property values");
    	xset.close();
    	
        // Now test getting the properties
        XSet xset2 = system.openXSet(xuid, XSet.MODE_READ_ONLY);
        System.out.println(booleanPropName + "=" + 
                xset2.getBoolean(booleanPropName));
        System.out.println(doublePropName + "=" + 
                xset2.getDouble(doublePropName));
        System.out.println(stringPropName + "=" + 
                xset2.getString(stringPropName));
        System.out.println("Closing XSet and XSystem");
        xset2.close();
    	system.close();
    }
    
}
