/*
 *  Copyright (c) 2009, Sun Microsystems, Inc.
 *  Copyright (c) 2009, The Storage Networking Industry Association.
 * 
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *  *  Neither the name of The Storage Networking Industry Association (SNIA) nor 
 *     the names of its contributors may be used to endorse or promote products 
 *     derived from this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.snia.xam.vim.reference;

/**
 * Access Policy used to represent access to XSets. Each policy represents two activities:
 *    1) Read
 *    2) Write (including delete)
 * An access policy will also specify a user, with a special user known as "*", or ALL.
 *
 */
public class AccessPolicy
{
   public static final String ALL = "*";
   
   private String  m_user;
   private boolean m_read;
   private boolean m_write;

   public AccessPolicy( String user, boolean read, boolean write )
   {
      m_user  = user;
      m_read  = read;
      m_write = write;
   }
   
   /** 
    * Check a potential reader against the access policy 
    * @param readingUser The user attempting to read the XSet.
    */
   public boolean canRead( String readingUser )
   {
      if (m_read && m_user.equals(readingUser))
         return true;
      else if( m_read && m_user.equals(ALL))
         return true;
      else
         return false;
   }

   /** 
    * Check a potential reader against the access policy 
    * @param writingUser The user attempting to write the XSet.
    */
   public boolean canWrite( String writingUser )
   {
      if( m_write && m_user.equals(writingUser) )
         return true;
      else if( m_write && m_user.equals(ALL) )
         return true;
      else
         return false;
   }
}
