/*
 * Copyright (c) 2008, Sun Microsystems, Inc.
 * Copyright (c) 2008, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference;

/**
 * XSession/XSystem Authentication States
 * 
 */
public enum AuthenticationState{
    /**
     * Session is connected but not authenticated. Either an authentication has
     * has not been attempted, the authentication has expired or the
     * application's attempt to authenticate has failed with a non-fatal
     * error. (For example the credentials provided were rejected.) 
     * The application does not have access to VIM except to perform 
     * authentication or to abandon or close the current session.
     */
    UNAUTHENTICATED, 
    
    /**
     * Session has valid authentication. Access is granted to VIM based on
     * authorization level of authentication credentials used.
     */
    AUTHENTICATED, 
    
    /**
     * Session is in the process of reauthenticating (proactive 
     * reauthentication). The application has initiated a re-authentication 
     * before the current authentication has expired. While in the process 
     * of re-authenticating this session is still considered to have valid 
     * authentication credentials and is granted the same access as when 
     * in the AUTHENTICATED state.
     */
    REAUTHENTICATING, 
    
    /**
     * The session enteres the FAILED state if the XSystem.authenticate method 
     * or any XAM API method returns a fatal error. Can not do anything with 
     * current session except call XSystem.abandon.
     */
    FAILED, 
    
    /**
     * Session has been abandoned. Application can not do anything with 
     * current session except call close.
     */
    ABANDONED, 
    
    /**
     * Session has been closed and resources have been released. The
     * application can not access the session. Must create new session with 
     * valid authentication to access VIM.
     * (Strictly speaking this is not an authentication state as per the 
     * XAM spec but some state was needed to identify closed sessions and 
     * disallow access.)
     */
    CLOSED 
}
