/*
 * Copyright (c) 2008, Sun Microsystems, Inc.
 * Copyright (c) 2008, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference;

import org.snia.xam.XAMException;

/**
 * The Job interface defines methods that enables the process to do work.
 */
public interface Job extends Runnable {
    
    /**
     * This method performs the work
     */
    void execute() throws XAMException;

    /**
     * Make a cancelation request to the Job.
     * @throws XAMException When an error is detected canceling this service.
     */
    void cancel() throws XAMException;
   
    /**
     * Get the job identifier.    
     * @return Returns the job ID.
     */
    String getID();

    /**
     * Close the resources used by this Job.
     * This method is invoked after the last dependent Job has been
     * invoked.
     * @throws XAMException When an error is detected releasing any resources.
     */
    void close() throws XAMException;  
    
    /**
     * Determines if this process has been requested to be canceled --
     * This method is called and the value checked when the process may be
     * able to stop execution.
     * @return Returns true if the Job should stop running, otherwise false
     */
    boolean isCancelRequested();
    
    /**
     * Determines if this process has finished processing/preforming its work.
     * @return Returns true if the Job has completed its work.
     */
    boolean isExecuteDone();
    
    /**
     * Gets the job type.
     * @return Returns the string value of the org.snia.xam.job.command field 
     * that was specified when submitting the job
     */
    String getJobType();
    
    /**
     * Gets the current job status object
     * @return the current job status
     */
    JobStatus getJobStatus();

}
