/*
 *  Copyright (c) 2008-2009, The Storage Networking Industry Association.
 *  Copyright (c) 2008-2009, Sun Microsystems, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *  *  Neither the name of The Storage Networking Industry Association (SNIA) nor 
 *     the names of its contributors may be used to endorse or promote products 
 *     derived from this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.snia.xam.vim.reference;

import java.lang.reflect.Proxy;
import org.snia.xam.XAMException;
import org.snia.xam.XSystem;
import org.snia.xam.vim.VIM;

/**
 * Implementation of the VIM methods allowing the XAM library to
 * communicate to VIMs. These methods should NOT be called by application 
 * programs as results are unpredictable.
 *
 * A VIM object implements the vendor specific logic to connect to a particular
 * XAM Storage System. The connect factory method is used by the XAM library to
 * cause the VIM to perform vendor specific connection logic, and provide the
 * XSystem. The provided XSystem instance is implemented in the vendor's package
 * space but implements org.snia.xam.XSystem interface.
 */
public class ReferenceVIM
        implements VIM {
    /**
     * Called in response to XAMLibrary connect method.
     * @see VIM#createXSystem() 
     */
    public XSystem createXSystem()
            throws XAMException {
        ReferenceXSystem xsystem = new ReferenceXSystem();
        return (XSystem) Proxy.newProxyInstance(
                xsystem.getClass().getClassLoader(),
                xsystem.getClass().getInterfaces(),
                new XSystemClientInvocationHandler(xsystem));
    }
}
