/*
 * Copyright (c) 2009, The Storage Networking Industry Association.
 * Copyright (c) 2009, Sun Microsystems, Inc.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.snia.xam.vim.reference;

import java.util.HashMap;

import org.snia.xam.XAMException;
import org.snia.xam.base.Property;

/**
 * A RetentionPolicyReference list a policy, via the XSystem field which refers to it. 
 * The reference also yields the list of the referring properties. This list can be 
 * obtained by calling the <i>elements()</i> method.
 */
public class RetentionPolicyReference
   extends Property
{
   private HashMap<String,Property> m_references = new HashMap<String,Property>();
   
   /**
    * Constructs a new policy reference.
    * 
    * @param name The policy field's name
    * @param type The policy field's mime-type
    * @param value The policy field's value, which is the name of the policy
    * @param binding The policy field's binding state
    * @param readonly The policy field's read state
    * @throws XAMException If the mime-type is invalid or the value does not
    *            match the mime-type
    */
   public RetentionPolicyReference( String  name, 
                                    String  type, 
                                    Object  value, 
                                    boolean binding,
                                    boolean readonly)
   throws XAMException
   {
      super( name, type, value );
      setBinding(binding);
      setReadonly(readonly);
   }
   
   /**
    * Add a reference to the property.
    * @param prop
    */
   public void addReference( Property prop )
   {
      m_references.put(prop.getName(), prop);
   }
   
   public HashMap<String,Property> getReferences()
   {
      return m_references;
   }
}
