/*
 * Copyright (c) 2008, Sun Microsystems, Inc.
 * Copyright (c) 2008, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference;

import java.io.File;
import org.snia.xam.XUID;

/**
 *
 * Data structure containing XOP data for imported XStream and utility 
 * methods.
 * 
 * This structure associates both the TOC type information about an XStream
 * and the information from the <XStream> tag in the XSet XML.
 * 
 */
public class XStreamXopData {
    // TOC Information
    public XUID xuid;
    public String xuidString;
    public String fieldName;
    public String cid;
    public long offset = -1;

    // Data Content Information
    public String primaryType = "";
    public String subType = "";
    public String encodingType = "";
    // Tmp Location of data on disk
    public String tmpFilename = "";
    
    
    /**
     * Locates the cid or field name of the XStream field in the list and
     * returns the index.
     * 
     * @param cidOrFieldName
     * @param streams
     * @return index of field in the array; -1 if not found
     */
    protected static int findFieldEntry(String cidOrFieldName, 
            XStreamXopData[] streams) {
        int index = -1;
        if (streams != null) {
            int pos = cidOrFieldName.indexOf("<");
            if (pos != -1) {
                cidOrFieldName = cidOrFieldName.substring(pos+1);
            }
            pos = cidOrFieldName.indexOf(">");
            if (pos != -1) {
                cidOrFieldName = cidOrFieldName.substring(0, pos);
            }
            
            for (int i = 0; i < streams.length; i++) {
                if (streams[i].cid.equals(cidOrFieldName) ||
                        streams[i].fieldName.equals(cidOrFieldName)) {
                    index = i;
                    break;
                }
            }
        }
        return index;
        
    }

    protected static boolean cleanup(XStreamXopData[] streams) {
        boolean success = true;
        if (streams != null) {
            for (int i = 0; i < streams.length; i++) {
                if (streams[i] != null && streams[i].tmpFilename != null) {
                    File file = new File(streams[i].tmpFilename);
                    if (!ReferenceImportXStream.debug)
                        file.delete();
                }
            }
        }
        return success;
    }
}
