/*
 * Copyright (c) 2008, Sun Microsystems, Inc.
 * Copyright (c) 2008, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import org.snia.xam.XAMException;
import org.snia.xam.XSystem;
import org.snia.xam.XSystemException;
//import org.snia.xam.vim.reference.ReferenceXSystem;

/**
 *
 */
public class XSystemClientInvocationHandler implements InvocationHandler {
    private XSystem xsystem = null;
    private ReferenceXSystem refXSystem = null;
    private Method[] methodList = XSystem.class.getDeclaredMethods();
    
    XSystemClientInvocationHandler(XSystem xsys) throws XSystemException {
        if (xsys instanceof ReferenceXSystem) {
            refXSystem = (ReferenceXSystem) xsys;
        } else {
            // change to runtime exception?
           String msg = "XSystem class type not supported: " + xsys.getClass().getName();
           throw new XSystemException(msg);
        }
        xsystem = xsys;
    }
    
    /**
     * 
     * @param proxy
     * @param method
     * @param args
     * @return the actual object used to invoke the method
     * @throws java.lang.Throwable
     */
    public Object invoke(Object proxy, Method method, Object[] args) 
            throws Throwable {
            String methodName = method.getName();
            Object returnObj = null;
            try {
                /*
                 * All these methods can call without being authenticated
                 */
                if (methodName.equals("abandon") ||
                        methodName.equals("close") ||
                        methodName.equals("authenticate") ||
                        methodName.equals("connect")) {
                    return method.invoke(xsystem, args);
                }

                // Otherwise check access is allowed
                boolean checkAccess = false;

                for (int i = 0; i < methodList.length; i++) {
                    if (methodList[i].getName().equals(methodName)) {
                        checkAccess = true;
                        break;
                    }
                }

                if (!checkAccess) {
                    // These are the field container methods - read allowed
                    if (!refXSystem.isInitializing()) {
                        if ((methodName.startsWith("get") ||
                                methodName.equals("openFieldIterator") ||
                                methodName.equals("containsField"))) {
                            refXSystem.checkFieldReadAccessAllowed();
                        } else {
                            refXSystem.checkAccessAllowed();
                        }
                    }
                } else {
                    refXSystem.checkAccessAllowed();
                }
                // finally call the method
                returnObj = method.invoke(xsystem, args);
            } catch (InvocationTargetException ex) {
                Throwable th = ex.getCause();
                if (th != null && th instanceof XAMException) {
                    throw th;
                } else {
                    throw ex;
                }
            }
            return returnObj;
    }

}
