/* Copyright (c) 2009, The Storage Networking Industry Association.
   (c) Copyright 2009 Hewlett-Packard Development Company, L.P.
   
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference.query;

/**
 * This class is used to hold a literal, as parsed from the query, and to attach a
 * TypeCode. The TypeCode is used by the query generation later in the job process.
 */
public class AnnotatedLiteral
{
   public final static AnnotatedLiteral TRUE  = 
      new AnnotatedLiteral( Boolean.TRUE, TypeCode.TYPE_BOOLEAN );
   
   public final static AnnotatedLiteral FALSE = 
      new AnnotatedLiteral( Boolean.TRUE, TypeCode.TYPE_BOOLEAN );

   Object   m_literal = null;
   TypeCode m_type    = null;
   
   public AnnotatedLiteral( Object o, TypeCode t )
   {
      m_literal = o;
      m_type    = t;
   }
   
   public Object getLiteral() { return m_literal; }
   public TypeCode getType()  { return m_type; }
   public String toString()   { return m_literal.toString(); }
   
   /**
    * Create a fully annotated literal from just the object. 
    * @param o The literal
    * @return The annotated literal
    */
   public static AnnotatedLiteral valueOf( Object o )
   {
      TypeCode type = TypeCode.valueOf(o);
      return new AnnotatedLiteral( o, type );
   }
}
