/* Copyright (c) 2009, The Storage Networking Industry Association.
   (c) Copyright 2009 Hewlett-Packard Development Company, L.P.
   
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference.query;

public class ParsedTermNode
   implements TermNode
{
   protected WhereNode m_left;

   protected boolean m_not;

   protected int m_op;

   protected WhereNode m_right;

   protected boolean m_isParenthesis;

   public ParsedTermNode()
   {
      super();
   }

   public int getOperator()
   {
      return m_op;
   }

   public boolean isNotExpression()
   {
      return m_not;
   }

   public WhereNode getLeftNode()
   {
      return m_left;
   }

   public WhereNode getRightNode()
   {
      return m_right;
   }

   public boolean isTerm()
   {
      return true;
   }

   public boolean isFactor()
   {
      return false;
   }

   public boolean isParenthesis()
   {
      return m_isParenthesis;
   }

   public void setNot(boolean not)
   {
      m_not = not;
   }

   public void setLeftNode(WhereNode node)
   {
      m_left = node;
   }

   public void setRightNode(WhereNode node)
   {
      m_right = node;
   }

   public void setOperator(int op)
   {
      m_op = op;
   }

   public void setParenthesis(boolean paren)
   {
      m_isParenthesis = paren;
   }

   String operatorToString()
   {
      switch (m_op)
      {
         case OPERATOR_NONE :
            return "";
         case OPERATOR_AND :
            return " AND ";
         case OPERATOR_OR :
            return " OR ";
         default :
            return "?";
      }
   }

   public String toString()
   {
      StringBuffer sb = new StringBuffer("T(");
      sb.append(m_left == null ? "NULL" : m_left.toString());
      sb.append(operatorToString());
      sb.append(m_right == null ? "NULL" : m_right.toString());
      sb.append(")");
      return sb.toString();
   }

   /**
    * Called from printDebug, with an indent level
    * I don't like the instanceof crap, but didn't feel like garbaging up 
    * the interface with some methods only for debuggging. This is pretty 
    * ugly but it will work. It can be changed if the presence of instanceof
    * will be chapping someone's hide.
    */
   void printDebug(int indent)
   {
      final String pad = "                                             ";
      System.out.print(pad.substring(0, indent));
      System.out.print("T(");
      if( isNotExpression() )
      {
         System.out.print( "not ");
      }
      if (m_left instanceof ParsedTermNode) ((ParsedTermNode) m_left).printDebug(indent + 2);
      else
         System.out.print(m_left.toString());
      
      if( isNotExpression() )
      {
         System.out.print( ")" );
      }
      else
      {
         System.out.println(operatorToString());

         if (m_right instanceof ParsedTermNode) ((ParsedTermNode) m_right).printDebug(indent + 2);
         else
         {
            System.out.print(pad.substring(0, indent));
            System.out.print(m_right.toString());
         }
         System.out.print(")");
      }
   }
   
   /**
    * Pretty print an expression
    */
   public void printDebug()
   {
      printDebug(0);
      System.out.println("");
   }
}
