/* Copyright (c) 2009, The Storage Networking Industry Association.
   (c) Copyright 2009 Hewlett-Packard Development Company, L.P.
   
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference.query;

import java.lang.RuntimeException;

/**
 * Class containing utility methods for handling query XSets.
 */
public class QueryUtils
{
   public static final String PROPERTY_XUID = ".xset.xuid";

   /** XamQL Escape Character \"\\\" */
   public static final char ESCAPE = '\\';

   /**
    * Replaces escape sequences in XamQL field names and string literals.
    * 
    * @param value XamQL field name or string literal
    * @return String with escape sequences replaced
    * @throws ParseException If an invalid sequence is found
    */
   public static String escapeXamQl(String value)
   {
      int index = value.indexOf(ESCAPE);
      if (index < 0) return value;

      int prev = 0;
      int length = value.length();
      StringBuffer buf = new StringBuffer(length);
      while (index > -1)
      {
         buf.append(value.subSequence(prev, index));
         try
         {
            char ch = value.charAt(index + 1);
            switch (ch)
            {
               case '\\' :
               case '"' :
               case '\'' :
                  buf.append(ch);
                  prev = index + 2;
                  break;
               case 'u' :
                  String code = value.substring(index + 2, index + 6);
                  try
                  {
                     buf.append((char) Integer.parseInt(code, 16));
                  }
                  catch (Exception xx)
                  {
                     throw new Exception("Invalid unicode character \""
                        + value.substring(index) + "\"");
                  }
                  prev = index + 6;
                  break;
               default :
                  throw new Exception("Invalid escape character \""
                     + value.substring(index) + "\"");
            }
         }
         catch (Exception x)
         {
            throw new RuntimeException(x);
         }
         index = value.indexOf(ESCAPE, prev);
      }
      buf.append(value.subSequence(prev, length));
      return buf.toString();
   }
}