/*
 *  Copyright (c) 2008, The Storage Networking Industry Association.
 * 
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *  *  Neither the name of The Storage Networking Industry Association (SNIA) nor 
 *     the names of its contributors may be used to endorse or promote products 
 *     derived from this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference.utils;

import org.snia.xam.FieldContainer;
import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.XIterator;

/**
 *
 */
public class Output {
/**
     * Debug helper method for outputing all fields for the specified container
     * that have the specified prefix
     * @param container the container (XAM, XSystem, XSet) to output all the 
     * associated fields for
     * @param prefix the prefix to limit the field output to.  May be null or
     * empty string.
     */
    public static void outputFields(FieldContainer container, String prefix) {
        try {
            XIterator iterator = container.openFieldIterator(prefix);
            while (iterator.hasNext()) {
                String fieldName = (String)iterator.next();
                System.out.print(fieldName + " = ");
                try {
                    String type = container.getFieldType(fieldName);
                    if (XAMLibrary.STYPE_BOOLEAN_MIME_TYPE.equals(type))
                        System.out.println(container.getBoolean(fieldName));
                    else if (XAMLibrary.STYPE_XUID_MIME_TYPE.equals(type)) 
                        System.out.println(container.getXUID(fieldName));
                    else if (XAMLibrary.STYPE_DOUBLE_MIME_TYPE.equals(type))
                        System.out.println(container.getDouble(fieldName));
                    else if (XAMLibrary.STYPE_INT_MIME_TYPE.equals(type))
                        System.out.println(container.getLong(fieldName));
                    else if (XAMLibrary.STYPE_STRING_MIME_TYPE.equals(type))
                        System.out.println(container.getString(fieldName));
                    else if (XAMLibrary.STYPE_DATETIME_MIME_TYPE.equals(type))
                        System.out.println(container.getDateTime(fieldName).getTime().toString());
                    else if (XAMLibrary.DEFAULT_MIME_TYPE.equals(type))
                        System.out.println("NA");
                }
                catch (XAMException xe) {
                    System.out.println(xe.getMessage());
                }
            }
        } catch (XAMException ex) {
            System.out.println("Output of container fields names and there values failed.  Reason:");
            System.out.println(ex.getMessage());
        }
    }
    
}
