/*
 * Copyright (c) 2008, Sun Microsystems, Inc.
 * Copyright (c) 2008, The Storage Networking Industry Association.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *  
 * Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 *  
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *  
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 * the names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.snia.xam.vim.reference;

import java.util.Calendar;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.snia.xam.FieldContainer;
import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.XIterator;
import org.snia.xam.XSet;
import org.snia.xam.XSystem;
import org.snia.xam.XUID;
import org.snia.xam.util.ISO8601Date;

/**
 *  Test suite for Exporting/Importing XSet
 *
 */
public class ReferenceIteratorTest extends ReferenceXAMTest {
    
    public ReferenceIteratorTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        System.exit(runTest(args, suite()));
    }
    
    public static Test suite() {
    	TestSuite suite = new TestSuite();
    	suite.addTest(new ReferenceIteratorTest("testSimpleIterator"));
        return suite;
    }
    
    public void testSimpleIterator() throws Exception {
        final String prefix = "test.simple.iterator.";
        int expectedUserFields = 5;
        int expectedSystemFields = 8; // this may change
        announce("ReferenceIteratorTest.testSimpleIterator");

        XSystem system = connectAndAuthenticate(true);
        XSet xset = system.createXSet(XSet.MODE_UNRESTRICTED);
        xset.createProperty("testBoolean", false, true);
        xset.createProperty("testInteger", true, 125);
        xset.createProperty("testDouble", true, (double) 1.4);
        String sValue = "StringValue";
        xset.createProperty("testString", false, sValue);
        Calendar calendar = Calendar.getInstance();
        xset.createProperty("testDate", true, calendar);
        
        // commit 
        XUID xuid1 = xset.commit();
        int userDefinedFields = 0;
        int systemFields = 0;
        // Create field iterator and get all the fields
        XIterator iterator = xset.openFieldIterator("");
        while (iterator.hasNext()) {
            String fieldName = (String) iterator.next();
            if (!fieldName.startsWith(".")) {
                userDefinedFields++;
            } else {
                systemFields++;
            }
            
            try {
                String type = xset.getFieldType(fieldName);
                
                //System.out.print("Field name: " + fieldName);
                //System.out.print("   type: " + type);
                //System.out.print("   value: ");
                if (type.equals(XAMLibrary.STYPE_BOOLEAN_MIME_TYPE)) {
                    boolean b = xset.getBoolean(fieldName);
                } else if (type.equals(XAMLibrary.STYPE_DATETIME_MIME_TYPE)) {
                    Calendar date = xset.getDateTime(fieldName);
                } else if (type.equals(XAMLibrary.STYPE_DOUBLE_MIME_TYPE)) {
                    double d = xset.getDouble(fieldName);
                } else if (type.equals(XAMLibrary.STYPE_INT_MIME_TYPE)) {
                    long l = xset.getLong(fieldName);
                } else if (type.equals(XAMLibrary.STYPE_STRING_MIME_TYPE)) {
                    String text = xset.getString(fieldName);
                } else if (type.equals(XAMLibrary.STYPE_XUID_MIME_TYPE)) {
                    XUID xuidTmp = xset.getXUID(fieldName);
                } else {
                    // Skip stream fields and other system fields for now
                }
            } catch (XAMException ex) {
                ex.printStackTrace();
                fail("Error when attempting to retrieve fields");
            }
        }

        
        xset.close();
        system.close();
        assertTrue("Incorrect number of user-defined fields, expected: " +
                expectedUserFields + "   Found: " + userDefinedFields,
                (userDefinedFields == expectedUserFields));
        assertTrue("Missing system fields, Found: " + systemFields,
                (systemFields >= expectedSystemFields));
    }

}
