/* Copyright (c) 2009, The Storage Networking Industry Association.
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */
/* (c) Copyright 2009 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.vim.reference.query;

import junit.framework.Test;
import junit.framework.TestSuite;

import java.io.StringBufferInputStream;
import java.util.List;
import java.util.Properties;

import org.snia.xam.vim.reference.ReferenceXAMTest;

public class QueryFactoryTest
   extends ReferenceXAMTest
{
   private final static String EASY_QUERY = "select \".xset.xuid\"";

   private static final String COM_EXAMPLE_PROP = "com.example.prop_a";

   private final static String SIMPLE_WHERE_QUERY = "select \".xset.xuid\" where \"" + 
        COM_EXAMPLE_PROP + "\" = 42";

   public QueryFactoryTest(String test)
   {
      super(test);
   }

   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new QueryFactoryTest("testSimplestQueryInTheWorld"));
      suite.addTest(new QueryFactoryTest("testSimpleWhere"));
      return suite;
   }

   /**
    * Test the world's easiest query. Make sure the select list is correct. Make
    * sure the where node is empty.
    */
   public void testSimplestQueryInTheWorld()
      throws Exception
   {
      announce( "testSimplestQueryInTheWorld" );
      XamQlParser xql = new XamQlParser(new StringBufferInputStream(EASY_QUERY));

      ParsedQuery q = xql.Start();

      List pl = q.getSelectedPropertyList();

      FactorNode wn = (FactorNode) q.getWhereExpression();
      assertNull("Should be no where node", wn);
   }

   /**
    * Test a query with a simple Where clause
    */
   public void testSimpleWhere()
      throws Exception
   {
      announce( "testSimpleWhere" );
      XamQlParser xql = new XamQlParser(new StringBufferInputStream(
         SIMPLE_WHERE_QUERY));

      ParsedQuery q = xql.Start();

      List pl = q.getSelectedPropertyList();

      FactorNode wn = (FactorNode) q.getWhereExpression();

      assertNotNull("Should be a where node", wn);

      String prop = wn.getProperty();
      assertEquals("Incorrect property name", "com.example.prop_a", prop);

      assertTrue("Where node should be a factor", wn.isFactor());
      FactorNode fn = (FactorNode) wn;

      assertEquals("Incorrect operation", FactorNode.EQUALS, fn.getOperator());

      assertTrue("Incorrect type of literal expression",
         fn.getExpression().getLiteral() instanceof Long);
      
      Properties whereProps = q.getWherePropertyList();
      assertEquals( "Wrong number of properties detected in where clause",
         1, whereProps.size() );
      assertTrue( "Expected to see " + COM_EXAMPLE_PROP + " in where clause",
         whereProps.containsKey( COM_EXAMPLE_PROP + TypeCode.TYPE_INTEGER ));
   }
}
