/* Copyright (c) 2008, The Storage Networking Industry Association.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.base;

/**
 * Default implementation of the {@link Field} interface. This will be used a
 * base class for both {@link Property} and {@link StreamField} classes.
 */
public class DefaultField
   implements Field
{
   private boolean m_binding;

   private long m_length;

   private boolean m_modified;

   private String m_name;

   private boolean m_readonly;

   private String m_type;

   /**
    * Constructor that takes just the field name accessible only to sub-classes.
    * 
    * @param name The field name
    */
   protected DefaultField(String name)
   {
      m_name = name;
   }

   /**
    * Constructor that takes the field name, mime-type, length in bytes, binding
    * state and read state. This is accessible only to sub-classes
    * 
    * @param name The field name
    * @param type The field mime-type
    * @param length The field length in bytes
    * @param binding The field binding state
    * @param readonly The field read state
    */
   protected DefaultField(String name, String type, long length,
      boolean binding, boolean readonly)
   {
      m_name = name;
      m_type = type;
      m_length = length;
      m_binding = binding;
      m_readonly = readonly;
   }

   public boolean getBinding()
   {
      return m_binding;
   }

   public long getLength()
   {
      return m_length;
   }

   public String getName()
   {
      return m_name;
   }

   public boolean getReadOnly()
   {
      return m_readonly;
   }

   public String getType()
   {
      return m_type;
   }

   public boolean isModified()
   {
      return m_modified;
   }

   /**
    * Sets the field binding state.
    * 
    * @param binding The field binding state
    */
   public void setBinding(boolean binding)
   {
      m_binding = binding;
   }

   /**
    * Sets the field length in bytes.
    * 
    * @param length The field length in bytes.
    */
   public void setLength(long length)
   {
      m_length = length;
   }

   /**
    * Sets the field name
    * 
    * @param name The field name
    */
   public void setName(String name)
   {
      m_name = name;
   }

   /**
    * Sets the field read state.
    * 
    * @param readonly The field read state.
    */
   public void setReadonly(boolean readonly)
   {
      m_readonly = readonly;
   }

   /**
    * Sets the field mime-type.
    * 
    * @param type The field mime-type.
    */
   public void setType(String type)
   {
      m_type = type;
   }

   /**
    * Sets the field modified flag to indicate that this field has been modified
    * by the client.
    * 
    * @param modified The modified flag.
    */
   public void setModified(boolean modified)
   {
      m_modified = modified;
   }
}
