/* Copyright (c) 2008, The Storage Networking Industry Association.

   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA) nor 
   the names of its contributors may be used to endorse or promote products 
   derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
   THE POSSIBILITY OF SUCH DAMAGE.
*/

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.base;

/**
 * Interface describing a XAM field which can be either
 * a property or a stream.
 */
public interface Field
{
	/**
	 * Returns the binding state of the field.
	 * 
	 * @return true if the field is binding, false otherwise
	 */
	public boolean getBinding();
	
	/**
	 * Returns the length of the field in bytes.
	 * 
	 * @return The length of the field in bytes.
	 */
	public long getLength();
	
	/**
	 * Returns the name of the field.
	 * 
	 * @return The name of a field
	 */
	public String getName();
	
	/**
	 * Returns the read state of the field.
	 * 
	 * @return true if the field is read only, false otherwise
	 */
	public boolean getReadOnly();
	
	/**
	 * Returns the mime-type of the field.
	 * 
	 * @return The mime-type of the field.
	 */
	public String getType();
	
	/**
	 * Returns true if the field has been modified by the
	 * application, false otherwise.
	 * 
	 * @return true if the field has been modified by the application, false otherwise
	 */
	public boolean isModified();
}
