/* Copyright (c) 2008-2009, The Storage Networking Industry Association.

   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA) nor 
   the names of its contributors may be used to endorse or promote products 
   derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
   THE POSSIBILITY OF SUCH DAMAGE.
*/

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */
/* (c) Copyright 2009 Sun Microsystems                          */

package org.snia.xam.base;

import java.util.Iterator;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.XAMException;
import org.snia.xam.XIterator;

/**
 * Wrapped Iterator is used to provide XIterator functionality, but wraps a normal Java Iterator.
 * This is used when having Java classes internally would generate an iterator, and makes it available
 * to the XAM crowd.
 */
public class WrappedIterator
   implements XIterator
{
   Iterator<String> m_iter;
   
   /**
    * Create an XIterator based around a native Java Iterator.
    * @param i The native Java iterator.
    */
   public WrappedIterator( Iterator<String> i )
   {
      m_iter = i;
   }
   
   /**
    * When the application is done with the iterator, they must close it, per XAM specifications.
    * For this type of iterator, we release the base Iterator and that's it. This will cause later
    * accesses to this XIterator to fail and throw a NullPointerException. Tough luck, eh?
    */
   public void close()
      throws AuthenticationExpiredException, XAMException
   {
      m_iter = null;
   }

   /**
    * Indicates if there is one more item in the collection to be retrieved.
    */
   public boolean hasNext()
   {
      return m_iter.hasNext();
   }

   /**
    * Next() returns the next item in the collection.
    */
   public String next()
   {
      return m_iter.next();
   }

   /**
    * The remove method is present to satisfy the Iterator interface, but per spec shall always
    * throw an error.
    */
   public void remove()
   {
      throw new java.lang.UnsupportedOperationException("Remove NOT supported by XIterator");
   }
}
