/* Copyright (c) 2008, The Storage Networking Industry Association.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

/**
 * Based on "Free CRC source code in C"
 * Copyright (c) 2000 by Michael Barr.  This software is placed into
 * the public domain and may be used for any purpose.  However, this
 * notice must not be changed or removed and no warranty is either
 * expressed or implied by its publication or distribution.
 */
package org.snia.xam.base;

/**
 * Implements the CRC-16 algorithm used to verify a SNIA XAM Xuid.
 */
public class XAMCrc
   extends Crc
{
   private static final int POLYNOMIAL = 0x8005;

   private static final int INITIAL_REMAINDER = 0x0000;

   private static final int FINAL_XOR_VALUE = 0x0000;

   private static final int CHECK_VALUE = 0xbb3d;

   private static final boolean REFLECT_DATA = true;

   private static final boolean REFLECT_REMAINDER = true;

   /**
    * Creates a new XamCrc instance and performs a self-test using
    * the check value. Throws a RuntimeException if the test fails.
    * The self-test should NEVER fail unless the algorithm parameters
    * are modified incorrectly.
    */
   public XAMCrc()
   {
      super(16, POLYNOMIAL, INITIAL_REMAINDER, FINAL_XOR_VALUE, CHECK_VALUE,
         REFLECT_DATA, REFLECT_REMAINDER);
      // Perform self-test
      if (getCrc(Crc.CHECK_BYTES) != getCheckValue())
         throw new RuntimeException("XamCrc self-test failed!");
   }
}