/* Copyright (c) 2008, The Storage Networking Industry Association.

   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA) nor 
   the names of its contributors may be used to endorse or promote products 
   derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
   THE POSSIBILITY OF SUCH DAMAGE.
*/

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.util;

import java.util.Calendar;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.FieldContainer;
import org.snia.xam.FieldDoesNotExistException;
import org.snia.xam.FieldExistsException;
import org.snia.xam.InvalidArgumentException;
import org.snia.xam.InvalidFieldNameException;
import org.snia.xam.InvalidFieldTypeException;
import org.snia.xam.InvalidOperationException;
import org.snia.xam.MaximumFieldException;
import org.snia.xam.ObjectInUseException;
import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.XAsync;
import org.snia.xam.XAsyncListener;
import org.snia.xam.XIterator;
import org.snia.xam.XSetAbandonException;
import org.snia.xam.XSetCorruptException;
import org.snia.xam.XStream;
import org.snia.xam.XSystemAbandonException;
import org.snia.xam.XSystemCorruptException;
import org.snia.xam.XUID;

public class ExtendedFieldContainer
   implements FieldContainer
{
   FieldContainer m_field_container;
   
   public ExtendedFieldContainer( FieldContainer fc )
   {
      m_field_container = fc;
   }
   
   public XAsync asyncOpenXStream(String arg0,
                                  String arg1,
                                  long arg2,
                                  XAsyncListener arg3)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, FieldDoesNotExistException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      return m_field_container.asyncOpenXStream( arg0, arg1, arg2, arg3 );
   }

   public boolean containsField(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      return m_field_container.containsField(arg0);
   }

   public void createProperty(String arg0, boolean arg1, boolean arg2)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldExistsException, InvalidOperationException, MaximumFieldException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      m_field_container.createProperty( arg0, arg1, arg2 );
   }

   public void createProperty(String arg0, boolean arg1, long arg2)
      throws AuthenticationExpiredException, FieldExistsException,
      InvalidOperationException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      m_field_container.createProperty( arg0, arg1, arg2 );
   }

   public void createProperty(String arg0, boolean arg1, double arg2)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldExistsException, InvalidOperationException, MaximumFieldException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      m_field_container.createProperty( arg0, arg1, arg2 );
   }

   public void createProperty(String arg0, boolean arg1, XUID arg2)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldExistsException, InvalidArgumentException,
      InvalidOperationException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      m_field_container.createProperty( arg0, arg1, arg2 );
   }

   public void createProperty(String arg0, boolean arg1, String arg2)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldExistsException, InvalidArgumentException,
      InvalidOperationException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      m_field_container.createProperty( arg0, arg1, arg2 );
   }

   public void createProperty(String arg0, boolean arg1, Calendar arg2)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldExistsException, InvalidArgumentException,
      InvalidOperationException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      m_field_container.createProperty( arg0, arg1, arg2 );
   }

   public XStream createXStream(String arg0, boolean arg1, String arg2)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, InvalidOperationException,
      FieldExistsException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      return m_field_container.createXStream( arg0, arg1, arg2 );
   }

   public void deleteField(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      m_field_container.deleteField(arg0);
   }

   public boolean getBoolean(String arg0)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSetCorruptException, XAMException
   {
      return m_field_container.getBoolean(arg0);
   }

   public Calendar getDateTime(String arg0)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSetCorruptException, XAMException
   {
      return m_field_container.getDateTime(arg0);
   }

   public double getDouble(String arg0)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSetCorruptException, XAMException
   {
       return m_field_container.getDouble(arg0);
   }

   public boolean getFieldBinding(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      return m_field_container.getFieldBinding(arg0);
   }

   public long getFieldLength(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      return m_field_container.getFieldLength(arg0);
   }

   public boolean getFieldReadOnly(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      return m_field_container.getFieldReadOnly(arg0);
   }

   public String getFieldType(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      return m_field_container.getFieldType(arg0);
   }

   public long getLong(String arg0)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSetCorruptException, XAMException
   {
      return m_field_container.getLong(arg0);
   }

   public String getString(String arg0)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSetCorruptException, XAMException
   {
      return m_field_container.getString(arg0);
   }

   public XUID getXUID(String arg0)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSetCorruptException, XAMException
   {
      return m_field_container.getXUID(arg0);
   }

   public XIterator openFieldIterator(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      return m_field_container.openFieldIterator(arg0);
   }

   public XStream openXStream(String arg0, String arg1)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      return m_field_container.openXStream(arg0, arg1);
   }

   public void setFieldAsBinding(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      InvalidOperationException, FieldDoesNotExistException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      m_field_container.setFieldAsBinding(arg0);
   }

   public void setFieldAsNonbinding(String arg0)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      m_field_container.setFieldAsNonbinding(arg0);
   }

   public void setProperty(String arg0, boolean arg1)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      InvalidFieldTypeException, FieldDoesNotExistException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      m_field_container.setProperty(arg0, arg1);
   }

   public void setProperty(String arg0, Calendar arg1)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidArgumentException, InvalidFieldNameException,
      InvalidFieldTypeException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      m_field_container.setProperty(arg0, arg1);
   }

   public void setProperty(String arg0, double arg1)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
     m_field_container.setProperty(arg0, arg1 );
   }

   public void setProperty(String arg0, long arg1)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
     m_field_container.setProperty(arg0, arg1);
   }

   public void setProperty(String arg0, String arg1)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidArgumentException, InvalidFieldNameException,
      InvalidFieldTypeException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
     m_field_container.setProperty(arg0, arg1);
   }

   public void setProperty(String arg0, XUID arg1)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      InvalidArgumentException, InvalidFieldNameException,
      InvalidFieldTypeException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      m_field_container.setProperty(arg0, arg1);
   }

   /**
    * Create a property by being passed an object, which is one of Long, 
    * Double, Boolean, String, Calendar, XUID. All others will throw an exception.
    * This method is to be used when dealing with arbitrary collections of objects
    * which yield Object classed instances.
    * @throws InvalidArgumentException Unrecognized value class.
    */
   public void createProperty(String fieldName, boolean binding, Object value)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldExistsException, InvalidArgumentException, MaximumFieldException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      if( value instanceof Long )
         m_field_container.setProperty( fieldName, ((Long)value).longValue() );
      else if( value instanceof Double )
         m_field_container.setProperty( fieldName, ((Double)value).doubleValue() );
      else if( value instanceof Boolean )
         m_field_container.setProperty( fieldName, ((Boolean)value).booleanValue() );
      else if( value instanceof String )
         m_field_container.setProperty( fieldName, (String)value );
      else if( value instanceof Calendar )
         m_field_container.setProperty( fieldName, (Calendar) value );
      else if( value instanceof XUID )
         m_field_container.setProperty( fieldName, (XUID) value );
      else throw new InvalidArgumentException( "createProperty value is not acceptable XAM type" );
   }
   
   
   /**
    * Set the value of property by being passed an object, which is one of Long,
    * Double, String, Calendar, XUID, Boolean. All other will throw an
    * exception.
    */
   public void setProperty(String fieldName, Object value)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, InvalidFieldTypeException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
         if( value instanceof Long )
            m_field_container.setProperty( fieldName, ((Long)value).longValue() );
         else if( value instanceof Double )
            m_field_container.setProperty( fieldName, ((Double)value).doubleValue() );
         else if( value instanceof Boolean )
            m_field_container.setProperty( fieldName, ((Boolean)value).booleanValue() );
         else if( value instanceof String )
            m_field_container.setProperty( fieldName, (String)value );
         else if( value instanceof Calendar )
            m_field_container.setProperty( fieldName, (Calendar) value );
         else if( value instanceof XUID )
            m_field_container.setProperty( fieldName, (XUID) value );
         else throw new InvalidArgumentException( "createProperty value is not acceptable XAM type" );
   }

   /** Get the value of property returning an object,
    * which is one of Long, Double, String, Calendar, XUID,
    * Boolean. Applications must perform conversions if they wish
    * wish to use Integer, Float, or Date classes.
    */
   public java.lang.Object getProperty(String fieldName)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      InvalidFieldTypeException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSetCorruptException,
      XAMException
   {
      String prop_type = m_field_container.getFieldType(fieldName);
      if( prop_type.equals( XAMLibrary.STYPE_BOOLEAN_MIME_TYPE ))
         return new Boolean(m_field_container.getBoolean(fieldName));
      
      else if( prop_type.equals( XAMLibrary.STYPE_DATETIME_MIME_TYPE ))
         return m_field_container.getDateTime(fieldName);
      
      else if( prop_type.equals( XAMLibrary.STYPE_DOUBLE_MIME_TYPE))
         return new Double(m_field_container.getDouble(fieldName));
      
      else if( prop_type.equals( XAMLibrary.STYPE_INT_MIME_TYPE))
         return new Long( m_field_container.getLong(fieldName) );
      
      else if( prop_type.equals( XAMLibrary.STYPE_STRING_MIME_TYPE))
         return m_field_container.getString(fieldName);
      
      else if( prop_type.equals( XAMLibrary.STYPE_XUID_MIME_TYPE ))
         return m_field_container.getXUID(fieldName );
      else
         throw new InvalidFieldTypeException( "invalid field type for getProperty Object " + prop_type );
   }

   /** Convenience functions to determine if a field is a property. */
   public boolean fieldIsProperty(String fieldName)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      String prop_type = m_field_container.getFieldType(fieldName);
      return( prop_type.equals( XAMLibrary.STYPE_BOOLEAN_MIME_TYPE )) ||
              prop_type.equals( XAMLibrary.STYPE_DATETIME_MIME_TYPE ) ||
              prop_type.equals( XAMLibrary.STYPE_DOUBLE_MIME_TYPE) ||
              prop_type.equals( XAMLibrary.STYPE_INT_MIME_TYPE) ||
              prop_type.equals( XAMLibrary.STYPE_STRING_MIME_TYPE) ||
              prop_type.equals( XAMLibrary.STYPE_XUID_MIME_TYPE );
    }

   /** Convenience function to determine if a field is an XStream. */
   public boolean fieldIsStream(String fieldName)
      throws AuthenticationExpiredException, FieldDoesNotExistException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      return ! fieldIsProperty( fieldName );
   }
}
