/* Copyright (c) 2008, The Storage Networking Industry Association.

   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA) nor 
   the names of its contributors may be used to endorse or promote products 
   derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
   THE POSSIBILITY OF SUCH DAMAGE.
*/

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.util;

import java.io.UnsupportedEncodingException;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.AuthorizationException;
import org.snia.xam.FieldDoesNotExistException;
import org.snia.xam.InvalidArgumentException;
import org.snia.xam.InvalidFieldNameException;
import org.snia.xam.InvalidFieldTypeException;
import org.snia.xam.JobCommandException;
import org.snia.xam.ObjectInUseException;
import org.snia.xam.XAMException;
import org.snia.xam.XSet;
import org.snia.xam.XSetAbandonException;
import org.snia.xam.XSetCorruptException;
import org.snia.xam.XStream;
import org.snia.xam.XSystem;
import org.snia.xam.XSystemAbandonException;
import org.snia.xam.XSystemCorruptException;

public class QueryFactory
{
   public static XSet Query( XSystem sys, String query_string ) throws XSystemAbandonException, XSystemCorruptException, AuthenticationExpiredException, AuthorizationException, InvalidArgumentException, XAMException
   {
      try
      {
         XSet job = sys.createXSet(XSet.MODE_RESTRICTED);
         XStream queryStream = job.createXStream(XSet.XAM_JOB_QUERY_COMMAND,
            false,
            "text/plain;charset=utf-8");
         byte qBytes[] = query_string.getBytes("UTF-8");
         queryStream.write(qBytes, 0, qBytes.length);
         queryStream.close();
         job.createProperty(XSet.XAM_JOB_COMMAND, false, XSet.XAM_JOB_QUERY );
         job.submitJob();
         return job;
      }
      catch (UnsupportedEncodingException use)
      {
         throw new JobCommandException(
            "Unable to get UTF-8 bytes from query string");
      }
   }
   
   public static boolean QueryIsComplete( XSet job ) throws InvalidFieldNameException, InvalidFieldTypeException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, AuthenticationExpiredException, ObjectInUseException, XAMException
   {
      try
      {
         // Error health is set on completion
         String status = job.getString( XSet.XAM_JOB_STATUS );
         return status.equals( XSet.XAM_JOB_STATUS_COMPLETE) ||
                status.equals( XSet.XAM_JOB_STATUS_HALTED)   ||
                status.equals( XSet.XAM_JOB_STATUS_KILLED);
      }
      catch( FieldDoesNotExistException fdne )
      {
         // If the field ERRORHEALTH does not exist, the job isn't done.
         return false;
      }
   }   
}
