/* Copyright (c) 2008-2009, The Storage Networking Industry Association.

   Redistribution and use in source and binary forms, with or without modification, 
   are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA) nor 
   the names of its contributors may be used to endorse or promote products 
   derived from this software without specific prior written permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
   THE POSSIBILITY OF SUCH DAMAGE.
*/

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */
/* (c) Copyright (c) 2009, Sun Microsystems, Inc. */


package org.snia.xam;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.logging.Logger;

import org.snia.xam.base.DefaultXUID;
import org.snia.xam.base.Property;
import org.snia.xam.base.XAMConfiguration;
import org.snia.xam.base.AbstractFieldContainer;
import org.snia.xam.base.XAMImplementation;
import org.snia.xam.util.ISO8601Date;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;

public class TestVIMConfiguration
   extends XAMTest
{
   private static String CONFIG_FILE = "VIMConfigUnitTest";
   
   // Will connect to testVim in testExtraAndTyped
   private static final String TEST_XRI_TESTVIM = "snia-xam://vim3!localhost";

   // Fake VIM Classes
   private static String EXAMPLE_VIM = "com.example.Vim";
   private static String TEST_VIM    = "org.snia.xam.testvim.TestVim";
   
   // Fake VIM Aliases
   private static String VIM1 = "vim1";
   private static String VIM2 = "vim2";
   private static String VIM3 = "vim3";
   private static String VIM99 = "vim99";
   
   // Fake logging values 
   private static long   TEST_LEVEL = 3;
   private static long   TEST_VERBOSITY = 99;
   private static String TEST_PATH = "VimConfigUnitTestLog.txt";
   
   // Typed values
   private static final String EXTRA_PROP         = "com.example.extra.1" ;
   private static final String TYPED_INT_PROP     = "com.example.int" ;
   private static final String TYPED_DOUBLE_PROP  = "com.example.double" ;
   private static final String TYPED_BOOLEAN_PROP = "com.example.boolean" ;
   private static final String TYPED_DATE_PROP    = "com.example.date";
   private static final String TYPED_XUID_PROP    = "com.example.xuid" ;
   private static final String TYPED_STR_PROP     = "com.example.str" ;

   private static final String  EXTRA_VALUE          = "extra property value";
   private static final long    TYPED_INT_VALUE      = 137;
   private static final double  TYPED_DOUBLE_VALUE   = 42.42;
   private static final boolean TYPED_BOOLEAN_VALUE  = true;
   private static final String  TYPED_DATE_VALUE     = "2009-02-11T11:44:00-08:00"; 
   private static final String  TYPED_XUID_VALUE     = "AAA6AwA0ymgwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OF8zMl80ODMtMDhfMDAtYQ==";
   private static final String  TYPED_STR_VALUE      = "The universe ends with a whimper, not a bang.";
   
   class testFieldContainer extends AbstractFieldContainer
   {
      public testFieldContainer() throws XAMException
      { 
         setProtectNamespace(false);
         createDefaultProperties();
      }
      
      /**
       * Create the typical default params one might see in a XAMLibrary implementation. 
       * These are bogus values and are tested later to make sure that values are overwritten
       * by the configuration values. The default values then must be different than the constants
       * defined above. 
       * 
       * @throws XAMException
       */
      private void createDefaultProperties()
         throws XAMException
      {
         addField(new Property(XAMLibrary.XAM_IDENTITY,
            XAMLibrary.STYPE_STRING_MIME_TYPE, "FAKE FAKE FAKE", false, true));
         addField(new Property(XAMLibrary.XAM_API_LEVEL,
            XAMLibrary.STYPE_STRING_MIME_TYPE, "0.0.0", false, true));
         addField(new Property(XAMLibrary.XAM_LOG_LEVEL,
            XAMLibrary.STYPE_INT_MIME_TYPE, new Long(0), false, false));
         addField(new Property(XAMLibrary.XAM_LOG_PATH,
            XAMLibrary.STYPE_STRING_MIME_TYPE, "    ", false, false));
         addField(new Property(XAMLibrary.XAM_LOG_VERBOSITY,
            XAMLibrary.STYPE_INT_MIME_TYPE, new Long(-1), false, false));
         addField(new Property(XAMLibrary.XAM_LOG_LEVEL,
            XAMLibrary.STYPE_INT_MIME_TYPE, new Long(-1), false, false));
      }
      
      public XAsync asyncOpenXStream(String name, String mode, long xopid, XAsyncListener listener)
         throws AuthenticationExpiredException, InvalidArgumentException, InvalidFieldNameException, FieldDoesNotExistException, ObjectInUseException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, XSystemCorruptException, XAMException
      {
         throw new InvalidOperationException();
      }

      public XStream createXStream(String fieldName, boolean binding, String mimeType)
         throws AuthenticationExpiredException, InvalidArgumentException, InvalidFieldNameException, InvalidOperationException, FieldExistsException, MaximumFieldException, ObjectInUseException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, XSystemCorruptException, XAMException
      {
         throw new InvalidOperationException();
      }

      public XStream openXStream(String fieldName, String mode)
         throws AuthenticationExpiredException, InvalidFieldNameException, FieldDoesNotExistException, ObjectInUseException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, XSystemCorruptException, XAMException
      {
         throw new InvalidOperationException();
      }
   }
   
   public TestVIMConfiguration( String name )
   { 
      super(name);
   }
   
   /**
    * Create an example VIM Config file. This is a legal config file which contains
    * a series of config file options, typing, preload order, vim alias, and
    * extra properties to be passed to the XSystem.
    * @throws IOException 
    */
   String createLegalConfigFile() throws IOException
   {
      File configFile = File.createTempFile(CONFIG_FILE, null);
      try
      {
         PrintWriter configWriter = new PrintWriter(new FileWriter(configFile));
         configWriter.println( ".xam.config.vim.alias." + VIM1 + "=" + EXAMPLE_VIM );
         configWriter.println( ".xam.config.vim.alias." + VIM2 + "=" + EXAMPLE_VIM );
         configWriter.println( ".xam.config.vim.alias." + VIM3 + "=" + TEST_VIM );
         configWriter.println( ".xam.config.vim.lookuporder.2=" + VIM2 );
         configWriter.println( ".xam.config.vim.lookuporder.3=" + VIM1 );
         configWriter.println( ".xam.config.vim.lookuporder.99=" + VIM99 );
         configWriter.println( ".xam.config.vim.lookuporder.1=" + VIM3 );
         configWriter.println( ".xam.config.vim.preload.77=" + VIM1 );
         configWriter.println( ".xam.config.vim.preload.1=" + VIM2 );
         configWriter.println( ".xam.config.vim.preload.2=" + VIM3 );
         configWriter.println( "xam_int..xam.log.level=" + TEST_LEVEL );
         configWriter.println( "xam_int..xam.log.verbosity=" + TEST_VERBOSITY );
         configWriter.println( ".xam.log.path=" + TEST_PATH );
         configWriter.println( EXTRA_PROP +         "=" + EXTRA_VALUE );
         configWriter.println( XAMConfiguration.TYPE_INT + 
                               TYPED_INT_PROP +     "=" + TYPED_INT_VALUE );
         configWriter.println( XAMConfiguration.TYPE_DOUBLE + 
                               TYPED_DOUBLE_PROP +  "=" + TYPED_DOUBLE_VALUE );
         configWriter.println( XAMConfiguration.TYPE_BOOLEAN + 
                               TYPED_BOOLEAN_PROP + "=" + TYPED_BOOLEAN_VALUE );
         configWriter.println( XAMConfiguration.TYPE_DATETIME + 
                               TYPED_DATE_PROP +    "=" + TYPED_DATE_VALUE );
         configWriter.println( XAMConfiguration.TYPE_XUID + 
                               TYPED_XUID_PROP +    "=" + TYPED_XUID_VALUE );
         configWriter.println( XAMConfiguration.TYPE_STRING + 
                               TYPED_STR_PROP +     "=" + TYPED_STR_VALUE );
         configWriter.close();
      }
      catch( Exception e )
      {
         fail( "Unexpected error in setup of configuration file" );
      }
      return configFile.getAbsolutePath();
   }

   /**
    * Test the ability to load the configuratino file.
    */
   public void testStartup() throws Exception
   {
      String configFile = createLegalConfigFile();
      try
      {
         Logger logger = Logger.getLogger(TestVIMConfiguration.class.getName());
         logger.setUseParentHandlers(false);
         
         XAMConfiguration vc = new XAMConfiguration(logger);
         AbstractFieldContainer fc = new testFieldContainer();
         vc.load(configFile, fc);
         assertEquals(vc.classForVIMName(VIM1), EXAMPLE_VIM);
         assertEquals(vc.classForVIMName(VIM2), EXAMPLE_VIM);
         assertEquals(vc.classForVIMName(VIM3), TEST_VIM);
      }
      finally
      {
         File f = new File(configFile);
         f.delete();
      }
   }
   
   /**
    * Test the order list.
    */
   public void testOrder() throws Exception
   {
      String configFile = createLegalConfigFile();
      try
      {
         Logger logger = Logger.getLogger(TestVIMConfiguration.class.getName());
         logger.setUseParentHandlers(false);
         
         XAMConfiguration vc = new XAMConfiguration(logger);
         AbstractFieldContainer fc = new testFieldContainer();
         vc.load(configFile, fc);
         String vims[] = vc.getLookupOrder();
         assertEquals(VIM3, vims[0]);
         assertEquals(VIM2, vims[1]);
         assertEquals(VIM1, vims[2]);
         assertEquals(VIM99, vims[3]);
      }
      finally
      {
         File f = new File(configFile);
         f.delete();
      }
   }
   
   /**
    * Test the log file params made it in.
    */
   public void testLogging() throws Exception
   {
      String configFile = createLegalConfigFile();
      try
      {
         Logger logger = Logger.getLogger(TestVIMConfiguration.class.getName());
         logger.setUseParentHandlers(false);
         
         XAMConfiguration vc = new XAMConfiguration(logger);
         AbstractFieldContainer fc = new testFieldContainer();
         vc.load(configFile, fc);
         long level = fc.getLong(XAMLibrary.XAM_LOG_LEVEL);
         assertEquals(XAMLibrary.XAM_LOG_LEVEL, TEST_LEVEL, level);
         long verbosity = fc.getLong(XAMLibrary.XAM_LOG_VERBOSITY);
         assertEquals(XAMLibrary.XAM_LOG_VERBOSITY, TEST_VERBOSITY, verbosity);
         String path = fc.getString(XAMLibrary.XAM_LOG_PATH);
         assertEquals(XAMLibrary.XAM_LOG_PATH, TEST_PATH, path);
      }
      finally
      {
         File f = new File(configFile);
         f.delete();
      }
   }
   
   /**
    * Test to make sure the preload order is properly parsed
    * @throws Exception
    */
   public void testPreload() throws Exception
   {
      String configFile = createLegalConfigFile();
      try
      {
         Logger logger = Logger.getLogger(TestVIMConfiguration.class.getName());
         logger.setUseParentHandlers(false);
         
         XAMConfiguration vc = new XAMConfiguration(logger);
         AbstractFieldContainer fc = new testFieldContainer();
         vc.load(configFile, fc);
         String order[] = vc.getPreloadOrder();
         assertEquals("Preload order", VIM2, order[0]);
         assertEquals("Preload order", VIM3, order[1]);
         assertEquals("Preload order", VIM1, order[2]);
      }
      finally
      {
         File f = new File(configFile);
         f.delete();
      }
   }
   
   /**
    * Test to make sure that properties not predefined by the library can be passed to the XSystem instance at load.
    * Also checks to make sure that the typed variants of properties are parsed appropriately.
    * @throws Exception
    */
   public void testExtraAndTyped() throws Exception
   {
      String configFile = createLegalConfigFile();
      try
      {
         XAMImplementation xam = new XAMImplementation( configFile );
         XSystem system = xam.connect( TEST_XRI_TESTVIM );
         
         assertEquals("Extra non typed property didn't pass properly", 
                      EXTRA_VALUE, system.getString(EXTRA_PROP ));
         
         assertEquals("Typed Int property didn't parse properly",
            TYPED_INT_VALUE, system.getLong(TYPED_INT_PROP));

         assertEquals("Typed Double property didn't parse properly",
            TYPED_DOUBLE_VALUE, system.getDouble(TYPED_DOUBLE_PROP));

         assertEquals("Typed Boolean property didn't parse properly",
            TYPED_BOOLEAN_VALUE, system.getBoolean(TYPED_BOOLEAN_PROP));

         assertEquals("Typed Datetime property didn't parse properly",
            ISO8601Date.toCalendar(TYPED_DATE_VALUE), system.getDateTime(TYPED_DATE_PROP));

         assertEquals("Typed XUID property didn't parse properly",
            DefaultXUID.valueOf(TYPED_XUID_VALUE), system.getXUID(TYPED_XUID_PROP));

         assertEquals("Typed String property didn't parse properly",
            TYPED_STR_VALUE, system.getString(TYPED_STR_PROP));
      }
      finally
      {
         File f = new File(configFile);
         f.delete();
      }
   }
   
   /**
    * Standalone tests.
    */
   public static void main(String[] args)
   {
      System.exit(runTest(args, suite()));
   }

    public static int runTest(String[] args, Test test)
    {
        TestResult res = junit.textui.TestRunner.run(test);
        return Math.max(res.failureCount(), res.errorCount());
    }
    
   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new TestVIMConfiguration("testStartup"));
      suite.addTest(new TestVIMConfiguration("testOrder"));
      suite.addTest(new TestVIMConfiguration("testLogging"));
      suite.addTest(new TestVIMConfiguration("testPreload"));
      suite.addTest(new TestVIMConfiguration("testExtraAndTyped"));
      return suite;
   }
}
