/* Copyright (c) 2008, The Storage Networking Industry Association.
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */
/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.base;

import java.util.Arrays;
import org.snia.xam.InvalidXUIDException;
import org.snia.xam.XUID;
import org.snia.xam.base.Base64;
import org.snia.xam.testvim.TestXUID;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

public class DefaultXUIDTest
   extends TestCase
{
   private static final String VALID_XUID[] = {
      // XUID values donated by Hewlett Packard Co.
      "AAAACwAb25cAAHhzZXQxMjE2MjI5ODgwNTAz",
      "AAAACwAb6dYAAHhzZXQxMjE2MjI5ODgwNTM0",
      "AAAACwAbitQAAHhzZXQxMjE2MjI5ODgwNTUw",
      "AAAACwAbjOQAAHhzZXQxMjE2MjI5ODgwNjU5",
      "AAAACwAb6eUAAHhzZXQxMjE2MjI5ODgwNjc1",
      "AAAACwAbiiEAAHhzZXQxMjE2MjI5ODgwNjkw",
      "AAAACwAbGPYAAHhzZXQxMjE2MjI5ODgwNzA2",
      "AAAACwAbGUYAAHhzZXQxMjE2MjI5ODgwODAw",
      "AAAACwAbiocAAHhzZXQxMjE2MjI5ODgwODE1",
      "AAAACwAbKYcAAHhzZXQxMjE2MjI5ODgwODMx",
   
      // XUID values from org.snia.xam.testvim.TestXUID
      "AAA6AwA0ymgwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OF8zMl80ODMtMDhfMDAtYQ==",
      "AAA6AwA0RmowMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MTUtMDhfMDAtMQ==",
      "AAA6AwA0RyowMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MTUtMDhfMDAtMg==",
      "AAA6AwA0h+swMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MTUtMDhfMDAtMw==",
      "AAA6AwA0RaowMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MTUtMDhfMDAtNA==",
      "AAA6AwA0hWswMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MTUtMDhfMDAtNQ==",
      "AAA6AwA0hCswMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MTUtMDhfMDAtNg==",
      "AAA6AwA03nQwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MzAtMDhfMDAtNw==",
      "AAA6AwA02jQwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MzAtMDhfMDAtOA==",
      "AAA6AwA0GvUwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MzAtMDhfMDAtOQ==",
      "AAA6AwA04PQwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OV8wMl85MzAtMDhfMDAtYQ==",

      // XUID Values donated by EMC
      "AAAEcwA9sb8wTUdOTjE2ME1LRktUZUVKVVBQOEpFUEYwOUNHNDEzRjE1VjkzRTFKRlBGQkU4U1FLM040SQ==",
      "AAAEcwA9419GVk1LQ05PNlRPQlE0ZTVJUTc1UzNNSlQ2Q0pHNDEzRjE1VkcySDFLNjFHMFFUTVRLOFNWNw==",
      "AAAEcwA9PyM3TU5GTFFOMFNBSERGZUNTM09BVDExUjJERk9HNDEzRjE1VksxNzE2RkNLS0cxUUdLSlRRVg==",
      "AAAEcwA9S5NGNzYzRTJCUzZLT0xDZUJLSjFLMDVHRzhLOFVHNDEzRjE1Vk8xQjFLQTVCS0JSUFRJT01RTg==",
      "AAAEcwA9EF1GOUI4SjdTNERLMTlWZTBNREFVQ0lHSUk3MkJHNDEzRjE2MDIzRjE5Qk9PSE1DNk1CNTlSNQ=="
   };

   public static void main(String[] args)
   {
      TestResult res = junit.textui.TestRunner.run(suite());
      System.exit(Math.max(res.failureCount(), res.errorCount()));
   }

   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new DefaultXUIDTest("testValidXuid"));
      suite.addTest(new DefaultXUIDTest("testInvalidXuid"));
      suite.addTest(new DefaultXUIDTest("testBase64RoundTrip"));
      suite.addTest(new DefaultXUIDTest("testPaddedXUID"));
      return suite;
   }

   public DefaultXUIDTest(String test)
   {
      super(test);
   }

   /** Check to make sure that a base 64 version encodes and decodes correctly. */
   public void testBase64RoundTrip()
      throws Exception
   {
      final String base64XUID = "AAA6AwA0ymgwMDUwNTZDMDAwMDgtMjAwOC0wNy0xOFQxMV81OF8zMl80ODMtMDhfMDAtYQ==";
      final byte[] raw = Base64.base64Decode(base64XUID);
      DefaultXUID dx = new DefaultXUID(raw);
      
      final String test64XUID = dx.toString();
      
      assertTrue( "Base64 strings compare", test64XUID.equals( base64XUID ));
   }
   
   public void testValidXuid()
      throws Exception
   {
      for( int x = 0; x < VALID_XUID.length; x++ )
      {
         XUID xuid1 = DefaultXUID.valueOf(VALID_XUID[x]);
         assertEquals("XUID.toString()", VALID_XUID[x], xuid1.toString());
         XUID xuid2 = DefaultXUID.valueOf(xuid1.toBytes());
         byte[] bytes1 = xuid1.toBytes();
         byte[] bytes2 = xuid2.toBytes();
         assertTrue("XUID bytes are the same", Arrays.equals(bytes1, bytes2));
      }
   }

   public void testInvalidXuid()
      throws Exception
   {
      String invalid = "ABCDEFGHIJKLMNOPQ";
      String invalidEnc = Base64.base64Encode(invalid.getBytes());
      try
      {
         DefaultXUID.valueOf(invalid);
         fail("Should not have been able to decode XUID string: " + invalid);
      }
      catch (InvalidXUIDException x)
      {
      }
      try
      {
         DefaultXUID.valueOf(invalidEnc);
         fail("Should not have been able to decode XUID string: " + invalidEnc);
      }
      catch (InvalidXUIDException x)
      {
      }
      try
      {
         DefaultXUID.valueOf(invalid.getBytes());
         fail("Should not have been able to decode invalid bytes");
      }
      catch (InvalidXUIDException x)
      {
      }
   }

    /**
     * Test PaddedXUID. Will test that DefaultXUID can accept padded byte '
     * array for input. Both Base64 and byte arrays are allowed.
     */
   public void testPaddedXUID()
      throws Exception
   {
      final int MAX = 80;
      
      XUID x = TestXUID.newXUID();
      byte x_bytes[] = x.toBytes();
      
      if (x_bytes.length >= MAX)
         fail("Test is unable to pad the TestXUID, it is already MAX length (80 bytes)");

      int extra = MAX - x_bytes.length;

      // Try to pad it byte one byte and see what happens.
      byte x2_bytes[] = new byte[x_bytes.length + 1];
      for( int i = 0; i < x_bytes.length; i++ )
         x2_bytes[i] = x_bytes[i];
      x2_bytes[x_bytes.length] = 0;  // Pad
      XUID x2 = DefaultXUID.valueOf( x2_bytes );
      assertEquals( "Byte array, single pad equals orig", x, x2);

      String x2_str = Base64.base64Encode(x2_bytes);
      x2 = DefaultXUID.valueOf( x2_str );
      assertEquals( "Base64, single pad equals orig", x, x2 );

      x2_bytes = null;
      x2 = null;

      // Now to try padding all the way to 80.
      // This might replicate the previous test, if the test XUID was 
      // length 79. Doesn't hurt to rerun it, but may not cover all 
      // of the test conditions I'd like to see - mja.
      if( extra == 1 )
         System.out.println( "TestXUIDPadding really wants a test XUID less than 79" );

      byte x3_bytes[] = new byte[MAX];
      for( int i = 0; i < x_bytes.length; i++ )
      {
         x3_bytes[i] = x_bytes[i];
      }

      // Pad it.
      for( int p = x_bytes.length; p < MAX; p++ )
	          x3_bytes[p] = 0;
      XUID x3 = DefaultXUID.valueOf( x3_bytes );
      assertEquals( "Byte array, multi pad equals orig", x, x3 );

      String x3_str = Base64.base64Encode(x3_bytes);
      x3 = DefaultXUID.valueOf( x3_str );
      assertEquals( "Base64, multi pad equals orig", x, x3 );
	
      x3_bytes = null;
      x3 = null;
   }
}
