/* Copyright (c) 2008, The Storage Networking Industry Association.
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */
/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */
package org.snia.xam.base;

import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.FieldDoesNotExistException;
import org.snia.xam.FieldExistsException;
import org.snia.xam.InvalidArgumentException;
import org.snia.xam.InvalidFieldNameException;
import org.snia.xam.InvalidOperationException;
import org.snia.xam.MaximumFieldException;
import org.snia.xam.ObjectInUseException;
import org.snia.xam.XAMException;
import org.snia.xam.XAsync;
import org.snia.xam.XAsyncListener;
import org.snia.xam.XSetAbandonException;
import org.snia.xam.XSetCorruptException;
import org.snia.xam.XStream;
import org.snia.xam.XSystemAbandonException;
import org.snia.xam.XSystemCorruptException;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * This test class is to test that AbstractFieldContainer throws the appropriate Exceptions at the 
 * appropriate time. Since there are so many exceptions that may be thrown, not all possibilities are
 * tests. This tests only those that the AbstractFieldContainer is going to throw. This test suite 
 * is likely never "complete" and may be added to as time permits.
 * 
 * @author allisonm
 *
 */
public class ExceptionTest
   extends TestCase
{
   class testFieldContainer extends AbstractFieldContainer
   {
      testFieldContainer() { super(); }

      /** Added to satisfy the interface */
      public XAsync asyncOpenXStream(String name, String mode, long xopid, XAsyncListener listener)
         throws AuthenticationExpiredException, InvalidArgumentException, InvalidFieldNameException, FieldDoesNotExistException, ObjectInUseException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, XSystemCorruptException, XAMException
      {
         return null;
      }

      /** Added to satisfy the interface */
      public XStream createXStream(String fieldName, boolean binding, String mimeType)
         throws AuthenticationExpiredException, InvalidArgumentException, InvalidFieldNameException, InvalidOperationException, FieldExistsException, MaximumFieldException, ObjectInUseException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, XSystemCorruptException, XAMException
      {
         return null;
      }

      /** Added to satisfy the interface */
      public XStream openXStream(String fieldName, String mode)
         throws AuthenticationExpiredException, InvalidFieldNameException, FieldDoesNotExistException, ObjectInUseException, XSetAbandonException, XSetCorruptException, XSystemAbandonException, XSystemCorruptException, XAMException
      {
         return null;
      }
   }
   
   public static void main(String[] args)
   {
   }

   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new ExceptionTest("testInvalidFieldNameException"));
      suite.addTest(new ExceptionTest("testFieldExistsException"));
      return suite;
   }

   public ExceptionTest(String test)
   {
      super(test);
   }

   /** 
    * Test that the AbstractFieldContainer throws InvalidFieldNameException where expected.
    * This particular test is long, as there are many places the InvalidFieldNameException can
    * be thrown.
    * 
    * Please note that get/set for each field stype is not tested. This is because this test 
    * knows that they all end up calling a core routine. Be aware that this is clearly "whitebox"
    * thinking for this test.
    */
   public void testInvalidFieldNameException() throws Exception
   {
      AbstractFieldContainer fc = new testFieldContainer();
      
      // Make sure the name is not null
      try 
      { 
         fc.createProperty( null, false, 42 );
         fail( "Expected InvalidFieldException to be thrown by createProperty(null)");
      } catch (InvalidFieldNameException ifne) {}
      
      // getFieldValie
      try 
      { 
         fc.getBoolean(null);
         fail( "Expected InvalidFieldException to be thrown by getBoolean(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // Make sure the protected name space is protected.
      try 
      { 
         fc.createProperty( ".com.example", false, "a string" );
         fail( "Expected InvalidFieldException to be thrown by createProperty(protect space)" );
      } catch (InvalidFieldNameException ifne) {}
      
      // setProperty needs to throw InvalidFieldName on null
      try 
      { 
         fc.setProperty( null, 42.0 );
         fail( "Expected InvalidFieldException to be thrown by setProperty( null, 42.0)" );
      } catch (InvalidFieldNameException ifne) {}
      
      // Field level attributes should throw InvalidFieldNameException on null name
      try 
      { 
         fc.getFieldBinding(null);
         fail( "Expected InvalidFieldNameException to be thrown by getFieldBinding(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // Field level attributes should throw InvalidFieldNameException on null name
      try 
      { 
         fc.getFieldReadOnly(null);
         fail( "Expected InvalidFieldNameException to be thrown by getFieldReadOnly(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // Field level attributes should throw InvalidFieldNameException on null name
      try 
      { 
         fc.getFieldLength(null);
         fail( "Expected InvalidFieldNameException to be thrown by getFieldLength(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // Field level attributes should throw InvalidFieldNameException on null name
      try 
      { 
         fc.getFieldType(null);
         fail( "Expected InvalidFieldNameException to be thrown by getFieldType(null" );
      } catch (InvalidFieldNameException ifne) {}

      // Set fieldBinding/Nonbinding
      try 
      { 
         fc.setFieldAsBinding(null);
         fail( "Expected InvalidFieldNameException to be thrown by setFieldAsBinding(null)" );
      } catch (InvalidFieldNameException ifne) {}

      try 
      { 
         fc.setFieldAsNonbinding(null);
         fail( "Expected InvalidFieldNameException to be thrown by setFieldAsNonbinind(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // Delete field
      try 
      { 
         fc.deleteField(null);
         fail( "Expected InvalidFieldNameException to be thrown by deleteField(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // Delete field
      try 
      { 
         fc.openFieldIterator(null);
         fail( "Expected InvalidFieldNameException to be thrown by openFieldIterator(null)" );
      } catch (InvalidFieldNameException ifne) {}

      // containsField
      try 
      { 
         fc.containsField(null);
         fail( "Expected InvalidFieldNameException to be thrown by containsField(null)" );
      } catch (InvalidFieldNameException ifne) {}
   }
   
   /**
    * Test to make sure that the FieldExistsException is thrown by AbstractFieldContainer when appropriate
    */
   public void testFieldExistsException() throws Exception
   {
      AbstractFieldContainer fc = new testFieldContainer();
      
      // Existing field name
      try 
      { 
         fc.createProperty( "com.example.prop", false, true );
         fc.createProperty( "com.example.prop", false, true );
         fail( "Expected FieldExistsException to be thrown by createProperty(com.example.prop, false, true)" );
      }
      catch( FieldExistsException fee )
      {
      }
      
   }
}
