/* Copyright (c) 2008, The Storage Networking Industry Association.
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */
/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */
package org.snia.xam.base;

import java.util.Calendar;

import org.snia.xam.util.ISO8601Date;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class ISO8601Test
   extends TestCase
{
   private static final String VALID_DATES[] = {
      "2008-08-18T14:13:38.938-07:00", // A random, but good date
      "2008-08-18T00:00:00.000-07:00", // A midnight date
      "2008-08-18T00:00:00.000Z", // A zulu (GMT) date
      "-9999-01-01T00:00:00.000Z", // A really long time ago
      "9999-12-31T23:59:59.999+23:59", // A long time from now.
      "2008-08-19T00:00:00.000+11:27" // A goofy time zone
   };

   // There are too many to enumerate, but pick the most reasonable ones.
   private static final String INVALID_DATES[] = {
      "2008-08-19T00:00:00.000T+7:00", // TZ offset needs 2 digits
      "2008-08-19T00:00:00.000T+07:77", // TZ minute offset illegal
      "20080-08-19T00:00:00.000Z", // Years have five digits
      "2008-44-19T00:00:00.000T+12:00", // Illegal month
      "2007-08-42T00:00:00.000T+01:01", // Illegal day
      "2009-08-18T24:00:00.000T+01:00", // Illegal time, 24:00 not allowed
      "2008-08-18T00:60:00.000T+03:00", // Illegal minutes
      "2008-08-18T00:00:61.000T+04:00", // Ilegal seconds
      "2008-08-18T00:00:00.000T+13:00", // Illegal TZ hours
      "2008-08-18T00:00:00.000T+00:89" // Illegal TZ Minutes
   };

   public static void main(String[] args)
   {
   }

   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new ISO8601Test("testValidDates"));
      suite.addTest(new ISO8601Test("testInvalidDates"));
      suite.addTest(new ISO8601Test("testDateRoundTrip"));
      return suite;
   }

   public ISO8601Test(String test)
   {
      super(test);
   }

   public void testValidDates()
      throws Exception
   {
      for (int x = 0; x < VALID_DATES.length; x++)
      {
         try
         {
            ISO8601Date.toCalendar(VALID_DATES[x]);
         }
         catch (Exception e)
         {
            fail("Valid date parse failure: #" + x + " " + e);
         }
      }
   }

   /** Check to make sure that invalid dates throw an exception. */
   public void testInvalidDates()
      throws Exception
   {
      for (int x = 0; x < INVALID_DATES.length; x++)
      {
         try
         {
            ISO8601Date.toCalendar(INVALID_DATES[x]);
            fail("Invalid date did not throw an exception: #" + x + " "
               + INVALID_DATES[x]);
         }
         catch (Exception e)
         {
            assertTrue("Excpected a ParseErrorException: ",
               e instanceof java.text.ParseException);
         }
      }
   }

   /** Check to make sure that dates endcode then decode correctly */
   public void testDateRoundTrip()
      throws Exception
   {
      for (int x = 0; x < VALID_DATES.length; x++)
      {
         try
         {
            Calendar c = ISO8601Date.toCalendar(VALID_DATES[x]);
            String test = ISO8601Date.toISO8601String(c);
            assertTrue("Dates must match round trip, #" + x + "/"
               + VALID_DATES[x] + "/" + test, VALID_DATES[x].equals(test));
         }
         catch (Exception e)
         {
            fail("Valid date parse failure: #" + x + " " + e);
         }
      }
   }
}
