/* Copyright (c) 2008, The Storage Networking Industry Association.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.testvim;

import java.util.Calendar;
import java.io.ByteArrayOutputStream;

import org.snia.xam.AuthenticationException;
import org.snia.xam.AuthenticationExpiredException;
import org.snia.xam.AuthorizationException;
import org.snia.xam.FieldDoesNotExistException;
import org.snia.xam.FieldExistsException;
import org.snia.xam.HoldIdException;
import org.snia.xam.InvalidArgumentException;
import org.snia.xam.InvalidFieldNameException;
import org.snia.xam.InvalidOperationException;
import org.snia.xam.InvalidXSetModeException;
import org.snia.xam.InvalidXUIDException;
import org.snia.xam.MaximumFieldException;
import org.snia.xam.ObjectInUseException;
import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.XAsync;
import org.snia.xam.XAsyncListener;
import org.snia.xam.XSet;
import org.snia.xam.XSetAbandonException;
import org.snia.xam.XSetCorruptException;
import org.snia.xam.XSetInaccessibleException;
import org.snia.xam.XSetUnderHoldException;
import org.snia.xam.XSetUnderRetentionException;
import org.snia.xam.XSystem;
import org.snia.xam.XStream;
import org.snia.xam.XSystemAbandonException;
import org.snia.xam.XSystemCorruptException;
import org.snia.xam.XSystemException;
import org.snia.xam.XUID;
import org.snia.xam.base.AbstractFieldContainer;
import org.snia.xam.base.DefaultXUID;
import org.snia.xam.base.Property;
import org.snia.xam.util.SASLUtils;

public class TestXSystem
   extends AbstractFieldContainer
   implements XSystem
{
   /** Default SASL authentication mechanism */
   protected static final String SASL_DEFAULT = SASLUtils.SASL_PLAIN;

   /** Indicates if the system is authtenticated. */
   boolean m_authenticated = false;

   /** Record what authentication may be in progress */
   protected String m_auth;

   /** User credentials for PLAIN */
   final String TEST_USER = "test";

   /** User crednetials for PLAIN */
   final String TEST_PASS = "pass";

   public TestXSystem()
      throws XAMException
   {
      addPreAuthProperties();
   }

   void addPreAuthProperties()
      throws XAMException
   {
      addField(new Property(XAM_XSYSTEM_AUTH_SASL_DEFAULT,
         XAMLibrary.STYPE_STRING_MIME_TYPE, SASLUtils.SASL_PLAIN, false, true));
      addField(new Property(XAM_XSYSTEM_AUTH_SASL_LIST
         + SASLUtils.SASL_ANONYMOUS, XAMLibrary.STYPE_BOOLEAN_MIME_TYPE,
         Boolean.valueOf(true), false, true));
      addField(new Property(XAM_XSYSTEM_AUTH_SASL_LIST + SASLUtils.SASL_PLAIN,
         XAMLibrary.STYPE_BOOLEAN_MIME_TYPE, Boolean.valueOf(true), false, true));
      addField(new Property(XAM_XSYSTEM_INITIALIZING,
         XAMLibrary.STYPE_BOOLEAN_MIME_TYPE, Boolean.valueOf(true), false,
         false));
   }

   protected void checkProtectedNameSpace(String fieldName)
      throws XAMException
   {
      if (!getBoolean(XSystem.XAM_XSYSTEM_INITIALIZING))
         super.checkProtectedNameSpace(fieldName);
   }

   public XAsync asyncOpenXStream(String name,
                                  String mode,
                                  long xopid,
                                  XAsyncListener listener)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, FieldDoesNotExistException,
      ObjectInUseException, XSetAbandonException, XSetCorruptException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   /**
    * This XSystem will always use DefaultXUID, since there is no way of
    * telling if the XUID is specific to this XSystem or a "foreign" system.
    * Since this is a test system, and doesn't really do anything with the
    * XUIDs, it is not important. A mature XSystem implementation might 
    * use a more sophisticated scheme.
    */
   public XUID createXUID(String base64String)
      throws InvalidXUIDException, XAMException
   {
      return DefaultXUID.valueOf(base64String);
   }

   /**
    * This XSystem will always use DefaultXUID, since there is no way of
    * telling if the XUID is specific to this XSystem or a "foreign" system.
    * Since this is a test system, and doesn't really do anything with the
    * XUIDs, it is not important. A mature XSystem implementation might 
    * use a more sophisticated scheme.
    */
   public XUID createXUID(byte byteBuffer[])
      throws InvalidXUIDException, XAMException
   {
      return DefaultXUID.valueOf(byteBuffer);
   }

   public XStream createXStream(String fieldName,
                                boolean binding,
                                String mimeType)
      throws AuthenticationExpiredException, InvalidArgumentException,
      InvalidFieldNameException, InvalidOperationException,
      FieldExistsException, MaximumFieldException, ObjectInUseException,
      XSetAbandonException, XSetCorruptException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public XStream openXStream(String fieldName, String mode)
      throws AuthenticationExpiredException, InvalidFieldNameException,
      FieldDoesNotExistException, ObjectInUseException, XSetAbandonException,
      XSetCorruptException, XSystemAbandonException, XSystemCorruptException,
      XAMException
   {
      throw new InvalidOperationException();
   }

   public void abandon()
      throws AuthenticationExpiredException, XAMException
   {
      throw new InvalidOperationException();
   }

   public boolean accessXSet(XUID xsetid, long mode)
      throws AuthenticationExpiredException, InvalidArgumentException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public XAsync asyncCopyXSet(XUID inXuid,
                               String mode,
                               long xopid,
                               XAsyncListener listener)
      throws AuthenticationExpiredException, AuthorizationException,
      InvalidArgumentException, XSetInaccessibleException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public XAsync asyncOpenXSet(XUID inXuid,
                               String mode,
                               long xopid,
                               XAsyncListener listener)
      throws AuthorizationException, AuthenticationExpiredException,
      InvalidArgumentException, XSetInaccessibleException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   /**
    * Validate that the byte buffers exactly match.
    * 
    * @param value Expected bytes
    * @param buf Actual bytes
    * @param count Number of bytes to check
    * @return True if the bytes exactly match
    */
   protected int checkBuffer(byte[] value, byte[] buf )
   {
      for (int i = 0; i < value.length; i++)
         if (value[i] != buf[i]) return -1;
      return value.length;
   }

   /**
    * Authentication for the TestXSystem. This XSystem will support ANONYMOUS and PLAIN.
    * For Plain to work out, the application must supply the username "test". The password
    * is ignored.
    */
   public byte[] authenticate(byte[] buffer)
      throws AuthenticationException, InvalidArgumentException, XAMException
   {
      ByteArrayOutputStream response = new ByteArrayOutputStream();
      authenticate(buffer, response);
      return response.toByteArray();
   }

   public int authenticate(byte[] buffer, ByteArrayOutputStream response)
      throws AuthenticationException, InvalidArgumentException, XAMException
   {
      response.reset();
      synchronized (this)
      {
         if (m_authenticated)
            throw new XSystemException("XSystem already authenticated");

         int start = 0;
         // Check to see if buffer contains a SASL mechanism
         if (m_auth == null)
         {
            if( (start = checkBuffer(SASLUtils.SASL_PLAIN_BYTES, buffer)) > -1)
            {
               m_auth = SASLUtils.SASL_PLAIN;
               return XSystem.XAM_SASL_IN_PROCESS;
            }

            else if( (start = checkBuffer(SASLUtils.SASL_ANONYMOUS_BYTES,  buffer)) > -1)
            {
               m_auth = SASLUtils.SASL_ANONYMOUS;
               return XSystem.XAM_SASL_IN_PROCESS;
            }
         }

         // If we did not get a SASL mechanism then use the default
         if (m_auth == null) m_auth = SASL_DEFAULT;

         if (m_auth.equals(SASLUtils.SASL_ANONYMOUS))
         {
            m_authenticated = true;
         }
         else if (m_auth.equals(SASLUtils.SASL_PLAIN))
         {
            String[] strs = SASLUtils.decodeSASLPlain(buffer, start, buffer.length);

            // Just need a username and password
            // Anything will do!
            m_authenticated = true;
         }
         else
         {
            throw new AuthenticationException("SASL mechanism " + m_auth
               + " unsupported");
         }

         byte[] resp_bytes = SASLUtils.SASL_SUCCESS.getBytes();
         response.write(resp_bytes, 0, resp_bytes.length);
         return XAM_SASL_COMPLETE;
      }
   }

   public void setProperty(String fieldName, boolean value)
      throws XAMException
   {
      if (fieldName.equals(XSystem.XAM_XSYSTEM_INITIALIZING) && !value)
      {
         Property p = (Property) this.getField(fieldName);
         removeField(p);
         addField(new Property(XAM_XSYSTEM_INITIALIZING,
            XAMLibrary.STYPE_BOOLEAN_MIME_TYPE, Boolean.valueOf(false), false,
            true));
      }
   }

   public void close()
      throws AuthenticationExpiredException, ObjectInUseException, XAMException
   {
      throw new InvalidOperationException();
   }

   public void connect(String xri)
      throws XAMException
   {
      // The test system doesn't do much, so a connection is happy.
   }

   public XSet copyXSet(XUID xsetid, String copyMode)
      throws AuthenticationExpiredException, AuthorizationException,
      InvalidArgumentException, InvalidXSetModeException,
      XSetInaccessibleException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   /**
    * Create a new XSet, let the XSet deal with the post commit mode.
    */
   public XSet createXSet(String createMode)
      throws AuthenticationExpiredException, AuthorizationException,
      InvalidArgumentException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      return new TestXSet(this, createMode);
   }

   public void deleteXSet(XUID xsetid)
      throws AuthorizationException, AuthenticationExpiredException,
      XSetUnderRetentionException, XSetUnderHoldException,
      InvalidArgumentException, XSetInaccessibleException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public Calendar getXSetAccessTime(XUID xsetid)
      throws AuthenticationExpiredException, InvalidArgumentException,
      XSetInaccessibleException, XSystemAbandonException, XSetCorruptException,
      XAMException
   {
      throw new InvalidOperationException();
   }

   public void holdXSet(XUID xsetid, String holdID)
      throws AuthenticationExpiredException, AuthorizationException,
      InvalidFieldNameException, HoldIdException, InvalidArgumentException,
      XSetInaccessibleException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public boolean isXSetRetained(XUID xsetid)
      throws AuthenticationExpiredException, InvalidArgumentException,
      XSetInaccessibleException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public XSet openXSet(XUID xsetid, String openMode)
      throws AuthenticationExpiredException, AuthorizationException,
      InvalidArgumentException, InvalidXSetModeException,
      XSetInaccessibleException, XSystemAbandonException,
      XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }

   public void releaseXSet(XUID xsetid, String holdID)
      throws AuthenticationExpiredException, AuthorizationException,
      HoldIdException, InvalidArgumentException, XSetInaccessibleException,
      XSystemAbandonException, XSystemCorruptException, XAMException
   {
      throw new InvalidOperationException();
   }
}
