/* Copyright (c) 2008, The Storage Networking Industry Association.
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */
/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */

package org.snia.xam.util;

import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.XSet;
import org.snia.xam.XSystem;
import org.snia.xam.util.QueryFactory;
import org.snia.xam.util.XAMLibraryFactory;
import org.snia.xam.base.XAMImplementation;
import org.snia.xam.testvim.TestXSet;
import org.snia.xam.testvim.TestXSystem;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import java.io.File;
import java.io.PrintWriter;

/**
 * Test the QueryFactory class. This test suite tests the query factory methods.
 */
public class QueryFactoryTest
   extends TestCase
{
   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new QueryFactoryTest("testQueryCreation"));
      suite.addTest(new QueryFactoryTest("testQueryComplete"));
      return suite;
   }

   public QueryFactoryTest(String test)
   {
      super(test);
   }

   /**
    * Simple test, checks out the default implementation creation, and 
    * named class creations. Also tests a config file parameterized creation.
    * This test only checks to make sure that basic Job is set up correctly.  It does
    * not check to make sure that the result fields are correct.
    */
   public void testQueryCreation()
      throws Exception
   {
      XSystem xsys = new TestXSystem();
      XSet job = QueryFactory.Query( xsys, "select \".xset.xuid\"" );
    
      assertTrue( "Job contains " + XSet.XAM_JOB_COMMAND,       job.containsField(XSet.XAM_JOB_COMMAND ));
      assertTrue( "Job contains " + XSet.XAM_JOB_QUERY_COMMAND, job.containsField(XSet.XAM_JOB_QUERY_COMMAND ));
      assertEquals( "Query string is UTF-8",
         "text/plain;charset=utf-8" , job.getFieldType(XSet.XAM_JOB_QUERY_COMMAND) );
   }

   /**
    * Test a bad creation, with a bad class value. 
    */
   public void testQueryComplete()
      throws Exception
   {
      XSystem xsys = new TestXSystem();
      TestXSet job = (TestXSet) QueryFactory.Query( xsys, "select \".xset.xuid\"" );
      
      assertFalse( "Query should not be complete ", QueryFactory.QueryIsComplete(job));
      
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_COMPLETE );
     assertTrue( "Query is complete in status == " + XSet.XAM_JOB_STATUS_COMPLETE,
        QueryFactory.QueryIsComplete(job));
     
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_HALTED );
     assertTrue( "Query is complete in status == " + XSet.XAM_JOB_STATUS_HALTED, 
        QueryFactory.QueryIsComplete(job));
     
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_KILLED );
     assertTrue( "Query is complete in status == " + XSet.XAM_JOB_STATUS_KILLED, 
        QueryFactory.QueryIsComplete(job));
     
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_RUNNING );
     assertFalse( "Query is complete in status == " + XSet.XAM_JOB_STATUS_RUNNING, 
        QueryFactory.QueryIsComplete(job));

     // Try a few non-complete status values
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_SHUTTING_DOWN );
     assertFalse( "Query is complete in status == " +  XSet.XAM_JOB_STATUS_SHUTTING_DOWN, 
        QueryFactory.QueryIsComplete(job));

     job.testJobSetStatus( XSet.XAM_JOB_STATUS_STARTING );
     assertFalse( "Query is complete in status == " +  XSet.XAM_JOB_STATUS_STARTING, 
        QueryFactory.QueryIsComplete(job));
     
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_SUSPENDED );
     assertFalse( "Query is complete in status == " +  XSet.XAM_JOB_STATUS_SUSPENDED, 
        QueryFactory.QueryIsComplete(job));
     
     job.testJobSetStatus( XSet.XAM_JOB_STATUS_NEW );
     assertFalse( "Query is complete in status == " +  XSet.XAM_JOB_STATUS_NEW, 
        QueryFactory.QueryIsComplete(job));
   }
}
