/* Copyright (c) 2008, The Storage Networking Industry Association.
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice, 
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.
 
 * Neither the name of The Storage Networking Industry Association (SNIA) nor 
 the names of its contributors may be used to endorse or promote products 
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 THE POSSIBILITY OF SUCH DAMAGE.
 */
/* (c) Copyright 2008 Hewlett-Packard Development Company, L.P. */
package org.snia.xam.util;

import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.util.XAMLibraryFactory;
import org.snia.xam.base.XAMImplementation;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import java.io.File;
import java.io.PrintWriter;

/**
 * Test the XAMLibraryFactory class. This test suite will test the positive
 * create path, with no arguments, and with a property path.
 */
public class XAMLibraryFactoryTest
   extends TestCase
{
   public static final String TEST_PROP = "test.time";

   public static Test suite()
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new XAMLibraryFactoryTest("testCreation"));
      suite.addTest(new XAMLibraryFactoryTest("testInvalidClass"));
      return suite;
   }

   public XAMLibraryFactoryTest(String test)
   {
      super(test);
   }

   /**
    * Simple test, checks out the default implementation creation, and 
    * named class creations. Also tests a config file parameterized creation.
    */
   public void testCreation()
      throws Exception
   {
      final String TEST_LIB = "org.snia.xam.util.XAMLibraryFactoryTestLib";
      XAMLibrary lib = XAMLibraryFactory.newXAMLibrary();
      assertTrue("Default library is Java XAM Library",
         lib instanceof org.snia.xam.base.XAMImplementation);
      ((XAMImplementation) lib).close();
      
      XAMLibrary lib2 = XAMLibraryFactory.newXAMLibrary(XAMLibraryFactory.JAVA_XAM_LIBRARY);
      assertTrue("newLibrary instance of " + XAMLibraryFactory.JAVA_XAM_LIBRARY,
         lib2 instanceof org.snia.xam.base.XAMImplementation);
      ((XAMImplementation) lib2).close();
      
      XAMLibrary lib3 = XAMLibraryFactory.newXAMLibrary(TEST_LIB);
      assertTrue("newLibrary instance of " + TEST_LIB,
         lib3 instanceof org.snia.xam.util.XAMLibraryFactoryTestLib);
 
      File f = File.createTempFile("XAM", "xam");
      PrintWriter testProps = new PrintWriter(f);
      long time = System.currentTimeMillis();
      testProps.println(TEST_PROP + "=" + time);
      testProps.close();
      XAMLibrary lib4 = XAMLibraryFactory.newXAMLibrary(TEST_LIB,
         f.getAbsolutePath());
      assertTrue("newLibrary instance of " + TEST_LIB,
         lib4 instanceof org.snia.xam.util.XAMLibraryFactoryTestLib);
      XAMLibraryFactoryTestLib tl4 = (XAMLibraryFactoryTestLib) lib4;
      assertTrue("Properties path match to factory create",
         time == tl4.m_prop_time);
   }

   /**
    * Test a bad creation, with a bad class value. 
    */
   public void testInvalidClass()
      throws Exception
   {
      try
      {
         XAMLibrary lib = XAMLibraryFactory.newXAMLibrary("classWhichDoesntExist");
         fail("Factory should have thrown a XAMException. ");
         lib = null;
      }
      catch (XAMException xe)
      {
         // This is a good thing, for this test.
      }
   }
}
